# frozen_string_literal: true

# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Auto-generated by gapic-generator-ruby. DO NOT EDIT!


module Google
  module Cloud
    module Compute
      module V1
        # Messages
        #
        # Contains the configurations necessary to generate a signature for access to private storage buckets that support Signature Version 4 for authentication. The service name for generating the authentication header will always default to 's3'.
        # @!attribute [rw] access_key
        #   @return [::String]
        #     The access key used for s3 bucket authentication. Required for updating or creating a backend that uses AWS v4 signature authentication, but will not be returned as part of the configuration when queried with a REST API GET request. @InputOnly
        # @!attribute [rw] access_key_id
        #   @return [::String]
        #     The identifier of an access key used for s3 bucket authentication.
        # @!attribute [rw] access_key_version
        #   @return [::String]
        #     The optional version identifier for the access key. You can use this to keep track of different iterations of your access key.
        # @!attribute [rw] origin_region
        #   @return [::String]
        #     The name of the cloud region of your origin. This is a free-form field with the name of the region your cloud uses to host your origin. For example, "us-east-1" for AWS or "us-ashburn-1" for OCI.
        class AWSV4Signature
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceGroupManagers.AbandonInstances. See the method description for details.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     The name of the managed instance group.
        # @!attribute [rw] instance_group_managers_abandon_instances_request_resource
        #   @return [::Google::Cloud::Compute::V1::InstanceGroupManagersAbandonInstancesRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where the managed instance group is located.
        class AbandonInstancesInstanceGroupManagerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionInstanceGroupManagers.AbandonInstances. See the method description for details.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     Name of the managed instance group.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] region_instance_group_managers_abandon_instances_request_resource
        #   @return [::Google::Cloud::Compute::V1::RegionInstanceGroupManagersAbandonInstancesRequest]
        #     The body resource for this request
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class AbandonInstancesRegionInstanceGroupManagerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A specification of the type and number of accelerator cards attached to the instance.
        # @!attribute [rw] accelerator_count
        #   @return [::Integer]
        #     The number of the guest accelerator cards exposed to this instance.
        # @!attribute [rw] accelerator_type
        #   @return [::String]
        #     Full or partial URL of the accelerator type resource to attach to this instance. For example: projects/my-project/zones/us-central1-c/acceleratorTypes/nvidia-tesla-p100 If you are creating an instance template, specify only the accelerator name. See GPUs on Compute Engine for a full list of accelerator types.
        class AcceleratorConfig
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents an Accelerator Type resource. Google Cloud Platform provides graphics processing units (accelerators) that you can add to VM instances to improve or accelerate performance when working with intensive workloads. For more information, read GPUs on Compute Engine.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] deprecated
        #   @return [::Google::Cloud::Compute::V1::DeprecationStatus]
        #     [Output Only] The deprecation status associated with this accelerator type.
        # @!attribute [rw] description
        #   @return [::String]
        #     [Output Only] An optional textual description of the resource.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] The type of the resource. Always compute#acceleratorType for accelerator types.
        # @!attribute [rw] maximum_cards_per_instance
        #   @return [::Integer]
        #     [Output Only] Maximum number of accelerator cards allowed per instance.
        # @!attribute [rw] name
        #   @return [::String]
        #     [Output Only] Name of the resource.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined, fully qualified URL for this resource.
        # @!attribute [rw] zone
        #   @return [::String]
        #     [Output Only] The name of the zone where the accelerator type resides, such as us-central1-a. You must specify this field as part of the HTTP request URL. It is not settable as a field in the request body.
        class AcceleratorType
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::AcceleratorTypesScopedList}]
        #     A list of AcceleratorTypesScopedList resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#acceleratorTypeAggregatedList for aggregated lists of accelerator types.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class AcceleratorTypeAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::AcceleratorTypesScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # Contains a list of accelerator types.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::AcceleratorType>]
        #     A list of AcceleratorType resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#acceleratorTypeList for lists of accelerator types.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class AcceleratorTypeList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] accelerator_types
        #   @return [::Array<::Google::Cloud::Compute::V1::AcceleratorType>]
        #     [Output Only] A list of accelerator types contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] An informational warning that appears when the accelerator types list is empty.
        class AcceleratorTypesScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] guest_accelerator_count
        #   @return [::Integer]
        #     Number of accelerator cards exposed to the guest.
        # @!attribute [rw] guest_accelerator_type
        #   @return [::String]
        #     The accelerator type resource name, not a full URL, e.g. nvidia-tesla-t4.
        class Accelerators
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # An access configuration attached to an instance's network interface. Only one access config per instance is supported.
        # @!attribute [rw] external_ipv6
        #   @return [::String]
        #     Applies to ipv6AccessConfigs only. The first IPv6 address of the external IPv6 range associated with this instance, prefix length is stored in externalIpv6PrefixLength in ipv6AccessConfig. To use a static external IP address, it must be unused and in the same region as the instance's zone. If not specified, Google Cloud will automatically assign an external IPv6 address from the instance's subnetwork.
        # @!attribute [rw] external_ipv6_prefix_length
        #   @return [::Integer]
        #     Applies to ipv6AccessConfigs only. The prefix length of the external IPv6 range.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#accessConfig for access configs.
        # @!attribute [rw] name
        #   @return [::String]
        #     The name of this access configuration. In accessConfigs (IPv4), the default and recommended name is External NAT, but you can use any arbitrary string, such as My external IP or Network Access. In ipv6AccessConfigs, the recommend name is External IPv6.
        # @!attribute [rw] nat_i_p
        #   @return [::String]
        #     Applies to accessConfigs (IPv4) only. An external IP address associated with this instance. Specify an unused static external IP address available to the project or leave this field undefined to use an IP from a shared ephemeral IP address pool. If you specify a static external IP address, it must live in the same region as the zone of the instance.
        # @!attribute [rw] network_tier
        #   @return [::String]
        #     This signifies the networking tier used for configuring this access configuration and can only take the following values: PREMIUM, STANDARD. If an AccessConfig is specified without a valid external IP address, an ephemeral IP will be created with this networkTier. If an AccessConfig with a valid external IP address is specified, it must match that of the networkTier associated with the Address resource owning that IP.
        #     Check the NetworkTier enum for the list of possible values.
        # @!attribute [rw] public_ptr_domain_name
        #   @return [::String]
        #     The DNS domain name for the public PTR record. You can set this field only if the `setPublicPtr` field is enabled in accessConfig. If this field is unspecified in ipv6AccessConfig, a default PTR record will be createc for first IP in associated external IPv6 range.
        # @!attribute [rw] security_policy
        #   @return [::String]
        #     [Output Only] The resource URL for the security policy associated with this access config.
        # @!attribute [rw] set_public_ptr
        #   @return [::Boolean]
        #     Specifies whether a public DNS 'PTR' record should be created to map the external IP address of the instance to a DNS domain name. This field is not used in ipv6AccessConfig. A default PTR record will be created if the VM has external IPv6 range associated.
        # @!attribute [rw] type
        #   @return [::String]
        #     The type of configuration. In accessConfigs (IPv4), the default and only option is ONE_TO_ONE_NAT. In ipv6AccessConfigs, the default and only option is DIRECT_IPV6.
        #     Check the Type enum for the list of possible values.
        class AccessConfig
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # This signifies the networking tier used for configuring this access configuration and can only take the following values: PREMIUM, STANDARD. If an AccessConfig is specified without a valid external IP address, an ephemeral IP will be created with this networkTier. If an AccessConfig with a valid external IP address is specified, it must match that of the networkTier associated with the Address resource owning that IP.
          module NetworkTier
            # A value indicating that the enum field is not set.
            UNDEFINED_NETWORK_TIER = 0

            # Public internet quality with fixed bandwidth.
            FIXED_STANDARD = 310_464_328

            # High quality, Google-grade network tier, support for all networking products.
            PREMIUM = 399_530_551

            # Public internet quality, only limited support for other networking products.
            STANDARD = 484_642_493

            # (Output only) Temporary tier for FIXED_STANDARD when fixed standard tier is expired or not configured.
            STANDARD_OVERRIDES_FIXED_STANDARD = 465_847_234
          end

          # The type of configuration. In accessConfigs (IPv4), the default and only option is ONE_TO_ONE_NAT. In ipv6AccessConfigs, the default and only option is DIRECT_IPV6.
          module Type
            # A value indicating that the enum field is not set.
            UNDEFINED_TYPE = 0

            DIRECT_IPV6 = 4_397_213

            ONE_TO_ONE_NAT = 84_090_205
          end
        end

        # A request message for Instances.AddAccessConfig. See the method description for details.
        # @!attribute [rw] access_config_resource
        #   @return [::Google::Cloud::Compute::V1::AccessConfig]
        #     The body resource for this request
        # @!attribute [rw] instance
        #   @return [::String]
        #     The instance name for this request.
        # @!attribute [rw] network_interface
        #   @return [::String]
        #     The name of the network interface to add to this instance.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class AddAccessConfigInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for FirewallPolicies.AddAssociation. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to update.
        # @!attribute [rw] firewall_policy_association_resource
        #   @return [::Google::Cloud::Compute::V1::FirewallPolicyAssociation]
        #     The body resource for this request
        # @!attribute [rw] replace_existing_association
        #   @return [::Boolean]
        #     Indicates whether or not to replace it if an association of the attachment already exists. This is false by default, in which case an error will be returned if an association already exists.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class AddAssociationFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkFirewallPolicies.AddAssociation. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to update.
        # @!attribute [rw] firewall_policy_association_resource
        #   @return [::Google::Cloud::Compute::V1::FirewallPolicyAssociation]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] replace_existing_association
        #   @return [::Boolean]
        #     Indicates whether or not to replace it if an association of the attachment already exists. This is false by default, in which case an error will be returned if an association already exists.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class AddAssociationNetworkFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionNetworkFirewallPolicies.AddAssociation. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to update.
        # @!attribute [rw] firewall_policy_association_resource
        #   @return [::Google::Cloud::Compute::V1::FirewallPolicyAssociation]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] replace_existing_association
        #   @return [::Boolean]
        #     Indicates whether or not to replace it if an association already exists. This is false by default, in which case an error will be returned if an association already exists.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class AddAssociationRegionNetworkFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetPools.AddHealthCheck. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_pool
        #   @return [::String]
        #     Name of the target pool to add a health check to.
        # @!attribute [rw] target_pools_add_health_check_request_resource
        #   @return [::Google::Cloud::Compute::V1::TargetPoolsAddHealthCheckRequest]
        #     The body resource for this request
        class AddHealthCheckTargetPoolRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetPools.AddInstance. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_pool
        #   @return [::String]
        #     Name of the TargetPool resource to add instances to.
        # @!attribute [rw] target_pools_add_instance_request_resource
        #   @return [::Google::Cloud::Compute::V1::TargetPoolsAddInstanceRequest]
        #     The body resource for this request
        class AddInstanceTargetPoolRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceGroups.AddInstances. See the method description for details.
        # @!attribute [rw] instance_group
        #   @return [::String]
        #     The name of the instance group where you are adding instances.
        # @!attribute [rw] instance_groups_add_instances_request_resource
        #   @return [::Google::Cloud::Compute::V1::InstanceGroupsAddInstancesRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where the instance group is located.
        class AddInstancesInstanceGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NodeGroups.AddNodes. See the method description for details.
        # @!attribute [rw] node_group
        #   @return [::String]
        #     Name of the NodeGroup resource.
        # @!attribute [rw] node_groups_add_nodes_request_resource
        #   @return [::Google::Cloud::Compute::V1::NodeGroupsAddNodesRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class AddNodesNodeGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Networks.AddPeering. See the method description for details.
        # @!attribute [rw] network
        #   @return [::String]
        #     Name of the network resource to add peering to.
        # @!attribute [rw] networks_add_peering_request_resource
        #   @return [::Google::Cloud::Compute::V1::NetworksAddPeeringRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class AddPeeringNetworkRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Disks.AddResourcePolicies. See the method description for details.
        # @!attribute [rw] disk
        #   @return [::String]
        #     The disk name for this request.
        # @!attribute [rw] disks_add_resource_policies_request_resource
        #   @return [::Google::Cloud::Compute::V1::DisksAddResourcePoliciesRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class AddResourcePoliciesDiskRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.AddResourcePolicies. See the method description for details.
        # @!attribute [rw] instance
        #   @return [::String]
        #     The instance name for this request.
        # @!attribute [rw] instances_add_resource_policies_request_resource
        #   @return [::Google::Cloud::Compute::V1::InstancesAddResourcePoliciesRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class AddResourcePoliciesInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionDisks.AddResourcePolicies. See the method description for details.
        # @!attribute [rw] disk
        #   @return [::String]
        #     The disk name for this request.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] region_disks_add_resource_policies_request_resource
        #   @return [::Google::Cloud::Compute::V1::RegionDisksAddResourcePoliciesRequest]
        #     The body resource for this request
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class AddResourcePoliciesRegionDiskRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for FirewallPolicies.AddRule. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to update.
        # @!attribute [rw] firewall_policy_rule_resource
        #   @return [::Google::Cloud::Compute::V1::FirewallPolicyRule]
        #     The body resource for this request
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class AddRuleFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkFirewallPolicies.AddRule. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to update.
        # @!attribute [rw] firewall_policy_rule_resource
        #   @return [::Google::Cloud::Compute::V1::FirewallPolicyRule]
        #     The body resource for this request
        # @!attribute [rw] max_priority
        #   @return [::Integer]
        #     When rule.priority is not specified, auto choose a unused priority between minPriority and maxPriority>. This field is exclusive with rule.priority.
        # @!attribute [rw] min_priority
        #   @return [::Integer]
        #     When rule.priority is not specified, auto choose a unused priority between minPriority and maxPriority>. This field is exclusive with rule.priority.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class AddRuleNetworkFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionNetworkFirewallPolicies.AddRule. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to update.
        # @!attribute [rw] firewall_policy_rule_resource
        #   @return [::Google::Cloud::Compute::V1::FirewallPolicyRule]
        #     The body resource for this request
        # @!attribute [rw] max_priority
        #   @return [::Integer]
        #     When rule.priority is not specified, auto choose a unused priority between minPriority and maxPriority>. This field is exclusive with rule.priority.
        # @!attribute [rw] min_priority
        #   @return [::Integer]
        #     When rule.priority is not specified, auto choose a unused priority between minPriority and maxPriority>. This field is exclusive with rule.priority.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class AddRuleRegionNetworkFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionSecurityPolicies.AddRule. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] security_policy
        #   @return [::String]
        #     Name of the security policy to update.
        # @!attribute [rw] security_policy_rule_resource
        #   @return [::Google::Cloud::Compute::V1::SecurityPolicyRule]
        #     The body resource for this request
        # @!attribute [rw] validate_only
        #   @return [::Boolean]
        #     If true, the request will not be committed.
        class AddRuleRegionSecurityPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for SecurityPolicies.AddRule. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] security_policy
        #   @return [::String]
        #     Name of the security policy to update.
        # @!attribute [rw] security_policy_rule_resource
        #   @return [::Google::Cloud::Compute::V1::SecurityPolicyRule]
        #     The body resource for this request
        # @!attribute [rw] validate_only
        #   @return [::Boolean]
        #     If true, the request will not be committed.
        class AddRuleSecurityPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for BackendBuckets.AddSignedUrlKey. See the method description for details.
        # @!attribute [rw] backend_bucket
        #   @return [::String]
        #     Name of the BackendBucket resource to which the Signed URL Key should be added. The name should conform to RFC1035.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] signed_url_key_resource
        #   @return [::Google::Cloud::Compute::V1::SignedUrlKey]
        #     The body resource for this request
        class AddSignedUrlKeyBackendBucketRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for BackendServices.AddSignedUrlKey. See the method description for details.
        # @!attribute [rw] backend_service
        #   @return [::String]
        #     Name of the BackendService resource to which the Signed URL Key should be added. The name should conform to RFC1035.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] signed_url_key_resource
        #   @return [::Google::Cloud::Compute::V1::SignedUrlKey]
        #     The body resource for this request
        class AddSignedUrlKeyBackendServiceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents an IP Address resource. Google Compute Engine has two IP Address resources: * [Global (external and internal)](https://cloud.google.com/compute/docs/reference/rest/v1/globalAddresses) * [Regional (external and internal)](https://cloud.google.com/compute/docs/reference/rest/v1/addresses) For more information, see Reserving a static external IP address.
        # @!attribute [rw] address
        #   @return [::String]
        #     The static IP address represented by this resource.
        # @!attribute [rw] address_type
        #   @return [::String]
        #     The type of address to reserve, either INTERNAL or EXTERNAL. If unspecified, defaults to EXTERNAL.
        #     Check the AddressType enum for the list of possible values.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this field when you create the resource.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] ip_version
        #   @return [::String]
        #     The IP version that will be used by this address. Valid options are IPV4 or IPV6.
        #     Check the IpVersion enum for the list of possible values.
        # @!attribute [rw] ipv6_endpoint_type
        #   @return [::String]
        #     The endpoint type of this address, which should be VM or NETLB. This is used for deciding which type of endpoint this address can be used after the external IPv6 address reservation.
        #     Check the Ipv6EndpointType enum for the list of possible values.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#address for addresses.
        # @!attribute [rw] label_fingerprint
        #   @return [::String]
        #     A fingerprint for the labels being applied to this Address, which is essentially a hash of the labels set used for optimistic locking. The fingerprint is initially generated by Compute Engine and changes after every request to modify or update labels. You must always provide an up-to-date fingerprint hash in order to update or change labels, otherwise the request will fail with error 412 conditionNotMet. To see the latest fingerprint, make a get() request to retrieve an Address.
        # @!attribute [rw] labels
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     Labels for this resource. These can only be added or modified by the setLabels method. Each label key/value pair must comply with RFC1035. Label values may be empty.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?`. The first character must be a lowercase letter, and all following characters (except for the last character) must be a dash, lowercase letter, or digit. The last character must be a lowercase letter or digit.
        # @!attribute [rw] network
        #   @return [::String]
        #     The URL of the network in which to reserve the address. This field can only be used with INTERNAL type with the VPC_PEERING purpose.
        # @!attribute [rw] network_tier
        #   @return [::String]
        #     This signifies the networking tier used for configuring this address and can only take the following values: PREMIUM or STANDARD. Internal IP addresses are always Premium Tier; global external IP addresses are always Premium Tier; regional external IP addresses can be either Standard or Premium Tier. If this field is not specified, it is assumed to be PREMIUM.
        #     Check the NetworkTier enum for the list of possible values.
        # @!attribute [rw] prefix_length
        #   @return [::Integer]
        #     The prefix length if the resource represents an IP range.
        # @!attribute [rw] purpose
        #   @return [::String]
        #     The purpose of this resource, which can be one of the following values: - GCE_ENDPOINT for addresses that are used by VM instances, alias IP ranges, load balancers, and similar resources. - DNS_RESOLVER for a DNS resolver address in a subnetwork for a Cloud DNS inbound forwarder IP addresses (regional internal IP address in a subnet of a VPC network) - VPC_PEERING for global internal IP addresses used for private services access allocated ranges. - NAT_AUTO for the regional external IP addresses used by Cloud NAT when allocating addresses using automatic NAT IP address allocation. - IPSEC_INTERCONNECT for addresses created from a private IP range that are reserved for a VLAN attachment in an *HA VPN over Cloud Interconnect* configuration. These addresses are regional resources. - `SHARED_LOADBALANCER_VIP` for an internal IP address that is assigned to multiple internal forwarding rules. - `PRIVATE_SERVICE_CONNECT` for a private network address that is used to configure Private Service Connect. Only global internal addresses can use this purpose.
        #     Check the Purpose enum for the list of possible values.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] The URL of the region where a regional address resides. For regional addresses, you must specify the region as a path parameter in the HTTP request URL. *This field is not applicable to global addresses.*
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] status
        #   @return [::String]
        #     [Output Only] The status of the address, which can be one of RESERVING, RESERVED, or IN_USE. An address that is RESERVING is currently in the process of being reserved. A RESERVED address is currently reserved and available to use. An IN_USE address is currently being used by another resource and is not available.
        #     Check the Status enum for the list of possible values.
        # @!attribute [rw] subnetwork
        #   @return [::String]
        #     The URL of the subnetwork in which to reserve the address. If an IP address is specified, it must be within the subnetwork's IP range. This field can only be used with INTERNAL type with a GCE_ENDPOINT or DNS_RESOLVER purpose.
        # @!attribute [rw] users
        #   @return [::Array<::String>]
        #     [Output Only] The URLs of the resources that are using this address.
        class Address
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class LabelsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # The type of address to reserve, either INTERNAL or EXTERNAL. If unspecified, defaults to EXTERNAL.
          module AddressType
            # A value indicating that the enum field is not set.
            UNDEFINED_ADDRESS_TYPE = 0

            # A publicly visible external IP address.
            EXTERNAL = 35_607_499

            # A private network IP address, for use with an Instance or Internal Load Balancer forwarding rule.
            INTERNAL = 279_295_677

            UNSPECIFIED_TYPE = 53_933_922
          end

          # The IP version that will be used by this address. Valid options are IPV4 or IPV6.
          module IpVersion
            # A value indicating that the enum field is not set.
            UNDEFINED_IP_VERSION = 0

            IPV4 = 2_254_341

            IPV6 = 2_254_343

            UNSPECIFIED_VERSION = 21_850_000
          end

          # The endpoint type of this address, which should be VM or NETLB. This is used for deciding which type of endpoint this address can be used after the external IPv6 address reservation.
          module Ipv6EndpointType
            # A value indicating that the enum field is not set.
            UNDEFINED_IPV6_ENDPOINT_TYPE = 0

            # Reserved IPv6 address can be used on network load balancer.
            NETLB = 74_173_363

            # Reserved IPv6 address can be used on VM.
            VM = 2743
          end

          # This signifies the networking tier used for configuring this address and can only take the following values: PREMIUM or STANDARD. Internal IP addresses are always Premium Tier; global external IP addresses are always Premium Tier; regional external IP addresses can be either Standard or Premium Tier. If this field is not specified, it is assumed to be PREMIUM.
          module NetworkTier
            # A value indicating that the enum field is not set.
            UNDEFINED_NETWORK_TIER = 0

            # Public internet quality with fixed bandwidth.
            FIXED_STANDARD = 310_464_328

            # High quality, Google-grade network tier, support for all networking products.
            PREMIUM = 399_530_551

            # Public internet quality, only limited support for other networking products.
            STANDARD = 484_642_493

            # (Output only) Temporary tier for FIXED_STANDARD when fixed standard tier is expired or not configured.
            STANDARD_OVERRIDES_FIXED_STANDARD = 465_847_234
          end

          # The purpose of this resource, which can be one of the following values: - GCE_ENDPOINT for addresses that are used by VM instances, alias IP ranges, load balancers, and similar resources. - DNS_RESOLVER for a DNS resolver address in a subnetwork for a Cloud DNS inbound forwarder IP addresses (regional internal IP address in a subnet of a VPC network) - VPC_PEERING for global internal IP addresses used for private services access allocated ranges. - NAT_AUTO for the regional external IP addresses used by Cloud NAT when allocating addresses using automatic NAT IP address allocation. - IPSEC_INTERCONNECT for addresses created from a private IP range that are reserved for a VLAN attachment in an *HA VPN over Cloud Interconnect* configuration. These addresses are regional resources. - `SHARED_LOADBALANCER_VIP` for an internal IP address that is assigned to multiple internal forwarding rules. - `PRIVATE_SERVICE_CONNECT` for a private network address that is used to configure Private Service Connect. Only global internal addresses can use this purpose.
          module Purpose
            # A value indicating that the enum field is not set.
            UNDEFINED_PURPOSE = 0

            # DNS resolver address in the subnetwork.
            DNS_RESOLVER = 476_114_556

            # VM internal/alias IP, Internal LB service IP, etc.
            GCE_ENDPOINT = 230_515_243

            # A regional internal IP address range reserved for the VLAN attachment that is used in HA VPN over Cloud Interconnect. This regional internal IP address range must not overlap with any IP address range of subnet/route in the VPC network and its peering networks. After the VLAN attachment is created with the reserved IP address range, when creating a new VPN gateway, its interface IP address is allocated from the associated VLAN attachment’s IP address range.
            IPSEC_INTERCONNECT = 340_437_251

            # External IP automatically reserved for Cloud NAT.
            NAT_AUTO = 163_666_477

            # A private network IP address that can be used to configure Private Service Connect. This purpose can be specified only for GLOBAL addresses of Type INTERNAL
            PRIVATE_SERVICE_CONNECT = 48_134_724

            # A regional internal IP address range reserved for Serverless.
            SERVERLESS = 270_492_508

            # A private network IP address that can be shared by multiple Internal Load Balancer forwarding rules.
            SHARED_LOADBALANCER_VIP = 294_447_572

            # IP range for peer networks.
            VPC_PEERING = 400_800_170
          end

          # [Output Only] The status of the address, which can be one of RESERVING, RESERVED, or IN_USE. An address that is RESERVING is currently in the process of being reserved. A RESERVED address is currently reserved and available to use. An IN_USE address is currently being used by another resource and is not available.
          module Status
            # A value indicating that the enum field is not set.
            UNDEFINED_STATUS = 0

            # Address is being used by another resource and is not available.
            IN_USE = 17_393_485

            # Address is reserved and available to use.
            RESERVED = 432_241_448

            # Address is being reserved.
            RESERVING = 514_587_225
          end
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::AddressesScopedList}]
        #     A list of AddressesScopedList resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#addressAggregatedList for aggregated lists of addresses.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class AddressAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::AddressesScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # Contains a list of addresses.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::Address>]
        #     A list of Address resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#addressList for lists of addresses.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class AddressList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] addresses
        #   @return [::Array<::Google::Cloud::Compute::V1::Address>]
        #     [Output Only] A list of addresses contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning which replaces the list of addresses when the list is empty.
        class AddressesScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Specifies options for controlling advanced machine features. Options that would traditionally be configured in a BIOS belong here. Features that require operating system support may have corresponding entries in the GuestOsFeatures of an Image (e.g., whether or not the OS in the Image supports nested virtualization being enabled or disabled).
        # @!attribute [rw] enable_nested_virtualization
        #   @return [::Boolean]
        #     Whether to enable nested virtualization or not (default is false).
        # @!attribute [rw] enable_uefi_networking
        #   @return [::Boolean]
        #     Whether to enable UEFI networking for instance creation.
        # @!attribute [rw] threads_per_core
        #   @return [::Integer]
        #     The number of threads per physical core. To disable simultaneous multithreading (SMT) set this to 1. If unset, the maximum number of threads supported per core by the underlying processor is assumed.
        # @!attribute [rw] visible_core_count
        #   @return [::Integer]
        #     The number of physical cores to expose to an instance. Multiply by the number of threads per core to compute the total number of virtual CPUs to expose to the instance. If unset, the number of cores is inferred from the instance's nominal CPU count and the underlying platform's SMT width.
        class AdvancedMachineFeatures
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for AcceleratorTypes.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListAcceleratorTypesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Addresses.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListAddressesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Autoscalers.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListAutoscalersRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for BackendServices.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Name of the project scoping this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListBackendServicesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for DiskTypes.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListDiskTypesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Disks.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListDisksRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ForwardingRules.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListForwardingRulesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for GlobalOperations.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListGlobalOperationsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for HealthChecks.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Name of the project scoping this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListHealthChecksRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceGroupManagers.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListInstanceGroupManagersRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceGroups.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListInstanceGroupsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceTemplates.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Name of the project scoping this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListInstanceTemplatesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListInstancesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InterconnectAttachments.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListInterconnectAttachmentsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for MachineTypes.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListMachineTypesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkAttachments.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListNetworkAttachmentsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkEdgeSecurityServices.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Name of the project scoping this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListNetworkEdgeSecurityServicesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkEndpointGroups.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListNetworkEndpointGroupsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NodeGroups.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListNodeGroupsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NodeTemplates.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListNodeTemplatesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NodeTypes.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListNodeTypesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for PacketMirrorings.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListPacketMirroringsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for PublicDelegatedPrefixes.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Name of the project scoping this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListPublicDelegatedPrefixesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionCommitments.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListRegionCommitmentsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Reservations.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListReservationsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ResourcePolicies.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListResourcePoliciesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Routers.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListRoutersRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for SecurityPolicies.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Name of the project scoping this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListSecurityPoliciesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ServiceAttachments.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Name of the project scoping this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListServiceAttachmentsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for SslCertificates.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Name of the project scoping this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListSslCertificatesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for SslPolicies.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Name of the project scoping this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListSslPoliciesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Subnetworks.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListSubnetworksRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetHttpProxies.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Name of the project scoping this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListTargetHttpProxiesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetHttpsProxies.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Name of the project scoping this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListTargetHttpsProxiesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetInstances.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListTargetInstancesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetPools.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListTargetPoolsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetTcpProxies.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Name of the project scoping this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListTargetTcpProxiesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetVpnGateways.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListTargetVpnGatewaysRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for UrlMaps.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Name of the project scoping this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListUrlMapsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for VpnGateways.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListVpnGatewaysRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for VpnTunnels.AggregatedList. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] include_all_scopes
        #   @return [::Boolean]
        #     Indicates whether every visible scope for each scope type (zone, region, global) should be included in the response. For new resource types added after this field, the flag has no effect as new resource types will always include every visible scope for each scope type in response. For resource types which predate this field, if this flag is omitted or false, only scopes of the scope types where the resource type is expected to be found will be included.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] service_project_number
        #   @return [::Integer]
        class AggregatedListVpnTunnelsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # An alias IP range attached to an instance's network interface.
        # @!attribute [rw] ip_cidr_range
        #   @return [::String]
        #     The IP alias ranges to allocate for this interface. This IP CIDR range must belong to the specified subnetwork and cannot contain IP addresses reserved by system or used by other network interfaces. This range may be a single IP address (such as 10.2.3.4), a netmask (such as /24) or a CIDR-formatted string (such as 10.1.2.0/24).
        # @!attribute [rw] subnetwork_range_name
        #   @return [::String]
        #     The name of a subnetwork secondary IP range from which to allocate an IP alias range. If not specified, the primary range of the subnetwork is used.
        class AliasIpRange
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # [Output Only] Contains output only fields.
        # @!attribute [rw] specific_sku_allocation
        #   @return [::Google::Cloud::Compute::V1::AllocationResourceStatusSpecificSKUAllocation]
        #     Allocation Properties of this reservation.
        class AllocationResourceStatus
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Contains Properties set for the reservation.
        # @!attribute [rw] source_instance_template_id
        #   @return [::String]
        #     ID of the instance template used to populate reservation properties.
        class AllocationResourceStatusSpecificSKUAllocation
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] disk_size_gb
        #   @return [::Integer]
        #     Specifies the size of the disk in base-2 GB.
        # @!attribute [rw] interface
        #   @return [::String]
        #     Specifies the disk interface to use for attaching this disk, which is either SCSI or NVME. The default is SCSI. For performance characteristics of SCSI over NVMe, see Local SSD performance.
        #     Check the Interface enum for the list of possible values.
        class AllocationSpecificSKUAllocationAllocatedInstancePropertiesReservedDisk
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Specifies the disk interface to use for attaching this disk, which is either SCSI or NVME. The default is SCSI. For performance characteristics of SCSI over NVMe, see Local SSD performance.
          module Interface
            # A value indicating that the enum field is not set.
            UNDEFINED_INTERFACE = 0

            NVME = 2_408_800

            SCSI = 2_539_686
          end
        end

        # Properties of the SKU instances being reserved. Next ID: 9
        # @!attribute [rw] guest_accelerators
        #   @return [::Array<::Google::Cloud::Compute::V1::AcceleratorConfig>]
        #     Specifies accelerator type and count.
        # @!attribute [rw] local_ssds
        #   @return [::Array<::Google::Cloud::Compute::V1::AllocationSpecificSKUAllocationAllocatedInstancePropertiesReservedDisk>]
        #     Specifies amount of local ssd to reserve with each instance. The type of disk is local-ssd.
        # @!attribute [rw] location_hint
        #   @return [::String]
        #     An opaque location hint used to place the allocation close to other resources. This field is for use by internal tools that use the public API.
        # @!attribute [rw] machine_type
        #   @return [::String]
        #     Specifies type of machine (name only) which has fixed number of vCPUs and fixed amount of memory. This also includes specifying custom machine type following custom-NUMBER_OF_CPUS-AMOUNT_OF_MEMORY pattern.
        # @!attribute [rw] min_cpu_platform
        #   @return [::String]
        #     Minimum cpu platform the reservation.
        class AllocationSpecificSKUAllocationReservedInstanceProperties
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # This reservation type allows to pre allocate specific instance configuration. Next ID: 6
        # @!attribute [rw] assured_count
        #   @return [::Integer]
        #     [Output Only] Indicates how many instances are actually usable currently.
        # @!attribute [rw] count
        #   @return [::Integer]
        #     Specifies the number of resources that are allocated.
        # @!attribute [rw] in_use_count
        #   @return [::Integer]
        #     [Output Only] Indicates how many instances are in use.
        # @!attribute [rw] instance_properties
        #   @return [::Google::Cloud::Compute::V1::AllocationSpecificSKUAllocationReservedInstanceProperties]
        #     The instance properties for the reservation.
        # @!attribute [rw] source_instance_template
        #   @return [::String]
        #     Specifies the instance template to create the reservation. If you use this field, you must exclude the instanceProperties field. This field is optional, and it can be a full or partial URL. For example, the following are all valid URLs to an instance template: - https://www.googleapis.com/compute/v1/projects/project /global/instanceTemplates/instanceTemplate - projects/project/global/instanceTemplates/instanceTemplate - global/instanceTemplates/instanceTemplate
        class AllocationSpecificSKUReservation
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] I_p_protocol
        #   @return [::String]
        #     The IP protocol to which this rule applies. The protocol type is required when creating a firewall rule. This value can either be one of the following well known protocol strings (tcp, udp, icmp, esp, ah, ipip, sctp) or the IP protocol number.
        # @!attribute [rw] ports
        #   @return [::Array<::String>]
        #     An optional list of ports to which this rule applies. This field is only applicable for the UDP or TCP protocol. Each entry must be either an integer or a range. If not specified, this rule applies to connections through any port. Example inputs include: ["22"], ["80","443"], and ["12345-12349"].
        class Allowed
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for PublicAdvertisedPrefixes.Announce. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] public_advertised_prefix
        #   @return [::String]
        #     The name of the public advertised prefix. It should comply with RFC1035.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class AnnouncePublicAdvertisedPrefixeRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for PublicDelegatedPrefixes.Announce. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] public_delegated_prefix
        #   @return [::String]
        #     The name of the public delegated prefix. It should comply with RFC1035.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region where the public delegated prefix is located. It should comply with RFC1035.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class AnnouncePublicDelegatedPrefixeRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceGroupManagers.ApplyUpdatesToInstances. See the method description for details.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     The name of the managed instance group, should conform to RFC1035.
        # @!attribute [rw] instance_group_managers_apply_updates_request_resource
        #   @return [::Google::Cloud::Compute::V1::InstanceGroupManagersApplyUpdatesRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where the managed instance group is located. Should conform to RFC1035.
        class ApplyUpdatesToInstancesInstanceGroupManagerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionInstanceGroupManagers.ApplyUpdatesToInstances. See the method description for details.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     The name of the managed instance group, should conform to RFC1035.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request, should conform to RFC1035.
        # @!attribute [rw] region_instance_group_managers_apply_updates_request_resource
        #   @return [::Google::Cloud::Compute::V1::RegionInstanceGroupManagersApplyUpdatesRequest]
        #     The body resource for this request
        class ApplyUpdatesToInstancesRegionInstanceGroupManagerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.AttachDisk. See the method description for details.
        # @!attribute [rw] attached_disk_resource
        #   @return [::Google::Cloud::Compute::V1::AttachedDisk]
        #     The body resource for this request
        # @!attribute [rw] force_attach
        #   @return [::Boolean]
        #     Whether to force attach the regional disk even if it's currently attached to another instance. If you try to force attach a zonal disk to an instance, you will receive an error.
        # @!attribute [rw] instance
        #   @return [::String]
        #     The instance name for this request.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class AttachDiskInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for GlobalNetworkEndpointGroups.AttachNetworkEndpoints. See the method description for details.
        # @!attribute [rw] global_network_endpoint_groups_attach_endpoints_request_resource
        #   @return [::Google::Cloud::Compute::V1::GlobalNetworkEndpointGroupsAttachEndpointsRequest]
        #     The body resource for this request
        # @!attribute [rw] network_endpoint_group
        #   @return [::String]
        #     The name of the network endpoint group where you are attaching network endpoints to. It should comply with RFC1035.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class AttachNetworkEndpointsGlobalNetworkEndpointGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkEndpointGroups.AttachNetworkEndpoints. See the method description for details.
        # @!attribute [rw] network_endpoint_group
        #   @return [::String]
        #     The name of the network endpoint group where you are attaching network endpoints to. It should comply with RFC1035.
        # @!attribute [rw] network_endpoint_groups_attach_endpoints_request_resource
        #   @return [::Google::Cloud::Compute::V1::NetworkEndpointGroupsAttachEndpointsRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where the network endpoint group is located. It should comply with RFC1035.
        class AttachNetworkEndpointsNetworkEndpointGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionNetworkEndpointGroups.AttachNetworkEndpoints. See the method description for details.
        # @!attribute [rw] network_endpoint_group
        #   @return [::String]
        #     The name of the network endpoint group where you are attaching network endpoints to. It should comply with RFC1035.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region where you want to create the network endpoint group. It should comply with RFC1035.
        # @!attribute [rw] region_network_endpoint_groups_attach_endpoints_request_resource
        #   @return [::Google::Cloud::Compute::V1::RegionNetworkEndpointGroupsAttachEndpointsRequest]
        #     The body resource for this request
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class AttachNetworkEndpointsRegionNetworkEndpointGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # An instance-attached disk resource.
        # @!attribute [rw] architecture
        #   @return [::String]
        #     [Output Only] The architecture of the attached disk. Valid values are ARM64 or X86_64.
        #     Check the Architecture enum for the list of possible values.
        # @!attribute [rw] auto_delete
        #   @return [::Boolean]
        #     Specifies whether the disk will be auto-deleted when the instance is deleted (but not when the disk is detached from the instance).
        # @!attribute [rw] boot
        #   @return [::Boolean]
        #     Indicates that this is a boot disk. The virtual machine will use the first partition of the disk for its root filesystem.
        # @!attribute [rw] device_name
        #   @return [::String]
        #     Specifies a unique device name of your choice that is reflected into the /dev/disk/by-id/google-* tree of a Linux operating system running within the instance. This name can be used to reference the device for mounting, resizing, and so on, from within the instance. If not specified, the server chooses a default device name to apply to this disk, in the form persistent-disk-x, where x is a number assigned by Google Compute Engine. This field is only applicable for persistent disks.
        # @!attribute [rw] disk_encryption_key
        #   @return [::Google::Cloud::Compute::V1::CustomerEncryptionKey]
        #     Encrypts or decrypts a disk using a customer-supplied encryption key. If you are creating a new disk, this field encrypts the new disk using an encryption key that you provide. If you are attaching an existing disk that is already encrypted, this field decrypts the disk using the customer-supplied encryption key. If you encrypt a disk using a customer-supplied key, you must provide the same key again when you attempt to use this resource at a later time. For example, you must provide the key when you create a snapshot or an image from the disk or when you attach the disk to a virtual machine instance. If you do not provide an encryption key, then the disk will be encrypted using an automatically generated key and you do not need to provide a key to use the disk later. Instance templates do not store customer-supplied encryption keys, so you cannot use your own keys to encrypt disks in a managed instance group.
        # @!attribute [rw] disk_size_gb
        #   @return [::Integer]
        #     The size of the disk in GB.
        # @!attribute [rw] force_attach
        #   @return [::Boolean]
        #     [Input Only] Whether to force attach the regional disk even if it's currently attached to another instance. If you try to force attach a zonal disk to an instance, you will receive an error.
        # @!attribute [rw] guest_os_features
        #   @return [::Array<::Google::Cloud::Compute::V1::GuestOsFeature>]
        #     A list of features to enable on the guest operating system. Applicable only for bootable images. Read Enabling guest operating system features to see a list of available options.
        # @!attribute [rw] index
        #   @return [::Integer]
        #     [Output Only] A zero-based index to this disk, where 0 is reserved for the boot disk. If you have many disks attached to an instance, each disk would have a unique index number.
        # @!attribute [rw] initialize_params
        #   @return [::Google::Cloud::Compute::V1::AttachedDiskInitializeParams]
        #     [Input Only] Specifies the parameters for a new disk that will be created alongside the new instance. Use initialization parameters to create boot disks or local SSDs attached to the new instance. This property is mutually exclusive with the source property; you can only define one or the other, but not both.
        # @!attribute [rw] interface
        #   @return [::String]
        #     Specifies the disk interface to use for attaching this disk, which is either SCSI or NVME. For most machine types, the default is SCSI. Local SSDs can use either NVME or SCSI. In certain configurations, persistent disks can use NVMe. For more information, see About persistent disks.
        #     Check the Interface enum for the list of possible values.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#attachedDisk for attached disks.
        # @!attribute [rw] licenses
        #   @return [::Array<::String>]
        #     [Output Only] Any valid publicly visible licenses.
        # @!attribute [rw] mode
        #   @return [::String]
        #     The mode in which to attach this disk, either READ_WRITE or READ_ONLY. If not specified, the default is to attach the disk in READ_WRITE mode.
        #     Check the Mode enum for the list of possible values.
        # @!attribute [rw] saved_state
        #   @return [::String]
        #     For LocalSSD disks on VM Instances in STOPPED or SUSPENDED state, this field is set to PRESERVED if the LocalSSD data has been saved to a persistent location by customer request. (see the discard_local_ssd option on Stop/Suspend). Read-only in the api.
        #     Check the SavedState enum for the list of possible values.
        # @!attribute [rw] shielded_instance_initial_state
        #   @return [::Google::Cloud::Compute::V1::InitialStateConfig]
        #     [Output Only] shielded vm initial state stored on disk
        # @!attribute [rw] source
        #   @return [::String]
        #     Specifies a valid partial or full URL to an existing Persistent Disk resource. When creating a new instance, one of initializeParams.sourceImage or initializeParams.sourceSnapshot or disks.source is required except for local SSD. If desired, you can also attach existing non-root persistent disks using this property. This field is only applicable for persistent disks. Note that for InstanceTemplate, specify the disk name for zonal disk, and the URL for regional disk.
        # @!attribute [rw] type
        #   @return [::String]
        #     Specifies the type of the disk, either SCRATCH or PERSISTENT. If not specified, the default is PERSISTENT.
        #     Check the Type enum for the list of possible values.
        class AttachedDisk
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # [Output Only] The architecture of the attached disk. Valid values are ARM64 or X86_64.
          module Architecture
            # A value indicating that the enum field is not set.
            UNDEFINED_ARCHITECTURE = 0

            # Default value indicating Architecture is not set.
            ARCHITECTURE_UNSPECIFIED = 394_750_507

            # Machines with architecture ARM64
            ARM64 = 62_547_450

            # Machines with architecture X86_64
            X86_64 = 425_300_551
          end

          # Specifies the disk interface to use for attaching this disk, which is either SCSI or NVME. For most machine types, the default is SCSI. Local SSDs can use either NVME or SCSI. In certain configurations, persistent disks can use NVMe. For more information, see About persistent disks.
          module Interface
            # A value indicating that the enum field is not set.
            UNDEFINED_INTERFACE = 0

            NVME = 2_408_800

            SCSI = 2_539_686
          end

          # The mode in which to attach this disk, either READ_WRITE or READ_ONLY. If not specified, the default is to attach the disk in READ_WRITE mode.
          module Mode
            # A value indicating that the enum field is not set.
            UNDEFINED_MODE = 0

            # Attaches this disk in read-only mode. Multiple virtual machines can use a disk in read-only mode at a time.
            READ_ONLY = 91_950_261

            # *[Default]* Attaches this disk in read-write mode. Only one virtual machine at a time can be attached to a disk in read-write mode.
            READ_WRITE = 173_607_894
          end

          # For LocalSSD disks on VM Instances in STOPPED or SUSPENDED state, this field is set to PRESERVED if the LocalSSD data has been saved to a persistent location by customer request. (see the discard_local_ssd option on Stop/Suspend). Read-only in the api.
          module SavedState
            # A value indicating that the enum field is not set.
            UNDEFINED_SAVED_STATE = 0

            # *[Default]* Disk state has not been preserved.
            DISK_SAVED_STATE_UNSPECIFIED = 391_290_831

            # Disk state has been preserved.
            PRESERVED = 254_159_736
          end

          # Specifies the type of the disk, either SCRATCH or PERSISTENT. If not specified, the default is PERSISTENT.
          module Type
            # A value indicating that the enum field is not set.
            UNDEFINED_TYPE = 0

            PERSISTENT = 460_683_927

            SCRATCH = 496_778_970
          end
        end

        # [Input Only] Specifies the parameters for a new disk that will be created alongside the new instance. Use initialization parameters to create boot disks or local SSDs attached to the new instance. This field is persisted and returned for instanceTemplate and not returned in the context of instance. This property is mutually exclusive with the source property; you can only define one or the other, but not both.
        # @!attribute [rw] architecture
        #   @return [::String]
        #     The architecture of the attached disk. Valid values are arm64 or x86_64.
        #     Check the Architecture enum for the list of possible values.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description. Provide this property when creating the disk.
        # @!attribute [rw] disk_name
        #   @return [::String]
        #     Specifies the disk name. If not specified, the default is to use the name of the instance. If a disk with the same name already exists in the given region, the existing disk is attached to the new instance and the new disk is not created.
        # @!attribute [rw] disk_size_gb
        #   @return [::Integer]
        #     Specifies the size of the disk in base-2 GB. The size must be at least 10 GB. If you specify a sourceImage, which is required for boot disks, the default size is the size of the sourceImage. If you do not specify a sourceImage, the default disk size is 500 GB.
        # @!attribute [rw] disk_type
        #   @return [::String]
        #     Specifies the disk type to use to create the instance. If not specified, the default is pd-standard, specified using the full URL. For example: https://www.googleapis.com/compute/v1/projects/project/zones/zone /diskTypes/pd-standard For a full list of acceptable values, see Persistent disk types. If you specify this field when creating a VM, you can provide either the full or partial URL. For example, the following values are valid: - https://www.googleapis.com/compute/v1/projects/project/zones/zone /diskTypes/diskType - projects/project/zones/zone/diskTypes/diskType - zones/zone/diskTypes/diskType If you specify this field when creating or updating an instance template or all-instances configuration, specify the type of the disk, not the URL. For example: pd-standard.
        # @!attribute [rw] labels
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     Labels to apply to this disk. These can be later modified by the disks.setLabels method. This field is only applicable for persistent disks.
        # @!attribute [rw] licenses
        #   @return [::Array<::String>]
        #     A list of publicly visible licenses. Reserved for Google's use.
        # @!attribute [rw] on_update_action
        #   @return [::String]
        #     Specifies which action to take on instance update with this disk. Default is to use the existing disk.
        #     Check the OnUpdateAction enum for the list of possible values.
        # @!attribute [rw] provisioned_iops
        #   @return [::Integer]
        #     Indicates how many IOPS to provision for the disk. This sets the number of I/O operations per second that the disk can handle. Values must be between 10,000 and 120,000. For more details, see the Extreme persistent disk documentation.
        # @!attribute [rw] provisioned_throughput
        #   @return [::Integer]
        #     Indicates how much throughput to provision for the disk. This sets the number of throughput mb per second that the disk can handle. Values must be between 1 and 7,124.
        # @!attribute [rw] replica_zones
        #   @return [::Array<::String>]
        #     Required for each regional disk associated with the instance. Specify the URLs of the zones where the disk should be replicated to. You must provide exactly two replica zones, and one zone must be the same as the instance zone.
        # @!attribute [rw] resource_manager_tags
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     Resource manager tags to be bound to the disk. Tag keys and values have the same definition as resource manager tags. Keys must be in the format `tagKeys/{tag_key_id}`, and values are in the format `tagValues/456`. The field is ignored (both PUT & PATCH) when empty.
        # @!attribute [rw] resource_policies
        #   @return [::Array<::String>]
        #     Resource policies applied to this disk for automatic snapshot creations. Specified using the full or partial URL. For instance template, specify only the resource policy name.
        # @!attribute [rw] source_image
        #   @return [::String]
        #     The source image to create this disk. When creating a new instance, one of initializeParams.sourceImage or initializeParams.sourceSnapshot or disks.source is required except for local SSD. To create a disk with one of the public operating system images, specify the image by its family name. For example, specify family/debian-9 to use the latest Debian 9 image: projects/debian-cloud/global/images/family/debian-9 Alternatively, use a specific version of a public operating system image: projects/debian-cloud/global/images/debian-9-stretch-vYYYYMMDD To create a disk with a custom image that you created, specify the image name in the following format: global/images/my-custom-image You can also specify a custom image by its image family, which returns the latest version of the image in that family. Replace the image name with family/family-name: global/images/family/my-image-family If the source image is deleted later, this field will not be set.
        # @!attribute [rw] source_image_encryption_key
        #   @return [::Google::Cloud::Compute::V1::CustomerEncryptionKey]
        #     The customer-supplied encryption key of the source image. Required if the source image is protected by a customer-supplied encryption key. InstanceTemplate and InstancePropertiesPatch do not store customer-supplied encryption keys, so you cannot create disks for instances in a managed instance group if the source images are encrypted with your own keys.
        # @!attribute [rw] source_snapshot
        #   @return [::String]
        #     The source snapshot to create this disk. When creating a new instance, one of initializeParams.sourceSnapshot or initializeParams.sourceImage or disks.source is required except for local SSD. To create a disk with a snapshot that you created, specify the snapshot name in the following format: global/snapshots/my-backup If the source snapshot is deleted later, this field will not be set.
        # @!attribute [rw] source_snapshot_encryption_key
        #   @return [::Google::Cloud::Compute::V1::CustomerEncryptionKey]
        #     The customer-supplied encryption key of the source snapshot.
        class AttachedDiskInitializeParams
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class LabelsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class ResourceManagerTagsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # The architecture of the attached disk. Valid values are arm64 or x86_64.
          module Architecture
            # A value indicating that the enum field is not set.
            UNDEFINED_ARCHITECTURE = 0

            # Default value indicating Architecture is not set.
            ARCHITECTURE_UNSPECIFIED = 394_750_507

            # Machines with architecture ARM64
            ARM64 = 62_547_450

            # Machines with architecture X86_64
            X86_64 = 425_300_551
          end

          # Specifies which action to take on instance update with this disk. Default is to use the existing disk.
          module OnUpdateAction
            # A value indicating that the enum field is not set.
            UNDEFINED_ON_UPDATE_ACTION = 0

            # Always recreate the disk.
            RECREATE_DISK = 494_767_853

            # Recreate the disk if source (image, snapshot) of this disk is different from source of existing disk.
            RECREATE_DISK_IF_SOURCE_CHANGED = 398_099_712

            # Use the existing disk, this is the default behaviour.
            USE_EXISTING_DISK = 232_682_233
          end
        end

        # Specifies the audit configuration for a service. The configuration determines which permission types are logged, and what identities, if any, are exempted from logging. An AuditConfig must have one or more AuditLogConfigs. If there are AuditConfigs for both `allServices` and a specific service, the union of the two AuditConfigs is used for that service: the log_types specified in each AuditConfig are enabled, and the exempted_members in each AuditLogConfig are exempted. Example Policy with multiple AuditConfigs: { "audit_configs": [ { "service": "allServices", "audit_log_configs": [ { "log_type": "DATA_READ", "exempted_members": [ "user:jose@example.com" ] }, { "log_type": "DATA_WRITE" }, { "log_type": "ADMIN_READ" } ] }, { "service": "sampleservice.googleapis.com", "audit_log_configs": [ { "log_type": "DATA_READ" }, { "log_type": "DATA_WRITE", "exempted_members": [ "user:aliya@example.com" ] } ] } ] } For sampleservice, this policy enables DATA_READ, DATA_WRITE and ADMIN_READ logging. It also exempts jose@example.com from DATA_READ logging, and aliya@example.com from DATA_WRITE logging.
        # @!attribute [rw] audit_log_configs
        #   @return [::Array<::Google::Cloud::Compute::V1::AuditLogConfig>]
        #     The configuration for logging of each type of permission.
        # @!attribute [rw] exempted_members
        #   @return [::Array<::String>]
        #     This is deprecated and has no effect. Do not use.
        # @!attribute [rw] service
        #   @return [::String]
        #     Specifies a service that will be enabled for audit logging. For example, `storage.googleapis.com`, `cloudsql.googleapis.com`. `allServices` is a special value that covers all services.
        class AuditConfig
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Provides the configuration for logging a type of permissions. Example: { "audit_log_configs": [ { "log_type": "DATA_READ", "exempted_members": [ "user:jose@example.com" ] }, { "log_type": "DATA_WRITE" } ] } This enables 'DATA_READ' and 'DATA_WRITE' logging, while exempting jose@example.com from DATA_READ logging.
        # @!attribute [rw] exempted_members
        #   @return [::Array<::String>]
        #     Specifies the identities that do not cause logging for this type of permission. Follows the same format of Binding.members.
        # @!attribute [rw] ignore_child_exemptions
        #   @return [::Boolean]
        #     This is deprecated and has no effect. Do not use.
        # @!attribute [rw] log_type
        #   @return [::String]
        #     The log type that this config enables.
        #     Check the LogType enum for the list of possible values.
        class AuditLogConfig
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The log type that this config enables.
          module LogType
            # A value indicating that the enum field is not set.
            UNDEFINED_LOG_TYPE = 0

            # Admin reads. Example: CloudIAM getIamPolicy
            ADMIN_READ = 128_951_462

            # Data reads. Example: CloudSQL Users list
            DATA_READ = 305_224_971

            # Data writes. Example: CloudSQL Users create
            DATA_WRITE = 340_181_738

            # Default case. Should never be this.
            LOG_TYPE_UNSPECIFIED = 154_527_053
          end
        end

        # This is deprecated and has no effect. Do not use.
        # @!attribute [rw] permission_type
        #   @return [::String]
        #     This is deprecated and has no effect. Do not use.
        #     Check the PermissionType enum for the list of possible values.
        class AuthorizationLoggingOptions
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # This is deprecated and has no effect. Do not use.
          module PermissionType
            # A value indicating that the enum field is not set.
            UNDEFINED_PERMISSION_TYPE = 0

            # This is deprecated and has no effect. Do not use.
            ADMIN_READ = 128_951_462

            # This is deprecated and has no effect. Do not use.
            ADMIN_WRITE = 244_412_079

            # This is deprecated and has no effect. Do not use.
            DATA_READ = 305_224_971

            # This is deprecated and has no effect. Do not use.
            DATA_WRITE = 340_181_738

            # This is deprecated and has no effect. Do not use.
            PERMISSION_TYPE_UNSPECIFIED = 440_313_346
          end
        end

        # Represents an Autoscaler resource. Google Compute Engine has two Autoscaler resources: * [Zonal](/compute/docs/reference/rest/v1/autoscalers) * [Regional](/compute/docs/reference/rest/v1/regionAutoscalers) Use autoscalers to automatically add or delete instances from a managed instance group according to your defined autoscaling policy. For more information, read Autoscaling Groups of Instances. For zonal managed instance groups resource, use the autoscaler resource. For regional managed instance groups, use the regionAutoscalers resource.
        # @!attribute [rw] autoscaling_policy
        #   @return [::Google::Cloud::Compute::V1::AutoscalingPolicy]
        #     The configuration parameters for the autoscaling algorithm. You can define one or more signals for an autoscaler: cpuUtilization, customMetricUtilizations, and loadBalancingUtilization. If none of these are specified, the default will be to autoscale based on cpuUtilization to 0.6 or 60%.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#autoscaler for autoscalers.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] recommended_size
        #   @return [::Integer]
        #     [Output Only] Target recommended MIG size (number of instances) computed by autoscaler. Autoscaler calculates the recommended MIG size even when the autoscaling policy mode is different from ON. This field is empty when autoscaler is not connected to an existing managed instance group or autoscaler did not generate its prediction.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] URL of the region where the instance group resides (for autoscalers living in regional scope).
        # @!attribute [rw] scaling_schedule_status
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::ScalingScheduleStatus}]
        #     [Output Only] Status information of existing scaling schedules.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] status
        #   @return [::String]
        #     [Output Only] The status of the autoscaler configuration. Current set of possible values: - PENDING: Autoscaler backend hasn't read new/updated configuration. - DELETING: Configuration is being deleted. - ACTIVE: Configuration is acknowledged to be effective. Some warnings might be present in the statusDetails field. - ERROR: Configuration has errors. Actionable for users. Details are present in the statusDetails field. New values might be added in the future.
        #     Check the Status enum for the list of possible values.
        # @!attribute [rw] status_details
        #   @return [::Array<::Google::Cloud::Compute::V1::AutoscalerStatusDetails>]
        #     [Output Only] Human-readable details about the current state of the autoscaler. Read the documentation for Commonly returned status messages for examples of status messages you might encounter.
        # @!attribute [rw] target
        #   @return [::String]
        #     URL of the managed instance group that this autoscaler will scale. This field is required when creating an autoscaler.
        # @!attribute [rw] zone
        #   @return [::String]
        #     [Output Only] URL of the zone where the instance group resides (for autoscalers living in zonal scope).
        class Autoscaler
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::ScalingScheduleStatus]
          class ScalingScheduleStatusEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # [Output Only] The status of the autoscaler configuration. Current set of possible values: - PENDING: Autoscaler backend hasn't read new/updated configuration. - DELETING: Configuration is being deleted. - ACTIVE: Configuration is acknowledged to be effective. Some warnings might be present in the statusDetails field. - ERROR: Configuration has errors. Actionable for users. Details are present in the statusDetails field. New values might be added in the future.
          module Status
            # A value indicating that the enum field is not set.
            UNDEFINED_STATUS = 0

            # Configuration is acknowledged to be effective
            ACTIVE = 314_733_318

            # Configuration is being deleted
            DELETING = 528_602_024

            # Configuration has errors. Actionable for users.
            ERROR = 66_247_144

            # Autoscaler backend hasn't read new/updated configuration
            PENDING = 35_394_935
          end
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::AutoscalersScopedList}]
        #     A list of AutoscalersScopedList resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#autoscalerAggregatedList for aggregated lists of autoscalers.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources. end_interface: MixerListResponseWithEtagBuilder
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class AutoscalerAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::AutoscalersScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # Contains a list of Autoscaler resources.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::Autoscaler>]
        #     A list of Autoscaler resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#autoscalerList for lists of autoscalers.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class AutoscalerList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] message
        #   @return [::String]
        #     The status message.
        # @!attribute [rw] type
        #   @return [::String]
        #     The type of error, warning, or notice returned. Current set of possible values: - ALL_INSTANCES_UNHEALTHY (WARNING): All instances in the instance group are unhealthy (not in RUNNING state). - BACKEND_SERVICE_DOES_NOT_EXIST (ERROR): There is no backend service attached to the instance group. - CAPPED_AT_MAX_NUM_REPLICAS (WARNING): Autoscaler recommends a size greater than maxNumReplicas. - CUSTOM_METRIC_DATA_POINTS_TOO_SPARSE (WARNING): The custom metric samples are not exported often enough to be a credible base for autoscaling. - CUSTOM_METRIC_INVALID (ERROR): The custom metric that was specified does not exist or does not have the necessary labels. - MIN_EQUALS_MAX (WARNING): The minNumReplicas is equal to maxNumReplicas. This means the autoscaler cannot add or remove instances from the instance group. - MISSING_CUSTOM_METRIC_DATA_POINTS (WARNING): The autoscaler did not receive any data from the custom metric configured for autoscaling. - MISSING_LOAD_BALANCING_DATA_POINTS (WARNING): The autoscaler is configured to scale based on a load balancing signal but the instance group has not received any requests from the load balancer. - MODE_OFF (WARNING): Autoscaling is turned off. The number of instances in the group won't change automatically. The autoscaling configuration is preserved. - MODE_ONLY_UP (WARNING): Autoscaling is in the "Autoscale only out" mode. The autoscaler can add instances but not remove any. - MORE_THAN_ONE_BACKEND_SERVICE (ERROR): The instance group cannot be autoscaled because it has more than one backend service attached to it. - NOT_ENOUGH_QUOTA_AVAILABLE (ERROR): There is insufficient quota for the necessary resources, such as CPU or number of instances. - REGION_RESOURCE_STOCKOUT (ERROR): Shown only for regional autoscalers: there is a resource stockout in the chosen region. - SCALING_TARGET_DOES_NOT_EXIST (ERROR): The target to be scaled does not exist. - UNSUPPORTED_MAX_RATE_LOAD_BALANCING_CONFIGURATION (ERROR): Autoscaling does not work with an HTTP/S load balancer that has been configured for maxRate. - ZONE_RESOURCE_STOCKOUT (ERROR): For zonal autoscalers: there is a resource stockout in the chosen zone. For regional autoscalers: in at least one of the zones you're using there is a resource stockout. New values might be added in the future. Some of the values might not be available in all API versions.
        #     Check the Type enum for the list of possible values.
        class AutoscalerStatusDetails
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The type of error, warning, or notice returned. Current set of possible values: - ALL_INSTANCES_UNHEALTHY (WARNING): All instances in the instance group are unhealthy (not in RUNNING state). - BACKEND_SERVICE_DOES_NOT_EXIST (ERROR): There is no backend service attached to the instance group. - CAPPED_AT_MAX_NUM_REPLICAS (WARNING): Autoscaler recommends a size greater than maxNumReplicas. - CUSTOM_METRIC_DATA_POINTS_TOO_SPARSE (WARNING): The custom metric samples are not exported often enough to be a credible base for autoscaling. - CUSTOM_METRIC_INVALID (ERROR): The custom metric that was specified does not exist or does not have the necessary labels. - MIN_EQUALS_MAX (WARNING): The minNumReplicas is equal to maxNumReplicas. This means the autoscaler cannot add or remove instances from the instance group. - MISSING_CUSTOM_METRIC_DATA_POINTS (WARNING): The autoscaler did not receive any data from the custom metric configured for autoscaling. - MISSING_LOAD_BALANCING_DATA_POINTS (WARNING): The autoscaler is configured to scale based on a load balancing signal but the instance group has not received any requests from the load balancer. - MODE_OFF (WARNING): Autoscaling is turned off. The number of instances in the group won't change automatically. The autoscaling configuration is preserved. - MODE_ONLY_UP (WARNING): Autoscaling is in the "Autoscale only out" mode. The autoscaler can add instances but not remove any. - MORE_THAN_ONE_BACKEND_SERVICE (ERROR): The instance group cannot be autoscaled because it has more than one backend service attached to it. - NOT_ENOUGH_QUOTA_AVAILABLE (ERROR): There is insufficient quota for the necessary resources, such as CPU or number of instances. - REGION_RESOURCE_STOCKOUT (ERROR): Shown only for regional autoscalers: there is a resource stockout in the chosen region. - SCALING_TARGET_DOES_NOT_EXIST (ERROR): The target to be scaled does not exist. - UNSUPPORTED_MAX_RATE_LOAD_BALANCING_CONFIGURATION (ERROR): Autoscaling does not work with an HTTP/S load balancer that has been configured for maxRate. - ZONE_RESOURCE_STOCKOUT (ERROR): For zonal autoscalers: there is a resource stockout in the chosen zone. For regional autoscalers: in at least one of the zones you're using there is a resource stockout. New values might be added in the future. Some of the values might not be available in all API versions.
          module Type
            # A value indicating that the enum field is not set.
            UNDEFINED_TYPE = 0

            # All instances in the instance group are unhealthy (not in RUNNING state).
            ALL_INSTANCES_UNHEALTHY = 404_965_477

            # There is no backend service attached to the instance group.
            BACKEND_SERVICE_DOES_NOT_EXIST = 191_417_626

            # Autoscaler recommends a size greater than maxNumReplicas.
            CAPPED_AT_MAX_NUM_REPLICAS = 518_617

            # The custom metric samples are not exported often enough to be a credible base for autoscaling.
            CUSTOM_METRIC_DATA_POINTS_TOO_SPARSE = 328_964_659

            # The custom metric that was specified does not exist or does not have the necessary labels.
            CUSTOM_METRIC_INVALID = 204_430_550

            # The minNumReplicas is equal to maxNumReplicas. This means the autoscaler cannot add or remove instances from the instance group.
            MIN_EQUALS_MAX = 2_821_361

            # The autoscaler did not receive any data from the custom metric configured for autoscaling.
            MISSING_CUSTOM_METRIC_DATA_POINTS = 94_885_086

            # The autoscaler is configured to scale based on a load balancing signal but the instance group has not received any requests from the load balancer.
            MISSING_LOAD_BALANCING_DATA_POINTS = 509_858_898

            # Autoscaling is turned off. The number of instances in the group won't change automatically. The autoscaling configuration is preserved.
            MODE_OFF = 164_169_907

            # Autoscaling is in the "Autoscale only scale out" mode. Instances in the group will be only added.
            MODE_ONLY_SCALE_OUT = 3_840_994

            # Autoscaling is in the "Autoscale only out" mode. Instances in the group will be only added.
            MODE_ONLY_UP = 100_969_842

            # The instance group cannot be autoscaled because it has more than one backend service attached to it.
            MORE_THAN_ONE_BACKEND_SERVICE = 151_922_141

            # There is insufficient quota for the necessary resources, such as CPU or number of instances.
            NOT_ENOUGH_QUOTA_AVAILABLE = 403_101_631

            # Showed only for regional autoscalers: there is a resource stockout in the chosen region.
            REGION_RESOURCE_STOCKOUT = 528_622_846

            # The target to be scaled does not exist.
            SCALING_TARGET_DOES_NOT_EXIST = 122_636_699

            # For some scaling schedules minRequiredReplicas is greater than maxNumReplicas. Autoscaler always recommends at most maxNumReplicas instances.
            SCHEDULED_INSTANCES_GREATER_THAN_AUTOSCALER_MAX = 29_275_586

            # For some scaling schedules minRequiredReplicas is less than minNumReplicas. Autoscaler always recommends at least minNumReplicas instances.
            SCHEDULED_INSTANCES_LESS_THAN_AUTOSCALER_MIN = 398_287_669

            UNKNOWN = 433_141_802

            # Autoscaling does not work with an HTTP/S load balancer that has been configured for maxRate.
            UNSUPPORTED_MAX_RATE_LOAD_BALANCING_CONFIGURATION = 330_845_009

            # For zonal autoscalers: there is a resource stockout in the chosen zone. For regional autoscalers: in at least one of the zones you're using there is a resource stockout.
            ZONE_RESOURCE_STOCKOUT = 210_200_502
          end
        end

        # @!attribute [rw] autoscalers
        #   @return [::Array<::Google::Cloud::Compute::V1::Autoscaler>]
        #     [Output Only] A list of autoscalers contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning which replaces the list of autoscalers when the list is empty.
        class AutoscalersScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Cloud Autoscaler policy.
        # @!attribute [rw] cool_down_period_sec
        #   @return [::Integer]
        #     The number of seconds that your application takes to initialize on a VM instance. This is referred to as the [initialization period](/compute/docs/autoscaler#cool_down_period). Specifying an accurate initialization period improves autoscaler decisions. For example, when scaling out, the autoscaler ignores data from VMs that are still initializing because those VMs might not yet represent normal usage of your application. The default initialization period is 60 seconds. Initialization periods might vary because of numerous factors. We recommend that you test how long your application takes to initialize. To do this, create a VM and time your application's startup process.
        # @!attribute [rw] cpu_utilization
        #   @return [::Google::Cloud::Compute::V1::AutoscalingPolicyCpuUtilization]
        #     Defines the CPU utilization policy that allows the autoscaler to scale based on the average CPU utilization of a managed instance group.
        # @!attribute [rw] custom_metric_utilizations
        #   @return [::Array<::Google::Cloud::Compute::V1::AutoscalingPolicyCustomMetricUtilization>]
        #     Configuration parameters of autoscaling based on a custom metric.
        # @!attribute [rw] load_balancing_utilization
        #   @return [::Google::Cloud::Compute::V1::AutoscalingPolicyLoadBalancingUtilization]
        #     Configuration parameters of autoscaling based on load balancer.
        # @!attribute [rw] max_num_replicas
        #   @return [::Integer]
        #     The maximum number of instances that the autoscaler can scale out to. This is required when creating or updating an autoscaler. The maximum number of replicas must not be lower than minimal number of replicas.
        # @!attribute [rw] min_num_replicas
        #   @return [::Integer]
        #     The minimum number of replicas that the autoscaler can scale in to. This cannot be less than 0. If not provided, autoscaler chooses a default value depending on maximum number of instances allowed.
        # @!attribute [rw] mode
        #   @return [::String]
        #     Defines the operating mode for this policy. The following modes are available: - OFF: Disables the autoscaler but maintains its configuration. - ONLY_SCALE_OUT: Restricts the autoscaler to add VM instances only. - ON: Enables all autoscaler activities according to its policy. For more information, see "Turning off or restricting an autoscaler"
        #     Check the Mode enum for the list of possible values.
        # @!attribute [rw] scale_in_control
        #   @return [::Google::Cloud::Compute::V1::AutoscalingPolicyScaleInControl]
        # @!attribute [rw] scaling_schedules
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::AutoscalingPolicyScalingSchedule}]
        #     Scaling schedules defined for an autoscaler. Multiple schedules can be set on an autoscaler, and they can overlap. During overlapping periods the greatest min_required_replicas of all scaling schedules is applied. Up to 128 scaling schedules are allowed.
        class AutoscalingPolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::AutoscalingPolicyScalingSchedule]
          class ScalingSchedulesEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # Defines the operating mode for this policy. The following modes are available: - OFF: Disables the autoscaler but maintains its configuration. - ONLY_SCALE_OUT: Restricts the autoscaler to add VM instances only. - ON: Enables all autoscaler activities according to its policy. For more information, see "Turning off or restricting an autoscaler"
          module Mode
            # A value indicating that the enum field is not set.
            UNDEFINED_MODE = 0

            # Do not automatically scale the MIG in or out. The recommended_size field contains the size of MIG that would be set if the actuation mode was enabled.
            OFF = 78_159

            # Automatically scale the MIG in and out according to the policy.
            ON = 2527

            # Automatically create VMs according to the policy, but do not scale the MIG in.
            ONLY_SCALE_OUT = 152_713_670

            # Automatically create VMs according to the policy, but do not scale the MIG in.
            ONLY_UP = 478_095_374
          end
        end

        # CPU utilization policy.
        # @!attribute [rw] predictive_method
        #   @return [::String]
        #     Indicates whether predictive autoscaling based on CPU metric is enabled. Valid values are: * NONE (default). No predictive method is used. The autoscaler scales the group to meet current demand based on real-time metrics. * OPTIMIZE_AVAILABILITY. Predictive autoscaling improves availability by monitoring daily and weekly load patterns and scaling out ahead of anticipated demand.
        #     Check the PredictiveMethod enum for the list of possible values.
        # @!attribute [rw] utilization_target
        #   @return [::Float]
        #     The target CPU utilization that the autoscaler maintains. Must be a float value in the range (0, 1]. If not specified, the default is 0.6. If the CPU level is below the target utilization, the autoscaler scales in the number of instances until it reaches the minimum number of instances you specified or until the average CPU of your instances reaches the target utilization. If the average CPU is above the target utilization, the autoscaler scales out until it reaches the maximum number of instances you specified or until the average utilization reaches the target utilization.
        class AutoscalingPolicyCpuUtilization
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Indicates whether predictive autoscaling based on CPU metric is enabled. Valid values are: * NONE (default). No predictive method is used. The autoscaler scales the group to meet current demand based on real-time metrics. * OPTIMIZE_AVAILABILITY. Predictive autoscaling improves availability by monitoring daily and weekly load patterns and scaling out ahead of anticipated demand.
          module PredictiveMethod
            # A value indicating that the enum field is not set.
            UNDEFINED_PREDICTIVE_METHOD = 0

            # No predictive method is used. The autoscaler scales the group to meet current demand based on real-time metrics
            NONE = 2_402_104

            # Predictive autoscaling improves availability by monitoring daily and weekly load patterns and scaling out ahead of anticipated demand.
            OPTIMIZE_AVAILABILITY = 11_629_437
          end
        end

        # Custom utilization metric policy.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter string, compatible with a Stackdriver Monitoring filter string for TimeSeries.list API call. This filter is used to select a specific TimeSeries for the purpose of autoscaling and to determine whether the metric is exporting per-instance or per-group data. For the filter to be valid for autoscaling purposes, the following rules apply: - You can only use the AND operator for joining selectors. - You can only use direct equality comparison operator (=) without any functions for each selector. - You can specify the metric in both the filter string and in the metric field. However, if specified in both places, the metric must be identical. - The monitored resource type determines what kind of values are expected for the metric. If it is a gce_instance, the autoscaler expects the metric to include a separate TimeSeries for each instance in a group. In such a case, you cannot filter on resource labels. If the resource type is any other value, the autoscaler expects this metric to contain values that apply to the entire autoscaled instance group and resource label filtering can be performed to point autoscaler at the correct TimeSeries to scale upon. This is called a *per-group metric* for the purpose of autoscaling. If not specified, the type defaults to gce_instance. Try to provide a filter that is selective enough to pick just one TimeSeries for the autoscaled group or for each of the instances (if you are using gce_instance resource type). If multiple TimeSeries are returned upon the query execution, the autoscaler will sum their respective values to obtain its scaling value.
        # @!attribute [rw] metric
        #   @return [::String]
        #     The identifier (type) of the Stackdriver Monitoring metric. The metric cannot have negative values. The metric must have a value type of INT64 or DOUBLE.
        # @!attribute [rw] single_instance_assignment
        #   @return [::Float]
        #     If scaling is based on a per-group metric value that represents the total amount of work to be done or resource usage, set this value to an amount assigned for a single instance of the scaled group. Autoscaler keeps the number of instances proportional to the value of this metric. The metric itself does not change value due to group resizing. A good metric to use with the target is for example pubsub.googleapis.com/subscription/num_undelivered_messages or a custom metric exporting the total number of requests coming to your instances. A bad example would be a metric exporting an average or median latency, since this value can't include a chunk assignable to a single instance, it could be better used with utilization_target instead.
        # @!attribute [rw] utilization_target
        #   @return [::Float]
        #     The target value of the metric that autoscaler maintains. This must be a positive value. A utilization metric scales number of virtual machines handling requests to increase or decrease proportionally to the metric. For example, a good metric to use as a utilization_target is https://www.googleapis.com/compute/v1/instance/network/received_bytes_count. The autoscaler works to keep this value constant for each of the instances.
        # @!attribute [rw] utilization_target_type
        #   @return [::String]
        #     Defines how target utilization value is expressed for a Stackdriver Monitoring metric. Either GAUGE, DELTA_PER_SECOND, or DELTA_PER_MINUTE.
        #     Check the UtilizationTargetType enum for the list of possible values.
        class AutoscalingPolicyCustomMetricUtilization
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Defines how target utilization value is expressed for a Stackdriver Monitoring metric. Either GAUGE, DELTA_PER_SECOND, or DELTA_PER_MINUTE.
          module UtilizationTargetType
            # A value indicating that the enum field is not set.
            UNDEFINED_UTILIZATION_TARGET_TYPE = 0

            # Sets the utilization target value for a cumulative or delta metric, expressed as the rate of growth per minute.
            DELTA_PER_MINUTE = 87_432_861

            # Sets the utilization target value for a cumulative or delta metric, expressed as the rate of growth per second.
            DELTA_PER_SECOND = 255_180_029

            # Sets the utilization target value for a gauge metric. The autoscaler will collect the average utilization of the virtual machines from the last couple of minutes, and compare the value to the utilization target value to perform autoscaling.
            GAUGE = 67_590_361
          end
        end

        # Configuration parameters of autoscaling based on load balancing.
        # @!attribute [rw] utilization_target
        #   @return [::Float]
        #     Fraction of backend capacity utilization (set in HTTP(S) load balancing configuration) that the autoscaler maintains. Must be a positive float value. If not defined, the default is 0.8.
        class AutoscalingPolicyLoadBalancingUtilization
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Configuration that allows for slower scale in so that even if Autoscaler recommends an abrupt scale in of a MIG, it will be throttled as specified by the parameters below.
        # @!attribute [rw] max_scaled_in_replicas
        #   @return [::Google::Cloud::Compute::V1::FixedOrPercent]
        #     Maximum allowed number (or %) of VMs that can be deducted from the peak recommendation during the window autoscaler looks at when computing recommendations. Possibly all these VMs can be deleted at once so user service needs to be prepared to lose that many VMs in one step.
        # @!attribute [rw] time_window_sec
        #   @return [::Integer]
        #     How far back autoscaling looks when computing recommendations to include directives regarding slower scale in, as described above.
        class AutoscalingPolicyScaleInControl
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Scaling based on user-defined schedule. The message describes a single scaling schedule. A scaling schedule changes the minimum number of VM instances an autoscaler can recommend, which can trigger scaling out.
        # @!attribute [rw] description
        #   @return [::String]
        #     A description of a scaling schedule.
        # @!attribute [rw] disabled
        #   @return [::Boolean]
        #     A boolean value that specifies whether a scaling schedule can influence autoscaler recommendations. If set to true, then a scaling schedule has no effect. This field is optional, and its value is false by default.
        # @!attribute [rw] duration_sec
        #   @return [::Integer]
        #     The duration of time intervals, in seconds, for which this scaling schedule is to run. The minimum allowed value is 300. This field is required.
        # @!attribute [rw] min_required_replicas
        #   @return [::Integer]
        #     The minimum number of VM instances that the autoscaler will recommend in time intervals starting according to schedule. This field is required.
        # @!attribute [rw] schedule
        #   @return [::String]
        #     The start timestamps of time intervals when this scaling schedule is to provide a scaling signal. This field uses the extended cron format (with an optional year field). The expression can describe a single timestamp if the optional year is set, in which case the scaling schedule runs once. The schedule is interpreted with respect to time_zone. This field is required. Note: These timestamps only describe when autoscaler starts providing the scaling signal. The VMs need additional time to become serving.
        # @!attribute [rw] time_zone
        #   @return [::String]
        #     The time zone to use when interpreting the schedule. The value of this field must be a time zone name from the tz database: https://en.wikipedia.org/wiki/Tz_database. This field is assigned a default value of "UTC" if left empty.
        class AutoscalingPolicyScalingSchedule
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Message containing information of one individual backend.
        # @!attribute [rw] balancing_mode
        #   @return [::String]
        #     Specifies how to determine whether the backend of a load balancer can handle additional traffic or is fully loaded. For usage guidelines, see Connection balancing mode. Backends must use compatible balancing modes. For more information, see Supported balancing modes and target capacity settings and Restrictions and guidance for instance groups. Note: Currently, if you use the API to configure incompatible balancing modes, the configuration might be accepted even though it has no impact and is ignored. Specifically, Backend.maxUtilization is ignored when Backend.balancingMode is RATE. In the future, this incompatible combination will be rejected.
        #     Check the BalancingMode enum for the list of possible values.
        # @!attribute [rw] capacity_scaler
        #   @return [::Float]
        #     A multiplier applied to the backend's target capacity of its balancing mode. The default value is 1, which means the group serves up to 100% of its configured capacity (depending on balancingMode). A setting of 0 means the group is completely drained, offering 0% of its available capacity. The valid ranges are 0.0 and [0.1,1.0]. You cannot configure a setting larger than 0 and smaller than 0.1. You cannot configure a setting of 0 when there is only one backend attached to the backend service. Not available with backends that don't support using a balancingMode. This includes backends such as global internet NEGs, regional serverless NEGs, and PSC NEGs.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] failover
        #   @return [::Boolean]
        #     This field designates whether this is a failover backend. More than one failover backend can be configured for a given BackendService.
        # @!attribute [rw] group
        #   @return [::String]
        #     The fully-qualified URL of an instance group or network endpoint group (NEG) resource. To determine what types of backends a load balancer supports, see the [Backend services overview](https://cloud.google.com/load-balancing/docs/backend-service#backends). You must use the *fully-qualified* URL (starting with https://www.googleapis.com/) to specify the instance group or NEG. Partial URLs are not supported.
        # @!attribute [rw] max_connections
        #   @return [::Integer]
        #     Defines a target maximum number of simultaneous connections. For usage guidelines, see Connection balancing mode and Utilization balancing mode. Not available if the backend's balancingMode is RATE.
        # @!attribute [rw] max_connections_per_endpoint
        #   @return [::Integer]
        #     Defines a target maximum number of simultaneous connections. For usage guidelines, see Connection balancing mode and Utilization balancing mode. Not available if the backend's balancingMode is RATE.
        # @!attribute [rw] max_connections_per_instance
        #   @return [::Integer]
        #     Defines a target maximum number of simultaneous connections. For usage guidelines, see Connection balancing mode and Utilization balancing mode. Not available if the backend's balancingMode is RATE.
        # @!attribute [rw] max_rate
        #   @return [::Integer]
        #     Defines a maximum number of HTTP requests per second (RPS). For usage guidelines, see Rate balancing mode and Utilization balancing mode. Not available if the backend's balancingMode is CONNECTION.
        # @!attribute [rw] max_rate_per_endpoint
        #   @return [::Float]
        #     Defines a maximum target for requests per second (RPS). For usage guidelines, see Rate balancing mode and Utilization balancing mode. Not available if the backend's balancingMode is CONNECTION.
        # @!attribute [rw] max_rate_per_instance
        #   @return [::Float]
        #     Defines a maximum target for requests per second (RPS). For usage guidelines, see Rate balancing mode and Utilization balancing mode. Not available if the backend's balancingMode is CONNECTION.
        # @!attribute [rw] max_utilization
        #   @return [::Float]
        #     Optional parameter to define a target capacity for the UTILIZATION balancing mode. The valid range is [0.0, 1.0]. For usage guidelines, see Utilization balancing mode.
        class Backend
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Specifies how to determine whether the backend of a load balancer can handle additional traffic or is fully loaded. For usage guidelines, see Connection balancing mode. Backends must use compatible balancing modes. For more information, see Supported balancing modes and target capacity settings and Restrictions and guidance for instance groups. Note: Currently, if you use the API to configure incompatible balancing modes, the configuration might be accepted even though it has no impact and is ignored. Specifically, Backend.maxUtilization is ignored when Backend.balancingMode is RATE. In the future, this incompatible combination will be rejected.
          module BalancingMode
            # A value indicating that the enum field is not set.
            UNDEFINED_BALANCING_MODE = 0

            # Balance based on the number of simultaneous connections.
            CONNECTION = 246_311_646

            # Balance based on requests per second (RPS).
            RATE = 2_508_000

            # Balance based on the backend utilization.
            UTILIZATION = 157_008_386
          end
        end

        # Represents a Cloud Storage Bucket resource. This Cloud Storage bucket resource is referenced by a URL map of a load balancer. For more information, read Backend Buckets.
        # @!attribute [rw] bucket_name
        #   @return [::String]
        #     Cloud Storage bucket name.
        # @!attribute [rw] cdn_policy
        #   @return [::Google::Cloud::Compute::V1::BackendBucketCdnPolicy]
        #     Cloud CDN configuration for this BackendBucket.
        # @!attribute [rw] compression_mode
        #   @return [::String]
        #     Compress text responses using Brotli or gzip compression, based on the client's Accept-Encoding header.
        #     Check the CompressionMode enum for the list of possible values.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] custom_response_headers
        #   @return [::Array<::String>]
        #     Headers that the Application Load Balancer should add to proxied responses.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional textual description of the resource; provided by the client when the resource is created.
        # @!attribute [rw] edge_security_policy
        #   @return [::String]
        #     [Output Only] The resource URL for the edge security policy associated with this backend bucket.
        # @!attribute [rw] enable_cdn
        #   @return [::Boolean]
        #     If true, enable Cloud CDN for this BackendBucket.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     Type of the resource.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        class BackendBucket
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Compress text responses using Brotli or gzip compression, based on the client's Accept-Encoding header.
          module CompressionMode
            # A value indicating that the enum field is not set.
            UNDEFINED_COMPRESSION_MODE = 0

            # Automatically uses the best compression based on the Accept-Encoding header sent by the client.
            AUTOMATIC = 165_298_699

            # Disables compression. Existing compressed responses cached by Cloud CDN will not be served to clients.
            DISABLED = 516_696_700
          end
        end

        # Message containing Cloud CDN configuration for a backend bucket.
        # @!attribute [rw] bypass_cache_on_request_headers
        #   @return [::Array<::Google::Cloud::Compute::V1::BackendBucketCdnPolicyBypassCacheOnRequestHeader>]
        #     Bypass the cache when the specified request headers are matched - e.g. Pragma or Authorization headers. Up to 5 headers can be specified. The cache is bypassed for all cdnPolicy.cacheMode settings.
        # @!attribute [rw] cache_key_policy
        #   @return [::Google::Cloud::Compute::V1::BackendBucketCdnPolicyCacheKeyPolicy]
        #     The CacheKeyPolicy for this CdnPolicy.
        # @!attribute [rw] cache_mode
        #   @return [::String]
        #     Specifies the cache setting for all responses from this backend. The possible values are: USE_ORIGIN_HEADERS Requires the origin to set valid caching headers to cache content. Responses without these headers will not be cached at Google's edge, and will require a full trip to the origin on every request, potentially impacting performance and increasing load on the origin server. FORCE_CACHE_ALL Cache all content, ignoring any "private", "no-store" or "no-cache" directives in Cache-Control response headers. Warning: this may result in Cloud CDN caching private, per-user (user identifiable) content. CACHE_ALL_STATIC Automatically cache static content, including common image formats, media (video and audio), and web assets (JavaScript and CSS). Requests and responses that are marked as uncacheable, as well as dynamic content (including HTML), will not be cached.
        #     Check the CacheMode enum for the list of possible values.
        # @!attribute [rw] client_ttl
        #   @return [::Integer]
        #     Specifies a separate client (e.g. browser client) maximum TTL. This is used to clamp the max-age (or Expires) value sent to the client. With FORCE_CACHE_ALL, the lesser of client_ttl and default_ttl is used for the response max-age directive, along with a "public" directive. For cacheable content in CACHE_ALL_STATIC mode, client_ttl clamps the max-age from the origin (if specified), or else sets the response max-age directive to the lesser of the client_ttl and default_ttl, and also ensures a "public" cache-control directive is present. If a client TTL is not specified, a default value (1 hour) will be used. The maximum allowed value is 31,622,400s (1 year).
        # @!attribute [rw] default_ttl
        #   @return [::Integer]
        #     Specifies the default TTL for cached content served by this origin for responses that do not have an existing valid TTL (max-age or s-max-age). Setting a TTL of "0" means "always revalidate". The value of defaultTTL cannot be set to a value greater than that of maxTTL, but can be equal. When the cacheMode is set to FORCE_CACHE_ALL, the defaultTTL will overwrite the TTL set in all responses. The maximum allowed value is 31,622,400s (1 year), noting that infrequently accessed objects may be evicted from the cache before the defined TTL.
        # @!attribute [rw] max_ttl
        #   @return [::Integer]
        #     Specifies the maximum allowed TTL for cached content served by this origin. Cache directives that attempt to set a max-age or s-maxage higher than this, or an Expires header more than maxTTL seconds in the future will be capped at the value of maxTTL, as if it were the value of an s-maxage Cache-Control directive. Headers sent to the client will not be modified. Setting a TTL of "0" means "always revalidate". The maximum allowed value is 31,622,400s (1 year), noting that infrequently accessed objects may be evicted from the cache before the defined TTL.
        # @!attribute [rw] negative_caching
        #   @return [::Boolean]
        #     Negative caching allows per-status code TTLs to be set, in order to apply fine-grained caching for common errors or redirects. This can reduce the load on your origin and improve end-user experience by reducing response latency. When the cache mode is set to CACHE_ALL_STATIC or USE_ORIGIN_HEADERS, negative caching applies to responses with the specified response code that lack any Cache-Control, Expires, or Pragma: no-cache directives. When the cache mode is set to FORCE_CACHE_ALL, negative caching applies to all responses with the specified response code, and override any caching headers. By default, Cloud CDN will apply the following default TTLs to these status codes: HTTP 300 (Multiple Choice), 301, 308 (Permanent Redirects): 10m HTTP 404 (Not Found), 410 (Gone), 451 (Unavailable For Legal Reasons): 120s HTTP 405 (Method Not Found), 421 (Misdirected Request), 501 (Not Implemented): 60s. These defaults can be overridden in negative_caching_policy.
        # @!attribute [rw] negative_caching_policy
        #   @return [::Array<::Google::Cloud::Compute::V1::BackendBucketCdnPolicyNegativeCachingPolicy>]
        #     Sets a cache TTL for the specified HTTP status code. negative_caching must be enabled to configure negative_caching_policy. Omitting the policy and leaving negative_caching enabled will use Cloud CDN's default cache TTLs. Note that when specifying an explicit negative_caching_policy, you should take care to specify a cache TTL for all response codes that you wish to cache. Cloud CDN will not apply any default negative caching when a policy exists.
        # @!attribute [rw] request_coalescing
        #   @return [::Boolean]
        #     If true then Cloud CDN will combine multiple concurrent cache fill requests into a small number of requests to the origin.
        # @!attribute [rw] serve_while_stale
        #   @return [::Integer]
        #     Serve existing content from the cache (if available) when revalidating content with the origin, or when an error is encountered when refreshing the cache. This setting defines the default "max-stale" duration for any cached responses that do not specify a max-stale directive. Stale responses that exceed the TTL configured here will not be served. The default limit (max-stale) is 86400s (1 day), which will allow stale content to be served up to this limit beyond the max-age (or s-max-age) of a cached response. The maximum allowed value is 604800 (1 week). Set this to zero (0) to disable serve-while-stale.
        # @!attribute [rw] signed_url_cache_max_age_sec
        #   @return [::Integer]
        #     Maximum number of seconds the response to a signed URL request will be considered fresh. After this time period, the response will be revalidated before being served. Defaults to 1hr (3600s). When serving responses to signed URL requests, Cloud CDN will internally behave as though all responses from this backend had a "Cache-Control: public, max-age=[TTL]" header, regardless of any existing Cache-Control header. The actual headers served in responses will not be altered.
        # @!attribute [rw] signed_url_key_names
        #   @return [::Array<::String>]
        #     [Output Only] Names of the keys for signing request URLs.
        class BackendBucketCdnPolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Specifies the cache setting for all responses from this backend. The possible values are: USE_ORIGIN_HEADERS Requires the origin to set valid caching headers to cache content. Responses without these headers will not be cached at Google's edge, and will require a full trip to the origin on every request, potentially impacting performance and increasing load on the origin server. FORCE_CACHE_ALL Cache all content, ignoring any "private", "no-store" or "no-cache" directives in Cache-Control response headers. Warning: this may result in Cloud CDN caching private, per-user (user identifiable) content. CACHE_ALL_STATIC Automatically cache static content, including common image formats, media (video and audio), and web assets (JavaScript and CSS). Requests and responses that are marked as uncacheable, as well as dynamic content (including HTML), will not be cached.
          module CacheMode
            # A value indicating that the enum field is not set.
            UNDEFINED_CACHE_MODE = 0

            # Automatically cache static content, including common image formats, media (video and audio), and web assets (JavaScript and CSS). Requests and responses that are marked as uncacheable, as well as dynamic content (including HTML), will not be cached.
            CACHE_ALL_STATIC = 355_027_945

            # Cache all content, ignoring any "private", "no-store" or "no-cache" directives in Cache-Control response headers. Warning: this may result in Cloud CDN caching private, per-user (user identifiable) content.
            FORCE_CACHE_ALL = 486_026_928

            INVALID_CACHE_MODE = 381_295_560

            # Requires the origin to set valid caching headers to cache content. Responses without these headers will not be cached at Google's edge, and will require a full trip to the origin on every request, potentially impacting performance and increasing load on the origin server.
            USE_ORIGIN_HEADERS = 55_380_261
          end
        end

        # Bypass the cache when the specified request headers are present, e.g. Pragma or Authorization headers. Values are case insensitive. The presence of such a header overrides the cache_mode setting.
        # @!attribute [rw] header_name
        #   @return [::String]
        #     The header field name to match on when bypassing cache. Values are case-insensitive.
        class BackendBucketCdnPolicyBypassCacheOnRequestHeader
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Message containing what to include in the cache key for a request for Cloud CDN.
        # @!attribute [rw] include_http_headers
        #   @return [::Array<::String>]
        #     Allows HTTP request headers (by name) to be used in the cache key.
        # @!attribute [rw] query_string_whitelist
        #   @return [::Array<::String>]
        #     Names of query string parameters to include in cache keys. Default parameters are always included. '&' and '=' will be percent encoded and not treated as delimiters.
        class BackendBucketCdnPolicyCacheKeyPolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Specify CDN TTLs for response error codes.
        # @!attribute [rw] code
        #   @return [::Integer]
        #     The HTTP status code to define a TTL against. Only HTTP status codes 300, 301, 302, 307, 308, 404, 405, 410, 421, 451 and 501 are can be specified as values, and you cannot specify a status code more than once.
        # @!attribute [rw] ttl
        #   @return [::Integer]
        #     The TTL (in seconds) for which to cache responses with the corresponding status code. The maximum allowed value is 1800s (30 minutes), noting that infrequently accessed objects may be evicted from the cache before the defined TTL.
        class BackendBucketCdnPolicyNegativeCachingPolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Contains a list of BackendBucket resources.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::BackendBucket>]
        #     A list of BackendBucket resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     Type of resource.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class BackendBucketList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a Backend Service resource. A backend service defines how Google Cloud load balancers distribute traffic. The backend service configuration contains a set of values, such as the protocol used to connect to backends, various distribution and session settings, health checks, and timeouts. These settings provide fine-grained control over how your load balancer behaves. Most of the settings have default values that allow for easy configuration if you need to get started quickly. Backend services in Google Compute Engine can be either regionally or globally scoped. * [Global](https://cloud.google.com/compute/docs/reference/rest/v1/backendServices) * [Regional](https://cloud.google.com/compute/docs/reference/rest/v1/regionBackendServices) For more information, see Backend Services.
        # @!attribute [rw] affinity_cookie_ttl_sec
        #   @return [::Integer]
        #     Lifetime of cookies in seconds. This setting is applicable to external and internal HTTP(S) load balancers and Traffic Director and requires GENERATED_COOKIE or HTTP_COOKIE session affinity. If set to 0, the cookie is non-persistent and lasts only until the end of the browser session (or equivalent). The maximum allowed value is two weeks (1,209,600). Not supported when the backend service is referenced by a URL map that is bound to target gRPC proxy that has validateForProxyless field set to true.
        # @!attribute [rw] backends
        #   @return [::Array<::Google::Cloud::Compute::V1::Backend>]
        #     The list of backends that serve this BackendService.
        # @!attribute [rw] cdn_policy
        #   @return [::Google::Cloud::Compute::V1::BackendServiceCdnPolicy]
        #     Cloud CDN configuration for this BackendService. Only available for specified load balancer types.
        # @!attribute [rw] circuit_breakers
        #   @return [::Google::Cloud::Compute::V1::CircuitBreakers]
        # @!attribute [rw] compression_mode
        #   @return [::String]
        #     Compress text responses using Brotli or gzip compression, based on the client's Accept-Encoding header.
        #     Check the CompressionMode enum for the list of possible values.
        # @!attribute [rw] connection_draining
        #   @return [::Google::Cloud::Compute::V1::ConnectionDraining]
        # @!attribute [rw] connection_tracking_policy
        #   @return [::Google::Cloud::Compute::V1::BackendServiceConnectionTrackingPolicy]
        #     Connection Tracking configuration for this BackendService. Connection tracking policy settings are only available for Network Load Balancing and Internal TCP/UDP Load Balancing.
        # @!attribute [rw] consistent_hash
        #   @return [::Google::Cloud::Compute::V1::ConsistentHashLoadBalancerSettings]
        #     Consistent Hash-based load balancing can be used to provide soft session affinity based on HTTP headers, cookies or other properties. This load balancing policy is applicable only for HTTP connections. The affinity to a particular destination host will be lost when one or more hosts are added/removed from the destination service. This field specifies parameters that control consistent hashing. This field is only applicable when localityLbPolicy is set to MAGLEV or RING_HASH. This field is applicable to either: - A regional backend service with the service_protocol set to HTTP, HTTPS, or HTTP2, and load_balancing_scheme set to INTERNAL_MANAGED. - A global backend service with the load_balancing_scheme set to INTERNAL_SELF_MANAGED.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] custom_request_headers
        #   @return [::Array<::String>]
        #     Headers that the load balancer adds to proxied requests. See [Creating custom headers](https://cloud.google.com/load-balancing/docs/custom-headers).
        # @!attribute [rw] custom_response_headers
        #   @return [::Array<::String>]
        #     Headers that the load balancer adds to proxied responses. See [Creating custom headers](https://cloud.google.com/load-balancing/docs/custom-headers).
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] edge_security_policy
        #   @return [::String]
        #     [Output Only] The resource URL for the edge security policy associated with this backend service.
        # @!attribute [rw] enable_c_d_n
        #   @return [::Boolean]
        #     If true, enables Cloud CDN for the backend service of an external HTTP(S) load balancer.
        # @!attribute [rw] failover_policy
        #   @return [::Google::Cloud::Compute::V1::BackendServiceFailoverPolicy]
        #     Requires at least one backend instance group to be defined as a backup (failover) backend. For load balancers that have configurable failover: [Internal TCP/UDP Load Balancing](https://cloud.google.com/load-balancing/docs/internal/failover-overview) and [external TCP/UDP Load Balancing](https://cloud.google.com/load-balancing/docs/network/networklb-failover-overview).
        # @!attribute [rw] fingerprint
        #   @return [::String]
        #     Fingerprint of this resource. A hash of the contents stored in this object. This field is used in optimistic locking. This field will be ignored when inserting a BackendService. An up-to-date fingerprint must be provided in order to update the BackendService, otherwise the request will fail with error 412 conditionNotMet. To see the latest fingerprint, make a get() request to retrieve a BackendService.
        # @!attribute [rw] health_checks
        #   @return [::Array<::String>]
        #     The list of URLs to the healthChecks, httpHealthChecks (legacy), or httpsHealthChecks (legacy) resource for health checking this backend service. Not all backend services support legacy health checks. See Load balancer guide. Currently, at most one health check can be specified for each backend service. Backend services with instance group or zonal NEG backends must have a health check. Backend services with internet or serverless NEG backends must not have a health check.
        # @!attribute [rw] iap
        #   @return [::Google::Cloud::Compute::V1::BackendServiceIAP]
        #     The configurations for Identity-Aware Proxy on this resource. Not available for Internal TCP/UDP Load Balancing and Network Load Balancing.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#backendService for backend services.
        # @!attribute [rw] load_balancing_scheme
        #   @return [::String]
        #     Specifies the load balancer type. A backend service created for one type of load balancer cannot be used with another. For more information, refer to Choosing a load balancer.
        #     Check the LoadBalancingScheme enum for the list of possible values.
        # @!attribute [rw] locality_lb_policies
        #   @return [::Array<::Google::Cloud::Compute::V1::BackendServiceLocalityLoadBalancingPolicyConfig>]
        #     A list of locality load-balancing policies to be used in order of preference. When you use localityLbPolicies, you must set at least one value for either the localityLbPolicies[].policy or the localityLbPolicies[].customPolicy field. localityLbPolicies overrides any value set in the localityLbPolicy field. For an example of how to use this field, see Define a list of preferred policies. Caution: This field and its children are intended for use in a service mesh that includes gRPC clients only. Envoy proxies can't use backend services that have this configuration.
        # @!attribute [rw] locality_lb_policy
        #   @return [::String]
        #     The load balancing algorithm used within the scope of the locality. The possible values are: - ROUND_ROBIN: This is a simple policy in which each healthy backend is selected in round robin order. This is the default. - LEAST_REQUEST: An O(1) algorithm which selects two random healthy hosts and picks the host which has fewer active requests. - RING_HASH: The ring/modulo hash load balancer implements consistent hashing to backends. The algorithm has the property that the addition/removal of a host from a set of N hosts only affects 1/N of the requests. - RANDOM: The load balancer selects a random healthy host. - ORIGINAL_DESTINATION: Backend host is selected based on the client connection metadata, i.e., connections are opened to the same address as the destination address of the incoming connection before the connection was redirected to the load balancer. - MAGLEV: used as a drop in replacement for the ring hash load balancer. Maglev is not as stable as ring hash but has faster table lookup build times and host selection times. For more information about Maglev, see https://ai.google/research/pubs/pub44824 This field is applicable to either: - A regional backend service with the service_protocol set to HTTP, HTTPS, or HTTP2, and load_balancing_scheme set to INTERNAL_MANAGED. - A global backend service with the load_balancing_scheme set to INTERNAL_SELF_MANAGED, INTERNAL_MANAGED, or EXTERNAL_MANAGED. If sessionAffinity is not NONE, and this field is not set to MAGLEV or RING_HASH, session affinity settings will not take effect. Only ROUND_ROBIN and RING_HASH are supported when the backend service is referenced by a URL map that is bound to target gRPC proxy that has validateForProxyless field set to true.
        #     Check the LocalityLbPolicy enum for the list of possible values.
        # @!attribute [rw] log_config
        #   @return [::Google::Cloud::Compute::V1::BackendServiceLogConfig]
        #     This field denotes the logging options for the load balancer traffic served by this backend service. If logging is enabled, logs will be exported to Stackdriver.
        # @!attribute [rw] max_stream_duration
        #   @return [::Google::Cloud::Compute::V1::Duration]
        #     Specifies the default maximum duration (timeout) for streams to this service. Duration is computed from the beginning of the stream until the response has been completely processed, including all retries. A stream that does not complete in this duration is closed. If not specified, there will be no timeout limit, i.e. the maximum duration is infinite. This value can be overridden in the PathMatcher configuration of the UrlMap that references this backend service. This field is only allowed when the loadBalancingScheme of the backend service is INTERNAL_SELF_MANAGED.
        # @!attribute [rw] metadatas
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     Deployment metadata associated with the resource to be set by a GKE hub controller and read by the backend RCTH
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] network
        #   @return [::String]
        #     The URL of the network to which this backend service belongs. This field can only be specified when the load balancing scheme is set to INTERNAL.
        # @!attribute [rw] outlier_detection
        #   @return [::Google::Cloud::Compute::V1::OutlierDetection]
        #     Settings controlling the ejection of unhealthy backend endpoints from the load balancing pool of each individual proxy instance that processes the traffic for the given backend service. If not set, this feature is considered disabled. Results of the outlier detection algorithm (ejection of endpoints from the load balancing pool and returning them back to the pool) are executed independently by each proxy instance of the load balancer. In most cases, more than one proxy instance handles the traffic received by a backend service. Thus, it is possible that an unhealthy endpoint is detected and ejected by only some of the proxies, and while this happens, other proxies may continue to send requests to the same unhealthy endpoint until they detect and eject the unhealthy endpoint. Applicable backend endpoints can be: - VM instances in an Instance Group - Endpoints in a Zonal NEG (GCE_VM_IP, GCE_VM_IP_PORT) - Endpoints in a Hybrid Connectivity NEG (NON_GCP_PRIVATE_IP_PORT) - Serverless NEGs, that resolve to Cloud Run, App Engine, or Cloud Functions Services - Private Service Connect NEGs, that resolve to Google-managed regional API endpoints or managed services published using Private Service Connect Applicable backend service types can be: - A global backend service with the loadBalancingScheme set to INTERNAL_SELF_MANAGED or EXTERNAL_MANAGED. - A regional backend service with the serviceProtocol set to HTTP, HTTPS, or HTTP2, and loadBalancingScheme set to INTERNAL_MANAGED or EXTERNAL_MANAGED. Not supported for Serverless NEGs. Not supported when the backend service is referenced by a URL map that is bound to target gRPC proxy that has validateForProxyless field set to true.
        # @!attribute [rw] port
        #   @return [::Integer]
        #     Deprecated in favor of portName. The TCP port to connect on the backend. The default value is 80. For Internal TCP/UDP Load Balancing and Network Load Balancing, omit port.
        # @!attribute [rw] port_name
        #   @return [::String]
        #     A named port on a backend instance group representing the port for communication to the backend VMs in that group. The named port must be [defined on each backend instance group](https://cloud.google.com/load-balancing/docs/backend-service#named_ports). This parameter has no meaning if the backends are NEGs. For Internal TCP/UDP Load Balancing and Network Load Balancing, omit port_name.
        # @!attribute [rw] protocol
        #   @return [::String]
        #     The protocol this BackendService uses to communicate with backends. Possible values are HTTP, HTTPS, HTTP2, TCP, SSL, UDP or GRPC. depending on the chosen load balancer or Traffic Director configuration. Refer to the documentation for the load balancers or for Traffic Director for more information. Must be set to GRPC when the backend service is referenced by a URL map that is bound to target gRPC proxy.
        #     Check the Protocol enum for the list of possible values.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] URL of the region where the regional backend service resides. This field is not applicable to global backend services. You must specify this field as part of the HTTP request URL. It is not settable as a field in the request body.
        # @!attribute [rw] security_policy
        #   @return [::String]
        #     [Output Only] The resource URL for the security policy associated with this backend service.
        # @!attribute [rw] security_settings
        #   @return [::Google::Cloud::Compute::V1::SecuritySettings]
        #     This field specifies the security settings that apply to this backend service. This field is applicable to a global backend service with the load_balancing_scheme set to INTERNAL_SELF_MANAGED.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] service_bindings
        #   @return [::Array<::String>]
        #     URLs of networkservices.ServiceBinding resources. Can only be set if load balancing scheme is INTERNAL_SELF_MANAGED. If set, lists of backends and health checks must be both empty.
        # @!attribute [rw] session_affinity
        #   @return [::String]
        #     Type of session affinity to use. The default is NONE. Only NONE and HEADER_FIELD are supported when the backend service is referenced by a URL map that is bound to target gRPC proxy that has validateForProxyless field set to true. For more details, see: [Session Affinity](https://cloud.google.com/load-balancing/docs/backend-service#session_affinity).
        #     Check the SessionAffinity enum for the list of possible values.
        # @!attribute [rw] subsetting
        #   @return [::Google::Cloud::Compute::V1::Subsetting]
        # @!attribute [rw] timeout_sec
        #   @return [::Integer]
        #     The backend service timeout has a different meaning depending on the type of load balancer. For more information see, Backend service settings. The default is 30 seconds. The full range of timeout values allowed goes from 1 through 2,147,483,647 seconds. This value can be overridden in the PathMatcher configuration of the UrlMap that references this backend service. Not supported when the backend service is referenced by a URL map that is bound to target gRPC proxy that has validateForProxyless field set to true. Instead, use maxStreamDuration.
        # @!attribute [rw] used_by
        #   @return [::Array<::Google::Cloud::Compute::V1::BackendServiceUsedBy>]
        class BackendService
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class MetadatasEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # Compress text responses using Brotli or gzip compression, based on the client's Accept-Encoding header.
          module CompressionMode
            # A value indicating that the enum field is not set.
            UNDEFINED_COMPRESSION_MODE = 0

            # Automatically uses the best compression based on the Accept-Encoding header sent by the client.
            AUTOMATIC = 165_298_699

            # Disables compression. Existing compressed responses cached by Cloud CDN will not be served to clients.
            DISABLED = 516_696_700
          end

          # Specifies the load balancer type. A backend service created for one type of load balancer cannot be used with another. For more information, refer to Choosing a load balancer.
          module LoadBalancingScheme
            # A value indicating that the enum field is not set.
            UNDEFINED_LOAD_BALANCING_SCHEME = 0

            # Signifies that this will be used for external HTTP(S), SSL Proxy, TCP Proxy, or Network Load Balancing
            EXTERNAL = 35_607_499

            # Signifies that this will be used for External Managed HTTP(S) Load Balancing.
            EXTERNAL_MANAGED = 512_006_923

            # Signifies that this will be used for Internal TCP/UDP Load Balancing.
            INTERNAL = 279_295_677

            # Signifies that this will be used for Internal HTTP(S) Load Balancing.
            INTERNAL_MANAGED = 37_350_397

            # Signifies that this will be used by Traffic Director.
            INTERNAL_SELF_MANAGED = 236_211_150

            INVALID_LOAD_BALANCING_SCHEME = 275_352_060
          end

          # The load balancing algorithm used within the scope of the locality. The possible values are: - ROUND_ROBIN: This is a simple policy in which each healthy backend is selected in round robin order. This is the default. - LEAST_REQUEST: An O(1) algorithm which selects two random healthy hosts and picks the host which has fewer active requests. - RING_HASH: The ring/modulo hash load balancer implements consistent hashing to backends. The algorithm has the property that the addition/removal of a host from a set of N hosts only affects 1/N of the requests. - RANDOM: The load balancer selects a random healthy host. - ORIGINAL_DESTINATION: Backend host is selected based on the client connection metadata, i.e., connections are opened to the same address as the destination address of the incoming connection before the connection was redirected to the load balancer. - MAGLEV: used as a drop in replacement for the ring hash load balancer. Maglev is not as stable as ring hash but has faster table lookup build times and host selection times. For more information about Maglev, see https://ai.google/research/pubs/pub44824 This field is applicable to either: - A regional backend service with the service_protocol set to HTTP, HTTPS, or HTTP2, and load_balancing_scheme set to INTERNAL_MANAGED. - A global backend service with the load_balancing_scheme set to INTERNAL_SELF_MANAGED, INTERNAL_MANAGED, or EXTERNAL_MANAGED. If sessionAffinity is not NONE, and this field is not set to MAGLEV or RING_HASH, session affinity settings will not take effect. Only ROUND_ROBIN and RING_HASH are supported when the backend service is referenced by a URL map that is bound to target gRPC proxy that has validateForProxyless field set to true.
          module LocalityLbPolicy
            # A value indicating that the enum field is not set.
            UNDEFINED_LOCALITY_LB_POLICY = 0

            INVALID_LB_POLICY = 323_318_707

            # An O(1) algorithm which selects two random healthy hosts and picks the host which has fewer active requests.
            LEAST_REQUEST = 46_604_921

            # This algorithm implements consistent hashing to backends. Maglev can be used as a drop in replacement for the ring hash load balancer. Maglev is not as stable as ring hash but has faster table lookup build times and host selection times. For more information about Maglev, see https://ai.google/research/pubs/pub44824
            MAGLEV = 119_180_266

            # Backend host is selected based on the client connection metadata, i.e., connections are opened to the same address as the destination address of the incoming connection before the connection was redirected to the load balancer.
            ORIGINAL_DESTINATION = 166_297_216

            # The load balancer selects a random healthy host.
            RANDOM = 262_527_171

            # The ring/modulo hash load balancer implements consistent hashing to backends. The algorithm has the property that the addition/removal of a host from a set of N hosts only affects 1/N of the requests.
            RING_HASH = 432_795_069

            # This is a simple policy in which each healthy backend is selected in round robin order. This is the default.
            ROUND_ROBIN = 153_895_801

            # Per-instance weighted Load Balancing via health check reported weights. If set, the Backend Service must configure a non legacy HTTP-based Health Check, and health check replies are expected to contain non-standard HTTP response header field X-Load-Balancing-Endpoint-Weight to specify the per-instance weights. If set, Load Balancing is weighted based on the per-instance weights reported in the last processed health check replies, as long as every instance either reported a valid weight or had UNAVAILABLE_WEIGHT. Otherwise, Load Balancing remains equal-weight. This option is only supported in Network Load Balancing.
            WEIGHTED_MAGLEV = 254_930_962
          end

          # The protocol this BackendService uses to communicate with backends. Possible values are HTTP, HTTPS, HTTP2, TCP, SSL, UDP or GRPC. depending on the chosen load balancer or Traffic Director configuration. Refer to the documentation for the load balancers or for Traffic Director for more information. Must be set to GRPC when the backend service is referenced by a URL map that is bound to target gRPC proxy.
          module Protocol
            # A value indicating that the enum field is not set.
            UNDEFINED_PROTOCOL = 0

            # gRPC (available for Traffic Director).
            GRPC = 2_196_510

            HTTP = 2_228_360

            # HTTP/2 with SSL.
            HTTP2 = 69_079_210

            HTTPS = 69_079_243

            # TCP proxying with SSL.
            SSL = 82_412

            # TCP proxying or TCP pass-through.
            TCP = 82_881

            # UDP.
            UDP = 83_873

            # If a Backend Service has UNSPECIFIED as its protocol, it can be used with any L3/L4 Forwarding Rules.
            UNSPECIFIED = 526_786_327
          end

          # Type of session affinity to use. The default is NONE. Only NONE and HEADER_FIELD are supported when the backend service is referenced by a URL map that is bound to target gRPC proxy that has validateForProxyless field set to true. For more details, see: [Session Affinity](https://cloud.google.com/load-balancing/docs/backend-service#session_affinity).
          module SessionAffinity
            # A value indicating that the enum field is not set.
            UNDEFINED_SESSION_AFFINITY = 0

            # 2-tuple hash on packet's source and destination IP addresses. Connections from the same source IP address to the same destination IP address will be served by the same backend VM while that VM remains healthy.
            CLIENT_IP = 345_665_051

            # 1-tuple hash only on packet's source IP address. Connections from the same source IP address will be served by the same backend VM while that VM remains healthy. This option can only be used for Internal TCP/UDP Load Balancing.
            CLIENT_IP_NO_DESTINATION = 106_122_516

            # 5-tuple hash on packet's source and destination IP addresses, IP protocol, and source and destination ports. Connections for the same IP protocol from the same source IP address and port to the same destination IP address and port will be served by the same backend VM while that VM remains healthy. This option cannot be used for HTTP(S) load balancing.
            CLIENT_IP_PORT_PROTO = 221_722_926

            # 3-tuple hash on packet's source and destination IP addresses, and IP protocol. Connections for the same IP protocol from the same source IP address to the same destination IP address will be served by the same backend VM while that VM remains healthy. This option cannot be used for HTTP(S) load balancing.
            CLIENT_IP_PROTO = 25_322_148

            # Hash based on a cookie generated by the L7 loadbalancer. Only valid for HTTP(S) load balancing.
            GENERATED_COOKIE = 370_321_204

            # The hash is based on a user specified header field.
            HEADER_FIELD = 200_737_960

            # The hash is based on a user provided cookie.
            HTTP_COOKIE = 494_981_627

            # No session affinity. Connections from the same client IP may go to any instance in the pool.
            NONE = 2_402_104
          end
        end

        # Contains a list of BackendServicesScopedList.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::BackendServicesScopedList}]
        #     A list of BackendServicesScopedList resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     Type of resource.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class BackendServiceAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::BackendServicesScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # Message containing Cloud CDN configuration for a backend service.
        # @!attribute [rw] bypass_cache_on_request_headers
        #   @return [::Array<::Google::Cloud::Compute::V1::BackendServiceCdnPolicyBypassCacheOnRequestHeader>]
        #     Bypass the cache when the specified request headers are matched - e.g. Pragma or Authorization headers. Up to 5 headers can be specified. The cache is bypassed for all cdnPolicy.cacheMode settings.
        # @!attribute [rw] cache_key_policy
        #   @return [::Google::Cloud::Compute::V1::CacheKeyPolicy]
        #     The CacheKeyPolicy for this CdnPolicy.
        # @!attribute [rw] cache_mode
        #   @return [::String]
        #     Specifies the cache setting for all responses from this backend. The possible values are: USE_ORIGIN_HEADERS Requires the origin to set valid caching headers to cache content. Responses without these headers will not be cached at Google's edge, and will require a full trip to the origin on every request, potentially impacting performance and increasing load on the origin server. FORCE_CACHE_ALL Cache all content, ignoring any "private", "no-store" or "no-cache" directives in Cache-Control response headers. Warning: this may result in Cloud CDN caching private, per-user (user identifiable) content. CACHE_ALL_STATIC Automatically cache static content, including common image formats, media (video and audio), and web assets (JavaScript and CSS). Requests and responses that are marked as uncacheable, as well as dynamic content (including HTML), will not be cached.
        #     Check the CacheMode enum for the list of possible values.
        # @!attribute [rw] client_ttl
        #   @return [::Integer]
        #     Specifies a separate client (e.g. browser client) maximum TTL. This is used to clamp the max-age (or Expires) value sent to the client. With FORCE_CACHE_ALL, the lesser of client_ttl and default_ttl is used for the response max-age directive, along with a "public" directive. For cacheable content in CACHE_ALL_STATIC mode, client_ttl clamps the max-age from the origin (if specified), or else sets the response max-age directive to the lesser of the client_ttl and default_ttl, and also ensures a "public" cache-control directive is present. If a client TTL is not specified, a default value (1 hour) will be used. The maximum allowed value is 31,622,400s (1 year).
        # @!attribute [rw] default_ttl
        #   @return [::Integer]
        #     Specifies the default TTL for cached content served by this origin for responses that do not have an existing valid TTL (max-age or s-max-age). Setting a TTL of "0" means "always revalidate". The value of defaultTTL cannot be set to a value greater than that of maxTTL, but can be equal. When the cacheMode is set to FORCE_CACHE_ALL, the defaultTTL will overwrite the TTL set in all responses. The maximum allowed value is 31,622,400s (1 year), noting that infrequently accessed objects may be evicted from the cache before the defined TTL.
        # @!attribute [rw] max_ttl
        #   @return [::Integer]
        #     Specifies the maximum allowed TTL for cached content served by this origin. Cache directives that attempt to set a max-age or s-maxage higher than this, or an Expires header more than maxTTL seconds in the future will be capped at the value of maxTTL, as if it were the value of an s-maxage Cache-Control directive. Headers sent to the client will not be modified. Setting a TTL of "0" means "always revalidate". The maximum allowed value is 31,622,400s (1 year), noting that infrequently accessed objects may be evicted from the cache before the defined TTL.
        # @!attribute [rw] negative_caching
        #   @return [::Boolean]
        #     Negative caching allows per-status code TTLs to be set, in order to apply fine-grained caching for common errors or redirects. This can reduce the load on your origin and improve end-user experience by reducing response latency. When the cache mode is set to CACHE_ALL_STATIC or USE_ORIGIN_HEADERS, negative caching applies to responses with the specified response code that lack any Cache-Control, Expires, or Pragma: no-cache directives. When the cache mode is set to FORCE_CACHE_ALL, negative caching applies to all responses with the specified response code, and override any caching headers. By default, Cloud CDN will apply the following default TTLs to these status codes: HTTP 300 (Multiple Choice), 301, 308 (Permanent Redirects): 10m HTTP 404 (Not Found), 410 (Gone), 451 (Unavailable For Legal Reasons): 120s HTTP 405 (Method Not Found), 421 (Misdirected Request), 501 (Not Implemented): 60s. These defaults can be overridden in negative_caching_policy.
        # @!attribute [rw] negative_caching_policy
        #   @return [::Array<::Google::Cloud::Compute::V1::BackendServiceCdnPolicyNegativeCachingPolicy>]
        #     Sets a cache TTL for the specified HTTP status code. negative_caching must be enabled to configure negative_caching_policy. Omitting the policy and leaving negative_caching enabled will use Cloud CDN's default cache TTLs. Note that when specifying an explicit negative_caching_policy, you should take care to specify a cache TTL for all response codes that you wish to cache. Cloud CDN will not apply any default negative caching when a policy exists.
        # @!attribute [rw] request_coalescing
        #   @return [::Boolean]
        #     If true then Cloud CDN will combine multiple concurrent cache fill requests into a small number of requests to the origin.
        # @!attribute [rw] serve_while_stale
        #   @return [::Integer]
        #     Serve existing content from the cache (if available) when revalidating content with the origin, or when an error is encountered when refreshing the cache. This setting defines the default "max-stale" duration for any cached responses that do not specify a max-stale directive. Stale responses that exceed the TTL configured here will not be served. The default limit (max-stale) is 86400s (1 day), which will allow stale content to be served up to this limit beyond the max-age (or s-max-age) of a cached response. The maximum allowed value is 604800 (1 week). Set this to zero (0) to disable serve-while-stale.
        # @!attribute [rw] signed_url_cache_max_age_sec
        #   @return [::Integer]
        #     Maximum number of seconds the response to a signed URL request will be considered fresh. After this time period, the response will be revalidated before being served. Defaults to 1hr (3600s). When serving responses to signed URL requests, Cloud CDN will internally behave as though all responses from this backend had a "Cache-Control: public, max-age=[TTL]" header, regardless of any existing Cache-Control header. The actual headers served in responses will not be altered.
        # @!attribute [rw] signed_url_key_names
        #   @return [::Array<::String>]
        #     [Output Only] Names of the keys for signing request URLs.
        class BackendServiceCdnPolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Specifies the cache setting for all responses from this backend. The possible values are: USE_ORIGIN_HEADERS Requires the origin to set valid caching headers to cache content. Responses without these headers will not be cached at Google's edge, and will require a full trip to the origin on every request, potentially impacting performance and increasing load on the origin server. FORCE_CACHE_ALL Cache all content, ignoring any "private", "no-store" or "no-cache" directives in Cache-Control response headers. Warning: this may result in Cloud CDN caching private, per-user (user identifiable) content. CACHE_ALL_STATIC Automatically cache static content, including common image formats, media (video and audio), and web assets (JavaScript and CSS). Requests and responses that are marked as uncacheable, as well as dynamic content (including HTML), will not be cached.
          module CacheMode
            # A value indicating that the enum field is not set.
            UNDEFINED_CACHE_MODE = 0

            # Automatically cache static content, including common image formats, media (video and audio), and web assets (JavaScript and CSS). Requests and responses that are marked as uncacheable, as well as dynamic content (including HTML), will not be cached.
            CACHE_ALL_STATIC = 355_027_945

            # Cache all content, ignoring any "private", "no-store" or "no-cache" directives in Cache-Control response headers. Warning: this may result in Cloud CDN caching private, per-user (user identifiable) content.
            FORCE_CACHE_ALL = 486_026_928

            INVALID_CACHE_MODE = 381_295_560

            # Requires the origin to set valid caching headers to cache content. Responses without these headers will not be cached at Google's edge, and will require a full trip to the origin on every request, potentially impacting performance and increasing load on the origin server.
            USE_ORIGIN_HEADERS = 55_380_261
          end
        end

        # Bypass the cache when the specified request headers are present, e.g. Pragma or Authorization headers. Values are case insensitive. The presence of such a header overrides the cache_mode setting.
        # @!attribute [rw] header_name
        #   @return [::String]
        #     The header field name to match on when bypassing cache. Values are case-insensitive.
        class BackendServiceCdnPolicyBypassCacheOnRequestHeader
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Specify CDN TTLs for response error codes.
        # @!attribute [rw] code
        #   @return [::Integer]
        #     The HTTP status code to define a TTL against. Only HTTP status codes 300, 301, 302, 307, 308, 404, 405, 410, 421, 451 and 501 are can be specified as values, and you cannot specify a status code more than once.
        # @!attribute [rw] ttl
        #   @return [::Integer]
        #     The TTL (in seconds) for which to cache responses with the corresponding status code. The maximum allowed value is 1800s (30 minutes), noting that infrequently accessed objects may be evicted from the cache before the defined TTL.
        class BackendServiceCdnPolicyNegativeCachingPolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Connection Tracking configuration for this BackendService.
        # @!attribute [rw] connection_persistence_on_unhealthy_backends
        #   @return [::String]
        #     Specifies connection persistence when backends are unhealthy. The default value is DEFAULT_FOR_PROTOCOL. If set to DEFAULT_FOR_PROTOCOL, the existing connections persist on unhealthy backends only for connection-oriented protocols (TCP and SCTP) and only if the Tracking Mode is PER_CONNECTION (default tracking mode) or the Session Affinity is configured for 5-tuple. They do not persist for UDP. If set to NEVER_PERSIST, after a backend becomes unhealthy, the existing connections on the unhealthy backend are never persisted on the unhealthy backend. They are always diverted to newly selected healthy backends (unless all backends are unhealthy). If set to ALWAYS_PERSIST, existing connections always persist on unhealthy backends regardless of protocol and session affinity. It is generally not recommended to use this mode overriding the default. For more details, see [Connection Persistence for Network Load Balancing](https://cloud.google.com/load-balancing/docs/network/networklb-backend-service#connection-persistence) and [Connection Persistence for Internal TCP/UDP Load Balancing](https://cloud.google.com/load-balancing/docs/internal#connection-persistence).
        #     Check the ConnectionPersistenceOnUnhealthyBackends enum for the list of possible values.
        # @!attribute [rw] enable_strong_affinity
        #   @return [::Boolean]
        #     Enable Strong Session Affinity for Network Load Balancing. This option is not available publicly.
        # @!attribute [rw] idle_timeout_sec
        #   @return [::Integer]
        #     Specifies how long to keep a Connection Tracking entry while there is no matching traffic (in seconds). For Internal TCP/UDP Load Balancing: - The minimum (default) is 10 minutes and the maximum is 16 hours. - It can be set only if Connection Tracking is less than 5-tuple (i.e. Session Affinity is CLIENT_IP_NO_DESTINATION, CLIENT_IP or CLIENT_IP_PROTO, and Tracking Mode is PER_SESSION). For Network Load Balancer the default is 60 seconds. This option is not available publicly.
        # @!attribute [rw] tracking_mode
        #   @return [::String]
        #     Specifies the key used for connection tracking. There are two options: - PER_CONNECTION: This is the default mode. The Connection Tracking is performed as per the Connection Key (default Hash Method) for the specific protocol. - PER_SESSION: The Connection Tracking is performed as per the configured Session Affinity. It matches the configured Session Affinity. For more details, see [Tracking Mode for Network Load Balancing](https://cloud.google.com/load-balancing/docs/network/networklb-backend-service#tracking-mode) and [Tracking Mode for Internal TCP/UDP Load Balancing](https://cloud.google.com/load-balancing/docs/internal#tracking-mode).
        #     Check the TrackingMode enum for the list of possible values.
        class BackendServiceConnectionTrackingPolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Specifies connection persistence when backends are unhealthy. The default value is DEFAULT_FOR_PROTOCOL. If set to DEFAULT_FOR_PROTOCOL, the existing connections persist on unhealthy backends only for connection-oriented protocols (TCP and SCTP) and only if the Tracking Mode is PER_CONNECTION (default tracking mode) or the Session Affinity is configured for 5-tuple. They do not persist for UDP. If set to NEVER_PERSIST, after a backend becomes unhealthy, the existing connections on the unhealthy backend are never persisted on the unhealthy backend. They are always diverted to newly selected healthy backends (unless all backends are unhealthy). If set to ALWAYS_PERSIST, existing connections always persist on unhealthy backends regardless of protocol and session affinity. It is generally not recommended to use this mode overriding the default. For more details, see [Connection Persistence for Network Load Balancing](https://cloud.google.com/load-balancing/docs/network/networklb-backend-service#connection-persistence) and [Connection Persistence for Internal TCP/UDP Load Balancing](https://cloud.google.com/load-balancing/docs/internal#connection-persistence).
          module ConnectionPersistenceOnUnhealthyBackends
            # A value indicating that the enum field is not set.
            UNDEFINED_CONNECTION_PERSISTENCE_ON_UNHEALTHY_BACKENDS = 0

            ALWAYS_PERSIST = 38_400_900

            DEFAULT_FOR_PROTOCOL = 145_265_356

            NEVER_PERSIST = 138_646_241
          end

          # Specifies the key used for connection tracking. There are two options: - PER_CONNECTION: This is the default mode. The Connection Tracking is performed as per the Connection Key (default Hash Method) for the specific protocol. - PER_SESSION: The Connection Tracking is performed as per the configured Session Affinity. It matches the configured Session Affinity. For more details, see [Tracking Mode for Network Load Balancing](https://cloud.google.com/load-balancing/docs/network/networklb-backend-service#tracking-mode) and [Tracking Mode for Internal TCP/UDP Load Balancing](https://cloud.google.com/load-balancing/docs/internal#tracking-mode).
          module TrackingMode
            # A value indicating that the enum field is not set.
            UNDEFINED_TRACKING_MODE = 0

            INVALID_TRACKING_MODE = 49_234_371

            PER_CONNECTION = 85_162_848

            PER_SESSION = 182_099_252
          end
        end

        # For load balancers that have configurable failover: [Internal TCP/UDP Load Balancing](https://cloud.google.com/load-balancing/docs/internal/failover-overview) and [external TCP/UDP Load Balancing](https://cloud.google.com/load-balancing/docs/network/networklb-failover-overview). On failover or failback, this field indicates whether connection draining will be honored. Google Cloud has a fixed connection draining timeout of 10 minutes. A setting of true terminates existing TCP connections to the active pool during failover and failback, immediately draining traffic. A setting of false allows existing TCP connections to persist, even on VMs no longer in the active pool, for up to the duration of the connection draining timeout (10 minutes).
        # @!attribute [rw] disable_connection_drain_on_failover
        #   @return [::Boolean]
        #     This can be set to true only if the protocol is TCP. The default is false.
        # @!attribute [rw] drop_traffic_if_unhealthy
        #   @return [::Boolean]
        #     If set to true, connections to the load balancer are dropped when all primary and all backup backend VMs are unhealthy.If set to false, connections are distributed among all primary VMs when all primary and all backup backend VMs are unhealthy. For load balancers that have configurable failover: [Internal TCP/UDP Load Balancing](https://cloud.google.com/load-balancing/docs/internal/failover-overview) and [external TCP/UDP Load Balancing](https://cloud.google.com/load-balancing/docs/network/networklb-failover-overview). The default is false.
        # @!attribute [rw] failover_ratio
        #   @return [::Float]
        #     The value of the field must be in the range [0, 1]. If the value is 0, the load balancer performs a failover when the number of healthy primary VMs equals zero. For all other values, the load balancer performs a failover when the total number of healthy primary VMs is less than this ratio. For load balancers that have configurable failover: [Internal TCP/UDP Load Balancing](https://cloud.google.com/load-balancing/docs/internal/failover-overview) and [external TCP/UDP Load Balancing](https://cloud.google.com/load-balancing/docs/network/networklb-failover-overview).
        class BackendServiceFailoverPolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] annotations
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     Metadata defined as annotations on the network endpoint group.
        # @!attribute [rw] health_status
        #   @return [::Array<::Google::Cloud::Compute::V1::HealthStatus>]
        #     Health state of the backend instances or endpoints in requested instance or network endpoint group, determined based on configured health checks.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#backendServiceGroupHealth for the health of backend services.
        class BackendServiceGroupHealth
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class AnnotationsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # Identity-Aware Proxy
        # @!attribute [rw] enabled
        #   @return [::Boolean]
        #     Whether the serving infrastructure will authenticate and authorize all incoming requests.
        # @!attribute [rw] oauth2_client_id
        #   @return [::String]
        #     OAuth2 client ID to use for the authentication flow.
        # @!attribute [rw] oauth2_client_secret
        #   @return [::String]
        #     OAuth2 client secret to use for the authentication flow. For security reasons, this value cannot be retrieved via the API. Instead, the SHA-256 hash of the value is returned in the oauth2ClientSecretSha256 field. @InputOnly
        # @!attribute [rw] oauth2_client_secret_sha256
        #   @return [::String]
        #     [Output Only] SHA256 hash value for the field oauth2_client_secret above.
        class BackendServiceIAP
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Contains a list of BackendService resources.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::BackendService>]
        #     A list of BackendService resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#backendServiceList for lists of backend services.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class BackendServiceList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Contains a list of usable BackendService resources.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::BackendService>]
        #     A list of BackendService resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#usableBackendServiceList for lists of usable backend services.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class BackendServiceListUsable
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Container for either a built-in LB policy supported by gRPC or Envoy or a custom one implemented by the end user.
        # @!attribute [rw] custom_policy
        #   @return [::Google::Cloud::Compute::V1::BackendServiceLocalityLoadBalancingPolicyConfigCustomPolicy]
        # @!attribute [rw] policy
        #   @return [::Google::Cloud::Compute::V1::BackendServiceLocalityLoadBalancingPolicyConfigPolicy]
        class BackendServiceLocalityLoadBalancingPolicyConfig
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # The configuration for a custom policy implemented by the user and deployed with the client.
        # @!attribute [rw] data
        #   @return [::String]
        #     An optional, arbitrary JSON object with configuration data, understood by a locally installed custom policy implementation.
        # @!attribute [rw] name
        #   @return [::String]
        #     Identifies the custom policy. The value should match the name of a custom implementation registered on the gRPC clients. It should follow protocol buffer message naming conventions and include the full path (for example, myorg.CustomLbPolicy). The maximum length is 256 characters. Do not specify the same custom policy more than once for a backend. If you do, the configuration is rejected. For an example of how to use this field, see Use a custom policy.
        class BackendServiceLocalityLoadBalancingPolicyConfigCustomPolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # The configuration for a built-in load balancing policy.
        # @!attribute [rw] name
        #   @return [::String]
        #     The name of a locality load-balancing policy. Valid values include ROUND_ROBIN and, for Java clients, LEAST_REQUEST. For information about these values, see the description of localityLbPolicy. Do not specify the same policy more than once for a backend. If you do, the configuration is rejected.
        #     Check the Name enum for the list of possible values.
        class BackendServiceLocalityLoadBalancingPolicyConfigPolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The name of a locality load-balancing policy. Valid values include ROUND_ROBIN and, for Java clients, LEAST_REQUEST. For information about these values, see the description of localityLbPolicy. Do not specify the same policy more than once for a backend. If you do, the configuration is rejected.
          module Name
            # A value indicating that the enum field is not set.
            UNDEFINED_NAME = 0

            INVALID_LB_POLICY = 323_318_707

            # An O(1) algorithm which selects two random healthy hosts and picks the host which has fewer active requests.
            LEAST_REQUEST = 46_604_921

            # This algorithm implements consistent hashing to backends. Maglev can be used as a drop in replacement for the ring hash load balancer. Maglev is not as stable as ring hash but has faster table lookup build times and host selection times. For more information about Maglev, see https://ai.google/research/pubs/pub44824
            MAGLEV = 119_180_266

            # Backend host is selected based on the client connection metadata, i.e., connections are opened to the same address as the destination address of the incoming connection before the connection was redirected to the load balancer.
            ORIGINAL_DESTINATION = 166_297_216

            # The load balancer selects a random healthy host.
            RANDOM = 262_527_171

            # The ring/modulo hash load balancer implements consistent hashing to backends. The algorithm has the property that the addition/removal of a host from a set of N hosts only affects 1/N of the requests.
            RING_HASH = 432_795_069

            # This is a simple policy in which each healthy backend is selected in round robin order. This is the default.
            ROUND_ROBIN = 153_895_801

            # Per-instance weighted Load Balancing via health check reported weights. If set, the Backend Service must configure a non legacy HTTP-based Health Check, and health check replies are expected to contain non-standard HTTP response header field X-Load-Balancing-Endpoint-Weight to specify the per-instance weights. If set, Load Balancing is weighted based on the per-instance weights reported in the last processed health check replies, as long as every instance either reported a valid weight or had UNAVAILABLE_WEIGHT. Otherwise, Load Balancing remains equal-weight. This option is only supported in Network Load Balancing.
            WEIGHTED_MAGLEV = 254_930_962
          end
        end

        # The available logging options for the load balancer traffic served by this backend service.
        # @!attribute [rw] enable
        #   @return [::Boolean]
        #     Denotes whether to enable logging for the load balancer traffic served by this backend service. The default value is false.
        # @!attribute [rw] optional_fields
        #   @return [::Array<::String>]
        #     This field can only be specified if logging is enabled for this backend service and "logConfig.optionalMode" was set to CUSTOM. Contains a list of optional fields you want to include in the logs. For example: serverInstance, serverGkeDetails.cluster, serverGkeDetails.pod.podNamespace
        # @!attribute [rw] optional_mode
        #   @return [::String]
        #     This field can only be specified if logging is enabled for this backend service. Configures whether all, none or a subset of optional fields should be added to the reported logs. One of [INCLUDE_ALL_OPTIONAL, EXCLUDE_ALL_OPTIONAL, CUSTOM]. Default is EXCLUDE_ALL_OPTIONAL.
        #     Check the OptionalMode enum for the list of possible values.
        # @!attribute [rw] sample_rate
        #   @return [::Float]
        #     This field can only be specified if logging is enabled for this backend service. The value of the field must be in [0, 1]. This configures the sampling rate of requests to the load balancer where 1.0 means all logged requests are reported and 0.0 means no logged requests are reported. The default value is 1.0.
        class BackendServiceLogConfig
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # This field can only be specified if logging is enabled for this backend service. Configures whether all, none or a subset of optional fields should be added to the reported logs. One of [INCLUDE_ALL_OPTIONAL, EXCLUDE_ALL_OPTIONAL, CUSTOM]. Default is EXCLUDE_ALL_OPTIONAL.
          module OptionalMode
            # A value indicating that the enum field is not set.
            UNDEFINED_OPTIONAL_MODE = 0

            # A subset of optional fields.
            CUSTOM = 388_595_569

            # None optional fields.
            EXCLUDE_ALL_OPTIONAL = 168_636_099

            # All optional fields.
            INCLUDE_ALL_OPTIONAL = 535_606_965
          end
        end

        # @!attribute [rw] backend_service
        #   @return [::String]
        class BackendServiceReference
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] reference
        #   @return [::String]
        class BackendServiceUsedBy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] backend_services
        #   @return [::Array<::Google::Cloud::Compute::V1::BackendService>]
        #     A list of BackendServices contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     Informational warning which replaces the list of backend services when the list is empty.
        class BackendServicesScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] authentication_present
        #   @return [::Boolean]
        #     The Authentication Present bit of the BFD packet. This is specified in section 4.1 of RFC5880
        # @!attribute [rw] control_plane_independent
        #   @return [::Boolean]
        #     The Control Plane Independent bit of the BFD packet. This is specified in section 4.1 of RFC5880
        # @!attribute [rw] demand
        #   @return [::Boolean]
        #     The demand bit of the BFD packet. This is specified in section 4.1 of RFC5880
        # @!attribute [rw] diagnostic
        #   @return [::String]
        #     The diagnostic code specifies the local system's reason for the last change in session state. This allows remote systems to determine the reason that the previous session failed, for example. These diagnostic codes are specified in section 4.1 of RFC5880
        #     Check the Diagnostic enum for the list of possible values.
        # @!attribute [rw] final
        #   @return [::Boolean]
        #     The Final bit of the BFD packet. This is specified in section 4.1 of RFC5880
        # @!attribute [rw] length
        #   @return [::Integer]
        #     The length of the BFD Control packet in bytes. This is specified in section 4.1 of RFC5880
        # @!attribute [rw] min_echo_rx_interval_ms
        #   @return [::Integer]
        #     The Required Min Echo RX Interval value in the BFD packet. This is specified in section 4.1 of RFC5880
        # @!attribute [rw] min_rx_interval_ms
        #   @return [::Integer]
        #     The Required Min RX Interval value in the BFD packet. This is specified in section 4.1 of RFC5880
        # @!attribute [rw] min_tx_interval_ms
        #   @return [::Integer]
        #     The Desired Min TX Interval value in the BFD packet. This is specified in section 4.1 of RFC5880
        # @!attribute [rw] multiplier
        #   @return [::Integer]
        #     The detection time multiplier of the BFD packet. This is specified in section 4.1 of RFC5880
        # @!attribute [rw] multipoint
        #   @return [::Boolean]
        #     The multipoint bit of the BFD packet. This is specified in section 4.1 of RFC5880
        # @!attribute [rw] my_discriminator
        #   @return [::Integer]
        #     The My Discriminator value in the BFD packet. This is specified in section 4.1 of RFC5880
        # @!attribute [rw] poll
        #   @return [::Boolean]
        #     The Poll bit of the BFD packet. This is specified in section 4.1 of RFC5880
        # @!attribute [rw] state
        #   @return [::String]
        #     The current BFD session state as seen by the transmitting system. These states are specified in section 4.1 of RFC5880
        #     Check the State enum for the list of possible values.
        # @!attribute [rw] version
        #   @return [::Integer]
        #     The version number of the BFD protocol, as specified in section 4.1 of RFC5880.
        # @!attribute [rw] your_discriminator
        #   @return [::Integer]
        #     The Your Discriminator value in the BFD packet. This is specified in section 4.1 of RFC5880
        class BfdPacket
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The diagnostic code specifies the local system's reason for the last change in session state. This allows remote systems to determine the reason that the previous session failed, for example. These diagnostic codes are specified in section 4.1 of RFC5880
          module Diagnostic
            # A value indicating that the enum field is not set.
            UNDEFINED_DIAGNOSTIC = 0

            ADMINISTRATIVELY_DOWN = 121_685_798

            CONCATENATED_PATH_DOWN = 26_186_892

            CONTROL_DETECTION_TIME_EXPIRED = 135_478_383

            DIAGNOSTIC_UNSPECIFIED = 58_720_895

            ECHO_FUNCTION_FAILED = 220_687_466

            FORWARDING_PLANE_RESET = 19_715_882

            NEIGHBOR_SIGNALED_SESSION_DOWN = 374_226_742

            NO_DIAGNOSTIC = 222_503_141

            PATH_DOWN = 290_605_180

            REVERSE_CONCATENATED_PATH_DOWN = 479_337_129
          end

          # The current BFD session state as seen by the transmitting system. These states are specified in section 4.1 of RFC5880
          module State
            # A value indicating that the enum field is not set.
            UNDEFINED_STATE = 0

            ADMIN_DOWN = 128_544_690

            DOWN = 2_104_482

            INIT = 2_252_048

            STATE_UNSPECIFIED = 470_755_401

            UP = 2715
          end
        end

        # Next free: 15
        # @!attribute [rw] bfd_session_initialization_mode
        #   @return [::String]
        #     The BFD session initialization mode for this BGP peer. If set to ACTIVE, the Cloud Router will initiate the BFD session for this BGP peer. If set to PASSIVE, the Cloud Router will wait for the peer router to initiate the BFD session for this BGP peer. If set to DISABLED, BFD is disabled for this BGP peer.
        #     Check the BfdSessionInitializationMode enum for the list of possible values.
        # @!attribute [rw] config_update_timestamp_micros
        #   @return [::Integer]
        #     Unix timestamp of the most recent config update.
        # @!attribute [rw] control_packet_counts
        #   @return [::Google::Cloud::Compute::V1::BfdStatusPacketCounts]
        #     Control packet counts for the current BFD session.
        # @!attribute [rw] control_packet_intervals
        #   @return [::Array<::Google::Cloud::Compute::V1::PacketIntervals>]
        #     Inter-packet time interval statistics for control packets.
        # @!attribute [rw] local_diagnostic
        #   @return [::String]
        #     The diagnostic code specifies the local system's reason for the last change in session state. This allows remote systems to determine the reason that the previous session failed, for example. These diagnostic codes are specified in section 4.1 of RFC5880
        #     Check the LocalDiagnostic enum for the list of possible values.
        # @!attribute [rw] local_state
        #   @return [::String]
        #     The current BFD session state as seen by the transmitting system. These states are specified in section 4.1 of RFC5880
        #     Check the LocalState enum for the list of possible values.
        # @!attribute [rw] negotiated_local_control_tx_interval_ms
        #   @return [::Integer]
        #     Negotiated transmit interval for control packets.
        # @!attribute [rw] rx_packet
        #   @return [::Google::Cloud::Compute::V1::BfdPacket]
        #     The most recent Rx control packet for this BFD session.
        # @!attribute [rw] tx_packet
        #   @return [::Google::Cloud::Compute::V1::BfdPacket]
        #     The most recent Tx control packet for this BFD session.
        # @!attribute [rw] uptime_ms
        #   @return [::Integer]
        #     Session uptime in milliseconds. Value will be 0 if session is not up.
        class BfdStatus
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The BFD session initialization mode for this BGP peer. If set to ACTIVE, the Cloud Router will initiate the BFD session for this BGP peer. If set to PASSIVE, the Cloud Router will wait for the peer router to initiate the BFD session for this BGP peer. If set to DISABLED, BFD is disabled for this BGP peer.
          module BfdSessionInitializationMode
            # A value indicating that the enum field is not set.
            UNDEFINED_BFD_SESSION_INITIALIZATION_MODE = 0

            ACTIVE = 314_733_318

            DISABLED = 516_696_700

            PASSIVE = 462_813_959
          end

          # The diagnostic code specifies the local system's reason for the last change in session state. This allows remote systems to determine the reason that the previous session failed, for example. These diagnostic codes are specified in section 4.1 of RFC5880
          module LocalDiagnostic
            # A value indicating that the enum field is not set.
            UNDEFINED_LOCAL_DIAGNOSTIC = 0

            ADMINISTRATIVELY_DOWN = 121_685_798

            CONCATENATED_PATH_DOWN = 26_186_892

            CONTROL_DETECTION_TIME_EXPIRED = 135_478_383

            DIAGNOSTIC_UNSPECIFIED = 58_720_895

            ECHO_FUNCTION_FAILED = 220_687_466

            FORWARDING_PLANE_RESET = 19_715_882

            NEIGHBOR_SIGNALED_SESSION_DOWN = 374_226_742

            NO_DIAGNOSTIC = 222_503_141

            PATH_DOWN = 290_605_180

            REVERSE_CONCATENATED_PATH_DOWN = 479_337_129
          end

          # The current BFD session state as seen by the transmitting system. These states are specified in section 4.1 of RFC5880
          module LocalState
            # A value indicating that the enum field is not set.
            UNDEFINED_LOCAL_STATE = 0

            ADMIN_DOWN = 128_544_690

            DOWN = 2_104_482

            INIT = 2_252_048

            STATE_UNSPECIFIED = 470_755_401

            UP = 2715
          end
        end

        # @!attribute [rw] num_rx
        #   @return [::Integer]
        #     Number of packets received since the beginning of the current BFD session.
        # @!attribute [rw] num_rx_rejected
        #   @return [::Integer]
        #     Number of packets received that were rejected because of errors since the beginning of the current BFD session.
        # @!attribute [rw] num_rx_successful
        #   @return [::Integer]
        #     Number of packets received that were successfully processed since the beginning of the current BFD session.
        # @!attribute [rw] num_tx
        #   @return [::Integer]
        #     Number of packets transmitted since the beginning of the current BFD session.
        class BfdStatusPacketCounts
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Associates `members`, or principals, with a `role`.
        # @!attribute [rw] binding_id
        #   @return [::String]
        #     This is deprecated and has no effect. Do not use.
        # @!attribute [rw] condition
        #   @return [::Google::Cloud::Compute::V1::Expr]
        #     The condition that is associated with this binding. If the condition evaluates to `true`, then this binding applies to the current request. If the condition evaluates to `false`, then this binding does not apply to the current request. However, a different role binding might grant the same role to one or more of the principals in this binding. To learn which resources support conditions in their IAM policies, see the [IAM documentation](https://cloud.google.com/iam/help/conditions/resource-policies).
        # @!attribute [rw] members
        #   @return [::Array<::String>]
        #     Specifies the principals requesting access for a Google Cloud resource. `members` can have the following values: * `allUsers`: A special identifier that represents anyone who is on the internet; with or without a Google account. * `allAuthenticatedUsers`: A special identifier that represents anyone who is authenticated with a Google account or a service account. Does not include identities that come from external identity providers (IdPs) through identity federation. * `user:{emailid}`: An email address that represents a specific Google account. For example, `alice@example.com` . * `serviceAccount:{emailid}`: An email address that represents a Google service account. For example, `my-other-app@appspot.gserviceaccount.com`. * `serviceAccount:{projectid}.svc.id.goog[{namespace}/{kubernetes-sa}]`: An identifier for a [Kubernetes service account](https://cloud.google.com/kubernetes-engine/docs/how-to/kubernetes-service-accounts). For example, `my-project.svc.id.goog[my-namespace/my-kubernetes-sa]`. * `group:{emailid}`: An email address that represents a Google group. For example, `admins@example.com`. * `domain:{domain}`: The G Suite domain (primary) that represents all the users of that domain. For example, `google.com` or `example.com`. * `deleted:user:{emailid}?uid={uniqueid}`: An email address (plus unique identifier) representing a user that has been recently deleted. For example, `alice@example.com?uid=123456789012345678901`. If the user is recovered, this value reverts to `user:{emailid}` and the recovered user retains the role in the binding. * `deleted:serviceAccount:{emailid}?uid={uniqueid}`: An email address (plus unique identifier) representing a service account that has been recently deleted. For example, `my-other-app@appspot.gserviceaccount.com?uid=123456789012345678901`. If the service account is undeleted, this value reverts to `serviceAccount:{emailid}` and the undeleted service account retains the role in the binding. * `deleted:group:{emailid}?uid={uniqueid}`: An email address (plus unique identifier) representing a Google group that has been recently deleted. For example, `admins@example.com?uid=123456789012345678901`. If the group is recovered, this value reverts to `group:{emailid}` and the recovered group retains the role in the binding.
        # @!attribute [rw] role
        #   @return [::String]
        #     Role that is assigned to the list of `members`, or principals. For example, `roles/viewer`, `roles/editor`, or `roles/owner`.
        class Binding
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Disks.BulkInsert. See the method description for details.
        # @!attribute [rw] bulk_insert_disk_resource_resource
        #   @return [::Google::Cloud::Compute::V1::BulkInsertDiskResource]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class BulkInsertDiskRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A transient resource used in compute.disks.bulkInsert and compute.regionDisks.bulkInsert. It is only used to process requests and is not persisted.
        # @!attribute [rw] source_consistency_group_policy
        #   @return [::String]
        #     The URL of the DiskConsistencyGroupPolicy for the group of disks to clone. This may be a full or partial URL, such as: - https://www.googleapis.com/compute/v1/projects/project/regions/region /resourcePolicies/resourcePolicy - projects/project/regions/region/resourcePolicies/resourcePolicy - regions/region/resourcePolicies/resourcePolicy
        class BulkInsertDiskResource
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.BulkInsert. See the method description for details.
        # @!attribute [rw] bulk_insert_instance_resource_resource
        #   @return [::Google::Cloud::Compute::V1::BulkInsertInstanceResource]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class BulkInsertInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A transient resource used in compute.instances.bulkInsert and compute.regionInstances.bulkInsert . This resource is not persisted anywhere, it is used only for processing the requests.
        # @!attribute [rw] count
        #   @return [::Integer]
        #     The maximum number of instances to create.
        # @!attribute [rw] instance_properties
        #   @return [::Google::Cloud::Compute::V1::InstanceProperties]
        #     The instance properties defining the VM instances to be created. Required if sourceInstanceTemplate is not provided.
        # @!attribute [rw] location_policy
        #   @return [::Google::Cloud::Compute::V1::LocationPolicy]
        #     Policy for chosing target zone. For more information, see Create VMs in bulk .
        # @!attribute [rw] min_count
        #   @return [::Integer]
        #     The minimum number of instances to create. If no min_count is specified then count is used as the default value. If min_count instances cannot be created, then no instances will be created and instances already created will be deleted.
        # @!attribute [rw] name_pattern
        #   @return [::String]
        #     The string pattern used for the names of the VMs. Either name_pattern or per_instance_properties must be set. The pattern must contain one continuous sequence of placeholder hash characters (#) with each character corresponding to one digit of the generated instance name. Example: a name_pattern of inst-#### generates instance names such as inst-0001 and inst-0002. If existing instances in the same project and zone have names that match the name pattern then the generated instance numbers start after the biggest existing number. For example, if there exists an instance with name inst-0050, then instance names generated using the pattern inst-#### begin with inst-0051. The name pattern placeholder #...# can contain up to 18 characters.
        # @!attribute [rw] per_instance_properties
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::BulkInsertInstanceResourcePerInstanceProperties}]
        #     Per-instance properties to be set on individual instances. Keys of this map specify requested instance names. Can be empty if name_pattern is used.
        # @!attribute [rw] source_instance_template
        #   @return [::String]
        #     Specifies the instance template from which to create instances. You may combine sourceInstanceTemplate with instanceProperties to override specific values from an existing instance template. Bulk API follows the semantics of JSON Merge Patch described by RFC 7396. It can be a full or partial URL. For example, the following are all valid URLs to an instance template: - https://www.googleapis.com/compute/v1/projects/project /global/instanceTemplates/instanceTemplate - projects/project/global/instanceTemplates/instanceTemplate - global/instanceTemplates/instanceTemplate This field is optional.
        class BulkInsertInstanceResource
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::BulkInsertInstanceResourcePerInstanceProperties]
          class PerInstancePropertiesEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # Per-instance properties to be set on individual instances. To be extended in the future.
        # @!attribute [rw] hostname
        #   @return [::String]
        #     Specifies the hostname of the instance. More details in: https://cloud.google.com/compute/docs/instances/custom-hostname-vm#naming_convention
        # @!attribute [rw] name
        #   @return [::String]
        #     This field is only temporary. It will be removed. Do not use it.
        class BulkInsertInstanceResourcePerInstanceProperties
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] created_vm_count
        #   @return [::Integer]
        #     [Output Only] Count of VMs successfully created so far.
        # @!attribute [rw] deleted_vm_count
        #   @return [::Integer]
        #     [Output Only] Count of VMs that got deleted during rollback.
        # @!attribute [rw] failed_to_create_vm_count
        #   @return [::Integer]
        #     [Output Only] Count of VMs that started creating but encountered an error.
        # @!attribute [rw] status
        #   @return [::String]
        #     [Output Only] Creation status of BulkInsert operation - information if the flow is rolling forward or rolling back.
        #     Check the Status enum for the list of possible values.
        # @!attribute [rw] target_vm_count
        #   @return [::Integer]
        #     [Output Only] Count of VMs originally planned to be created.
        class BulkInsertOperationStatus
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # [Output Only] Creation status of BulkInsert operation - information if the flow is rolling forward or rolling back.
          module Status
            # A value indicating that the enum field is not set.
            UNDEFINED_STATUS = 0

            # Rolling forward - creating VMs.
            CREATING = 455_564_985

            # Done
            DONE = 2_104_194

            # Rolling back - cleaning up after an error.
            ROLLING_BACK = 259_411_649

            STATUS_UNSPECIFIED = 42_133_066
          end
        end

        # A request message for RegionDisks.BulkInsert. See the method description for details.
        # @!attribute [rw] bulk_insert_disk_resource_resource
        #   @return [::Google::Cloud::Compute::V1::BulkInsertDiskResource]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class BulkInsertRegionDiskRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionInstances.BulkInsert. See the method description for details.
        # @!attribute [rw] bulk_insert_instance_resource_resource
        #   @return [::Google::Cloud::Compute::V1::BulkInsertInstanceResource]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class BulkInsertRegionInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] host
        #   @return [::String]
        #     If set, this invalidation rule will only apply to requests with a Host header matching host.
        # @!attribute [rw] path
        #   @return [::String]
        class CacheInvalidationRule
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Message containing what to include in the cache key for a request for Cloud CDN.
        # @!attribute [rw] include_host
        #   @return [::Boolean]
        #     If true, requests to different hosts will be cached separately.
        # @!attribute [rw] include_http_headers
        #   @return [::Array<::String>]
        #     Allows HTTP request headers (by name) to be used in the cache key.
        # @!attribute [rw] include_named_cookies
        #   @return [::Array<::String>]
        #     Allows HTTP cookies (by name) to be used in the cache key. The name=value pair will be used in the cache key Cloud CDN generates.
        # @!attribute [rw] include_protocol
        #   @return [::Boolean]
        #     If true, http and https requests will be cached separately.
        # @!attribute [rw] include_query_string
        #   @return [::Boolean]
        #     If true, include query string parameters in the cache key according to query_string_whitelist and query_string_blacklist. If neither is set, the entire query string will be included. If false, the query string will be excluded from the cache key entirely.
        # @!attribute [rw] query_string_blacklist
        #   @return [::Array<::String>]
        #     Names of query string parameters to exclude in cache keys. All other parameters will be included. Either specify query_string_whitelist or query_string_blacklist, not both. '&' and '=' will be percent encoded and not treated as delimiters.
        # @!attribute [rw] query_string_whitelist
        #   @return [::Array<::String>]
        #     Names of query string parameters to include in cache keys. All other parameters will be excluded. Either specify query_string_whitelist or query_string_blacklist, not both. '&' and '=' will be percent encoded and not treated as delimiters.
        class CacheKeyPolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Settings controlling the volume of requests, connections and retries to this backend service.
        # @!attribute [rw] max_connections
        #   @return [::Integer]
        #     The maximum number of connections to the backend service. If not specified, there is no limit. Not supported when the backend service is referenced by a URL map that is bound to target gRPC proxy that has validateForProxyless field set to true.
        # @!attribute [rw] max_pending_requests
        #   @return [::Integer]
        #     The maximum number of pending requests allowed to the backend service. If not specified, there is no limit. Not supported when the backend service is referenced by a URL map that is bound to target gRPC proxy that has validateForProxyless field set to true.
        # @!attribute [rw] max_requests
        #   @return [::Integer]
        #     The maximum number of parallel requests that allowed to the backend service. If not specified, there is no limit.
        # @!attribute [rw] max_requests_per_connection
        #   @return [::Integer]
        #     Maximum requests for a single connection to the backend service. This parameter is respected by both the HTTP/1.1 and HTTP/2 implementations. If not specified, there is no limit. Setting this parameter to 1 will effectively disable keep alive. Not supported when the backend service is referenced by a URL map that is bound to target gRPC proxy that has validateForProxyless field set to true.
        # @!attribute [rw] max_retries
        #   @return [::Integer]
        #     The maximum number of parallel retries allowed to the backend cluster. If not specified, the default is 1. Not supported when the backend service is referenced by a URL map that is bound to target gRPC proxy that has validateForProxyless field set to true.
        class CircuitBreakers
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for FirewallPolicies.CloneRules. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to update.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] source_firewall_policy
        #   @return [::String]
        #     The firewall policy from which to copy rules.
        class CloneRulesFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkFirewallPolicies.CloneRules. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to update.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] source_firewall_policy
        #   @return [::String]
        #     The firewall policy from which to copy rules.
        class CloneRulesNetworkFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionNetworkFirewallPolicies.CloneRules. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to update.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] source_firewall_policy
        #   @return [::String]
        #     The firewall policy from which to copy rules.
        class CloneRulesRegionNetworkFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a regional Commitment resource. Creating a commitment resource means that you are purchasing a committed use contract with an explicit start and end time. You can create commitments based on vCPUs and memory usage and receive discounted rates. For full details, read Signing Up for Committed Use Discounts.
        # @!attribute [rw] auto_renew
        #   @return [::Boolean]
        #     Specifies whether to enable automatic renewal for the commitment. The default value is false if not specified. The field can be updated until the day of the commitment expiration at 12:00am PST. If the field is set to true, the commitment will be automatically renewed for either one or three years according to the terms of the existing commitment.
        # @!attribute [rw] category
        #   @return [::String]
        #     The category of the commitment. Category MACHINE specifies commitments composed of machine resources such as VCPU or MEMORY, listed in resources. Category LICENSE specifies commitments composed of software licenses, listed in licenseResources. Note that only MACHINE commitments should have a Type specified.
        #     Check the Category enum for the list of possible values.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] end_timestamp
        #   @return [::String]
        #     [Output Only] Commitment end time in RFC3339 text format.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#commitment for commitments.
        # @!attribute [rw] license_resource
        #   @return [::Google::Cloud::Compute::V1::LicenseResourceCommitment]
        #     The license specification required as part of a license commitment.
        # @!attribute [rw] merge_source_commitments
        #   @return [::Array<::String>]
        #     List of source commitments to be merged into a new commitment.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] plan
        #   @return [::String]
        #     The plan for this commitment, which determines duration and discount rate. The currently supported plans are TWELVE_MONTH (1 year), and THIRTY_SIX_MONTH (3 years).
        #     Check the Plan enum for the list of possible values.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] URL of the region where this commitment may be used.
        # @!attribute [rw] reservations
        #   @return [::Array<::Google::Cloud::Compute::V1::Reservation>]
        #     List of create-on-create reseravtions for this commitment.
        # @!attribute [rw] resources
        #   @return [::Array<::Google::Cloud::Compute::V1::ResourceCommitment>]
        #     A list of commitment amounts for particular resources. Note that VCPU and MEMORY resource commitments must occur together.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] split_source_commitment
        #   @return [::String]
        #     Source commitment to be split into a new commitment.
        # @!attribute [rw] start_timestamp
        #   @return [::String]
        #     [Output Only] Commitment start time in RFC3339 text format.
        # @!attribute [rw] status
        #   @return [::String]
        #     [Output Only] Status of the commitment with regards to eventual expiration (each commitment has an end date defined). One of the following values: NOT_YET_ACTIVE, ACTIVE, EXPIRED.
        #     Check the Status enum for the list of possible values.
        # @!attribute [rw] status_message
        #   @return [::String]
        #     [Output Only] An optional, human-readable explanation of the status.
        # @!attribute [rw] type
        #   @return [::String]
        #     The type of commitment, which affects the discount rate and the eligible resources. Type MEMORY_OPTIMIZED specifies a commitment that will only apply to memory optimized machines. Type ACCELERATOR_OPTIMIZED specifies a commitment that will only apply to accelerator optimized machines.
        #     Check the Type enum for the list of possible values.
        class Commitment
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The category of the commitment. Category MACHINE specifies commitments composed of machine resources such as VCPU or MEMORY, listed in resources. Category LICENSE specifies commitments composed of software licenses, listed in licenseResources. Note that only MACHINE commitments should have a Type specified.
          module Category
            # A value indicating that the enum field is not set.
            UNDEFINED_CATEGORY = 0

            CATEGORY_UNSPECIFIED = 509_189_462

            LICENSE = 347_869_217

            MACHINE = 469_553_191
          end

          # The plan for this commitment, which determines duration and discount rate. The currently supported plans are TWELVE_MONTH (1 year), and THIRTY_SIX_MONTH (3 years).
          module Plan
            # A value indicating that the enum field is not set.
            UNDEFINED_PLAN = 0

            INVALID = 530_283_991

            THIRTY_SIX_MONTH = 266_295_942

            TWELVE_MONTH = 173_083_962
          end

          # [Output Only] Status of the commitment with regards to eventual expiration (each commitment has an end date defined). One of the following values: NOT_YET_ACTIVE, ACTIVE, EXPIRED.
          module Status
            # A value indicating that the enum field is not set.
            UNDEFINED_STATUS = 0

            ACTIVE = 314_733_318

            # Deprecate CANCELED status. Will use separate status to differentiate cancel by mergeCud or manual cancellation.
            CANCELLED = 41_957_681

            CREATING = 455_564_985

            EXPIRED = 482_489_093

            NOT_YET_ACTIVE = 20_607_337
          end

          # The type of commitment, which affects the discount rate and the eligible resources. Type MEMORY_OPTIMIZED specifies a commitment that will only apply to memory optimized machines. Type ACCELERATOR_OPTIMIZED specifies a commitment that will only apply to accelerator optimized machines.
          module Type
            # A value indicating that the enum field is not set.
            UNDEFINED_TYPE = 0

            ACCELERATOR_OPTIMIZED = 280_848_403

            ACCELERATOR_OPTIMIZED_A3 = 158_574_526

            COMPUTE_OPTIMIZED = 158_349_023

            COMPUTE_OPTIMIZED_C2D = 383_246_453

            COMPUTE_OPTIMIZED_C3 = 428_004_784

            COMPUTE_OPTIMIZED_C3D = 383_246_484

            COMPUTE_OPTIMIZED_H3 = 428_004_939

            GENERAL_PURPOSE = 299_793_543

            GENERAL_PURPOSE_E2 = 301_911_877

            GENERAL_PURPOSE_N2 = 301_912_156

            GENERAL_PURPOSE_N2D = 232_471_400

            GENERAL_PURPOSE_T2D = 232_477_166

            GRAPHICS_OPTIMIZED = 68_500_563

            MEMORY_OPTIMIZED = 281_753_417

            MEMORY_OPTIMIZED_M3 = 276_301_372

            TYPE_UNSPECIFIED = 437_714_322
          end
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::CommitmentsScopedList}]
        #     A list of CommitmentsScopedList resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#commitmentAggregatedList for aggregated lists of commitments.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class CommitmentAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::CommitmentsScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # Contains a list of Commitment resources.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::Commitment>]
        #     A list of Commitment resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#commitmentList for lists of commitments.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class CommitmentList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] commitments
        #   @return [::Array<::Google::Cloud::Compute::V1::Commitment>]
        #     [Output Only] A list of commitments contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning which replaces the list of commitments when the list is empty.
        class CommitmentsScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # This is deprecated and has no effect. Do not use.
        # @!attribute [rw] iam
        #   @return [::String]
        #     This is deprecated and has no effect. Do not use.
        #     Check the Iam enum for the list of possible values.
        # @!attribute [rw] op
        #   @return [::String]
        #     This is deprecated and has no effect. Do not use.
        #     Check the Op enum for the list of possible values.
        # @!attribute [rw] svc
        #   @return [::String]
        #     This is deprecated and has no effect. Do not use.
        # @!attribute [rw] sys
        #   @return [::String]
        #     This is deprecated and has no effect. Do not use.
        #     Check the Sys enum for the list of possible values.
        # @!attribute [rw] values
        #   @return [::Array<::String>]
        #     This is deprecated and has no effect. Do not use.
        class Condition
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # This is deprecated and has no effect. Do not use.
          # Additional supported values which may be not listed in the enum directly due to technical reasons:
          # NO_ATTR
          module Iam
            # A value indicating that the enum field is not set.
            UNDEFINED_IAM = 0

            # This is deprecated and has no effect. Do not use.
            APPROVER = 357_258_949

            # This is deprecated and has no effect. Do not use.
            ATTRIBUTION = 232_465_503

            # This is deprecated and has no effect. Do not use.
            AUTHORITY = 504_108_835

            # This is deprecated and has no effect. Do not use.
            CREDENTIALS_TYPE = 348_222_141

            # This is deprecated and has no effect. Do not use.
            CREDS_ASSERTION = 332_343_272

            # This is deprecated and has no effect. Do not use.
            JUSTIFICATION_TYPE = 206_147_719

            # This is deprecated and has no effect. Do not use.
            SECURITY_REALM = 526_269_616
          end

          # This is deprecated and has no effect. Do not use.
          module Op
            # A value indicating that the enum field is not set.
            UNDEFINED_OP = 0

            # This is deprecated and has no effect. Do not use.
            DISCHARGED = 266_338_274

            # This is deprecated and has no effect. Do not use.
            EQUALS = 442_201_023

            # This is deprecated and has no effect. Do not use.
            IN = 2341

            # This is deprecated and has no effect. Do not use.
            NOT_EQUALS = 19_718_859

            # This is deprecated and has no effect. Do not use.
            NOT_IN = 161_144_369

            # This is deprecated and has no effect. Do not use.
            NO_OP = 74_481_951
          end

          # This is deprecated and has no effect. Do not use.
          # Additional supported values which may be not listed in the enum directly due to technical reasons:
          # NO_ATTR
          module Sys
            # A value indicating that the enum field is not set.
            UNDEFINED_SYS = 0

            # This is deprecated and has no effect. Do not use.
            IP = 2343

            # This is deprecated and has no effect. Do not use.
            NAME = 2_388_619

            # This is deprecated and has no effect. Do not use.
            REGION = 266_017_524

            # This is deprecated and has no effect. Do not use.
            SERVICE = 17_781_397
          end
        end

        # A set of Confidential Instance options.
        # @!attribute [rw] enable_confidential_compute
        #   @return [::Boolean]
        #     Defines whether the instance should have confidential compute enabled.
        class ConfidentialInstanceConfig
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Message containing connection draining configuration.
        # @!attribute [rw] draining_timeout_sec
        #   @return [::Integer]
        #     Configures a duration timeout for existing requests on a removed backend instance. For supported load balancers and protocols, as described in Enabling connection draining.
        class ConnectionDraining
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # This message defines settings for a consistent hash style load balancer.
        # @!attribute [rw] http_cookie
        #   @return [::Google::Cloud::Compute::V1::ConsistentHashLoadBalancerSettingsHttpCookie]
        #     Hash is based on HTTP Cookie. This field describes a HTTP cookie that will be used as the hash key for the consistent hash load balancer. If the cookie is not present, it will be generated. This field is applicable if the sessionAffinity is set to HTTP_COOKIE. Not supported when the backend service is referenced by a URL map that is bound to target gRPC proxy that has validateForProxyless field set to true.
        # @!attribute [rw] http_header_name
        #   @return [::String]
        #     The hash based on the value of the specified header field. This field is applicable if the sessionAffinity is set to HEADER_FIELD.
        # @!attribute [rw] minimum_ring_size
        #   @return [::Integer]
        #     The minimum number of virtual nodes to use for the hash ring. Defaults to 1024. Larger ring sizes result in more granular load distributions. If the number of hosts in the load balancing pool is larger than the ring size, each host will be assigned a single virtual node.
        class ConsistentHashLoadBalancerSettings
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # The information about the HTTP Cookie on which the hash function is based for load balancing policies that use a consistent hash.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the cookie.
        # @!attribute [rw] path
        #   @return [::String]
        #     Path to set for the cookie.
        # @!attribute [rw] ttl
        #   @return [::Google::Cloud::Compute::V1::Duration]
        #     Lifetime of the cookie.
        class ConsistentHashLoadBalancerSettingsHttpCookie
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # The specification for allowing client-side cross-origin requests. For more information about the W3C recommendation for cross-origin resource sharing (CORS), see Fetch API Living Standard.
        # @!attribute [rw] allow_credentials
        #   @return [::Boolean]
        #     In response to a preflight request, setting this to true indicates that the actual request can include user credentials. This field translates to the Access-Control-Allow-Credentials header. Default is false.
        # @!attribute [rw] allow_headers
        #   @return [::Array<::String>]
        #     Specifies the content for the Access-Control-Allow-Headers header.
        # @!attribute [rw] allow_methods
        #   @return [::Array<::String>]
        #     Specifies the content for the Access-Control-Allow-Methods header.
        # @!attribute [rw] allow_origin_regexes
        #   @return [::Array<::String>]
        #     Specifies a regular expression that matches allowed origins. For more information about the regular expression syntax, see Syntax. An origin is allowed if it matches either an item in allowOrigins or an item in allowOriginRegexes. Regular expressions can only be used when the loadBalancingScheme is set to INTERNAL_SELF_MANAGED.
        # @!attribute [rw] allow_origins
        #   @return [::Array<::String>]
        #     Specifies the list of origins that is allowed to do CORS requests. An origin is allowed if it matches either an item in allowOrigins or an item in allowOriginRegexes.
        # @!attribute [rw] disabled
        #   @return [::Boolean]
        #     If true, the setting specifies the CORS policy is disabled. The default value of false, which indicates that the CORS policy is in effect.
        # @!attribute [rw] expose_headers
        #   @return [::Array<::String>]
        #     Specifies the content for the Access-Control-Expose-Headers header.
        # @!attribute [rw] max_age
        #   @return [::Integer]
        #     Specifies how long results of a preflight request can be cached in seconds. This field translates to the Access-Control-Max-Age header.
        class CorsPolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceGroupManagers.CreateInstances. See the method description for details.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     The name of the managed instance group. It should conform to RFC1035.
        # @!attribute [rw] instance_group_managers_create_instances_request_resource
        #   @return [::Google::Cloud::Compute::V1::InstanceGroupManagersCreateInstancesRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where the managed instance group is located. It should conform to RFC1035.
        class CreateInstancesInstanceGroupManagerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionInstanceGroupManagers.CreateInstances. See the method description for details.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     The name of the managed instance group. It should conform to RFC1035.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region where the managed instance group is located. It should conform to RFC1035.
        # @!attribute [rw] region_instance_group_managers_create_instances_request_resource
        #   @return [::Google::Cloud::Compute::V1::RegionInstanceGroupManagersCreateInstancesRequest]
        #     The body resource for this request
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class CreateInstancesRegionInstanceGroupManagerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Disks.CreateSnapshot. See the method description for details.
        # @!attribute [rw] disk
        #   @return [::String]
        #     Name of the persistent disk to snapshot.
        # @!attribute [rw] guest_flush
        #   @return [::Boolean]
        #     [Input Only] Whether to attempt an application consistent snapshot by informing the OS to prepare for the snapshot process.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] snapshot_resource
        #   @return [::Google::Cloud::Compute::V1::Snapshot]
        #     The body resource for this request
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class CreateSnapshotDiskRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionDisks.CreateSnapshot. See the method description for details.
        # @!attribute [rw] disk
        #   @return [::String]
        #     Name of the regional persistent disk to snapshot.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] snapshot_resource
        #   @return [::Google::Cloud::Compute::V1::Snapshot]
        #     The body resource for this request
        class CreateSnapshotRegionDiskRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] kms_key_name
        #   @return [::String]
        #     The name of the encryption key that is stored in Google Cloud KMS. For example: "kmsKeyName": "projects/kms_project_id/locations/region/keyRings/ key_region/cryptoKeys/key The fully-qualifed key name may be returned for resource GET requests. For example: "kmsKeyName": "projects/kms_project_id/locations/region/keyRings/ key_region/cryptoKeys/key /cryptoKeyVersions/1
        # @!attribute [rw] kms_key_service_account
        #   @return [::String]
        #     The service account being used for the encryption request for the given KMS key. If absent, the Compute Engine default service account is used. For example: "kmsKeyServiceAccount": "name@project_id.iam.gserviceaccount.com/
        # @!attribute [rw] raw_key
        #   @return [::String]
        #     Specifies a 256-bit customer-supplied encryption key, encoded in RFC 4648 base64 to either encrypt or decrypt this resource. You can provide either the rawKey or the rsaEncryptedKey. For example: "rawKey": "SGVsbG8gZnJvbSBHb29nbGUgQ2xvdWQgUGxhdGZvcm0="
        # @!attribute [rw] rsa_encrypted_key
        #   @return [::String]
        #     Specifies an RFC 4648 base64 encoded, RSA-wrapped 2048-bit customer-supplied encryption key to either encrypt or decrypt this resource. You can provide either the rawKey or the rsaEncryptedKey. For example: "rsaEncryptedKey": "ieCx/NcW06PcT7Ep1X6LUTc/hLvUDYyzSZPPVCVPTVEohpeHASqC8uw5TzyO9U+Fka9JFH z0mBibXUInrC/jEk014kCK/NPjYgEMOyssZ4ZINPKxlUh2zn1bV+MCaTICrdmuSBTWlUUiFoD D6PYznLwh8ZNdaheCeZ8ewEXgFQ8V+sDroLaN3Xs3MDTXQEMMoNUXMCZEIpg9Vtp9x2oe==" The key must meet the following requirements before you can provide it to Compute Engine: 1. The key is wrapped using a RSA public key certificate provided by Google. 2. After being wrapped, the key must be encoded in RFC 4648 base64 encoding. Gets the RSA public key certificate provided by Google at: https://cloud-certs.storage.googleapis.com/google-cloud-csek-ingress.pem
        # @!attribute [rw] sha256
        #   @return [::String]
        #     [Output only] The RFC 4648 base64 encoded SHA-256 hash of the customer-supplied encryption key that protects this resource.
        class CustomerEncryptionKey
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] disk_encryption_key
        #   @return [::Google::Cloud::Compute::V1::CustomerEncryptionKey]
        #     Decrypts data associated with the disk with a customer-supplied encryption key.
        # @!attribute [rw] source
        #   @return [::String]
        #     Specifies a valid partial or full URL to an existing Persistent Disk resource. This field is only applicable for persistent disks. For example: "source": "/compute/v1/projects/project_id/zones/zone/disks/ disk_name
        class CustomerEncryptionKeyProtectedDisk
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] key
        #   @return [::String]
        #     [Output Only] A key that provides more detail on the warning being returned. For example, for warnings where there are no results in a list request for a particular zone, this key might be scope and the key value might be the zone name. Other examples might be a key indicating a deprecated resource and a suggested replacement, or a warning about invalid network settings (for example, if an instance attempts to perform IP forwarding but is not enabled for IP forwarding).
        # @!attribute [rw] value
        #   @return [::String]
        #     [Output Only] A warning data value corresponding to the key.
        class Data
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.DeleteAccessConfig. See the method description for details.
        # @!attribute [rw] access_config
        #   @return [::String]
        #     The name of the access config to delete.
        # @!attribute [rw] instance
        #   @return [::String]
        #     The instance name for this request.
        # @!attribute [rw] network_interface
        #   @return [::String]
        #     The name of the network interface.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class DeleteAccessConfigInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Addresses.Delete. See the method description for details.
        # @!attribute [rw] address
        #   @return [::String]
        #     Name of the address resource to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteAddressRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Autoscalers.Delete. See the method description for details.
        # @!attribute [rw] autoscaler
        #   @return [::String]
        #     Name of the autoscaler to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     Name of the zone for this request.
        class DeleteAutoscalerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for BackendBuckets.Delete. See the method description for details.
        # @!attribute [rw] backend_bucket
        #   @return [::String]
        #     Name of the BackendBucket resource to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteBackendBucketRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for BackendServices.Delete. See the method description for details.
        # @!attribute [rw] backend_service
        #   @return [::String]
        #     Name of the BackendService resource to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteBackendServiceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Disks.Delete. See the method description for details.
        # @!attribute [rw] disk
        #   @return [::String]
        #     Name of the persistent disk to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class DeleteDiskRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ExternalVpnGateways.Delete. See the method description for details.
        # @!attribute [rw] external_vpn_gateway
        #   @return [::String]
        #     Name of the externalVpnGateways to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteExternalVpnGatewayRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for FirewallPolicies.Delete. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to delete.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Firewalls.Delete. See the method description for details.
        # @!attribute [rw] firewall
        #   @return [::String]
        #     Name of the firewall rule to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteFirewallRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ForwardingRules.Delete. See the method description for details.
        # @!attribute [rw] forwarding_rule
        #   @return [::String]
        #     Name of the ForwardingRule resource to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteForwardingRuleRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for GlobalAddresses.Delete. See the method description for details.
        # @!attribute [rw] address
        #   @return [::String]
        #     Name of the address resource to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteGlobalAddressRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for GlobalForwardingRules.Delete. See the method description for details.
        # @!attribute [rw] forwarding_rule
        #   @return [::String]
        #     Name of the ForwardingRule resource to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteGlobalForwardingRuleRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for GlobalNetworkEndpointGroups.Delete. See the method description for details.
        # @!attribute [rw] network_endpoint_group
        #   @return [::String]
        #     The name of the network endpoint group to delete. It should comply with RFC1035.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteGlobalNetworkEndpointGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for GlobalOperations.Delete. See the method description for details.
        # @!attribute [rw] operation
        #   @return [::String]
        #     Name of the Operations resource to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        class DeleteGlobalOperationRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A response message for GlobalOperations.Delete. See the method description for details.
        class DeleteGlobalOperationResponse
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for GlobalOrganizationOperations.Delete. See the method description for details.
        # @!attribute [rw] operation
        #   @return [::String]
        #     Name of the Operations resource to delete.
        # @!attribute [rw] parent_id
        #   @return [::String]
        #     Parent ID for this request.
        class DeleteGlobalOrganizationOperationRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A response message for GlobalOrganizationOperations.Delete. See the method description for details.
        class DeleteGlobalOrganizationOperationResponse
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for GlobalPublicDelegatedPrefixes.Delete. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] public_delegated_prefix
        #   @return [::String]
        #     Name of the PublicDelegatedPrefix resource to delete.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteGlobalPublicDelegatedPrefixeRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for HealthChecks.Delete. See the method description for details.
        # @!attribute [rw] health_check
        #   @return [::String]
        #     Name of the HealthCheck resource to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteHealthCheckRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Images.Delete. See the method description for details.
        # @!attribute [rw] image
        #   @return [::String]
        #     Name of the image resource to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteImageRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceGroupManagers.Delete. See the method description for details.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     The name of the managed instance group to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where the managed instance group is located.
        class DeleteInstanceGroupManagerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceGroups.Delete. See the method description for details.
        # @!attribute [rw] instance_group
        #   @return [::String]
        #     The name of the instance group to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where the instance group is located.
        class DeleteInstanceGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.Delete. See the method description for details.
        # @!attribute [rw] instance
        #   @return [::String]
        #     Name of the instance resource to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class DeleteInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceTemplates.Delete. See the method description for details.
        # @!attribute [rw] instance_template
        #   @return [::String]
        #     The name of the instance template to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteInstanceTemplateRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceGroupManagers.DeleteInstances. See the method description for details.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     The name of the managed instance group.
        # @!attribute [rw] instance_group_managers_delete_instances_request_resource
        #   @return [::Google::Cloud::Compute::V1::InstanceGroupManagersDeleteInstancesRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where the managed instance group is located.
        class DeleteInstancesInstanceGroupManagerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionInstanceGroupManagers.DeleteInstances. See the method description for details.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     Name of the managed instance group.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] region_instance_group_managers_delete_instances_request_resource
        #   @return [::Google::Cloud::Compute::V1::RegionInstanceGroupManagersDeleteInstancesRequest]
        #     The body resource for this request
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteInstancesRegionInstanceGroupManagerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InterconnectAttachments.Delete. See the method description for details.
        # @!attribute [rw] interconnect_attachment
        #   @return [::String]
        #     Name of the interconnect attachment to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteInterconnectAttachmentRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Interconnects.Delete. See the method description for details.
        # @!attribute [rw] interconnect
        #   @return [::String]
        #     Name of the interconnect to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteInterconnectRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Licenses.Delete. See the method description for details.
        # @!attribute [rw] license
        #   @return [::String]
        #     Name of the license resource to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteLicenseRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for MachineImages.Delete. See the method description for details.
        # @!attribute [rw] machine_image
        #   @return [::String]
        #     The name of the machine image to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteMachineImageRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkAttachments.Delete. See the method description for details.
        # @!attribute [rw] network_attachment
        #   @return [::String]
        #     Name of the NetworkAttachment resource to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region of this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000). end_interface: MixerMutationRequestBuilder
        class DeleteNetworkAttachmentRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkEdgeSecurityServices.Delete. See the method description for details.
        # @!attribute [rw] network_edge_security_service
        #   @return [::String]
        #     Name of the network edge security service to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteNetworkEdgeSecurityServiceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkEndpointGroups.Delete. See the method description for details.
        # @!attribute [rw] network_endpoint_group
        #   @return [::String]
        #     The name of the network endpoint group to delete. It should comply with RFC1035.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where the network endpoint group is located. It should comply with RFC1035.
        class DeleteNetworkEndpointGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkFirewallPolicies.Delete. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteNetworkFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Networks.Delete. See the method description for details.
        # @!attribute [rw] network
        #   @return [::String]
        #     Name of the network to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteNetworkRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NodeGroups.Delete. See the method description for details.
        # @!attribute [rw] node_group
        #   @return [::String]
        #     Name of the NodeGroup resource to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class DeleteNodeGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NodeTemplates.Delete. See the method description for details.
        # @!attribute [rw] node_template
        #   @return [::String]
        #     Name of the NodeTemplate resource to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteNodeTemplateRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NodeGroups.DeleteNodes. See the method description for details.
        # @!attribute [rw] node_group
        #   @return [::String]
        #     Name of the NodeGroup resource whose nodes will be deleted.
        # @!attribute [rw] node_groups_delete_nodes_request_resource
        #   @return [::Google::Cloud::Compute::V1::NodeGroupsDeleteNodesRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class DeleteNodesNodeGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for PacketMirrorings.Delete. See the method description for details.
        # @!attribute [rw] packet_mirroring
        #   @return [::String]
        #     Name of the PacketMirroring resource to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeletePacketMirroringRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceGroupManagers.DeletePerInstanceConfigs. See the method description for details.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     The name of the managed instance group. It should conform to RFC1035.
        # @!attribute [rw] instance_group_managers_delete_per_instance_configs_req_resource
        #   @return [::Google::Cloud::Compute::V1::InstanceGroupManagersDeletePerInstanceConfigsReq]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where the managed instance group is located. It should conform to RFC1035.
        class DeletePerInstanceConfigsInstanceGroupManagerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionInstanceGroupManagers.DeletePerInstanceConfigs. See the method description for details.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     The name of the managed instance group. It should conform to RFC1035.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request, should conform to RFC1035.
        # @!attribute [rw] region_instance_group_manager_delete_instance_config_req_resource
        #   @return [::Google::Cloud::Compute::V1::RegionInstanceGroupManagerDeleteInstanceConfigReq]
        #     The body resource for this request
        class DeletePerInstanceConfigsRegionInstanceGroupManagerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for PublicAdvertisedPrefixes.Delete. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] public_advertised_prefix
        #   @return [::String]
        #     Name of the PublicAdvertisedPrefix resource to delete.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeletePublicAdvertisedPrefixeRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for PublicDelegatedPrefixes.Delete. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] public_delegated_prefix
        #   @return [::String]
        #     Name of the PublicDelegatedPrefix resource to delete.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region of this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeletePublicDelegatedPrefixeRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionAutoscalers.Delete. See the method description for details.
        # @!attribute [rw] autoscaler
        #   @return [::String]
        #     Name of the autoscaler to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteRegionAutoscalerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionBackendServices.Delete. See the method description for details.
        # @!attribute [rw] backend_service
        #   @return [::String]
        #     Name of the BackendService resource to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteRegionBackendServiceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionDisks.Delete. See the method description for details.
        # @!attribute [rw] disk
        #   @return [::String]
        #     Name of the regional persistent disk to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteRegionDiskRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionHealthChecks.Delete. See the method description for details.
        # @!attribute [rw] health_check
        #   @return [::String]
        #     Name of the HealthCheck resource to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteRegionHealthCheckRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionHealthCheckServices.Delete. See the method description for details.
        # @!attribute [rw] health_check_service
        #   @return [::String]
        #     Name of the HealthCheckService to delete. The name must be 1-63 characters long, and comply with RFC1035.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteRegionHealthCheckServiceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionInstanceGroupManagers.Delete. See the method description for details.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     Name of the managed instance group to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteRegionInstanceGroupManagerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionInstanceTemplates.Delete. See the method description for details.
        # @!attribute [rw] instance_template
        #   @return [::String]
        #     The name of the instance template to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteRegionInstanceTemplateRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionNetworkEndpointGroups.Delete. See the method description for details.
        # @!attribute [rw] network_endpoint_group
        #   @return [::String]
        #     The name of the network endpoint group to delete. It should comply with RFC1035.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region where the network endpoint group is located. It should comply with RFC1035.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteRegionNetworkEndpointGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionNetworkFirewallPolicies.Delete. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteRegionNetworkFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionNotificationEndpoints.Delete. See the method description for details.
        # @!attribute [rw] notification_endpoint
        #   @return [::String]
        #     Name of the NotificationEndpoint resource to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteRegionNotificationEndpointRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionOperations.Delete. See the method description for details.
        # @!attribute [rw] operation
        #   @return [::String]
        #     Name of the Operations resource to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        class DeleteRegionOperationRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A response message for RegionOperations.Delete. See the method description for details.
        class DeleteRegionOperationResponse
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionSecurityPolicies.Delete. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] security_policy
        #   @return [::String]
        #     Name of the security policy to delete.
        class DeleteRegionSecurityPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionSslCertificates.Delete. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] ssl_certificate
        #   @return [::String]
        #     Name of the SslCertificate resource to delete.
        class DeleteRegionSslCertificateRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionSslPolicies.Delete. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] ssl_policy
        #   @return [::String]
        #     Name of the SSL policy to delete. The name must be 1-63 characters long, and comply with RFC1035.
        class DeleteRegionSslPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionTargetHttpProxies.Delete. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_http_proxy
        #   @return [::String]
        #     Name of the TargetHttpProxy resource to delete.
        class DeleteRegionTargetHttpProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionTargetHttpsProxies.Delete. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_https_proxy
        #   @return [::String]
        #     Name of the TargetHttpsProxy resource to delete.
        class DeleteRegionTargetHttpsProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionTargetTcpProxies.Delete. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_tcp_proxy
        #   @return [::String]
        #     Name of the TargetTcpProxy resource to delete.
        class DeleteRegionTargetTcpProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionUrlMaps.Delete. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     begin_interface: MixerMutationRequestBuilder Request ID to support idempotency.
        # @!attribute [rw] url_map
        #   @return [::String]
        #     Name of the UrlMap resource to delete.
        class DeleteRegionUrlMapRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Reservations.Delete. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] reservation
        #   @return [::String]
        #     Name of the reservation to delete.
        # @!attribute [rw] zone
        #   @return [::String]
        #     Name of the zone for this request.
        class DeleteReservationRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ResourcePolicies.Delete. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] resource_policy
        #   @return [::String]
        #     Name of the resource policy to delete.
        class DeleteResourcePolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Routes.Delete. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] route
        #   @return [::String]
        #     Name of the Route resource to delete.
        class DeleteRouteRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Routers.Delete. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] router
        #   @return [::String]
        #     Name of the Router resource to delete.
        class DeleteRouterRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for SecurityPolicies.Delete. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] security_policy
        #   @return [::String]
        #     Name of the security policy to delete.
        class DeleteSecurityPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ServiceAttachments.Delete. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region of this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] service_attachment
        #   @return [::String]
        #     Name of the ServiceAttachment resource to delete.
        class DeleteServiceAttachmentRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for BackendBuckets.DeleteSignedUrlKey. See the method description for details.
        # @!attribute [rw] backend_bucket
        #   @return [::String]
        #     Name of the BackendBucket resource to which the Signed URL Key should be added. The name should conform to RFC1035.
        # @!attribute [rw] key_name
        #   @return [::String]
        #     The name of the Signed URL Key to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteSignedUrlKeyBackendBucketRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for BackendServices.DeleteSignedUrlKey. See the method description for details.
        # @!attribute [rw] backend_service
        #   @return [::String]
        #     Name of the BackendService resource to which the Signed URL Key should be added. The name should conform to RFC1035.
        # @!attribute [rw] key_name
        #   @return [::String]
        #     The name of the Signed URL Key to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeleteSignedUrlKeyBackendServiceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Snapshots.Delete. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] snapshot
        #   @return [::String]
        #     Name of the Snapshot resource to delete.
        class DeleteSnapshotRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for SslCertificates.Delete. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] ssl_certificate
        #   @return [::String]
        #     Name of the SslCertificate resource to delete.
        class DeleteSslCertificateRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for SslPolicies.Delete. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] ssl_policy
        #   @return [::String]
        #     Name of the SSL policy to delete. The name must be 1-63 characters long, and comply with RFC1035.
        class DeleteSslPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Subnetworks.Delete. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] subnetwork
        #   @return [::String]
        #     Name of the Subnetwork resource to delete.
        class DeleteSubnetworkRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetGrpcProxies.Delete. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_grpc_proxy
        #   @return [::String]
        #     Name of the TargetGrpcProxy resource to delete.
        class DeleteTargetGrpcProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetHttpProxies.Delete. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_http_proxy
        #   @return [::String]
        #     Name of the TargetHttpProxy resource to delete.
        class DeleteTargetHttpProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetHttpsProxies.Delete. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_https_proxy
        #   @return [::String]
        #     Name of the TargetHttpsProxy resource to delete.
        class DeleteTargetHttpsProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetInstances.Delete. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_instance
        #   @return [::String]
        #     Name of the TargetInstance resource to delete.
        # @!attribute [rw] zone
        #   @return [::String]
        #     Name of the zone scoping this request.
        class DeleteTargetInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetPools.Delete. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_pool
        #   @return [::String]
        #     Name of the TargetPool resource to delete.
        class DeleteTargetPoolRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetSslProxies.Delete. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_ssl_proxy
        #   @return [::String]
        #     Name of the TargetSslProxy resource to delete.
        class DeleteTargetSslProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetTcpProxies.Delete. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_tcp_proxy
        #   @return [::String]
        #     Name of the TargetTcpProxy resource to delete.
        class DeleteTargetTcpProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetVpnGateways.Delete. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_vpn_gateway
        #   @return [::String]
        #     Name of the target VPN gateway to delete.
        class DeleteTargetVpnGatewayRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for UrlMaps.Delete. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] url_map
        #   @return [::String]
        #     Name of the UrlMap resource to delete.
        class DeleteUrlMapRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for VpnGateways.Delete. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] vpn_gateway
        #   @return [::String]
        #     Name of the VPN gateway to delete.
        class DeleteVpnGatewayRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for VpnTunnels.Delete. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] vpn_tunnel
        #   @return [::String]
        #     Name of the VpnTunnel resource to delete.
        class DeleteVpnTunnelRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ZoneOperations.Delete. See the method description for details.
        # @!attribute [rw] operation
        #   @return [::String]
        #     Name of the Operations resource to delete.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     Name of the zone for this request.
        class DeleteZoneOperationRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A response message for ZoneOperations.Delete. See the method description for details.
        class DeleteZoneOperationResponse
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] I_p_protocol
        #   @return [::String]
        #     The IP protocol to which this rule applies. The protocol type is required when creating a firewall rule. This value can either be one of the following well known protocol strings (tcp, udp, icmp, esp, ah, ipip, sctp) or the IP protocol number.
        # @!attribute [rw] ports
        #   @return [::Array<::String>]
        #     An optional list of ports to which this rule applies. This field is only applicable for the UDP or TCP protocol. Each entry must be either an integer or a range. If not specified, this rule applies to connections through any port. Example inputs include: ["22"], ["80","443"], and ["12345-12349"].
        class Denied
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Images.Deprecate. See the method description for details.
        # @!attribute [rw] deprecation_status_resource
        #   @return [::Google::Cloud::Compute::V1::DeprecationStatus]
        #     The body resource for this request
        # @!attribute [rw] image
        #   @return [::String]
        #     Image name.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DeprecateImageRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Deprecation status for a public resource.
        # @!attribute [rw] deleted
        #   @return [::String]
        #     An optional RFC3339 timestamp on or after which the state of this resource is intended to change to DELETED. This is only informational and the status will not change unless the client explicitly changes it.
        # @!attribute [rw] deprecated
        #   @return [::String]
        #     An optional RFC3339 timestamp on or after which the state of this resource is intended to change to DEPRECATED. This is only informational and the status will not change unless the client explicitly changes it.
        # @!attribute [rw] obsolete
        #   @return [::String]
        #     An optional RFC3339 timestamp on or after which the state of this resource is intended to change to OBSOLETE. This is only informational and the status will not change unless the client explicitly changes it.
        # @!attribute [rw] replacement
        #   @return [::String]
        #     The URL of the suggested replacement for a deprecated resource. The suggested replacement resource must be the same kind of resource as the deprecated resource.
        # @!attribute [rw] state
        #   @return [::String]
        #     The deprecation state of this resource. This can be ACTIVE, DEPRECATED, OBSOLETE, or DELETED. Operations which communicate the end of life date for an image, can use ACTIVE. Operations which create a new resource using a DEPRECATED resource will return successfully, but with a warning indicating the deprecated resource and recommending its replacement. Operations which use OBSOLETE or DELETED resources will be rejected and result in an error.
        #     Check the State enum for the list of possible values.
        class DeprecationStatus
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The deprecation state of this resource. This can be ACTIVE, DEPRECATED, OBSOLETE, or DELETED. Operations which communicate the end of life date for an image, can use ACTIVE. Operations which create a new resource using a DEPRECATED resource will return successfully, but with a warning indicating the deprecated resource and recommending its replacement. Operations which use OBSOLETE or DELETED resources will be rejected and result in an error.
          module State
            # A value indicating that the enum field is not set.
            UNDEFINED_STATE = 0

            ACTIVE = 314_733_318

            DELETED = 120_962_041

            DEPRECATED = 463_360_435

            OBSOLETE = 66_532_761
          end
        end

        # A request message for Instances.DetachDisk. See the method description for details.
        # @!attribute [rw] device_name
        #   @return [::String]
        #     The device name of the disk to detach. Make a get() request on the instance to view currently attached disks and device names.
        # @!attribute [rw] instance
        #   @return [::String]
        #     Instance name for this request.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class DetachDiskInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for GlobalNetworkEndpointGroups.DetachNetworkEndpoints. See the method description for details.
        # @!attribute [rw] global_network_endpoint_groups_detach_endpoints_request_resource
        #   @return [::Google::Cloud::Compute::V1::GlobalNetworkEndpointGroupsDetachEndpointsRequest]
        #     The body resource for this request
        # @!attribute [rw] network_endpoint_group
        #   @return [::String]
        #     The name of the network endpoint group where you are removing network endpoints. It should comply with RFC1035.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DetachNetworkEndpointsGlobalNetworkEndpointGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkEndpointGroups.DetachNetworkEndpoints. See the method description for details.
        # @!attribute [rw] network_endpoint_group
        #   @return [::String]
        #     The name of the network endpoint group where you are removing network endpoints. It should comply with RFC1035.
        # @!attribute [rw] network_endpoint_groups_detach_endpoints_request_resource
        #   @return [::Google::Cloud::Compute::V1::NetworkEndpointGroupsDetachEndpointsRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where the network endpoint group is located. It should comply with RFC1035.
        class DetachNetworkEndpointsNetworkEndpointGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionNetworkEndpointGroups.DetachNetworkEndpoints. See the method description for details.
        # @!attribute [rw] network_endpoint_group
        #   @return [::String]
        #     The name of the network endpoint group you are detaching network endpoints from. It should comply with RFC1035.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region where the network endpoint group is located. It should comply with RFC1035.
        # @!attribute [rw] region_network_endpoint_groups_detach_endpoints_request_resource
        #   @return [::Google::Cloud::Compute::V1::RegionNetworkEndpointGroupsDetachEndpointsRequest]
        #     The body resource for this request
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000). end_interface: MixerMutationRequestBuilder
        class DetachNetworkEndpointsRegionNetworkEndpointGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Projects.DisableXpnHost. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DisableXpnHostProjectRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Projects.DisableXpnResource. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] projects_disable_xpn_resource_request_resource
        #   @return [::Google::Cloud::Compute::V1::ProjectsDisableXpnResourceRequest]
        #     The body resource for this request
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class DisableXpnResourceProjectRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a Persistent Disk resource. Google Compute Engine has two Disk resources: * [Zonal](/compute/docs/reference/rest/v1/disks) * [Regional](/compute/docs/reference/rest/v1/regionDisks) Persistent disks are required for running your VM instances. Create both boot and non-boot (data) persistent disks. For more information, read Persistent Disks. For more storage options, read Storage options. The disks resource represents a zonal persistent disk. For more information, read Zonal persistent disks. The regionDisks resource represents a regional persistent disk. For more information, read Regional resources.
        # @!attribute [rw] architecture
        #   @return [::String]
        #     The architecture of the disk. Valid values are ARM64 or X86_64.
        #     Check the Architecture enum for the list of possible values.
        # @!attribute [rw] async_primary_disk
        #   @return [::Google::Cloud::Compute::V1::DiskAsyncReplication]
        #     Disk asynchronously replicated into this disk.
        # @!attribute [rw] async_secondary_disks
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::DiskAsyncReplicationList}]
        #     [Output Only] A list of disks this disk is asynchronously replicated to.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] disk_encryption_key
        #   @return [::Google::Cloud::Compute::V1::CustomerEncryptionKey]
        #     Encrypts the disk using a customer-supplied encryption key or a customer-managed encryption key. Encryption keys do not protect access to metadata of the disk. After you encrypt a disk with a customer-supplied key, you must provide the same key if you use the disk later. For example, to create a disk snapshot, to create a disk image, to create a machine image, or to attach the disk to a virtual machine. After you encrypt a disk with a customer-managed key, the diskEncryptionKey.kmsKeyName is set to a key *version* name once the disk is created. The disk is encrypted with this version of the key. In the response, diskEncryptionKey.kmsKeyName appears in the following format: "diskEncryptionKey.kmsKeyName": "projects/kms_project_id/locations/region/keyRings/ key_region/cryptoKeys/key /cryptoKeysVersions/version If you do not provide an encryption key when creating the disk, then the disk is encrypted using an automatically generated key and you don't need to provide a key to use the disk later.
        # @!attribute [rw] guest_os_features
        #   @return [::Array<::Google::Cloud::Compute::V1::GuestOsFeature>]
        #     A list of features to enable on the guest operating system. Applicable only for bootable images. Read Enabling guest operating system features to see a list of available options.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#disk for disks.
        # @!attribute [rw] label_fingerprint
        #   @return [::String]
        #     A fingerprint for the labels being applied to this disk, which is essentially a hash of the labels set used for optimistic locking. The fingerprint is initially generated by Compute Engine and changes after every request to modify or update labels. You must always provide an up-to-date fingerprint hash in order to update or change labels, otherwise the request will fail with error 412 conditionNotMet. To see the latest fingerprint, make a get() request to retrieve a disk.
        # @!attribute [rw] labels
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     Labels to apply to this disk. These can be later modified by the setLabels method.
        # @!attribute [rw] last_attach_timestamp
        #   @return [::String]
        #     [Output Only] Last attach timestamp in RFC3339 text format.
        # @!attribute [rw] last_detach_timestamp
        #   @return [::String]
        #     [Output Only] Last detach timestamp in RFC3339 text format.
        # @!attribute [rw] license_codes
        #   @return [::Array<::Integer>]
        #     Integer license codes indicating which licenses are attached to this disk.
        # @!attribute [rw] licenses
        #   @return [::Array<::String>]
        #     A list of publicly visible licenses. Reserved for Google's use.
        # @!attribute [rw] location_hint
        #   @return [::String]
        #     An opaque location hint used to place the disk close to other resources. This field is for use by internal tools that use the public API.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] options
        #   @return [::String]
        #     Internal use only.
        # @!attribute [rw] params
        #   @return [::Google::Cloud::Compute::V1::DiskParams]
        #     Input only. [Input Only] Additional params passed with the request, but not persisted as part of resource payload.
        # @!attribute [rw] physical_block_size_bytes
        #   @return [::Integer]
        #     Physical block size of the persistent disk, in bytes. If not present in a request, a default value is used. The currently supported size is 4096, other sizes may be added in the future. If an unsupported value is requested, the error message will list the supported values for the caller's project.
        # @!attribute [rw] provisioned_iops
        #   @return [::Integer]
        #     Indicates how many IOPS to provision for the disk. This sets the number of I/O operations per second that the disk can handle. Values must be between 10,000 and 120,000. For more details, see the Extreme persistent disk documentation.
        # @!attribute [rw] provisioned_throughput
        #   @return [::Integer]
        #     Indicates how much throughput to provision for the disk. This sets the number of throughput mb per second that the disk can handle. Values must be between 1 and 7,124.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] URL of the region where the disk resides. Only applicable for regional resources. You must specify this field as part of the HTTP request URL. It is not settable as a field in the request body.
        # @!attribute [rw] replica_zones
        #   @return [::Array<::String>]
        #     URLs of the zones where the disk should be replicated to. Only applicable for regional resources.
        # @!attribute [rw] resource_policies
        #   @return [::Array<::String>]
        #     Resource policies applied to this disk for automatic snapshot creations.
        # @!attribute [rw] resource_status
        #   @return [::Google::Cloud::Compute::V1::DiskResourceStatus]
        #     [Output Only] Status information for the disk resource.
        # @!attribute [rw] satisfies_pzs
        #   @return [::Boolean]
        #     [Output Only] Reserved for future use.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined fully-qualified URL for this resource.
        # @!attribute [rw] size_gb
        #   @return [::Integer]
        #     Size, in GB, of the persistent disk. You can specify this field when creating a persistent disk using the sourceImage, sourceSnapshot, or sourceDisk parameter, or specify it alone to create an empty persistent disk. If you specify this field along with a source, the value of sizeGb must not be less than the size of the source. Acceptable values are greater than 0.
        # @!attribute [rw] source_consistency_group_policy
        #   @return [::String]
        #     [Output Only] URL of the DiskConsistencyGroupPolicy for a secondary disk that was created using a consistency group.
        # @!attribute [rw] source_consistency_group_policy_id
        #   @return [::String]
        #     [Output Only] ID of the DiskConsistencyGroupPolicy for a secondary disk that was created using a consistency group.
        # @!attribute [rw] source_disk
        #   @return [::String]
        #     The source disk used to create this disk. You can provide this as a partial or full URL to the resource. For example, the following are valid values: - https://www.googleapis.com/compute/v1/projects/project/zones/zone /disks/disk - https://www.googleapis.com/compute/v1/projects/project/regions/region /disks/disk - projects/project/zones/zone/disks/disk - projects/project/regions/region/disks/disk - zones/zone/disks/disk - regions/region/disks/disk
        # @!attribute [rw] source_disk_id
        #   @return [::String]
        #     [Output Only] The unique ID of the disk used to create this disk. This value identifies the exact disk that was used to create this persistent disk. For example, if you created the persistent disk from a disk that was later deleted and recreated under the same name, the source disk ID would identify the exact version of the disk that was used.
        # @!attribute [rw] source_image
        #   @return [::String]
        #     The source image used to create this disk. If the source image is deleted, this field will not be set. To create a disk with one of the public operating system images, specify the image by its family name. For example, specify family/debian-9 to use the latest Debian 9 image: projects/debian-cloud/global/images/family/debian-9 Alternatively, use a specific version of a public operating system image: projects/debian-cloud/global/images/debian-9-stretch-vYYYYMMDD To create a disk with a custom image that you created, specify the image name in the following format: global/images/my-custom-image You can also specify a custom image by its image family, which returns the latest version of the image in that family. Replace the image name with family/family-name: global/images/family/my-image-family
        # @!attribute [rw] source_image_encryption_key
        #   @return [::Google::Cloud::Compute::V1::CustomerEncryptionKey]
        #     The customer-supplied encryption key of the source image. Required if the source image is protected by a customer-supplied encryption key.
        # @!attribute [rw] source_image_id
        #   @return [::String]
        #     [Output Only] The ID value of the image used to create this disk. This value identifies the exact image that was used to create this persistent disk. For example, if you created the persistent disk from an image that was later deleted and recreated under the same name, the source image ID would identify the exact version of the image that was used.
        # @!attribute [rw] source_snapshot
        #   @return [::String]
        #     The source snapshot used to create this disk. You can provide this as a partial or full URL to the resource. For example, the following are valid values: - https://www.googleapis.com/compute/v1/projects/project /global/snapshots/snapshot - projects/project/global/snapshots/snapshot - global/snapshots/snapshot
        # @!attribute [rw] source_snapshot_encryption_key
        #   @return [::Google::Cloud::Compute::V1::CustomerEncryptionKey]
        #     The customer-supplied encryption key of the source snapshot. Required if the source snapshot is protected by a customer-supplied encryption key.
        # @!attribute [rw] source_snapshot_id
        #   @return [::String]
        #     [Output Only] The unique ID of the snapshot used to create this disk. This value identifies the exact snapshot that was used to create this persistent disk. For example, if you created the persistent disk from a snapshot that was later deleted and recreated under the same name, the source snapshot ID would identify the exact version of the snapshot that was used.
        # @!attribute [rw] source_storage_object
        #   @return [::String]
        #     The full Google Cloud Storage URI where the disk image is stored. This file must be a gzip-compressed tarball whose name ends in .tar.gz or virtual machine disk whose name ends in vmdk. Valid URIs may start with gs:// or https://storage.googleapis.com/. This flag is not optimized for creating multiple disks from a source storage object. To create many disks from a source storage object, use gcloud compute images import instead.
        # @!attribute [rw] status
        #   @return [::String]
        #     [Output Only] The status of disk creation. - CREATING: Disk is provisioning. - RESTORING: Source data is being copied into the disk. - FAILED: Disk creation failed. - READY: Disk is ready for use. - DELETING: Disk is deleting.
        #     Check the Status enum for the list of possible values.
        # @!attribute [rw] type
        #   @return [::String]
        #     URL of the disk type resource describing which disk type to use to create the disk. Provide this when creating the disk. For example: projects/project /zones/zone/diskTypes/pd-ssd . See Persistent disk types.
        # @!attribute [rw] users
        #   @return [::Array<::String>]
        #     [Output Only] Links to the users of the disk (attached instances) in form: projects/project/zones/zone/instances/instance
        # @!attribute [rw] zone
        #   @return [::String]
        #     [Output Only] URL of the zone where the disk resides. You must specify this field as part of the HTTP request URL. It is not settable as a field in the request body.
        class Disk
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::DiskAsyncReplicationList]
          class AsyncSecondaryDisksEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class LabelsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # The architecture of the disk. Valid values are ARM64 or X86_64.
          module Architecture
            # A value indicating that the enum field is not set.
            UNDEFINED_ARCHITECTURE = 0

            # Default value indicating Architecture is not set.
            ARCHITECTURE_UNSPECIFIED = 394_750_507

            # Machines with architecture ARM64
            ARM64 = 62_547_450

            # Machines with architecture X86_64
            X86_64 = 425_300_551
          end

          # [Output Only] The status of disk creation. - CREATING: Disk is provisioning. - RESTORING: Source data is being copied into the disk. - FAILED: Disk creation failed. - READY: Disk is ready for use. - DELETING: Disk is deleting.
          module Status
            # A value indicating that the enum field is not set.
            UNDEFINED_STATUS = 0

            # Disk is provisioning
            CREATING = 455_564_985

            # Disk is deleting.
            DELETING = 528_602_024

            # Disk creation failed.
            FAILED = 455_706_685

            # Disk is ready for use.
            READY = 77_848_963

            # Source data is being copied into the disk.
            RESTORING = 404_263_851
          end
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::DisksScopedList}]
        #     A list of DisksScopedList resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#diskAggregatedList for aggregated lists of persistent disks.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class DiskAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::DisksScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # @!attribute [rw] consistency_group_policy
        #   @return [::String]
        #     [Output Only] URL of the DiskConsistencyGroupPolicy if replication was started on the disk as a member of a group.
        # @!attribute [rw] consistency_group_policy_id
        #   @return [::String]
        #     [Output Only] ID of the DiskConsistencyGroupPolicy if replication was started on the disk as a member of a group.
        # @!attribute [rw] disk
        #   @return [::String]
        #     The other disk asynchronously replicated to or from the current disk. You can provide this as a partial or full URL to the resource. For example, the following are valid values: - https://www.googleapis.com/compute/v1/projects/project/zones/zone /disks/disk - projects/project/zones/zone/disks/disk - zones/zone/disks/disk
        # @!attribute [rw] disk_id
        #   @return [::String]
        #     [Output Only] The unique ID of the other disk asynchronously replicated to or from the current disk. This value identifies the exact disk that was used to create this replication. For example, if you started replicating the persistent disk from a disk that was later deleted and recreated under the same name, the disk ID would identify the exact version of the disk that was used.
        class DiskAsyncReplication
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] async_replication_disk
        #   @return [::Google::Cloud::Compute::V1::DiskAsyncReplication]
        class DiskAsyncReplicationList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A specification of the desired way to instantiate a disk in the instance template when its created from a source instance.
        # @!attribute [rw] auto_delete
        #   @return [::Boolean]
        #     Specifies whether the disk will be auto-deleted when the instance is deleted (but not when the disk is detached from the instance).
        # @!attribute [rw] custom_image
        #   @return [::String]
        #     The custom source image to be used to restore this disk when instantiating this instance template.
        # @!attribute [rw] device_name
        #   @return [::String]
        #     Specifies the device name of the disk to which the configurations apply to.
        # @!attribute [rw] instantiate_from
        #   @return [::String]
        #     Specifies whether to include the disk and what image to use. Possible values are: - source-image: to use the same image that was used to create the source instance's corresponding disk. Applicable to the boot disk and additional read-write disks. - source-image-family: to use the same image family that was used to create the source instance's corresponding disk. Applicable to the boot disk and additional read-write disks. - custom-image: to use a user-provided image url for disk creation. Applicable to the boot disk and additional read-write disks. - attach-read-only: to attach a read-only disk. Applicable to read-only disks. - do-not-include: to exclude a disk from the template. Applicable to additional read-write disks, local SSDs, and read-only disks.
        #     Check the InstantiateFrom enum for the list of possible values.
        class DiskInstantiationConfig
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Specifies whether to include the disk and what image to use. Possible values are: - source-image: to use the same image that was used to create the source instance's corresponding disk. Applicable to the boot disk and additional read-write disks. - source-image-family: to use the same image family that was used to create the source instance's corresponding disk. Applicable to the boot disk and additional read-write disks. - custom-image: to use a user-provided image url for disk creation. Applicable to the boot disk and additional read-write disks. - attach-read-only: to attach a read-only disk. Applicable to read-only disks. - do-not-include: to exclude a disk from the template. Applicable to additional read-write disks, local SSDs, and read-only disks.
          module InstantiateFrom
            # A value indicating that the enum field is not set.
            UNDEFINED_INSTANTIATE_FROM = 0

            # Attach the existing disk in read-only mode. The request will fail if the disk was attached in read-write mode on the source instance. Applicable to: read-only disks.
            ATTACH_READ_ONLY = 513_775_419

            # Create a blank disk. The disk will be created unformatted. Applicable to: additional read-write disks, local SSDs.
            BLANK = 63_281_460

            # Use the custom image specified in the custom_image field. Applicable to: boot disk, additional read-write disks.
            CUSTOM_IMAGE = 196_311_789

            # Use the default instantiation option for the corresponding type of disk. For boot disk and any other R/W disks, new custom images will be created from each disk. For read-only disks, they will be attached in read-only mode. Local SSD disks will be created as blank volumes.
            DEFAULT = 115_302_945

            # Do not include the disk in the instance template. Applicable to: additional read-write disks, local SSDs, read-only disks.
            DO_NOT_INCLUDE = 104_218_952

            # Use the same source image used for creation of the source instance's corresponding disk. The request will fail if the source VM's disk was created from a snapshot. Applicable to: boot disk, additional read-write disks.
            SOURCE_IMAGE = 62_631_959

            # Use the same source image family used for creation of the source instance's corresponding disk. The request will fail if the source image of the source disk does not belong to any image family. Applicable to: boot disk, additional read-write disks.
            SOURCE_IMAGE_FAMILY = 76_850_316
          end
        end

        # A list of Disk resources.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::Disk>]
        #     A list of Disk resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#diskList for lists of disks.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class DiskList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] destination_zone
        #   @return [::String]
        #     The URL of the destination zone to move the disk. This can be a full or partial URL. For example, the following are all valid URLs to a zone: - https://www.googleapis.com/compute/v1/projects/project/zones/zone - projects/project/zones/zone - zones/zone
        # @!attribute [rw] target_disk
        #   @return [::String]
        #     The URL of the target disk to move. This can be a full or partial URL. For example, the following are all valid URLs to a disk: - https://www.googleapis.com/compute/v1/projects/project/zones/zone /disks/disk - projects/project/zones/zone/disks/disk - zones/zone/disks/disk
        class DiskMoveRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Additional disk params.
        # @!attribute [rw] resource_manager_tags
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     Resource manager tags to be bound to the disk. Tag keys and values have the same definition as resource manager tags. Keys must be in the format `tagKeys/{tag_key_id}`, and values are in the format `tagValues/456`. The field is ignored (both PUT & PATCH) when empty.
        class DiskParams
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class ResourceManagerTagsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # @!attribute [rw] async_primary_disk
        #   @return [::Google::Cloud::Compute::V1::DiskResourceStatusAsyncReplicationStatus]
        # @!attribute [rw] async_secondary_disks
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::DiskResourceStatusAsyncReplicationStatus}]
        #     Key: disk, value: AsyncReplicationStatus message
        class DiskResourceStatus
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::DiskResourceStatusAsyncReplicationStatus]
          class AsyncSecondaryDisksEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # @!attribute [rw] state
        #   @return [::String]
        #     Check the State enum for the list of possible values.
        class DiskResourceStatusAsyncReplicationStatus
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods


          module State
            # A value indicating that the enum field is not set.
            UNDEFINED_STATE = 0

            # Replication is active.
            ACTIVE = 314_733_318

            # Secondary disk is created and is waiting for replication to start.
            CREATED = 135_924_424

            # Replication is starting.
            STARTING = 488_820_800

            STATE_UNSPECIFIED = 470_755_401

            # Replication is stopped.
            STOPPED = 444_276_141

            # Replication is stopping.
            STOPPING = 350_791_796
          end
        end

        # Represents a Disk Type resource. Google Compute Engine has two Disk Type resources: * [Regional](/compute/docs/reference/rest/v1/regionDiskTypes) * [Zonal](/compute/docs/reference/rest/v1/diskTypes) You can choose from a variety of disk types based on your needs. For more information, read Storage options. The diskTypes resource represents disk types for a zonal persistent disk. For more information, read Zonal persistent disks. The regionDiskTypes resource represents disk types for a regional persistent disk. For more information, read Regional persistent disks.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] default_disk_size_gb
        #   @return [::Integer]
        #     [Output Only] Server-defined default disk size in GB.
        # @!attribute [rw] deprecated
        #   @return [::Google::Cloud::Compute::V1::DeprecationStatus]
        #     [Output Only] The deprecation status associated with this disk type.
        # @!attribute [rw] description
        #   @return [::String]
        #     [Output Only] An optional description of this resource.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#diskType for disk types.
        # @!attribute [rw] name
        #   @return [::String]
        #     [Output Only] Name of the resource.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] URL of the region where the disk type resides. Only applicable for regional resources. You must specify this field as part of the HTTP request URL. It is not settable as a field in the request body.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] valid_disk_size
        #   @return [::String]
        #     [Output Only] An optional textual description of the valid disk size, such as "10GB-10TB".
        # @!attribute [rw] zone
        #   @return [::String]
        #     [Output Only] URL of the zone where the disk type resides. You must specify this field as part of the HTTP request URL. It is not settable as a field in the request body.
        class DiskType
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::DiskTypesScopedList}]
        #     A list of DiskTypesScopedList resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#diskTypeAggregatedList.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class DiskTypeAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::DiskTypesScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # Contains a list of disk types.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::DiskType>]
        #     A list of DiskType resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#diskTypeList for disk types.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class DiskTypeList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] disk_types
        #   @return [::Array<::Google::Cloud::Compute::V1::DiskType>]
        #     [Output Only] A list of disk types contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning which replaces the list of disk types when the list is empty.
        class DiskTypesScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] resource_policies
        #   @return [::Array<::String>]
        #     Full or relative path to the resource policy to be added to this disk. You can only specify one resource policy.
        class DisksAddResourcePoliciesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] resource_policies
        #   @return [::Array<::String>]
        #     Resource policies to be removed from this disk.
        class DisksRemoveResourcePoliciesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] size_gb
        #   @return [::Integer]
        #     The new size of the persistent disk, which is specified in GB.
        class DisksResizeRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] disks
        #   @return [::Array<::Google::Cloud::Compute::V1::Disk>]
        #     [Output Only] A list of disks contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning which replaces the list of disks when the list is empty.
        class DisksScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] async_secondary_disk
        #   @return [::String]
        #     The secondary disk to start asynchronous replication to. You can provide this as a partial or full URL to the resource. For example, the following are valid values: - https://www.googleapis.com/compute/v1/projects/project/zones/zone /disks/disk - https://www.googleapis.com/compute/v1/projects/project/regions/region /disks/disk - projects/project/zones/zone/disks/disk - projects/project/regions/region/disks/disk - zones/zone/disks/disk - regions/region/disks/disk
        class DisksStartAsyncReplicationRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A transient resource used in compute.disks.stopGroupAsyncReplication and compute.regionDisks.stopGroupAsyncReplication. It is only used to process requests and is not persisted.
        # @!attribute [rw] resource_policy
        #   @return [::String]
        #     The URL of the DiskConsistencyGroupPolicy for the group of disks to stop. This may be a full or partial URL, such as: - https://www.googleapis.com/compute/v1/projects/project/regions/region /resourcePolicies/resourcePolicy - projects/project/regions/region/resourcePolicies/resourcePolicy - regions/region/resourcePolicies/resourcePolicy
        class DisksStopGroupAsyncReplicationResource
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A set of Display Device options
        # @!attribute [rw] enable_display
        #   @return [::Boolean]
        #     Defines whether the instance has Display enabled.
        class DisplayDevice
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] target_shape
        #   @return [::String]
        #     The distribution shape to which the group converges either proactively or on resize events (depending on the value set in updatePolicy.instanceRedistributionType).
        #     Check the TargetShape enum for the list of possible values.
        # @!attribute [rw] zones
        #   @return [::Array<::Google::Cloud::Compute::V1::DistributionPolicyZoneConfiguration>]
        #     Zones where the regional managed instance group will create and manage its instances.
        class DistributionPolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The distribution shape to which the group converges either proactively or on resize events (depending on the value set in updatePolicy.instanceRedistributionType).
          module TargetShape
            # A value indicating that the enum field is not set.
            UNDEFINED_TARGET_SHAPE = 0

            # The group picks zones for creating VM instances to fulfill the requested number of VMs within present resource constraints and to maximize utilization of unused zonal reservations. Recommended for batch workloads that do not require high availability.
            ANY = 64_972

            # The group creates all VM instances within a single zone. The zone is selected based on the present resource constraints and to maximize utilization of unused zonal reservations. Recommended for batch workloads with heavy interprocess communication.
            ANY_SINGLE_ZONE = 61_100_880

            # The group prioritizes acquisition of resources, scheduling VMs in zones where resources are available while distributing VMs as evenly as possible across selected zones to minimize the impact of zonal failure. Recommended for highly available serving workloads.
            BALANCED = 468_409_608

            # The group schedules VM instance creation and deletion to achieve and maintain an even number of managed instances across the selected zones. The distribution is even when the number of managed instances does not differ by more than 1 between any two zones. Recommended for highly available serving workloads.
            EVEN = 2_140_442
          end
        end

        # @!attribute [rw] zone
        #   @return [::String]
        #     The URL of the zone. The zone must exist in the region where the managed instance group is located.
        class DistributionPolicyZoneConfiguration
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A Duration represents a fixed-length span of time represented as a count of seconds and fractions of seconds at nanosecond resolution. It is independent of any calendar and concepts like "day" or "month". Range is approximately 10,000 years.
        # @!attribute [rw] nanos
        #   @return [::Integer]
        #     Span of time that's a fraction of a second at nanosecond resolution. Durations less than one second are represented with a 0 `seconds` field and a positive `nanos` field. Must be from 0 to 999,999,999 inclusive.
        # @!attribute [rw] seconds
        #   @return [::Integer]
        #     Span of time at a resolution of a second. Must be from 0 to 315,576,000,000 inclusive. Note: these bounds are computed from: 60 sec/min * 60 min/hr * 24 hr/day * 365.25 days/year * 10000 years
        class Duration
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Projects.EnableXpnHost. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class EnableXpnHostProjectRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Projects.EnableXpnResource. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] projects_enable_xpn_resource_request_resource
        #   @return [::Google::Cloud::Compute::V1::ProjectsEnableXpnResourceRequest]
        #     The body resource for this request
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class EnableXpnResourceProjectRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # [Output Only] If errors are generated during processing of the operation, this field will be populated.
        # @!attribute [rw] errors
        #   @return [::Array<::Google::Cloud::Compute::V1::Errors>]
        #     [Output Only] The array of errors encountered while processing this operation.
        class Error
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] error_info
        #   @return [::Google::Cloud::Compute::V1::ErrorInfo]
        # @!attribute [rw] help
        #   @return [::Google::Cloud::Compute::V1::Help]
        # @!attribute [rw] localized_message
        #   @return [::Google::Cloud::Compute::V1::LocalizedMessage]
        # @!attribute [rw] quota_info
        #   @return [::Google::Cloud::Compute::V1::QuotaExceededInfo]
        class ErrorDetails
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Describes the cause of the error with structured details. Example of an error when contacting the "pubsub.googleapis.com" API when it is not enabled: { "reason": "API_DISABLED" "domain": "googleapis.com" "metadata": { "resource": "projects/123", "service": "pubsub.googleapis.com" } } This response indicates that the pubsub.googleapis.com API is not enabled. Example of an error that is returned when attempting to create a Spanner instance in a region that is out of stock: { "reason": "STOCKOUT" "domain": "spanner.googleapis.com", "metadata": { "availableRegions": "us-central1,us-east2" } }
        # @!attribute [rw] domain
        #   @return [::String]
        #     The logical grouping to which the "reason" belongs. The error domain is typically the registered service name of the tool or product that generates the error. Example: "pubsub.googleapis.com". If the error is generated by some common infrastructure, the error domain must be a globally unique value that identifies the infrastructure. For Google API infrastructure, the error domain is "googleapis.com".
        # @!attribute [rw] metadatas
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     Additional structured details about this error. Keys should match /[a-zA-Z0-9-_]/ and be limited to 64 characters in length. When identifying the current value of an exceeded limit, the units should be contained in the key, not the value. For example, rather than \\{"instanceLimit": "100/request"}, should be returned as, \\{"instanceLimitPerRequest": "100"}, if the client exceeds the number of instances that can be created in a single (batch) request.
        # @!attribute [rw] reason
        #   @return [::String]
        #     The reason of the error. This is a constant value that identifies the proximate cause of the error. Error reasons are unique within a particular domain of errors. This should be at most 63 characters and match a regular expression of `A-Z+[A-Z0-9]`, which represents UPPER_SNAKE_CASE.
        class ErrorInfo
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class MetadatasEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # @!attribute [rw] code
        #   @return [::String]
        #     [Output Only] The error type identifier for this error.
        # @!attribute [rw] error_details
        #   @return [::Array<::Google::Cloud::Compute::V1::ErrorDetails>]
        #     [Output Only] An optional list of messages that contain the error details. There is a set of defined message types to use for providing details.The syntax depends on the error code. For example, QuotaExceededInfo will have details when the error code is QUOTA_EXCEEDED.
        # @!attribute [rw] location
        #   @return [::String]
        #     [Output Only] Indicates the field in the request that caused the error. This property is optional.
        # @!attribute [rw] message
        #   @return [::String]
        #     [Output Only] An optional, human-readable error message.
        class Errors
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] dest_range
        #   @return [::String]
        #     The destination range of the route.
        # @!attribute [rw] imported
        #   @return [::Boolean]
        #     True if the peering route has been imported from a peer. The actual import happens if the field networkPeering.importCustomRoutes is true for this network, and networkPeering.exportCustomRoutes is true for the peer network, and the import does not result in a route conflict.
        # @!attribute [rw] next_hop_region
        #   @return [::String]
        #     The region of peering route next hop, only applies to dynamic routes.
        # @!attribute [rw] priority
        #   @return [::Integer]
        #     The priority of the peering route.
        # @!attribute [rw] type
        #   @return [::String]
        #     The type of the peering route.
        #     Check the Type enum for the list of possible values.
        class ExchangedPeeringRoute
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The type of the peering route.
          module Type
            # A value indicating that the enum field is not set.
            UNDEFINED_TYPE = 0

            # For routes exported from local network.
            DYNAMIC_PEERING_ROUTE = 469_794_858

            # The peering route.
            STATIC_PEERING_ROUTE = 473_407_545

            # The peering route corresponding to subnetwork range.
            SUBNET_PEERING_ROUTE = 465_782_504
          end
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::ExchangedPeeringRoute>]
        #     A list of ExchangedPeeringRoute resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#exchangedPeeringRoutesList for exchanged peering routes lists.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class ExchangedPeeringRoutesList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Subnetworks.ExpandIpCidrRange. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] subnetwork
        #   @return [::String]
        #     Name of the Subnetwork resource to update.
        # @!attribute [rw] subnetworks_expand_ip_cidr_range_request_resource
        #   @return [::Google::Cloud::Compute::V1::SubnetworksExpandIpCidrRangeRequest]
        #     The body resource for this request
        class ExpandIpCidrRangeSubnetworkRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a textual expression in the Common Expression Language (CEL) syntax. CEL is a C-like expression language. The syntax and semantics of CEL are documented at https://github.com/google/cel-spec. Example (Comparison): title: "Summary size limit" description: "Determines if a summary is less than 100 chars" expression: "document.summary.size() < 100" Example (Equality): title: "Requestor is owner" description: "Determines if requestor is the document owner" expression: "document.owner == request.auth.claims.email" Example (Logic): title: "Public documents" description: "Determine whether the document should be publicly visible" expression: "document.type != 'private' && document.type != 'internal'" Example (Data Manipulation): title: "Notification string" description: "Create a notification string with a timestamp." expression: "'New message received at ' + string(document.create_time)" The exact variables and functions that may be referenced within an expression are determined by the service that evaluates it. See the service documentation for additional information.
        # @!attribute [rw] description
        #   @return [::String]
        #     Optional. Description of the expression. This is a longer text which describes the expression, e.g. when hovered over it in a UI.
        # @!attribute [rw] expression
        #   @return [::String]
        #     Textual representation of an expression in Common Expression Language syntax.
        # @!attribute [rw] location
        #   @return [::String]
        #     Optional. String indicating the location of the expression for error reporting, e.g. a file name and a position in the file.
        # @!attribute [rw] title
        #   @return [::String]
        #     Optional. Title for the expression, i.e. a short string describing its purpose. This can be used e.g. in UIs which allow to enter the expression.
        class Expr
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents an external VPN gateway. External VPN gateway is the on-premises VPN gateway(s) or another cloud provider's VPN gateway that connects to your Google Cloud VPN gateway. To create a highly available VPN from Google Cloud Platform to your VPN gateway or another cloud provider's VPN gateway, you must create a external VPN gateway resource with information about the other gateway. For more information about using external VPN gateways, see Creating an HA VPN gateway and tunnel pair to a peer VPN.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] interfaces
        #   @return [::Array<::Google::Cloud::Compute::V1::ExternalVpnGatewayInterface>]
        #     A list of interfaces for this external VPN gateway. If your peer-side gateway is an on-premises gateway and non-AWS cloud providers' gateway, at most two interfaces can be provided for an external VPN gateway. If your peer side is an AWS virtual private gateway, four interfaces should be provided for an external VPN gateway.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#externalVpnGateway for externalVpnGateways.
        # @!attribute [rw] label_fingerprint
        #   @return [::String]
        #     A fingerprint for the labels being applied to this ExternalVpnGateway, which is essentially a hash of the labels set used for optimistic locking. The fingerprint is initially generated by Compute Engine and changes after every request to modify or update labels. You must always provide an up-to-date fingerprint hash in order to update or change labels, otherwise the request will fail with error 412 conditionNotMet. To see the latest fingerprint, make a get() request to retrieve an ExternalVpnGateway.
        # @!attribute [rw] labels
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     Labels for this resource. These can only be added or modified by the setLabels method. Each label key/value pair must comply with RFC1035. Label values may be empty.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] redundancy_type
        #   @return [::String]
        #     Indicates the user-supplied redundancy type of this external VPN gateway.
        #     Check the RedundancyType enum for the list of possible values.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        class ExternalVpnGateway
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class LabelsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # Indicates the user-supplied redundancy type of this external VPN gateway.
          module RedundancyType
            # A value indicating that the enum field is not set.
            UNDEFINED_REDUNDANCY_TYPE = 0

            # The external VPN gateway has four public IP addresses; at the time of writing this API, the AWS virtual private gateway is an example which has four public IP addresses for high availability connections; there should be two VPN connections in the AWS virtual private gateway , each AWS VPN connection has two public IP addresses; please make sure to put two public IP addresses from one AWS VPN connection into interfaces 0 and 1 of this external VPN gateway, and put the other two public IP addresses from another AWS VPN connection into interfaces 2 and 3 of this external VPN gateway. When displaying highly available configuration status for the VPN tunnels connected to FOUR_IPS_REDUNDANCY external VPN gateway, Google will always detect whether interfaces 0 and 1 are connected on one interface of HA Cloud VPN gateway, and detect whether interfaces 2 and 3 are connected to another interface of the HA Cloud VPN gateway.
            FOUR_IPS_REDUNDANCY = 520_087_913

            # The external VPN gateway has only one public IP address which internally provide redundancy or failover.
            SINGLE_IP_INTERNALLY_REDUNDANT = 133_914_873

            # The external VPN gateway has two public IP addresses which are redundant with each other, the following two types of setup on your on-premises side would have this type of redundancy: (1) Two separate on-premises gateways, each with one public IP address, the two on-premises gateways are redundant with each other. (2) A single on-premise gateway with two public IP addresses that are redundant with eatch other.
            TWO_IPS_REDUNDANCY = 367_049_635
          end
        end

        # The interface for the external VPN gateway.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     The numeric ID of this interface. The allowed input values for this id for different redundancy types of external VPN gateway: - SINGLE_IP_INTERNALLY_REDUNDANT - 0 - TWO_IPS_REDUNDANCY - 0, 1 - FOUR_IPS_REDUNDANCY - 0, 1, 2, 3
        # @!attribute [rw] ip_address
        #   @return [::String]
        #     IP address of the interface in the external VPN gateway. Only IPv4 is supported. This IP address can be either from your on-premise gateway or another Cloud provider's VPN gateway, it cannot be an IP address from Google Compute Engine.
        class ExternalVpnGatewayInterface
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Response to the list request, and contains a list of externalVpnGateways.
        # @!attribute [rw] etag
        #   @return [::String]
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::ExternalVpnGateway>]
        #     A list of ExternalVpnGateway resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#externalVpnGatewayList for lists of externalVpnGateways.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class ExternalVpnGatewayList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] content
        #   @return [::String]
        #     The raw content in the secure keys file.
        # @!attribute [rw] file_type
        #   @return [::String]
        #     The file type of source file.
        #     Check the FileType enum for the list of possible values.
        class FileContentBuffer
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The file type of source file.
          module FileType
            # A value indicating that the enum field is not set.
            UNDEFINED_FILE_TYPE = 0

            BIN = 65_767

            UNDEFINED = 137_851_184

            X509 = 2_674_086
          end
        end

        # Represents a Firewall Rule resource. Firewall rules allow or deny ingress traffic to, and egress traffic from your instances. For more information, read Firewall rules.
        # @!attribute [rw] allowed
        #   @return [::Array<::Google::Cloud::Compute::V1::Allowed>]
        #     The list of ALLOW rules specified by this firewall. Each rule specifies a protocol and port-range tuple that describes a permitted connection.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] denied
        #   @return [::Array<::Google::Cloud::Compute::V1::Denied>]
        #     The list of DENY rules specified by this firewall. Each rule specifies a protocol and port-range tuple that describes a denied connection.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this field when you create the resource.
        # @!attribute [rw] destination_ranges
        #   @return [::Array<::String>]
        #     If destination ranges are specified, the firewall rule applies only to traffic that has destination IP address in these ranges. These ranges must be expressed in CIDR format. Both IPv4 and IPv6 are supported.
        # @!attribute [rw] direction
        #   @return [::String]
        #     Direction of traffic to which this firewall applies, either `INGRESS` or `EGRESS`. The default is `INGRESS`. For `EGRESS` traffic, you cannot specify the sourceTags fields.
        #     Check the Direction enum for the list of possible values.
        # @!attribute [rw] disabled
        #   @return [::Boolean]
        #     Denotes whether the firewall rule is disabled. When set to true, the firewall rule is not enforced and the network behaves as if it did not exist. If this is unspecified, the firewall rule will be enabled.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#firewall for firewall rules.
        # @!attribute [rw] log_config
        #   @return [::Google::Cloud::Compute::V1::FirewallLogConfig]
        #     This field denotes the logging options for a particular firewall rule. If logging is enabled, logs will be exported to Cloud Logging.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource; provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?`. The first character must be a lowercase letter, and all following characters (except for the last character) must be a dash, lowercase letter, or digit. The last character must be a lowercase letter or digit.
        # @!attribute [rw] network
        #   @return [::String]
        #     URL of the network resource for this firewall rule. If not specified when creating a firewall rule, the default network is used: global/networks/default If you choose to specify this field, you can specify the network as a full or partial URL. For example, the following are all valid URLs: - https://www.googleapis.com/compute/v1/projects/myproject/global/networks/my-network - projects/myproject/global/networks/my-network - global/networks/default
        # @!attribute [rw] priority
        #   @return [::Integer]
        #     Priority for this rule. This is an integer between `0` and `65535`, both inclusive. The default value is `1000`. Relative priorities determine which rule takes effect if multiple rules apply. Lower values indicate higher priority. For example, a rule with priority `0` has higher precedence than a rule with priority `1`. DENY rules take precedence over ALLOW rules if they have equal priority. Note that VPC networks have implied rules with a priority of `65535`. To avoid conflicts with the implied rules, use a priority number less than `65535`.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] source_ranges
        #   @return [::Array<::String>]
        #     If source ranges are specified, the firewall rule applies only to traffic that has a source IP address in these ranges. These ranges must be expressed in CIDR format. One or both of sourceRanges and sourceTags may be set. If both fields are set, the rule applies to traffic that has a source IP address within sourceRanges OR a source IP from a resource with a matching tag listed in the sourceTags field. The connection does not need to match both fields for the rule to apply. Both IPv4 and IPv6 are supported.
        # @!attribute [rw] source_service_accounts
        #   @return [::Array<::String>]
        #     If source service accounts are specified, the firewall rules apply only to traffic originating from an instance with a service account in this list. Source service accounts cannot be used to control traffic to an instance's external IP address because service accounts are associated with an instance, not an IP address. sourceRanges can be set at the same time as sourceServiceAccounts. If both are set, the firewall applies to traffic that has a source IP address within the sourceRanges OR a source IP that belongs to an instance with service account listed in sourceServiceAccount. The connection does not need to match both fields for the firewall to apply. sourceServiceAccounts cannot be used at the same time as sourceTags or targetTags.
        # @!attribute [rw] source_tags
        #   @return [::Array<::String>]
        #     If source tags are specified, the firewall rule applies only to traffic with source IPs that match the primary network interfaces of VM instances that have the tag and are in the same VPC network. Source tags cannot be used to control traffic to an instance's external IP address, it only applies to traffic between instances in the same virtual network. Because tags are associated with instances, not IP addresses. One or both of sourceRanges and sourceTags may be set. If both fields are set, the firewall applies to traffic that has a source IP address within sourceRanges OR a source IP from a resource with a matching tag listed in the sourceTags field. The connection does not need to match both fields for the firewall to apply.
        # @!attribute [rw] target_service_accounts
        #   @return [::Array<::String>]
        #     A list of service accounts indicating sets of instances located in the network that may make network connections as specified in allowed[]. targetServiceAccounts cannot be used at the same time as targetTags or sourceTags. If neither targetServiceAccounts nor targetTags are specified, the firewall rule applies to all instances on the specified network.
        # @!attribute [rw] target_tags
        #   @return [::Array<::String>]
        #     A list of tags that controls which instances the firewall rule applies to. If targetTags are specified, then the firewall rule applies only to instances in the VPC network that have one of those tags. If no targetTags are specified, the firewall rule applies to all instances on the specified network.
        class Firewall
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Direction of traffic to which this firewall applies, either `INGRESS` or `EGRESS`. The default is `INGRESS`. For `EGRESS` traffic, you cannot specify the sourceTags fields.
          module Direction
            # A value indicating that the enum field is not set.
            UNDEFINED_DIRECTION = 0

            # Indicates that firewall should apply to outgoing traffic.
            EGRESS = 432_880_501

            # Indicates that firewall should apply to incoming traffic.
            INGRESS = 516_931_221
          end
        end

        # Contains a list of firewalls.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::Firewall>]
        #     A list of Firewall resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#firewallList for lists of firewalls.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class FirewallList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # The available logging options for a firewall rule.
        # @!attribute [rw] enable
        #   @return [::Boolean]
        #     This field denotes whether to enable logging for a particular firewall rule.
        # @!attribute [rw] metadata
        #   @return [::String]
        #     This field can only be specified for a particular firewall rule if logging is enabled for that rule. This field denotes whether to include or exclude metadata for firewall logs.
        #     Check the Metadata enum for the list of possible values.
        class FirewallLogConfig
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # This field can only be specified for a particular firewall rule if logging is enabled for that rule. This field denotes whether to include or exclude metadata for firewall logs.
          module Metadata
            # A value indicating that the enum field is not set.
            UNDEFINED_METADATA = 0

            EXCLUDE_ALL_METADATA = 334_519_954

            INCLUDE_ALL_METADATA = 164_619_908
          end
        end

        # @!attribute [rw] associations
        #   @return [::Array<::Google::Cloud::Compute::V1::FirewallPolicyAssociation>]
        #     A list of associations.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of firewallPolicy associations. Always compute#FirewallPoliciesListAssociations for lists of firewallPolicy associations.
        class FirewallPoliciesListAssociationsResponse
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a Firewall Policy resource.
        # @!attribute [rw] associations
        #   @return [::Array<::Google::Cloud::Compute::V1::FirewallPolicyAssociation>]
        #     A list of associations that belong to this firewall policy.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] display_name
        #   @return [::String]
        #     Deprecated, please use short name instead. User-provided name of the Organization firewall policy. The name should be unique in the organization in which the firewall policy is created. This field is not applicable to network firewall policies. This name must be set on creation and cannot be changed. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] fingerprint
        #   @return [::String]
        #     Specifies a fingerprint for this resource, which is essentially a hash of the metadata's contents and used for optimistic locking. The fingerprint is initially generated by Compute Engine and changes after every request to modify or update metadata. You must always provide an up-to-date fingerprint hash in order to update or change metadata, otherwise the request will fail with error 412 conditionNotMet. To see the latest fingerprint, make get() request to the firewall policy.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output only] Type of the resource. Always compute#firewallPolicyfor firewall policies
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. For Organization Firewall Policies it's a [Output Only] numeric ID allocated by Google Cloud which uniquely identifies the Organization Firewall Policy.
        # @!attribute [rw] parent
        #   @return [::String]
        #     [Output Only] The parent of the firewall policy. This field is not applicable to network firewall policies.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] URL of the region where the regional firewall policy resides. This field is not applicable to global firewall policies. You must specify this field as part of the HTTP request URL. It is not settable as a field in the request body.
        # @!attribute [rw] rule_tuple_count
        #   @return [::Integer]
        #     [Output Only] Total count of all firewall policy rule tuples. A firewall policy can not exceed a set number of tuples.
        # @!attribute [rw] rules
        #   @return [::Array<::Google::Cloud::Compute::V1::FirewallPolicyRule>]
        #     A list of rules that belong to this policy. There must always be a default rule (rule with priority 2147483647 and match "*"). If no rules are provided when creating a firewall policy, a default rule with action "allow" will be added.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] self_link_with_id
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource with the resource id.
        # @!attribute [rw] short_name
        #   @return [::String]
        #     User-provided name of the Organization firewall policy. The name should be unique in the organization in which the firewall policy is created. This field is not applicable to network firewall policies. This name must be set on creation and cannot be changed. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        class FirewallPolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] attachment_target
        #   @return [::String]
        #     The target that the firewall policy is attached to.
        # @!attribute [rw] display_name
        #   @return [::String]
        #     [Output Only] Deprecated, please use short name instead. The display name of the firewall policy of the association.
        # @!attribute [rw] firewall_policy_id
        #   @return [::String]
        #     [Output Only] The firewall policy ID of the association.
        # @!attribute [rw] name
        #   @return [::String]
        #     The name for an association.
        # @!attribute [rw] short_name
        #   @return [::String]
        #     [Output Only] The short name of the firewall policy of the association.
        class FirewallPolicyAssociation
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::FirewallPolicy>]
        #     A list of FirewallPolicy resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#firewallPolicyList for listsof FirewallPolicies
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class FirewallPolicyList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a rule that describes one or more match conditions along with the action to be taken when traffic matches this condition (allow or deny).
        # @!attribute [rw] action
        #   @return [::String]
        #     The Action to perform when the client connection triggers the rule. Valid actions are "allow", "deny" and "goto_next".
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description for this resource.
        # @!attribute [rw] direction
        #   @return [::String]
        #     The direction in which this rule applies.
        #     Check the Direction enum for the list of possible values.
        # @!attribute [rw] disabled
        #   @return [::Boolean]
        #     Denotes whether the firewall policy rule is disabled. When set to true, the firewall policy rule is not enforced and traffic behaves as if it did not exist. If this is unspecified, the firewall policy rule will be enabled.
        # @!attribute [rw] enable_logging
        #   @return [::Boolean]
        #     Denotes whether to enable logging for a particular rule. If logging is enabled, logs will be exported to the configured export destination in Stackdriver. Logs may be exported to BigQuery or Pub/Sub. Note: you cannot enable logging on "goto_next" rules.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output only] Type of the resource. Always compute#firewallPolicyRule for firewall policy rules
        # @!attribute [rw] match
        #   @return [::Google::Cloud::Compute::V1::FirewallPolicyRuleMatcher]
        #     A match condition that incoming traffic is evaluated against. If it evaluates to true, the corresponding 'action' is enforced.
        # @!attribute [rw] priority
        #   @return [::Integer]
        #     An integer indicating the priority of a rule in the list. The priority must be a positive value between 0 and 2147483647. Rules are evaluated from highest to lowest priority where 0 is the highest priority and 2147483647 is the lowest prority.
        # @!attribute [rw] rule_name
        #   @return [::String]
        #     An optional name for the rule. This field is not a unique identifier and can be updated.
        # @!attribute [rw] rule_tuple_count
        #   @return [::Integer]
        #     [Output Only] Calculation of the complexity of a single firewall policy rule.
        # @!attribute [rw] target_resources
        #   @return [::Array<::String>]
        #     A list of network resource URLs to which this rule applies. This field allows you to control which network's VMs get this rule. If this field is left blank, all VMs within the organization will receive the rule.
        # @!attribute [rw] target_secure_tags
        #   @return [::Array<::Google::Cloud::Compute::V1::FirewallPolicyRuleSecureTag>]
        #     A list of secure tags that controls which instances the firewall rule applies to. If targetSecureTag are specified, then the firewall rule applies only to instances in the VPC network that have one of those EFFECTIVE secure tags, if all the target_secure_tag are in INEFFECTIVE state, then this rule will be ignored. targetSecureTag may not be set at the same time as targetServiceAccounts. If neither targetServiceAccounts nor targetSecureTag are specified, the firewall rule applies to all instances on the specified network. Maximum number of target label tags allowed is 256.
        # @!attribute [rw] target_service_accounts
        #   @return [::Array<::String>]
        #     A list of service accounts indicating the sets of instances that are applied with this rule.
        class FirewallPolicyRule
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The direction in which this rule applies.
          module Direction
            # A value indicating that the enum field is not set.
            UNDEFINED_DIRECTION = 0

            EGRESS = 432_880_501

            INGRESS = 516_931_221
          end
        end

        # Represents a match condition that incoming traffic is evaluated against. Exactly one field must be specified.
        # @!attribute [rw] dest_address_groups
        #   @return [::Array<::String>]
        #     Address groups which should be matched against the traffic destination. Maximum number of destination address groups is 10.
        # @!attribute [rw] dest_fqdns
        #   @return [::Array<::String>]
        #     Fully Qualified Domain Name (FQDN) which should be matched against traffic destination. Maximum number of destination fqdn allowed is 100.
        # @!attribute [rw] dest_ip_ranges
        #   @return [::Array<::String>]
        #     CIDR IP address range. Maximum number of destination CIDR IP ranges allowed is 5000.
        # @!attribute [rw] dest_region_codes
        #   @return [::Array<::String>]
        #     Region codes whose IP addresses will be used to match for destination of traffic. Should be specified as 2 letter country code defined as per ISO 3166 alpha-2 country codes. ex."US" Maximum number of dest region codes allowed is 5000.
        # @!attribute [rw] dest_threat_intelligences
        #   @return [::Array<::String>]
        #     Names of Network Threat Intelligence lists. The IPs in these lists will be matched against traffic destination.
        # @!attribute [rw] layer4_configs
        #   @return [::Array<::Google::Cloud::Compute::V1::FirewallPolicyRuleMatcherLayer4Config>]
        #     Pairs of IP protocols and ports that the rule should match.
        # @!attribute [rw] src_address_groups
        #   @return [::Array<::String>]
        #     Address groups which should be matched against the traffic source. Maximum number of source address groups is 10.
        # @!attribute [rw] src_fqdns
        #   @return [::Array<::String>]
        #     Fully Qualified Domain Name (FQDN) which should be matched against traffic source. Maximum number of source fqdn allowed is 100.
        # @!attribute [rw] src_ip_ranges
        #   @return [::Array<::String>]
        #     CIDR IP address range. Maximum number of source CIDR IP ranges allowed is 5000.
        # @!attribute [rw] src_region_codes
        #   @return [::Array<::String>]
        #     Region codes whose IP addresses will be used to match for source of traffic. Should be specified as 2 letter country code defined as per ISO 3166 alpha-2 country codes. ex."US" Maximum number of source region codes allowed is 5000.
        # @!attribute [rw] src_secure_tags
        #   @return [::Array<::Google::Cloud::Compute::V1::FirewallPolicyRuleSecureTag>]
        #     List of secure tag values, which should be matched at the source of the traffic. For INGRESS rule, if all the srcSecureTag are INEFFECTIVE, and there is no srcIpRange, this rule will be ignored. Maximum number of source tag values allowed is 256.
        # @!attribute [rw] src_threat_intelligences
        #   @return [::Array<::String>]
        #     Names of Network Threat Intelligence lists. The IPs in these lists will be matched against traffic source.
        class FirewallPolicyRuleMatcher
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] ip_protocol
        #   @return [::String]
        #     The IP protocol to which this rule applies. The protocol type is required when creating a firewall rule. This value can either be one of the following well known protocol strings (tcp, udp, icmp, esp, ah, ipip, sctp), or the IP protocol number.
        # @!attribute [rw] ports
        #   @return [::Array<::String>]
        #     An optional list of ports to which this rule applies. This field is only applicable for UDP or TCP protocol. Each entry must be either an integer or a range. If not specified, this rule applies to connections through any port. Example inputs include: ["22"], ["80","443"], and ["12345-12349"].
        class FirewallPolicyRuleMatcherLayer4Config
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the secure tag, created with TagManager's TagValue API.
        # @!attribute [rw] state
        #   @return [::String]
        #     [Output Only] State of the secure tag, either `EFFECTIVE` or `INEFFECTIVE`. A secure tag is `INEFFECTIVE` when it is deleted or its network is deleted.
        #     Check the State enum for the list of possible values.
        class FirewallPolicyRuleSecureTag
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # [Output Only] State of the secure tag, either `EFFECTIVE` or `INEFFECTIVE`. A secure tag is `INEFFECTIVE` when it is deleted or its network is deleted.
          module State
            # A value indicating that the enum field is not set.
            UNDEFINED_STATE = 0

            EFFECTIVE = 244_201_863

            INEFFECTIVE = 304_458_242
          end
        end

        # Encapsulates numeric value that can be either absolute or relative.
        # @!attribute [rw] calculated
        #   @return [::Integer]
        #     [Output Only] Absolute value of VM instances calculated based on the specific mode. - If the value is fixed, then the calculated value is equal to the fixed value. - If the value is a percent, then the calculated value is percent/100 * targetSize. For example, the calculated value of a 80% of a managed instance group with 150 instances would be (80/100 * 150) = 120 VM instances. If there is a remainder, the number is rounded.
        # @!attribute [rw] fixed
        #   @return [::Integer]
        #     Specifies a fixed number of VM instances. This must be a positive integer.
        # @!attribute [rw] percent
        #   @return [::Integer]
        #     Specifies a percentage of instances between 0 to 100%, inclusive. For example, specify 80 for 80%.
        class FixedOrPercent
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a Forwarding Rule resource. Forwarding rule resources in Google Cloud can be either regional or global in scope: * [Global](https://cloud.google.com/compute/docs/reference/rest/v1/globalForwardingRules) * [Regional](https://cloud.google.com/compute/docs/reference/rest/v1/forwardingRules) A forwarding rule and its corresponding IP address represent the frontend configuration of a Google Cloud Platform load balancer. Forwarding rules can also reference target instances and Cloud VPN Classic gateways (targetVpnGateway). For more information, read Forwarding rule concepts and Using protocol forwarding.
        # @!attribute [rw] I_p_address
        #   @return [::String]
        #     IP address for which this forwarding rule accepts traffic. When a client sends traffic to this IP address, the forwarding rule directs the traffic to the referenced target or backendService. While creating a forwarding rule, specifying an IPAddress is required under the following circumstances: - When the target is set to targetGrpcProxy and validateForProxyless is set to true, the IPAddress should be set to 0.0.0.0. - When the target is a Private Service Connect Google APIs bundle, you must specify an IPAddress. Otherwise, you can optionally specify an IP address that references an existing static (reserved) IP address resource. When omitted, Google Cloud assigns an ephemeral IP address. Use one of the following formats to specify an IP address while creating a forwarding rule: * IP address number, as in `100.1.2.3` * IPv6 address range, as in `2600:1234::/96` * Full resource URL, as in https://www.googleapis.com/compute/v1/projects/ project_id/regions/region/addresses/address-name * Partial URL or by name, as in: - projects/project_id/regions/region/addresses/address-name - regions/region/addresses/address-name - global/addresses/address-name - address-name The forwarding rule's target or backendService, and in most cases, also the loadBalancingScheme, determine the type of IP address that you can use. For detailed information, see [IP address specifications](https://cloud.google.com/load-balancing/docs/forwarding-rule-concepts#ip_address_specifications). When reading an IPAddress, the API always returns the IP address number.
        # @!attribute [rw] I_p_protocol
        #   @return [::String]
        #     The IP protocol to which this rule applies. For protocol forwarding, valid options are TCP, UDP, ESP, AH, SCTP, ICMP and L3_DEFAULT. The valid IP protocols are different for different load balancing products as described in [Load balancing features](https://cloud.google.com/load-balancing/docs/features#protocols_from_the_load_balancer_to_the_backends).
        #     Check the IPProtocolEnum enum for the list of possible values.
        # @!attribute [rw] all_ports
        #   @return [::Boolean]
        #     The ports, portRange, and allPorts fields are mutually exclusive. Only packets addressed to ports in the specified range will be forwarded to the backends configured with this forwarding rule. The allPorts field has the following limitations: - It requires that the forwarding rule IPProtocol be TCP, UDP, SCTP, or L3_DEFAULT. - It's applicable only to the following products: internal passthrough Network Load Balancers, backend service-based external passthrough Network Load Balancers, and internal and external protocol forwarding. - Set this field to true to allow packets addressed to any port or packets lacking destination port information (for example, UDP fragments after the first fragment) to be forwarded to the backends configured with this forwarding rule. The L3_DEFAULT protocol requires allPorts be set to true.
        # @!attribute [rw] allow_global_access
        #   @return [::Boolean]
        #     This field is used along with the backend_service field for internal load balancing or with the target field for internal TargetInstance. If set to true, clients can access the Internal TCP/UDP Load Balancer, Internal HTTP(S) and TCP Proxy Load Balancer from all regions. If false, only allows access from the local region the load balancer is located at. Note that for INTERNAL_MANAGED forwarding rules, this field cannot be changed after the forwarding rule is created.
        # @!attribute [rw] allow_psc_global_access
        #   @return [::Boolean]
        #     This is used in PSC consumer ForwardingRule to control whether the PSC endpoint can be accessed from another region.
        # @!attribute [rw] backend_service
        #   @return [::String]
        #     Identifies the backend service to which the forwarding rule sends traffic. Required for Internal TCP/UDP Load Balancing and Network Load Balancing; must be omitted for all other load balancer types.
        # @!attribute [rw] base_forwarding_rule
        #   @return [::String]
        #     [Output Only] The URL for the corresponding base Forwarding Rule. By base Forwarding Rule, we mean the Forwarding Rule that has the same IP address, protocol, and port settings with the current Forwarding Rule, but without sourceIPRanges specified. Always empty if the current Forwarding Rule does not have sourceIPRanges specified.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] fingerprint
        #   @return [::String]
        #     Fingerprint of this resource. A hash of the contents stored in this object. This field is used in optimistic locking. This field will be ignored when inserting a ForwardingRule. Include the fingerprint in patch request to ensure that you do not overwrite changes that were applied from another concurrent request. To see the latest fingerprint, make a get() request to retrieve a ForwardingRule.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] ip_version
        #   @return [::String]
        #     The IP Version that will be used by this forwarding rule. Valid options are IPV4 or IPV6.
        #     Check the IpVersion enum for the list of possible values.
        # @!attribute [rw] is_mirroring_collector
        #   @return [::Boolean]
        #     Indicates whether or not this load balancer can be used as a collector for packet mirroring. To prevent mirroring loops, instances behind this load balancer will not have their traffic mirrored even if a PacketMirroring rule applies to them. This can only be set to true for load balancers that have their loadBalancingScheme set to INTERNAL.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#forwardingRule for Forwarding Rule resources.
        # @!attribute [rw] label_fingerprint
        #   @return [::String]
        #     A fingerprint for the labels being applied to this resource, which is essentially a hash of the labels set used for optimistic locking. The fingerprint is initially generated by Compute Engine and changes after every request to modify or update labels. You must always provide an up-to-date fingerprint hash in order to update or change labels, otherwise the request will fail with error 412 conditionNotMet. To see the latest fingerprint, make a get() request to retrieve a ForwardingRule.
        # @!attribute [rw] labels
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     Labels for this resource. These can only be added or modified by the setLabels method. Each label key/value pair must comply with RFC1035. Label values may be empty.
        # @!attribute [rw] load_balancing_scheme
        #   @return [::String]
        #     Specifies the forwarding rule type. For more information about forwarding rules, refer to Forwarding rule concepts.
        #     Check the LoadBalancingScheme enum for the list of possible values.
        # @!attribute [rw] metadata_filters
        #   @return [::Array<::Google::Cloud::Compute::V1::MetadataFilter>]
        #     Opaque filter criteria used by load balancer to restrict routing configuration to a limited set of xDS compliant clients. In their xDS requests to load balancer, xDS clients present node metadata. When there is a match, the relevant configuration is made available to those proxies. Otherwise, all the resources (e.g. TargetHttpProxy, UrlMap) referenced by the ForwardingRule are not visible to those proxies. For each metadataFilter in this list, if its filterMatchCriteria is set to MATCH_ANY, at least one of the filterLabels must match the corresponding label provided in the metadata. If its filterMatchCriteria is set to MATCH_ALL, then all of its filterLabels must match with corresponding labels provided in the metadata. If multiple metadataFilters are specified, all of them need to be satisfied in order to be considered a match. metadataFilters specified here will be applifed before those specified in the UrlMap that this ForwardingRule references. metadataFilters only applies to Loadbalancers that have their loadBalancingScheme set to INTERNAL_SELF_MANAGED.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource; provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash. For Private Service Connect forwarding rules that forward traffic to Google APIs, the forwarding rule name must be a 1-20 characters string with lowercase letters and numbers and must start with a letter.
        # @!attribute [rw] network
        #   @return [::String]
        #     This field is not used for global external load balancing. For Internal TCP/UDP Load Balancing, this field identifies the network that the load balanced IP should belong to for this Forwarding Rule. If the subnetwork is specified, the network of the subnetwork will be used. If neither subnetwork nor this field is specified, the default network will be used. For Private Service Connect forwarding rules that forward traffic to Google APIs, a network must be provided.
        # @!attribute [rw] network_tier
        #   @return [::String]
        #     This signifies the networking tier used for configuring this load balancer and can only take the following values: PREMIUM, STANDARD. For regional ForwardingRule, the valid values are PREMIUM and STANDARD. For GlobalForwardingRule, the valid value is PREMIUM. If this field is not specified, it is assumed to be PREMIUM. If IPAddress is specified, this value must be equal to the networkTier of the Address.
        #     Check the NetworkTier enum for the list of possible values.
        # @!attribute [rw] no_automate_dns_zone
        #   @return [::Boolean]
        #     This is used in PSC consumer ForwardingRule to control whether it should try to auto-generate a DNS zone or not. Non-PSC forwarding rules do not use this field. Once set, this field is not mutable.
        # @!attribute [rw] port_range
        #   @return [::String]
        #     The ports, portRange, and allPorts fields are mutually exclusive. Only packets addressed to ports in the specified range will be forwarded to the backends configured with this forwarding rule. The portRange field has the following limitations: - It requires that the forwarding rule IPProtocol be TCP, UDP, or SCTP, and - It's applicable only to the following products: external passthrough Network Load Balancers, internal and external proxy Network Load Balancers, internal and external Application Load Balancers, external protocol forwarding, and Classic VPN. - Some products have restrictions on what ports can be used. See port specifications for details. For external forwarding rules, two or more forwarding rules cannot use the same [IPAddress, IPProtocol] pair, and cannot have overlapping portRanges. For internal forwarding rules within the same VPC network, two or more forwarding rules cannot use the same [IPAddress, IPProtocol] pair, and cannot have overlapping portRanges. @pattern: \\d+(?:-\\d+)?
        # @!attribute [rw] ports
        #   @return [::Array<::String>]
        #     The ports, portRange, and allPorts fields are mutually exclusive. Only packets addressed to ports in the specified range will be forwarded to the backends configured with this forwarding rule. The ports field has the following limitations: - It requires that the forwarding rule IPProtocol be TCP, UDP, or SCTP, and - It's applicable only to the following products: internal passthrough Network Load Balancers, backend service-based external passthrough Network Load Balancers, and internal protocol forwarding. - You can specify a list of up to five ports by number, separated by commas. The ports can be contiguous or discontiguous. For external forwarding rules, two or more forwarding rules cannot use the same [IPAddress, IPProtocol] pair if they share at least one port number. For internal forwarding rules within the same VPC network, two or more forwarding rules cannot use the same [IPAddress, IPProtocol] pair if they share at least one port number. @pattern: \\d+(?:-\\d+)?
        # @!attribute [rw] psc_connection_id
        #   @return [::Integer]
        #     [Output Only] The PSC connection id of the PSC Forwarding Rule.
        # @!attribute [rw] psc_connection_status
        #   @return [::String]
        #     Check the PscConnectionStatus enum for the list of possible values.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] URL of the region where the regional forwarding rule resides. This field is not applicable to global forwarding rules. You must specify this field as part of the HTTP request URL. It is not settable as a field in the request body.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] service_directory_registrations
        #   @return [::Array<::Google::Cloud::Compute::V1::ForwardingRuleServiceDirectoryRegistration>]
        #     Service Directory resources to register this forwarding rule with. Currently, only supports a single Service Directory resource.
        # @!attribute [rw] service_label
        #   @return [::String]
        #     An optional prefix to the service name for this Forwarding Rule. If specified, the prefix is the first label of the fully qualified service name. The label must be 1-63 characters long, and comply with RFC1035. Specifically, the label must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash. This field is only used for internal load balancing.
        # @!attribute [rw] service_name
        #   @return [::String]
        #     [Output Only] The internal fully qualified service name for this Forwarding Rule. This field is only used for internal load balancing.
        # @!attribute [rw] source_ip_ranges
        #   @return [::Array<::String>]
        #     If not empty, this Forwarding Rule will only forward the traffic when the source IP address matches one of the IP addresses or CIDR ranges set here. Note that a Forwarding Rule can only have up to 64 source IP ranges, and this field can only be used with a regional Forwarding Rule whose scheme is EXTERNAL. Each source_ip_range entry should be either an IP address (for example, 1.2.3.4) or a CIDR range (for example, 1.2.3.0/24).
        # @!attribute [rw] subnetwork
        #   @return [::String]
        #     This field identifies the subnetwork that the load balanced IP should belong to for this Forwarding Rule, used in internal load balancing and network load balancing with IPv6. If the network specified is in auto subnet mode, this field is optional. However, a subnetwork must be specified if the network is in custom subnet mode or when creating external forwarding rule with IPv6.
        # @!attribute [rw] target
        #   @return [::String]
        #     The URL of the target resource to receive the matched traffic. For regional forwarding rules, this target must be in the same region as the forwarding rule. For global forwarding rules, this target must be a global load balancing resource. The forwarded traffic must be of a type appropriate to the target object. - For load balancers, see the "Target" column in [Port specifications](https://cloud.google.com/load-balancing/docs/forwarding-rule-concepts#ip_address_specifications). - For Private Service Connect forwarding rules that forward traffic to Google APIs, provide the name of a supported Google API bundle: - vpc-sc - APIs that support VPC Service Controls. - all-apis - All supported Google APIs. - For Private Service Connect forwarding rules that forward traffic to managed services, the target must be a service attachment. The target is not mutable once set as a service attachment.
        class ForwardingRule
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class LabelsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # The IP protocol to which this rule applies. For protocol forwarding, valid options are TCP, UDP, ESP, AH, SCTP, ICMP and L3_DEFAULT. The valid IP protocols are different for different load balancing products as described in [Load balancing features](https://cloud.google.com/load-balancing/docs/features#protocols_from_the_load_balancer_to_the_backends).
          module IPProtocolEnum
            # A value indicating that the enum field is not set.
            UNDEFINED_I_P_PROTOCOL_ENUM = 0

            AH = 2087

            ESP = 68_962

            ICMP = 2_241_597

            L3_DEFAULT = 48_151_369

            SCTP = 2_539_724

            TCP = 82_881

            UDP = 83_873
          end

          # The IP Version that will be used by this forwarding rule. Valid options are IPV4 or IPV6.
          module IpVersion
            # A value indicating that the enum field is not set.
            UNDEFINED_IP_VERSION = 0

            IPV4 = 2_254_341

            IPV6 = 2_254_343

            UNSPECIFIED_VERSION = 21_850_000
          end

          # Specifies the forwarding rule type. For more information about forwarding rules, refer to Forwarding rule concepts.
          module LoadBalancingScheme
            # A value indicating that the enum field is not set.
            UNDEFINED_LOAD_BALANCING_SCHEME = 0

            EXTERNAL = 35_607_499

            EXTERNAL_MANAGED = 512_006_923

            INTERNAL = 279_295_677

            INTERNAL_MANAGED = 37_350_397

            INTERNAL_SELF_MANAGED = 236_211_150

            INVALID = 530_283_991
          end

          # This signifies the networking tier used for configuring this load balancer and can only take the following values: PREMIUM, STANDARD. For regional ForwardingRule, the valid values are PREMIUM and STANDARD. For GlobalForwardingRule, the valid value is PREMIUM. If this field is not specified, it is assumed to be PREMIUM. If IPAddress is specified, this value must be equal to the networkTier of the Address.
          module NetworkTier
            # A value indicating that the enum field is not set.
            UNDEFINED_NETWORK_TIER = 0

            # Public internet quality with fixed bandwidth.
            FIXED_STANDARD = 310_464_328

            # High quality, Google-grade network tier, support for all networking products.
            PREMIUM = 399_530_551

            # Public internet quality, only limited support for other networking products.
            STANDARD = 484_642_493

            # (Output only) Temporary tier for FIXED_STANDARD when fixed standard tier is expired or not configured.
            STANDARD_OVERRIDES_FIXED_STANDARD = 465_847_234
          end

          module PscConnectionStatus
            # A value indicating that the enum field is not set.
            UNDEFINED_PSC_CONNECTION_STATUS = 0

            # The connection has been accepted by the producer.
            ACCEPTED = 246_714_279

            # The connection has been closed by the producer and will not serve traffic going forward.
            CLOSED = 380_163_436

            # The connection has been accepted by the producer, but the producer needs to take further action before the forwarding rule can serve traffic.
            NEEDS_ATTENTION = 344_491_452

            # The connection is pending acceptance by the producer.
            PENDING = 35_394_935

            # The connection has been rejected by the producer.
            REJECTED = 174_130_302

            STATUS_UNSPECIFIED = 42_133_066
          end
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::ForwardingRulesScopedList}]
        #     A list of ForwardingRulesScopedList resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#forwardingRuleAggregatedList for lists of forwarding rules.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class ForwardingRuleAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::ForwardingRulesScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # Contains a list of ForwardingRule resources.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::ForwardingRule>]
        #     A list of ForwardingRule resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     Type of resource.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class ForwardingRuleList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] forwarding_rule
        #   @return [::String]
        class ForwardingRuleReference
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Describes the auto-registration of the Forwarding Rule to Service Directory. The region and project of the Service Directory resource generated from this registration will be the same as this Forwarding Rule.
        # @!attribute [rw] namespace
        #   @return [::String]
        #     Service Directory namespace to register the forwarding rule under.
        # @!attribute [rw] service
        #   @return [::String]
        #     Service Directory service to register the forwarding rule under.
        # @!attribute [rw] service_directory_region
        #   @return [::String]
        #     [Optional] Service Directory region to register this global forwarding rule under. Default to "us-central1". Only used for PSC for Google APIs. All PSC for Google APIs Forwarding Rules on the same network should use the same Service Directory region.
        class ForwardingRuleServiceDirectoryRegistration
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] forwarding_rules
        #   @return [::Array<::Google::Cloud::Compute::V1::ForwardingRule>]
        #     A list of forwarding rules contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     Informational warning which replaces the list of forwarding rules when the list is empty.
        class ForwardingRulesScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] grpc_service_name
        #   @return [::String]
        #     The gRPC service name for the health check. This field is optional. The value of grpc_service_name has the following meanings by convention: - Empty service_name means the overall status of all services at the backend. - Non-empty service_name means the health of that gRPC service, as defined by the owner of the service. The grpc_service_name can only be ASCII.
        # @!attribute [rw] port
        #   @return [::Integer]
        #     The TCP port number to which the health check prober sends packets. Valid values are 1 through 65535.
        # @!attribute [rw] port_name
        #   @return [::String]
        #     Not supported.
        # @!attribute [rw] port_specification
        #   @return [::String]
        #     Specifies how a port is selected for health checking. Can be one of the following values: USE_FIXED_PORT: Specifies a port number explicitly using the port field in the health check. Supported by backend services for pass-through load balancers and backend services for proxy load balancers. Not supported by target pools. The health check supports all backends supported by the backend service provided the backend can be health checked. For example, GCE_VM_IP network endpoint groups, GCE_VM_IP_PORT network endpoint groups, and instance group backends. USE_NAMED_PORT: Not supported. USE_SERVING_PORT: Provides an indirect method of specifying the health check port by referring to the backend service. Only supported by backend services for proxy load balancers. Not supported by target pools. Not supported by backend services for pass-through load balancers. Supports all backends that can be health checked; for example, GCE_VM_IP_PORT network endpoint groups and instance group backends. For GCE_VM_IP_PORT network endpoint group backends, the health check uses the port number specified for each endpoint in the network endpoint group. For instance group backends, the health check uses the port number determined by looking up the backend service's named port in the instance group's list of named ports.
        #     Check the PortSpecification enum for the list of possible values.
        class GRPCHealthCheck
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Specifies how a port is selected for health checking. Can be one of the following values: USE_FIXED_PORT: Specifies a port number explicitly using the port field in the health check. Supported by backend services for pass-through load balancers and backend services for proxy load balancers. Not supported by target pools. The health check supports all backends supported by the backend service provided the backend can be health checked. For example, GCE_VM_IP network endpoint groups, GCE_VM_IP_PORT network endpoint groups, and instance group backends. USE_NAMED_PORT: Not supported. USE_SERVING_PORT: Provides an indirect method of specifying the health check port by referring to the backend service. Only supported by backend services for proxy load balancers. Not supported by target pools. Not supported by backend services for pass-through load balancers. Supports all backends that can be health checked; for example, GCE_VM_IP_PORT network endpoint groups and instance group backends. For GCE_VM_IP_PORT network endpoint group backends, the health check uses the port number specified for each endpoint in the network endpoint group. For instance group backends, the health check uses the port number determined by looking up the backend service's named port in the instance group's list of named ports.
          module PortSpecification
            # A value indicating that the enum field is not set.
            UNDEFINED_PORT_SPECIFICATION = 0

            # The port number in the health check's port is used for health checking. Applies to network endpoint group and instance group backends.
            USE_FIXED_PORT = 190_235_748

            # Not supported.
            USE_NAMED_PORT = 349_300_671

            # For network endpoint group backends, the health check uses the port number specified on each endpoint in the network endpoint group. For instance group backends, the health check uses the port number specified for the backend service's named port defined in the instance group's named ports.
            USE_SERVING_PORT = 362_637_516
          end
        end

        # A request message for AcceleratorTypes.Get. See the method description for details.
        # @!attribute [rw] accelerator_type
        #   @return [::String]
        #     Name of the accelerator type to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class GetAcceleratorTypeRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Addresses.Get. See the method description for details.
        # @!attribute [rw] address
        #   @return [::String]
        #     Name of the address resource to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        class GetAddressRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for FirewallPolicies.GetAssociation. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to which the queried rule belongs.
        # @!attribute [rw] name
        #   @return [::String]
        #     The name of the association to get from the firewall policy.
        class GetAssociationFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkFirewallPolicies.GetAssociation. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to which the queried association belongs.
        # @!attribute [rw] name
        #   @return [::String]
        #     The name of the association to get from the firewall policy.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        class GetAssociationNetworkFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionNetworkFirewallPolicies.GetAssociation. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to which the queried association belongs.
        # @!attribute [rw] name
        #   @return [::String]
        #     The name of the association to get from the firewall policy.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        class GetAssociationRegionNetworkFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Autoscalers.Get. See the method description for details.
        # @!attribute [rw] autoscaler
        #   @return [::String]
        #     Name of the autoscaler to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     Name of the zone for this request.
        class GetAutoscalerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for BackendBuckets.Get. See the method description for details.
        # @!attribute [rw] backend_bucket
        #   @return [::String]
        #     Name of the BackendBucket resource to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        class GetBackendBucketRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for BackendServices.Get. See the method description for details.
        # @!attribute [rw] backend_service
        #   @return [::String]
        #     Name of the BackendService resource to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        class GetBackendServiceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Interconnects.GetDiagnostics. See the method description for details.
        # @!attribute [rw] interconnect
        #   @return [::String]
        #     Name of the interconnect resource to query.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        class GetDiagnosticsInterconnectRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Disks.Get. See the method description for details.
        # @!attribute [rw] disk
        #   @return [::String]
        #     Name of the persistent disk to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class GetDiskRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for DiskTypes.Get. See the method description for details.
        # @!attribute [rw] disk_type
        #   @return [::String]
        #     Name of the disk type to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class GetDiskTypeRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.GetEffectiveFirewalls. See the method description for details.
        # @!attribute [rw] instance
        #   @return [::String]
        #     Name of the instance scoping this request.
        # @!attribute [rw] network_interface
        #   @return [::String]
        #     The name of the network interface to get the effective firewalls.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class GetEffectiveFirewallsInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Networks.GetEffectiveFirewalls. See the method description for details.
        # @!attribute [rw] network
        #   @return [::String]
        #     Name of the network for this request.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        class GetEffectiveFirewallsNetworkRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionNetworkFirewallPolicies.GetEffectiveFirewalls. See the method description for details.
        # @!attribute [rw] network
        #   @return [::String]
        #     Network reference
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        class GetEffectiveFirewallsRegionNetworkFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ExternalVpnGateways.Get. See the method description for details.
        # @!attribute [rw] external_vpn_gateway
        #   @return [::String]
        #     Name of the externalVpnGateway to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        class GetExternalVpnGatewayRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for FirewallPolicies.Get. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to get.
        class GetFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Firewalls.Get. See the method description for details.
        # @!attribute [rw] firewall
        #   @return [::String]
        #     Name of the firewall rule to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        class GetFirewallRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ForwardingRules.Get. See the method description for details.
        # @!attribute [rw] forwarding_rule
        #   @return [::String]
        #     Name of the ForwardingRule resource to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        class GetForwardingRuleRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Images.GetFromFamily. See the method description for details.
        # @!attribute [rw] family
        #   @return [::String]
        #     Name of the image family to search for.
        # @!attribute [rw] project
        #   @return [::String]
        #     The image project that the image belongs to. For example, to get a CentOS image, specify centos-cloud as the image project.
        class GetFromFamilyImageRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for GlobalAddresses.Get. See the method description for details.
        # @!attribute [rw] address
        #   @return [::String]
        #     Name of the address resource to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        class GetGlobalAddressRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for GlobalForwardingRules.Get. See the method description for details.
        # @!attribute [rw] forwarding_rule
        #   @return [::String]
        #     Name of the ForwardingRule resource to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        class GetGlobalForwardingRuleRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for GlobalNetworkEndpointGroups.Get. See the method description for details.
        # @!attribute [rw] network_endpoint_group
        #   @return [::String]
        #     The name of the network endpoint group. It should comply with RFC1035.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        class GetGlobalNetworkEndpointGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for GlobalOperations.Get. See the method description for details.
        # @!attribute [rw] operation
        #   @return [::String]
        #     Name of the Operations resource to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        class GetGlobalOperationRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for GlobalOrganizationOperations.Get. See the method description for details.
        # @!attribute [rw] operation
        #   @return [::String]
        #     Name of the Operations resource to return.
        # @!attribute [rw] parent_id
        #   @return [::String]
        #     Parent ID for this request.
        class GetGlobalOrganizationOperationRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for GlobalPublicDelegatedPrefixes.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] public_delegated_prefix
        #   @return [::String]
        #     Name of the PublicDelegatedPrefix resource to return.
        class GetGlobalPublicDelegatedPrefixeRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.GetGuestAttributes. See the method description for details.
        # @!attribute [rw] instance
        #   @return [::String]
        #     Name of the instance scoping this request.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] query_path
        #   @return [::String]
        #     Specifies the guest attributes path to be queried.
        # @!attribute [rw] variable_key
        #   @return [::String]
        #     Specifies the key for the guest attributes entry.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class GetGuestAttributesInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for BackendServices.GetHealth. See the method description for details.
        # @!attribute [rw] backend_service
        #   @return [::String]
        #     Name of the BackendService resource to which the queried instance belongs.
        # @!attribute [rw] project
        #   @return [::String]
        # @!attribute [rw] resource_group_reference_resource
        #   @return [::Google::Cloud::Compute::V1::ResourceGroupReference]
        #     The body resource for this request
        class GetHealthBackendServiceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for HealthChecks.Get. See the method description for details.
        # @!attribute [rw] health_check
        #   @return [::String]
        #     Name of the HealthCheck resource to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        class GetHealthCheckRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionBackendServices.GetHealth. See the method description for details.
        # @!attribute [rw] backend_service
        #   @return [::String]
        #     Name of the BackendService resource for which to get health.
        # @!attribute [rw] project
        #   @return [::String]
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] resource_group_reference_resource
        #   @return [::Google::Cloud::Compute::V1::ResourceGroupReference]
        #     The body resource for this request
        class GetHealthRegionBackendServiceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetPools.GetHealth. See the method description for details.
        # @!attribute [rw] instance_reference_resource
        #   @return [::Google::Cloud::Compute::V1::InstanceReference]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] target_pool
        #   @return [::String]
        #     Name of the TargetPool resource to which the queried instance belongs.
        class GetHealthTargetPoolRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for BackendBuckets.GetIamPolicy. See the method description for details.
        # @!attribute [rw] options_requested_policy_version
        #   @return [::Integer]
        #     Requested IAM Policy version.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class GetIamPolicyBackendBucketRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for BackendServices.GetIamPolicy. See the method description for details.
        # @!attribute [rw] options_requested_policy_version
        #   @return [::Integer]
        #     Requested IAM Policy version.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class GetIamPolicyBackendServiceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Disks.GetIamPolicy. See the method description for details.
        # @!attribute [rw] options_requested_policy_version
        #   @return [::Integer]
        #     Requested IAM Policy version.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class GetIamPolicyDiskRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for FirewallPolicies.GetIamPolicy. See the method description for details.
        # @!attribute [rw] options_requested_policy_version
        #   @return [::Integer]
        #     Requested IAM Policy version.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class GetIamPolicyFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Images.GetIamPolicy. See the method description for details.
        # @!attribute [rw] options_requested_policy_version
        #   @return [::Integer]
        #     Requested IAM Policy version.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class GetIamPolicyImageRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.GetIamPolicy. See the method description for details.
        # @!attribute [rw] options_requested_policy_version
        #   @return [::Integer]
        #     Requested IAM Policy version.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class GetIamPolicyInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceTemplates.GetIamPolicy. See the method description for details.
        # @!attribute [rw] options_requested_policy_version
        #   @return [::Integer]
        #     Requested IAM Policy version.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class GetIamPolicyInstanceTemplateRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Licenses.GetIamPolicy. See the method description for details.
        # @!attribute [rw] options_requested_policy_version
        #   @return [::Integer]
        #     Requested IAM Policy version.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class GetIamPolicyLicenseRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for MachineImages.GetIamPolicy. See the method description for details.
        # @!attribute [rw] options_requested_policy_version
        #   @return [::Integer]
        #     Requested IAM Policy version.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class GetIamPolicyMachineImageRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkAttachments.GetIamPolicy. See the method description for details.
        # @!attribute [rw] options_requested_policy_version
        #   @return [::Integer]
        #     Requested IAM Policy version.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class GetIamPolicyNetworkAttachmentRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkFirewallPolicies.GetIamPolicy. See the method description for details.
        # @!attribute [rw] options_requested_policy_version
        #   @return [::Integer]
        #     Requested IAM Policy version.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class GetIamPolicyNetworkFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NodeGroups.GetIamPolicy. See the method description for details.
        # @!attribute [rw] options_requested_policy_version
        #   @return [::Integer]
        #     Requested IAM Policy version.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class GetIamPolicyNodeGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NodeTemplates.GetIamPolicy. See the method description for details.
        # @!attribute [rw] options_requested_policy_version
        #   @return [::Integer]
        #     Requested IAM Policy version.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class GetIamPolicyNodeTemplateRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionBackendServices.GetIamPolicy. See the method description for details.
        # @!attribute [rw] options_requested_policy_version
        #   @return [::Integer]
        #     Requested IAM Policy version.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class GetIamPolicyRegionBackendServiceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionDisks.GetIamPolicy. See the method description for details.
        # @!attribute [rw] options_requested_policy_version
        #   @return [::Integer]
        #     Requested IAM Policy version.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class GetIamPolicyRegionDiskRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionNetworkFirewallPolicies.GetIamPolicy. See the method description for details.
        # @!attribute [rw] options_requested_policy_version
        #   @return [::Integer]
        #     Requested IAM Policy version.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class GetIamPolicyRegionNetworkFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Reservations.GetIamPolicy. See the method description for details.
        # @!attribute [rw] options_requested_policy_version
        #   @return [::Integer]
        #     Requested IAM Policy version.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class GetIamPolicyReservationRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ResourcePolicies.GetIamPolicy. See the method description for details.
        # @!attribute [rw] options_requested_policy_version
        #   @return [::Integer]
        #     Requested IAM Policy version.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class GetIamPolicyResourcePolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ServiceAttachments.GetIamPolicy. See the method description for details.
        # @!attribute [rw] options_requested_policy_version
        #   @return [::Integer]
        #     Requested IAM Policy version.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class GetIamPolicyServiceAttachmentRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Snapshots.GetIamPolicy. See the method description for details.
        # @!attribute [rw] options_requested_policy_version
        #   @return [::Integer]
        #     Requested IAM Policy version.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class GetIamPolicySnapshotRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Subnetworks.GetIamPolicy. See the method description for details.
        # @!attribute [rw] options_requested_policy_version
        #   @return [::Integer]
        #     Requested IAM Policy version.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class GetIamPolicySubnetworkRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ImageFamilyViews.Get. See the method description for details.
        # @!attribute [rw] family
        #   @return [::String]
        #     Name of the image family to search for.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class GetImageFamilyViewRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Images.Get. See the method description for details.
        # @!attribute [rw] image
        #   @return [::String]
        #     Name of the image resource to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        class GetImageRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceGroupManagers.Get. See the method description for details.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     The name of the managed instance group.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where the managed instance group is located.
        class GetInstanceGroupManagerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceGroups.Get. See the method description for details.
        # @!attribute [rw] instance_group
        #   @return [::String]
        #     The name of the instance group.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where the instance group is located.
        class GetInstanceGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.Get. See the method description for details.
        # @!attribute [rw] instance
        #   @return [::String]
        #     Name of the instance resource to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class GetInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceTemplates.Get. See the method description for details.
        # @!attribute [rw] instance_template
        #   @return [::String]
        #     The name of the instance template.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        class GetInstanceTemplateRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InterconnectAttachments.Get. See the method description for details.
        # @!attribute [rw] interconnect_attachment
        #   @return [::String]
        #     Name of the interconnect attachment to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        class GetInterconnectAttachmentRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InterconnectLocations.Get. See the method description for details.
        # @!attribute [rw] interconnect_location
        #   @return [::String]
        #     Name of the interconnect location to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        class GetInterconnectLocationRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InterconnectRemoteLocations.Get. See the method description for details.
        # @!attribute [rw] interconnect_remote_location
        #   @return [::String]
        #     Name of the interconnect remote location to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        class GetInterconnectRemoteLocationRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Interconnects.Get. See the method description for details.
        # @!attribute [rw] interconnect
        #   @return [::String]
        #     Name of the interconnect to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        class GetInterconnectRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for LicenseCodes.Get. See the method description for details.
        # @!attribute [rw] license_code
        #   @return [::String]
        #     Number corresponding to the License code resource to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        class GetLicenseCodeRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Licenses.Get. See the method description for details.
        # @!attribute [rw] license
        #   @return [::String]
        #     Name of the License resource to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        class GetLicenseRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for MachineImages.Get. See the method description for details.
        # @!attribute [rw] machine_image
        #   @return [::String]
        #     The name of the machine image.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        class GetMachineImageRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for MachineTypes.Get. See the method description for details.
        # @!attribute [rw] machine_type
        #   @return [::String]
        #     Name of the machine type to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class GetMachineTypeRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Interconnects.GetMacsecConfig. See the method description for details.
        # @!attribute [rw] interconnect
        #   @return [::String]
        #     Name of the interconnect resource to query.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        class GetMacsecConfigInterconnectRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Routers.GetNatIpInfo. See the method description for details.
        # @!attribute [rw] nat_name
        #   @return [::String]
        #     Name of the nat service to filter the NAT IP information. If it is omitted, all nats for this router will be returned. Name should conform to RFC1035.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] router
        #   @return [::String]
        #     Name of the Router resource to query for Nat IP information. The name should conform to RFC1035.
        class GetNatIpInfoRouterRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Routers.GetNatMappingInfo. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] nat_name
        #   @return [::String]
        #     Name of the nat service to filter the Nat Mapping information. If it is omitted, all nats for this router will be returned. Name should conform to RFC1035.
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] router
        #   @return [::String]
        #     Name of the Router resource to query for Nat Mapping information of VM endpoints.
        class GetNatMappingInfoRoutersRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkAttachments.Get. See the method description for details.
        # @!attribute [rw] network_attachment
        #   @return [::String]
        #     Name of the NetworkAttachment resource to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region of this request.
        class GetNetworkAttachmentRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkEdgeSecurityServices.Get. See the method description for details.
        # @!attribute [rw] network_edge_security_service
        #   @return [::String]
        #     Name of the network edge security service to get.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        class GetNetworkEdgeSecurityServiceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkEndpointGroups.Get. See the method description for details.
        # @!attribute [rw] network_endpoint_group
        #   @return [::String]
        #     The name of the network endpoint group. It should comply with RFC1035.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where the network endpoint group is located. It should comply with RFC1035.
        class GetNetworkEndpointGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkFirewallPolicies.Get. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to get.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        class GetNetworkFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Networks.Get. See the method description for details.
        # @!attribute [rw] network
        #   @return [::String]
        #     Name of the network to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        class GetNetworkRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NodeGroups.Get. See the method description for details.
        # @!attribute [rw] node_group
        #   @return [::String]
        #     Name of the node group to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class GetNodeGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NodeTemplates.Get. See the method description for details.
        # @!attribute [rw] node_template
        #   @return [::String]
        #     Name of the node template to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        class GetNodeTemplateRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NodeTypes.Get. See the method description for details.
        # @!attribute [rw] node_type
        #   @return [::String]
        #     Name of the node type to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class GetNodeTypeRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for PacketMirrorings.Get. See the method description for details.
        # @!attribute [rw] packet_mirroring
        #   @return [::String]
        #     Name of the PacketMirroring resource to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        class GetPacketMirroringRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Projects.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        class GetProjectRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for PublicAdvertisedPrefixes.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] public_advertised_prefix
        #   @return [::String]
        #     Name of the PublicAdvertisedPrefix resource to return.
        class GetPublicAdvertisedPrefixeRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for PublicDelegatedPrefixes.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] public_delegated_prefix
        #   @return [::String]
        #     Name of the PublicDelegatedPrefix resource to return.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region of this request.
        class GetPublicDelegatedPrefixeRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionAutoscalers.Get. See the method description for details.
        # @!attribute [rw] autoscaler
        #   @return [::String]
        #     Name of the autoscaler to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        class GetRegionAutoscalerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionBackendServices.Get. See the method description for details.
        # @!attribute [rw] backend_service
        #   @return [::String]
        #     Name of the BackendService resource to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        class GetRegionBackendServiceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionCommitments.Get. See the method description for details.
        # @!attribute [rw] commitment
        #   @return [::String]
        #     Name of the commitment to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        class GetRegionCommitmentRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionDisks.Get. See the method description for details.
        # @!attribute [rw] disk
        #   @return [::String]
        #     Name of the regional persistent disk to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        class GetRegionDiskRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionDiskTypes.Get. See the method description for details.
        # @!attribute [rw] disk_type
        #   @return [::String]
        #     Name of the disk type to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        class GetRegionDiskTypeRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionHealthChecks.Get. See the method description for details.
        # @!attribute [rw] health_check
        #   @return [::String]
        #     Name of the HealthCheck resource to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        class GetRegionHealthCheckRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionHealthCheckServices.Get. See the method description for details.
        # @!attribute [rw] health_check_service
        #   @return [::String]
        #     Name of the HealthCheckService to update. The name must be 1-63 characters long, and comply with RFC1035.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        class GetRegionHealthCheckServiceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionInstanceGroupManagers.Get. See the method description for details.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     Name of the managed instance group to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        class GetRegionInstanceGroupManagerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionInstanceGroups.Get. See the method description for details.
        # @!attribute [rw] instance_group
        #   @return [::String]
        #     Name of the instance group resource to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        class GetRegionInstanceGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionInstanceTemplates.Get. See the method description for details.
        # @!attribute [rw] instance_template
        #   @return [::String]
        #     The name of the instance template.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        class GetRegionInstanceTemplateRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionNetworkEndpointGroups.Get. See the method description for details.
        # @!attribute [rw] network_endpoint_group
        #   @return [::String]
        #     The name of the network endpoint group. It should comply with RFC1035.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region where the network endpoint group is located. It should comply with RFC1035.
        class GetRegionNetworkEndpointGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionNetworkFirewallPolicies.Get. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to get.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        class GetRegionNetworkFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionNotificationEndpoints.Get. See the method description for details.
        # @!attribute [rw] notification_endpoint
        #   @return [::String]
        #     Name of the NotificationEndpoint resource to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        class GetRegionNotificationEndpointRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionOperations.Get. See the method description for details.
        # @!attribute [rw] operation
        #   @return [::String]
        #     Name of the Operations resource to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        class GetRegionOperationRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Regions.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region resource to return.
        class GetRegionRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionSecurityPolicies.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] security_policy
        #   @return [::String]
        #     Name of the security policy to get.
        class GetRegionSecurityPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionSslCertificates.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] ssl_certificate
        #   @return [::String]
        #     Name of the SslCertificate resource to return.
        class GetRegionSslCertificateRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionSslPolicies.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] ssl_policy
        #   @return [::String]
        #     Name of the SSL policy to update. The name must be 1-63 characters long, and comply with RFC1035.
        class GetRegionSslPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionTargetHttpProxies.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] target_http_proxy
        #   @return [::String]
        #     Name of the TargetHttpProxy resource to return.
        class GetRegionTargetHttpProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionTargetHttpsProxies.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] target_https_proxy
        #   @return [::String]
        #     Name of the TargetHttpsProxy resource to return.
        class GetRegionTargetHttpsProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionTargetTcpProxies.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] target_tcp_proxy
        #   @return [::String]
        #     Name of the TargetTcpProxy resource to return.
        class GetRegionTargetTcpProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionUrlMaps.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] url_map
        #   @return [::String]
        #     Name of the UrlMap resource to return.
        class GetRegionUrlMapRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Reservations.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] reservation
        #   @return [::String]
        #     Name of the reservation to retrieve.
        # @!attribute [rw] zone
        #   @return [::String]
        #     Name of the zone for this request.
        class GetReservationRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ResourcePolicies.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] resource_policy
        #   @return [::String]
        #     Name of the resource policy to retrieve.
        class GetResourcePolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Routes.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] route
        #   @return [::String]
        #     Name of the Route resource to return.
        class GetRouteRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Routers.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] router
        #   @return [::String]
        #     Name of the Router resource to return.
        class GetRouterRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Routers.GetRouterStatus. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] router
        #   @return [::String]
        #     Name of the Router resource to query.
        class GetRouterStatusRouterRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for FirewallPolicies.GetRule. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to which the queried rule belongs.
        # @!attribute [rw] priority
        #   @return [::Integer]
        #     The priority of the rule to get from the firewall policy.
        class GetRuleFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkFirewallPolicies.GetRule. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to which the queried rule belongs.
        # @!attribute [rw] priority
        #   @return [::Integer]
        #     The priority of the rule to get from the firewall policy.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        class GetRuleNetworkFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionNetworkFirewallPolicies.GetRule. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to which the queried rule belongs.
        # @!attribute [rw] priority
        #   @return [::Integer]
        #     The priority of the rule to get from the firewall policy.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        class GetRuleRegionNetworkFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionSecurityPolicies.GetRule. See the method description for details.
        # @!attribute [rw] priority
        #   @return [::Integer]
        #     The priority of the rule to get from the security policy.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] security_policy
        #   @return [::String]
        #     Name of the security policy to which the queried rule belongs.
        class GetRuleRegionSecurityPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for SecurityPolicies.GetRule. See the method description for details.
        # @!attribute [rw] priority
        #   @return [::Integer]
        #     The priority of the rule to get from the security policy.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] security_policy
        #   @return [::String]
        #     Name of the security policy to which the queried rule belongs.
        class GetRuleSecurityPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.GetScreenshot. See the method description for details.
        # @!attribute [rw] instance
        #   @return [::String]
        #     Name of the instance scoping this request.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class GetScreenshotInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for SecurityPolicies.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] security_policy
        #   @return [::String]
        #     Name of the security policy to get.
        class GetSecurityPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.GetSerialPortOutput. See the method description for details.
        # @!attribute [rw] instance
        #   @return [::String]
        #     Name of the instance for this request.
        # @!attribute [rw] port
        #   @return [::Integer]
        #     Specifies which COM or serial port to retrieve data from.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] start
        #   @return [::Integer]
        #     Specifies the starting byte position of the output to return. To start with the first byte of output to the specified port, omit this field or set it to `0`. If the output for that byte position is available, this field matches the `start` parameter sent with the request. If the amount of serial console output exceeds the size of the buffer (1 MB), the oldest output is discarded and is no longer available. If the requested start position refers to discarded output, the start position is adjusted to the oldest output still available, and the adjusted start position is returned as the `start` property value. You can also provide a negative start position, which translates to the most recent number of bytes written to the serial port. For example, -3 is interpreted as the most recent 3 bytes written to the serial console.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class GetSerialPortOutputInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ServiceAttachments.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region of this request.
        # @!attribute [rw] service_attachment
        #   @return [::String]
        #     Name of the ServiceAttachment resource to return.
        class GetServiceAttachmentRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.GetShieldedInstanceIdentity. See the method description for details.
        # @!attribute [rw] instance
        #   @return [::String]
        #     Name or id of the instance scoping this request.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class GetShieldedInstanceIdentityInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Snapshots.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] snapshot
        #   @return [::String]
        #     Name of the Snapshot resource to return.
        class GetSnapshotRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for SnapshotSettingsService.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        class GetSnapshotSettingRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for SslCertificates.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] ssl_certificate
        #   @return [::String]
        #     Name of the SslCertificate resource to return.
        class GetSslCertificateRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for SslPolicies.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] ssl_policy
        #   @return [::String]
        #     Name of the SSL policy to update. The name must be 1-63 characters long, and comply with RFC1035.
        class GetSslPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for VpnGateways.GetStatus. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] vpn_gateway
        #   @return [::String]
        #     Name of the VPN gateway to return.
        class GetStatusVpnGatewayRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Subnetworks.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] subnetwork
        #   @return [::String]
        #     Name of the Subnetwork resource to return.
        class GetSubnetworkRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetGrpcProxies.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] target_grpc_proxy
        #   @return [::String]
        #     Name of the TargetGrpcProxy resource to return.
        class GetTargetGrpcProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetHttpProxies.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] target_http_proxy
        #   @return [::String]
        #     Name of the TargetHttpProxy resource to return.
        class GetTargetHttpProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetHttpsProxies.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] target_https_proxy
        #   @return [::String]
        #     Name of the TargetHttpsProxy resource to return.
        class GetTargetHttpsProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetInstances.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] target_instance
        #   @return [::String]
        #     Name of the TargetInstance resource to return.
        # @!attribute [rw] zone
        #   @return [::String]
        #     Name of the zone scoping this request.
        class GetTargetInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetPools.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] target_pool
        #   @return [::String]
        #     Name of the TargetPool resource to return.
        class GetTargetPoolRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetSslProxies.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] target_ssl_proxy
        #   @return [::String]
        #     Name of the TargetSslProxy resource to return.
        class GetTargetSslProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetTcpProxies.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] target_tcp_proxy
        #   @return [::String]
        #     Name of the TargetTcpProxy resource to return.
        class GetTargetTcpProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetVpnGateways.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] target_vpn_gateway
        #   @return [::String]
        #     Name of the target VPN gateway to return.
        class GetTargetVpnGatewayRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for UrlMaps.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] url_map
        #   @return [::String]
        #     Name of the UrlMap resource to return.
        class GetUrlMapRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for VpnGateways.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] vpn_gateway
        #   @return [::String]
        #     Name of the VPN gateway to return.
        class GetVpnGatewayRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for VpnTunnels.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] vpn_tunnel
        #   @return [::String]
        #     Name of the VpnTunnel resource to return.
        class GetVpnTunnelRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Projects.GetXpnHost. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        class GetXpnHostProjectRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Projects.GetXpnResources. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class GetXpnResourcesProjectsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ZoneOperations.Get. See the method description for details.
        # @!attribute [rw] operation
        #   @return [::String]
        #     Name of the Operations resource to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     Name of the zone for this request.
        class GetZoneOperationRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Zones.Get. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     Name of the zone resource to return.
        class GetZoneRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] description
        #   @return [::String]
        #     An optional destination address description if intended to be different from the source.
        # @!attribute [rw] destination_address
        #   @return [::String]
        #     The URL of the destination address to move to. This can be a full or partial URL. For example, the following are all valid URLs to a address: - https://www.googleapis.com/compute/v1/projects/project /global/addresses/address - projects/project/global/addresses/address Note that destination project must be different from the source project. So /global/addresses/address is not valid partial url.
        class GlobalAddressesMoveRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] network_endpoints
        #   @return [::Array<::Google::Cloud::Compute::V1::NetworkEndpoint>]
        #     The list of network endpoints to be attached.
        class GlobalNetworkEndpointGroupsAttachEndpointsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] network_endpoints
        #   @return [::Array<::Google::Cloud::Compute::V1::NetworkEndpoint>]
        #     The list of network endpoints to be detached.
        class GlobalNetworkEndpointGroupsDetachEndpointsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] bindings
        #   @return [::Array<::Google::Cloud::Compute::V1::Binding>]
        #     Flatten Policy to create a backward compatible wire-format. Deprecated. Use 'policy' to specify bindings.
        # @!attribute [rw] etag
        #   @return [::String]
        #     Flatten Policy to create a backward compatible wire-format. Deprecated. Use 'policy' to specify the etag.
        # @!attribute [rw] policy
        #   @return [::Google::Cloud::Compute::V1::Policy]
        #     REQUIRED: The complete policy to be applied to the 'resource'. The size of the policy is limited to a few 10s of KB. An empty policy is in general a valid policy but certain services (like Projects) might reject them.
        class GlobalOrganizationSetPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] label_fingerprint
        #   @return [::String]
        #     The fingerprint of the previous set of labels for this resource, used to detect conflicts. The fingerprint is initially generated by Compute Engine and changes after every request to modify or update labels. You must always provide an up-to-date fingerprint hash when updating or changing labels, otherwise the request will fail with error 412 conditionNotMet. Make a get() request to the resource to get the latest fingerprint.
        # @!attribute [rw] labels
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     A list of labels to apply for this resource. Each label must comply with the requirements for labels. For example, "webserver-frontend": "images". A label value can also be empty (e.g. "my-label": "").
        class GlobalSetLabelsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class LabelsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # @!attribute [rw] bindings
        #   @return [::Array<::Google::Cloud::Compute::V1::Binding>]
        #     Flatten Policy to create a backward compatible wire-format. Deprecated. Use 'policy' to specify bindings.
        # @!attribute [rw] etag
        #   @return [::String]
        #     Flatten Policy to create a backward compatible wire-format. Deprecated. Use 'policy' to specify the etag.
        # @!attribute [rw] policy
        #   @return [::Google::Cloud::Compute::V1::Policy]
        #     REQUIRED: The complete policy to be applied to the 'resource'. The size of the policy is limited to a few 10s of KB. An empty policy is in general a valid policy but certain services (like Projects) might reject them.
        class GlobalSetPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A guest attributes entry.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#guestAttributes for guest attributes entry.
        # @!attribute [rw] query_path
        #   @return [::String]
        #     The path to be queried. This can be the default namespace ('') or a nested namespace ('\/') or a specified key ('\/\').
        # @!attribute [rw] query_value
        #   @return [::Google::Cloud::Compute::V1::GuestAttributesValue]
        #     [Output Only] The value of the requested queried path.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] variable_key
        #   @return [::String]
        #     The key to search for.
        # @!attribute [rw] variable_value
        #   @return [::String]
        #     [Output Only] The value found for the requested key.
        class GuestAttributes
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A guest attributes namespace/key/value entry.
        # @!attribute [rw] key
        #   @return [::String]
        #     Key for the guest attribute entry.
        # @!attribute [rw] namespace
        #   @return [::String]
        #     Namespace for the guest attribute entry.
        # @!attribute [rw] value
        #   @return [::String]
        #     Value for the guest attribute entry.
        class GuestAttributesEntry
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Array of guest attribute namespace/key/value tuples.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::GuestAttributesEntry>]
        class GuestAttributesValue
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Guest OS features.
        # @!attribute [rw] type
        #   @return [::String]
        #     The ID of a supported feature. To add multiple values, use commas to separate values. Set to one or more of the following values: - VIRTIO_SCSI_MULTIQUEUE - WINDOWS - MULTI_IP_SUBNET - UEFI_COMPATIBLE - GVNIC - SEV_CAPABLE - SUSPEND_RESUME_COMPATIBLE - SEV_LIVE_MIGRATABLE - SEV_SNP_CAPABLE For more information, see Enabling guest operating system features.
        #     Check the Type enum for the list of possible values.
        class GuestOsFeature
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The ID of a supported feature. To add multiple values, use commas to separate values. Set to one or more of the following values: - VIRTIO_SCSI_MULTIQUEUE - WINDOWS - MULTI_IP_SUBNET - UEFI_COMPATIBLE - GVNIC - SEV_CAPABLE - SUSPEND_RESUME_COMPATIBLE - SEV_LIVE_MIGRATABLE - SEV_SNP_CAPABLE For more information, see Enabling guest operating system features.
          module Type
            # A value indicating that the enum field is not set.
            UNDEFINED_TYPE = 0

            FEATURE_TYPE_UNSPECIFIED = 531_767_259

            GVNIC = 68_209_305

            MULTI_IP_SUBNET = 151_776_719

            SECURE_BOOT = 376_811_194

            SEV_CAPABLE = 87_083_793

            SEV_LIVE_MIGRATABLE = 392_039_820

            SEV_LIVE_MIGRATABLE_V2 = 168_551_983

            SEV_SNP_CAPABLE = 426_919

            UEFI_COMPATIBLE = 195_865_408

            VIRTIO_SCSI_MULTIQUEUE = 201_597_069

            WINDOWS = 456_863_331
          end
        end

        # @!attribute [rw] host
        #   @return [::String]
        #     The value of the host header in the HTTP/2 health check request. If left empty (default value), the host header is set to the destination IP address to which health check packets are sent. The destination IP address depends on the type of load balancer. For details, see: https://cloud.google.com/load-balancing/docs/health-check-concepts#hc-packet-dest
        # @!attribute [rw] port
        #   @return [::Integer]
        #     The TCP port number to which the health check prober sends packets. The default value is 443. Valid values are 1 through 65535.
        # @!attribute [rw] port_name
        #   @return [::String]
        #     Not supported.
        # @!attribute [rw] port_specification
        #   @return [::String]
        #     Specifies how a port is selected for health checking. Can be one of the following values: USE_FIXED_PORT: Specifies a port number explicitly using the port field in the health check. Supported by backend services for pass-through load balancers and backend services for proxy load balancers. Not supported by target pools. The health check supports all backends supported by the backend service provided the backend can be health checked. For example, GCE_VM_IP network endpoint groups, GCE_VM_IP_PORT network endpoint groups, and instance group backends. USE_NAMED_PORT: Not supported. USE_SERVING_PORT: Provides an indirect method of specifying the health check port by referring to the backend service. Only supported by backend services for proxy load balancers. Not supported by target pools. Not supported by backend services for pass-through load balancers. Supports all backends that can be health checked; for example, GCE_VM_IP_PORT network endpoint groups and instance group backends. For GCE_VM_IP_PORT network endpoint group backends, the health check uses the port number specified for each endpoint in the network endpoint group. For instance group backends, the health check uses the port number determined by looking up the backend service's named port in the instance group's list of named ports.
        #     Check the PortSpecification enum for the list of possible values.
        # @!attribute [rw] proxy_header
        #   @return [::String]
        #     Specifies the type of proxy header to append before sending data to the backend, either NONE or PROXY_V1. The default is NONE.
        #     Check the ProxyHeader enum for the list of possible values.
        # @!attribute [rw] request_path
        #   @return [::String]
        #     The request path of the HTTP/2 health check request. The default value is /.
        # @!attribute [rw] response
        #   @return [::String]
        #     Creates a content-based HTTP/2 health check. In addition to the required HTTP 200 (OK) status code, you can configure the health check to pass only when the backend sends this specific ASCII response string within the first 1024 bytes of the HTTP response body. For details, see: https://cloud.google.com/load-balancing/docs/health-check-concepts#criteria-protocol-http
        class HTTP2HealthCheck
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Specifies how a port is selected for health checking. Can be one of the following values: USE_FIXED_PORT: Specifies a port number explicitly using the port field in the health check. Supported by backend services for pass-through load balancers and backend services for proxy load balancers. Not supported by target pools. The health check supports all backends supported by the backend service provided the backend can be health checked. For example, GCE_VM_IP network endpoint groups, GCE_VM_IP_PORT network endpoint groups, and instance group backends. USE_NAMED_PORT: Not supported. USE_SERVING_PORT: Provides an indirect method of specifying the health check port by referring to the backend service. Only supported by backend services for proxy load balancers. Not supported by target pools. Not supported by backend services for pass-through load balancers. Supports all backends that can be health checked; for example, GCE_VM_IP_PORT network endpoint groups and instance group backends. For GCE_VM_IP_PORT network endpoint group backends, the health check uses the port number specified for each endpoint in the network endpoint group. For instance group backends, the health check uses the port number determined by looking up the backend service's named port in the instance group's list of named ports.
          module PortSpecification
            # A value indicating that the enum field is not set.
            UNDEFINED_PORT_SPECIFICATION = 0

            # The port number in the health check's port is used for health checking. Applies to network endpoint group and instance group backends.
            USE_FIXED_PORT = 190_235_748

            # Not supported.
            USE_NAMED_PORT = 349_300_671

            # For network endpoint group backends, the health check uses the port number specified on each endpoint in the network endpoint group. For instance group backends, the health check uses the port number specified for the backend service's named port defined in the instance group's named ports.
            USE_SERVING_PORT = 362_637_516
          end

          # Specifies the type of proxy header to append before sending data to the backend, either NONE or PROXY_V1. The default is NONE.
          module ProxyHeader
            # A value indicating that the enum field is not set.
            UNDEFINED_PROXY_HEADER = 0

            NONE = 2_402_104

            PROXY_V1 = 334_352_940
          end
        end

        # @!attribute [rw] host
        #   @return [::String]
        #     The value of the host header in the HTTP health check request. If left empty (default value), the host header is set to the destination IP address to which health check packets are sent. The destination IP address depends on the type of load balancer. For details, see: https://cloud.google.com/load-balancing/docs/health-check-concepts#hc-packet-dest
        # @!attribute [rw] port
        #   @return [::Integer]
        #     The TCP port number to which the health check prober sends packets. The default value is 80. Valid values are 1 through 65535.
        # @!attribute [rw] port_name
        #   @return [::String]
        #     Not supported.
        # @!attribute [rw] port_specification
        #   @return [::String]
        #     Specifies how a port is selected for health checking. Can be one of the following values: USE_FIXED_PORT: Specifies a port number explicitly using the port field in the health check. Supported by backend services for pass-through load balancers and backend services for proxy load balancers. Also supported in legacy HTTP health checks for target pools. The health check supports all backends supported by the backend service provided the backend can be health checked. For example, GCE_VM_IP network endpoint groups, GCE_VM_IP_PORT network endpoint groups, and instance group backends. USE_NAMED_PORT: Not supported. USE_SERVING_PORT: Provides an indirect method of specifying the health check port by referring to the backend service. Only supported by backend services for proxy load balancers. Not supported by target pools. Not supported by backend services for pass-through load balancers. Supports all backends that can be health checked; for example, GCE_VM_IP_PORT network endpoint groups and instance group backends. For GCE_VM_IP_PORT network endpoint group backends, the health check uses the port number specified for each endpoint in the network endpoint group. For instance group backends, the health check uses the port number determined by looking up the backend service's named port in the instance group's list of named ports.
        #     Check the PortSpecification enum for the list of possible values.
        # @!attribute [rw] proxy_header
        #   @return [::String]
        #     Specifies the type of proxy header to append before sending data to the backend, either NONE or PROXY_V1. The default is NONE.
        #     Check the ProxyHeader enum for the list of possible values.
        # @!attribute [rw] request_path
        #   @return [::String]
        #     The request path of the HTTP health check request. The default value is /.
        # @!attribute [rw] response
        #   @return [::String]
        #     Creates a content-based HTTP health check. In addition to the required HTTP 200 (OK) status code, you can configure the health check to pass only when the backend sends this specific ASCII response string within the first 1024 bytes of the HTTP response body. For details, see: https://cloud.google.com/load-balancing/docs/health-check-concepts#criteria-protocol-http
        class HTTPHealthCheck
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Specifies how a port is selected for health checking. Can be one of the following values: USE_FIXED_PORT: Specifies a port number explicitly using the port field in the health check. Supported by backend services for pass-through load balancers and backend services for proxy load balancers. Also supported in legacy HTTP health checks for target pools. The health check supports all backends supported by the backend service provided the backend can be health checked. For example, GCE_VM_IP network endpoint groups, GCE_VM_IP_PORT network endpoint groups, and instance group backends. USE_NAMED_PORT: Not supported. USE_SERVING_PORT: Provides an indirect method of specifying the health check port by referring to the backend service. Only supported by backend services for proxy load balancers. Not supported by target pools. Not supported by backend services for pass-through load balancers. Supports all backends that can be health checked; for example, GCE_VM_IP_PORT network endpoint groups and instance group backends. For GCE_VM_IP_PORT network endpoint group backends, the health check uses the port number specified for each endpoint in the network endpoint group. For instance group backends, the health check uses the port number determined by looking up the backend service's named port in the instance group's list of named ports.
          module PortSpecification
            # A value indicating that the enum field is not set.
            UNDEFINED_PORT_SPECIFICATION = 0

            # The port number in the health check's port is used for health checking. Applies to network endpoint group and instance group backends.
            USE_FIXED_PORT = 190_235_748

            # Not supported.
            USE_NAMED_PORT = 349_300_671

            # For network endpoint group backends, the health check uses the port number specified on each endpoint in the network endpoint group. For instance group backends, the health check uses the port number specified for the backend service's named port defined in the instance group's named ports.
            USE_SERVING_PORT = 362_637_516
          end

          # Specifies the type of proxy header to append before sending data to the backend, either NONE or PROXY_V1. The default is NONE.
          module ProxyHeader
            # A value indicating that the enum field is not set.
            UNDEFINED_PROXY_HEADER = 0

            NONE = 2_402_104

            PROXY_V1 = 334_352_940
          end
        end

        # @!attribute [rw] host
        #   @return [::String]
        #     The value of the host header in the HTTPS health check request. If left empty (default value), the host header is set to the destination IP address to which health check packets are sent. The destination IP address depends on the type of load balancer. For details, see: https://cloud.google.com/load-balancing/docs/health-check-concepts#hc-packet-dest
        # @!attribute [rw] port
        #   @return [::Integer]
        #     The TCP port number to which the health check prober sends packets. The default value is 443. Valid values are 1 through 65535.
        # @!attribute [rw] port_name
        #   @return [::String]
        #     Not supported.
        # @!attribute [rw] port_specification
        #   @return [::String]
        #     Specifies how a port is selected for health checking. Can be one of the following values: USE_FIXED_PORT: Specifies a port number explicitly using the port field in the health check. Supported by backend services for pass-through load balancers and backend services for proxy load balancers. Not supported by target pools. The health check supports all backends supported by the backend service provided the backend can be health checked. For example, GCE_VM_IP network endpoint groups, GCE_VM_IP_PORT network endpoint groups, and instance group backends. USE_NAMED_PORT: Not supported. USE_SERVING_PORT: Provides an indirect method of specifying the health check port by referring to the backend service. Only supported by backend services for proxy load balancers. Not supported by target pools. Not supported by backend services for pass-through load balancers. Supports all backends that can be health checked; for example, GCE_VM_IP_PORT network endpoint groups and instance group backends. For GCE_VM_IP_PORT network endpoint group backends, the health check uses the port number specified for each endpoint in the network endpoint group. For instance group backends, the health check uses the port number determined by looking up the backend service's named port in the instance group's list of named ports.
        #     Check the PortSpecification enum for the list of possible values.
        # @!attribute [rw] proxy_header
        #   @return [::String]
        #     Specifies the type of proxy header to append before sending data to the backend, either NONE or PROXY_V1. The default is NONE.
        #     Check the ProxyHeader enum for the list of possible values.
        # @!attribute [rw] request_path
        #   @return [::String]
        #     The request path of the HTTPS health check request. The default value is /.
        # @!attribute [rw] response
        #   @return [::String]
        #     Creates a content-based HTTPS health check. In addition to the required HTTP 200 (OK) status code, you can configure the health check to pass only when the backend sends this specific ASCII response string within the first 1024 bytes of the HTTP response body. For details, see: https://cloud.google.com/load-balancing/docs/health-check-concepts#criteria-protocol-http
        class HTTPSHealthCheck
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Specifies how a port is selected for health checking. Can be one of the following values: USE_FIXED_PORT: Specifies a port number explicitly using the port field in the health check. Supported by backend services for pass-through load balancers and backend services for proxy load balancers. Not supported by target pools. The health check supports all backends supported by the backend service provided the backend can be health checked. For example, GCE_VM_IP network endpoint groups, GCE_VM_IP_PORT network endpoint groups, and instance group backends. USE_NAMED_PORT: Not supported. USE_SERVING_PORT: Provides an indirect method of specifying the health check port by referring to the backend service. Only supported by backend services for proxy load balancers. Not supported by target pools. Not supported by backend services for pass-through load balancers. Supports all backends that can be health checked; for example, GCE_VM_IP_PORT network endpoint groups and instance group backends. For GCE_VM_IP_PORT network endpoint group backends, the health check uses the port number specified for each endpoint in the network endpoint group. For instance group backends, the health check uses the port number determined by looking up the backend service's named port in the instance group's list of named ports.
          module PortSpecification
            # A value indicating that the enum field is not set.
            UNDEFINED_PORT_SPECIFICATION = 0

            # The port number in the health check's port is used for health checking. Applies to network endpoint group and instance group backends.
            USE_FIXED_PORT = 190_235_748

            # Not supported.
            USE_NAMED_PORT = 349_300_671

            # For network endpoint group backends, the health check uses the port number specified on each endpoint in the network endpoint group. For instance group backends, the health check uses the port number specified for the backend service's named port defined in the instance group's named ports.
            USE_SERVING_PORT = 362_637_516
          end

          # Specifies the type of proxy header to append before sending data to the backend, either NONE or PROXY_V1. The default is NONE.
          module ProxyHeader
            # A value indicating that the enum field is not set.
            UNDEFINED_PROXY_HEADER = 0

            NONE = 2_402_104

            PROXY_V1 = 334_352_940
          end
        end

        # Represents a health check resource. Google Compute Engine has two health check resources: * [Regional](/compute/docs/reference/rest/v1/regionHealthChecks) * [Global](/compute/docs/reference/rest/v1/healthChecks) These health check resources can be used for load balancing and for autohealing VMs in a managed instance group (MIG). **Load balancing** The following load balancer can use either regional or global health check: * Internal TCP/UDP load balancer The following load balancers require regional health check: * Internal HTTP(S) load balancer * Backend service-based network load balancer Traffic Director and the following load balancers require global health check: * External HTTP(S) load balancer * TCP proxy load balancer * SSL proxy load balancer The following load balancer require [legacy HTTP health checks](/compute/docs/reference/rest/v1/httpHealthChecks): * Target pool-based network load balancer **Autohealing in MIGs** The health checks that you use for autohealing VMs in a MIG can be either regional or global. For more information, see Set up an application health check and autohealing. For more information, see Health checks overview.
        # @!attribute [rw] check_interval_sec
        #   @return [::Integer]
        #     How often (in seconds) to send a health check. The default value is 5 seconds.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in 3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] grpc_health_check
        #   @return [::Google::Cloud::Compute::V1::GRPCHealthCheck]
        # @!attribute [rw] healthy_threshold
        #   @return [::Integer]
        #     A so-far unhealthy instance will be marked healthy after this many consecutive successes. The default value is 2.
        # @!attribute [rw] http2_health_check
        #   @return [::Google::Cloud::Compute::V1::HTTP2HealthCheck]
        # @!attribute [rw] http_health_check
        #   @return [::Google::Cloud::Compute::V1::HTTPHealthCheck]
        # @!attribute [rw] https_health_check
        #   @return [::Google::Cloud::Compute::V1::HTTPSHealthCheck]
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     Type of the resource.
        # @!attribute [rw] log_config
        #   @return [::Google::Cloud::Compute::V1::HealthCheckLogConfig]
        #     Configure logging on this health check.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. For example, a name that is 1-63 characters long, matches the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?`, and otherwise complies with RFC1035. This regular expression describes a name where the first character is a lowercase letter, and all following characters are a dash, lowercase letter, or digit, except the last character, which isn't a dash.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] Region where the health check resides. Not applicable to global health checks.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] ssl_health_check
        #   @return [::Google::Cloud::Compute::V1::SSLHealthCheck]
        # @!attribute [rw] tcp_health_check
        #   @return [::Google::Cloud::Compute::V1::TCPHealthCheck]
        # @!attribute [rw] timeout_sec
        #   @return [::Integer]
        #     How long (in seconds) to wait before claiming failure. The default value is 5 seconds. It is invalid for timeoutSec to have greater value than checkIntervalSec.
        # @!attribute [rw] type
        #   @return [::String]
        #     Specifies the type of the healthCheck, either TCP, SSL, HTTP, HTTPS, HTTP2 or GRPC. Exactly one of the protocol-specific health check fields must be specified, which must match type field.
        #     Check the Type enum for the list of possible values.
        # @!attribute [rw] unhealthy_threshold
        #   @return [::Integer]
        #     A so-far healthy instance will be marked unhealthy after this many consecutive failures. The default value is 2.
        class HealthCheck
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Specifies the type of the healthCheck, either TCP, SSL, HTTP, HTTPS, HTTP2 or GRPC. Exactly one of the protocol-specific health check fields must be specified, which must match type field.
          module Type
            # A value indicating that the enum field is not set.
            UNDEFINED_TYPE = 0

            GRPC = 2_196_510

            HTTP = 2_228_360

            HTTP2 = 69_079_210

            HTTPS = 69_079_243

            INVALID = 530_283_991

            SSL = 82_412

            TCP = 82_881
          end
        end

        # Contains a list of HealthCheck resources.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::HealthCheck>]
        #     A list of HealthCheck resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     Type of resource.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class HealthCheckList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Configuration of logging on a health check. If logging is enabled, logs will be exported to Stackdriver.
        # @!attribute [rw] enable
        #   @return [::Boolean]
        #     Indicates whether or not to export logs. This is false by default, which means no health check logging will be done.
        class HealthCheckLogConfig
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A full or valid partial URL to a health check. For example, the following are valid URLs: - https://www.googleapis.com/compute/beta/projects/project-id/global/httpHealthChecks/health-check - projects/project-id/global/httpHealthChecks/health-check - global/httpHealthChecks/health-check
        # @!attribute [rw] health_check
        #   @return [::String]
        class HealthCheckReference
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a Health-Check as a Service resource.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] fingerprint
        #   @return [::String]
        #     Fingerprint of this resource. A hash of the contents stored in this object. This field is used in optimistic locking. This field will be ignored when inserting a HealthCheckService. An up-to-date fingerprint must be provided in order to patch/update the HealthCheckService; Otherwise, the request will fail with error 412 conditionNotMet. To see the latest fingerprint, make a get() request to retrieve the HealthCheckService.
        # @!attribute [rw] health_checks
        #   @return [::Array<::String>]
        #     A list of URLs to the HealthCheck resources. Must have at least one HealthCheck, and not more than 10 for regional HealthCheckService, and not more than 1 for global HealthCheckService. HealthCheck resources must have portSpecification=USE_SERVING_PORT or portSpecification=USE_FIXED_PORT. For regional HealthCheckService, the HealthCheck must be regional and in the same region. For global HealthCheckService, HealthCheck must be global. Mix of regional and global HealthChecks is not supported. Multiple regional HealthChecks must belong to the same region. Regional HealthChecks must belong to the same region as zones of NetworkEndpointGroups. For global HealthCheckService using global INTERNET_IP_PORT NetworkEndpointGroups, the global HealthChecks must specify sourceRegions, and HealthChecks that specify sourceRegions can only be used with global INTERNET_IP_PORT NetworkEndpointGroups.
        # @!attribute [rw] health_status_aggregation_policy
        #   @return [::String]
        #     Optional. Policy for how the results from multiple health checks for the same endpoint are aggregated. Defaults to NO_AGGREGATION if unspecified. - NO_AGGREGATION. An EndpointHealth message is returned for each pair in the health check service. - AND. If any health check of an endpoint reports UNHEALTHY, then UNHEALTHY is the HealthState of the endpoint. If all health checks report HEALTHY, the HealthState of the endpoint is HEALTHY. . This is only allowed with regional HealthCheckService.
        #     Check the HealthStatusAggregationPolicy enum for the list of possible values.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output only] Type of the resource. Always compute#healthCheckServicefor health check services.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] network_endpoint_groups
        #   @return [::Array<::String>]
        #     A list of URLs to the NetworkEndpointGroup resources. Must not have more than 100. For regional HealthCheckService, NEGs must be in zones in the region of the HealthCheckService. For global HealthCheckServices, the NetworkEndpointGroups must be global INTERNET_IP_PORT.
        # @!attribute [rw] notification_endpoints
        #   @return [::Array<::String>]
        #     A list of URLs to the NotificationEndpoint resources. Must not have more than 10. A list of endpoints for receiving notifications of change in health status. For regional HealthCheckService, NotificationEndpoint must be regional and in the same region. For global HealthCheckService, NotificationEndpoint must be global.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] URL of the region where the health check service resides. This field is not applicable to global health check services. You must specify this field as part of the HTTP request URL. It is not settable as a field in the request body.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        class HealthCheckService
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Optional. Policy for how the results from multiple health checks for the same endpoint are aggregated. Defaults to NO_AGGREGATION if unspecified. - NO_AGGREGATION. An EndpointHealth message is returned for each pair in the health check service. - AND. If any health check of an endpoint reports UNHEALTHY, then UNHEALTHY is the HealthState of the endpoint. If all health checks report HEALTHY, the HealthState of the endpoint is HEALTHY. . This is only allowed with regional HealthCheckService.
          module HealthStatusAggregationPolicy
            # A value indicating that the enum field is not set.
            UNDEFINED_HEALTH_STATUS_AGGREGATION_POLICY = 0

            # If any backend's health check reports UNHEALTHY, then UNHEALTHY is the HealthState of the entire health check service. If all backend's are healthy, the HealthState of the health check service is HEALTHY.
            AND = 64_951

            # An EndpointHealth message is returned for each backend in the health check service.
            NO_AGGREGATION = 426_445_124
          end
        end

        # A full or valid partial URL to a health check service. For example, the following are valid URLs: - https://www.googleapis.com/compute/beta/projects/project-id/regions/us-west1/healthCheckServices/health-check-service - projects/project-id/regions/us-west1/healthCheckServices/health-check-service - regions/us-west1/healthCheckServices/health-check-service
        # @!attribute [rw] health_check_service
        #   @return [::String]
        class HealthCheckServiceReference
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::HealthCheckService>]
        #     A list of HealthCheckService resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#healthCheckServicesList for lists of HealthCheckServices.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class HealthCheckServicesList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::HealthChecksScopedList}]
        #     A list of HealthChecksScopedList resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     Type of resource.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class HealthChecksAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::HealthChecksScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # @!attribute [rw] health_checks
        #   @return [::Array<::Google::Cloud::Compute::V1::HealthCheck>]
        #     A list of HealthChecks contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     Informational warning which replaces the list of backend services when the list is empty.
        class HealthChecksScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] annotations
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     Metadata defined as annotations for network endpoint.
        # @!attribute [rw] forwarding_rule
        #   @return [::String]
        #     URL of the forwarding rule associated with the health status of the instance.
        # @!attribute [rw] forwarding_rule_ip
        #   @return [::String]
        #     A forwarding rule IP address assigned to this instance.
        # @!attribute [rw] health_state
        #   @return [::String]
        #     Health state of the IPv4 address of the instance.
        #     Check the HealthState enum for the list of possible values.
        # @!attribute [rw] instance
        #   @return [::String]
        #     URL of the instance resource.
        # @!attribute [rw] ip_address
        #   @return [::String]
        #     For target pool based Network Load Balancing, it indicates the forwarding rule's IP address assigned to this instance. For other types of load balancing, the field indicates VM internal ip.
        # @!attribute [rw] port
        #   @return [::Integer]
        #     The named port of the instance group, not necessarily the port that is health-checked.
        # @!attribute [rw] weight
        #   @return [::String]
        # @!attribute [rw] weight_error
        #   @return [::String]
        #     Check the WeightError enum for the list of possible values.
        class HealthStatus
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class AnnotationsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # Health state of the IPv4 address of the instance.
          module HealthState
            # A value indicating that the enum field is not set.
            UNDEFINED_HEALTH_STATE = 0

            HEALTHY = 439_801_213

            UNHEALTHY = 462_118_084
          end

          module WeightError
            # A value indicating that the enum field is not set.
            UNDEFINED_WEIGHT_ERROR = 0

            # The response to a Health Check probe had the HTTP response header field X-Load-Balancing-Endpoint-Weight, but its content was invalid (i.e., not a non-negative single-precision floating-point number in decimal string representation).
            INVALID_WEIGHT = 383_698_400

            # The response to a Health Check probe did not have the HTTP response header field X-Load-Balancing-Endpoint-Weight.
            MISSING_WEIGHT = 384_027_537

            # This is the value when the accompanied health status is either TIMEOUT (i.e.,the Health Check probe was not able to get a response in time) or UNKNOWN. For the latter, it should be typically because there has not been sufficient time to parse and report the weight for a new backend (which is with 0.0.0.0 ip address). However, it can be also due to an outage case for which the health status is explicitly reset to UNKNOWN.
            UNAVAILABLE_WEIGHT = 439_464_295

            # This is the default value when WeightReportMode is DISABLE, and is also the initial value when WeightReportMode has just updated to ENABLE or DRY_RUN and there has not been sufficient time to parse and report the backend weight.
            WEIGHT_NONE = 502_428_831
          end
        end

        # @!attribute [rw] backend_service
        #   @return [::Google::Cloud::Compute::V1::BackendServiceReference]
        #     URL of the backend service associated with the health state of the network endpoint.
        # @!attribute [rw] forwarding_rule
        #   @return [::Google::Cloud::Compute::V1::ForwardingRuleReference]
        #     URL of the forwarding rule associated with the health state of the network endpoint.
        # @!attribute [rw] health_check
        #   @return [::Google::Cloud::Compute::V1::HealthCheckReference]
        #     URL of the health check associated with the health state of the network endpoint.
        # @!attribute [rw] health_check_service
        #   @return [::Google::Cloud::Compute::V1::HealthCheckServiceReference]
        #     URL of the health check service associated with the health state of the network endpoint.
        # @!attribute [rw] health_state
        #   @return [::String]
        #     Health state of the network endpoint determined based on the health checks configured.
        #     Check the HealthState enum for the list of possible values.
        class HealthStatusForNetworkEndpoint
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Health state of the network endpoint determined based on the health checks configured.
          module HealthState
            # A value indicating that the enum field is not set.
            UNDEFINED_HEALTH_STATE = 0

            # Endpoint is being drained.
            DRAINING = 480_455_402

            # Endpoint is healthy.
            HEALTHY = 439_801_213

            # Endpoint is unhealthy.
            UNHEALTHY = 462_118_084

            # Health status of the endpoint is unknown.
            UNKNOWN = 433_141_802
          end
        end

        # Provides links to documentation or for performing an out of band action. For example, if a quota check failed with an error indicating the calling project hasn't enabled the accessed service, this can contain a URL pointing directly to the right place in the developer console to flip the bit.
        # @!attribute [rw] links
        #   @return [::Array<::Google::Cloud::Compute::V1::HelpLink>]
        #     URL(s) pointing to additional information on handling the current error.
        class Help
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Describes a URL link.
        # @!attribute [rw] description
        #   @return [::String]
        #     Describes what the link offers.
        # @!attribute [rw] url
        #   @return [::String]
        #     The URL of the link.
        class HelpLink
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # UrlMaps A host-matching rule for a URL. If matched, will use the named PathMatcher to select the BackendService.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] hosts
        #   @return [::Array<::String>]
        #     The list of host patterns to match. They must be valid hostnames with optional port numbers in the format host:port. * matches any string of ([a-z0-9-.]*). In that case, * must be the first character, and if followed by anything, the immediate following character must be either - or .. * based matching is not supported when the URL map is bound to a target gRPC proxy that has the validateForProxyless field set to true.
        # @!attribute [rw] path_matcher
        #   @return [::String]
        #     The name of the PathMatcher to use to match the path portion of the URL if the hostRule matches the URL's host portion.
        class HostRule
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Specification for how requests are aborted as part of fault injection.
        # @!attribute [rw] http_status
        #   @return [::Integer]
        #     The HTTP status code used to abort the request. The value must be from 200 to 599 inclusive. For gRPC protocol, the gRPC status code is mapped to HTTP status code according to this mapping table. HTTP status 200 is mapped to gRPC status UNKNOWN. Injecting an OK status is currently not supported by Traffic Director.
        # @!attribute [rw] percentage
        #   @return [::Float]
        #     The percentage of traffic for connections, operations, or requests that is aborted as part of fault injection. The value must be from 0.0 to 100.0 inclusive.
        class HttpFaultAbort
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Specifies the delay introduced by the load balancer before forwarding the request to the backend service as part of fault injection.
        # @!attribute [rw] fixed_delay
        #   @return [::Google::Cloud::Compute::V1::Duration]
        #     Specifies the value of the fixed delay interval.
        # @!attribute [rw] percentage
        #   @return [::Float]
        #     The percentage of traffic for connections, operations, or requests for which a delay is introduced as part of fault injection. The value must be from 0.0 to 100.0 inclusive.
        class HttpFaultDelay
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # The specification for fault injection introduced into traffic to test the resiliency of clients to backend service failure. As part of fault injection, when clients send requests to a backend service, delays can be introduced by the load balancer on a percentage of requests before sending those request to the backend service. Similarly requests from clients can be aborted by the load balancer for a percentage of requests.
        # @!attribute [rw] abort
        #   @return [::Google::Cloud::Compute::V1::HttpFaultAbort]
        #     The specification for how client requests are aborted as part of fault injection.
        # @!attribute [rw] delay
        #   @return [::Google::Cloud::Compute::V1::HttpFaultDelay]
        #     The specification for how client requests are delayed as part of fault injection, before being sent to a backend service.
        class HttpFaultInjection
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # The request and response header transformations that take effect before the request is passed along to the selected backendService.
        # @!attribute [rw] request_headers_to_add
        #   @return [::Array<::Google::Cloud::Compute::V1::HttpHeaderOption>]
        #     Headers to add to a matching request before forwarding the request to the backendService.
        # @!attribute [rw] request_headers_to_remove
        #   @return [::Array<::String>]
        #     A list of header names for headers that need to be removed from the request before forwarding the request to the backendService.
        # @!attribute [rw] response_headers_to_add
        #   @return [::Array<::Google::Cloud::Compute::V1::HttpHeaderOption>]
        #     Headers to add the response before sending the response back to the client.
        # @!attribute [rw] response_headers_to_remove
        #   @return [::Array<::String>]
        #     A list of header names for headers that need to be removed from the response before sending the response back to the client.
        class HttpHeaderAction
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # matchRule criteria for request header matches.
        # @!attribute [rw] exact_match
        #   @return [::String]
        #     The value should exactly match contents of exactMatch. Only one of exactMatch, prefixMatch, suffixMatch, regexMatch, presentMatch or rangeMatch must be set.
        # @!attribute [rw] header_name
        #   @return [::String]
        #     The name of the HTTP header to match. For matching against the HTTP request's authority, use a headerMatch with the header name ":authority". For matching a request's method, use the headerName ":method". When the URL map is bound to a target gRPC proxy that has the validateForProxyless field set to true, only non-binary user-specified custom metadata and the `content-type` header are supported. The following transport-level headers cannot be used in header matching rules: `:authority`, `:method`, `:path`, `:scheme`, `user-agent`, `accept-encoding`, `content-encoding`, `grpc-accept-encoding`, `grpc-encoding`, `grpc-previous-rpc-attempts`, `grpc-tags-bin`, `grpc-timeout` and `grpc-trace-bin`.
        # @!attribute [rw] invert_match
        #   @return [::Boolean]
        #     If set to false, the headerMatch is considered a match if the preceding match criteria are met. If set to true, the headerMatch is considered a match if the preceding match criteria are NOT met. The default setting is false.
        # @!attribute [rw] prefix_match
        #   @return [::String]
        #     The value of the header must start with the contents of prefixMatch. Only one of exactMatch, prefixMatch, suffixMatch, regexMatch, presentMatch or rangeMatch must be set.
        # @!attribute [rw] present_match
        #   @return [::Boolean]
        #     A header with the contents of headerName must exist. The match takes place whether or not the request's header has a value. Only one of exactMatch, prefixMatch, suffixMatch, regexMatch, presentMatch or rangeMatch must be set.
        # @!attribute [rw] range_match
        #   @return [::Google::Cloud::Compute::V1::Int64RangeMatch]
        #     The header value must be an integer and its value must be in the range specified in rangeMatch. If the header does not contain an integer, number or is empty, the match fails. For example for a range [-5, 0] - -3 will match. - 0 will not match. - 0.25 will not match. - -3someString will not match. Only one of exactMatch, prefixMatch, suffixMatch, regexMatch, presentMatch or rangeMatch must be set. rangeMatch is not supported for load balancers that have loadBalancingScheme set to EXTERNAL.
        # @!attribute [rw] regex_match
        #   @return [::String]
        #     The value of the header must match the regular expression specified in regexMatch. For more information about regular expression syntax, see Syntax. For matching against a port specified in the HTTP request, use a headerMatch with headerName set to PORT and a regular expression that satisfies the RFC2616 Host header's port specifier. Only one of exactMatch, prefixMatch, suffixMatch, regexMatch, presentMatch or rangeMatch must be set. Regular expressions can only be used when the loadBalancingScheme is set to INTERNAL_SELF_MANAGED.
        # @!attribute [rw] suffix_match
        #   @return [::String]
        #     The value of the header must end with the contents of suffixMatch. Only one of exactMatch, prefixMatch, suffixMatch, regexMatch, presentMatch or rangeMatch must be set.
        class HttpHeaderMatch
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Specification determining how headers are added to requests or responses.
        # @!attribute [rw] header_name
        #   @return [::String]
        #     The name of the header.
        # @!attribute [rw] header_value
        #   @return [::String]
        #     The value of the header to add.
        # @!attribute [rw] replace
        #   @return [::Boolean]
        #     If false, headerValue is appended to any values that already exist for the header. If true, headerValue is set for the header, discarding any values that were set for that header. The default value is false.
        class HttpHeaderOption
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # HttpRouteRuleMatch criteria for a request's query parameter.
        # @!attribute [rw] exact_match
        #   @return [::String]
        #     The queryParameterMatch matches if the value of the parameter exactly matches the contents of exactMatch. Only one of presentMatch, exactMatch, or regexMatch must be set.
        # @!attribute [rw] name
        #   @return [::String]
        #     The name of the query parameter to match. The query parameter must exist in the request, in the absence of which the request match fails.
        # @!attribute [rw] present_match
        #   @return [::Boolean]
        #     Specifies that the queryParameterMatch matches if the request contains the query parameter, irrespective of whether the parameter has a value or not. Only one of presentMatch, exactMatch, or regexMatch must be set.
        # @!attribute [rw] regex_match
        #   @return [::String]
        #     The queryParameterMatch matches if the value of the parameter matches the regular expression specified by regexMatch. For more information about regular expression syntax, see Syntax. Only one of presentMatch, exactMatch, or regexMatch must be set. Regular expressions can only be used when the loadBalancingScheme is set to INTERNAL_SELF_MANAGED.
        class HttpQueryParameterMatch
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Specifies settings for an HTTP redirect.
        # @!attribute [rw] host_redirect
        #   @return [::String]
        #     The host that is used in the redirect response instead of the one that was supplied in the request. The value must be from 1 to 255 characters.
        # @!attribute [rw] https_redirect
        #   @return [::Boolean]
        #     If set to true, the URL scheme in the redirected request is set to HTTPS. If set to false, the URL scheme of the redirected request remains the same as that of the request. This must only be set for URL maps used in TargetHttpProxys. Setting this true for TargetHttpsProxy is not permitted. The default is set to false.
        # @!attribute [rw] path_redirect
        #   @return [::String]
        #     The path that is used in the redirect response instead of the one that was supplied in the request. pathRedirect cannot be supplied together with prefixRedirect. Supply one alone or neither. If neither is supplied, the path of the original request is used for the redirect. The value must be from 1 to 1024 characters.
        # @!attribute [rw] prefix_redirect
        #   @return [::String]
        #     The prefix that replaces the prefixMatch specified in the HttpRouteRuleMatch, retaining the remaining portion of the URL before redirecting the request. prefixRedirect cannot be supplied together with pathRedirect. Supply one alone or neither. If neither is supplied, the path of the original request is used for the redirect. The value must be from 1 to 1024 characters.
        # @!attribute [rw] redirect_response_code
        #   @return [::String]
        #     The HTTP Status code to use for this RedirectAction. Supported values are: - MOVED_PERMANENTLY_DEFAULT, which is the default value and corresponds to 301. - FOUND, which corresponds to 302. - SEE_OTHER which corresponds to 303. - TEMPORARY_REDIRECT, which corresponds to 307. In this case, the request method is retained. - PERMANENT_REDIRECT, which corresponds to 308. In this case, the request method is retained.
        #     Check the RedirectResponseCode enum for the list of possible values.
        # @!attribute [rw] strip_query
        #   @return [::Boolean]
        #     If set to true, any accompanying query portion of the original URL is removed before redirecting the request. If set to false, the query portion of the original URL is retained. The default is set to false.
        class HttpRedirectAction
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The HTTP Status code to use for this RedirectAction. Supported values are: - MOVED_PERMANENTLY_DEFAULT, which is the default value and corresponds to 301. - FOUND, which corresponds to 302. - SEE_OTHER which corresponds to 303. - TEMPORARY_REDIRECT, which corresponds to 307. In this case, the request method is retained. - PERMANENT_REDIRECT, which corresponds to 308. In this case, the request method is retained.
          module RedirectResponseCode
            # A value indicating that the enum field is not set.
            UNDEFINED_REDIRECT_RESPONSE_CODE = 0

            # Http Status Code 302 - Found.
            FOUND = 67_084_130

            # Http Status Code 301 - Moved Permanently.
            MOVED_PERMANENTLY_DEFAULT = 386_698_449

            # Http Status Code 308 - Permanent Redirect maintaining HTTP method.
            PERMANENT_REDIRECT = 382_006_381

            # Http Status Code 303 - See Other.
            SEE_OTHER = 445_380_580

            # Http Status Code 307 - Temporary Redirect maintaining HTTP method.
            TEMPORARY_REDIRECT = 489_550_378
          end
        end

        # The retry policy associates with HttpRouteRule
        # @!attribute [rw] num_retries
        #   @return [::Integer]
        #     Specifies the allowed number retries. This number must be > 0. If not specified, defaults to 1.
        # @!attribute [rw] per_try_timeout
        #   @return [::Google::Cloud::Compute::V1::Duration]
        #     Specifies a non-zero timeout per retry attempt. If not specified, will use the timeout set in the HttpRouteAction field. If timeout in the HttpRouteAction field is not set, this field uses the largest timeout among all backend services associated with the route. Not supported when the URL map is bound to a target gRPC proxy that has the validateForProxyless field set to true.
        # @!attribute [rw] retry_conditions
        #   @return [::Array<::String>]
        #     Specifies one or more conditions when this retry policy applies. Valid values are: - 5xx: retry is attempted if the instance or endpoint responds with any 5xx response code, or if the instance or endpoint does not respond at all. For example, disconnects, reset, read timeout, connection failure, and refused streams. - gateway-error: Similar to 5xx, but only applies to response codes 502, 503 or 504. - connect-failure: a retry is attempted on failures connecting to the instance or endpoint. For example, connection timeouts. - retriable-4xx: a retry is attempted if the instance or endpoint responds with a 4xx response code. The only error that you can retry is error code 409. - refused-stream: a retry is attempted if the instance or endpoint resets the stream with a REFUSED_STREAM error code. This reset type indicates that it is safe to retry. - cancelled: a retry is attempted if the gRPC status code in the response header is set to cancelled. - deadline-exceeded: a retry is attempted if the gRPC status code in the response header is set to deadline-exceeded. - internal: a retry is attempted if the gRPC status code in the response header is set to internal. - resource-exhausted: a retry is attempted if the gRPC status code in the response header is set to resource-exhausted. - unavailable: a retry is attempted if the gRPC status code in the response header is set to unavailable. Only the following codes are supported when the URL map is bound to target gRPC proxy that has validateForProxyless field set to true. - cancelled - deadline-exceeded - internal - resource-exhausted - unavailable
        class HttpRetryPolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] cors_policy
        #   @return [::Google::Cloud::Compute::V1::CorsPolicy]
        #     The specification for allowing client-side cross-origin requests. For more information about the W3C recommendation for cross-origin resource sharing (CORS), see Fetch API Living Standard. Not supported when the URL map is bound to a target gRPC proxy.
        # @!attribute [rw] fault_injection_policy
        #   @return [::Google::Cloud::Compute::V1::HttpFaultInjection]
        #     The specification for fault injection introduced into traffic to test the resiliency of clients to backend service failure. As part of fault injection, when clients send requests to a backend service, delays can be introduced by a load balancer on a percentage of requests before sending those requests to the backend service. Similarly requests from clients can be aborted by the load balancer for a percentage of requests. timeout and retry_policy is ignored by clients that are configured with a fault_injection_policy if: 1. The traffic is generated by fault injection AND 2. The fault injection is not a delay fault injection. Fault injection is not supported with the classic Application Load Balancer . To see which load balancers support fault injection, see Load balancing: Routing and traffic management features.
        # @!attribute [rw] max_stream_duration
        #   @return [::Google::Cloud::Compute::V1::Duration]
        #     Specifies the maximum duration (timeout) for streams on the selected route. Unlike the timeout field where the timeout duration starts from the time the request has been fully processed (known as *end-of-stream*), the duration in this field is computed from the beginning of the stream until the response has been processed, including all retries. A stream that does not complete in this duration is closed. If not specified, this field uses the maximum maxStreamDuration value among all backend services associated with the route. This field is only allowed if the Url map is used with backend services with loadBalancingScheme set to INTERNAL_SELF_MANAGED.
        # @!attribute [rw] request_mirror_policy
        #   @return [::Google::Cloud::Compute::V1::RequestMirrorPolicy]
        #     Specifies the policy on how requests intended for the route's backends are shadowed to a separate mirrored backend service. The load balancer does not wait for responses from the shadow service. Before sending traffic to the shadow service, the host / authority header is suffixed with -shadow. Not supported when the URL map is bound to a target gRPC proxy that has the validateForProxyless field set to true.
        # @!attribute [rw] retry_policy
        #   @return [::Google::Cloud::Compute::V1::HttpRetryPolicy]
        #     Specifies the retry policy associated with this route.
        # @!attribute [rw] timeout
        #   @return [::Google::Cloud::Compute::V1::Duration]
        #     Specifies the timeout for the selected route. Timeout is computed from the time the request has been fully processed (known as *end-of-stream*) up until the response has been processed. Timeout includes all retries. If not specified, this field uses the largest timeout among all backend services associated with the route. Not supported when the URL map is bound to a target gRPC proxy that has validateForProxyless field set to true.
        # @!attribute [rw] url_rewrite
        #   @return [::Google::Cloud::Compute::V1::UrlRewrite]
        #     The spec to modify the URL of the request, before forwarding the request to the matched service. urlRewrite is the only action supported in UrlMaps for classic Application Load Balancers. Not supported when the URL map is bound to a target gRPC proxy that has the validateForProxyless field set to true.
        # @!attribute [rw] weighted_backend_services
        #   @return [::Array<::Google::Cloud::Compute::V1::WeightedBackendService>]
        #     A list of weighted backend services to send traffic to when a route match occurs. The weights determine the fraction of traffic that flows to their corresponding backend service. If all traffic needs to go to a single backend service, there must be one weightedBackendService with weight set to a non-zero number. After a backend service is identified and before forwarding the request to the backend service, advanced routing actions such as URL rewrites and header transformations are applied depending on additional settings specified in this HttpRouteAction.
        class HttpRouteAction
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # The HttpRouteRule setting specifies how to match an HTTP request and the corresponding routing action that load balancing proxies perform.
        # @!attribute [rw] description
        #   @return [::String]
        #     The short description conveying the intent of this routeRule. The description can have a maximum length of 1024 characters.
        # @!attribute [rw] header_action
        #   @return [::Google::Cloud::Compute::V1::HttpHeaderAction]
        #     Specifies changes to request and response headers that need to take effect for the selected backendService. The headerAction value specified here is applied before the matching pathMatchers[].headerAction and after pathMatchers[].routeRules[].routeAction.weightedBackendService.backendServiceWeightAction[].headerAction HeaderAction is not supported for load balancers that have their loadBalancingScheme set to EXTERNAL. Not supported when the URL map is bound to a target gRPC proxy that has validateForProxyless field set to true.
        # @!attribute [rw] match_rules
        #   @return [::Array<::Google::Cloud::Compute::V1::HttpRouteRuleMatch>]
        #     The list of criteria for matching attributes of a request to this routeRule. This list has OR semantics: the request matches this routeRule when any of the matchRules are satisfied. However predicates within a given matchRule have AND semantics. All predicates within a matchRule must match for the request to match the rule.
        # @!attribute [rw] priority
        #   @return [::Integer]
        #     For routeRules within a given pathMatcher, priority determines the order in which a load balancer interprets routeRules. RouteRules are evaluated in order of priority, from the lowest to highest number. The priority of a rule decreases as its number increases (1, 2, 3, N+1). The first rule that matches the request is applied. You cannot configure two or more routeRules with the same priority. Priority for each rule must be set to a number from 0 to 2147483647 inclusive. Priority numbers can have gaps, which enable you to add or remove rules in the future without affecting the rest of the rules. For example, 1, 2, 3, 4, 5, 9, 12, 16 is a valid series of priority numbers to which you could add rules numbered from 6 to 8, 10 to 11, and 13 to 15 in the future without any impact on existing rules.
        # @!attribute [rw] route_action
        #   @return [::Google::Cloud::Compute::V1::HttpRouteAction]
        #     In response to a matching matchRule, the load balancer performs advanced routing actions, such as URL rewrites and header transformations, before forwarding the request to the selected backend. If routeAction specifies any weightedBackendServices, service must not be set. Conversely if service is set, routeAction cannot contain any weightedBackendServices. Only one of urlRedirect, service or routeAction.weightedBackendService must be set. URL maps for classic Application Load Balancers only support the urlRewrite action within a route rule's routeAction.
        # @!attribute [rw] service
        #   @return [::String]
        #     The full or partial URL of the backend service resource to which traffic is directed if this rule is matched. If routeAction is also specified, advanced routing actions, such as URL rewrites, take effect before sending the request to the backend. However, if service is specified, routeAction cannot contain any weightedBackendServices. Conversely, if routeAction specifies any weightedBackendServices, service must not be specified. Only one of urlRedirect, service or routeAction.weightedBackendService must be set.
        # @!attribute [rw] url_redirect
        #   @return [::Google::Cloud::Compute::V1::HttpRedirectAction]
        #     When this rule is matched, the request is redirected to a URL specified by urlRedirect. If urlRedirect is specified, service or routeAction must not be set. Not supported when the URL map is bound to a target gRPC proxy.
        class HttpRouteRule
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # HttpRouteRuleMatch specifies a set of criteria for matching requests to an HttpRouteRule. All specified criteria must be satisfied for a match to occur.
        # @!attribute [rw] full_path_match
        #   @return [::String]
        #     For satisfying the matchRule condition, the path of the request must exactly match the value specified in fullPathMatch after removing any query parameters and anchor that may be part of the original URL. fullPathMatch must be from 1 to 1024 characters. Only one of prefixMatch, fullPathMatch or regexMatch must be specified.
        # @!attribute [rw] header_matches
        #   @return [::Array<::Google::Cloud::Compute::V1::HttpHeaderMatch>]
        #     Specifies a list of header match criteria, all of which must match corresponding headers in the request.
        # @!attribute [rw] ignore_case
        #   @return [::Boolean]
        #     Specifies that prefixMatch and fullPathMatch matches are case sensitive. The default value is false. ignoreCase must not be used with regexMatch. Not supported when the URL map is bound to a target gRPC proxy.
        # @!attribute [rw] metadata_filters
        #   @return [::Array<::Google::Cloud::Compute::V1::MetadataFilter>]
        #     Opaque filter criteria used by the load balancer to restrict routing configuration to a limited set of xDS compliant clients. In their xDS requests to the load balancer, xDS clients present node metadata. When there is a match, the relevant routing configuration is made available to those proxies. For each metadataFilter in this list, if its filterMatchCriteria is set to MATCH_ANY, at least one of the filterLabels must match the corresponding label provided in the metadata. If its filterMatchCriteria is set to MATCH_ALL, then all of its filterLabels must match with corresponding labels provided in the metadata. If multiple metadata filters are specified, all of them need to be satisfied in order to be considered a match. metadataFilters specified here is applied after those specified in ForwardingRule that refers to the UrlMap this HttpRouteRuleMatch belongs to. metadataFilters only applies to load balancers that have loadBalancingScheme set to INTERNAL_SELF_MANAGED. Not supported when the URL map is bound to a target gRPC proxy that has validateForProxyless field set to true.
        # @!attribute [rw] path_template_match
        #   @return [::String]
        #     If specified, the route is a pattern match expression that must match the :path header once the query string is removed. A pattern match allows you to match - The value must be between 1 and 1024 characters - The pattern must start with a leading slash ("/") - There may be no more than 5 operators in pattern Precisely one of prefix_match, full_path_match, regex_match or path_template_match must be set.
        # @!attribute [rw] prefix_match
        #   @return [::String]
        #     For satisfying the matchRule condition, the request's path must begin with the specified prefixMatch. prefixMatch must begin with a /. The value must be from 1 to 1024 characters. Only one of prefixMatch, fullPathMatch or regexMatch must be specified.
        # @!attribute [rw] query_parameter_matches
        #   @return [::Array<::Google::Cloud::Compute::V1::HttpQueryParameterMatch>]
        #     Specifies a list of query parameter match criteria, all of which must match corresponding query parameters in the request. Not supported when the URL map is bound to a target gRPC proxy.
        # @!attribute [rw] regex_match
        #   @return [::String]
        #     For satisfying the matchRule condition, the path of the request must satisfy the regular expression specified in regexMatch after removing any query parameters and anchor supplied with the original URL. For more information about regular expression syntax, see Syntax. Only one of prefixMatch, fullPathMatch or regexMatch must be specified. Regular expressions can only be used when the loadBalancingScheme is set to INTERNAL_SELF_MANAGED.
        class HttpRouteRuleMatch
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents an Image resource. You can use images to create boot disks for your VM instances. For more information, read Images.
        # @!attribute [rw] architecture
        #   @return [::String]
        #     The architecture of the image. Valid values are ARM64 or X86_64.
        #     Check the Architecture enum for the list of possible values.
        # @!attribute [rw] archive_size_bytes
        #   @return [::Integer]
        #     Size of the image tar.gz archive stored in Google Cloud Storage (in bytes).
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] deprecated
        #   @return [::Google::Cloud::Compute::V1::DeprecationStatus]
        #     The deprecation status associated with this image.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] disk_size_gb
        #   @return [::Integer]
        #     Size of the image when restored onto a persistent disk (in GB).
        # @!attribute [rw] family
        #   @return [::String]
        #     The name of the image family to which this image belongs. The image family name can be from a publicly managed image family provided by Compute Engine, or from a custom image family you create. For example, centos-stream-9 is a publicly available image family. For more information, see Image family best practices. When creating disks, you can specify an image family instead of a specific image name. The image family always returns its latest image that is not deprecated. The name of the image family must comply with RFC1035.
        # @!attribute [rw] guest_os_features
        #   @return [::Array<::Google::Cloud::Compute::V1::GuestOsFeature>]
        #     A list of features to enable on the guest operating system. Applicable only for bootable images. To see a list of available options, see the guestOSfeatures[].type parameter.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] image_encryption_key
        #   @return [::Google::Cloud::Compute::V1::CustomerEncryptionKey]
        #     Encrypts the image using a customer-supplied encryption key. After you encrypt an image with a customer-supplied key, you must provide the same key if you use the image later (e.g. to create a disk from the image). Customer-supplied encryption keys do not protect access to metadata of the disk. If you do not provide an encryption key when creating the image, then the disk will be encrypted using an automatically generated key and you do not need to provide a key to use the image later.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#image for images.
        # @!attribute [rw] label_fingerprint
        #   @return [::String]
        #     A fingerprint for the labels being applied to this image, which is essentially a hash of the labels used for optimistic locking. The fingerprint is initially generated by Compute Engine and changes after every request to modify or update labels. You must always provide an up-to-date fingerprint hash in order to update or change labels, otherwise the request will fail with error 412 conditionNotMet. To see the latest fingerprint, make a get() request to retrieve an image.
        # @!attribute [rw] labels
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     Labels to apply to this image. These can be later modified by the setLabels method.
        # @!attribute [rw] license_codes
        #   @return [::Array<::Integer>]
        #     Integer license codes indicating which licenses are attached to this image.
        # @!attribute [rw] licenses
        #   @return [::Array<::String>]
        #     Any applicable license URI.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource; provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] raw_disk
        #   @return [::Google::Cloud::Compute::V1::RawDisk]
        #     The parameters of the raw disk image.
        # @!attribute [rw] satisfies_pzs
        #   @return [::Boolean]
        #     [Output Only] Reserved for future use.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] shielded_instance_initial_state
        #   @return [::Google::Cloud::Compute::V1::InitialStateConfig]
        #     Set the secure boot keys of shielded instance.
        # @!attribute [rw] source_disk
        #   @return [::String]
        #     URL of the source disk used to create this image. For example, the following are valid values: - https://www.googleapis.com/compute/v1/projects/project/zones/zone /disks/disk - projects/project/zones/zone/disks/disk - zones/zone/disks/disk In order to create an image, you must provide the full or partial URL of one of the following: - The rawDisk.source URL - The sourceDisk URL - The sourceImage URL - The sourceSnapshot URL
        # @!attribute [rw] source_disk_encryption_key
        #   @return [::Google::Cloud::Compute::V1::CustomerEncryptionKey]
        #     The customer-supplied encryption key of the source disk. Required if the source disk is protected by a customer-supplied encryption key.
        # @!attribute [rw] source_disk_id
        #   @return [::String]
        #     [Output Only] The ID value of the disk used to create this image. This value may be used to determine whether the image was taken from the current or a previous instance of a given disk name.
        # @!attribute [rw] source_image
        #   @return [::String]
        #     URL of the source image used to create this image. The following are valid formats for the URL: - https://www.googleapis.com/compute/v1/projects/project_id/global/ images/image_name - projects/project_id/global/images/image_name In order to create an image, you must provide the full or partial URL of one of the following: - The rawDisk.source URL - The sourceDisk URL - The sourceImage URL - The sourceSnapshot URL
        # @!attribute [rw] source_image_encryption_key
        #   @return [::Google::Cloud::Compute::V1::CustomerEncryptionKey]
        #     The customer-supplied encryption key of the source image. Required if the source image is protected by a customer-supplied encryption key.
        # @!attribute [rw] source_image_id
        #   @return [::String]
        #     [Output Only] The ID value of the image used to create this image. This value may be used to determine whether the image was taken from the current or a previous instance of a given image name.
        # @!attribute [rw] source_snapshot
        #   @return [::String]
        #     URL of the source snapshot used to create this image. The following are valid formats for the URL: - https://www.googleapis.com/compute/v1/projects/project_id/global/ snapshots/snapshot_name - projects/project_id/global/snapshots/snapshot_name In order to create an image, you must provide the full or partial URL of one of the following: - The rawDisk.source URL - The sourceDisk URL - The sourceImage URL - The sourceSnapshot URL
        # @!attribute [rw] source_snapshot_encryption_key
        #   @return [::Google::Cloud::Compute::V1::CustomerEncryptionKey]
        #     The customer-supplied encryption key of the source snapshot. Required if the source snapshot is protected by a customer-supplied encryption key.
        # @!attribute [rw] source_snapshot_id
        #   @return [::String]
        #     [Output Only] The ID value of the snapshot used to create this image. This value may be used to determine whether the snapshot was taken from the current or a previous instance of a given snapshot name.
        # @!attribute [rw] source_type
        #   @return [::String]
        #     The type of the image used to create this disk. The default and only valid value is RAW.
        #     Check the SourceType enum for the list of possible values.
        # @!attribute [rw] status
        #   @return [::String]
        #     [Output Only] The status of the image. An image can be used to create other resources, such as instances, only after the image has been successfully created and the status is set to READY. Possible values are FAILED, PENDING, or READY.
        #     Check the Status enum for the list of possible values.
        # @!attribute [rw] storage_locations
        #   @return [::Array<::String>]
        #     Cloud Storage bucket storage location of the image (regional or multi-regional).
        class Image
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class LabelsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # The architecture of the image. Valid values are ARM64 or X86_64.
          module Architecture
            # A value indicating that the enum field is not set.
            UNDEFINED_ARCHITECTURE = 0

            # Default value indicating Architecture is not set.
            ARCHITECTURE_UNSPECIFIED = 394_750_507

            # Machines with architecture ARM64
            ARM64 = 62_547_450

            # Machines with architecture X86_64
            X86_64 = 425_300_551
          end

          # The type of the image used to create this disk. The default and only valid value is RAW.
          module SourceType
            # A value indicating that the enum field is not set.
            UNDEFINED_SOURCE_TYPE = 0

            RAW = 80_904
          end

          # [Output Only] The status of the image. An image can be used to create other resources, such as instances, only after the image has been successfully created and the status is set to READY. Possible values are FAILED, PENDING, or READY.
          module Status
            # A value indicating that the enum field is not set.
            UNDEFINED_STATUS = 0

            # Image is deleting.
            DELETING = 528_602_024

            # Image creation failed due to an error.
            FAILED = 455_706_685

            # Image hasn't been created as yet.
            PENDING = 35_394_935

            # Image has been successfully created.
            READY = 77_848_963
          end
        end

        # @!attribute [rw] image
        #   @return [::Google::Cloud::Compute::V1::Image]
        #     The latest image that is part of the specified image family in the requested location, and that is not deprecated.
        class ImageFamilyView
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Contains a list of images.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::Image>]
        #     A list of Image resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     Type of resource.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class ImageList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Initial State for shielded instance, these are public keys which are safe to store in public
        # @!attribute [rw] dbs
        #   @return [::Array<::Google::Cloud::Compute::V1::FileContentBuffer>]
        #     The Key Database (db).
        # @!attribute [rw] dbxs
        #   @return [::Array<::Google::Cloud::Compute::V1::FileContentBuffer>]
        #     The forbidden key database (dbx).
        # @!attribute [rw] keks
        #   @return [::Array<::Google::Cloud::Compute::V1::FileContentBuffer>]
        #     The Key Exchange Key (KEK).
        # @!attribute [rw] pk
        #   @return [::Google::Cloud::Compute::V1::FileContentBuffer]
        #     The Platform Key (PK).
        class InitialStateConfig
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Addresses.Insert. See the method description for details.
        # @!attribute [rw] address_resource
        #   @return [::Google::Cloud::Compute::V1::Address]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class InsertAddressRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Autoscalers.Insert. See the method description for details.
        # @!attribute [rw] autoscaler_resource
        #   @return [::Google::Cloud::Compute::V1::Autoscaler]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     Name of the zone for this request.
        class InsertAutoscalerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for BackendBuckets.Insert. See the method description for details.
        # @!attribute [rw] backend_bucket_resource
        #   @return [::Google::Cloud::Compute::V1::BackendBucket]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class InsertBackendBucketRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for BackendServices.Insert. See the method description for details.
        # @!attribute [rw] backend_service_resource
        #   @return [::Google::Cloud::Compute::V1::BackendService]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class InsertBackendServiceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Disks.Insert. See the method description for details.
        # @!attribute [rw] disk_resource
        #   @return [::Google::Cloud::Compute::V1::Disk]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] source_image
        #   @return [::String]
        #     Source image to restore onto a disk. This field is optional.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class InsertDiskRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ExternalVpnGateways.Insert. See the method description for details.
        # @!attribute [rw] external_vpn_gateway_resource
        #   @return [::Google::Cloud::Compute::V1::ExternalVpnGateway]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class InsertExternalVpnGatewayRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for FirewallPolicies.Insert. See the method description for details.
        # @!attribute [rw] firewall_policy_resource
        #   @return [::Google::Cloud::Compute::V1::FirewallPolicy]
        #     The body resource for this request
        # @!attribute [rw] parent_id
        #   @return [::String]
        #     Parent ID for this request. The ID can be either be "folders/[FOLDER_ID]" if the parent is a folder or "organizations/[ORGANIZATION_ID]" if the parent is an organization.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class InsertFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Firewalls.Insert. See the method description for details.
        # @!attribute [rw] firewall_resource
        #   @return [::Google::Cloud::Compute::V1::Firewall]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class InsertFirewallRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ForwardingRules.Insert. See the method description for details.
        # @!attribute [rw] forwarding_rule_resource
        #   @return [::Google::Cloud::Compute::V1::ForwardingRule]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class InsertForwardingRuleRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for GlobalAddresses.Insert. See the method description for details.
        # @!attribute [rw] address_resource
        #   @return [::Google::Cloud::Compute::V1::Address]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class InsertGlobalAddressRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for GlobalForwardingRules.Insert. See the method description for details.
        # @!attribute [rw] forwarding_rule_resource
        #   @return [::Google::Cloud::Compute::V1::ForwardingRule]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class InsertGlobalForwardingRuleRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for GlobalNetworkEndpointGroups.Insert. See the method description for details.
        # @!attribute [rw] network_endpoint_group_resource
        #   @return [::Google::Cloud::Compute::V1::NetworkEndpointGroup]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class InsertGlobalNetworkEndpointGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for GlobalPublicDelegatedPrefixes.Insert. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] public_delegated_prefix_resource
        #   @return [::Google::Cloud::Compute::V1::PublicDelegatedPrefix]
        #     The body resource for this request
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class InsertGlobalPublicDelegatedPrefixeRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for HealthChecks.Insert. See the method description for details.
        # @!attribute [rw] health_check_resource
        #   @return [::Google::Cloud::Compute::V1::HealthCheck]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class InsertHealthCheckRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Images.Insert. See the method description for details.
        # @!attribute [rw] force_create
        #   @return [::Boolean]
        #     Force image creation if true.
        # @!attribute [rw] image_resource
        #   @return [::Google::Cloud::Compute::V1::Image]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class InsertImageRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceGroupManagers.Insert. See the method description for details.
        # @!attribute [rw] instance_group_manager_resource
        #   @return [::Google::Cloud::Compute::V1::InstanceGroupManager]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where you want to create the managed instance group.
        class InsertInstanceGroupManagerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceGroups.Insert. See the method description for details.
        # @!attribute [rw] instance_group_resource
        #   @return [::Google::Cloud::Compute::V1::InstanceGroup]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where you want to create the instance group.
        class InsertInstanceGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.Insert. See the method description for details.
        # @!attribute [rw] instance_resource
        #   @return [::Google::Cloud::Compute::V1::Instance]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] source_instance_template
        #   @return [::String]
        #     Specifies instance template to create the instance. This field is optional. It can be a full or partial URL. For example, the following are all valid URLs to an instance template: - https://www.googleapis.com/compute/v1/projects/project /global/instanceTemplates/instanceTemplate - projects/project/global/instanceTemplates/instanceTemplate - global/instanceTemplates/instanceTemplate
        # @!attribute [rw] source_machine_image
        #   @return [::String]
        #     Specifies the machine image to use to create the instance. This field is optional. It can be a full or partial URL. For example, the following are all valid URLs to a machine image: - https://www.googleapis.com/compute/v1/projects/project/global/global /machineImages/machineImage - projects/project/global/global/machineImages/machineImage - global/machineImages/machineImage
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class InsertInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceTemplates.Insert. See the method description for details.
        # @!attribute [rw] instance_template_resource
        #   @return [::Google::Cloud::Compute::V1::InstanceTemplate]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class InsertInstanceTemplateRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InterconnectAttachments.Insert. See the method description for details.
        # @!attribute [rw] interconnect_attachment_resource
        #   @return [::Google::Cloud::Compute::V1::InterconnectAttachment]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] validate_only
        #   @return [::Boolean]
        #     If true, the request will not be committed.
        class InsertInterconnectAttachmentRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Interconnects.Insert. See the method description for details.
        # @!attribute [rw] interconnect_resource
        #   @return [::Google::Cloud::Compute::V1::Interconnect]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class InsertInterconnectRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Licenses.Insert. See the method description for details.
        # @!attribute [rw] license_resource
        #   @return [::Google::Cloud::Compute::V1::License]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class InsertLicenseRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for MachineImages.Insert. See the method description for details.
        # @!attribute [rw] machine_image_resource
        #   @return [::Google::Cloud::Compute::V1::MachineImage]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] source_instance
        #   @return [::String]
        #     Required. Source instance that is used to create the machine image from.
        class InsertMachineImageRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkAttachments.Insert. See the method description for details.
        # @!attribute [rw] network_attachment_resource
        #   @return [::Google::Cloud::Compute::V1::NetworkAttachment]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region of this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000). end_interface: MixerMutationRequestBuilder
        class InsertNetworkAttachmentRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkEdgeSecurityServices.Insert. See the method description for details.
        # @!attribute [rw] network_edge_security_service_resource
        #   @return [::Google::Cloud::Compute::V1::NetworkEdgeSecurityService]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] validate_only
        #   @return [::Boolean]
        #     If true, the request will not be committed.
        class InsertNetworkEdgeSecurityServiceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkEndpointGroups.Insert. See the method description for details.
        # @!attribute [rw] network_endpoint_group_resource
        #   @return [::Google::Cloud::Compute::V1::NetworkEndpointGroup]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where you want to create the network endpoint group. It should comply with RFC1035.
        class InsertNetworkEndpointGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkFirewallPolicies.Insert. See the method description for details.
        # @!attribute [rw] firewall_policy_resource
        #   @return [::Google::Cloud::Compute::V1::FirewallPolicy]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class InsertNetworkFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Networks.Insert. See the method description for details.
        # @!attribute [rw] network_resource
        #   @return [::Google::Cloud::Compute::V1::Network]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class InsertNetworkRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NodeGroups.Insert. See the method description for details.
        # @!attribute [rw] initial_node_count
        #   @return [::Integer]
        #     Initial count of nodes in the node group.
        # @!attribute [rw] node_group_resource
        #   @return [::Google::Cloud::Compute::V1::NodeGroup]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class InsertNodeGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NodeTemplates.Insert. See the method description for details.
        # @!attribute [rw] node_template_resource
        #   @return [::Google::Cloud::Compute::V1::NodeTemplate]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class InsertNodeTemplateRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for PacketMirrorings.Insert. See the method description for details.
        # @!attribute [rw] packet_mirroring_resource
        #   @return [::Google::Cloud::Compute::V1::PacketMirroring]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class InsertPacketMirroringRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for PublicAdvertisedPrefixes.Insert. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] public_advertised_prefix_resource
        #   @return [::Google::Cloud::Compute::V1::PublicAdvertisedPrefix]
        #     The body resource for this request
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class InsertPublicAdvertisedPrefixeRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for PublicDelegatedPrefixes.Insert. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] public_delegated_prefix_resource
        #   @return [::Google::Cloud::Compute::V1::PublicDelegatedPrefix]
        #     The body resource for this request
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region of this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class InsertPublicDelegatedPrefixeRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionAutoscalers.Insert. See the method description for details.
        # @!attribute [rw] autoscaler_resource
        #   @return [::Google::Cloud::Compute::V1::Autoscaler]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class InsertRegionAutoscalerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionBackendServices.Insert. See the method description for details.
        # @!attribute [rw] backend_service_resource
        #   @return [::Google::Cloud::Compute::V1::BackendService]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class InsertRegionBackendServiceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionCommitments.Insert. See the method description for details.
        # @!attribute [rw] commitment_resource
        #   @return [::Google::Cloud::Compute::V1::Commitment]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class InsertRegionCommitmentRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionDisks.Insert. See the method description for details.
        # @!attribute [rw] disk_resource
        #   @return [::Google::Cloud::Compute::V1::Disk]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] source_image
        #   @return [::String]
        #     Source image to restore onto a disk. This field is optional.
        class InsertRegionDiskRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionHealthChecks.Insert. See the method description for details.
        # @!attribute [rw] health_check_resource
        #   @return [::Google::Cloud::Compute::V1::HealthCheck]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class InsertRegionHealthCheckRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionHealthCheckServices.Insert. See the method description for details.
        # @!attribute [rw] health_check_service_resource
        #   @return [::Google::Cloud::Compute::V1::HealthCheckService]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class InsertRegionHealthCheckServiceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionInstanceGroupManagers.Insert. See the method description for details.
        # @!attribute [rw] instance_group_manager_resource
        #   @return [::Google::Cloud::Compute::V1::InstanceGroupManager]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class InsertRegionInstanceGroupManagerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionInstanceTemplates.Insert. See the method description for details.
        # @!attribute [rw] instance_template_resource
        #   @return [::Google::Cloud::Compute::V1::InstanceTemplate]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class InsertRegionInstanceTemplateRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionNetworkEndpointGroups.Insert. See the method description for details.
        # @!attribute [rw] network_endpoint_group_resource
        #   @return [::Google::Cloud::Compute::V1::NetworkEndpointGroup]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region where you want to create the network endpoint group. It should comply with RFC1035.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class InsertRegionNetworkEndpointGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionNetworkFirewallPolicies.Insert. See the method description for details.
        # @!attribute [rw] firewall_policy_resource
        #   @return [::Google::Cloud::Compute::V1::FirewallPolicy]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class InsertRegionNetworkFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionNotificationEndpoints.Insert. See the method description for details.
        # @!attribute [rw] notification_endpoint_resource
        #   @return [::Google::Cloud::Compute::V1::NotificationEndpoint]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class InsertRegionNotificationEndpointRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionSecurityPolicies.Insert. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] security_policy_resource
        #   @return [::Google::Cloud::Compute::V1::SecurityPolicy]
        #     The body resource for this request
        # @!attribute [rw] validate_only
        #   @return [::Boolean]
        #     If true, the request will not be committed.
        class InsertRegionSecurityPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionSslCertificates.Insert. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] ssl_certificate_resource
        #   @return [::Google::Cloud::Compute::V1::SslCertificate]
        #     The body resource for this request
        class InsertRegionSslCertificateRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionSslPolicies.Insert. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] ssl_policy_resource
        #   @return [::Google::Cloud::Compute::V1::SslPolicy]
        #     The body resource for this request
        class InsertRegionSslPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionTargetHttpProxies.Insert. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_http_proxy_resource
        #   @return [::Google::Cloud::Compute::V1::TargetHttpProxy]
        #     The body resource for this request
        class InsertRegionTargetHttpProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionTargetHttpsProxies.Insert. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_https_proxy_resource
        #   @return [::Google::Cloud::Compute::V1::TargetHttpsProxy]
        #     The body resource for this request
        class InsertRegionTargetHttpsProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionTargetTcpProxies.Insert. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_tcp_proxy_resource
        #   @return [::Google::Cloud::Compute::V1::TargetTcpProxy]
        #     The body resource for this request
        class InsertRegionTargetTcpProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionUrlMaps.Insert. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     begin_interface: MixerMutationRequestBuilder Request ID to support idempotency.
        # @!attribute [rw] url_map_resource
        #   @return [::Google::Cloud::Compute::V1::UrlMap]
        #     The body resource for this request
        class InsertRegionUrlMapRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Reservations.Insert. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] reservation_resource
        #   @return [::Google::Cloud::Compute::V1::Reservation]
        #     The body resource for this request
        # @!attribute [rw] zone
        #   @return [::String]
        #     Name of the zone for this request.
        class InsertReservationRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ResourcePolicies.Insert. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] resource_policy_resource
        #   @return [::Google::Cloud::Compute::V1::ResourcePolicy]
        #     The body resource for this request
        class InsertResourcePolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Routes.Insert. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] route_resource
        #   @return [::Google::Cloud::Compute::V1::Route]
        #     The body resource for this request
        class InsertRouteRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Routers.Insert. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] router_resource
        #   @return [::Google::Cloud::Compute::V1::Router]
        #     The body resource for this request
        class InsertRouterRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for SecurityPolicies.Insert. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] security_policy_resource
        #   @return [::Google::Cloud::Compute::V1::SecurityPolicy]
        #     The body resource for this request
        # @!attribute [rw] validate_only
        #   @return [::Boolean]
        #     If true, the request will not be committed.
        class InsertSecurityPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ServiceAttachments.Insert. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region of this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] service_attachment_resource
        #   @return [::Google::Cloud::Compute::V1::ServiceAttachment]
        #     The body resource for this request
        class InsertServiceAttachmentRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Snapshots.Insert. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] snapshot_resource
        #   @return [::Google::Cloud::Compute::V1::Snapshot]
        #     The body resource for this request
        class InsertSnapshotRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for SslCertificates.Insert. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] ssl_certificate_resource
        #   @return [::Google::Cloud::Compute::V1::SslCertificate]
        #     The body resource for this request
        class InsertSslCertificateRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for SslPolicies.Insert. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] ssl_policy_resource
        #   @return [::Google::Cloud::Compute::V1::SslPolicy]
        #     The body resource for this request
        class InsertSslPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Subnetworks.Insert. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] subnetwork_resource
        #   @return [::Google::Cloud::Compute::V1::Subnetwork]
        #     The body resource for this request
        class InsertSubnetworkRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetGrpcProxies.Insert. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_grpc_proxy_resource
        #   @return [::Google::Cloud::Compute::V1::TargetGrpcProxy]
        #     The body resource for this request
        class InsertTargetGrpcProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetHttpProxies.Insert. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_http_proxy_resource
        #   @return [::Google::Cloud::Compute::V1::TargetHttpProxy]
        #     The body resource for this request
        class InsertTargetHttpProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetHttpsProxies.Insert. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_https_proxy_resource
        #   @return [::Google::Cloud::Compute::V1::TargetHttpsProxy]
        #     The body resource for this request
        class InsertTargetHttpsProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetInstances.Insert. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_instance_resource
        #   @return [::Google::Cloud::Compute::V1::TargetInstance]
        #     The body resource for this request
        # @!attribute [rw] zone
        #   @return [::String]
        #     Name of the zone scoping this request.
        class InsertTargetInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetPools.Insert. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_pool_resource
        #   @return [::Google::Cloud::Compute::V1::TargetPool]
        #     The body resource for this request
        class InsertTargetPoolRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetSslProxies.Insert. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_ssl_proxy_resource
        #   @return [::Google::Cloud::Compute::V1::TargetSslProxy]
        #     The body resource for this request
        class InsertTargetSslProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetTcpProxies.Insert. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_tcp_proxy_resource
        #   @return [::Google::Cloud::Compute::V1::TargetTcpProxy]
        #     The body resource for this request
        class InsertTargetTcpProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetVpnGateways.Insert. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_vpn_gateway_resource
        #   @return [::Google::Cloud::Compute::V1::TargetVpnGateway]
        #     The body resource for this request
        class InsertTargetVpnGatewayRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for UrlMaps.Insert. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] url_map_resource
        #   @return [::Google::Cloud::Compute::V1::UrlMap]
        #     The body resource for this request
        class InsertUrlMapRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for VpnGateways.Insert. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] vpn_gateway_resource
        #   @return [::Google::Cloud::Compute::V1::VpnGateway]
        #     The body resource for this request
        class InsertVpnGatewayRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for VpnTunnels.Insert. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] vpn_tunnel_resource
        #   @return [::Google::Cloud::Compute::V1::VpnTunnel]
        #     The body resource for this request
        class InsertVpnTunnelRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents an Instance resource. An instance is a virtual machine that is hosted on Google Cloud Platform. For more information, read Virtual Machine Instances.
        # @!attribute [rw] advanced_machine_features
        #   @return [::Google::Cloud::Compute::V1::AdvancedMachineFeatures]
        #     Controls for advanced machine-related behavior features.
        # @!attribute [rw] can_ip_forward
        #   @return [::Boolean]
        #     Allows this instance to send and receive packets with non-matching destination or source IPs. This is required if you plan to use this instance to forward routes. For more information, see Enabling IP Forwarding .
        # @!attribute [rw] confidential_instance_config
        #   @return [::Google::Cloud::Compute::V1::ConfidentialInstanceConfig]
        # @!attribute [rw] cpu_platform
        #   @return [::String]
        #     [Output Only] The CPU platform used by this instance.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] deletion_protection
        #   @return [::Boolean]
        #     Whether the resource should be protected against deletion.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] disks
        #   @return [::Array<::Google::Cloud::Compute::V1::AttachedDisk>]
        #     Array of disks associated with this instance. Persistent disks must be created before you can assign them.
        # @!attribute [rw] display_device
        #   @return [::Google::Cloud::Compute::V1::DisplayDevice]
        #     Enables display device for the instance.
        # @!attribute [rw] fingerprint
        #   @return [::String]
        #     Specifies a fingerprint for this resource, which is essentially a hash of the instance's contents and used for optimistic locking. The fingerprint is initially generated by Compute Engine and changes after every request to modify or update the instance. You must always provide an up-to-date fingerprint hash in order to update the instance. To see the latest fingerprint, make get() request to the instance.
        # @!attribute [rw] guest_accelerators
        #   @return [::Array<::Google::Cloud::Compute::V1::AcceleratorConfig>]
        #     A list of the type and count of accelerator cards attached to the instance.
        # @!attribute [rw] hostname
        #   @return [::String]
        #     Specifies the hostname of the instance. The specified hostname must be RFC1035 compliant. If hostname is not specified, the default hostname is [INSTANCE_NAME].c.[PROJECT_ID].internal when using the global DNS, and [INSTANCE_NAME].[ZONE].c.[PROJECT_ID].internal when using zonal DNS.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] instance_encryption_key
        #   @return [::Google::Cloud::Compute::V1::CustomerEncryptionKey]
        #     Encrypts suspended data for an instance with a customer-managed encryption key. If you are creating a new instance, this field will encrypt the local SSD and in-memory contents of the instance during the suspend operation. If you do not provide an encryption key when creating the instance, then the local SSD and in-memory contents will be encrypted using an automatically generated key during the suspend operation.
        # @!attribute [rw] key_revocation_action_type
        #   @return [::String]
        #     KeyRevocationActionType of the instance. Supported options are "STOP" and "NONE". The default value is "NONE" if it is not specified.
        #     Check the KeyRevocationActionType enum for the list of possible values.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#instance for instances.
        # @!attribute [rw] label_fingerprint
        #   @return [::String]
        #     A fingerprint for this request, which is essentially a hash of the label's contents and used for optimistic locking. The fingerprint is initially generated by Compute Engine and changes after every request to modify or update labels. You must always provide an up-to-date fingerprint hash in order to update or change labels. To see the latest fingerprint, make get() request to the instance.
        # @!attribute [rw] labels
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     Labels to apply to this instance. These can be later modified by the setLabels method.
        # @!attribute [rw] last_start_timestamp
        #   @return [::String]
        #     [Output Only] Last start timestamp in RFC3339 text format.
        # @!attribute [rw] last_stop_timestamp
        #   @return [::String]
        #     [Output Only] Last stop timestamp in RFC3339 text format.
        # @!attribute [rw] last_suspended_timestamp
        #   @return [::String]
        #     [Output Only] Last suspended timestamp in RFC3339 text format.
        # @!attribute [rw] machine_type
        #   @return [::String]
        #     Full or partial URL of the machine type resource to use for this instance, in the format: zones/zone/machineTypes/machine-type. This is provided by the client when the instance is created. For example, the following is a valid partial url to a predefined machine type: zones/us-central1-f/machineTypes/n1-standard-1 To create a custom machine type, provide a URL to a machine type in the following format, where CPUS is 1 or an even number up to 32 (2, 4, 6, ... 24, etc), and MEMORY is the total memory for this instance. Memory must be a multiple of 256 MB and must be supplied in MB (e.g. 5 GB of memory is 5120 MB): zones/zone/machineTypes/custom-CPUS-MEMORY For example: zones/us-central1-f/machineTypes/custom-4-5120 For a full list of restrictions, read the Specifications for custom machine types.
        # @!attribute [rw] metadata
        #   @return [::Google::Cloud::Compute::V1::Metadata]
        #     The metadata key/value pairs assigned to this instance. This includes custom metadata and predefined keys.
        # @!attribute [rw] min_cpu_platform
        #   @return [::String]
        #     Specifies a minimum CPU platform for the VM instance. Applicable values are the friendly names of CPU platforms, such as minCpuPlatform: "Intel Haswell" or minCpuPlatform: "Intel Sandy Bridge".
        # @!attribute [rw] name
        #   @return [::String]
        #     The name of the resource, provided by the client when initially creating the resource. The resource name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] network_interfaces
        #   @return [::Array<::Google::Cloud::Compute::V1::NetworkInterface>]
        #     An array of network configurations for this instance. These specify how interfaces are configured to interact with other network services, such as connecting to the internet. Multiple interfaces are supported per instance.
        # @!attribute [rw] network_performance_config
        #   @return [::Google::Cloud::Compute::V1::NetworkPerformanceConfig]
        # @!attribute [rw] params
        #   @return [::Google::Cloud::Compute::V1::InstanceParams]
        #     Input only. [Input Only] Additional params passed with the request, but not persisted as part of resource payload.
        # @!attribute [rw] private_ipv6_google_access
        #   @return [::String]
        #     The private IPv6 google access type for the VM. If not specified, use INHERIT_FROM_SUBNETWORK as default.
        #     Check the PrivateIpv6GoogleAccess enum for the list of possible values.
        # @!attribute [rw] reservation_affinity
        #   @return [::Google::Cloud::Compute::V1::ReservationAffinity]
        #     Specifies the reservations that this instance can consume from.
        # @!attribute [rw] resource_policies
        #   @return [::Array<::String>]
        #     Resource policies applied to this instance.
        # @!attribute [rw] resource_status
        #   @return [::Google::Cloud::Compute::V1::ResourceStatus]
        #     [Output Only] Specifies values set for instance attributes as compared to the values requested by user in the corresponding input only field.
        # @!attribute [rw] satisfies_pzs
        #   @return [::Boolean]
        #     [Output Only] Reserved for future use.
        # @!attribute [rw] scheduling
        #   @return [::Google::Cloud::Compute::V1::Scheduling]
        #     Sets the scheduling options for this instance.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] service_accounts
        #   @return [::Array<::Google::Cloud::Compute::V1::ServiceAccount>]
        #     A list of service accounts, with their specified scopes, authorized for this instance. Only one service account per VM instance is supported. Service accounts generate access tokens that can be accessed through the metadata server and used to authenticate applications on the instance. See Service Accounts for more information.
        # @!attribute [rw] shielded_instance_config
        #   @return [::Google::Cloud::Compute::V1::ShieldedInstanceConfig]
        # @!attribute [rw] shielded_instance_integrity_policy
        #   @return [::Google::Cloud::Compute::V1::ShieldedInstanceIntegrityPolicy]
        # @!attribute [rw] source_machine_image
        #   @return [::String]
        #     Source machine image
        # @!attribute [rw] source_machine_image_encryption_key
        #   @return [::Google::Cloud::Compute::V1::CustomerEncryptionKey]
        #     Source machine image encryption key when creating an instance from a machine image.
        # @!attribute [rw] start_restricted
        #   @return [::Boolean]
        #     [Output Only] Whether a VM has been restricted for start because Compute Engine has detected suspicious activity.
        # @!attribute [rw] status
        #   @return [::String]
        #     [Output Only] The status of the instance. One of the following values: PROVISIONING, STAGING, RUNNING, STOPPING, SUSPENDING, SUSPENDED, REPAIRING, and TERMINATED. For more information about the status of the instance, see Instance life cycle.
        #     Check the Status enum for the list of possible values.
        # @!attribute [rw] status_message
        #   @return [::String]
        #     [Output Only] An optional, human-readable explanation of the status.
        # @!attribute [rw] tags
        #   @return [::Google::Cloud::Compute::V1::Tags]
        #     Tags to apply to this instance. Tags are used to identify valid sources or targets for network firewalls and are specified by the client during instance creation. The tags can be later modified by the setTags method. Each tag within the list must comply with RFC1035. Multiple tags can be specified via the 'tags.items' field.
        # @!attribute [rw] zone
        #   @return [::String]
        #     [Output Only] URL of the zone where the instance resides. You must specify this field as part of the HTTP request URL. It is not settable as a field in the request body.
        class Instance
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class LabelsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # KeyRevocationActionType of the instance. Supported options are "STOP" and "NONE". The default value is "NONE" if it is not specified.
          module KeyRevocationActionType
            # A value indicating that the enum field is not set.
            UNDEFINED_KEY_REVOCATION_ACTION_TYPE = 0

            # Default value. This value is unused.
            KEY_REVOCATION_ACTION_TYPE_UNSPECIFIED = 467_110_106

            # Indicates user chose no operation.
            NONE = 2_402_104

            # Indicates user chose to opt for VM shutdown on key revocation.
            STOP = 2_555_906
          end

          # The private IPv6 google access type for the VM. If not specified, use INHERIT_FROM_SUBNETWORK as default.
          module PrivateIpv6GoogleAccess
            # A value indicating that the enum field is not set.
            UNDEFINED_PRIVATE_IPV6_GOOGLE_ACCESS = 0

            # Bidirectional private IPv6 access to/from Google services. If specified, the subnetwork who is attached to the instance's default network interface will be assigned an internal IPv6 prefix if it doesn't have before.
            ENABLE_BIDIRECTIONAL_ACCESS_TO_GOOGLE = 427_975_994

            # Outbound private IPv6 access from VMs in this subnet to Google services. If specified, the subnetwork who is attached to the instance's default network interface will be assigned an internal IPv6 prefix if it doesn't have before.
            ENABLE_OUTBOUND_VM_ACCESS_TO_GOOGLE = 288_210_263

            # Each network interface inherits PrivateIpv6GoogleAccess from its subnetwork.
            INHERIT_FROM_SUBNETWORK = 530_256_959
          end

          # [Output Only] The status of the instance. One of the following values: PROVISIONING, STAGING, RUNNING, STOPPING, SUSPENDING, SUSPENDED, REPAIRING, and TERMINATED. For more information about the status of the instance, see Instance life cycle.
          module Status
            # A value indicating that the enum field is not set.
            UNDEFINED_STATUS = 0

            # The instance is halted and we are performing tear down tasks like network deprogramming, releasing quota, IP, tearing down disks etc.
            DEPROVISIONING = 428_935_662

            # Resources are being allocated for the instance.
            PROVISIONING = 290_896_621

            # The instance is in repair.
            REPAIRING = 413_483_285

            # The instance is running.
            RUNNING = 121_282_975

            # All required resources have been allocated and the instance is being started.
            STAGING = 431_072_283

            # The instance has stopped successfully.
            STOPPED = 444_276_141

            # The instance is currently stopping (either being deleted or killed).
            STOPPING = 350_791_796

            # The instance has suspended.
            SUSPENDED = 51_223_995

            # The instance is suspending.
            SUSPENDING = 514_206_246

            # The instance has stopped (either by explicit action or underlying failure).
            TERMINATED = 250_018_339
          end
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::InstancesScopedList}]
        #     An object that contains a list of instances scoped by zone.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#instanceAggregatedList for aggregated lists of Instance resources.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class InstanceAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::InstancesScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # @!attribute [rw] consumption_info
        #   @return [::Google::Cloud::Compute::V1::InstanceConsumptionInfo]
        #     Resources consumed by the instance.
        # @!attribute [rw] instance
        #   @return [::String]
        #     Server-defined URL for the instance.
        class InstanceConsumptionData
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] guest_cpus
        #   @return [::Integer]
        #     The number of virtual CPUs that are available to the instance.
        # @!attribute [rw] local_ssd_gb
        #   @return [::Integer]
        #     The amount of local SSD storage available to the instance, defined in GiB.
        # @!attribute [rw] memory_mb
        #   @return [::Integer]
        #     The amount of physical memory available to the instance, defined in MiB.
        # @!attribute [rw] min_node_cpus
        #   @return [::Integer]
        #     The minimal guaranteed number of virtual CPUs that are reserved.
        class InstanceConsumptionInfo
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents an Instance Group resource. Instance Groups can be used to configure a target for load balancing. Instance groups can either be managed or unmanaged. To create managed instance groups, use the instanceGroupManager or regionInstanceGroupManager resource instead. Use zonal unmanaged instance groups if you need to apply load balancing to groups of heterogeneous instances or if you need to manage the instances yourself. You cannot create regional unmanaged instance groups. For more information, read Instance groups.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] The creation timestamp for this instance group in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] fingerprint
        #   @return [::String]
        #     [Output Only] The fingerprint of the named ports. The system uses this fingerprint to detect conflicts when multiple users change the named ports concurrently.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] A unique identifier for this instance group, generated by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] The resource type, which is always compute#instanceGroup for instance groups.
        # @!attribute [rw] name
        #   @return [::String]
        #     The name of the instance group. The name must be 1-63 characters long, and comply with RFC1035.
        # @!attribute [rw] named_ports
        #   @return [::Array<::Google::Cloud::Compute::V1::NamedPort>]
        #     Assigns a name to a port number. For example: \\{name: "http", port: 80} This allows the system to reference ports by the assigned name instead of a port number. Named ports can also contain multiple ports. For example: [\\{name: "app1", port: 8080}, \\{name: "app1", port: 8081}, \\{name: "app2", port: 8082}] Named ports apply to all instances in this instance group.
        # @!attribute [rw] network
        #   @return [::String]
        #     [Output Only] The URL of the network to which all instances in the instance group belong. If your instance has multiple network interfaces, then the network and subnetwork fields only refer to the network and subnet used by your primary interface (nic0).
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] The URL of the region where the instance group is located (for regional resources).
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] The URL for this instance group. The server generates this URL.
        # @!attribute [rw] size
        #   @return [::Integer]
        #     [Output Only] The total number of instances in the instance group.
        # @!attribute [rw] subnetwork
        #   @return [::String]
        #     [Output Only] The URL of the subnetwork to which all instances in the instance group belong. If your instance has multiple network interfaces, then the network and subnetwork fields only refer to the network and subnet used by your primary interface (nic0).
        # @!attribute [rw] zone
        #   @return [::String]
        #     [Output Only] The URL of the zone where the instance group is located (for zonal resources).
        class InstanceGroup
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::InstanceGroupsScopedList}]
        #     A list of InstanceGroupsScopedList resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] The resource type, which is always compute#instanceGroupAggregatedList for aggregated lists of instance groups.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class InstanceGroupAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::InstanceGroupsScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # A list of InstanceGroup resources.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::InstanceGroup>]
        #     A list of InstanceGroup resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] The resource type, which is always compute#instanceGroupList for instance group lists.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class InstanceGroupList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a Managed Instance Group resource. An instance group is a collection of VM instances that you can manage as a single entity. For more information, read Instance groups. For zonal Managed Instance Group, use the instanceGroupManagers resource. For regional Managed Instance Group, use the regionInstanceGroupManagers resource.
        # @!attribute [rw] auto_healing_policies
        #   @return [::Array<::Google::Cloud::Compute::V1::InstanceGroupManagerAutoHealingPolicy>]
        #     The autohealing policy for this managed instance group. You can specify only one value.
        # @!attribute [rw] base_instance_name
        #   @return [::String]
        #     The base instance name to use for instances in this group. The value must be 1-58 characters long. Instances are named by appending a hyphen and a random four-character string to the base instance name. The base instance name must comply with RFC1035.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] The creation timestamp for this managed instance group in RFC3339 text format.
        # @!attribute [rw] current_actions
        #   @return [::Google::Cloud::Compute::V1::InstanceGroupManagerActionsSummary]
        #     [Output Only] The list of instance actions and the number of instances in this managed instance group that are scheduled for each of those actions.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource.
        # @!attribute [rw] distribution_policy
        #   @return [::Google::Cloud::Compute::V1::DistributionPolicy]
        #     Policy specifying the intended distribution of managed instances across zones in a regional managed instance group.
        # @!attribute [rw] fingerprint
        #   @return [::String]
        #     Fingerprint of this resource. This field may be used in optimistic locking. It will be ignored when inserting an InstanceGroupManager. An up-to-date fingerprint must be provided in order to update the InstanceGroupManager, otherwise the request will fail with error 412 conditionNotMet. To see the latest fingerprint, make a get() request to retrieve an InstanceGroupManager.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] A unique identifier for this resource type. The server generates this identifier.
        # @!attribute [rw] instance_group
        #   @return [::String]
        #     [Output Only] The URL of the Instance Group resource.
        # @!attribute [rw] instance_lifecycle_policy
        #   @return [::Google::Cloud::Compute::V1::InstanceGroupManagerInstanceLifecyclePolicy]
        #     The repair policy for this managed instance group.
        # @!attribute [rw] instance_template
        #   @return [::String]
        #     The URL of the instance template that is specified for this managed instance group. The group uses this template to create all new instances in the managed instance group. The templates for existing instances in the group do not change unless you run recreateInstances, run applyUpdatesToInstances, or set the group's updatePolicy.type to PROACTIVE.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] The resource type, which is always compute#instanceGroupManager for managed instance groups.
        # @!attribute [rw] list_managed_instances_results
        #   @return [::String]
        #     Pagination behavior of the listManagedInstances API method for this managed instance group.
        #     Check the ListManagedInstancesResults enum for the list of possible values.
        # @!attribute [rw] name
        #   @return [::String]
        #     The name of the managed instance group. The name must be 1-63 characters long, and comply with RFC1035.
        # @!attribute [rw] named_ports
        #   @return [::Array<::Google::Cloud::Compute::V1::NamedPort>]
        #     Named ports configured for the Instance Groups complementary to this Instance Group Manager.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] The URL of the region where the managed instance group resides (for regional resources).
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] The URL for this managed instance group. The server defines this URL.
        # @!attribute [rw] stateful_policy
        #   @return [::Google::Cloud::Compute::V1::StatefulPolicy]
        #     Stateful configuration for this Instanced Group Manager
        # @!attribute [rw] status
        #   @return [::Google::Cloud::Compute::V1::InstanceGroupManagerStatus]
        #     [Output Only] The status of this managed instance group.
        # @!attribute [rw] target_pools
        #   @return [::Array<::String>]
        #     The URLs for all TargetPool resources to which instances in the instanceGroup field are added. The target pools automatically apply to all of the instances in the managed instance group.
        # @!attribute [rw] target_size
        #   @return [::Integer]
        #     The target number of running instances for this managed instance group. You can reduce this number by using the instanceGroupManager deleteInstances or abandonInstances methods. Resizing the group also changes this number.
        # @!attribute [rw] update_policy
        #   @return [::Google::Cloud::Compute::V1::InstanceGroupManagerUpdatePolicy]
        #     The update policy for this managed instance group.
        # @!attribute [rw] versions
        #   @return [::Array<::Google::Cloud::Compute::V1::InstanceGroupManagerVersion>]
        #     Specifies the instance templates used by this managed instance group to create instances. Each version is defined by an instanceTemplate and a name. Every version can appear at most once per instance group. This field overrides the top-level instanceTemplate field. Read more about the relationships between these fields. Exactly one version must leave the targetSize field unset. That version will be applied to all remaining instances. For more information, read about canary updates.
        # @!attribute [rw] zone
        #   @return [::String]
        #     [Output Only] The URL of a zone where the managed instance group is located (for zonal resources).
        class InstanceGroupManager
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Pagination behavior of the listManagedInstances API method for this managed instance group.
          module ListManagedInstancesResults
            # A value indicating that the enum field is not set.
            UNDEFINED_LIST_MANAGED_INSTANCES_RESULTS = 0

            # (Default) Pagination is disabled for the group's listManagedInstances API method. maxResults and pageToken query parameters are ignored and all instances are returned in a single response.
            PAGELESS = 32_183_464

            # Pagination is enabled for the group's listManagedInstances API method. maxResults and pageToken query parameters are respected.
            PAGINATED = 40_190_637
          end
        end

        # @!attribute [rw] abandoning
        #   @return [::Integer]
        #     [Output Only] The total number of instances in the managed instance group that are scheduled to be abandoned. Abandoning an instance removes it from the managed instance group without deleting it.
        # @!attribute [rw] creating
        #   @return [::Integer]
        #     [Output Only] The number of instances in the managed instance group that are scheduled to be created or are currently being created. If the group fails to create any of these instances, it tries again until it creates the instance successfully. If you have disabled creation retries, this field will not be populated; instead, the creatingWithoutRetries field will be populated.
        # @!attribute [rw] creating_without_retries
        #   @return [::Integer]
        #     [Output Only] The number of instances that the managed instance group will attempt to create. The group attempts to create each instance only once. If the group fails to create any of these instances, it decreases the group's targetSize value accordingly.
        # @!attribute [rw] deleting
        #   @return [::Integer]
        #     [Output Only] The number of instances in the managed instance group that are scheduled to be deleted or are currently being deleted.
        # @!attribute [rw] none
        #   @return [::Integer]
        #     [Output Only] The number of instances in the managed instance group that are running and have no scheduled actions.
        # @!attribute [rw] recreating
        #   @return [::Integer]
        #     [Output Only] The number of instances in the managed instance group that are scheduled to be recreated or are currently being being recreated. Recreating an instance deletes the existing root persistent disk and creates a new disk from the image that is defined in the instance template.
        # @!attribute [rw] refreshing
        #   @return [::Integer]
        #     [Output Only] The number of instances in the managed instance group that are being reconfigured with properties that do not require a restart or a recreate action. For example, setting or removing target pools for the instance.
        # @!attribute [rw] restarting
        #   @return [::Integer]
        #     [Output Only] The number of instances in the managed instance group that are scheduled to be restarted or are currently being restarted.
        # @!attribute [rw] resuming
        #   @return [::Integer]
        #     [Output Only] The number of instances in the managed instance group that are scheduled to be resumed or are currently being resumed.
        # @!attribute [rw] starting
        #   @return [::Integer]
        #     [Output Only] The number of instances in the managed instance group that are scheduled to be started or are currently being started.
        # @!attribute [rw] stopping
        #   @return [::Integer]
        #     [Output Only] The number of instances in the managed instance group that are scheduled to be stopped or are currently being stopped.
        # @!attribute [rw] suspending
        #   @return [::Integer]
        #     [Output Only] The number of instances in the managed instance group that are scheduled to be suspended or are currently being suspended.
        # @!attribute [rw] verifying
        #   @return [::Integer]
        #     [Output Only] The number of instances in the managed instance group that are being verified. See the managedInstances[].currentAction property in the listManagedInstances method documentation.
        class InstanceGroupManagerActionsSummary
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::InstanceGroupManagersScopedList}]
        #     A list of InstanceGroupManagersScopedList resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] The resource type, which is always compute#instanceGroupManagerAggregatedList for an aggregated list of managed instance groups.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class InstanceGroupManagerAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::InstanceGroupManagersScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # @!attribute [rw] health_check
        #   @return [::String]
        #     The URL for the health check that signals autohealing.
        # @!attribute [rw] initial_delay_sec
        #   @return [::Integer]
        #     The initial delay is the number of seconds that a new VM takes to initialize and run its startup script. During a VM's initial delay period, the MIG ignores unsuccessful health checks because the VM might be in the startup process. This prevents the MIG from prematurely recreating a VM. If the health check receives a healthy response during the initial delay, it indicates that the startup process is complete and the VM is ready. The value of initial delay must be between 0 and 3600 seconds. The default value is 0.
        class InstanceGroupManagerAutoHealingPolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] force_update_on_repair
        #   @return [::String]
        #     A bit indicating whether to forcefully apply the group's latest configuration when repairing a VM. Valid options are: - NO (default): If configuration updates are available, they are not forcefully applied during repair. Instead, configuration updates are applied according to the group's update policy. - YES: If configuration updates are available, they are applied during repair.
        #     Check the ForceUpdateOnRepair enum for the list of possible values.
        class InstanceGroupManagerInstanceLifecyclePolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # A bit indicating whether to forcefully apply the group's latest configuration when repairing a VM. Valid options are: - NO (default): If configuration updates are available, they are not forcefully applied during repair. Instead, configuration updates are applied according to the group's update policy. - YES: If configuration updates are available, they are applied during repair.
          module ForceUpdateOnRepair
            # A value indicating that the enum field is not set.
            UNDEFINED_FORCE_UPDATE_ON_REPAIR = 0

            NO = 2497

            YES = 87_751
          end
        end

        # [Output Only] A list of managed instance groups.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::InstanceGroupManager>]
        #     A list of InstanceGroupManager resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] The resource type, which is always compute#instanceGroupManagerList for a list of managed instance groups.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class InstanceGroupManagerList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] autoscaler
        #   @return [::String]
        #     [Output Only] The URL of the Autoscaler that targets this instance group manager.
        # @!attribute [rw] is_stable
        #   @return [::Boolean]
        #     [Output Only] A bit indicating whether the managed instance group is in a stable state. A stable state means that: none of the instances in the managed instance group is currently undergoing any type of change (for example, creation, restart, or deletion); no future changes are scheduled for instances in the managed instance group; and the managed instance group itself is not being modified.
        # @!attribute [rw] stateful
        #   @return [::Google::Cloud::Compute::V1::InstanceGroupManagerStatusStateful]
        #     [Output Only] Stateful status of the given Instance Group Manager.
        # @!attribute [rw] version_target
        #   @return [::Google::Cloud::Compute::V1::InstanceGroupManagerStatusVersionTarget]
        #     [Output Only] A status of consistency of Instances' versions with their target version specified by version field on Instance Group Manager.
        class InstanceGroupManagerStatus
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] has_stateful_config
        #   @return [::Boolean]
        #     [Output Only] A bit indicating whether the managed instance group has stateful configuration, that is, if you have configured any items in a stateful policy or in per-instance configs. The group might report that it has no stateful configuration even when there is still some preserved state on a managed instance, for example, if you have deleted all PICs but not yet applied those deletions.
        # @!attribute [rw] per_instance_configs
        #   @return [::Google::Cloud::Compute::V1::InstanceGroupManagerStatusStatefulPerInstanceConfigs]
        #     [Output Only] Status of per-instance configurations on the instance.
        class InstanceGroupManagerStatusStateful
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] all_effective
        #   @return [::Boolean]
        #     A bit indicating if all of the group's per-instance configurations (listed in the output of a listPerInstanceConfigs API call) have status EFFECTIVE or there are no per-instance-configs.
        class InstanceGroupManagerStatusStatefulPerInstanceConfigs
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] is_reached
        #   @return [::Boolean]
        #     [Output Only] A bit indicating whether version target has been reached in this managed instance group, i.e. all instances are in their target version. Instances' target version are specified by version field on Instance Group Manager.
        class InstanceGroupManagerStatusVersionTarget
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] instance_redistribution_type
        #   @return [::String]
        #     The instance redistribution policy for regional managed instance groups. Valid values are: - PROACTIVE (default): The group attempts to maintain an even distribution of VM instances across zones in the region. - NONE: For non-autoscaled groups, proactive redistribution is disabled.
        #     Check the InstanceRedistributionType enum for the list of possible values.
        # @!attribute [rw] max_surge
        #   @return [::Google::Cloud::Compute::V1::FixedOrPercent]
        #     The maximum number of instances that can be created above the specified targetSize during the update process. This value can be either a fixed number or, if the group has 10 or more instances, a percentage. If you set a percentage, the number of instances is rounded if necessary. The default value for maxSurge is a fixed value equal to the number of zones in which the managed instance group operates. At least one of either maxSurge or maxUnavailable must be greater than 0. Learn more about maxSurge.
        # @!attribute [rw] max_unavailable
        #   @return [::Google::Cloud::Compute::V1::FixedOrPercent]
        #     The maximum number of instances that can be unavailable during the update process. An instance is considered available if all of the following conditions are satisfied: - The instance's status is RUNNING. - If there is a health check on the instance group, the instance's health check status must be HEALTHY at least once. If there is no health check on the group, then the instance only needs to have a status of RUNNING to be considered available. This value can be either a fixed number or, if the group has 10 or more instances, a percentage. If you set a percentage, the number of instances is rounded if necessary. The default value for maxUnavailable is a fixed value equal to the number of zones in which the managed instance group operates. At least one of either maxSurge or maxUnavailable must be greater than 0. Learn more about maxUnavailable.
        # @!attribute [rw] minimal_action
        #   @return [::String]
        #     Minimal action to be taken on an instance. Use this option to minimize disruption as much as possible or to apply a more disruptive action than is necessary. - To limit disruption as much as possible, set the minimal action to REFRESH. If your update requires a more disruptive action, Compute Engine performs the necessary action to execute the update. - To apply a more disruptive action than is strictly necessary, set the minimal action to RESTART or REPLACE. For example, Compute Engine does not need to restart a VM to change its metadata. But if your application reads instance metadata only when a VM is restarted, you can set the minimal action to RESTART in order to pick up metadata changes.
        #     Check the MinimalAction enum for the list of possible values.
        # @!attribute [rw] most_disruptive_allowed_action
        #   @return [::String]
        #     Most disruptive action that is allowed to be taken on an instance. You can specify either NONE to forbid any actions, REFRESH to avoid restarting the VM and to limit disruption as much as possible. RESTART to allow actions that can be applied without instance replacing or REPLACE to allow all possible actions. If the Updater determines that the minimal update action needed is more disruptive than most disruptive allowed action you specify it will not perform the update at all.
        #     Check the MostDisruptiveAllowedAction enum for the list of possible values.
        # @!attribute [rw] replacement_method
        #   @return [::String]
        #     What action should be used to replace instances. See minimal_action.REPLACE
        #     Check the ReplacementMethod enum for the list of possible values.
        # @!attribute [rw] type
        #   @return [::String]
        #     The type of update process. You can specify either PROACTIVE so that the MIG automatically updates VMs to the latest configurations or OPPORTUNISTIC so that you can select the VMs that you want to update.
        #     Check the Type enum for the list of possible values.
        class InstanceGroupManagerUpdatePolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The instance redistribution policy for regional managed instance groups. Valid values are: - PROACTIVE (default): The group attempts to maintain an even distribution of VM instances across zones in the region. - NONE: For non-autoscaled groups, proactive redistribution is disabled.
          # Additional supported values which may be not listed in the enum directly due to technical reasons:
          # NONE
          # PROACTIVE
          module InstanceRedistributionType
            # A value indicating that the enum field is not set.
            UNDEFINED_INSTANCE_REDISTRIBUTION_TYPE = 0
          end

          # Minimal action to be taken on an instance. Use this option to minimize disruption as much as possible or to apply a more disruptive action than is necessary. - To limit disruption as much as possible, set the minimal action to REFRESH. If your update requires a more disruptive action, Compute Engine performs the necessary action to execute the update. - To apply a more disruptive action than is strictly necessary, set the minimal action to RESTART or REPLACE. For example, Compute Engine does not need to restart a VM to change its metadata. But if your application reads instance metadata only when a VM is restarted, you can set the minimal action to RESTART in order to pick up metadata changes.
          # Additional supported values which may be not listed in the enum directly due to technical reasons:
          # NONE
          # REFRESH
          # REPLACE
          # RESTART
          module MinimalAction
            # A value indicating that the enum field is not set.
            UNDEFINED_MINIMAL_ACTION = 0
          end

          # Most disruptive action that is allowed to be taken on an instance. You can specify either NONE to forbid any actions, REFRESH to avoid restarting the VM and to limit disruption as much as possible. RESTART to allow actions that can be applied without instance replacing or REPLACE to allow all possible actions. If the Updater determines that the minimal update action needed is more disruptive than most disruptive allowed action you specify it will not perform the update at all.
          # Additional supported values which may be not listed in the enum directly due to technical reasons:
          # NONE
          # REFRESH
          # REPLACE
          # RESTART
          module MostDisruptiveAllowedAction
            # A value indicating that the enum field is not set.
            UNDEFINED_MOST_DISRUPTIVE_ALLOWED_ACTION = 0
          end

          # What action should be used to replace instances. See minimal_action.REPLACE
          module ReplacementMethod
            # A value indicating that the enum field is not set.
            UNDEFINED_REPLACEMENT_METHOD = 0

            # Instances will be recreated (with the same name)
            RECREATE = 522_644_719

            # Default option: instances will be deleted and created (with a new name)
            SUBSTITUTE = 280_924_314
          end

          # The type of update process. You can specify either PROACTIVE so that the MIG automatically updates VMs to the latest configurations or OPPORTUNISTIC so that you can select the VMs that you want to update.
          # Additional supported values which may be not listed in the enum directly due to technical reasons:
          # PROACTIVE
          module Type
            # A value indicating that the enum field is not set.
            UNDEFINED_TYPE = 0

            # MIG will apply new configurations to existing VMs only when you selectively target specific or all VMs to be updated.
            OPPORTUNISTIC = 429_530_089
          end
        end

        # @!attribute [rw] instance_template
        #   @return [::String]
        #     The URL of the instance template that is specified for this managed instance group. The group uses this template to create new instances in the managed instance group until the `targetSize` for this version is reached. The templates for existing instances in the group do not change unless you run recreateInstances, run applyUpdatesToInstances, or set the group's updatePolicy.type to PROACTIVE; in those cases, existing instances are updated until the `targetSize` for this version is reached.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the version. Unique among all versions in the scope of this managed instance group.
        # @!attribute [rw] target_size
        #   @return [::Google::Cloud::Compute::V1::FixedOrPercent]
        #     Specifies the intended number of instances to be created from the instanceTemplate. The final number of instances created from the template will be equal to: - If expressed as a fixed number, the minimum of either targetSize.fixed or instanceGroupManager.targetSize is used. - if expressed as a percent, the targetSize would be (targetSize.percent/100 * InstanceGroupManager.targetSize) If there is a remainder, the number is rounded. If unset, this version will update any remaining instances not updated by another version. Read Starting a canary update for more information.
        class InstanceGroupManagerVersion
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] instances
        #   @return [::Array<::String>]
        #     The URLs of one or more instances to abandon. This can be a full URL or a partial URL, such as zones/[ZONE]/instances/[INSTANCE_NAME].
        class InstanceGroupManagersAbandonInstancesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # InstanceGroupManagers.applyUpdatesToInstances
        # @!attribute [rw] all_instances
        #   @return [::Boolean]
        #     Flag to update all instances instead of specified list of “instances”. If the flag is set to true then the instances may not be specified in the request.
        # @!attribute [rw] instances
        #   @return [::Array<::String>]
        #     The list of URLs of one or more instances for which you want to apply updates. Each URL can be a full URL or a partial URL, such as zones/[ZONE]/instances/[INSTANCE_NAME].
        # @!attribute [rw] minimal_action
        #   @return [::String]
        #     The minimal action that you want to perform on each instance during the update: - REPLACE: At minimum, delete the instance and create it again. - RESTART: Stop the instance and start it again. - REFRESH: Do not stop the instance and limit disruption as much as possible. - NONE: Do not disrupt the instance at all. By default, the minimum action is NONE. If your update requires a more disruptive action than you set with this flag, the necessary action is performed to execute the update.
        #     Check the MinimalAction enum for the list of possible values.
        # @!attribute [rw] most_disruptive_allowed_action
        #   @return [::String]
        #     The most disruptive action that you want to perform on each instance during the update: - REPLACE: Delete the instance and create it again. - RESTART: Stop the instance and start it again. - REFRESH: Do not stop the instance and limit disruption as much as possible. - NONE: Do not disrupt the instance at all. By default, the most disruptive allowed action is REPLACE. If your update requires a more disruptive action than you set with this flag, the update request will fail.
        #     Check the MostDisruptiveAllowedAction enum for the list of possible values.
        class InstanceGroupManagersApplyUpdatesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The minimal action that you want to perform on each instance during the update: - REPLACE: At minimum, delete the instance and create it again. - RESTART: Stop the instance and start it again. - REFRESH: Do not stop the instance and limit disruption as much as possible. - NONE: Do not disrupt the instance at all. By default, the minimum action is NONE. If your update requires a more disruptive action than you set with this flag, the necessary action is performed to execute the update.
          # Additional supported values which may be not listed in the enum directly due to technical reasons:
          # NONE
          # REFRESH
          # REPLACE
          # RESTART
          module MinimalAction
            # A value indicating that the enum field is not set.
            UNDEFINED_MINIMAL_ACTION = 0
          end

          # The most disruptive action that you want to perform on each instance during the update: - REPLACE: Delete the instance and create it again. - RESTART: Stop the instance and start it again. - REFRESH: Do not stop the instance and limit disruption as much as possible. - NONE: Do not disrupt the instance at all. By default, the most disruptive allowed action is REPLACE. If your update requires a more disruptive action than you set with this flag, the update request will fail.
          # Additional supported values which may be not listed in the enum directly due to technical reasons:
          # NONE
          # REFRESH
          # REPLACE
          # RESTART
          module MostDisruptiveAllowedAction
            # A value indicating that the enum field is not set.
            UNDEFINED_MOST_DISRUPTIVE_ALLOWED_ACTION = 0
          end
        end

        # InstanceGroupManagers.createInstances
        # @!attribute [rw] instances
        #   @return [::Array<::Google::Cloud::Compute::V1::PerInstanceConfig>]
        #     [Required] List of specifications of per-instance configs.
        class InstanceGroupManagersCreateInstancesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] instances
        #   @return [::Array<::String>]
        #     The URLs of one or more instances to delete. This can be a full URL or a partial URL, such as zones/[ZONE]/instances/[INSTANCE_NAME]. Queued instances do not have URL and can be deleted only by name. One cannot specify both URLs and names in a single request.
        # @!attribute [rw] skip_instances_on_validation_error
        #   @return [::Boolean]
        #     Specifies whether the request should proceed despite the inclusion of instances that are not members of the group or that are already in the process of being deleted or abandoned. If this field is set to `false` and such an instance is specified in the request, the operation fails. The operation always fails if the request contains a malformed instance URL or a reference to an instance that exists in a zone or region other than the group's zone or region.
        class InstanceGroupManagersDeleteInstancesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # InstanceGroupManagers.deletePerInstanceConfigs
        # @!attribute [rw] names
        #   @return [::Array<::String>]
        #     The list of instance names for which we want to delete per-instance configs on this managed instance group.
        class InstanceGroupManagersDeletePerInstanceConfigsReq
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::InstanceManagedByIgmError>]
        #     [Output Only] The list of errors of the managed instance group.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        class InstanceGroupManagersListErrorsResponse
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] managed_instances
        #   @return [::Array<::Google::Cloud::Compute::V1::ManagedInstance>]
        #     [Output Only] The list of instances in the managed instance group.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        class InstanceGroupManagersListManagedInstancesResponse
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::PerInstanceConfig>]
        #     [Output Only] The list of PerInstanceConfig.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class InstanceGroupManagersListPerInstanceConfigsResp
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # InstanceGroupManagers.patchPerInstanceConfigs
        # @!attribute [rw] per_instance_configs
        #   @return [::Array<::Google::Cloud::Compute::V1::PerInstanceConfig>]
        #     The list of per-instance configurations to insert or patch on this managed instance group.
        class InstanceGroupManagersPatchPerInstanceConfigsReq
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] instances
        #   @return [::Array<::String>]
        #     The URLs of one or more instances to recreate. This can be a full URL or a partial URL, such as zones/[ZONE]/instances/[INSTANCE_NAME].
        class InstanceGroupManagersRecreateInstancesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] instance_group_managers
        #   @return [::Array<::Google::Cloud::Compute::V1::InstanceGroupManager>]
        #     [Output Only] The list of managed instance groups that are contained in the specified project and zone.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] The warning that replaces the list of managed instance groups when the list is empty.
        class InstanceGroupManagersScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] instance_template
        #   @return [::String]
        #     The URL of the instance template that is specified for this managed instance group. The group uses this template to create all new instances in the managed instance group. The templates for existing instances in the group do not change unless you run recreateInstances, run applyUpdatesToInstances, or set the group's updatePolicy.type to PROACTIVE.
        class InstanceGroupManagersSetInstanceTemplateRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] fingerprint
        #   @return [::String]
        #     The fingerprint of the target pools information. Use this optional property to prevent conflicts when multiple users change the target pools settings concurrently. Obtain the fingerprint with the instanceGroupManagers.get method. Then, include the fingerprint in your request to ensure that you do not overwrite changes that were applied from another concurrent request.
        # @!attribute [rw] target_pools
        #   @return [::Array<::String>]
        #     The list of target pool URLs that instances in this managed instance group belong to. The managed instance group applies these target pools to all of the instances in the group. Existing instances and new instances in the group all receive these target pool settings.
        class InstanceGroupManagersSetTargetPoolsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # InstanceGroupManagers.updatePerInstanceConfigs
        # @!attribute [rw] per_instance_configs
        #   @return [::Array<::Google::Cloud::Compute::V1::PerInstanceConfig>]
        #     The list of per-instance configurations to insert or patch on this managed instance group.
        class InstanceGroupManagersUpdatePerInstanceConfigsReq
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] instances
        #   @return [::Array<::Google::Cloud::Compute::V1::InstanceReference>]
        #     The list of instances to add to the instance group.
        class InstanceGroupsAddInstancesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::InstanceWithNamedPorts>]
        #     A list of InstanceWithNamedPorts resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] The resource type, which is always compute#instanceGroupsListInstances for the list of instances in the specified instance group.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class InstanceGroupsListInstances
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] instance_state
        #   @return [::String]
        #     A filter for the state of the instances in the instance group. Valid options are ALL or RUNNING. If you do not specify this parameter the list includes all instances regardless of their state.
        #     Check the InstanceState enum for the list of possible values.
        class InstanceGroupsListInstancesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # A filter for the state of the instances in the instance group. Valid options are ALL or RUNNING. If you do not specify this parameter the list includes all instances regardless of their state.
          module InstanceState
            # A value indicating that the enum field is not set.
            UNDEFINED_INSTANCE_STATE = 0

            # Includes all instances in the generated list regardless of their state.
            ALL = 64_897

            # Includes instances in the generated list only if they have a RUNNING state.
            RUNNING = 121_282_975
          end
        end

        # @!attribute [rw] instances
        #   @return [::Array<::Google::Cloud::Compute::V1::InstanceReference>]
        #     The list of instances to remove from the instance group.
        class InstanceGroupsRemoveInstancesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] instance_groups
        #   @return [::Array<::Google::Cloud::Compute::V1::InstanceGroup>]
        #     [Output Only] The list of instance groups that are contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] An informational warning that replaces the list of instance groups when the list is empty.
        class InstanceGroupsScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] fingerprint
        #   @return [::String]
        #     The fingerprint of the named ports information for this instance group. Use this optional property to prevent conflicts when multiple users change the named ports settings concurrently. Obtain the fingerprint with the instanceGroups.get method. Then, include the fingerprint in your request to ensure that you do not overwrite changes that were applied from another concurrent request. A request with an incorrect fingerprint will fail with error 412 conditionNotMet.
        # @!attribute [rw] named_ports
        #   @return [::Array<::Google::Cloud::Compute::V1::NamedPort>]
        #     The list of named ports to set for this instance group.
        class InstanceGroupsSetNamedPortsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Contains a list of instances.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::Instance>]
        #     A list of Instance resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#instanceList for lists of Instance resources.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class InstanceList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Contains a list of instance referrers.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::Reference>]
        #     A list of Reference resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#instanceListReferrers for lists of Instance referrers.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class InstanceListReferrers
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] error
        #   @return [::Google::Cloud::Compute::V1::InstanceManagedByIgmErrorManagedInstanceError]
        #     [Output Only] Contents of the error.
        # @!attribute [rw] instance_action_details
        #   @return [::Google::Cloud::Compute::V1::InstanceManagedByIgmErrorInstanceActionDetails]
        #     [Output Only] Details of the instance action that triggered this error. May be null, if the error was not caused by an action on an instance. This field is optional.
        # @!attribute [rw] timestamp
        #   @return [::String]
        #     [Output Only] The time that this error occurred. This value is in RFC3339 text format.
        class InstanceManagedByIgmError
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] action
        #   @return [::String]
        #     [Output Only] Action that managed instance group was executing on the instance when the error occurred. Possible values:
        #     Check the Action enum for the list of possible values.
        # @!attribute [rw] instance
        #   @return [::String]
        #     [Output Only] The URL of the instance. The URL can be set even if the instance has not yet been created.
        # @!attribute [rw] version
        #   @return [::Google::Cloud::Compute::V1::ManagedInstanceVersion]
        #     [Output Only] Version this instance was created from, or was being created from, but the creation failed. Corresponds to one of the versions that were set on the Instance Group Manager resource at the time this instance was being created.
        class InstanceManagedByIgmErrorInstanceActionDetails
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # [Output Only] Action that managed instance group was executing on the instance when the error occurred. Possible values:
          module Action
            # A value indicating that the enum field is not set.
            UNDEFINED_ACTION = 0

            # The managed instance group is abandoning this instance. The instance will be removed from the instance group and from any target pools that are associated with this group.
            ABANDONING = 388_244_813

            # The managed instance group is creating this instance. If the group fails to create this instance, it will try again until it is successful.
            CREATING = 455_564_985

            # The managed instance group is attempting to create this instance only once. If the group fails to create this instance, it does not try again and the group's targetSize value is decreased.
            CREATING_WITHOUT_RETRIES = 428_843_785

            # The managed instance group is permanently deleting this instance.
            DELETING = 528_602_024

            # The managed instance group has not scheduled any actions for this instance.
            NONE = 2_402_104

            # The managed instance group is recreating this instance.
            RECREATING = 287_278_572

            # The managed instance group is applying configuration changes to the instance without stopping it. For example, the group can update the target pool list for an instance without stopping that instance.
            REFRESHING = 163_266_343

            # The managed instance group is restarting this instance.
            RESTARTING = 320_534_387

            # The managed instance group is resuming this instance.
            RESUMING = 446_856_618

            # The managed instance group is starting this instance.
            STARTING = 488_820_800

            # The managed instance group is stopping this instance.
            STOPPING = 350_791_796

            # The managed instance group is suspending this instance.
            SUSPENDING = 514_206_246

            # The managed instance group is verifying this already created instance. Verification happens every time the instance is (re)created or restarted and consists of: 1. Waiting until health check specified as part of this managed instance group's autohealing policy reports HEALTHY. Note: Applies only if autohealing policy has a health check specified 2. Waiting for addition verification steps performed as post-instance creation (subject to future extensions).
            VERIFYING = 16_982_185
          end
        end

        # @!attribute [rw] code
        #   @return [::String]
        #     [Output Only] Error code.
        # @!attribute [rw] message
        #   @return [::String]
        #     [Output Only] Error message.
        class InstanceManagedByIgmErrorManagedInstanceError
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] destination_zone
        #   @return [::String]
        #     The URL of the destination zone to move the instance. This can be a full or partial URL. For example, the following are all valid URLs to a zone: - https://www.googleapis.com/compute/v1/projects/project/zones/zone - projects/project/zones/zone - zones/zone
        # @!attribute [rw] target_instance
        #   @return [::String]
        #     The URL of the target instance to move. This can be a full or partial URL. For example, the following are all valid URLs to an instance: - https://www.googleapis.com/compute/v1/projects/project/zones/zone /instances/instance - projects/project/zones/zone/instances/instance - zones/zone/instances/instance
        class InstanceMoveRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Additional instance params.
        # @!attribute [rw] resource_manager_tags
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     Resource manager tags to be bound to the instance. Tag keys and values have the same definition as resource manager tags. Keys must be in the format `tagKeys/{tag_key_id}`, and values are in the format `tagValues/456`. The field is ignored (both PUT & PATCH) when empty.
        class InstanceParams
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class ResourceManagerTagsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # @!attribute [rw] advanced_machine_features
        #   @return [::Google::Cloud::Compute::V1::AdvancedMachineFeatures]
        #     Controls for advanced machine-related behavior features. Note that for MachineImage, this is not supported yet.
        # @!attribute [rw] can_ip_forward
        #   @return [::Boolean]
        #     Enables instances created based on these properties to send packets with source IP addresses other than their own and receive packets with destination IP addresses other than their own. If these instances will be used as an IP gateway or it will be set as the next-hop in a Route resource, specify true. If unsure, leave this set to false. See the Enable IP forwarding documentation for more information.
        # @!attribute [rw] confidential_instance_config
        #   @return [::Google::Cloud::Compute::V1::ConfidentialInstanceConfig]
        #     Specifies the Confidential Instance options. Note that for MachineImage, this is not supported yet.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional text description for the instances that are created from these properties.
        # @!attribute [rw] disks
        #   @return [::Array<::Google::Cloud::Compute::V1::AttachedDisk>]
        #     An array of disks that are associated with the instances that are created from these properties.
        # @!attribute [rw] guest_accelerators
        #   @return [::Array<::Google::Cloud::Compute::V1::AcceleratorConfig>]
        #     A list of guest accelerator cards' type and count to use for instances created from these properties.
        # @!attribute [rw] key_revocation_action_type
        #   @return [::String]
        #     KeyRevocationActionType of the instance. Supported options are "STOP" and "NONE". The default value is "NONE" if it is not specified.
        #     Check the KeyRevocationActionType enum for the list of possible values.
        # @!attribute [rw] labels
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     Labels to apply to instances that are created from these properties.
        # @!attribute [rw] machine_type
        #   @return [::String]
        #     The machine type to use for instances that are created from these properties.
        # @!attribute [rw] metadata
        #   @return [::Google::Cloud::Compute::V1::Metadata]
        #     The metadata key/value pairs to assign to instances that are created from these properties. These pairs can consist of custom metadata or predefined keys. See Project and instance metadata for more information.
        # @!attribute [rw] min_cpu_platform
        #   @return [::String]
        #     Minimum cpu/platform to be used by instances. The instance may be scheduled on the specified or newer cpu/platform. Applicable values are the friendly names of CPU platforms, such as minCpuPlatform: "Intel Haswell" or minCpuPlatform: "Intel Sandy Bridge". For more information, read Specifying a Minimum CPU Platform.
        # @!attribute [rw] network_interfaces
        #   @return [::Array<::Google::Cloud::Compute::V1::NetworkInterface>]
        #     An array of network access configurations for this interface.
        # @!attribute [rw] network_performance_config
        #   @return [::Google::Cloud::Compute::V1::NetworkPerformanceConfig]
        #     Note that for MachineImage, this is not supported yet.
        # @!attribute [rw] private_ipv6_google_access
        #   @return [::String]
        #     The private IPv6 google access type for VMs. If not specified, use INHERIT_FROM_SUBNETWORK as default. Note that for MachineImage, this is not supported yet.
        #     Check the PrivateIpv6GoogleAccess enum for the list of possible values.
        # @!attribute [rw] reservation_affinity
        #   @return [::Google::Cloud::Compute::V1::ReservationAffinity]
        #     Specifies the reservations that instances can consume from. Note that for MachineImage, this is not supported yet.
        # @!attribute [rw] resource_manager_tags
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     Resource manager tags to be bound to the instance. Tag keys and values have the same definition as resource manager tags. Keys must be in the format `tagKeys/{tag_key_id}`, and values are in the format `tagValues/456`. The field is ignored (both PUT & PATCH) when empty.
        # @!attribute [rw] resource_policies
        #   @return [::Array<::String>]
        #     Resource policies (names, not URLs) applied to instances created from these properties. Note that for MachineImage, this is not supported yet.
        # @!attribute [rw] scheduling
        #   @return [::Google::Cloud::Compute::V1::Scheduling]
        #     Specifies the scheduling options for the instances that are created from these properties.
        # @!attribute [rw] service_accounts
        #   @return [::Array<::Google::Cloud::Compute::V1::ServiceAccount>]
        #     A list of service accounts with specified scopes. Access tokens for these service accounts are available to the instances that are created from these properties. Use metadata queries to obtain the access tokens for these instances.
        # @!attribute [rw] shielded_instance_config
        #   @return [::Google::Cloud::Compute::V1::ShieldedInstanceConfig]
        #     Note that for MachineImage, this is not supported yet.
        # @!attribute [rw] tags
        #   @return [::Google::Cloud::Compute::V1::Tags]
        #     A list of tags to apply to the instances that are created from these properties. The tags identify valid sources or targets for network firewalls. The setTags method can modify this list of tags. Each tag within the list must comply with RFC1035.
        class InstanceProperties
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class LabelsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class ResourceManagerTagsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # KeyRevocationActionType of the instance. Supported options are "STOP" and "NONE". The default value is "NONE" if it is not specified.
          module KeyRevocationActionType
            # A value indicating that the enum field is not set.
            UNDEFINED_KEY_REVOCATION_ACTION_TYPE = 0

            # Default value. This value is unused.
            KEY_REVOCATION_ACTION_TYPE_UNSPECIFIED = 467_110_106

            # Indicates user chose no operation.
            NONE = 2_402_104

            # Indicates user chose to opt for VM shutdown on key revocation.
            STOP = 2_555_906
          end

          # The private IPv6 google access type for VMs. If not specified, use INHERIT_FROM_SUBNETWORK as default. Note that for MachineImage, this is not supported yet.
          module PrivateIpv6GoogleAccess
            # A value indicating that the enum field is not set.
            UNDEFINED_PRIVATE_IPV6_GOOGLE_ACCESS = 0

            # Bidirectional private IPv6 access to/from Google services. If specified, the subnetwork who is attached to the instance's default network interface will be assigned an internal IPv6 prefix if it doesn't have before.
            ENABLE_BIDIRECTIONAL_ACCESS_TO_GOOGLE = 427_975_994

            # Outbound private IPv6 access from VMs in this subnet to Google services. If specified, the subnetwork who is attached to the instance's default network interface will be assigned an internal IPv6 prefix if it doesn't have before.
            ENABLE_OUTBOUND_VM_ACCESS_TO_GOOGLE = 288_210_263

            # Each network interface inherits PrivateIpv6GoogleAccess from its subnetwork.
            INHERIT_FROM_SUBNETWORK = 530_256_959
          end
        end

        # @!attribute [rw] instance
        #   @return [::String]
        #     The URL for a specific instance. @required compute.instancegroups.addInstances/removeInstances
        class InstanceReference
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents an Instance Template resource. Google Compute Engine has two Instance Template resources: * [Global](/compute/docs/reference/rest/v1/instanceTemplates) * [Regional](/compute/docs/reference/rest/v1/regionInstanceTemplates) You can reuse a global instance template in different regions whereas you can use a regional instance template in a specified region only. If you want to reduce cross-region dependency or achieve data residency, use a regional instance template. To create VMs, managed instance groups, and reservations, you can use either global or regional instance templates. For more information, read Instance Templates.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] The creation timestamp for this instance template in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] A unique identifier for this instance template. The server defines this identifier.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] The resource type, which is always compute#instanceTemplate for instance templates.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource; provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] properties
        #   @return [::Google::Cloud::Compute::V1::InstanceProperties]
        #     The instance properties for this instance template.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] URL of the region where the instance template resides. Only applicable for regional resources.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] The URL for this instance template. The server defines this URL.
        # @!attribute [rw] source_instance
        #   @return [::String]
        #     The source instance used to create the template. You can provide this as a partial or full URL to the resource. For example, the following are valid values: - https://www.googleapis.com/compute/v1/projects/project/zones/zone /instances/instance - projects/project/zones/zone/instances/instance
        # @!attribute [rw] source_instance_params
        #   @return [::Google::Cloud::Compute::V1::SourceInstanceParams]
        #     The source instance params to use to create this instance template.
        class InstanceTemplate
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Contains a list of InstanceTemplatesScopedList.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::InstanceTemplatesScopedList}]
        #     A list of InstanceTemplatesScopedList resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     Type of resource.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class InstanceTemplateAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::InstanceTemplatesScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # A list of instance templates.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::InstanceTemplate>]
        #     A list of InstanceTemplate resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] The resource type, which is always compute#instanceTemplatesListResponse for instance template lists.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class InstanceTemplateList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] instance_templates
        #   @return [::Array<::Google::Cloud::Compute::V1::InstanceTemplate>]
        #     [Output Only] A list of instance templates that are contained within the specified project and zone.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] An informational warning that replaces the list of instance templates when the list is empty.
        class InstanceTemplatesScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] instance
        #   @return [::String]
        #     [Output Only] The URL of the instance.
        # @!attribute [rw] named_ports
        #   @return [::Array<::Google::Cloud::Compute::V1::NamedPort>]
        #     [Output Only] The named ports that belong to this instance group.
        # @!attribute [rw] status
        #   @return [::String]
        #     [Output Only] The status of the instance.
        #     Check the Status enum for the list of possible values.
        class InstanceWithNamedPorts
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # [Output Only] The status of the instance.
          module Status
            # A value indicating that the enum field is not set.
            UNDEFINED_STATUS = 0

            # The instance is halted and we are performing tear down tasks like network deprogramming, releasing quota, IP, tearing down disks etc.
            DEPROVISIONING = 428_935_662

            # Resources are being allocated for the instance.
            PROVISIONING = 290_896_621

            # The instance is in repair.
            REPAIRING = 413_483_285

            # The instance is running.
            RUNNING = 121_282_975

            # All required resources have been allocated and the instance is being started.
            STAGING = 431_072_283

            # The instance has stopped successfully.
            STOPPED = 444_276_141

            # The instance is currently stopping (either being deleted or killed).
            STOPPING = 350_791_796

            # The instance has suspended.
            SUSPENDED = 51_223_995

            # The instance is suspending.
            SUSPENDING = 514_206_246

            # The instance has stopped (either by explicit action or underlying failure).
            TERMINATED = 250_018_339
          end
        end

        # @!attribute [rw] resource_policies
        #   @return [::Array<::String>]
        #     Resource policies to be added to this instance.
        class InstancesAddResourcePoliciesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] per_location_status
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::BulkInsertOperationStatus}]
        #     Status information per location (location name is key). Example key: zones/us-central1-a
        class InstancesBulkInsertOperationMetadata
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::BulkInsertOperationStatus]
          class PerLocationStatusEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # @!attribute [rw] firewall_policys
        #   @return [::Array<::Google::Cloud::Compute::V1::InstancesGetEffectiveFirewallsResponseEffectiveFirewallPolicy>]
        #     Effective firewalls from firewall policies.
        # @!attribute [rw] firewalls
        #   @return [::Array<::Google::Cloud::Compute::V1::Firewall>]
        #     Effective firewalls on the instance.
        class InstancesGetEffectiveFirewallsResponse
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] display_name
        #   @return [::String]
        #     [Output Only] Deprecated, please use short name instead. The display name of the firewall policy.
        # @!attribute [rw] name
        #   @return [::String]
        #     [Output Only] The name of the firewall policy.
        # @!attribute [rw] rules
        #   @return [::Array<::Google::Cloud::Compute::V1::FirewallPolicyRule>]
        #     The rules that apply to the network.
        # @!attribute [rw] short_name
        #   @return [::String]
        #     [Output Only] The short name of the firewall policy.
        # @!attribute [rw] type
        #   @return [::String]
        #     [Output Only] The type of the firewall policy. Can be one of HIERARCHY, NETWORK, NETWORK_REGIONAL.
        #     Check the Type enum for the list of possible values.
        class InstancesGetEffectiveFirewallsResponseEffectiveFirewallPolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # [Output Only] The type of the firewall policy. Can be one of HIERARCHY, NETWORK, NETWORK_REGIONAL.
          module Type
            # A value indicating that the enum field is not set.
            UNDEFINED_TYPE = 0

            HIERARCHY = 69_902_869

            NETWORK = 413_984_270

            NETWORK_REGIONAL = 190_804_272

            UNSPECIFIED = 526_786_327
          end
        end

        # @!attribute [rw] resource_policies
        #   @return [::Array<::String>]
        #     Resource policies to be removed from this instance.
        class InstancesRemoveResourcePoliciesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] instances
        #   @return [::Array<::Google::Cloud::Compute::V1::Instance>]
        #     [Output Only] A list of instances contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning which replaces the list of instances when the list is empty.
        class InstancesScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] label_fingerprint
        #   @return [::String]
        #     Fingerprint of the previous set of labels for this resource, used to prevent conflicts. Provide the latest fingerprint value when making a request to add or change labels.
        # @!attribute [rw] labels
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        class InstancesSetLabelsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class LabelsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # @!attribute [rw] guest_accelerators
        #   @return [::Array<::Google::Cloud::Compute::V1::AcceleratorConfig>]
        #     A list of the type and count of accelerator cards attached to the instance.
        class InstancesSetMachineResourcesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] machine_type
        #   @return [::String]
        #     Full or partial URL of the machine type resource. See Machine Types for a full list of machine types. For example: zones/us-central1-f/machineTypes/n1-standard-1
        class InstancesSetMachineTypeRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] min_cpu_platform
        #   @return [::String]
        #     Minimum cpu/platform this instance should be started at.
        class InstancesSetMinCpuPlatformRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] current_name
        #   @return [::String]
        #     The current name of this resource, used to prevent conflicts. Provide the latest name when making a request to change name.
        # @!attribute [rw] name
        #   @return [::String]
        #     The name to be applied to the instance. Needs to be RFC 1035 compliant.
        class InstancesSetNameRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] network_interfaces
        #   @return [::Array<::String>]
        #     The network interfaces that the security policy will be applied to. Network interfaces use the nicN naming format. You can only set a security policy for network interfaces with an access config.
        # @!attribute [rw] security_policy
        #   @return [::String]
        #     A full or partial URL to a security policy to add to this instance. If this field is set to an empty string it will remove the associated security policy.
        class InstancesSetSecurityPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] email
        #   @return [::String]
        #     Email address of the service account.
        # @!attribute [rw] scopes
        #   @return [::Array<::String>]
        #     The list of scopes to be made available for this service account.
        class InstancesSetServiceAccountRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] disks
        #   @return [::Array<::Google::Cloud::Compute::V1::CustomerEncryptionKeyProtectedDisk>]
        #     Array of disks associated with this instance that are protected with a customer-supplied encryption key. In order to start the instance, the disk url and its corresponding key must be provided. If the disk is not protected with a customer-supplied encryption key it should not be specified.
        class InstancesStartWithEncryptionKeyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # HttpRouteRuleMatch criteria for field values that must stay within the specified integer range.
        # @!attribute [rw] range_end
        #   @return [::Integer]
        #     The end of the range (exclusive) in signed long integer format.
        # @!attribute [rw] range_start
        #   @return [::Integer]
        #     The start of the range (inclusive) in signed long integer format.
        class Int64RangeMatch
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents an Interconnect resource. An Interconnect resource is a dedicated connection between the Google Cloud network and your on-premises network. For more information, read the Dedicated Interconnect Overview.
        # @!attribute [rw] admin_enabled
        #   @return [::Boolean]
        #     Administrative status of the interconnect. When this is set to true, the Interconnect is functional and can carry traffic. When set to false, no packets can be carried over the interconnect and no BGP routes are exchanged over it. By default, the status is set to true.
        # @!attribute [rw] available_features
        #   @return [::Array<::String>]
        #     [Output only] List of features available for this Interconnect connection, which can take one of the following values: - MACSEC If present then the Interconnect connection is provisioned on MACsec capable hardware ports. If not present then the Interconnect connection is provisioned on non-MACsec capable ports and MACsec isn't supported and enabling MACsec fails.
        #     Check the AvailableFeatures enum for the list of possible values.
        # @!attribute [rw] circuit_infos
        #   @return [::Array<::Google::Cloud::Compute::V1::InterconnectCircuitInfo>]
        #     [Output Only] A list of CircuitInfo objects, that describe the individual circuits in this LAG.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] customer_name
        #   @return [::String]
        #     Customer name, to put in the Letter of Authorization as the party authorized to request a crossconnect.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] expected_outages
        #   @return [::Array<::Google::Cloud::Compute::V1::InterconnectOutageNotification>]
        #     [Output Only] A list of outages expected for this Interconnect.
        # @!attribute [rw] google_ip_address
        #   @return [::String]
        #     [Output Only] IP address configured on the Google side of the Interconnect link. This can be used only for ping tests.
        # @!attribute [rw] google_reference_id
        #   @return [::String]
        #     [Output Only] Google reference ID to be used when raising support tickets with Google or otherwise to debug backend connectivity issues.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] interconnect_attachments
        #   @return [::Array<::String>]
        #     [Output Only] A list of the URLs of all InterconnectAttachments configured to use this Interconnect.
        # @!attribute [rw] interconnect_type
        #   @return [::String]
        #     Type of interconnect, which can take one of the following values: - PARTNER: A partner-managed interconnection shared between customers though a partner. - DEDICATED: A dedicated physical interconnection with the customer. Note that a value IT_PRIVATE has been deprecated in favor of DEDICATED.
        #     Check the InterconnectType enum for the list of possible values.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#interconnect for interconnects.
        # @!attribute [rw] label_fingerprint
        #   @return [::String]
        #     A fingerprint for the labels being applied to this Interconnect, which is essentially a hash of the labels set used for optimistic locking. The fingerprint is initially generated by Compute Engine and changes after every request to modify or update labels. You must always provide an up-to-date fingerprint hash in order to update or change labels, otherwise the request will fail with error 412 conditionNotMet. To see the latest fingerprint, make a get() request to retrieve an Interconnect.
        # @!attribute [rw] labels
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     Labels for this resource. These can only be added or modified by the setLabels method. Each label key/value pair must comply with RFC1035. Label values may be empty.
        # @!attribute [rw] link_type
        #   @return [::String]
        #     Type of link requested, which can take one of the following values: - LINK_TYPE_ETHERNET_10G_LR: A 10G Ethernet with LR optics - LINK_TYPE_ETHERNET_100G_LR: A 100G Ethernet with LR optics. Note that this field indicates the speed of each of the links in the bundle, not the speed of the entire bundle.
        #     Check the LinkType enum for the list of possible values.
        # @!attribute [rw] location
        #   @return [::String]
        #     URL of the InterconnectLocation object that represents where this connection is to be provisioned.
        # @!attribute [rw] macsec
        #   @return [::Google::Cloud::Compute::V1::InterconnectMacsec]
        #     Configuration that enables Media Access Control security (MACsec) on the Cloud Interconnect connection between Google and your on-premises router.
        # @!attribute [rw] macsec_enabled
        #   @return [::Boolean]
        #     Enable or disable MACsec on this Interconnect connection. MACsec enablement fails if the MACsec object is not specified.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] noc_contact_email
        #   @return [::String]
        #     Email address to contact the customer NOC for operations and maintenance notifications regarding this Interconnect. If specified, this will be used for notifications in addition to all other forms described, such as Cloud Monitoring logs alerting and Cloud Notifications. This field is required for users who sign up for Cloud Interconnect using workforce identity federation.
        # @!attribute [rw] operational_status
        #   @return [::String]
        #     [Output Only] The current status of this Interconnect's functionality, which can take one of the following values: - OS_ACTIVE: A valid Interconnect, which is turned up and is ready to use. Attachments may be provisioned on this Interconnect. - OS_UNPROVISIONED: An Interconnect that has not completed turnup. No attachments may be provisioned on this Interconnect. - OS_UNDER_MAINTENANCE: An Interconnect that is undergoing internal maintenance. No attachments may be provisioned or updated on this Interconnect.
        #     Check the OperationalStatus enum for the list of possible values.
        # @!attribute [rw] peer_ip_address
        #   @return [::String]
        #     [Output Only] IP address configured on the customer side of the Interconnect link. The customer should configure this IP address during turnup when prompted by Google NOC. This can be used only for ping tests.
        # @!attribute [rw] provisioned_link_count
        #   @return [::Integer]
        #     [Output Only] Number of links actually provisioned in this interconnect.
        # @!attribute [rw] remote_location
        #   @return [::String]
        #     Indicates that this is a Cross-Cloud Interconnect. This field specifies the location outside of Google's network that the interconnect is connected to.
        # @!attribute [rw] requested_features
        #   @return [::Array<::String>]
        #     Optional. List of features requested for this Interconnect connection, which can take one of the following values: - MACSEC If specified then the connection is created on MACsec capable hardware ports. If not specified, the default value is false, which allocates non-MACsec capable ports first if available. This parameter can be provided only with Interconnect INSERT. It isn't valid for Interconnect PATCH.
        #     Check the RequestedFeatures enum for the list of possible values.
        # @!attribute [rw] requested_link_count
        #   @return [::Integer]
        #     Target number of physical links in the link bundle, as requested by the customer.
        # @!attribute [rw] satisfies_pzs
        #   @return [::Boolean]
        #     [Output Only] Reserved for future use.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] state
        #   @return [::String]
        #     [Output Only] The current state of Interconnect functionality, which can take one of the following values: - ACTIVE: The Interconnect is valid, turned up and ready to use. Attachments may be provisioned on this Interconnect. - UNPROVISIONED: The Interconnect has not completed turnup. No attachments may be provisioned on this Interconnect. - UNDER_MAINTENANCE: The Interconnect is undergoing internal maintenance. No attachments may be provisioned or updated on this Interconnect.
        #     Check the State enum for the list of possible values.
        class Interconnect
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class LabelsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # Additional supported values which may be not listed in the enum directly due to technical reasons:
          # IF_MACSEC
          module AvailableFeatures
            # A value indicating that the enum field is not set.
            UNDEFINED_AVAILABLE_FEATURES = 0
          end

          # Type of interconnect, which can take one of the following values: - PARTNER: A partner-managed interconnection shared between customers though a partner. - DEDICATED: A dedicated physical interconnection with the customer. Note that a value IT_PRIVATE has been deprecated in favor of DEDICATED.
          module InterconnectType
            # A value indicating that the enum field is not set.
            UNDEFINED_INTERCONNECT_TYPE = 0

            # A dedicated physical interconnection with the customer.
            DEDICATED = 258_411_983

            # [Deprecated] A private, physical interconnection with the customer.
            IT_PRIVATE = 335_677_007

            # A partner-managed interconnection shared between customers via partner.
            PARTNER = 461_924_520
          end

          # Type of link requested, which can take one of the following values: - LINK_TYPE_ETHERNET_10G_LR: A 10G Ethernet with LR optics - LINK_TYPE_ETHERNET_100G_LR: A 100G Ethernet with LR optics. Note that this field indicates the speed of each of the links in the bundle, not the speed of the entire bundle.
          module LinkType
            # A value indicating that the enum field is not set.
            UNDEFINED_LINK_TYPE = 0

            # 100G Ethernet, LR Optics.
            LINK_TYPE_ETHERNET_100G_LR = 337_672_551

            # 10G Ethernet, LR Optics. [(rate_bps) = 10000000000];
            LINK_TYPE_ETHERNET_10G_LR = 236_739_749
          end

          # [Output Only] The current status of this Interconnect's functionality, which can take one of the following values: - OS_ACTIVE: A valid Interconnect, which is turned up and is ready to use. Attachments may be provisioned on this Interconnect. - OS_UNPROVISIONED: An Interconnect that has not completed turnup. No attachments may be provisioned on this Interconnect. - OS_UNDER_MAINTENANCE: An Interconnect that is undergoing internal maintenance. No attachments may be provisioned or updated on this Interconnect.
          module OperationalStatus
            # A value indicating that the enum field is not set.
            UNDEFINED_OPERATIONAL_STATUS = 0

            # The interconnect is valid, turned up, and ready to use. Attachments may be provisioned on this interconnect.
            OS_ACTIVE = 55_721_409

            # The interconnect has not completed turnup. No attachments may be provisioned on this interconnect.
            OS_UNPROVISIONED = 239_771_840
          end

          # Additional supported values which may be not listed in the enum directly due to technical reasons:
          # IF_MACSEC
          module RequestedFeatures
            # A value indicating that the enum field is not set.
            UNDEFINED_REQUESTED_FEATURES = 0
          end

          # [Output Only] The current state of Interconnect functionality, which can take one of the following values: - ACTIVE: The Interconnect is valid, turned up and ready to use. Attachments may be provisioned on this Interconnect. - UNPROVISIONED: The Interconnect has not completed turnup. No attachments may be provisioned on this Interconnect. - UNDER_MAINTENANCE: The Interconnect is undergoing internal maintenance. No attachments may be provisioned or updated on this Interconnect.
          module State
            # A value indicating that the enum field is not set.
            UNDEFINED_STATE = 0

            # The interconnect is valid, turned up, and ready to use. Attachments may be provisioned on this interconnect.
            ACTIVE = 314_733_318

            # The interconnect has not completed turnup. No attachments may be provisioned on this interconnect.
            UNPROVISIONED = 517_333_979
          end
        end

        # Represents an Interconnect Attachment (VLAN) resource. You can use Interconnect attachments (VLANS) to connect your Virtual Private Cloud networks to your on-premises networks through an Interconnect. For more information, read Creating VLAN Attachments.
        # @!attribute [rw] admin_enabled
        #   @return [::Boolean]
        #     Determines whether this Attachment will carry packets. Not present for PARTNER_PROVIDER.
        # @!attribute [rw] bandwidth
        #   @return [::String]
        #     Provisioned bandwidth capacity for the interconnect attachment. For attachments of type DEDICATED, the user can set the bandwidth. For attachments of type PARTNER, the Google Partner that is operating the interconnect must set the bandwidth. Output only for PARTNER type, mutable for PARTNER_PROVIDER and DEDICATED, and can take one of the following values: - BPS_50M: 50 Mbit/s - BPS_100M: 100 Mbit/s - BPS_200M: 200 Mbit/s - BPS_300M: 300 Mbit/s - BPS_400M: 400 Mbit/s - BPS_500M: 500 Mbit/s - BPS_1G: 1 Gbit/s - BPS_2G: 2 Gbit/s - BPS_5G: 5 Gbit/s - BPS_10G: 10 Gbit/s - BPS_20G: 20 Gbit/s - BPS_50G: 50 Gbit/s
        #     Check the Bandwidth enum for the list of possible values.
        # @!attribute [rw] candidate_ipv6_subnets
        #   @return [::Array<::String>]
        #     This field is not available.
        # @!attribute [rw] candidate_subnets
        #   @return [::Array<::String>]
        #     Up to 16 candidate prefixes that can be used to restrict the allocation of cloudRouterIpAddress and customerRouterIpAddress for this attachment. All prefixes must be within link-local address space (169.254.0.0/16) and must be /29 or shorter (/28, /27, etc). Google will attempt to select an unused /29 from the supplied candidate prefix(es). The request will fail if all possible /29s are in use on Google's edge. If not supplied, Google will randomly select an unused /29 from all of link-local space.
        # @!attribute [rw] cloud_router_ip_address
        #   @return [::String]
        #     [Output Only] IPv4 address + prefix length to be configured on Cloud Router Interface for this interconnect attachment.
        # @!attribute [rw] cloud_router_ipv6_address
        #   @return [::String]
        #     [Output Only] IPv6 address + prefix length to be configured on Cloud Router Interface for this interconnect attachment.
        # @!attribute [rw] cloud_router_ipv6_interface_id
        #   @return [::String]
        #     This field is not available.
        # @!attribute [rw] configuration_constraints
        #   @return [::Google::Cloud::Compute::V1::InterconnectAttachmentConfigurationConstraints]
        #     [Output Only] Constraints for this attachment, if any. The attachment does not work if these constraints are not met.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] customer_router_ip_address
        #   @return [::String]
        #     [Output Only] IPv4 address + prefix length to be configured on the customer router subinterface for this interconnect attachment.
        # @!attribute [rw] customer_router_ipv6_address
        #   @return [::String]
        #     [Output Only] IPv6 address + prefix length to be configured on the customer router subinterface for this interconnect attachment.
        # @!attribute [rw] customer_router_ipv6_interface_id
        #   @return [::String]
        #     This field is not available.
        # @!attribute [rw] dataplane_version
        #   @return [::Integer]
        #     [Output Only] Dataplane version for this InterconnectAttachment. This field is only present for Dataplane version 2 and higher. Absence of this field in the API output indicates that the Dataplane is version 1.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource.
        # @!attribute [rw] edge_availability_domain
        #   @return [::String]
        #     Desired availability domain for the attachment. Only available for type PARTNER, at creation time, and can take one of the following values: - AVAILABILITY_DOMAIN_ANY - AVAILABILITY_DOMAIN_1 - AVAILABILITY_DOMAIN_2 For improved reliability, customers should configure a pair of attachments, one per availability domain. The selected availability domain will be provided to the Partner via the pairing key, so that the provisioned circuit will lie in the specified domain. If not specified, the value will default to AVAILABILITY_DOMAIN_ANY.
        #     Check the EdgeAvailabilityDomain enum for the list of possible values.
        # @!attribute [rw] encryption
        #   @return [::String]
        #     Indicates the user-supplied encryption option of this VLAN attachment (interconnectAttachment). Can only be specified at attachment creation for PARTNER or DEDICATED attachments. Possible values are: - NONE - This is the default value, which means that the VLAN attachment carries unencrypted traffic. VMs are able to send traffic to, or receive traffic from, such a VLAN attachment. - IPSEC - The VLAN attachment carries only encrypted traffic that is encrypted by an IPsec device, such as an HA VPN gateway or third-party IPsec VPN. VMs cannot directly send traffic to, or receive traffic from, such a VLAN attachment. To use *HA VPN over Cloud Interconnect*, the VLAN attachment must be created with this option.
        #     Check the Encryption enum for the list of possible values.
        # @!attribute [rw] google_reference_id
        #   @return [::String]
        #     [Output Only] Google reference ID, to be used when raising support tickets with Google or otherwise to debug backend connectivity issues. [Deprecated] This field is not used.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] interconnect
        #   @return [::String]
        #     URL of the underlying Interconnect object that this attachment's traffic will traverse through.
        # @!attribute [rw] ipsec_internal_addresses
        #   @return [::Array<::String>]
        #     A list of URLs of addresses that have been reserved for the VLAN attachment. Used only for the VLAN attachment that has the encryption option as IPSEC. The addresses must be regional internal IP address ranges. When creating an HA VPN gateway over the VLAN attachment, if the attachment is configured to use a regional internal IP address, then the VPN gateway's IP address is allocated from the IP address range specified here. For example, if the HA VPN gateway's interface 0 is paired to this VLAN attachment, then a regional internal IP address for the VPN gateway interface 0 will be allocated from the IP address specified for this VLAN attachment. If this field is not specified when creating the VLAN attachment, then later on when creating an HA VPN gateway on this VLAN attachment, the HA VPN gateway's IP address is allocated from the regional external IP address pool.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#interconnectAttachment for interconnect attachments.
        # @!attribute [rw] label_fingerprint
        #   @return [::String]
        #     A fingerprint for the labels being applied to this InterconnectAttachment, which is essentially a hash of the labels set used for optimistic locking. The fingerprint is initially generated by Compute Engine and changes after every request to modify or update labels. You must always provide an up-to-date fingerprint hash in order to update or change labels, otherwise the request will fail with error 412 conditionNotMet. To see the latest fingerprint, make a get() request to retrieve an InterconnectAttachment.
        # @!attribute [rw] labels
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     Labels for this resource. These can only be added or modified by the setLabels method. Each label key/value pair must comply with RFC1035. Label values may be empty.
        # @!attribute [rw] mtu
        #   @return [::Integer]
        #     Maximum Transmission Unit (MTU), in bytes, of packets passing through this interconnect attachment. Only 1440 and 1500 are allowed. If not specified, the value will default to 1440.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] operational_status
        #   @return [::String]
        #     [Output Only] The current status of whether or not this interconnect attachment is functional, which can take one of the following values: - OS_ACTIVE: The attachment has been turned up and is ready to use. - OS_UNPROVISIONED: The attachment is not ready to use yet, because turnup is not complete.
        #     Check the OperationalStatus enum for the list of possible values.
        # @!attribute [rw] pairing_key
        #   @return [::String]
        #     [Output only for type PARTNER. Input only for PARTNER_PROVIDER. Not present for DEDICATED]. The opaque identifier of a PARTNER attachment used to initiate provisioning with a selected partner. Of the form "XXXXX/region/domain"
        # @!attribute [rw] partner_asn
        #   @return [::Integer]
        #     Optional BGP ASN for the router supplied by a Layer 3 Partner if they configured BGP on behalf of the customer. Output only for PARTNER type, input only for PARTNER_PROVIDER, not available for DEDICATED.
        # @!attribute [rw] partner_metadata
        #   @return [::Google::Cloud::Compute::V1::InterconnectAttachmentPartnerMetadata]
        #     Informational metadata about Partner attachments from Partners to display to customers. Output only for PARTNER type, mutable for PARTNER_PROVIDER, not available for DEDICATED.
        # @!attribute [rw] private_interconnect_info
        #   @return [::Google::Cloud::Compute::V1::InterconnectAttachmentPrivateInfo]
        #     [Output Only] Information specific to an InterconnectAttachment. This property is populated if the interconnect that this is attached to is of type DEDICATED.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] URL of the region where the regional interconnect attachment resides. You must specify this field as part of the HTTP request URL. It is not settable as a field in the request body.
        # @!attribute [rw] remote_service
        #   @return [::String]
        #     [Output Only] If the attachment is on a Cross-Cloud Interconnect connection, this field contains the interconnect's remote location service provider. Example values: "Amazon Web Services" "Microsoft Azure". The field is set only for attachments on Cross-Cloud Interconnect connections. Its value is copied from the InterconnectRemoteLocation remoteService field.
        # @!attribute [rw] router
        #   @return [::String]
        #     URL of the Cloud Router to be used for dynamic routing. This router must be in the same region as this InterconnectAttachment. The InterconnectAttachment will automatically connect the Interconnect to the network & region within which the Cloud Router is configured.
        # @!attribute [rw] satisfies_pzs
        #   @return [::Boolean]
        #     [Output Only] Reserved for future use.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] stack_type
        #   @return [::String]
        #     The stack type for this interconnect attachment to identify whether the IPv6 feature is enabled or not. If not specified, IPV4_ONLY will be used. This field can be both set at interconnect attachments creation and update interconnect attachment operations.
        #     Check the StackType enum for the list of possible values.
        # @!attribute [rw] state
        #   @return [::String]
        #     [Output Only] The current state of this attachment's functionality. Enum values ACTIVE and UNPROVISIONED are shared by DEDICATED/PRIVATE, PARTNER, and PARTNER_PROVIDER interconnect attachments, while enum values PENDING_PARTNER, PARTNER_REQUEST_RECEIVED, and PENDING_CUSTOMER are used for only PARTNER and PARTNER_PROVIDER interconnect attachments. This state can take one of the following values: - ACTIVE: The attachment has been turned up and is ready to use. - UNPROVISIONED: The attachment is not ready to use yet, because turnup is not complete. - PENDING_PARTNER: A newly-created PARTNER attachment that has not yet been configured on the Partner side. - PARTNER_REQUEST_RECEIVED: A PARTNER attachment is in the process of provisioning after a PARTNER_PROVIDER attachment was created that references it. - PENDING_CUSTOMER: A PARTNER or PARTNER_PROVIDER attachment that is waiting for a customer to activate it. - DEFUNCT: The attachment was deleted externally and is no longer functional. This could be because the associated Interconnect was removed, or because the other side of a Partner attachment was deleted.
        #     Check the State enum for the list of possible values.
        # @!attribute [rw] subnet_length
        #   @return [::Integer]
        #     Length of the IPv4 subnet mask. Allowed values: - 29 (default) - 30 The default value is 29, except for Cross-Cloud Interconnect connections that use an InterconnectRemoteLocation with a constraints.subnetLengthRange.min equal to 30. For example, connections that use an Azure remote location fall into this category. In these cases, the default value is 30, and requesting 29 returns an error. Where both 29 and 30 are allowed, 29 is preferred, because it gives Google Cloud Support more debugging visibility.
        # @!attribute [rw] type
        #   @return [::String]
        #     The type of interconnect attachment this is, which can take one of the following values: - DEDICATED: an attachment to a Dedicated Interconnect. - PARTNER: an attachment to a Partner Interconnect, created by the customer. - PARTNER_PROVIDER: an attachment to a Partner Interconnect, created by the partner.
        #     Check the Type enum for the list of possible values.
        # @!attribute [rw] vlan_tag8021q
        #   @return [::Integer]
        #     The IEEE 802.1Q VLAN tag for this attachment, in the range 2-4093. Only specified at creation time.
        class InterconnectAttachment
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class LabelsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # Provisioned bandwidth capacity for the interconnect attachment. For attachments of type DEDICATED, the user can set the bandwidth. For attachments of type PARTNER, the Google Partner that is operating the interconnect must set the bandwidth. Output only for PARTNER type, mutable for PARTNER_PROVIDER and DEDICATED, and can take one of the following values: - BPS_50M: 50 Mbit/s - BPS_100M: 100 Mbit/s - BPS_200M: 200 Mbit/s - BPS_300M: 300 Mbit/s - BPS_400M: 400 Mbit/s - BPS_500M: 500 Mbit/s - BPS_1G: 1 Gbit/s - BPS_2G: 2 Gbit/s - BPS_5G: 5 Gbit/s - BPS_10G: 10 Gbit/s - BPS_20G: 20 Gbit/s - BPS_50G: 50 Gbit/s
          module Bandwidth
            # A value indicating that the enum field is not set.
            UNDEFINED_BANDWIDTH = 0

            # 100 Mbit/s
            BPS_100M = 49_547_958

            # 10 Gbit/s
            BPS_10G = 278_693_006

            # 1 Gbit/s
            BPS_1G = 355_358_448

            # 200 Mbit/s
            BPS_200M = 49_577_749

            # 20 Gbit/s
            BPS_20G = 278_693_967

            # 2 Gbit/s
            BPS_2G = 355_358_479

            # 300 Mbit/s
            BPS_300M = 49_607_540

            # 400 Mbit/s
            BPS_400M = 49_637_331

            # 500 Mbit/s
            BPS_500M = 49_667_122

            # 50 Gbit/s
            BPS_50G = 278_696_850

            # 50 Mbit/s
            BPS_50M = 278_696_856

            # 5 Gbit/s
            BPS_5G = 355_358_572
          end

          # Desired availability domain for the attachment. Only available for type PARTNER, at creation time, and can take one of the following values: - AVAILABILITY_DOMAIN_ANY - AVAILABILITY_DOMAIN_1 - AVAILABILITY_DOMAIN_2 For improved reliability, customers should configure a pair of attachments, one per availability domain. The selected availability domain will be provided to the Partner via the pairing key, so that the provisioned circuit will lie in the specified domain. If not specified, the value will default to AVAILABILITY_DOMAIN_ANY.
          module EdgeAvailabilityDomain
            # A value indicating that the enum field is not set.
            UNDEFINED_EDGE_AVAILABILITY_DOMAIN = 0

            AVAILABILITY_DOMAIN_1 = 349_552_090

            AVAILABILITY_DOMAIN_2 = 349_552_091

            AVAILABILITY_DOMAIN_ANY = 375_256_373
          end

          # Indicates the user-supplied encryption option of this VLAN attachment (interconnectAttachment). Can only be specified at attachment creation for PARTNER or DEDICATED attachments. Possible values are: - NONE - This is the default value, which means that the VLAN attachment carries unencrypted traffic. VMs are able to send traffic to, or receive traffic from, such a VLAN attachment. - IPSEC - The VLAN attachment carries only encrypted traffic that is encrypted by an IPsec device, such as an HA VPN gateway or third-party IPsec VPN. VMs cannot directly send traffic to, or receive traffic from, such a VLAN attachment. To use *HA VPN over Cloud Interconnect*, the VLAN attachment must be created with this option.
          module Encryption
            # A value indicating that the enum field is not set.
            UNDEFINED_ENCRYPTION = 0

            # The interconnect attachment will carry only encrypted traffic that is encrypted by an IPsec device such as HA VPN gateway; VMs cannot directly send traffic to or receive traffic from such an interconnect attachment. To use HA VPN over Cloud Interconnect, the interconnect attachment must be created with this option.
            IPSEC = 69_882_282

            # This is the default value, which means the Interconnect Attachment will carry unencrypted traffic. VMs will be able to send traffic to or receive traffic from such interconnect attachment.
            NONE = 2_402_104
          end

          # [Output Only] The current status of whether or not this interconnect attachment is functional, which can take one of the following values: - OS_ACTIVE: The attachment has been turned up and is ready to use. - OS_UNPROVISIONED: The attachment is not ready to use yet, because turnup is not complete.
          module OperationalStatus
            # A value indicating that the enum field is not set.
            UNDEFINED_OPERATIONAL_STATUS = 0

            # Indicates that attachment has been turned up and is ready to use.
            OS_ACTIVE = 55_721_409

            # Indicates that attachment is not ready to use yet, because turnup is not complete.
            OS_UNPROVISIONED = 239_771_840
          end

          # The stack type for this interconnect attachment to identify whether the IPv6 feature is enabled or not. If not specified, IPV4_ONLY will be used. This field can be both set at interconnect attachments creation and update interconnect attachment operations.
          module StackType
            # A value indicating that the enum field is not set.
            UNDEFINED_STACK_TYPE = 0

            # The interconnect attachment can have both IPv4 and IPv6 addresses.
            IPV4_IPV6 = 22_197_249

            # The interconnect attachment will only be assigned IPv4 addresses.
            IPV4_ONLY = 22_373_798
          end

          # [Output Only] The current state of this attachment's functionality. Enum values ACTIVE and UNPROVISIONED are shared by DEDICATED/PRIVATE, PARTNER, and PARTNER_PROVIDER interconnect attachments, while enum values PENDING_PARTNER, PARTNER_REQUEST_RECEIVED, and PENDING_CUSTOMER are used for only PARTNER and PARTNER_PROVIDER interconnect attachments. This state can take one of the following values: - ACTIVE: The attachment has been turned up and is ready to use. - UNPROVISIONED: The attachment is not ready to use yet, because turnup is not complete. - PENDING_PARTNER: A newly-created PARTNER attachment that has not yet been configured on the Partner side. - PARTNER_REQUEST_RECEIVED: A PARTNER attachment is in the process of provisioning after a PARTNER_PROVIDER attachment was created that references it. - PENDING_CUSTOMER: A PARTNER or PARTNER_PROVIDER attachment that is waiting for a customer to activate it. - DEFUNCT: The attachment was deleted externally and is no longer functional. This could be because the associated Interconnect was removed, or because the other side of a Partner attachment was deleted.
          module State
            # A value indicating that the enum field is not set.
            UNDEFINED_STATE = 0

            # Indicates that attachment has been turned up and is ready to use.
            ACTIVE = 314_733_318

            # The attachment was deleted externally and is no longer functional. This could be because the associated Interconnect was wiped out, or because the other side of a Partner attachment was deleted.
            DEFUNCT = 115_891_759

            # A PARTNER attachment is in the process of provisioning after a PARTNER_PROVIDER attachment was created that references it.
            PARTNER_REQUEST_RECEIVED = 513_587_304

            # PARTNER or PARTNER_PROVIDER attachment that is waiting for the customer to activate.
            PENDING_CUSTOMER = 167_494_054

            # A newly created PARTNER attachment that has not yet been configured on the Partner side.
            PENDING_PARTNER = 387_890_656

            STATE_UNSPECIFIED = 470_755_401

            # Indicates that attachment is not ready to use yet, because turnup is not complete.
            UNPROVISIONED = 517_333_979
          end

          # The type of interconnect attachment this is, which can take one of the following values: - DEDICATED: an attachment to a Dedicated Interconnect. - PARTNER: an attachment to a Partner Interconnect, created by the customer. - PARTNER_PROVIDER: an attachment to a Partner Interconnect, created by the partner.
          module Type
            # A value indicating that the enum field is not set.
            UNDEFINED_TYPE = 0

            # Attachment to a dedicated interconnect.
            DEDICATED = 258_411_983

            # Attachment to a partner interconnect, created by the customer.
            PARTNER = 461_924_520

            # Attachment to a partner interconnect, created by the partner.
            PARTNER_PROVIDER = 483_261_352
          end
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::InterconnectAttachmentsScopedList}]
        #     A list of InterconnectAttachmentsScopedList resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#interconnectAttachmentAggregatedList for aggregated lists of interconnect attachments.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class InterconnectAttachmentAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::InterconnectAttachmentsScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # @!attribute [rw] bgp_md5
        #   @return [::String]
        #     [Output Only] Whether the attachment's BGP session requires/allows/disallows BGP MD5 authentication. This can take one of the following values: MD5_OPTIONAL, MD5_REQUIRED, MD5_UNSUPPORTED. For example, a Cross-Cloud Interconnect connection to a remote cloud provider that requires BGP MD5 authentication has the interconnectRemoteLocation attachment_configuration_constraints.bgp_md5 field set to MD5_REQUIRED, and that property is propagated to the attachment. Similarly, if BGP MD5 is MD5_UNSUPPORTED, an error is returned if MD5 is requested.
        #     Check the BgpMd5 enum for the list of possible values.
        # @!attribute [rw] bgp_peer_asn_ranges
        #   @return [::Array<::Google::Cloud::Compute::V1::InterconnectAttachmentConfigurationConstraintsBgpPeerASNRange>]
        #     [Output Only] List of ASN ranges that the remote location is known to support. Formatted as an array of inclusive ranges \\{min: min-value, max: max-value}. For example, [\\{min: 123, max: 123}, \\{min: 64512, max: 65534}] allows the peer ASN to be 123 or anything in the range 64512-65534. This field is only advisory. Although the API accepts other ranges, these are the ranges that we recommend.
        class InterconnectAttachmentConfigurationConstraints
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # [Output Only] Whether the attachment's BGP session requires/allows/disallows BGP MD5 authentication. This can take one of the following values: MD5_OPTIONAL, MD5_REQUIRED, MD5_UNSUPPORTED. For example, a Cross-Cloud Interconnect connection to a remote cloud provider that requires BGP MD5 authentication has the interconnectRemoteLocation attachment_configuration_constraints.bgp_md5 field set to MD5_REQUIRED, and that property is propagated to the attachment. Similarly, if BGP MD5 is MD5_UNSUPPORTED, an error is returned if MD5 is requested.
          module BgpMd5
            # A value indicating that the enum field is not set.
            UNDEFINED_BGP_MD5 = 0

            # MD5_OPTIONAL: BGP MD5 authentication is supported and can optionally be configured.
            MD5_OPTIONAL = 532_156_673

            # MD5_REQUIRED: BGP MD5 authentication must be configured.
            MD5_REQUIRED = 218_034_496

            # MD5_UNSUPPORTED: BGP MD5 authentication must not be configured
            MD5_UNSUPPORTED = 86_962_388
          end
        end

        # @!attribute [rw] max
        #   @return [::Integer]
        # @!attribute [rw] min
        #   @return [::Integer]
        class InterconnectAttachmentConfigurationConstraintsBgpPeerASNRange
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Response to the list request, and contains a list of interconnect attachments.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::InterconnectAttachment>]
        #     A list of InterconnectAttachment resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#interconnectAttachmentList for lists of interconnect attachments.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class InterconnectAttachmentList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Informational metadata about Partner attachments from Partners to display to customers. These fields are propagated from PARTNER_PROVIDER attachments to their corresponding PARTNER attachments.
        # @!attribute [rw] interconnect_name
        #   @return [::String]
        #     Plain text name of the Interconnect this attachment is connected to, as displayed in the Partner's portal. For instance "Chicago 1". This value may be validated to match approved Partner values.
        # @!attribute [rw] partner_name
        #   @return [::String]
        #     Plain text name of the Partner providing this attachment. This value may be validated to match approved Partner values.
        # @!attribute [rw] portal_url
        #   @return [::String]
        #     URL of the Partner's portal for this Attachment. Partners may customise this to be a deep link to the specific resource on the Partner portal. This value may be validated to match approved Partner values.
        class InterconnectAttachmentPartnerMetadata
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Information for an interconnect attachment when this belongs to an interconnect of type DEDICATED.
        # @!attribute [rw] tag8021q
        #   @return [::Integer]
        #     [Output Only] 802.1q encapsulation tag to be used for traffic between Google and the customer, going to and from this network and region.
        class InterconnectAttachmentPrivateInfo
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] interconnect_attachments
        #   @return [::Array<::Google::Cloud::Compute::V1::InterconnectAttachment>]
        #     A list of interconnect attachments contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     Informational warning which replaces the list of addresses when the list is empty.
        class InterconnectAttachmentsScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Describes a single physical circuit between the Customer and Google. CircuitInfo objects are created by Google, so all fields are output only.
        # @!attribute [rw] customer_demarc_id
        #   @return [::String]
        #     Customer-side demarc ID for this circuit.
        # @!attribute [rw] google_circuit_id
        #   @return [::String]
        #     Google-assigned unique ID for this circuit. Assigned at circuit turn-up.
        # @!attribute [rw] google_demarc_id
        #   @return [::String]
        #     Google-side demarc ID for this circuit. Assigned at circuit turn-up and provided by Google to the customer in the LOA.
        class InterconnectCircuitInfo
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Diagnostics information about the Interconnect connection, which contains detailed and current technical information about Google's side of the connection.
        # @!attribute [rw] arp_caches
        #   @return [::Array<::Google::Cloud::Compute::V1::InterconnectDiagnosticsARPEntry>]
        #     A list of InterconnectDiagnostics.ARPEntry objects, describing individual neighbors currently seen by the Google router in the ARP cache for the Interconnect. This will be empty when the Interconnect is not bundled.
        # @!attribute [rw] bundle_aggregation_type
        #   @return [::String]
        #     The aggregation type of the bundle interface.
        #     Check the BundleAggregationType enum for the list of possible values.
        # @!attribute [rw] bundle_operational_status
        #   @return [::String]
        #     The operational status of the bundle interface.
        #     Check the BundleOperationalStatus enum for the list of possible values.
        # @!attribute [rw] links
        #   @return [::Array<::Google::Cloud::Compute::V1::InterconnectDiagnosticsLinkStatus>]
        #     A list of InterconnectDiagnostics.LinkStatus objects, describing the status for each link on the Interconnect.
        # @!attribute [rw] mac_address
        #   @return [::String]
        #     The MAC address of the Interconnect's bundle interface.
        class InterconnectDiagnostics
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The aggregation type of the bundle interface.
          module BundleAggregationType
            # A value indicating that the enum field is not set.
            UNDEFINED_BUNDLE_AGGREGATION_TYPE = 0

            # LACP is enabled.
            BUNDLE_AGGREGATION_TYPE_LACP = 27_758_925

            # LACP is disabled.
            BUNDLE_AGGREGATION_TYPE_STATIC = 50_678_873
          end

          # The operational status of the bundle interface.
          module BundleOperationalStatus
            # A value indicating that the enum field is not set.
            UNDEFINED_BUNDLE_OPERATIONAL_STATUS = 0

            # If bundleAggregationType is LACP: LACP is not established and/or all links in the bundle have DOWN operational status. If bundleAggregationType is STATIC: one or more links in the bundle has DOWN operational status.
            BUNDLE_OPERATIONAL_STATUS_DOWN = 453_842_693

            # If bundleAggregationType is LACP: LACP is established and at least one link in the bundle has UP operational status. If bundleAggregationType is STATIC: all links in the bundle (typically just one) have UP operational status.
            BUNDLE_OPERATIONAL_STATUS_UP = 161_366_462
          end
        end

        # Describing the ARP neighbor entries seen on this link
        # @!attribute [rw] ip_address
        #   @return [::String]
        #     The IP address of this ARP neighbor.
        # @!attribute [rw] mac_address
        #   @return [::String]
        #     The MAC address of this ARP neighbor.
        class InterconnectDiagnosticsARPEntry
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] google_system_id
        #   @return [::String]
        #     System ID of the port on Google's side of the LACP exchange.
        # @!attribute [rw] neighbor_system_id
        #   @return [::String]
        #     System ID of the port on the neighbor's side of the LACP exchange.
        # @!attribute [rw] state
        #   @return [::String]
        #     The state of a LACP link, which can take one of the following values: - ACTIVE: The link is configured and active within the bundle. - DETACHED: The link is not configured within the bundle. This means that the rest of the object should be empty.
        #     Check the State enum for the list of possible values.
        class InterconnectDiagnosticsLinkLACPStatus
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The state of a LACP link, which can take one of the following values: - ACTIVE: The link is configured and active within the bundle. - DETACHED: The link is not configured within the bundle. This means that the rest of the object should be empty.
          module State
            # A value indicating that the enum field is not set.
            UNDEFINED_STATE = 0

            # The link is configured and active within the bundle.
            ACTIVE = 314_733_318

            # The link is not configured within the bundle, this means the rest of the object should be empty.
            DETACHED = 216_562_546
          end
        end

        # @!attribute [rw] state
        #   @return [::String]
        #     The status of the current value when compared to the warning and alarm levels for the receiving or transmitting transceiver. Possible states include: - OK: The value has not crossed a warning threshold. - LOW_WARNING: The value has crossed below the low warning threshold. - HIGH_WARNING: The value has crossed above the high warning threshold. - LOW_ALARM: The value has crossed below the low alarm threshold. - HIGH_ALARM: The value has crossed above the high alarm threshold.
        #     Check the State enum for the list of possible values.
        # @!attribute [rw] value
        #   @return [::Float]
        #     Value of the current receiving or transmitting optical power, read in dBm. Take a known good optical value, give it a 10% margin and trigger warnings relative to that value. In general, a -7dBm warning and a -11dBm alarm are good optical value estimates for most links.
        class InterconnectDiagnosticsLinkOpticalPower
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The status of the current value when compared to the warning and alarm levels for the receiving or transmitting transceiver. Possible states include: - OK: The value has not crossed a warning threshold. - LOW_WARNING: The value has crossed below the low warning threshold. - HIGH_WARNING: The value has crossed above the high warning threshold. - LOW_ALARM: The value has crossed below the low alarm threshold. - HIGH_ALARM: The value has crossed above the high alarm threshold.
          module State
            # A value indicating that the enum field is not set.
            UNDEFINED_STATE = 0

            # The value has crossed above the high alarm threshold.
            HIGH_ALARM = 305_363_284

            # The value of the current optical power has crossed above the high warning threshold.
            HIGH_WARNING = 220_984_799

            # The value of the current optical power has crossed below the low alarm threshold.
            LOW_ALARM = 316_659_046

            # The value of the current optical power has crossed below the low warning threshold.
            LOW_WARNING = 338_793_841

            # The value of the current optical power has not crossed a warning threshold.
            OK = 2524
          end
        end

        # @!attribute [rw] arp_caches
        #   @return [::Array<::Google::Cloud::Compute::V1::InterconnectDiagnosticsARPEntry>]
        #     A list of InterconnectDiagnostics.ARPEntry objects, describing the ARP neighbor entries seen on this link. This will be empty if the link is bundled
        # @!attribute [rw] circuit_id
        #   @return [::String]
        #     The unique ID for this link assigned during turn up by Google.
        # @!attribute [rw] google_demarc
        #   @return [::String]
        #     The Demarc address assigned by Google and provided in the LoA.
        # @!attribute [rw] lacp_status
        #   @return [::Google::Cloud::Compute::V1::InterconnectDiagnosticsLinkLACPStatus]
        # @!attribute [rw] macsec
        #   @return [::Google::Cloud::Compute::V1::InterconnectDiagnosticsMacsecStatus]
        #     Describes the status of MACsec encryption on this link.
        # @!attribute [rw] operational_status
        #   @return [::String]
        #     The operational status of the link.
        #     Check the OperationalStatus enum for the list of possible values.
        # @!attribute [rw] receiving_optical_power
        #   @return [::Google::Cloud::Compute::V1::InterconnectDiagnosticsLinkOpticalPower]
        #     An InterconnectDiagnostics.LinkOpticalPower object, describing the current value and status of the received light level.
        # @!attribute [rw] transmitting_optical_power
        #   @return [::Google::Cloud::Compute::V1::InterconnectDiagnosticsLinkOpticalPower]
        #     An InterconnectDiagnostics.LinkOpticalPower object, describing the current value and status of the transmitted light level.
        class InterconnectDiagnosticsLinkStatus
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The operational status of the link.
          module OperationalStatus
            # A value indicating that the enum field is not set.
            UNDEFINED_OPERATIONAL_STATUS = 0

            # The interface is unable to communicate with the remote end.
            LINK_OPERATIONAL_STATUS_DOWN = 281_653_885

            # The interface has low level communication with the remote end.
            LINK_OPERATIONAL_STATUS_UP = 305_879_862
          end
        end

        # Describes the status of MACsec encryption on the link.
        # @!attribute [rw] ckn
        #   @return [::String]
        #     Indicates the Connectivity Association Key Name (CKN) currently being used if MACsec is operational.
        # @!attribute [rw] operational
        #   @return [::Boolean]
        #     Indicates whether or not MACsec is operational on this link.
        class InterconnectDiagnosticsMacsecStatus
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Response to the list request, and contains a list of interconnects.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::Interconnect>]
        #     A list of Interconnect resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#interconnectList for lists of interconnects.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class InterconnectList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents an Interconnect Attachment (VLAN) Location resource. You can use this resource to find location details about an Interconnect attachment (VLAN). For more information about interconnect attachments, read Creating VLAN Attachments.
        # @!attribute [rw] address
        #   @return [::String]
        #     [Output Only] The postal address of the Point of Presence, each line in the address is separated by a newline character.
        # @!attribute [rw] availability_zone
        #   @return [::String]
        #     [Output Only] Availability zone for this InterconnectLocation. Within a metropolitan area (metro), maintenance will not be simultaneously scheduled in more than one availability zone. Example: "zone1" or "zone2".
        # @!attribute [rw] available_features
        #   @return [::Array<::String>]
        #     [Output only] List of features available at this InterconnectLocation, which can take one of the following values: - MACSEC
        #     Check the AvailableFeatures enum for the list of possible values.
        # @!attribute [rw] available_link_types
        #   @return [::Array<::String>]
        #     [Output only] List of link types available at this InterconnectLocation, which can take one of the following values: - LINK_TYPE_ETHERNET_10G_LR - LINK_TYPE_ETHERNET_100G_LR
        #     Check the AvailableLinkTypes enum for the list of possible values.
        # @!attribute [rw] city
        #   @return [::String]
        #     [Output Only] Metropolitan area designator that indicates which city an interconnect is located. For example: "Chicago, IL", "Amsterdam, Netherlands".
        # @!attribute [rw] continent
        #   @return [::String]
        #     [Output Only] Continent for this location, which can take one of the following values: - AFRICA - ASIA_PAC - EUROPE - NORTH_AMERICA - SOUTH_AMERICA
        #     Check the Continent enum for the list of possible values.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     [Output Only] An optional description of the resource.
        # @!attribute [rw] facility_provider
        #   @return [::String]
        #     [Output Only] The name of the provider for this facility (e.g., EQUINIX).
        # @!attribute [rw] facility_provider_facility_id
        #   @return [::String]
        #     [Output Only] A provider-assigned Identifier for this facility (e.g., Ashburn-DC1).
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#interconnectLocation for interconnect locations.
        # @!attribute [rw] name
        #   @return [::String]
        #     [Output Only] Name of the resource.
        # @!attribute [rw] peeringdb_facility_id
        #   @return [::String]
        #     [Output Only] The peeringdb identifier for this facility (corresponding with a netfac type in peeringdb).
        # @!attribute [rw] region_infos
        #   @return [::Array<::Google::Cloud::Compute::V1::InterconnectLocationRegionInfo>]
        #     [Output Only] A list of InterconnectLocation.RegionInfo objects, that describe parameters pertaining to the relation between this InterconnectLocation and various Google Cloud regions.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] status
        #   @return [::String]
        #     [Output Only] The status of this InterconnectLocation, which can take one of the following values: - CLOSED: The InterconnectLocation is closed and is unavailable for provisioning new Interconnects. - AVAILABLE: The InterconnectLocation is available for provisioning new Interconnects.
        #     Check the Status enum for the list of possible values.
        # @!attribute [rw] supports_pzs
        #   @return [::Boolean]
        #     [Output Only] Reserved for future use.
        class InterconnectLocation
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods


          module AvailableFeatures
            # A value indicating that the enum field is not set.
            UNDEFINED_AVAILABLE_FEATURES = 0

            # Media Access Control security (MACsec)
            IF_MACSEC = 396_279_300
          end

          module AvailableLinkTypes
            # A value indicating that the enum field is not set.
            UNDEFINED_AVAILABLE_LINK_TYPES = 0

            # 100G Ethernet, LR Optics.
            LINK_TYPE_ETHERNET_100G_LR = 337_672_551

            # 10G Ethernet, LR Optics. [(rate_bps) = 10000000000];
            LINK_TYPE_ETHERNET_10G_LR = 236_739_749
          end

          # [Output Only] Continent for this location, which can take one of the following values: - AFRICA - ASIA_PAC - EUROPE - NORTH_AMERICA - SOUTH_AMERICA
          module Continent
            # A value indicating that the enum field is not set.
            UNDEFINED_CONTINENT = 0

            AFRICA = 317_443_706

            ASIA_PAC = 119_782_269

            C_AFRICA = 71_993_846

            C_ASIA_PAC = 465_668_089

            C_EUROPE = 200_369_438

            C_NORTH_AMERICA = 275_697_048

            C_SOUTH_AMERICA = 397_149_792

            EUROPE = 445_819_298

            NORTH_AMERICA = 448_015_508

            SOUTH_AMERICA = 32_597_340
          end

          # [Output Only] The status of this InterconnectLocation, which can take one of the following values: - CLOSED: The InterconnectLocation is closed and is unavailable for provisioning new Interconnects. - AVAILABLE: The InterconnectLocation is available for provisioning new Interconnects.
          module Status
            # A value indicating that the enum field is not set.
            UNDEFINED_STATUS = 0

            # The InterconnectLocation is available for provisioning new Interconnects.
            AVAILABLE = 442_079_913

            # The InterconnectLocation is closed for provisioning new Interconnects.
            CLOSED = 380_163_436
          end
        end

        # Response to the list request, and contains a list of interconnect locations.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::InterconnectLocation>]
        #     A list of InterconnectLocation resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#interconnectLocationList for lists of interconnect locations.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class InterconnectLocationList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Information about any potential InterconnectAttachments between an Interconnect at a specific InterconnectLocation, and a specific Cloud Region.
        # @!attribute [rw] expected_rtt_ms
        #   @return [::Integer]
        #     Expected round-trip time in milliseconds, from this InterconnectLocation to a VM in this region.
        # @!attribute [rw] location_presence
        #   @return [::String]
        #     Identifies the network presence of this location.
        #     Check the LocationPresence enum for the list of possible values.
        # @!attribute [rw] region
        #   @return [::String]
        #     URL for the region of this location.
        class InterconnectLocationRegionInfo
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Identifies the network presence of this location.
          module LocationPresence
            # A value indicating that the enum field is not set.
            UNDEFINED_LOCATION_PRESENCE = 0

            # This region is not in any common network presence with this InterconnectLocation.
            GLOBAL = 494_663_587

            # This region shares the same regional network presence as this InterconnectLocation.
            LOCAL_REGION = 403_535_464

            # [Deprecated] This region is not in any common network presence with this InterconnectLocation.
            LP_GLOBAL = 429_584_062

            # [Deprecated] This region shares the same regional network presence as this InterconnectLocation.
            LP_LOCAL_REGION = 488_598_851
          end
        end

        # Configuration information for enabling Media Access Control security (MACsec) on this Cloud Interconnect connection between Google and your on-premises router.
        # @!attribute [rw] fail_open
        #   @return [::Boolean]
        #     If set to true, the Interconnect connection is configured with a should-secure MACsec security policy, that allows the Google router to fallback to cleartext traffic if the MKA session cannot be established. By default, the Interconnect connection is configured with a must-secure security policy that drops all traffic if the MKA session cannot be established with your router.
        # @!attribute [rw] pre_shared_keys
        #   @return [::Array<::Google::Cloud::Compute::V1::InterconnectMacsecPreSharedKey>]
        #     Required. A keychain placeholder describing a set of named key objects along with their start times. A MACsec CKN/CAK is generated for each key in the key chain. Google router automatically picks the key with the most recent startTime when establishing or re-establishing a MACsec secure link.
        class InterconnectMacsec
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # MACsec configuration information for the Interconnect connection. Contains the generated Connectivity Association Key Name (CKN) and the key (CAK) for this Interconnect connection.
        # @!attribute [rw] pre_shared_keys
        #   @return [::Array<::Google::Cloud::Compute::V1::InterconnectMacsecConfigPreSharedKey>]
        #     A keychain placeholder describing a set of named key objects along with their start times. A MACsec CKN/CAK is generated for each key in the key chain. Google router automatically picks the key with the most recent startTime when establishing or re-establishing a MACsec secure link.
        class InterconnectMacsecConfig
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Describes a pre-shared key used to setup MACsec in static connectivity association key (CAK) mode.
        # @!attribute [rw] cak
        #   @return [::String]
        #     An auto-generated Connectivity Association Key (CAK) for this key.
        # @!attribute [rw] ckn
        #   @return [::String]
        #     An auto-generated Connectivity Association Key Name (CKN) for this key.
        # @!attribute [rw] name
        #   @return [::String]
        #     User provided name for this pre-shared key.
        # @!attribute [rw] start_time
        #   @return [::String]
        #     User provided timestamp on or after which this key is valid.
        class InterconnectMacsecConfigPreSharedKey
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Describes a pre-shared key used to setup MACsec in static connectivity association key (CAK) mode.
        # @!attribute [rw] name
        #   @return [::String]
        #     Required. A name for this pre-shared key. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] start_time
        #   @return [::String]
        #     A RFC3339 timestamp on or after which the key is valid. startTime can be in the future. If the keychain has a single key, startTime can be omitted. If the keychain has multiple keys, startTime is mandatory for each key. The start times of keys must be in increasing order. The start times of two consecutive keys must be at least 6 hours apart.
        class InterconnectMacsecPreSharedKey
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Description of a planned outage on this Interconnect.
        # @!attribute [rw] affected_circuits
        #   @return [::Array<::String>]
        #     If issue_type is IT_PARTIAL_OUTAGE, a list of the Google-side circuit IDs that will be affected.
        # @!attribute [rw] description
        #   @return [::String]
        #     A description about the purpose of the outage.
        # @!attribute [rw] end_time
        #   @return [::Integer]
        #     Scheduled end time for the outage (milliseconds since Unix epoch).
        # @!attribute [rw] issue_type
        #   @return [::String]
        #     Form this outage is expected to take, which can take one of the following values: - OUTAGE: The Interconnect may be completely out of service for some or all of the specified window. - PARTIAL_OUTAGE: Some circuits comprising the Interconnect as a whole should remain up, but with reduced bandwidth. Note that the versions of this enum prefixed with "IT_" have been deprecated in favor of the unprefixed values.
        #     Check the IssueType enum for the list of possible values.
        # @!attribute [rw] name
        #   @return [::String]
        #     Unique identifier for this outage notification.
        # @!attribute [rw] source
        #   @return [::String]
        #     The party that generated this notification, which can take the following value: - GOOGLE: this notification as generated by Google. Note that the value of NSRC_GOOGLE has been deprecated in favor of GOOGLE.
        #     Check the Source enum for the list of possible values.
        # @!attribute [rw] start_time
        #   @return [::Integer]
        #     Scheduled start time for the outage (milliseconds since Unix epoch).
        # @!attribute [rw] state
        #   @return [::String]
        #     State of this notification, which can take one of the following values: - ACTIVE: This outage notification is active. The event could be in the past, present, or future. See start_time and end_time for scheduling. - CANCELLED: The outage associated with this notification was cancelled before the outage was due to start. - COMPLETED: The outage associated with this notification is complete. Note that the versions of this enum prefixed with "NS_" have been deprecated in favor of the unprefixed values.
        #     Check the State enum for the list of possible values.
        class InterconnectOutageNotification
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Form this outage is expected to take, which can take one of the following values: - OUTAGE: The Interconnect may be completely out of service for some or all of the specified window. - PARTIAL_OUTAGE: Some circuits comprising the Interconnect as a whole should remain up, but with reduced bandwidth. Note that the versions of this enum prefixed with "IT_" have been deprecated in favor of the unprefixed values.
          module IssueType
            # A value indicating that the enum field is not set.
            UNDEFINED_ISSUE_TYPE = 0

            # [Deprecated] The Interconnect may be completely out of service for some or all of the specified window.
            IT_OUTAGE = 175_779_973

            # [Deprecated] Some circuits comprising the Interconnect will be out of service during the expected window. The interconnect as a whole should remain up, albeit with reduced bandwidth.
            IT_PARTIAL_OUTAGE = 92_103_971

            # The Interconnect may be completely out of service for some or all of the specified window.
            OUTAGE = 195_285_745

            # Some circuits comprising the Interconnect will be out of service during the expected window. The interconnect as a whole should remain up, albeit with reduced bandwidth.
            PARTIAL_OUTAGE = 147_053_455
          end

          # The party that generated this notification, which can take the following value: - GOOGLE: this notification as generated by Google. Note that the value of NSRC_GOOGLE has been deprecated in favor of GOOGLE.
          module Source
            # A value indicating that the enum field is not set.
            UNDEFINED_SOURCE = 0

            # This notification was generated by Google.
            GOOGLE = 497_439_289

            # [Deprecated] This notification was generated by Google.
            NSRC_GOOGLE = 510_574_562
          end

          # State of this notification, which can take one of the following values: - ACTIVE: This outage notification is active. The event could be in the past, present, or future. See start_time and end_time for scheduling. - CANCELLED: The outage associated with this notification was cancelled before the outage was due to start. - COMPLETED: The outage associated with this notification is complete. Note that the versions of this enum prefixed with "NS_" have been deprecated in favor of the unprefixed values.
          module State
            # A value indicating that the enum field is not set.
            UNDEFINED_STATE = 0

            # This outage notification is active. The event could be in the future, present, or past. See start_time and end_time for scheduling.
            ACTIVE = 314_733_318

            # The outage associated with this notification was cancelled before the outage was due to start.
            CANCELLED = 41_957_681

            # The outage associated with this notification is complete.
            COMPLETED = 309_921_323

            # [Deprecated] This outage notification is active. The event could be in the future, present, or past. See start_time and end_time for scheduling.
            NS_ACTIVE = 252_563_136

            # [Deprecated] The outage associated with this notification was canceled before the outage was due to start.
            NS_CANCELED = 506_579_411
          end
        end

        # Represents a Cross-Cloud Interconnect Remote Location resource. You can use this resource to find remote location details about an Interconnect attachment (VLAN).
        # @!attribute [rw] address
        #   @return [::String]
        #     [Output Only] The postal address of the Point of Presence, each line in the address is separated by a newline character.
        # @!attribute [rw] attachment_configuration_constraints
        #   @return [::Google::Cloud::Compute::V1::InterconnectAttachmentConfigurationConstraints]
        #     [Output Only] Subset of fields from InterconnectAttachment's |configurationConstraints| field that apply to all attachments for this remote location.
        # @!attribute [rw] city
        #   @return [::String]
        #     [Output Only] Metropolitan area designator that indicates which city an interconnect is located. For example: "Chicago, IL", "Amsterdam, Netherlands".
        # @!attribute [rw] constraints
        #   @return [::Google::Cloud::Compute::V1::InterconnectRemoteLocationConstraints]
        #     [Output Only] Constraints on the parameters for creating Cross-Cloud Interconnect and associated InterconnectAttachments.
        # @!attribute [rw] continent
        #   @return [::String]
        #     [Output Only] Continent for this location, which can take one of the following values: - AFRICA - ASIA_PAC - EUROPE - NORTH_AMERICA - SOUTH_AMERICA
        #     Check the Continent enum for the list of possible values.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     [Output Only] An optional description of the resource.
        # @!attribute [rw] facility_provider
        #   @return [::String]
        #     [Output Only] The name of the provider for this facility (e.g., EQUINIX).
        # @!attribute [rw] facility_provider_facility_id
        #   @return [::String]
        #     [Output Only] A provider-assigned Identifier for this facility (e.g., Ashburn-DC1).
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#interconnectRemoteLocation for interconnect remote locations.
        # @!attribute [rw] lacp
        #   @return [::String]
        #     [Output Only] Link Aggregation Control Protocol (LACP) constraints, which can take one of the following values: LACP_SUPPORTED, LACP_UNSUPPORTED
        #     Check the Lacp enum for the list of possible values.
        # @!attribute [rw] max_lag_size100_gbps
        #   @return [::Integer]
        #     [Output Only] The maximum number of 100 Gbps ports supported in a link aggregation group (LAG). When linkType is 100 Gbps, requestedLinkCount cannot exceed max_lag_size_100_gbps.
        # @!attribute [rw] max_lag_size10_gbps
        #   @return [::Integer]
        #     [Output Only] The maximum number of 10 Gbps ports supported in a link aggregation group (LAG). When linkType is 10 Gbps, requestedLinkCount cannot exceed max_lag_size_10_gbps.
        # @!attribute [rw] name
        #   @return [::String]
        #     [Output Only] Name of the resource.
        # @!attribute [rw] peeringdb_facility_id
        #   @return [::String]
        #     [Output Only] The peeringdb identifier for this facility (corresponding with a netfac type in peeringdb).
        # @!attribute [rw] permitted_connections
        #   @return [::Array<::Google::Cloud::Compute::V1::InterconnectRemoteLocationPermittedConnections>]
        #     [Output Only] Permitted connections.
        # @!attribute [rw] remote_service
        #   @return [::String]
        #     [Output Only] Indicates the service provider present at the remote location. Example values: "Amazon Web Services", "Microsoft Azure".
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] status
        #   @return [::String]
        #     [Output Only] The status of this InterconnectRemoteLocation, which can take one of the following values: - CLOSED: The InterconnectRemoteLocation is closed and is unavailable for provisioning new Cross-Cloud Interconnects. - AVAILABLE: The InterconnectRemoteLocation is available for provisioning new Cross-Cloud Interconnects.
        #     Check the Status enum for the list of possible values.
        class InterconnectRemoteLocation
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # [Output Only] Continent for this location, which can take one of the following values: - AFRICA - ASIA_PAC - EUROPE - NORTH_AMERICA - SOUTH_AMERICA
          module Continent
            # A value indicating that the enum field is not set.
            UNDEFINED_CONTINENT = 0

            AFRICA = 317_443_706

            ASIA_PAC = 119_782_269

            EUROPE = 445_819_298

            NORTH_AMERICA = 448_015_508

            SOUTH_AMERICA = 32_597_340
          end

          # [Output Only] Link Aggregation Control Protocol (LACP) constraints, which can take one of the following values: LACP_SUPPORTED, LACP_UNSUPPORTED
          module Lacp
            # A value indicating that the enum field is not set.
            UNDEFINED_LACP = 0

            # LACP_SUPPORTED: LACP is supported, and enabled by default on the Cross-Cloud Interconnect.
            LACP_SUPPORTED = 339_576_113

            # LACP_UNSUPPORTED: LACP is not supported and is not be enabled on this port. GetDiagnostics shows bundleAggregationType as "static". GCP does not support LAGs without LACP, so requestedLinkCount must be 1.
            LACP_UNSUPPORTED = 203_930_104
          end

          # [Output Only] The status of this InterconnectRemoteLocation, which can take one of the following values: - CLOSED: The InterconnectRemoteLocation is closed and is unavailable for provisioning new Cross-Cloud Interconnects. - AVAILABLE: The InterconnectRemoteLocation is available for provisioning new Cross-Cloud Interconnects.
          module Status
            # A value indicating that the enum field is not set.
            UNDEFINED_STATUS = 0

            # The InterconnectRemoteLocation is available for provisioning new Cross-Cloud Interconnects.
            AVAILABLE = 442_079_913

            # The InterconnectRemoteLocation is closed for provisioning new Cross-Cloud Interconnects.
            CLOSED = 380_163_436
          end
        end

        # @!attribute [rw] port_pair_remote_location
        #   @return [::String]
        #     [Output Only] Port pair remote location constraints, which can take one of the following values: PORT_PAIR_UNCONSTRAINED_REMOTE_LOCATION, PORT_PAIR_MATCHING_REMOTE_LOCATION. GCP's API refers only to individual ports, but the UI uses this field when ordering a pair of ports, to prevent users from accidentally ordering something that is incompatible with their cloud provider. Specifically, when ordering a redundant pair of Cross-Cloud Interconnect ports, and one of them uses a remote location with portPairMatchingRemoteLocation set to matching, the UI requires that both ports use the same remote location.
        #     Check the PortPairRemoteLocation enum for the list of possible values.
        # @!attribute [rw] port_pair_vlan
        #   @return [::String]
        #     [Output Only] Port pair VLAN constraints, which can take one of the following values: PORT_PAIR_UNCONSTRAINED_VLAN, PORT_PAIR_MATCHING_VLAN
        #     Check the PortPairVlan enum for the list of possible values.
        # @!attribute [rw] subnet_length_range
        #   @return [::Google::Cloud::Compute::V1::InterconnectRemoteLocationConstraintsSubnetLengthRange]
        #     [Output Only] [min-length, max-length] The minimum and maximum value (inclusive) for the IPv4 subnet length. For example, an interconnectRemoteLocation for Azure has \\{min: 30, max: 30} because Azure requires /30 subnets. This range specifies the values supported by both cloud providers. Interconnect currently supports /29 and /30 IPv4 subnet lengths. If a remote cloud has no constraint on IPv4 subnet length, the range would thus be \\{min: 29, max: 30}.
        class InterconnectRemoteLocationConstraints
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # [Output Only] Port pair remote location constraints, which can take one of the following values: PORT_PAIR_UNCONSTRAINED_REMOTE_LOCATION, PORT_PAIR_MATCHING_REMOTE_LOCATION. GCP's API refers only to individual ports, but the UI uses this field when ordering a pair of ports, to prevent users from accidentally ordering something that is incompatible with their cloud provider. Specifically, when ordering a redundant pair of Cross-Cloud Interconnect ports, and one of them uses a remote location with portPairMatchingRemoteLocation set to matching, the UI requires that both ports use the same remote location.
          module PortPairRemoteLocation
            # A value indicating that the enum field is not set.
            UNDEFINED_PORT_PAIR_REMOTE_LOCATION = 0

            # If PORT_PAIR_MATCHING_REMOTE_LOCATION, the remote cloud provider allocates ports in pairs, and the user should choose the same remote location for both ports.
            PORT_PAIR_MATCHING_REMOTE_LOCATION = 207_291_859

            # If PORT_PAIR_UNCONSTRAINED_REMOTE_LOCATION, a user may opt to provision a redundant pair of Cross-Cloud Interconnects using two different remote locations in the same city.
            PORT_PAIR_UNCONSTRAINED_REMOTE_LOCATION = 60_609_829
          end

          # [Output Only] Port pair VLAN constraints, which can take one of the following values: PORT_PAIR_UNCONSTRAINED_VLAN, PORT_PAIR_MATCHING_VLAN
          module PortPairVlan
            # A value indicating that the enum field is not set.
            UNDEFINED_PORT_PAIR_VLAN = 0

            # If PORT_PAIR_MATCHING_VLAN, the Interconnect for this attachment is part of a pair of ports that should have matching VLAN allocations. This occurs with Cross-Cloud Interconnect to Azure remote locations. While GCP's API does not explicitly group pairs of ports, the UI uses this field to ensure matching VLAN ids when configuring a redundant VLAN pair.
            PORT_PAIR_MATCHING_VLAN = 250_295_358

            # PORT_PAIR_UNCONSTRAINED_VLAN means there is no constraint.
            PORT_PAIR_UNCONSTRAINED_VLAN = 175_227_948
          end
        end

        # @!attribute [rw] max
        #   @return [::Integer]
        # @!attribute [rw] min
        #   @return [::Integer]
        class InterconnectRemoteLocationConstraintsSubnetLengthRange
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Response to the list request, and contains a list of interconnect remote locations.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::InterconnectRemoteLocation>]
        #     A list of InterconnectRemoteLocation resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#interconnectRemoteLocationList for lists of interconnect remote locations.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token lets you get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class InterconnectRemoteLocationList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] interconnect_location
        #   @return [::String]
        #     [Output Only] URL of an Interconnect location that is permitted to connect to this Interconnect remote location.
        class InterconnectRemoteLocationPermittedConnections
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Response for the InterconnectsGetDiagnosticsRequest.
        # @!attribute [rw] result
        #   @return [::Google::Cloud::Compute::V1::InterconnectDiagnostics]
        class InterconnectsGetDiagnosticsResponse
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Response for the InterconnectsGetMacsecConfigRequest.
        # @!attribute [rw] etag
        #   @return [::String]
        #     end_interface: MixerGetResponseWithEtagBuilder
        # @!attribute [rw] result
        #   @return [::Google::Cloud::Compute::V1::InterconnectMacsecConfig]
        class InterconnectsGetMacsecConfigResponse
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for UrlMaps.InvalidateCache. See the method description for details.
        # @!attribute [rw] cache_invalidation_rule_resource
        #   @return [::Google::Cloud::Compute::V1::CacheInvalidationRule]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] url_map
        #   @return [::String]
        #     Name of the UrlMap scoping this request.
        class InvalidateCacheUrlMapRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Metadata
        # @!attribute [rw] key
        #   @return [::String]
        #     Key for the metadata entry. Keys must conform to the following regexp: [a-zA-Z0-9-_]+, and be less than 128 bytes in length. This is reflected as part of a URL in the metadata server. Additionally, to avoid ambiguity, keys must not conflict with any other metadata keys for the project.
        # @!attribute [rw] value
        #   @return [::String]
        #     Value for the metadata entry. These are free-form strings, and only have meaning as interpreted by the image running in the instance. The only restriction placed on values is that their size must be less than or equal to 262144 bytes (256 KiB).
        class Items
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a License resource. A License represents billing and aggregate usage data for public and marketplace images. *Caution* This resource is intended for use only by third-party partners who are creating Cloud Marketplace images.
        # @!attribute [rw] charges_use_fee
        #   @return [::Boolean]
        #     [Output Only] Deprecated. This field no longer reflects whether a license charges a usage fee.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional textual description of the resource; provided by the client when the resource is created.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#license for licenses.
        # @!attribute [rw] license_code
        #   @return [::Integer]
        #     [Output Only] The unique code used to attach this license to images, snapshots, and disks.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. The name must be 1-63 characters long and comply with RFC1035.
        # @!attribute [rw] resource_requirements
        #   @return [::Google::Cloud::Compute::V1::LicenseResourceRequirements]
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] transferable
        #   @return [::Boolean]
        #     If false, licenses will not be copied from the source resource when creating an image from a disk, disk from snapshot, or snapshot from disk.
        class License
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a License Code resource. A License Code is a unique identifier used to represent a license resource. *Caution* This resource is intended for use only by third-party partners who are creating Cloud Marketplace images.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     [Output Only] Description of this License Code.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#licenseCode for licenses.
        # @!attribute [rw] license_alias
        #   @return [::Array<::Google::Cloud::Compute::V1::LicenseCodeLicenseAlias>]
        #     [Output Only] URL and description aliases of Licenses with the same License Code.
        # @!attribute [rw] name
        #   @return [::String]
        #     [Output Only] Name of the resource. The name is 1-20 characters long and must be a valid 64 bit integer.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] state
        #   @return [::String]
        #     [Output Only] Current state of this License Code.
        #     Check the State enum for the list of possible values.
        # @!attribute [rw] transferable
        #   @return [::Boolean]
        #     [Output Only] If true, the license will remain attached when creating images or snapshots from disks. Otherwise, the license is not transferred.
        class LicenseCode
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # [Output Only] Current state of this License Code.
          module State
            # A value indicating that the enum field is not set.
            UNDEFINED_STATE = 0

            # Machines are not allowed to attach boot disks with this License Code. Requests to create new resources with this license will be rejected.
            DISABLED = 516_696_700

            # Use is allowed for anyone with USE_READ_ONLY access to this License Code.
            ENABLED = 182_130_465

            # Use of this license is limited to a project whitelist.
            RESTRICTED = 261_551_195

            STATE_UNSPECIFIED = 470_755_401

            # Reserved state.
            TERMINATED = 250_018_339
          end
        end

        # @!attribute [rw] description
        #   @return [::String]
        #     [Output Only] Description of this License Code.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] URL of license corresponding to this License Code.
        class LicenseCodeLicenseAlias
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Commitment for a particular license resource.
        # @!attribute [rw] amount
        #   @return [::Integer]
        #     The number of licenses purchased.
        # @!attribute [rw] cores_per_license
        #   @return [::String]
        #     Specifies the core range of the instance for which this license applies.
        # @!attribute [rw] license
        #   @return [::String]
        #     Any applicable license URI.
        class LicenseResourceCommitment
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] min_guest_cpu_count
        #   @return [::Integer]
        #     Minimum number of guest cpus required to use the Instance. Enforced at Instance creation and Instance start.
        # @!attribute [rw] min_memory_mb
        #   @return [::Integer]
        #     Minimum memory required to use the Instance. Enforced at Instance creation and Instance start.
        class LicenseResourceRequirements
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::License>]
        #     A list of License resources.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class LicensesListResponse
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for AcceleratorTypes.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class ListAcceleratorTypesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Addresses.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListAddressesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for FirewallPolicies.ListAssociations. See the method description for details.
        # @!attribute [rw] target_resource
        #   @return [::String]
        #     The target resource to list associations. It is an organization, or a folder.
        class ListAssociationsFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Autoscalers.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] zone
        #   @return [::String]
        #     Name of the zone for this request.
        class ListAutoscalersRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionSslPolicies.ListAvailableFeatures. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListAvailableFeaturesRegionSslPoliciesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for SslPolicies.ListAvailableFeatures. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListAvailableFeaturesSslPoliciesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for BackendBuckets.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListBackendBucketsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for BackendServices.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListBackendServicesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for DiskTypes.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class ListDiskTypesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Disks.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class ListDisksRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceGroupManagers.ListErrors. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     The name of the managed instance group. It must be a string that meets the requirements in RFC1035, or an unsigned long integer: must match regexp pattern: (?:[a-z](?:[-a-z0-9]\\{0,61}[a-z0-9])?)|1-9\\{0,19}.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where the managed instance group is located. It should conform to RFC1035.
        class ListErrorsInstanceGroupManagersRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionInstanceGroupManagers.ListErrors. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     The name of the managed instance group. It must be a string that meets the requirements in RFC1035, or an unsigned long integer: must match regexp pattern: (?:[a-z](?:[-a-z0-9]\\{0,61}[a-z0-9])?)|1-9\\{0,19}.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request. This should conform to RFC1035.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListErrorsRegionInstanceGroupManagersRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ExternalVpnGateways.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListExternalVpnGatewaysRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for FirewallPolicies.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] parent_id
        #   @return [::String]
        #     Parent ID for this request. The ID can be either be "folders/[FOLDER_ID]" if the parent is a folder or "organizations/[ORGANIZATION_ID]" if the parent is an organization.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListFirewallPoliciesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Firewalls.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListFirewallsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ForwardingRules.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListForwardingRulesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for GlobalAddresses.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListGlobalAddressesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for GlobalForwardingRules.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListGlobalForwardingRulesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for GlobalNetworkEndpointGroups.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListGlobalNetworkEndpointGroupsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for GlobalOperations.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListGlobalOperationsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for GlobalOrganizationOperations.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] parent_id
        #   @return [::String]
        #     Parent ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListGlobalOrganizationOperationsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for GlobalPublicDelegatedPrefixes.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListGlobalPublicDelegatedPrefixesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for HealthChecks.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListHealthChecksRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Images.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListImagesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceGroupManagers.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where the managed instance group is located.
        class ListInstanceGroupManagersRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceGroups.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where the instance group is located.
        class ListInstanceGroupsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceTemplates.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListInstanceTemplatesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceGroups.ListInstances. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] instance_group
        #   @return [::String]
        #     The name of the instance group from which you want to generate a list of included instances.
        # @!attribute [rw] instance_groups_list_instances_request_resource
        #   @return [::Google::Cloud::Compute::V1::InstanceGroupsListInstancesRequest]
        #     The body resource for this request
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where the instance group is located.
        class ListInstancesInstanceGroupsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionInstanceGroups.ListInstances. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] instance_group
        #   @return [::String]
        #     Name of the regional instance group for which we want to list the instances.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] region_instance_groups_list_instances_request_resource
        #   @return [::Google::Cloud::Compute::V1::RegionInstanceGroupsListInstancesRequest]
        #     The body resource for this request
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListInstancesRegionInstanceGroupsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class ListInstancesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InterconnectAttachments.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListInterconnectAttachmentsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InterconnectLocations.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListInterconnectLocationsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InterconnectRemoteLocations.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListInterconnectRemoteLocationsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Interconnects.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListInterconnectsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Licenses.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListLicensesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for MachineImages.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListMachineImagesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for MachineTypes.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class ListMachineTypesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceGroupManagers.ListManagedInstances. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     The name of the managed instance group.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where the managed instance group is located.
        class ListManagedInstancesInstanceGroupManagersRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionInstanceGroupManagers.ListManagedInstances. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     The name of the managed instance group.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListManagedInstancesRegionInstanceGroupManagersRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkAttachments.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region of this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListNetworkAttachmentsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkEndpointGroups.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where the network endpoint group is located. It should comply with RFC1035.
        class ListNetworkEndpointGroupsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for GlobalNetworkEndpointGroups.ListNetworkEndpoints. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] network_endpoint_group
        #   @return [::String]
        #     The name of the network endpoint group from which you want to generate a list of included network endpoints. It should comply with RFC1035.
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListNetworkEndpointsGlobalNetworkEndpointGroupsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkEndpointGroups.ListNetworkEndpoints. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] network_endpoint_group
        #   @return [::String]
        #     The name of the network endpoint group from which you want to generate a list of included network endpoints. It should comply with RFC1035.
        # @!attribute [rw] network_endpoint_groups_list_endpoints_request_resource
        #   @return [::Google::Cloud::Compute::V1::NetworkEndpointGroupsListEndpointsRequest]
        #     The body resource for this request
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where the network endpoint group is located. It should comply with RFC1035.
        class ListNetworkEndpointsNetworkEndpointGroupsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionNetworkEndpointGroups.ListNetworkEndpoints. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] network_endpoint_group
        #   @return [::String]
        #     The name of the network endpoint group from which you want to generate a list of included network endpoints. It should comply with RFC1035.
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region where the network endpoint group is located. It should comply with RFC1035.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListNetworkEndpointsRegionNetworkEndpointGroupsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkFirewallPolicies.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListNetworkFirewallPoliciesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Networks.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListNetworksRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NodeGroups.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class ListNodeGroupsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NodeTemplates.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListNodeTemplatesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NodeTypes.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class ListNodeTypesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NodeGroups.ListNodes. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] node_group
        #   @return [::String]
        #     Name of the NodeGroup resource whose nodes you want to list.
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class ListNodesNodeGroupsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for PacketMirrorings.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListPacketMirroringsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Networks.ListPeeringRoutes. See the method description for details.
        # @!attribute [rw] direction
        #   @return [::String]
        #     The direction of the exchanged routes.
        #     Check the Direction enum for the list of possible values.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] network
        #   @return [::String]
        #     Name of the network for this request.
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] peering_name
        #   @return [::String]
        #     The response will show routes exchanged over the given peering connection.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The region of the request. The response will include all subnet routes, static routes and dynamic routes in the region.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListPeeringRoutesNetworksRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The direction of the exchanged routes.
          module Direction
            # A value indicating that the enum field is not set.
            UNDEFINED_DIRECTION = 0

            # For routes exported from peer network.
            INCOMING = 338_552_870

            # For routes exported from local network.
            OUTGOING = 307_438_444
          end
        end

        # A request message for InstanceGroupManagers.ListPerInstanceConfigs. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     The name of the managed instance group. It should conform to RFC1035.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where the managed instance group is located. It should conform to RFC1035.
        class ListPerInstanceConfigsInstanceGroupManagersRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionInstanceGroupManagers.ListPerInstanceConfigs. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     The name of the managed instance group. It should conform to RFC1035.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request, should conform to RFC1035.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListPerInstanceConfigsRegionInstanceGroupManagersRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for SecurityPolicies.ListPreconfiguredExpressionSets. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListPreconfiguredExpressionSetsSecurityPoliciesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for PublicAdvertisedPrefixes.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListPublicAdvertisedPrefixesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for PublicDelegatedPrefixes.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region of this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListPublicDelegatedPrefixesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.ListReferrers. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] instance
        #   @return [::String]
        #     Name of the target instance scoping this request, or '-' if the request should span over all instances in the container.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class ListReferrersInstancesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionAutoscalers.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListRegionAutoscalersRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionBackendServices.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListRegionBackendServicesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionCommitments.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListRegionCommitmentsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionDiskTypes.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListRegionDiskTypesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionDisks.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListRegionDisksRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionHealthCheckServices.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListRegionHealthCheckServicesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionHealthChecks.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListRegionHealthChecksRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionInstanceGroupManagers.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListRegionInstanceGroupManagersRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionInstanceGroups.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListRegionInstanceGroupsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionInstanceTemplates.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the regions for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListRegionInstanceTemplatesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionNetworkEndpointGroups.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region where the network endpoint group is located. It should comply with RFC1035.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListRegionNetworkEndpointGroupsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionNetworkFirewallPolicies.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListRegionNetworkFirewallPoliciesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionNotificationEndpoints.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListRegionNotificationEndpointsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionOperations.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListRegionOperationsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionSecurityPolicies.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListRegionSecurityPoliciesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionSslCertificates.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListRegionSslCertificatesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionSslPolicies.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListRegionSslPoliciesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionTargetHttpProxies.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListRegionTargetHttpProxiesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionTargetHttpsProxies.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListRegionTargetHttpsProxiesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionTargetTcpProxies.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListRegionTargetTcpProxiesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionUrlMaps.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListRegionUrlMapsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Regions.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListRegionsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Reservations.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] zone
        #   @return [::String]
        #     Name of the zone for this request.
        class ListReservationsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ResourcePolicies.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListResourcePoliciesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Routers.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListRoutersRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Routes.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListRoutesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for SecurityPolicies.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListSecurityPoliciesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ServiceAttachments.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region of this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListServiceAttachmentsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Snapshots.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListSnapshotsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for SslCertificates.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListSslCertificatesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for SslPolicies.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListSslPoliciesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Subnetworks.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListSubnetworksRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetGrpcProxies.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListTargetGrpcProxiesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetHttpProxies.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListTargetHttpProxiesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetHttpsProxies.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListTargetHttpsProxiesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetInstances.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] zone
        #   @return [::String]
        #     Name of the zone scoping this request.
        class ListTargetInstancesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetPools.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListTargetPoolsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetSslProxies.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListTargetSslProxiesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetTcpProxies.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListTargetTcpProxiesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetVpnGateways.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListTargetVpnGatewaysRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for UrlMaps.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListUrlMapsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for BackendServices.ListUsable. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListUsableBackendServicesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionBackendServices.ListUsable. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request. It must be a string that meets the requirements in RFC1035.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListUsableRegionBackendServicesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Subnetworks.ListUsable. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListUsableSubnetworksRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for VpnGateways.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListVpnGatewaysRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for VpnTunnels.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListVpnTunnelsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Projects.ListXpnHosts. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] projects_list_xpn_hosts_request_resource
        #   @return [::Google::Cloud::Compute::V1::ProjectsListXpnHostsRequest]
        #     The body resource for this request
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListXpnHostsProjectsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ZoneOperations.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        # @!attribute [rw] zone
        #   @return [::String]
        #     Name of the zone for request.
        class ListZoneOperationsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Zones.List. See the method description for details.
        # @!attribute [rw] filter
        #   @return [::String]
        #     A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. These two types of filter expressions cannot be mixed in one request. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`. You cannot combine constraints on multiple fields using regular expressions.
        # @!attribute [rw] max_results
        #   @return [::Integer]
        #     The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
        # @!attribute [rw] order_by
        #   @return [::String]
        #     Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
        # @!attribute [rw] page_token
        #   @return [::String]
        #     Specifies a page token to use. Set `pageToken` to the `nextPageToken` returned by a previous list request to get the next page of results.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] return_partial_success
        #   @return [::Boolean]
        #     Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
        class ListZonesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] disk_count
        #   @return [::Integer]
        #     Specifies the number of such disks.
        # @!attribute [rw] disk_size_gb
        #   @return [::Integer]
        #     Specifies the size of the disk in base-2 GB.
        # @!attribute [rw] disk_type
        #   @return [::String]
        #     Specifies the desired disk type on the node. This disk type must be a local storage type (e.g.: local-ssd). Note that for nodeTemplates, this should be the name of the disk type and not its URL.
        class LocalDisk
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Provides a localized error message that is safe to return to the user which can be attached to an RPC error.
        # @!attribute [rw] locale
        #   @return [::String]
        #     The locale used following the specification defined at https://www.rfc-editor.org/rfc/bcp/bcp47.txt. Examples are: "en-US", "fr-CH", "es-MX"
        # @!attribute [rw] message
        #   @return [::String]
        #     The localized error message in the above locale.
        class LocalizedMessage
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Configuration for location policy among multiple possible locations (e.g. preferences for zone selection among zones in a single region).
        # @!attribute [rw] locations
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::LocationPolicyLocation}]
        #     Location configurations mapped by location name. Currently only zone names are supported and must be represented as valid internal URLs, such as zones/us-central1-a.
        # @!attribute [rw] target_shape
        #   @return [::String]
        #     Strategy for distributing VMs across zones in a region.
        #     Check the TargetShape enum for the list of possible values.
        class LocationPolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::LocationPolicyLocation]
          class LocationsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # Strategy for distributing VMs across zones in a region.
          module TargetShape
            # A value indicating that the enum field is not set.
            UNDEFINED_TARGET_SHAPE = 0

            # GCE picks zones for creating VM instances to fulfill the requested number of VMs within present resource constraints and to maximize utilization of unused zonal reservations. Recommended for batch workloads that do not require high availability.
            ANY = 64_972

            # GCE always selects a single zone for all the VMs, optimizing for resource quotas, available reservations and general capacity. Recommended for batch workloads that cannot tollerate distribution over multiple zones. This the default shape in Bulk Insert and Capacity Advisor APIs.
            ANY_SINGLE_ZONE = 61_100_880

            # GCE prioritizes acquisition of resources, scheduling VMs in zones where resources are available while distributing VMs as evenly as possible across allowed zones to minimize the impact of zonal failure. Recommended for highly available serving workloads.
            BALANCED = 468_409_608
          end
        end

        # @!attribute [rw] constraints
        #   @return [::Google::Cloud::Compute::V1::LocationPolicyLocationConstraints]
        #     Constraints that the caller requires on the result distribution in this zone.
        # @!attribute [rw] preference
        #   @return [::String]
        #     Preference for a given location. Set to either ALLOW or DENY.
        #     Check the Preference enum for the list of possible values.
        class LocationPolicyLocation
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Preference for a given location. Set to either ALLOW or DENY.
          module Preference
            # A value indicating that the enum field is not set.
            UNDEFINED_PREFERENCE = 0

            # Location is allowed for use.
            ALLOW = 62_368_553

            # Location is prohibited.
            DENY = 2_094_604

            # Default value, unused.
            PREFERENCE_UNSPECIFIED = 496_219_571
          end
        end

        # Per-zone constraints on location policy for this zone.
        # @!attribute [rw] max_count
        #   @return [::Integer]
        #     Maximum number of items that are allowed to be placed in this zone. The value must be non-negative.
        class LocationPolicyLocationConstraints
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # This is deprecated and has no effect. Do not use.
        # @!attribute [rw] cloud_audit
        #   @return [::Google::Cloud::Compute::V1::LogConfigCloudAuditOptions]
        #     This is deprecated and has no effect. Do not use.
        # @!attribute [rw] counter
        #   @return [::Google::Cloud::Compute::V1::LogConfigCounterOptions]
        #     This is deprecated and has no effect. Do not use.
        # @!attribute [rw] data_access
        #   @return [::Google::Cloud::Compute::V1::LogConfigDataAccessOptions]
        #     This is deprecated and has no effect. Do not use.
        class LogConfig
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # This is deprecated and has no effect. Do not use.
        # @!attribute [rw] authorization_logging_options
        #   @return [::Google::Cloud::Compute::V1::AuthorizationLoggingOptions]
        #     This is deprecated and has no effect. Do not use.
        # @!attribute [rw] log_name
        #   @return [::String]
        #     This is deprecated and has no effect. Do not use.
        #     Check the LogName enum for the list of possible values.
        class LogConfigCloudAuditOptions
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # This is deprecated and has no effect. Do not use.
          module LogName
            # A value indicating that the enum field is not set.
            UNDEFINED_LOG_NAME = 0

            # This is deprecated and has no effect. Do not use.
            ADMIN_ACTIVITY = 427_503_135

            # This is deprecated and has no effect. Do not use.
            DATA_ACCESS = 238_070_681

            # This is deprecated and has no effect. Do not use.
            UNSPECIFIED_LOG_NAME = 410_515_182
          end
        end

        # This is deprecated and has no effect. Do not use.
        # @!attribute [rw] custom_fields
        #   @return [::Array<::Google::Cloud::Compute::V1::LogConfigCounterOptionsCustomField>]
        #     This is deprecated and has no effect. Do not use.
        # @!attribute [rw] field
        #   @return [::String]
        #     This is deprecated and has no effect. Do not use.
        # @!attribute [rw] metric
        #   @return [::String]
        #     This is deprecated and has no effect. Do not use.
        class LogConfigCounterOptions
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # This is deprecated and has no effect. Do not use.
        # @!attribute [rw] name
        #   @return [::String]
        #     This is deprecated and has no effect. Do not use.
        # @!attribute [rw] value
        #   @return [::String]
        #     This is deprecated and has no effect. Do not use.
        class LogConfigCounterOptionsCustomField
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # This is deprecated and has no effect. Do not use.
        # @!attribute [rw] log_mode
        #   @return [::String]
        #     This is deprecated and has no effect. Do not use.
        #     Check the LogMode enum for the list of possible values.
        class LogConfigDataAccessOptions
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # This is deprecated and has no effect. Do not use.
          module LogMode
            # A value indicating that the enum field is not set.
            UNDEFINED_LOG_MODE = 0

            # This is deprecated and has no effect. Do not use.
            LOG_FAIL_CLOSED = 360_469_778

            # This is deprecated and has no effect. Do not use.
            LOG_MODE_UNSPECIFIED = 88_160_822
          end
        end

        # Represents a machine image resource. A machine image is a Compute Engine resource that stores all the configuration, metadata, permissions, and data from one or more disks required to create a Virtual machine (VM) instance. For more information, see Machine images.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] The creation timestamp for this machine image in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] guest_flush
        #   @return [::Boolean]
        #     [Input Only] Whether to attempt an application consistent machine image by informing the OS to prepare for the snapshot process.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] A unique identifier for this machine image. The server defines this identifier.
        # @!attribute [rw] instance_properties
        #   @return [::Google::Cloud::Compute::V1::InstanceProperties]
        #     [Output Only] Properties of source instance
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] The resource type, which is always compute#machineImage for machine image.
        # @!attribute [rw] machine_image_encryption_key
        #   @return [::Google::Cloud::Compute::V1::CustomerEncryptionKey]
        #     Encrypts the machine image using a customer-supplied encryption key. After you encrypt a machine image using a customer-supplied key, you must provide the same key if you use the machine image later. For example, you must provide the encryption key when you create an instance from the encrypted machine image in a future request. Customer-supplied encryption keys do not protect access to metadata of the machine image. If you do not provide an encryption key when creating the machine image, then the machine image will be encrypted using an automatically generated key and you do not need to provide a key to use the machine image later.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource; provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] satisfies_pzs
        #   @return [::Boolean]
        #     [Output Only] Reserved for future use.
        # @!attribute [rw] saved_disks
        #   @return [::Array<::Google::Cloud::Compute::V1::SavedDisk>]
        #     An array of Machine Image specific properties for disks attached to the source instance
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] The URL for this machine image. The server defines this URL.
        # @!attribute [rw] source_disk_encryption_keys
        #   @return [::Array<::Google::Cloud::Compute::V1::SourceDiskEncryptionKey>]
        #     [Input Only] The customer-supplied encryption key of the disks attached to the source instance. Required if the source disk is protected by a customer-supplied encryption key.
        # @!attribute [rw] source_instance
        #   @return [::String]
        #     The source instance used to create the machine image. You can provide this as a partial or full URL to the resource. For example, the following are valid values: - https://www.googleapis.com/compute/v1/projects/project/zones/zone /instances/instance - projects/project/zones/zone/instances/instance
        # @!attribute [rw] source_instance_properties
        #   @return [::Google::Cloud::Compute::V1::SourceInstanceProperties]
        #     [Output Only] DEPRECATED: Please use instance_properties instead for source instance related properties. New properties will not be added to this field.
        # @!attribute [rw] status
        #   @return [::String]
        #     [Output Only] The status of the machine image. One of the following values: INVALID, CREATING, READY, DELETING, and UPLOADING.
        #     Check the Status enum for the list of possible values.
        # @!attribute [rw] storage_locations
        #   @return [::Array<::String>]
        #     The regional or multi-regional Cloud Storage bucket location where the machine image is stored.
        # @!attribute [rw] total_storage_bytes
        #   @return [::Integer]
        #     [Output Only] Total size of the storage used by the machine image.
        class MachineImage
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # [Output Only] The status of the machine image. One of the following values: INVALID, CREATING, READY, DELETING, and UPLOADING.
          module Status
            # A value indicating that the enum field is not set.
            UNDEFINED_STATUS = 0

            CREATING = 455_564_985

            DELETING = 528_602_024

            INVALID = 530_283_991

            READY = 77_848_963

            UPLOADING = 267_603_489
          end
        end

        # A list of machine images.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::MachineImage>]
        #     A list of MachineImage resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] The resource type, which is always compute#machineImagesListResponse for machine image lists.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class MachineImageList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a Machine Type resource. You can use specific machine types for your VM instances based on performance and pricing requirements. For more information, read Machine Types.
        # @!attribute [rw] accelerators
        #   @return [::Array<::Google::Cloud::Compute::V1::Accelerators>]
        #     [Output Only] A list of accelerator configurations assigned to this machine type.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] deprecated
        #   @return [::Google::Cloud::Compute::V1::DeprecationStatus]
        #     [Output Only] The deprecation status associated with this machine type. Only applicable if the machine type is unavailable.
        # @!attribute [rw] description
        #   @return [::String]
        #     [Output Only] An optional textual description of the resource.
        # @!attribute [rw] guest_cpus
        #   @return [::Integer]
        #     [Output Only] The number of virtual CPUs that are available to the instance.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] image_space_gb
        #   @return [::Integer]
        #     [Deprecated] This property is deprecated and will never be populated with any relevant values.
        # @!attribute [rw] is_shared_cpu
        #   @return [::Boolean]
        #     [Output Only] Whether this machine type has a shared CPU. See Shared-core machine types for more information.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] The type of the resource. Always compute#machineType for machine types.
        # @!attribute [rw] maximum_persistent_disks
        #   @return [::Integer]
        #     [Output Only] Maximum persistent disks allowed.
        # @!attribute [rw] maximum_persistent_disks_size_gb
        #   @return [::Integer]
        #     [Output Only] Maximum total persistent disks size (GB) allowed.
        # @!attribute [rw] memory_mb
        #   @return [::Integer]
        #     [Output Only] The amount of physical memory available to the instance, defined in MB.
        # @!attribute [rw] name
        #   @return [::String]
        #     [Output Only] Name of the resource.
        # @!attribute [rw] scratch_disks
        #   @return [::Array<::Google::Cloud::Compute::V1::ScratchDisks>]
        #     [Output Only] A list of extended scratch disks assigned to the instance.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] zone
        #   @return [::String]
        #     [Output Only] The name of the zone where the machine type resides, such as us-central1-a.
        class MachineType
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::MachineTypesScopedList}]
        #     A list of MachineTypesScopedList resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#machineTypeAggregatedList for aggregated lists of machine types.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class MachineTypeAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::MachineTypesScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # Contains a list of machine types.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::MachineType>]
        #     A list of MachineType resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#machineTypeList for lists of machine types.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class MachineTypeList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] machine_types
        #   @return [::Array<::Google::Cloud::Compute::V1::MachineType>]
        #     [Output Only] A list of machine types contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] An informational warning that appears when the machine types list is empty.
        class MachineTypesScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A Managed Instance resource.
        # @!attribute [rw] current_action
        #   @return [::String]
        #     [Output Only] The current action that the managed instance group has scheduled for the instance. Possible values: - NONE The instance is running, and the managed instance group does not have any scheduled actions for this instance. - CREATING The managed instance group is creating this instance. If the group fails to create this instance, it will try again until it is successful. - CREATING_WITHOUT_RETRIES The managed instance group is attempting to create this instance only once. If the group fails to create this instance, it does not try again and the group's targetSize value is decreased instead. - RECREATING The managed instance group is recreating this instance. - DELETING The managed instance group is permanently deleting this instance. - ABANDONING The managed instance group is abandoning this instance. The instance will be removed from the instance group and from any target pools that are associated with this group. - RESTARTING The managed instance group is restarting the instance. - REFRESHING The managed instance group is applying configuration changes to the instance without stopping it. For example, the group can update the target pool list for an instance without stopping that instance. - VERIFYING The managed instance group has created the instance and it is in the process of being verified.
        #     Check the CurrentAction enum for the list of possible values.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output only] The unique identifier for this resource. This field is empty when instance does not exist.
        # @!attribute [rw] instance
        #   @return [::String]
        #     [Output Only] The URL of the instance. The URL can exist even if the instance has not yet been created.
        # @!attribute [rw] instance_health
        #   @return [::Array<::Google::Cloud::Compute::V1::ManagedInstanceInstanceHealth>]
        #     [Output Only] Health state of the instance per health-check.
        # @!attribute [rw] instance_status
        #   @return [::String]
        #     [Output Only] The status of the instance. This field is empty when the instance does not exist.
        #     Check the InstanceStatus enum for the list of possible values.
        # @!attribute [rw] last_attempt
        #   @return [::Google::Cloud::Compute::V1::ManagedInstanceLastAttempt]
        #     [Output Only] Information about the last attempt to create or delete the instance.
        # @!attribute [rw] name
        #   @return [::String]
        #     [Output Only] The name of the instance. The name always exists even if the instance has not yet been created.
        # @!attribute [rw] preserved_state_from_config
        #   @return [::Google::Cloud::Compute::V1::PreservedState]
        #     [Output Only] Preserved state applied from per-instance config for this instance.
        # @!attribute [rw] preserved_state_from_policy
        #   @return [::Google::Cloud::Compute::V1::PreservedState]
        #     [Output Only] Preserved state generated based on stateful policy for this instance.
        # @!attribute [rw] version
        #   @return [::Google::Cloud::Compute::V1::ManagedInstanceVersion]
        #     [Output Only] Intended version of this instance.
        class ManagedInstance
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # [Output Only] The current action that the managed instance group has scheduled for the instance. Possible values: - NONE The instance is running, and the managed instance group does not have any scheduled actions for this instance. - CREATING The managed instance group is creating this instance. If the group fails to create this instance, it will try again until it is successful. - CREATING_WITHOUT_RETRIES The managed instance group is attempting to create this instance only once. If the group fails to create this instance, it does not try again and the group's targetSize value is decreased instead. - RECREATING The managed instance group is recreating this instance. - DELETING The managed instance group is permanently deleting this instance. - ABANDONING The managed instance group is abandoning this instance. The instance will be removed from the instance group and from any target pools that are associated with this group. - RESTARTING The managed instance group is restarting the instance. - REFRESHING The managed instance group is applying configuration changes to the instance without stopping it. For example, the group can update the target pool list for an instance without stopping that instance. - VERIFYING The managed instance group has created the instance and it is in the process of being verified.
          # Additional supported values which may be not listed in the enum directly due to technical reasons:
          # STOPPING
          # SUSPENDING
          module CurrentAction
            # A value indicating that the enum field is not set.
            UNDEFINED_CURRENT_ACTION = 0

            # The managed instance group is abandoning this instance. The instance will be removed from the instance group and from any target pools that are associated with this group.
            ABANDONING = 388_244_813

            # The managed instance group is creating this instance. If the group fails to create this instance, it will try again until it is successful.
            CREATING = 455_564_985

            # The managed instance group is attempting to create this instance only once. If the group fails to create this instance, it does not try again and the group's targetSize value is decreased.
            CREATING_WITHOUT_RETRIES = 428_843_785

            # The managed instance group is permanently deleting this instance.
            DELETING = 528_602_024

            # The managed instance group has not scheduled any actions for this instance.
            NONE = 2_402_104

            # The managed instance group is recreating this instance.
            RECREATING = 287_278_572

            # The managed instance group is applying configuration changes to the instance without stopping it. For example, the group can update the target pool list for an instance without stopping that instance.
            REFRESHING = 163_266_343

            # The managed instance group is restarting this instance.
            RESTARTING = 320_534_387

            # The managed instance group is resuming this instance.
            RESUMING = 446_856_618

            # The managed instance group is starting this instance.
            STARTING = 488_820_800

            # The managed instance group is verifying this already created instance. Verification happens every time the instance is (re)created or restarted and consists of: 1. Waiting until health check specified as part of this managed instance group's autohealing policy reports HEALTHY. Note: Applies only if autohealing policy has a health check specified 2. Waiting for addition verification steps performed as post-instance creation (subject to future extensions).
            VERIFYING = 16_982_185
          end

          # [Output Only] The status of the instance. This field is empty when the instance does not exist.
          # Additional supported values which may be not listed in the enum directly due to technical reasons:
          # STOPPING
          # SUSPENDING
          module InstanceStatus
            # A value indicating that the enum field is not set.
            UNDEFINED_INSTANCE_STATUS = 0

            # The instance is halted and we are performing tear down tasks like network deprogramming, releasing quota, IP, tearing down disks etc.
            DEPROVISIONING = 428_935_662

            # Resources are being allocated for the instance.
            PROVISIONING = 290_896_621

            # The instance is in repair.
            REPAIRING = 413_483_285

            # The instance is running.
            RUNNING = 121_282_975

            # All required resources have been allocated and the instance is being started.
            STAGING = 431_072_283

            # The instance has stopped successfully.
            STOPPED = 444_276_141

            STOPPING = 350_791_796

            # The instance has suspended.
            SUSPENDED = 51_223_995

            SUSPENDING = 514_206_246

            # The instance has stopped (either by explicit action or underlying failure).
            TERMINATED = 250_018_339
          end
        end

        # @!attribute [rw] detailed_health_state
        #   @return [::String]
        #     [Output Only] The current detailed instance health state.
        #     Check the DetailedHealthState enum for the list of possible values.
        # @!attribute [rw] health_check
        #   @return [::String]
        #     [Output Only] The URL for the health check that verifies whether the instance is healthy.
        class ManagedInstanceInstanceHealth
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # [Output Only] The current detailed instance health state.
          module DetailedHealthState
            # A value indicating that the enum field is not set.
            UNDEFINED_DETAILED_HEALTH_STATE = 0

            # The instance is being drained. The existing connections to the instance have time to complete, but the new ones are being refused.
            DRAINING = 480_455_402

            # The instance is reachable i.e. a connection to the application health checking endpoint can be established, and conforms to the requirements defined by the health check.
            HEALTHY = 439_801_213

            # The instance is unreachable i.e. a connection to the application health checking endpoint cannot be established, or the server does not respond within the specified timeout.
            TIMEOUT = 477_813_057

            # The instance is reachable, but does not conform to the requirements defined by the health check.
            UNHEALTHY = 462_118_084

            # The health checking system is aware of the instance but its health is not known at the moment.
            UNKNOWN = 433_141_802
          end
        end

        # @!attribute [rw] errors
        #   @return [::Google::Cloud::Compute::V1::Errors]
        #     [Output Only] Encountered errors during the last attempt to create or delete the instance.
        class ManagedInstanceLastAttempt
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] instance_template
        #   @return [::String]
        #     [Output Only] The intended template of the instance. This field is empty when current_action is one of { DELETING, ABANDONING }.
        # @!attribute [rw] name
        #   @return [::String]
        #     [Output Only] Name of the version.
        class ManagedInstanceVersion
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A metadata key/value entry.
        # @!attribute [rw] fingerprint
        #   @return [::String]
        #     Specifies a fingerprint for this request, which is essentially a hash of the metadata's contents and used for optimistic locking. The fingerprint is initially generated by Compute Engine and changes after every request to modify or update metadata. You must always provide an up-to-date fingerprint hash in order to update or change metadata, otherwise the request will fail with error 412 conditionNotMet. To see the latest fingerprint, make a get() request to retrieve the resource.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::Items>]
        #     Array of key/value pairs. The total size of all keys and values must be less than 512 KB.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#metadata for metadata.
        class Metadata
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Opaque filter criteria used by load balancers to restrict routing configuration to a limited set of load balancing proxies. Proxies and sidecars involved in load balancing would typically present metadata to the load balancers that need to match criteria specified here. If a match takes place, the relevant configuration is made available to those proxies. For each metadataFilter in this list, if its filterMatchCriteria is set to MATCH_ANY, at least one of the filterLabels must match the corresponding label provided in the metadata. If its filterMatchCriteria is set to MATCH_ALL, then all of its filterLabels must match with corresponding labels provided in the metadata. An example for using metadataFilters would be: if load balancing involves Envoys, they receive routing configuration when values in metadataFilters match values supplied in of their XDS requests to loadbalancers.
        # @!attribute [rw] filter_labels
        #   @return [::Array<::Google::Cloud::Compute::V1::MetadataFilterLabelMatch>]
        #     The list of label value pairs that must match labels in the provided metadata based on filterMatchCriteria This list must not be empty and can have at the most 64 entries.
        # @!attribute [rw] filter_match_criteria
        #   @return [::String]
        #     Specifies how individual filter label matches within the list of filterLabels and contributes toward the overall metadataFilter match. Supported values are: - MATCH_ANY: at least one of the filterLabels must have a matching label in the provided metadata. - MATCH_ALL: all filterLabels must have matching labels in the provided metadata.
        #     Check the FilterMatchCriteria enum for the list of possible values.
        class MetadataFilter
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Specifies how individual filter label matches within the list of filterLabels and contributes toward the overall metadataFilter match. Supported values are: - MATCH_ANY: at least one of the filterLabels must have a matching label in the provided metadata. - MATCH_ALL: all filterLabels must have matching labels in the provided metadata.
          module FilterMatchCriteria
            # A value indicating that the enum field is not set.
            UNDEFINED_FILTER_MATCH_CRITERIA = 0

            # Specifies that all filterLabels must match for the metadataFilter to be considered a match.
            MATCH_ALL = 180_663_271

            # Specifies that any filterLabel must match for the metadataFilter to be considered a match.
            MATCH_ANY = 180_663_346

            # Indicates that the match criteria was not set. A metadataFilter must never be created with this value.
            NOT_SET = 163_646_646
          end
        end

        # MetadataFilter label name value pairs that are expected to match corresponding labels presented as metadata to the load balancer.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of metadata label. The name can have a maximum length of 1024 characters and must be at least 1 character long.
        # @!attribute [rw] value
        #   @return [::String]
        #     The value of the label must match the specified value. value can have a maximum length of 1024 characters.
        class MetadataFilterLabelMatch
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Addresses.Move. See the method description for details.
        # @!attribute [rw] address
        #   @return [::String]
        #     Name of the address resource to move.
        # @!attribute [rw] project
        #   @return [::String]
        #     Source project ID which the Address is moved from.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] region_addresses_move_request_resource
        #   @return [::Google::Cloud::Compute::V1::RegionAddressesMoveRequest]
        #     The body resource for this request
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class MoveAddressRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Projects.MoveDisk. See the method description for details.
        # @!attribute [rw] disk_move_request_resource
        #   @return [::Google::Cloud::Compute::V1::DiskMoveRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class MoveDiskProjectRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for FirewallPolicies.Move. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to update.
        # @!attribute [rw] parent_id
        #   @return [::String]
        #     The new parent of the firewall policy. The ID can be either be "folders/[FOLDER_ID]" if the parent is a folder or "organizations/[ORGANIZATION_ID]" if the parent is an organization.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class MoveFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for GlobalAddresses.Move. See the method description for details.
        # @!attribute [rw] address
        #   @return [::String]
        #     Name of the address resource to move.
        # @!attribute [rw] global_addresses_move_request_resource
        #   @return [::Google::Cloud::Compute::V1::GlobalAddressesMoveRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Source project ID which the Address is moved from.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class MoveGlobalAddressRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Projects.MoveInstance. See the method description for details.
        # @!attribute [rw] instance_move_request_resource
        #   @return [::Google::Cloud::Compute::V1::InstanceMoveRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class MoveInstanceProjectRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # The named port. For example: <"http", 80>.
        # @!attribute [rw] name
        #   @return [::String]
        #     The name for this named port. The name must be 1-63 characters long, and comply with RFC1035.
        # @!attribute [rw] port
        #   @return [::Integer]
        #     The port number, which can be a value between 1 and 65535.
        class NamedPort
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Contains NAT IP information of a NAT config (i.e. usage status, mode).
        # @!attribute [rw] nat_ip_info_mappings
        #   @return [::Array<::Google::Cloud::Compute::V1::NatIpInfoNatIpInfoMapping>]
        #     A list of all NAT IPs assigned to this NAT config.
        # @!attribute [rw] nat_name
        #   @return [::String]
        #     Name of the NAT config which the NAT IP belongs to.
        class NatIpInfo
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Contains information of a NAT IP.
        # @!attribute [rw] mode
        #   @return [::String]
        #     Specifies whether NAT IP is auto or manual.
        #     Check the Mode enum for the list of possible values.
        # @!attribute [rw] nat_ip
        #   @return [::String]
        #     NAT IP address. For example: 203.0.113.11.
        # @!attribute [rw] usage
        #   @return [::String]
        #     Specifies whether NAT IP is currently serving at least one endpoint or not.
        #     Check the Usage enum for the list of possible values.
        class NatIpInfoNatIpInfoMapping
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Specifies whether NAT IP is auto or manual.
          module Mode
            # A value indicating that the enum field is not set.
            UNDEFINED_MODE = 0

            AUTO = 2_020_783

            MANUAL = 119_397_318
          end

          # Specifies whether NAT IP is currently serving at least one endpoint or not.
          module Usage
            # A value indicating that the enum field is not set.
            UNDEFINED_USAGE = 0

            IN_USE = 17_393_485

            UNUSED = 360_643_030
          end
        end

        # @!attribute [rw] result
        #   @return [::Array<::Google::Cloud::Compute::V1::NatIpInfo>]
        #     [Output Only] A list of NAT IP information.
        class NatIpInfoResponse
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a VPC Network resource. Networks connect resources to each other and to the internet. For more information, read Virtual Private Cloud (VPC) Network.
        # @!attribute [rw] I_pv4_range
        #   @return [::String]
        #     Deprecated in favor of subnet mode networks. The range of internal addresses that are legal on this network. This range is a CIDR specification, for example: 192.168.0.0/16. Provided by the client when the network is created.
        # @!attribute [rw] auto_create_subnetworks
        #   @return [::Boolean]
        #     Must be set to create a VPC network. If not set, a legacy network is created. When set to true, the VPC network is created in auto mode. When set to false, the VPC network is created in custom mode. An auto mode VPC network starts with one subnet per region. Each subnet has a predetermined range as described in Auto mode VPC network IP ranges. For custom mode VPC networks, you can add subnets using the subnetworks insert method.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this field when you create the resource.
        # @!attribute [rw] enable_ula_internal_ipv6
        #   @return [::Boolean]
        #     Enable ULA internal ipv6 on this network. Enabling this feature will assign a /48 from google defined ULA prefix fd20::/20. .
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     [Output Only] URL of the firewall policy the network is associated with.
        # @!attribute [rw] gateway_i_pv4
        #   @return [::String]
        #     [Output Only] The gateway address for default routing out of the network, selected by Google Cloud.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] internal_ipv6_range
        #   @return [::String]
        #     When enabling ula internal ipv6, caller optionally can specify the /48 range they want from the google defined ULA prefix fd20::/20. The input must be a valid /48 ULA IPv6 address and must be within the fd20::/20. Operation will fail if the speficied /48 is already in used by another resource. If the field is not speficied, then a /48 range will be randomly allocated from fd20::/20 and returned via this field. .
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#network for networks.
        # @!attribute [rw] mtu
        #   @return [::Integer]
        #     Maximum Transmission Unit in bytes. The minimum value for this field is 1300 and the maximum value is 8896. The suggested value is 1500, which is the default MTU used on the Internet, or 8896 if you want to use Jumbo frames. If unspecified, the value defaults to 1460.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?`. The first character must be a lowercase letter, and all following characters (except for the last character) must be a dash, lowercase letter, or digit. The last character must be a lowercase letter or digit.
        # @!attribute [rw] network_firewall_policy_enforcement_order
        #   @return [::String]
        #     The network firewall policy enforcement order. Can be either AFTER_CLASSIC_FIREWALL or BEFORE_CLASSIC_FIREWALL. Defaults to AFTER_CLASSIC_FIREWALL if the field is not specified.
        #     Check the NetworkFirewallPolicyEnforcementOrder enum for the list of possible values.
        # @!attribute [rw] peerings
        #   @return [::Array<::Google::Cloud::Compute::V1::NetworkPeering>]
        #     [Output Only] A list of network peerings for the resource.
        # @!attribute [rw] routing_config
        #   @return [::Google::Cloud::Compute::V1::NetworkRoutingConfig]
        #     The network-level routing configuration for this network. Used by Cloud Router to determine what type of network-wide routing behavior to enforce.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] self_link_with_id
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource with the resource id.
        # @!attribute [rw] subnetworks
        #   @return [::Array<::String>]
        #     [Output Only] Server-defined fully-qualified URLs for all subnetworks in this VPC network.
        class Network
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The network firewall policy enforcement order. Can be either AFTER_CLASSIC_FIREWALL or BEFORE_CLASSIC_FIREWALL. Defaults to AFTER_CLASSIC_FIREWALL if the field is not specified.
          module NetworkFirewallPolicyEnforcementOrder
            # A value indicating that the enum field is not set.
            UNDEFINED_NETWORK_FIREWALL_POLICY_ENFORCEMENT_ORDER = 0

            AFTER_CLASSIC_FIREWALL = 154_582_608

            BEFORE_CLASSIC_FIREWALL = 338_458_349
          end
        end

        # NetworkAttachments A network attachment resource ...
        # @!attribute [rw] connection_endpoints
        #   @return [::Array<::Google::Cloud::Compute::V1::NetworkAttachmentConnectedEndpoint>]
        #     [Output Only] An array of connections for all the producers connected to this network attachment.
        # @!attribute [rw] connection_preference
        #   @return [::String]
        #     Check the ConnectionPreference enum for the list of possible values.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] fingerprint
        #   @return [::String]
        #     Fingerprint of this resource. A hash of the contents stored in this object. This field is used in optimistic locking. An up-to-date fingerprint must be provided in order to patch.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource type. The server generates this identifier.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] network
        #   @return [::String]
        #     [Output Only] The URL of the network which the Network Attachment belongs to. Practically it is inferred by fetching the network of the first subnetwork associated. Because it is required that all the subnetworks must be from the same network, it is assured that the Network Attachment belongs to the same network as all the subnetworks.
        # @!attribute [rw] producer_accept_lists
        #   @return [::Array<::String>]
        #     Projects that are allowed to connect to this network attachment. The project can be specified using its id or number.
        # @!attribute [rw] producer_reject_lists
        #   @return [::Array<::String>]
        #     Projects that are not allowed to connect to this network attachment. The project can be specified using its id or number.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] URL of the region where the network attachment resides. This field applies only to the region resource. You must specify this field as part of the HTTP request URL. It is not settable as a field in the request body.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] self_link_with_id
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource's resource id.
        # @!attribute [rw] subnetworks
        #   @return [::Array<::String>]
        #     An array of URLs where each entry is the URL of a subnet provided by the service consumer to use for endpoints in the producers that connect to this network attachment.
        class NetworkAttachment
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods


          module ConnectionPreference
            # A value indicating that the enum field is not set.
            UNDEFINED_CONNECTION_PREFERENCE = 0

            ACCEPT_AUTOMATIC = 75_250_580

            ACCEPT_MANUAL = 373_061_341

            INVALID = 530_283_991
          end
        end

        # Contains a list of NetworkAttachmentsScopedList.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::NetworkAttachmentsScopedList}]
        #     A list of NetworkAttachmentsScopedList resources.
        # @!attribute [rw] kind
        #   @return [::String]
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class NetworkAttachmentAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::NetworkAttachmentsScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # [Output Only] A connection connected to this network attachment.
        # @!attribute [rw] ip_address
        #   @return [::String]
        #     The IPv4 address assigned to the producer instance network interface. This value will be a range in case of Serverless.
        # @!attribute [rw] ipv6_address
        #   @return [::String]
        #     The IPv6 address assigned to the producer instance network interface. This is only assigned when the stack types of both the instance network interface and the consumer subnet are IPv4_IPv6.
        # @!attribute [rw] project_id_or_num
        #   @return [::String]
        #     The project id or number of the interface to which the IP was assigned.
        # @!attribute [rw] secondary_ip_cidr_ranges
        #   @return [::Array<::String>]
        #     Alias IP ranges from the same subnetwork.
        # @!attribute [rw] status
        #   @return [::String]
        #     The status of a connected endpoint to this network attachment.
        #     Check the Status enum for the list of possible values.
        # @!attribute [rw] subnetwork
        #   @return [::String]
        #     The subnetwork used to assign the IP to the producer instance network interface.
        # @!attribute [rw] subnetwork_cidr_range
        #   @return [::String]
        #     [Output Only] The CIDR range of the subnet from which the IPv4 internal IP was allocated from.
        class NetworkAttachmentConnectedEndpoint
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The status of a connected endpoint to this network attachment.
          module Status
            # A value indicating that the enum field is not set.
            UNDEFINED_STATUS = 0

            # The consumer allows traffic from the producer to reach its VPC.
            ACCEPTED = 246_714_279

            # The consumer network attachment no longer exists.
            CLOSED = 380_163_436

            # The consumer needs to take further action before traffic can be served.
            NEEDS_ATTENTION = 344_491_452

            # The consumer neither allows nor prohibits traffic from the producer to reach its VPC.
            PENDING = 35_394_935

            # The consumer prohibits traffic from the producer to reach its VPC.
            REJECTED = 174_130_302

            STATUS_UNSPECIFIED = 42_133_066
          end
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::NetworkAttachment>]
        #     A list of NetworkAttachment resources.
        # @!attribute [rw] kind
        #   @return [::String]
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class NetworkAttachmentList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] network_attachments
        #   @return [::Array<::Google::Cloud::Compute::V1::NetworkAttachment>]
        #     A list of NetworkAttachments contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     Informational warning which replaces the list of network attachments when the list is empty.
        class NetworkAttachmentsScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a Google Cloud Armor network edge security service resource.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] fingerprint
        #   @return [::String]
        #     Fingerprint of this resource. A hash of the contents stored in this object. This field is used in optimistic locking. This field will be ignored when inserting a NetworkEdgeSecurityService. An up-to-date fingerprint must be provided in order to update the NetworkEdgeSecurityService, otherwise the request will fail with error 412 conditionNotMet. To see the latest fingerprint, make a get() request to retrieve a NetworkEdgeSecurityService.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output only] Type of the resource. Always compute#networkEdgeSecurityService for NetworkEdgeSecurityServices
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] URL of the region where the resource resides. You must specify this field as part of the HTTP request URL. It is not settable as a field in the request body.
        # @!attribute [rw] security_policy
        #   @return [::String]
        #     The resource URL for the network edge security service associated with this network edge security service.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] self_link_with_id
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource with the resource id.
        class NetworkEdgeSecurityService
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] etag
        #   @return [::String]
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::NetworkEdgeSecurityServicesScopedList}]
        #     A list of NetworkEdgeSecurityServicesScopedList resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#networkEdgeSecurityServiceAggregatedList for lists of Network Edge Security Services.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class NetworkEdgeSecurityServiceAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::NetworkEdgeSecurityServicesScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # @!attribute [rw] network_edge_security_services
        #   @return [::Array<::Google::Cloud::Compute::V1::NetworkEdgeSecurityService>]
        #     A list of NetworkEdgeSecurityServices contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     Informational warning which replaces the list of security policies when the list is empty.
        class NetworkEdgeSecurityServicesScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # The network endpoint.
        # @!attribute [rw] annotations
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     Metadata defined as annotations on the network endpoint.
        # @!attribute [rw] fqdn
        #   @return [::String]
        #     Optional fully qualified domain name of network endpoint. This can only be specified when NetworkEndpointGroup.network_endpoint_type is NON_GCP_FQDN_PORT.
        # @!attribute [rw] instance
        #   @return [::String]
        #     The name for a specific VM instance that the IP address belongs to. This is required for network endpoints of type GCE_VM_IP_PORT. The instance must be in the same zone of network endpoint group. The name must be 1-63 characters long, and comply with RFC1035.
        # @!attribute [rw] ip_address
        #   @return [::String]
        #     Optional IPv4 address of network endpoint. The IP address must belong to a VM in Compute Engine (either the primary IP or as part of an aliased IP range). If the IP address is not specified, then the primary IP address for the VM instance in the network that the network endpoint group belongs to will be used.
        # @!attribute [rw] port
        #   @return [::Integer]
        #     Optional port number of network endpoint. If not specified, the defaultPort for the network endpoint group will be used.
        class NetworkEndpoint
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class AnnotationsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # Represents a collection of network endpoints. A network endpoint group (NEG) defines how a set of endpoints should be reached, whether they are reachable, and where they are located. For more information about using NEGs for different use cases, see Network endpoint groups overview.
        # @!attribute [rw] annotations
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     Metadata defined as annotations on the network endpoint group.
        # @!attribute [rw] app_engine
        #   @return [::Google::Cloud::Compute::V1::NetworkEndpointGroupAppEngine]
        #     Only valid when networkEndpointType is "SERVERLESS". Only one of cloudRun, appEngine or cloudFunction may be set.
        # @!attribute [rw] cloud_function
        #   @return [::Google::Cloud::Compute::V1::NetworkEndpointGroupCloudFunction]
        #     Only valid when networkEndpointType is "SERVERLESS". Only one of cloudRun, appEngine or cloudFunction may be set.
        # @!attribute [rw] cloud_run
        #   @return [::Google::Cloud::Compute::V1::NetworkEndpointGroupCloudRun]
        #     Only valid when networkEndpointType is "SERVERLESS". Only one of cloudRun, appEngine or cloudFunction may be set.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] default_port
        #   @return [::Integer]
        #     The default port used if the port number is not specified in the network endpoint.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#networkEndpointGroup for network endpoint group.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource; provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] network
        #   @return [::String]
        #     The URL of the network to which all network endpoints in the NEG belong. Uses "default" project network if unspecified.
        # @!attribute [rw] network_endpoint_type
        #   @return [::String]
        #     Type of network endpoints in this network endpoint group. Can be one of GCE_VM_IP, GCE_VM_IP_PORT, NON_GCP_PRIVATE_IP_PORT, INTERNET_FQDN_PORT, INTERNET_IP_PORT, SERVERLESS, PRIVATE_SERVICE_CONNECT.
        #     Check the NetworkEndpointType enum for the list of possible values.
        # @!attribute [rw] psc_data
        #   @return [::Google::Cloud::Compute::V1::NetworkEndpointGroupPscData]
        # @!attribute [rw] psc_target_service
        #   @return [::String]
        #     The target service url used to set up private service connection to a Google API or a PSC Producer Service Attachment. An example value is: "asia-northeast3-cloudkms.googleapis.com"
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] The URL of the region where the network endpoint group is located.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] size
        #   @return [::Integer]
        #     [Output only] Number of network endpoints in the network endpoint group.
        # @!attribute [rw] subnetwork
        #   @return [::String]
        #     Optional URL of the subnetwork to which all network endpoints in the NEG belong.
        # @!attribute [rw] zone
        #   @return [::String]
        #     [Output Only] The URL of the zone where the network endpoint group is located.
        class NetworkEndpointGroup
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class AnnotationsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # Type of network endpoints in this network endpoint group. Can be one of GCE_VM_IP, GCE_VM_IP_PORT, NON_GCP_PRIVATE_IP_PORT, INTERNET_FQDN_PORT, INTERNET_IP_PORT, SERVERLESS, PRIVATE_SERVICE_CONNECT.
          module NetworkEndpointType
            # A value indicating that the enum field is not set.
            UNDEFINED_NETWORK_ENDPOINT_TYPE = 0

            # The network endpoint is represented by an IP address.
            GCE_VM_IP = 401_880_793

            # The network endpoint is represented by IP address and port pair.
            GCE_VM_IP_PORT = 501_838_375

            # The network endpoint is represented by fully qualified domain name and port.
            INTERNET_FQDN_PORT = 404_154_477

            # The network endpoint is represented by an internet IP address and port.
            INTERNET_IP_PORT = 477_719_963

            # The network endpoint is represented by an IP address and port. The endpoint belongs to a VM or pod running in a customer's on-premises.
            NON_GCP_PRIVATE_IP_PORT = 336_447_968

            # The network endpoint is either public Google APIs or services exposed by other GCP Project with a Service Attachment. The connection is set up by private service connect
            PRIVATE_SERVICE_CONNECT = 48_134_724

            # The network endpoint is handled by specified serverless infrastructure.
            SERVERLESS = 270_492_508
          end
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::NetworkEndpointGroupsScopedList}]
        #     A list of NetworkEndpointGroupsScopedList resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] The resource type, which is always compute#networkEndpointGroupAggregatedList for aggregated lists of network endpoint groups.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class NetworkEndpointGroupAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::NetworkEndpointGroupsScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # Configuration for an App Engine network endpoint group (NEG). The service is optional, may be provided explicitly or in the URL mask. The version is optional and can only be provided explicitly or in the URL mask when service is present. Note: App Engine service must be in the same project and located in the same region as the Serverless NEG.
        # @!attribute [rw] service
        #   @return [::String]
        #     Optional serving service. The service name is case-sensitive and must be 1-63 characters long. Example value: "default", "my-service".
        # @!attribute [rw] url_mask
        #   @return [::String]
        #     A template to parse service and version fields from a request URL. URL mask allows for routing to multiple App Engine services without having to create multiple Network Endpoint Groups and backend services. For example, the request URLs "foo1-dot-appname.appspot.com/v1" and "foo1-dot-appname.appspot.com/v2" can be backed by the same Serverless NEG with URL mask "<service>-dot-appname.appspot.com/<version>". The URL mask will parse them to { service = "foo1", version = "v1" } and { service = "foo1", version = "v2" } respectively.
        # @!attribute [rw] version
        #   @return [::String]
        #     Optional serving version. The version name is case-sensitive and must be 1-100 characters long. Example value: "v1", "v2".
        class NetworkEndpointGroupAppEngine
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Configuration for a Cloud Function network endpoint group (NEG). The function must be provided explicitly or in the URL mask. Note: Cloud Function must be in the same project and located in the same region as the Serverless NEG.
        # @!attribute [rw] function
        #   @return [::String]
        #     A user-defined name of the Cloud Function. The function name is case-sensitive and must be 1-63 characters long. Example value: "func1".
        # @!attribute [rw] url_mask
        #   @return [::String]
        #     A template to parse function field from a request URL. URL mask allows for routing to multiple Cloud Functions without having to create multiple Network Endpoint Groups and backend services. For example, request URLs " mydomain.com/function1" and "mydomain.com/function2" can be backed by the same Serverless NEG with URL mask "/<function>". The URL mask will parse them to { function = "function1" } and { function = "function2" } respectively.
        class NetworkEndpointGroupCloudFunction
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Configuration for a Cloud Run network endpoint group (NEG). The service must be provided explicitly or in the URL mask. The tag is optional, may be provided explicitly or in the URL mask. Note: Cloud Run service must be in the same project and located in the same region as the Serverless NEG.
        # @!attribute [rw] service
        #   @return [::String]
        #     Cloud Run service is the main resource of Cloud Run. The service must be 1-63 characters long, and comply with RFC1035. Example value: "run-service".
        # @!attribute [rw] tag
        #   @return [::String]
        #     Optional Cloud Run tag represents the "named-revision" to provide additional fine-grained traffic routing information. The tag must be 1-63 characters long, and comply with RFC1035. Example value: "revision-0010".
        # @!attribute [rw] url_mask
        #   @return [::String]
        #     A template to parse <service> and <tag> fields from a request URL. URL mask allows for routing to multiple Run services without having to create multiple network endpoint groups and backend services. For example, request URLs "foo1.domain.com/bar1" and "foo1.domain.com/bar2" can be backed by the same Serverless Network Endpoint Group (NEG) with URL mask "<tag>.domain.com/<service>". The URL mask will parse them to { service="bar1", tag="foo1" } and { service="bar2", tag="foo2" } respectively.
        class NetworkEndpointGroupCloudRun
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::NetworkEndpointGroup>]
        #     A list of NetworkEndpointGroup resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] The resource type, which is always compute#networkEndpointGroupList for network endpoint group lists.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class NetworkEndpointGroupList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # All data that is specifically relevant to only network endpoint groups of type PRIVATE_SERVICE_CONNECT.
        # @!attribute [rw] consumer_psc_address
        #   @return [::String]
        #     [Output Only] Address allocated from given subnetwork for PSC. This IP address acts as a VIP for a PSC NEG, allowing it to act as an endpoint in L7 PSC-XLB.
        # @!attribute [rw] psc_connection_id
        #   @return [::Integer]
        #     [Output Only] The PSC connection id of the PSC Network Endpoint Group Consumer.
        # @!attribute [rw] psc_connection_status
        #   @return [::String]
        #     [Output Only] The connection status of the PSC Forwarding Rule.
        #     Check the PscConnectionStatus enum for the list of possible values.
        class NetworkEndpointGroupPscData
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # [Output Only] The connection status of the PSC Forwarding Rule.
          module PscConnectionStatus
            # A value indicating that the enum field is not set.
            UNDEFINED_PSC_CONNECTION_STATUS = 0

            # The connection has been accepted by the producer.
            ACCEPTED = 246_714_279

            # The connection has been closed by the producer and will not serve traffic going forward.
            CLOSED = 380_163_436

            # The connection has been accepted by the producer, but the producer needs to take further action before the forwarding rule can serve traffic.
            NEEDS_ATTENTION = 344_491_452

            # The connection is pending acceptance by the producer.
            PENDING = 35_394_935

            # The connection has been rejected by the producer.
            REJECTED = 174_130_302

            STATUS_UNSPECIFIED = 42_133_066
          end
        end

        # @!attribute [rw] network_endpoints
        #   @return [::Array<::Google::Cloud::Compute::V1::NetworkEndpoint>]
        #     The list of network endpoints to be attached.
        class NetworkEndpointGroupsAttachEndpointsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] network_endpoints
        #   @return [::Array<::Google::Cloud::Compute::V1::NetworkEndpoint>]
        #     The list of network endpoints to be detached.
        class NetworkEndpointGroupsDetachEndpointsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] health_status
        #   @return [::String]
        #     Optional query parameter for showing the health status of each network endpoint. Valid options are SKIP or SHOW. If you don't specify this parameter, the health status of network endpoints will not be provided.
        #     Check the HealthStatus enum for the list of possible values.
        class NetworkEndpointGroupsListEndpointsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Optional query parameter for showing the health status of each network endpoint. Valid options are SKIP or SHOW. If you don't specify this parameter, the health status of network endpoints will not be provided.
          module HealthStatus
            # A value indicating that the enum field is not set.
            UNDEFINED_HEALTH_STATUS = 0

            # Show the health status for each network endpoint. Impacts latency of the call.
            SHOW = 2_544_381

            # Health status for network endpoints will not be provided.
            SKIP = 2_547_071
          end
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::NetworkEndpointWithHealthStatus>]
        #     A list of NetworkEndpointWithHealthStatus resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] The resource type, which is always compute#networkEndpointGroupsListNetworkEndpoints for the list of network endpoints in the specified network endpoint group.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class NetworkEndpointGroupsListNetworkEndpoints
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] network_endpoint_groups
        #   @return [::Array<::Google::Cloud::Compute::V1::NetworkEndpointGroup>]
        #     [Output Only] The list of network endpoint groups that are contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] An informational warning that replaces the list of network endpoint groups when the list is empty.
        class NetworkEndpointGroupsScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] healths
        #   @return [::Array<::Google::Cloud::Compute::V1::HealthStatusForNetworkEndpoint>]
        #     [Output only] The health status of network endpoint;
        # @!attribute [rw] network_endpoint
        #   @return [::Google::Cloud::Compute::V1::NetworkEndpoint]
        #     [Output only] The network endpoint;
        class NetworkEndpointWithHealthStatus
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A network interface resource attached to an instance.
        # @!attribute [rw] access_configs
        #   @return [::Array<::Google::Cloud::Compute::V1::AccessConfig>]
        #     An array of configurations for this interface. Currently, only one access config, ONE_TO_ONE_NAT, is supported. If there are no accessConfigs specified, then this instance will have no external internet access.
        # @!attribute [rw] alias_ip_ranges
        #   @return [::Array<::Google::Cloud::Compute::V1::AliasIpRange>]
        #     An array of alias IP ranges for this network interface. You can only specify this field for network interfaces in VPC networks.
        # @!attribute [rw] fingerprint
        #   @return [::String]
        #     Fingerprint hash of contents stored in this network interface. This field will be ignored when inserting an Instance or adding a NetworkInterface. An up-to-date fingerprint must be provided in order to update the NetworkInterface. The request will fail with error 400 Bad Request if the fingerprint is not provided, or 412 Precondition Failed if the fingerprint is out of date.
        # @!attribute [rw] internal_ipv6_prefix_length
        #   @return [::Integer]
        #     The prefix length of the primary internal IPv6 range.
        # @!attribute [rw] ipv6_access_configs
        #   @return [::Array<::Google::Cloud::Compute::V1::AccessConfig>]
        #     An array of IPv6 access configurations for this interface. Currently, only one IPv6 access config, DIRECT_IPV6, is supported. If there is no ipv6AccessConfig specified, then this instance will have no external IPv6 Internet access.
        # @!attribute [rw] ipv6_access_type
        #   @return [::String]
        #     [Output Only] One of EXTERNAL, INTERNAL to indicate whether the IP can be accessed from the Internet. This field is always inherited from its subnetwork. Valid only if stackType is IPV4_IPV6.
        #     Check the Ipv6AccessType enum for the list of possible values.
        # @!attribute [rw] ipv6_address
        #   @return [::String]
        #     An IPv6 internal network address for this network interface. To use a static internal IP address, it must be unused and in the same region as the instance's zone. If not specified, Google Cloud will automatically assign an internal IPv6 address from the instance's subnetwork.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#networkInterface for network interfaces.
        # @!attribute [rw] name
        #   @return [::String]
        #     [Output Only] The name of the network interface, which is generated by the server. For a VM, the network interface uses the nicN naming format. Where N is a value between 0 and 7. The default interface value is nic0.
        # @!attribute [rw] network
        #   @return [::String]
        #     URL of the VPC network resource for this instance. When creating an instance, if neither the network nor the subnetwork is specified, the default network global/networks/default is used. If the selected project doesn't have the default network, you must specify a network or subnet. If the network is not specified but the subnetwork is specified, the network is inferred. If you specify this property, you can specify the network as a full or partial URL. For example, the following are all valid URLs: - https://www.googleapis.com/compute/v1/projects/project/global/networks/ network - projects/project/global/networks/network - global/networks/default
        # @!attribute [rw] network_attachment
        #   @return [::String]
        #     The URL of the network attachment that this interface should connect to in the following format: projects/\\{project_number}/regions/\\{region_name}/networkAttachments/\\{network_attachment_name}.
        # @!attribute [rw] network_i_p
        #   @return [::String]
        #     An IPv4 internal IP address to assign to the instance for this network interface. If not specified by the user, an unused internal IP is assigned by the system.
        # @!attribute [rw] nic_type
        #   @return [::String]
        #     The type of vNIC to be used on this interface. This may be gVNIC or VirtioNet.
        #     Check the NicType enum for the list of possible values.
        # @!attribute [rw] queue_count
        #   @return [::Integer]
        #     The networking queue count that's specified by users for the network interface. Both Rx and Tx queues will be set to this number. It'll be empty if not specified by the users.
        # @!attribute [rw] stack_type
        #   @return [::String]
        #     The stack type for this network interface. To assign only IPv4 addresses, use IPV4_ONLY. To assign both IPv4 and IPv6 addresses, use IPV4_IPV6. If not specified, IPV4_ONLY is used. This field can be both set at instance creation and update network interface operations.
        #     Check the StackType enum for the list of possible values.
        # @!attribute [rw] subnetwork
        #   @return [::String]
        #     The URL of the Subnetwork resource for this instance. If the network resource is in legacy mode, do not specify this field. If the network is in auto subnet mode, specifying the subnetwork is optional. If the network is in custom subnet mode, specifying the subnetwork is required. If you specify this field, you can specify the subnetwork as a full or partial URL. For example, the following are all valid URLs: - https://www.googleapis.com/compute/v1/projects/project/regions/region /subnetworks/subnetwork - regions/region/subnetworks/subnetwork
        class NetworkInterface
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # [Output Only] One of EXTERNAL, INTERNAL to indicate whether the IP can be accessed from the Internet. This field is always inherited from its subnetwork. Valid only if stackType is IPV4_IPV6.
          module Ipv6AccessType
            # A value indicating that the enum field is not set.
            UNDEFINED_IPV6_ACCESS_TYPE = 0

            # This network interface can have external IPv6.
            EXTERNAL = 35_607_499

            # This network interface can have internal IPv6.
            INTERNAL = 279_295_677

            UNSPECIFIED_IPV6_ACCESS_TYPE = 313_080_613
          end

          # The type of vNIC to be used on this interface. This may be gVNIC or VirtioNet.
          module NicType
            # A value indicating that the enum field is not set.
            UNDEFINED_NIC_TYPE = 0

            # GVNIC
            GVNIC = 68_209_305

            # No type specified.
            UNSPECIFIED_NIC_TYPE = 67_411_801

            # VIRTIO
            VIRTIO_NET = 452_123_481
          end

          # The stack type for this network interface. To assign only IPv4 addresses, use IPV4_ONLY. To assign both IPv4 and IPv6 addresses, use IPV4_IPV6. If not specified, IPV4_ONLY is used. This field can be both set at instance creation and update network interface operations.
          module StackType
            # A value indicating that the enum field is not set.
            UNDEFINED_STACK_TYPE = 0

            # The network interface can have both IPv4 and IPv6 addresses.
            IPV4_IPV6 = 22_197_249

            # The network interface will be assigned IPv4 address.
            IPV4_ONLY = 22_373_798

            UNSPECIFIED_STACK_TYPE = 298_084_569
          end
        end

        # Contains a list of networks.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::Network>]
        #     A list of Network resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#networkList for lists of networks.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class NetworkList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A network peering attached to a network resource. The message includes the peering name, peer network, peering state, and a flag indicating whether Google Compute Engine should automatically create routes for the peering.
        # @!attribute [rw] auto_create_routes
        #   @return [::Boolean]
        #     This field will be deprecated soon. Use the exchange_subnet_routes field instead. Indicates whether full mesh connectivity is created and managed automatically between peered networks. Currently this field should always be true since Google Compute Engine will automatically create and manage subnetwork routes between two networks when peering state is ACTIVE.
        # @!attribute [rw] exchange_subnet_routes
        #   @return [::Boolean]
        #     Indicates whether full mesh connectivity is created and managed automatically between peered networks. Currently this field should always be true since Google Compute Engine will automatically create and manage subnetwork routes between two networks when peering state is ACTIVE.
        # @!attribute [rw] export_custom_routes
        #   @return [::Boolean]
        #     Whether to export the custom routes to peer network. The default value is false.
        # @!attribute [rw] export_subnet_routes_with_public_ip
        #   @return [::Boolean]
        #     Whether subnet routes with public IP range are exported. The default value is true, all subnet routes are exported. IPv4 special-use ranges are always exported to peers and are not controlled by this field.
        # @!attribute [rw] import_custom_routes
        #   @return [::Boolean]
        #     Whether to import the custom routes from peer network. The default value is false.
        # @!attribute [rw] import_subnet_routes_with_public_ip
        #   @return [::Boolean]
        #     Whether subnet routes with public IP range are imported. The default value is false. IPv4 special-use ranges are always imported from peers and are not controlled by this field.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of this peering. Provided by the client when the peering is created. The name must comply with RFC1035. Specifically, the name must be 1-63 characters long and match regular expression `[a-z]([-a-z0-9]*[a-z0-9])?`. The first character must be a lowercase letter, and all the following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] network
        #   @return [::String]
        #     The URL of the peer network. It can be either full URL or partial URL. The peer network may belong to a different project. If the partial URL does not contain project, it is assumed that the peer network is in the same project as the current network.
        # @!attribute [rw] peer_mtu
        #   @return [::Integer]
        #     Maximum Transmission Unit in bytes.
        # @!attribute [rw] stack_type
        #   @return [::String]
        #     Which IP version(s) of traffic and routes are allowed to be imported or exported between peer networks. The default value is IPV4_ONLY.
        #     Check the StackType enum for the list of possible values.
        # @!attribute [rw] state
        #   @return [::String]
        #     [Output Only] State for the peering, either `ACTIVE` or `INACTIVE`. The peering is `ACTIVE` when there's a matching configuration in the peer network.
        #     Check the State enum for the list of possible values.
        # @!attribute [rw] state_details
        #   @return [::String]
        #     [Output Only] Details about the current state of the peering.
        class NetworkPeering
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Which IP version(s) of traffic and routes are allowed to be imported or exported between peer networks. The default value is IPV4_ONLY.
          module StackType
            # A value indicating that the enum field is not set.
            UNDEFINED_STACK_TYPE = 0

            # This Peering will allow IPv4 traffic and routes to be exchanged. Additionally if the matching peering is IPV4_IPV6, IPv6 traffic and routes will be exchanged as well.
            IPV4_IPV6 = 22_197_249

            # This Peering will only allow IPv4 traffic and routes to be exchanged, even if the matching peering is IPV4_IPV6.
            IPV4_ONLY = 22_373_798
          end

          # [Output Only] State for the peering, either `ACTIVE` or `INACTIVE`. The peering is `ACTIVE` when there's a matching configuration in the peer network.
          module State
            # A value indicating that the enum field is not set.
            UNDEFINED_STATE = 0

            # Matching configuration exists on the peer.
            ACTIVE = 314_733_318

            # There is no matching configuration on the peer, including the case when peer does not exist.
            INACTIVE = 270_421_099
          end
        end

        # @!attribute [rw] total_egress_bandwidth_tier
        #   @return [::String]
        #     Check the TotalEgressBandwidthTier enum for the list of possible values.
        class NetworkPerformanceConfig
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods


          module TotalEgressBandwidthTier
            # A value indicating that the enum field is not set.
            UNDEFINED_TOTAL_EGRESS_BANDWIDTH_TIER = 0

            DEFAULT = 115_302_945

            TIER_1 = 326_919_444
          end
        end

        # A routing configuration attached to a network resource. The message includes the list of routers associated with the network, and a flag indicating the type of routing behavior to enforce network-wide.
        # @!attribute [rw] routing_mode
        #   @return [::String]
        #     The network-wide routing mode to use. If set to REGIONAL, this network's Cloud Routers will only advertise routes with subnets of this network in the same region as the router. If set to GLOBAL, this network's Cloud Routers will advertise routes with all subnets of this network, across regions.
        #     Check the RoutingMode enum for the list of possible values.
        class NetworkRoutingConfig
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The network-wide routing mode to use. If set to REGIONAL, this network's Cloud Routers will only advertise routes with subnets of this network in the same region as the router. If set to GLOBAL, this network's Cloud Routers will advertise routes with all subnets of this network, across regions.
          module RoutingMode
            # A value indicating that the enum field is not set.
            UNDEFINED_ROUTING_MODE = 0

            GLOBAL = 494_663_587

            REGIONAL = 92_288_543
          end
        end

        # @!attribute [rw] auto_create_routes
        #   @return [::Boolean]
        #     This field will be deprecated soon. Use exchange_subnet_routes in network_peering instead. Indicates whether full mesh connectivity is created and managed automatically between peered networks. Currently this field should always be true since Google Compute Engine will automatically create and manage subnetwork routes between two networks when peering state is ACTIVE.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the peering, which should conform to RFC1035.
        # @!attribute [rw] network_peering
        #   @return [::Google::Cloud::Compute::V1::NetworkPeering]
        #     Network peering parameters. In order to specify route policies for peering using import and export custom routes, you must specify all peering related parameters (name, peer network, exchange_subnet_routes) in the network_peering field. The corresponding fields in NetworksAddPeeringRequest will be deprecated soon.
        # @!attribute [rw] peer_network
        #   @return [::String]
        #     URL of the peer network. It can be either full URL or partial URL. The peer network may belong to a different project. If the partial URL does not contain project, it is assumed that the peer network is in the same project as the current network.
        class NetworksAddPeeringRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] firewall_policys
        #   @return [::Array<::Google::Cloud::Compute::V1::NetworksGetEffectiveFirewallsResponseEffectiveFirewallPolicy>]
        #     Effective firewalls from firewall policy.
        # @!attribute [rw] firewalls
        #   @return [::Array<::Google::Cloud::Compute::V1::Firewall>]
        #     Effective firewalls on the network.
        class NetworksGetEffectiveFirewallsResponse
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] display_name
        #   @return [::String]
        #     [Output Only] Deprecated, please use short name instead. The display name of the firewall policy.
        # @!attribute [rw] name
        #   @return [::String]
        #     [Output Only] The name of the firewall policy.
        # @!attribute [rw] rules
        #   @return [::Array<::Google::Cloud::Compute::V1::FirewallPolicyRule>]
        #     The rules that apply to the network.
        # @!attribute [rw] short_name
        #   @return [::String]
        #     [Output Only] The short name of the firewall policy.
        # @!attribute [rw] type
        #   @return [::String]
        #     [Output Only] The type of the firewall policy.
        #     Check the Type enum for the list of possible values.
        class NetworksGetEffectiveFirewallsResponseEffectiveFirewallPolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # [Output Only] The type of the firewall policy.
          module Type
            # A value indicating that the enum field is not set.
            UNDEFINED_TYPE = 0

            HIERARCHY = 69_902_869

            NETWORK = 413_984_270

            UNSPECIFIED = 526_786_327
          end
        end

        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the peering, which should conform to RFC1035.
        class NetworksRemovePeeringRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] network_peering
        #   @return [::Google::Cloud::Compute::V1::NetworkPeering]
        class NetworksUpdatePeeringRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a sole-tenant Node Group resource. A sole-tenant node is a physical server that is dedicated to hosting VM instances only for your specific project. Use sole-tenant nodes to keep your instances physically separated from instances in other projects, or to group your instances together on the same host hardware. For more information, read Sole-tenant nodes.
        # @!attribute [rw] autoscaling_policy
        #   @return [::Google::Cloud::Compute::V1::NodeGroupAutoscalingPolicy]
        #     Specifies how autoscaling should behave.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] fingerprint
        #   @return [::String]
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] The type of the resource. Always compute#nodeGroup for node group.
        # @!attribute [rw] location_hint
        #   @return [::String]
        #     An opaque location hint used to place the Node close to other resources. This field is for use by internal tools that use the public API. The location hint here on the NodeGroup overrides any location_hint present in the NodeTemplate.
        # @!attribute [rw] maintenance_policy
        #   @return [::String]
        #     Specifies how to handle instances when a node in the group undergoes maintenance. Set to one of: DEFAULT, RESTART_IN_PLACE, or MIGRATE_WITHIN_NODE_GROUP. The default value is DEFAULT. For more information, see Maintenance policies.
        #     Check the MaintenancePolicy enum for the list of possible values.
        # @!attribute [rw] maintenance_window
        #   @return [::Google::Cloud::Compute::V1::NodeGroupMaintenanceWindow]
        # @!attribute [rw] name
        #   @return [::String]
        #     The name of the resource, provided by the client when initially creating the resource. The resource name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] node_template
        #   @return [::String]
        #     URL of the node template to create the node group from.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] share_settings
        #   @return [::Google::Cloud::Compute::V1::ShareSettings]
        #     Share-settings for the node group
        # @!attribute [rw] size
        #   @return [::Integer]
        #     [Output Only] The total number of nodes in the node group.
        # @!attribute [rw] status
        #   @return [::String]
        #     Check the Status enum for the list of possible values.
        # @!attribute [rw] zone
        #   @return [::String]
        #     [Output Only] The name of the zone where the node group resides, such as us-central1-a.
        class NodeGroup
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Specifies how to handle instances when a node in the group undergoes maintenance. Set to one of: DEFAULT, RESTART_IN_PLACE, or MIGRATE_WITHIN_NODE_GROUP. The default value is DEFAULT. For more information, see Maintenance policies.
          module MaintenancePolicy
            # A value indicating that the enum field is not set.
            UNDEFINED_MAINTENANCE_POLICY = 0

            # Allow the node and corresponding instances to retain default maintenance behavior.
            DEFAULT = 115_302_945

            MAINTENANCE_POLICY_UNSPECIFIED = 72_964_182

            # When maintenance must be done on a node, the instances on that node will be moved to other nodes in the group. Instances with onHostMaintenance = MIGRATE will live migrate to their destinations while instances with onHostMaintenance = TERMINATE will terminate and then restart on their destination nodes if automaticRestart = true.
            MIGRATE_WITHIN_NODE_GROUP = 153_483_394

            # Instances in this group will restart on the same node when maintenance has completed. Instances must have onHostMaintenance = TERMINATE, and they will only restart if automaticRestart = true.
            RESTART_IN_PLACE = 228_647_325
          end

          module Status
            # A value indicating that the enum field is not set.
            UNDEFINED_STATUS = 0

            CREATING = 455_564_985

            DELETING = 528_602_024

            INVALID = 530_283_991

            READY = 77_848_963
          end
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::NodeGroupsScopedList}]
        #     A list of NodeGroupsScopedList resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource.Always compute#nodeGroupAggregatedList for aggregated lists of node groups.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class NodeGroupAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::NodeGroupsScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # @!attribute [rw] max_nodes
        #   @return [::Integer]
        #     The maximum number of nodes that the group should have. Must be set if autoscaling is enabled. Maximum value allowed is 100.
        # @!attribute [rw] min_nodes
        #   @return [::Integer]
        #     The minimum number of nodes that the group should have.
        # @!attribute [rw] mode
        #   @return [::String]
        #     The autoscaling mode. Set to one of: ON, OFF, or ONLY_SCALE_OUT. For more information, see Autoscaler modes.
        #     Check the Mode enum for the list of possible values.
        class NodeGroupAutoscalingPolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The autoscaling mode. Set to one of: ON, OFF, or ONLY_SCALE_OUT. For more information, see Autoscaler modes.
          module Mode
            # A value indicating that the enum field is not set.
            UNDEFINED_MODE = 0

            MODE_UNSPECIFIED = 371_348_091

            # Autoscaling is disabled.
            OFF = 78_159

            # Autocaling is fully enabled.
            ON = 2527

            # Autoscaling will only scale out and will not remove nodes.
            ONLY_SCALE_OUT = 152_713_670
          end
        end

        # Contains a list of nodeGroups.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::NodeGroup>]
        #     A list of NodeGroup resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource.Always compute#nodeGroupList for lists of node groups.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class NodeGroupList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Time window specified for daily maintenance operations. GCE's internal maintenance will be performed within this window.
        # @!attribute [rw] maintenance_duration
        #   @return [::Google::Cloud::Compute::V1::Duration]
        #     [Output only] A predetermined duration for the window, automatically chosen to be the smallest possible in the given scenario.
        # @!attribute [rw] start_time
        #   @return [::String]
        #     Start time of the window. This must be in UTC format that resolves to one of 00:00, 04:00, 08:00, 12:00, 16:00, or 20:00. For example, both 13:00-5 and 08:00 are valid.
        class NodeGroupMaintenanceWindow
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] accelerators
        #   @return [::Array<::Google::Cloud::Compute::V1::AcceleratorConfig>]
        #     Accelerators for this node.
        # @!attribute [rw] consumed_resources
        #   @return [::Google::Cloud::Compute::V1::InstanceConsumptionInfo]
        #     Node resources that are reserved by all instances.
        # @!attribute [rw] cpu_overcommit_type
        #   @return [::String]
        #     CPU overcommit.
        #     Check the CpuOvercommitType enum for the list of possible values.
        # @!attribute [rw] disks
        #   @return [::Array<::Google::Cloud::Compute::V1::LocalDisk>]
        #     Local disk configurations.
        # @!attribute [rw] instance_consumption_data
        #   @return [::Array<::Google::Cloud::Compute::V1::InstanceConsumptionData>]
        #     Instance data that shows consumed resources on the node.
        # @!attribute [rw] instances
        #   @return [::Array<::String>]
        #     Instances scheduled on this node.
        # @!attribute [rw] name
        #   @return [::String]
        #     The name of the node.
        # @!attribute [rw] node_type
        #   @return [::String]
        #     The type of this node.
        # @!attribute [rw] satisfies_pzs
        #   @return [::Boolean]
        #     [Output Only] Reserved for future use.
        # @!attribute [rw] server_binding
        #   @return [::Google::Cloud::Compute::V1::ServerBinding]
        #     Binding properties for the physical server.
        # @!attribute [rw] server_id
        #   @return [::String]
        #     Server ID associated with this node.
        # @!attribute [rw] status
        #   @return [::String]
        #     Check the Status enum for the list of possible values.
        # @!attribute [rw] total_resources
        #   @return [::Google::Cloud::Compute::V1::InstanceConsumptionInfo]
        #     Total amount of available resources on the node.
        class NodeGroupNode
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # CPU overcommit.
          module CpuOvercommitType
            # A value indicating that the enum field is not set.
            UNDEFINED_CPU_OVERCOMMIT_TYPE = 0

            CPU_OVERCOMMIT_TYPE_UNSPECIFIED = 520_665_615

            ENABLED = 182_130_465

            NONE = 2_402_104
          end

          module Status
            # A value indicating that the enum field is not set.
            UNDEFINED_STATUS = 0

            CREATING = 455_564_985

            DELETING = 528_602_024

            INVALID = 530_283_991

            READY = 77_848_963

            REPAIRING = 413_483_285
          end
        end

        # @!attribute [rw] additional_node_count
        #   @return [::Integer]
        #     Count of additional nodes to be added to the node group.
        class NodeGroupsAddNodesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] nodes
        #   @return [::Array<::String>]
        #     Names of the nodes to delete.
        class NodeGroupsDeleteNodesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::NodeGroupNode>]
        #     A list of Node resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] The resource type, which is always compute.nodeGroupsListNodes for the list of nodes in the specified node group.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class NodeGroupsListNodes
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] node_groups
        #   @return [::Array<::Google::Cloud::Compute::V1::NodeGroup>]
        #     [Output Only] A list of node groups contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] An informational warning that appears when the nodeGroup list is empty.
        class NodeGroupsScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] node_template
        #   @return [::String]
        #     Full or partial URL of the node template resource to be updated for this node group.
        class NodeGroupsSetNodeTemplateRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] nodes
        #   @return [::Array<::String>]
        #     Names of the nodes to go under maintenance simulation.
        class NodeGroupsSimulateMaintenanceEventRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represent a sole-tenant Node Template resource. You can use a template to define properties for nodes in a node group. For more information, read Creating node groups and instances.
        # @!attribute [rw] accelerators
        #   @return [::Array<::Google::Cloud::Compute::V1::AcceleratorConfig>]
        # @!attribute [rw] cpu_overcommit_type
        #   @return [::String]
        #     CPU overcommit.
        #     Check the CpuOvercommitType enum for the list of possible values.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] disks
        #   @return [::Array<::Google::Cloud::Compute::V1::LocalDisk>]
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] The type of the resource. Always compute#nodeTemplate for node templates.
        # @!attribute [rw] name
        #   @return [::String]
        #     The name of the resource, provided by the client when initially creating the resource. The resource name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] node_affinity_labels
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     Labels to use for node affinity, which will be used in instance scheduling.
        # @!attribute [rw] node_type
        #   @return [::String]
        #     The node type to use for nodes group that are created from this template.
        # @!attribute [rw] node_type_flexibility
        #   @return [::Google::Cloud::Compute::V1::NodeTemplateNodeTypeFlexibility]
        #     Do not use. Instead, use the node_type property.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] The name of the region where the node template resides, such as us-central1.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] server_binding
        #   @return [::Google::Cloud::Compute::V1::ServerBinding]
        #     Sets the binding properties for the physical server. Valid values include: - *[Default]* RESTART_NODE_ON_ANY_SERVER: Restarts VMs on any available physical server - RESTART_NODE_ON_MINIMAL_SERVER: Restarts VMs on the same physical server whenever possible See Sole-tenant node options for more information.
        # @!attribute [rw] status
        #   @return [::String]
        #     [Output Only] The status of the node template. One of the following values: CREATING, READY, and DELETING.
        #     Check the Status enum for the list of possible values.
        # @!attribute [rw] status_message
        #   @return [::String]
        #     [Output Only] An optional, human-readable explanation of the status.
        class NodeTemplate
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class NodeAffinityLabelsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # CPU overcommit.
          module CpuOvercommitType
            # A value indicating that the enum field is not set.
            UNDEFINED_CPU_OVERCOMMIT_TYPE = 0

            CPU_OVERCOMMIT_TYPE_UNSPECIFIED = 520_665_615

            ENABLED = 182_130_465

            NONE = 2_402_104
          end

          # [Output Only] The status of the node template. One of the following values: CREATING, READY, and DELETING.
          module Status
            # A value indicating that the enum field is not set.
            UNDEFINED_STATUS = 0

            # Resources are being allocated.
            CREATING = 455_564_985

            # The node template is currently being deleted.
            DELETING = 528_602_024

            # Invalid status.
            INVALID = 530_283_991

            # The node template is ready.
            READY = 77_848_963
          end
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::NodeTemplatesScopedList}]
        #     A list of NodeTemplatesScopedList resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource.Always compute#nodeTemplateAggregatedList for aggregated lists of node templates.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class NodeTemplateAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::NodeTemplatesScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # Contains a list of node templates.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::NodeTemplate>]
        #     A list of NodeTemplate resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource.Always compute#nodeTemplateList for lists of node templates.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class NodeTemplateList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] cpus
        #   @return [::String]
        # @!attribute [rw] local_ssd
        #   @return [::String]
        # @!attribute [rw] memory
        #   @return [::String]
        class NodeTemplateNodeTypeFlexibility
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] node_templates
        #   @return [::Array<::Google::Cloud::Compute::V1::NodeTemplate>]
        #     [Output Only] A list of node templates contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] An informational warning that appears when the node templates list is empty.
        class NodeTemplatesScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represent a sole-tenant Node Type resource. Each node within a node group must have a node type. A node type specifies the total amount of cores and memory for that node. Currently, the only available node type is n1-node-96-624 node type that has 96 vCPUs and 624 GB of memory, available in multiple zones. For more information read Node types.
        # @!attribute [rw] cpu_platform
        #   @return [::String]
        #     [Output Only] The CPU platform used by this node type.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] deprecated
        #   @return [::Google::Cloud::Compute::V1::DeprecationStatus]
        #     [Output Only] The deprecation status associated with this node type.
        # @!attribute [rw] description
        #   @return [::String]
        #     [Output Only] An optional textual description of the resource.
        # @!attribute [rw] guest_cpus
        #   @return [::Integer]
        #     [Output Only] The number of virtual CPUs that are available to the node type.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] The type of the resource. Always compute#nodeType for node types.
        # @!attribute [rw] local_ssd_gb
        #   @return [::Integer]
        #     [Output Only] Local SSD available to the node type, defined in GB.
        # @!attribute [rw] memory_mb
        #   @return [::Integer]
        #     [Output Only] The amount of physical memory available to the node type, defined in MB.
        # @!attribute [rw] name
        #   @return [::String]
        #     [Output Only] Name of the resource.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] zone
        #   @return [::String]
        #     [Output Only] The name of the zone where the node type resides, such as us-central1-a.
        class NodeType
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::NodeTypesScopedList}]
        #     A list of NodeTypesScopedList resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource.Always compute#nodeTypeAggregatedList for aggregated lists of node types.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class NodeTypeAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::NodeTypesScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # Contains a list of node types.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::NodeType>]
        #     A list of NodeType resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource.Always compute#nodeTypeList for lists of node types.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class NodeTypeList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] node_types
        #   @return [::Array<::Google::Cloud::Compute::V1::NodeType>]
        #     [Output Only] A list of node types contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] An informational warning that appears when the node types list is empty.
        class NodeTypesScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a notification endpoint. A notification endpoint resource defines an endpoint to receive notifications when there are status changes detected by the associated health check service. For more information, see Health checks overview.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] grpc_settings
        #   @return [::Google::Cloud::Compute::V1::NotificationEndpointGrpcSettings]
        #     Settings of the gRPC notification endpoint including the endpoint URL and the retry duration.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] A unique identifier for this resource type. The server generates this identifier.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#notificationEndpoint for notification endpoints.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] URL of the region where the notification endpoint resides. This field applies only to the regional resource. You must specify this field as part of the HTTP request URL. It is not settable as a field in the request body.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        class NotificationEndpoint
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a gRPC setting that describes one gRPC notification endpoint and the retry duration attempting to send notification to this endpoint.
        # @!attribute [rw] authority
        #   @return [::String]
        #     Optional. If specified, this field is used to set the authority header by the sender of notifications. See https://tools.ietf.org/html/rfc7540#section-8.1.2.3
        # @!attribute [rw] endpoint
        #   @return [::String]
        #     Endpoint to which gRPC notifications are sent. This must be a valid gRPCLB DNS name.
        # @!attribute [rw] payload_name
        #   @return [::String]
        #     Optional. If specified, this field is used to populate the "name" field in gRPC requests.
        # @!attribute [rw] resend_interval
        #   @return [::Google::Cloud::Compute::V1::Duration]
        #     Optional. This field is used to configure how often to send a full update of all non-healthy backends. If unspecified, full updates are not sent. If specified, must be in the range between 600 seconds to 3600 seconds. Nanos are disallowed. Can only be set for regional notification endpoints.
        # @!attribute [rw] retry_duration_sec
        #   @return [::Integer]
        #     How much time (in seconds) is spent attempting notification retries until a successful response is received. Default is 30s. Limit is 20m (1200s). Must be a positive number.
        class NotificationEndpointGrpcSettings
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::NotificationEndpoint>]
        #     A list of NotificationEndpoint resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#notificationEndpoint for notification endpoints.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class NotificationEndpointList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents an Operation resource. Google Compute Engine has three Operation resources: * [Global](/compute/docs/reference/rest/v1/globalOperations) * [Regional](/compute/docs/reference/rest/v1/regionOperations) * [Zonal](/compute/docs/reference/rest/v1/zoneOperations) You can use an operation resource to manage asynchronous API requests. For more information, read Handling API responses. Operations can be global, regional or zonal. - For global operations, use the `globalOperations` resource. - For regional operations, use the `regionOperations` resource. - For zonal operations, use the `zoneOperations` resource. For more information, read Global, Regional, and Zonal Resources.
        # @!attribute [rw] client_operation_id
        #   @return [::String]
        #     [Output Only] The value of `requestId` if you provided it in the request. Not present otherwise.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Deprecated] This field is deprecated.
        # @!attribute [rw] description
        #   @return [::String]
        #     [Output Only] A textual description of the operation, which is set when the operation is created.
        # @!attribute [rw] end_time
        #   @return [::String]
        #     [Output Only] The time that this operation was completed. This value is in RFC3339 text format.
        # @!attribute [rw] error
        #   @return [::Google::Cloud::Compute::V1::Error]
        #     [Output Only] If errors are generated during processing of the operation, this field will be populated.
        # @!attribute [rw] http_error_message
        #   @return [::String]
        #     [Output Only] If the operation fails, this field contains the HTTP error message that was returned, such as `NOT FOUND`.
        # @!attribute [rw] http_error_status_code
        #   @return [::Integer]
        #     [Output Only] If the operation fails, this field contains the HTTP error status code that was returned. For example, a `404` means the resource was not found.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the operation. This identifier is defined by the server.
        # @!attribute [rw] insert_time
        #   @return [::String]
        #     [Output Only] The time that this operation was requested. This value is in RFC3339 text format.
        # @!attribute [rw] instances_bulk_insert_operation_metadata
        #   @return [::Google::Cloud::Compute::V1::InstancesBulkInsertOperationMetadata]
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always `compute#operation` for Operation resources.
        # @!attribute [rw] name
        #   @return [::String]
        #     [Output Only] Name of the operation.
        # @!attribute [rw] operation_group_id
        #   @return [::String]
        #     [Output Only] An ID that represents a group of operations, such as when a group of operations results from a `bulkInsert` API request.
        # @!attribute [rw] operation_type
        #   @return [::String]
        #     [Output Only] The type of operation, such as `insert`, `update`, or `delete`, and so on.
        # @!attribute [rw] progress
        #   @return [::Integer]
        #     [Output Only] An optional progress indicator that ranges from 0 to 100. There is no requirement that this be linear or support any granularity of operations. This should not be used to guess when the operation will be complete. This number should monotonically increase as the operation progresses.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] The URL of the region where the operation resides. Only applicable when performing regional operations.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] set_common_instance_metadata_operation_metadata
        #   @return [::Google::Cloud::Compute::V1::SetCommonInstanceMetadataOperationMetadata]
        #     [Output Only] If the operation is for projects.setCommonInstanceMetadata, this field will contain information on all underlying zonal actions and their state.
        # @!attribute [rw] start_time
        #   @return [::String]
        #     [Output Only] The time that this operation was started by the server. This value is in RFC3339 text format.
        # @!attribute [rw] status
        #   @return [::Google::Cloud::Compute::V1::Operation::Status]
        #     [Output Only] The status of the operation, which can be one of the following: `PENDING`, `RUNNING`, or `DONE`.
        # @!attribute [rw] status_message
        #   @return [::String]
        #     [Output Only] An optional textual description of the current status of the operation.
        # @!attribute [rw] target_id
        #   @return [::Integer]
        #     [Output Only] The unique target ID, which identifies a specific incarnation of the target resource.
        # @!attribute [rw] target_link
        #   @return [::String]
        #     [Output Only] The URL of the resource that the operation modifies. For operations related to creating a snapshot, this points to the persistent disk that the snapshot was created from.
        # @!attribute [rw] user
        #   @return [::String]
        #     [Output Only] User who requested the operation, for example: `user@example.com` or `alice_smith_identifier (global/workforcePools/example-com-us-employees)`.
        # @!attribute [rw] warnings
        #   @return [::Array<::Google::Cloud::Compute::V1::Warnings>]
        #     [Output Only] If warning messages are generated during processing of the operation, this field will be populated.
        # @!attribute [rw] zone
        #   @return [::String]
        #     [Output Only] The URL of the zone where the operation resides. Only applicable when performing per-zone operations.
        class Operation
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # [Output Only] The status of the operation, which can be one of the following: `PENDING`, `RUNNING`, or `DONE`.
          module Status
            # A value indicating that the enum field is not set.
            UNDEFINED_STATUS = 0

            DONE = 2_104_194

            PENDING = 35_394_935

            RUNNING = 121_282_975
          end
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::OperationsScopedList}]
        #     [Output Only] A map of scoped operation lists.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always `compute#operationAggregatedList` for aggregated lists of operations.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than `maxResults`, use the `nextPageToken` as a value for the query parameter `pageToken` in the next list request. Subsequent list requests will have their own `nextPageToken` to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class OperationAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::OperationsScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # Contains a list of Operation resources.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::Operation>]
        #     [Output Only] A list of Operation resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always `compute#operations` for Operations resource.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than `maxResults`, use the `nextPageToken` as a value for the query parameter `pageToken` in the next list request. Subsequent list requests will have their own `nextPageToken` to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class OperationList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] operations
        #   @return [::Array<::Google::Cloud::Compute::V1::Operation>]
        #     [Output Only] A list of operations contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning which replaces the list of operations when the list is empty.
        class OperationsScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Settings controlling the eviction of unhealthy hosts from the load balancing pool for the backend service.
        # @!attribute [rw] base_ejection_time
        #   @return [::Google::Cloud::Compute::V1::Duration]
        #     The base time that a backend endpoint is ejected for. Defaults to 30000ms or 30s. After a backend endpoint is returned back to the load balancing pool, it can be ejected again in another ejection analysis. Thus, the total ejection time is equal to the base ejection time multiplied by the number of times the backend endpoint has been ejected. Defaults to 30000ms or 30s.
        # @!attribute [rw] consecutive_errors
        #   @return [::Integer]
        #     Number of consecutive errors before a backend endpoint is ejected from the load balancing pool. When the backend endpoint is accessed over HTTP, a 5xx return code qualifies as an error. Defaults to 5.
        # @!attribute [rw] consecutive_gateway_failure
        #   @return [::Integer]
        #     The number of consecutive gateway failures (502, 503, 504 status or connection errors that are mapped to one of those status codes) before a consecutive gateway failure ejection occurs. Defaults to 3.
        # @!attribute [rw] enforcing_consecutive_errors
        #   @return [::Integer]
        #     The percentage chance that a backend endpoint will be ejected when an outlier status is detected through consecutive 5xx. This setting can be used to disable ejection or to ramp it up slowly. Defaults to 0.
        # @!attribute [rw] enforcing_consecutive_gateway_failure
        #   @return [::Integer]
        #     The percentage chance that a backend endpoint will be ejected when an outlier status is detected through consecutive gateway failures. This setting can be used to disable ejection or to ramp it up slowly. Defaults to 100.
        # @!attribute [rw] enforcing_success_rate
        #   @return [::Integer]
        #     The percentage chance that a backend endpoint will be ejected when an outlier status is detected through success rate statistics. This setting can be used to disable ejection or to ramp it up slowly. Defaults to 100. Not supported when the backend service uses Serverless NEG.
        # @!attribute [rw] interval
        #   @return [::Google::Cloud::Compute::V1::Duration]
        #     Time interval between ejection analysis sweeps. This can result in both new ejections and backend endpoints being returned to service. The interval is equal to the number of seconds as defined in outlierDetection.interval.seconds plus the number of nanoseconds as defined in outlierDetection.interval.nanos. Defaults to 1 second.
        # @!attribute [rw] max_ejection_percent
        #   @return [::Integer]
        #     Maximum percentage of backend endpoints in the load balancing pool for the backend service that can be ejected if the ejection conditions are met. Defaults to 50%.
        # @!attribute [rw] success_rate_minimum_hosts
        #   @return [::Integer]
        #     The number of backend endpoints in the load balancing pool that must have enough request volume to detect success rate outliers. If the number of backend endpoints is fewer than this setting, outlier detection via success rate statistics is not performed for any backend endpoint in the load balancing pool. Defaults to 5. Not supported when the backend service uses Serverless NEG.
        # @!attribute [rw] success_rate_request_volume
        #   @return [::Integer]
        #     The minimum number of total requests that must be collected in one interval (as defined by the interval duration above) to include this backend endpoint in success rate based outlier detection. If the volume is lower than this setting, outlier detection via success rate statistics is not performed for that backend endpoint. Defaults to 100. Not supported when the backend service uses Serverless NEG.
        # @!attribute [rw] success_rate_stdev_factor
        #   @return [::Integer]
        #     This factor is used to determine the ejection threshold for success rate outlier ejection. The ejection threshold is the difference between the mean success rate, and the product of this factor and the standard deviation of the mean success rate: mean - (stdev * successRateStdevFactor). This factor is divided by a thousand to get a double. That is, if the desired factor is 1.9, the runtime value should be 1900. Defaults to 1900. Not supported when the backend service uses Serverless NEG.
        class OutlierDetection
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Next free: 7
        # @!attribute [rw] avg_ms
        #   @return [::Integer]
        #     Average observed inter-packet interval in milliseconds.
        # @!attribute [rw] duration
        #   @return [::String]
        #     From how long ago in the past these intervals were observed.
        #     Check the Duration enum for the list of possible values.
        # @!attribute [rw] max_ms
        #   @return [::Integer]
        #     Maximum observed inter-packet interval in milliseconds.
        # @!attribute [rw] min_ms
        #   @return [::Integer]
        #     Minimum observed inter-packet interval in milliseconds.
        # @!attribute [rw] num_intervals
        #   @return [::Integer]
        #     Number of inter-packet intervals from which these statistics were derived.
        # @!attribute [rw] type
        #   @return [::String]
        #     The type of packets for which inter-packet intervals were computed.
        #     Check the Type enum for the list of possible values.
        class PacketIntervals
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # From how long ago in the past these intervals were observed.
          module Duration
            # A value indicating that the enum field is not set.
            UNDEFINED_DURATION = 0

            DURATION_UNSPECIFIED = 529_071_340

            HOUR = 2_223_588

            # From BfdSession object creation time.
            MAX = 76_100

            MINUTE = 126_786_068
          end

          # The type of packets for which inter-packet intervals were computed.
          module Type
            # A value indicating that the enum field is not set.
            UNDEFINED_TYPE = 0

            # Only applies to Echo packets. This shows the intervals between sending and receiving the same packet.
            LOOPBACK = 356_174_219

            # Intervals between received packets.
            RECEIVE = 189_660_867

            # Intervals between transmitted packets.
            TRANSMIT = 452_903_600

            TYPE_UNSPECIFIED = 437_714_322
          end
        end

        # Represents a Packet Mirroring resource. Packet Mirroring clones the traffic of specified instances in your Virtual Private Cloud (VPC) network and forwards it to a collector destination, such as an instance group of an internal TCP/UDP load balancer, for analysis or examination. For more information about setting up Packet Mirroring, see Using Packet Mirroring.
        # @!attribute [rw] collector_ilb
        #   @return [::Google::Cloud::Compute::V1::PacketMirroringForwardingRuleInfo]
        #     The Forwarding Rule resource of type loadBalancingScheme=INTERNAL that will be used as collector for mirrored traffic. The specified forwarding rule must have isMirroringCollector set to true.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] enable
        #   @return [::String]
        #     Indicates whether or not this packet mirroring takes effect. If set to FALSE, this packet mirroring policy will not be enforced on the network. The default is TRUE.
        #     Check the Enable enum for the list of possible values.
        # @!attribute [rw] filter
        #   @return [::Google::Cloud::Compute::V1::PacketMirroringFilter]
        #     Filter for mirrored traffic. If unspecified, all traffic is mirrored.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#packetMirroring for packet mirrorings.
        # @!attribute [rw] mirrored_resources
        #   @return [::Google::Cloud::Compute::V1::PacketMirroringMirroredResourceInfo]
        #     PacketMirroring mirroredResourceInfos. MirroredResourceInfo specifies a set of mirrored VM instances, subnetworks and/or tags for which traffic from/to all VM instances will be mirrored.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource; provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] network
        #   @return [::Google::Cloud::Compute::V1::PacketMirroringNetworkInfo]
        #     Specifies the mirrored VPC network. Only packets in this network will be mirrored. All mirrored VMs should have a NIC in the given network. All mirrored subnetworks should belong to the given network.
        # @!attribute [rw] priority
        #   @return [::Integer]
        #     The priority of applying this configuration. Priority is used to break ties in cases where there is more than one matching rule. In the case of two rules that apply for a given Instance, the one with the lowest-numbered priority value wins. Default value is 1000. Valid range is 0 through 65535.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] URI of the region where the packetMirroring resides.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        class PacketMirroring
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Indicates whether or not this packet mirroring takes effect. If set to FALSE, this packet mirroring policy will not be enforced on the network. The default is TRUE.
          module Enable
            # A value indicating that the enum field is not set.
            UNDEFINED_ENABLE = 0

            FALSE = 66_658_563

            TRUE = 2_583_950
          end
        end

        # Contains a list of packetMirrorings.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::PacketMirroringsScopedList}]
        #     A list of PacketMirroring resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     Type of resource.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class PacketMirroringAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::PacketMirroringsScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # @!attribute [rw] I_p_protocols
        #   @return [::Array<::String>]
        #     Protocols that apply as filter on mirrored traffic. If no protocols are specified, all traffic that matches the specified CIDR ranges is mirrored. If neither cidrRanges nor IPProtocols is specified, all traffic is mirrored.
        # @!attribute [rw] cidr_ranges
        #   @return [::Array<::String>]
        #     IP CIDR ranges that apply as filter on the source (ingress) or destination (egress) IP in the IP header. Only IPv4 is supported. If no ranges are specified, all traffic that matches the specified IPProtocols is mirrored. If neither cidrRanges nor IPProtocols is specified, all traffic is mirrored.
        # @!attribute [rw] direction
        #   @return [::String]
        #     Direction of traffic to mirror, either INGRESS, EGRESS, or BOTH. The default is BOTH.
        #     Check the Direction enum for the list of possible values.
        class PacketMirroringFilter
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Direction of traffic to mirror, either INGRESS, EGRESS, or BOTH. The default is BOTH.
          module Direction
            # A value indicating that the enum field is not set.
            UNDEFINED_DIRECTION = 0

            # Default, both directions are mirrored.
            BOTH = 2_044_801

            # Only egress traffic is mirrored.
            EGRESS = 432_880_501

            # Only ingress traffic is mirrored.
            INGRESS = 516_931_221
          end
        end

        # @!attribute [rw] canonical_url
        #   @return [::String]
        #     [Output Only] Unique identifier for the forwarding rule; defined by the server.
        # @!attribute [rw] url
        #   @return [::String]
        #     Resource URL to the forwarding rule representing the ILB configured as destination of the mirrored traffic.
        class PacketMirroringForwardingRuleInfo
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Contains a list of PacketMirroring resources.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::PacketMirroring>]
        #     A list of PacketMirroring resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#packetMirroring for packetMirrorings.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class PacketMirroringList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] instances
        #   @return [::Array<::Google::Cloud::Compute::V1::PacketMirroringMirroredResourceInfoInstanceInfo>]
        #     A set of virtual machine instances that are being mirrored. They must live in zones contained in the same region as this packetMirroring. Note that this config will apply only to those network interfaces of the Instances that belong to the network specified in this packetMirroring. You may specify a maximum of 50 Instances.
        # @!attribute [rw] subnetworks
        #   @return [::Array<::Google::Cloud::Compute::V1::PacketMirroringMirroredResourceInfoSubnetInfo>]
        #     A set of subnetworks for which traffic from/to all VM instances will be mirrored. They must live in the same region as this packetMirroring. You may specify a maximum of 5 subnetworks.
        # @!attribute [rw] tags
        #   @return [::Array<::String>]
        #     A set of mirrored tags. Traffic from/to all VM instances that have one or more of these tags will be mirrored.
        class PacketMirroringMirroredResourceInfo
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] canonical_url
        #   @return [::String]
        #     [Output Only] Unique identifier for the instance; defined by the server.
        # @!attribute [rw] url
        #   @return [::String]
        #     Resource URL to the virtual machine instance which is being mirrored.
        class PacketMirroringMirroredResourceInfoInstanceInfo
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] canonical_url
        #   @return [::String]
        #     [Output Only] Unique identifier for the subnetwork; defined by the server.
        # @!attribute [rw] url
        #   @return [::String]
        #     Resource URL to the subnetwork for which traffic from/to all VM instances will be mirrored.
        class PacketMirroringMirroredResourceInfoSubnetInfo
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] canonical_url
        #   @return [::String]
        #     [Output Only] Unique identifier for the network; defined by the server.
        # @!attribute [rw] url
        #   @return [::String]
        #     URL of the network resource.
        class PacketMirroringNetworkInfo
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] packet_mirrorings
        #   @return [::Array<::Google::Cloud::Compute::V1::PacketMirroring>]
        #     A list of packetMirrorings contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     Informational warning which replaces the list of packetMirrorings when the list is empty.
        class PacketMirroringsScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Autoscalers.Patch. See the method description for details.
        # @!attribute [rw] autoscaler
        #   @return [::String]
        #     Name of the autoscaler to patch.
        # @!attribute [rw] autoscaler_resource
        #   @return [::Google::Cloud::Compute::V1::Autoscaler]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     Name of the zone for this request.
        class PatchAutoscalerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for BackendBuckets.Patch. See the method description for details.
        # @!attribute [rw] backend_bucket
        #   @return [::String]
        #     Name of the BackendBucket resource to patch.
        # @!attribute [rw] backend_bucket_resource
        #   @return [::Google::Cloud::Compute::V1::BackendBucket]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class PatchBackendBucketRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for BackendServices.Patch. See the method description for details.
        # @!attribute [rw] backend_service
        #   @return [::String]
        #     Name of the BackendService resource to patch.
        # @!attribute [rw] backend_service_resource
        #   @return [::Google::Cloud::Compute::V1::BackendService]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class PatchBackendServiceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for FirewallPolicies.Patch. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to update.
        # @!attribute [rw] firewall_policy_resource
        #   @return [::Google::Cloud::Compute::V1::FirewallPolicy]
        #     The body resource for this request
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class PatchFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Firewalls.Patch. See the method description for details.
        # @!attribute [rw] firewall
        #   @return [::String]
        #     Name of the firewall rule to patch.
        # @!attribute [rw] firewall_resource
        #   @return [::Google::Cloud::Compute::V1::Firewall]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class PatchFirewallRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ForwardingRules.Patch. See the method description for details.
        # @!attribute [rw] forwarding_rule
        #   @return [::String]
        #     Name of the ForwardingRule resource to patch.
        # @!attribute [rw] forwarding_rule_resource
        #   @return [::Google::Cloud::Compute::V1::ForwardingRule]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class PatchForwardingRuleRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for GlobalForwardingRules.Patch. See the method description for details.
        # @!attribute [rw] forwarding_rule
        #   @return [::String]
        #     Name of the ForwardingRule resource to patch.
        # @!attribute [rw] forwarding_rule_resource
        #   @return [::Google::Cloud::Compute::V1::ForwardingRule]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class PatchGlobalForwardingRuleRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for GlobalPublicDelegatedPrefixes.Patch. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] public_delegated_prefix
        #   @return [::String]
        #     Name of the PublicDelegatedPrefix resource to patch.
        # @!attribute [rw] public_delegated_prefix_resource
        #   @return [::Google::Cloud::Compute::V1::PublicDelegatedPrefix]
        #     The body resource for this request
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class PatchGlobalPublicDelegatedPrefixeRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for HealthChecks.Patch. See the method description for details.
        # @!attribute [rw] health_check
        #   @return [::String]
        #     Name of the HealthCheck resource to patch.
        # @!attribute [rw] health_check_resource
        #   @return [::Google::Cloud::Compute::V1::HealthCheck]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class PatchHealthCheckRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Images.Patch. See the method description for details.
        # @!attribute [rw] image
        #   @return [::String]
        #     Name of the image resource to patch.
        # @!attribute [rw] image_resource
        #   @return [::Google::Cloud::Compute::V1::Image]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class PatchImageRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceGroupManagers.Patch. See the method description for details.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     The name of the instance group manager.
        # @!attribute [rw] instance_group_manager_resource
        #   @return [::Google::Cloud::Compute::V1::InstanceGroupManager]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where you want to create the managed instance group.
        class PatchInstanceGroupManagerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InterconnectAttachments.Patch. See the method description for details.
        # @!attribute [rw] interconnect_attachment
        #   @return [::String]
        #     Name of the interconnect attachment to patch.
        # @!attribute [rw] interconnect_attachment_resource
        #   @return [::Google::Cloud::Compute::V1::InterconnectAttachment]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class PatchInterconnectAttachmentRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Interconnects.Patch. See the method description for details.
        # @!attribute [rw] interconnect
        #   @return [::String]
        #     Name of the interconnect to update.
        # @!attribute [rw] interconnect_resource
        #   @return [::Google::Cloud::Compute::V1::Interconnect]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class PatchInterconnectRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkAttachments.Patch. See the method description for details.
        # @!attribute [rw] network_attachment
        #   @return [::String]
        #     Name of the NetworkAttachment resource to patch.
        # @!attribute [rw] network_attachment_resource
        #   @return [::Google::Cloud::Compute::V1::NetworkAttachment]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000). end_interface: MixerMutationRequestBuilder
        class PatchNetworkAttachmentRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkEdgeSecurityServices.Patch. See the method description for details.
        # @!attribute [rw] network_edge_security_service
        #   @return [::String]
        #     Name of the network edge security service to update.
        # @!attribute [rw] network_edge_security_service_resource
        #   @return [::Google::Cloud::Compute::V1::NetworkEdgeSecurityService]
        #     The body resource for this request
        # @!attribute [rw] paths
        #   @return [::String]
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] update_mask
        #   @return [::String]
        #     Indicates fields to be updated as part of this request.
        class PatchNetworkEdgeSecurityServiceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkFirewallPolicies.Patch. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to update.
        # @!attribute [rw] firewall_policy_resource
        #   @return [::Google::Cloud::Compute::V1::FirewallPolicy]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class PatchNetworkFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Networks.Patch. See the method description for details.
        # @!attribute [rw] network
        #   @return [::String]
        #     Name of the network to update.
        # @!attribute [rw] network_resource
        #   @return [::Google::Cloud::Compute::V1::Network]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class PatchNetworkRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NodeGroups.Patch. See the method description for details.
        # @!attribute [rw] node_group
        #   @return [::String]
        #     Name of the NodeGroup resource to update.
        # @!attribute [rw] node_group_resource
        #   @return [::Google::Cloud::Compute::V1::NodeGroup]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class PatchNodeGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for PacketMirrorings.Patch. See the method description for details.
        # @!attribute [rw] packet_mirroring
        #   @return [::String]
        #     Name of the PacketMirroring resource to patch.
        # @!attribute [rw] packet_mirroring_resource
        #   @return [::Google::Cloud::Compute::V1::PacketMirroring]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class PatchPacketMirroringRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceGroupManagers.PatchPerInstanceConfigs. See the method description for details.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     The name of the managed instance group. It should conform to RFC1035.
        # @!attribute [rw] instance_group_managers_patch_per_instance_configs_req_resource
        #   @return [::Google::Cloud::Compute::V1::InstanceGroupManagersPatchPerInstanceConfigsReq]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where the managed instance group is located. It should conform to RFC1035.
        class PatchPerInstanceConfigsInstanceGroupManagerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionInstanceGroupManagers.PatchPerInstanceConfigs. See the method description for details.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     The name of the managed instance group. It should conform to RFC1035.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request, should conform to RFC1035.
        # @!attribute [rw] region_instance_group_manager_patch_instance_config_req_resource
        #   @return [::Google::Cloud::Compute::V1::RegionInstanceGroupManagerPatchInstanceConfigReq]
        #     The body resource for this request
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class PatchPerInstanceConfigsRegionInstanceGroupManagerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for PublicAdvertisedPrefixes.Patch. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] public_advertised_prefix
        #   @return [::String]
        #     Name of the PublicAdvertisedPrefix resource to patch.
        # @!attribute [rw] public_advertised_prefix_resource
        #   @return [::Google::Cloud::Compute::V1::PublicAdvertisedPrefix]
        #     The body resource for this request
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class PatchPublicAdvertisedPrefixeRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for PublicDelegatedPrefixes.Patch. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] public_delegated_prefix
        #   @return [::String]
        #     Name of the PublicDelegatedPrefix resource to patch.
        # @!attribute [rw] public_delegated_prefix_resource
        #   @return [::Google::Cloud::Compute::V1::PublicDelegatedPrefix]
        #     The body resource for this request
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class PatchPublicDelegatedPrefixeRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionAutoscalers.Patch. See the method description for details.
        # @!attribute [rw] autoscaler
        #   @return [::String]
        #     Name of the autoscaler to patch.
        # @!attribute [rw] autoscaler_resource
        #   @return [::Google::Cloud::Compute::V1::Autoscaler]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class PatchRegionAutoscalerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionBackendServices.Patch. See the method description for details.
        # @!attribute [rw] backend_service
        #   @return [::String]
        #     Name of the BackendService resource to patch.
        # @!attribute [rw] backend_service_resource
        #   @return [::Google::Cloud::Compute::V1::BackendService]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class PatchRegionBackendServiceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionHealthChecks.Patch. See the method description for details.
        # @!attribute [rw] health_check
        #   @return [::String]
        #     Name of the HealthCheck resource to patch.
        # @!attribute [rw] health_check_resource
        #   @return [::Google::Cloud::Compute::V1::HealthCheck]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class PatchRegionHealthCheckRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionHealthCheckServices.Patch. See the method description for details.
        # @!attribute [rw] health_check_service
        #   @return [::String]
        #     Name of the HealthCheckService to update. The name must be 1-63 characters long, and comply with RFC1035.
        # @!attribute [rw] health_check_service_resource
        #   @return [::Google::Cloud::Compute::V1::HealthCheckService]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class PatchRegionHealthCheckServiceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionInstanceGroupManagers.Patch. See the method description for details.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     The name of the instance group manager.
        # @!attribute [rw] instance_group_manager_resource
        #   @return [::Google::Cloud::Compute::V1::InstanceGroupManager]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class PatchRegionInstanceGroupManagerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionNetworkFirewallPolicies.Patch. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to update.
        # @!attribute [rw] firewall_policy_resource
        #   @return [::Google::Cloud::Compute::V1::FirewallPolicy]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class PatchRegionNetworkFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionSecurityPolicies.Patch. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] security_policy
        #   @return [::String]
        #     Name of the security policy to update.
        # @!attribute [rw] security_policy_resource
        #   @return [::Google::Cloud::Compute::V1::SecurityPolicy]
        #     The body resource for this request
        # @!attribute [rw] update_mask
        #   @return [::String]
        #     Indicates fields to be cleared as part of this request.
        class PatchRegionSecurityPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionSslPolicies.Patch. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] ssl_policy
        #   @return [::String]
        #     Name of the SSL policy to update. The name must be 1-63 characters long, and comply with RFC1035.
        # @!attribute [rw] ssl_policy_resource
        #   @return [::Google::Cloud::Compute::V1::SslPolicy]
        #     The body resource for this request
        class PatchRegionSslPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionTargetHttpsProxies.Patch. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_https_proxy
        #   @return [::String]
        #     Name of the TargetHttpsProxy resource to patch.
        # @!attribute [rw] target_https_proxy_resource
        #   @return [::Google::Cloud::Compute::V1::TargetHttpsProxy]
        #     The body resource for this request
        class PatchRegionTargetHttpsProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionUrlMaps.Patch. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     begin_interface: MixerMutationRequestBuilder Request ID to support idempotency.
        # @!attribute [rw] url_map
        #   @return [::String]
        #     Name of the UrlMap resource to patch.
        # @!attribute [rw] url_map_resource
        #   @return [::Google::Cloud::Compute::V1::UrlMap]
        #     The body resource for this request
        class PatchRegionUrlMapRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ResourcePolicies.Patch. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] resource_policy
        #   @return [::String]
        #     Id of the resource policy to patch.
        # @!attribute [rw] resource_policy_resource
        #   @return [::Google::Cloud::Compute::V1::ResourcePolicy]
        #     The body resource for this request
        # @!attribute [rw] update_mask
        #   @return [::String]
        #     update_mask indicates fields to be updated as part of this request.
        class PatchResourcePolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Routers.Patch. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] router
        #   @return [::String]
        #     Name of the Router resource to patch.
        # @!attribute [rw] router_resource
        #   @return [::Google::Cloud::Compute::V1::Router]
        #     The body resource for this request
        class PatchRouterRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for FirewallPolicies.PatchRule. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to update.
        # @!attribute [rw] firewall_policy_rule_resource
        #   @return [::Google::Cloud::Compute::V1::FirewallPolicyRule]
        #     The body resource for this request
        # @!attribute [rw] priority
        #   @return [::Integer]
        #     The priority of the rule to patch.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class PatchRuleFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkFirewallPolicies.PatchRule. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to update.
        # @!attribute [rw] firewall_policy_rule_resource
        #   @return [::Google::Cloud::Compute::V1::FirewallPolicyRule]
        #     The body resource for this request
        # @!attribute [rw] priority
        #   @return [::Integer]
        #     The priority of the rule to patch.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class PatchRuleNetworkFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionNetworkFirewallPolicies.PatchRule. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to update.
        # @!attribute [rw] firewall_policy_rule_resource
        #   @return [::Google::Cloud::Compute::V1::FirewallPolicyRule]
        #     The body resource for this request
        # @!attribute [rw] priority
        #   @return [::Integer]
        #     The priority of the rule to patch.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class PatchRuleRegionNetworkFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionSecurityPolicies.PatchRule. See the method description for details.
        # @!attribute [rw] priority
        #   @return [::Integer]
        #     The priority of the rule to patch.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] security_policy
        #   @return [::String]
        #     Name of the security policy to update.
        # @!attribute [rw] security_policy_rule_resource
        #   @return [::Google::Cloud::Compute::V1::SecurityPolicyRule]
        #     The body resource for this request
        # @!attribute [rw] update_mask
        #   @return [::String]
        #     Indicates fields to be cleared as part of this request.
        # @!attribute [rw] validate_only
        #   @return [::Boolean]
        #     If true, the request will not be committed.
        class PatchRuleRegionSecurityPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for SecurityPolicies.PatchRule. See the method description for details.
        # @!attribute [rw] priority
        #   @return [::Integer]
        #     The priority of the rule to patch.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] security_policy
        #   @return [::String]
        #     Name of the security policy to update.
        # @!attribute [rw] security_policy_rule_resource
        #   @return [::Google::Cloud::Compute::V1::SecurityPolicyRule]
        #     The body resource for this request
        # @!attribute [rw] update_mask
        #   @return [::String]
        #     Indicates fields to be cleared as part of this request.
        # @!attribute [rw] validate_only
        #   @return [::Boolean]
        #     If true, the request will not be committed.
        class PatchRuleSecurityPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for SecurityPolicies.Patch. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] security_policy
        #   @return [::String]
        #     Name of the security policy to update.
        # @!attribute [rw] security_policy_resource
        #   @return [::Google::Cloud::Compute::V1::SecurityPolicy]
        #     The body resource for this request
        # @!attribute [rw] update_mask
        #   @return [::String]
        #     Indicates fields to be cleared as part of this request.
        class PatchSecurityPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ServiceAttachments.Patch. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The region scoping this request and should conform to RFC1035.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] service_attachment
        #   @return [::String]
        #     The resource id of the ServiceAttachment to patch. It should conform to RFC1035 resource name or be a string form on an unsigned long number.
        # @!attribute [rw] service_attachment_resource
        #   @return [::Google::Cloud::Compute::V1::ServiceAttachment]
        #     The body resource for this request
        class PatchServiceAttachmentRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for SnapshotSettingsService.Patch. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] snapshot_settings_resource
        #   @return [::Google::Cloud::Compute::V1::SnapshotSettings]
        #     The body resource for this request
        # @!attribute [rw] update_mask
        #   @return [::String]
        #     update_mask indicates fields to be updated as part of this request.
        class PatchSnapshotSettingRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for SslPolicies.Patch. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] ssl_policy
        #   @return [::String]
        #     Name of the SSL policy to update. The name must be 1-63 characters long, and comply with RFC1035.
        # @!attribute [rw] ssl_policy_resource
        #   @return [::Google::Cloud::Compute::V1::SslPolicy]
        #     The body resource for this request
        class PatchSslPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Subnetworks.Patch. See the method description for details.
        # @!attribute [rw] drain_timeout_seconds
        #   @return [::Integer]
        #     The drain timeout specifies the upper bound in seconds on the amount of time allowed to drain connections from the current ACTIVE subnetwork to the current BACKUP subnetwork. The drain timeout is only applicable when the following conditions are true: - the subnetwork being patched has purpose = INTERNAL_HTTPS_LOAD_BALANCER - the subnetwork being patched has role = BACKUP - the patch request is setting the role to ACTIVE. Note that after this patch operation the roles of the ACTIVE and BACKUP subnetworks will be swapped.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] subnetwork
        #   @return [::String]
        #     Name of the Subnetwork resource to patch.
        # @!attribute [rw] subnetwork_resource
        #   @return [::Google::Cloud::Compute::V1::Subnetwork]
        #     The body resource for this request
        class PatchSubnetworkRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetGrpcProxies.Patch. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_grpc_proxy
        #   @return [::String]
        #     Name of the TargetGrpcProxy resource to patch.
        # @!attribute [rw] target_grpc_proxy_resource
        #   @return [::Google::Cloud::Compute::V1::TargetGrpcProxy]
        #     The body resource for this request
        class PatchTargetGrpcProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetHttpProxies.Patch. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_http_proxy
        #   @return [::String]
        #     Name of the TargetHttpProxy resource to patch.
        # @!attribute [rw] target_http_proxy_resource
        #   @return [::Google::Cloud::Compute::V1::TargetHttpProxy]
        #     The body resource for this request
        class PatchTargetHttpProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetHttpsProxies.Patch. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_https_proxy
        #   @return [::String]
        #     Name of the TargetHttpsProxy resource to patch.
        # @!attribute [rw] target_https_proxy_resource
        #   @return [::Google::Cloud::Compute::V1::TargetHttpsProxy]
        #     The body resource for this request
        class PatchTargetHttpsProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for UrlMaps.Patch. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] url_map
        #   @return [::String]
        #     Name of the UrlMap resource to patch.
        # @!attribute [rw] url_map_resource
        #   @return [::Google::Cloud::Compute::V1::UrlMap]
        #     The body resource for this request
        class PatchUrlMapRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A matcher for the path portion of the URL. The BackendService from the longest-matched rule will serve the URL. If no rule was matched, the default service is used.
        # @!attribute [rw] default_route_action
        #   @return [::Google::Cloud::Compute::V1::HttpRouteAction]
        #     defaultRouteAction takes effect when none of the pathRules or routeRules match. The load balancer performs advanced routing actions, such as URL rewrites and header transformations, before forwarding the request to the selected backend. If defaultRouteAction specifies any weightedBackendServices, defaultService must not be set. Conversely if defaultService is set, defaultRouteAction cannot contain any weightedBackendServices. Only one of defaultRouteAction or defaultUrlRedirect must be set. URL maps for classic Application Load Balancers only support the urlRewrite action within a path matcher's defaultRouteAction.
        # @!attribute [rw] default_service
        #   @return [::String]
        #     The full or partial URL to the BackendService resource. This URL is used if none of the pathRules or routeRules defined by this PathMatcher are matched. For example, the following are all valid URLs to a BackendService resource: - https://www.googleapis.com/compute/v1/projects/project /global/backendServices/backendService - compute/v1/projects/project/global/backendServices/backendService - global/backendServices/backendService If defaultRouteAction is also specified, advanced routing actions, such as URL rewrites, take effect before sending the request to the backend. However, if defaultService is specified, defaultRouteAction cannot contain any weightedBackendServices. Conversely, if defaultRouteAction specifies any weightedBackendServices, defaultService must not be specified. Only one of defaultService, defaultUrlRedirect , or defaultRouteAction.weightedBackendService must be set. Authorization requires one or more of the following Google IAM permissions on the specified resource default_service: - compute.backendBuckets.use - compute.backendServices.use
        # @!attribute [rw] default_url_redirect
        #   @return [::Google::Cloud::Compute::V1::HttpRedirectAction]
        #     When none of the specified pathRules or routeRules match, the request is redirected to a URL specified by defaultUrlRedirect. If defaultUrlRedirect is specified, defaultService or defaultRouteAction must not be set. Not supported when the URL map is bound to a target gRPC proxy.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] header_action
        #   @return [::Google::Cloud::Compute::V1::HttpHeaderAction]
        #     Specifies changes to request and response headers that need to take effect for the selected backend service. HeaderAction specified here are applied after the matching HttpRouteRule HeaderAction and before the HeaderAction in the UrlMap HeaderAction is not supported for load balancers that have their loadBalancingScheme set to EXTERNAL. Not supported when the URL map is bound to a target gRPC proxy that has validateForProxyless field set to true.
        # @!attribute [rw] name
        #   @return [::String]
        #     The name to which this PathMatcher is referred by the HostRule.
        # @!attribute [rw] path_rules
        #   @return [::Array<::Google::Cloud::Compute::V1::PathRule>]
        #     The list of path rules. Use this list instead of routeRules when routing based on simple path matching is all that's required. The order by which path rules are specified does not matter. Matches are always done on the longest-path-first basis. For example: a pathRule with a path /a/b/c/* will match before /a/b/* irrespective of the order in which those paths appear in this list. Within a given pathMatcher, only one of pathRules or routeRules must be set.
        # @!attribute [rw] route_rules
        #   @return [::Array<::Google::Cloud::Compute::V1::HttpRouteRule>]
        #     The list of HTTP route rules. Use this list instead of pathRules when advanced route matching and routing actions are desired. routeRules are evaluated in order of priority, from the lowest to highest number. Within a given pathMatcher, you can set only one of pathRules or routeRules.
        class PathMatcher
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A path-matching rule for a URL. If matched, will use the specified BackendService to handle the traffic arriving at this URL.
        # @!attribute [rw] paths
        #   @return [::Array<::String>]
        #     The list of path patterns to match. Each must start with / and the only place a * is allowed is at the end following a /. The string fed to the path matcher does not include any text after the first ? or #, and those chars are not allowed here.
        # @!attribute [rw] route_action
        #   @return [::Google::Cloud::Compute::V1::HttpRouteAction]
        #     In response to a matching path, the load balancer performs advanced routing actions, such as URL rewrites and header transformations, before forwarding the request to the selected backend. If routeAction specifies any weightedBackendServices, service must not be set. Conversely if service is set, routeAction cannot contain any weightedBackendServices. Only one of routeAction or urlRedirect must be set. URL maps for classic Application Load Balancers only support the urlRewrite action within a path rule's routeAction.
        # @!attribute [rw] service
        #   @return [::String]
        #     The full or partial URL of the backend service resource to which traffic is directed if this rule is matched. If routeAction is also specified, advanced routing actions, such as URL rewrites, take effect before sending the request to the backend. However, if service is specified, routeAction cannot contain any weightedBackendServices. Conversely, if routeAction specifies any weightedBackendServices, service must not be specified. Only one of urlRedirect, service or routeAction.weightedBackendService must be set.
        # @!attribute [rw] url_redirect
        #   @return [::Google::Cloud::Compute::V1::HttpRedirectAction]
        #     When a path pattern is matched, the request is redirected to a URL specified by urlRedirect. If urlRedirect is specified, service or routeAction must not be set. Not supported when the URL map is bound to a target gRPC proxy.
        class PathRule
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] fingerprint
        #   @return [::String]
        #     Fingerprint of this per-instance config. This field can be used in optimistic locking. It is ignored when inserting a per-instance config. An up-to-date fingerprint must be provided in order to update an existing per-instance configuration or the field needs to be unset.
        # @!attribute [rw] name
        #   @return [::String]
        #     The name of a per-instance configuration and its corresponding instance. Serves as a merge key during UpdatePerInstanceConfigs operations, that is, if a per-instance configuration with the same name exists then it will be updated, otherwise a new one will be created for the VM instance with the same name. An attempt to create a per-instance configconfiguration for a VM instance that either doesn't exist or is not part of the group will result in an error.
        # @!attribute [rw] preserved_state
        #   @return [::Google::Cloud::Compute::V1::PreservedState]
        #     The intended preserved state for the given instance. Does not contain preserved state generated from a stateful policy.
        # @!attribute [rw] status
        #   @return [::String]
        #     The status of applying this per-instance configuration on the corresponding managed instance.
        #     Check the Status enum for the list of possible values.
        class PerInstanceConfig
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The status of applying this per-instance configuration on the corresponding managed instance.
          module Status
            # A value indicating that the enum field is not set.
            UNDEFINED_STATUS = 0

            # The per-instance configuration is being applied to the instance, but is not yet effective, possibly waiting for the instance to, for example, REFRESH.
            APPLYING = 352_003_508

            # The per-instance configuration deletion is being applied on the instance, possibly waiting for the instance to, for example, REFRESH.
            DELETING = 528_602_024

            # The per-instance configuration is effective on the instance, meaning that all disks, ips and metadata specified in this configuration are attached or set on the instance.
            EFFECTIVE = 244_201_863

            # *[Default]* The default status, when no per-instance configuration exists.
            NONE = 2_402_104

            # The per-instance configuration is set on an instance but not been applied yet.
            UNAPPLIED = 483_935_140

            # The per-instance configuration has been deleted, but the deletion is not yet applied.
            UNAPPLIED_DELETION = 313_956_873
          end
        end

        # An Identity and Access Management (IAM) policy, which specifies access controls for Google Cloud resources. A `Policy` is a collection of `bindings`. A `binding` binds one or more `members`, or principals, to a single `role`. Principals can be user accounts, service accounts, Google groups, and domains (such as G Suite). A `role` is a named list of permissions; each `role` can be an IAM predefined role or a user-created custom role. For some types of Google Cloud resources, a `binding` can also specify a `condition`, which is a logical expression that allows access to a resource only if the expression evaluates to `true`. A condition can add constraints based on attributes of the request, the resource, or both. To learn which resources support conditions in their IAM policies, see the [IAM documentation](https://cloud.google.com/iam/help/conditions/resource-policies). **JSON example:** ``` { "bindings": [ { "role": "roles/resourcemanager.organizationAdmin", "members": [ "user:mike@example.com", "group:admins@example.com", "domain:google.com", "serviceAccount:my-project-id@appspot.gserviceaccount.com" ] }, { "role": "roles/resourcemanager.organizationViewer", "members": [ "user:eve@example.com" ], "condition": { "title": "expirable access", "description": "Does not grant access after Sep 2020", "expression": "request.time < timestamp('2020-10-01T00:00:00.000Z')", } } ], "etag": "BwWWja0YfJA=", "version": 3 } ``` **YAML example:** ``` bindings: - members: - user:mike@example.com - group:admins@example.com - domain:google.com - serviceAccount:my-project-id@appspot.gserviceaccount.com role: roles/resourcemanager.organizationAdmin - members: - user:eve@example.com role: roles/resourcemanager.organizationViewer condition: title: expirable access description: Does not grant access after Sep 2020 expression: request.time < timestamp('2020-10-01T00:00:00.000Z') etag: BwWWja0YfJA= version: 3 ``` For a description of IAM and its features, see the [IAM documentation](https://cloud.google.com/iam/docs/).
        # @!attribute [rw] audit_configs
        #   @return [::Array<::Google::Cloud::Compute::V1::AuditConfig>]
        #     Specifies cloud audit logging configuration for this policy.
        # @!attribute [rw] bindings
        #   @return [::Array<::Google::Cloud::Compute::V1::Binding>]
        #     Associates a list of `members`, or principals, with a `role`. Optionally, may specify a `condition` that determines how and when the `bindings` are applied. Each of the `bindings` must contain at least one principal. The `bindings` in a `Policy` can refer to up to 1,500 principals; up to 250 of these principals can be Google groups. Each occurrence of a principal counts towards these limits. For example, if the `bindings` grant 50 different roles to `user:alice@example.com`, and not to any other principal, then you can add another 1,450 principals to the `bindings` in the `Policy`.
        # @!attribute [rw] etag
        #   @return [::String]
        #     `etag` is used for optimistic concurrency control as a way to help prevent simultaneous updates of a policy from overwriting each other. It is strongly suggested that systems make use of the `etag` in the read-modify-write cycle to perform policy updates in order to avoid race conditions: An `etag` is returned in the response to `getIamPolicy`, and systems are expected to put that etag in the request to `setIamPolicy` to ensure that their change will be applied to the same version of the policy. **Important:** If you use IAM Conditions, you must include the `etag` field whenever you call `setIamPolicy`. If you omit this field, then IAM allows you to overwrite a version `3` policy with a version `1` policy, and all of the conditions in the version `3` policy are lost.
        # @!attribute [rw] iam_owned
        #   @return [::Boolean]
        # @!attribute [rw] rules
        #   @return [::Array<::Google::Cloud::Compute::V1::Rule>]
        #     This is deprecated and has no effect. Do not use.
        # @!attribute [rw] version
        #   @return [::Integer]
        #     Specifies the format of the policy. Valid values are `0`, `1`, and `3`. Requests that specify an invalid value are rejected. Any operation that affects conditional role bindings must specify version `3`. This requirement applies to the following operations: * Getting a policy that includes a conditional role binding * Adding a conditional role binding to a policy * Changing a conditional role binding in a policy * Removing any role binding, with or without a condition, from a policy that includes conditions **Important:** If you use IAM Conditions, you must include the `etag` field whenever you call `setIamPolicy`. If you omit this field, then IAM allows you to overwrite a version `3` policy with a version `1` policy, and all of the conditions in the version `3` policy are lost. If a policy does not include any conditions, operations on that policy may specify any valid version or leave the field unset. To learn which resources support conditions in their IAM policies, see the [IAM documentation](https://cloud.google.com/iam/help/conditions/resource-policies).
        class Policy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] expression_sets
        #   @return [::Array<::Google::Cloud::Compute::V1::WafExpressionSet>]
        #     List of entities that are currently supported for WAF rules.
        class PreconfiguredWafSet
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Preserved state for a given instance.
        # @!attribute [rw] disks
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::PreservedStatePreservedDisk}]
        #     Preserved disks defined for this instance. This map is keyed with the device names of the disks.
        # @!attribute [rw] external_i_ps
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::PreservedStatePreservedNetworkIp}]
        #     Preserved external IPs defined for this instance. This map is keyed with the name of the network interface.
        # @!attribute [rw] internal_i_ps
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::PreservedStatePreservedNetworkIp}]
        #     Preserved internal IPs defined for this instance. This map is keyed with the name of the network interface.
        # @!attribute [rw] metadata
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     Preserved metadata defined for this instance.
        class PreservedState
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::PreservedStatePreservedDisk]
          class DisksEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::PreservedStatePreservedNetworkIp]
          class ExternalIPsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::PreservedStatePreservedNetworkIp]
          class InternalIPsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class MetadataEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # @!attribute [rw] auto_delete
        #   @return [::String]
        #     These stateful disks will never be deleted during autohealing, update, instance recreate operations. This flag is used to configure if the disk should be deleted after it is no longer used by the group, e.g. when the given instance or the whole MIG is deleted. Note: disks attached in READ_ONLY mode cannot be auto-deleted.
        #     Check the AutoDelete enum for the list of possible values.
        # @!attribute [rw] mode
        #   @return [::String]
        #     The mode in which to attach this disk, either READ_WRITE or READ_ONLY. If not specified, the default is to attach the disk in READ_WRITE mode.
        #     Check the Mode enum for the list of possible values.
        # @!attribute [rw] source
        #   @return [::String]
        #     The URL of the disk resource that is stateful and should be attached to the VM instance.
        class PreservedStatePreservedDisk
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # These stateful disks will never be deleted during autohealing, update, instance recreate operations. This flag is used to configure if the disk should be deleted after it is no longer used by the group, e.g. when the given instance or the whole MIG is deleted. Note: disks attached in READ_ONLY mode cannot be auto-deleted.
          module AutoDelete
            # A value indicating that the enum field is not set.
            UNDEFINED_AUTO_DELETE = 0

            NEVER = 74_175_084

            ON_PERMANENT_INSTANCE_DELETION = 95_727_719
          end

          # The mode in which to attach this disk, either READ_WRITE or READ_ONLY. If not specified, the default is to attach the disk in READ_WRITE mode.
          module Mode
            # A value indicating that the enum field is not set.
            UNDEFINED_MODE = 0

            # Attaches this disk in read-only mode. Multiple VM instances can use a disk in READ_ONLY mode at a time.
            READ_ONLY = 91_950_261

            # *[Default]* Attaches this disk in READ_WRITE mode. Only one VM instance at a time can be attached to a disk in READ_WRITE mode.
            READ_WRITE = 173_607_894
          end
        end

        # @!attribute [rw] auto_delete
        #   @return [::String]
        #     These stateful IPs will never be released during autohealing, update or VM instance recreate operations. This flag is used to configure if the IP reservation should be deleted after it is no longer used by the group, e.g. when the given instance or the whole group is deleted.
        #     Check the AutoDelete enum for the list of possible values.
        # @!attribute [rw] ip_address
        #   @return [::Google::Cloud::Compute::V1::PreservedStatePreservedNetworkIpIpAddress]
        #     Ip address representation
        class PreservedStatePreservedNetworkIp
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # These stateful IPs will never be released during autohealing, update or VM instance recreate operations. This flag is used to configure if the IP reservation should be deleted after it is no longer used by the group, e.g. when the given instance or the whole group is deleted.
          module AutoDelete
            # A value indicating that the enum field is not set.
            UNDEFINED_AUTO_DELETE = 0

            NEVER = 74_175_084

            ON_PERMANENT_INSTANCE_DELETION = 95_727_719
          end
        end

        # @!attribute [rw] address
        #   @return [::String]
        #     The URL of the reservation for this IP address.
        # @!attribute [rw] literal
        #   @return [::String]
        #     An IPv4 internal network address to assign to the instance for this network interface.
        class PreservedStatePreservedNetworkIpIpAddress
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Routers.Preview. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] router
        #   @return [::String]
        #     Name of the Router resource to query.
        # @!attribute [rw] router_resource
        #   @return [::Google::Cloud::Compute::V1::Router]
        #     The body resource for this request
        class PreviewRouterRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a Project resource. A project is used to organize resources in a Google Cloud Platform environment. For more information, read about the Resource Hierarchy.
        # @!attribute [rw] common_instance_metadata
        #   @return [::Google::Cloud::Compute::V1::Metadata]
        #     Metadata key/value pairs available to all instances contained in this project. See Custom metadata for more information.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] default_network_tier
        #   @return [::String]
        #     This signifies the default network tier used for configuring resources of the project and can only take the following values: PREMIUM, STANDARD. Initially the default network tier is PREMIUM.
        #     Check the DefaultNetworkTier enum for the list of possible values.
        # @!attribute [rw] default_service_account
        #   @return [::String]
        #     [Output Only] Default service account used by VMs running in this project.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional textual description of the resource.
        # @!attribute [rw] enabled_features
        #   @return [::Array<::String>]
        #     Restricted features enabled for use on this project.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server. This is *not* the project ID, and is just a unique ID used by Compute Engine to identify resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#project for projects.
        # @!attribute [rw] name
        #   @return [::String]
        #     The project ID. For example: my-example-project. Use the project ID to make requests to Compute Engine.
        # @!attribute [rw] quotas
        #   @return [::Array<::Google::Cloud::Compute::V1::Quota>]
        #     [Output Only] Quotas assigned to this project.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] usage_export_location
        #   @return [::Google::Cloud::Compute::V1::UsageExportLocation]
        #     The naming prefix for daily usage reports and the Google Cloud Storage bucket where they are stored.
        # @!attribute [rw] vm_dns_setting
        #   @return [::String]
        #     [Output Only] Default internal DNS setting used by VMs running in this project.
        #     Check the VmDnsSetting enum for the list of possible values.
        # @!attribute [rw] xpn_project_status
        #   @return [::String]
        #     [Output Only] The role this project has in a shared VPC configuration. Currently, only projects with the host role, which is specified by the value HOST, are differentiated.
        #     Check the XpnProjectStatus enum for the list of possible values.
        class Project
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # This signifies the default network tier used for configuring resources of the project and can only take the following values: PREMIUM, STANDARD. Initially the default network tier is PREMIUM.
          module DefaultNetworkTier
            # A value indicating that the enum field is not set.
            UNDEFINED_DEFAULT_NETWORK_TIER = 0

            # Public internet quality with fixed bandwidth.
            FIXED_STANDARD = 310_464_328

            # High quality, Google-grade network tier, support for all networking products.
            PREMIUM = 399_530_551

            # Public internet quality, only limited support for other networking products.
            STANDARD = 484_642_493

            # (Output only) Temporary tier for FIXED_STANDARD when fixed standard tier is expired or not configured.
            STANDARD_OVERRIDES_FIXED_STANDARD = 465_847_234
          end

          # [Output Only] Default internal DNS setting used by VMs running in this project.
          module VmDnsSetting
            # A value indicating that the enum field is not set.
            UNDEFINED_VM_DNS_SETTING = 0

            GLOBAL_DEFAULT = 345_419_141

            UNSPECIFIED_VM_DNS_SETTING = 35_691_930

            ZONAL_DEFAULT = 368_475_782

            ZONAL_ONLY = 521_198_951
          end

          # [Output Only] The role this project has in a shared VPC configuration. Currently, only projects with the host role, which is specified by the value HOST, are differentiated.
          module XpnProjectStatus
            # A value indicating that the enum field is not set.
            UNDEFINED_XPN_PROJECT_STATUS = 0

            HOST = 2_223_528

            UNSPECIFIED_XPN_PROJECT_STATUS = 340_393_257
          end
        end

        # @!attribute [rw] xpn_resource
        #   @return [::Google::Cloud::Compute::V1::XpnResourceId]
        #     Service resource (a.k.a service project) ID.
        class ProjectsDisableXpnResourceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] xpn_resource
        #   @return [::Google::Cloud::Compute::V1::XpnResourceId]
        #     Service resource (a.k.a service project) ID.
        class ProjectsEnableXpnResourceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#projectsGetXpnResources for lists of service resources (a.k.a service projects)
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] resources
        #   @return [::Array<::Google::Cloud::Compute::V1::XpnResourceId>]
        #     Service resources (a.k.a service projects) attached to this project as their shared VPC host.
        class ProjectsGetXpnResources
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] organization
        #   @return [::String]
        #     Optional organization ID managed by Cloud Resource Manager, for which to list shared VPC host projects. If not specified, the organization will be inferred from the project.
        class ProjectsListXpnHostsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] network_tier
        #   @return [::String]
        #     Default network tier to be set.
        #     Check the NetworkTier enum for the list of possible values.
        class ProjectsSetDefaultNetworkTierRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Default network tier to be set.
          module NetworkTier
            # A value indicating that the enum field is not set.
            UNDEFINED_NETWORK_TIER = 0

            # Public internet quality with fixed bandwidth.
            FIXED_STANDARD = 310_464_328

            # High quality, Google-grade network tier, support for all networking products.
            PREMIUM = 399_530_551

            # Public internet quality, only limited support for other networking products.
            STANDARD = 484_642_493

            # (Output only) Temporary tier for FIXED_STANDARD when fixed standard tier is expired or not configured.
            STANDARD_OVERRIDES_FIXED_STANDARD = 465_847_234
          end
        end

        # A public advertised prefix represents an aggregated IP prefix or netblock which customers bring to cloud. The IP prefix is a single unit of route advertisement and is announced globally to the internet.
        # @!attribute [rw] byoip_api_version
        #   @return [::String]
        #     [Output Only] The version of BYOIP API.
        #     Check the ByoipApiVersion enum for the list of possible values.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] dns_verification_ip
        #   @return [::String]
        #     The address to be used for reverse DNS verification.
        # @!attribute [rw] fingerprint
        #   @return [::String]
        #     Fingerprint of this resource. A hash of the contents stored in this object. This field is used in optimistic locking. This field will be ignored when inserting a new PublicAdvertisedPrefix. An up-to-date fingerprint must be provided in order to update the PublicAdvertisedPrefix, otherwise the request will fail with error 412 conditionNotMet. To see the latest fingerprint, make a get() request to retrieve a PublicAdvertisedPrefix.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource type. The server generates this identifier.
        # @!attribute [rw] ip_cidr_range
        #   @return [::String]
        #     The address range, in CIDR format, represented by this public advertised prefix.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#publicAdvertisedPrefix for public advertised prefixes.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] pdp_scope
        #   @return [::String]
        #     Specifies how child public delegated prefix will be scoped. It could be one of following values: - `REGIONAL`: The public delegated prefix is regional only. The provisioning will take a few minutes. - `GLOBAL`: The public delegated prefix is global only. The provisioning will take ~4 weeks. - `GLOBAL_AND_REGIONAL` [output only]: The public delegated prefixes is BYOIP V1 legacy prefix. This is output only value and no longer supported in BYOIP V2.
        #     Check the PdpScope enum for the list of possible values.
        # @!attribute [rw] public_delegated_prefixs
        #   @return [::Array<::Google::Cloud::Compute::V1::PublicAdvertisedPrefixPublicDelegatedPrefix>]
        #     [Output Only] The list of public delegated prefixes that exist for this public advertised prefix.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] shared_secret
        #   @return [::String]
        #     [Output Only] The shared secret to be used for reverse DNS verification.
        # @!attribute [rw] status
        #   @return [::String]
        #     The status of the public advertised prefix. Possible values include: - `INITIAL`: RPKI validation is complete. - `PTR_CONFIGURED`: User has configured the PTR. - `VALIDATED`: Reverse DNS lookup is successful. - `REVERSE_DNS_LOOKUP_FAILED`: Reverse DNS lookup failed. - `PREFIX_CONFIGURATION_IN_PROGRESS`: The prefix is being configured. - `PREFIX_CONFIGURATION_COMPLETE`: The prefix is fully configured. - `PREFIX_REMOVAL_IN_PROGRESS`: The prefix is being removed.
        #     Check the Status enum for the list of possible values.
        class PublicAdvertisedPrefix
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # [Output Only] The version of BYOIP API.
          module ByoipApiVersion
            # A value indicating that the enum field is not set.
            UNDEFINED_BYOIP_API_VERSION = 0

            # This public advertised prefix can be used to create both regional and global public delegated prefixes. It usually takes 4 weeks to create or delete a public delegated prefix. The BGP status cannot be changed.
            V1 = 2715

            # This public advertised prefix can only be used to create regional public delegated prefixes. Public delegated prefix creation and deletion takes minutes and the BGP status can be modified.
            V2 = 2716
          end

          # Specifies how child public delegated prefix will be scoped. It could be one of following values: - `REGIONAL`: The public delegated prefix is regional only. The provisioning will take a few minutes. - `GLOBAL`: The public delegated prefix is global only. The provisioning will take ~4 weeks. - `GLOBAL_AND_REGIONAL` [output only]: The public delegated prefixes is BYOIP V1 legacy prefix. This is output only value and no longer supported in BYOIP V2.
          module PdpScope
            # A value indicating that the enum field is not set.
            UNDEFINED_PDP_SCOPE = 0

            # The public delegated prefix is global only. The provisioning will take ~4 weeks.
            GLOBAL = 494_663_587

            # The public delegated prefixes is BYOIP V1 legacy prefix. This is output only value and no longer supported in BYOIP V2.
            GLOBAL_AND_REGIONAL = 318_053_059

            # The public delegated prefix is regional only. The provisioning will take a few minutes.
            REGIONAL = 92_288_543
          end

          # The status of the public advertised prefix. Possible values include: - `INITIAL`: RPKI validation is complete. - `PTR_CONFIGURED`: User has configured the PTR. - `VALIDATED`: Reverse DNS lookup is successful. - `REVERSE_DNS_LOOKUP_FAILED`: Reverse DNS lookup failed. - `PREFIX_CONFIGURATION_IN_PROGRESS`: The prefix is being configured. - `PREFIX_CONFIGURATION_COMPLETE`: The prefix is fully configured. - `PREFIX_REMOVAL_IN_PROGRESS`: The prefix is being removed.
          module Status
            # A value indicating that the enum field is not set.
            UNDEFINED_STATUS = 0

            # The prefix is announced to Internet.
            ANNOUNCED_TO_INTERNET = 177_880_897

            # RPKI validation is complete.
            INITIAL = 518_841_124

            # The prefix is fully configured.
            PREFIX_CONFIGURATION_COMPLETE = 480_889_551

            # The prefix is being configured.
            PREFIX_CONFIGURATION_IN_PROGRESS = 378_550_961

            # The prefix is being removed.
            PREFIX_REMOVAL_IN_PROGRESS = 284_375_783

            # User has configured the PTR.
            PTR_CONFIGURED = 513_497_167

            # The prefix is currently withdrawn but ready to be announced.
            READY_TO_ANNOUNCE = 64_641_265

            # Reverse DNS lookup failed.
            REVERSE_DNS_LOOKUP_FAILED = 295_755_183

            # Reverse DNS lookup is successful.
            VALIDATED = 66_197_998
          end
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::PublicAdvertisedPrefix>]
        #     A list of PublicAdvertisedPrefix resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#publicAdvertisedPrefix for public advertised prefixes.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class PublicAdvertisedPrefixList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a CIDR range which can be used to assign addresses.
        # @!attribute [rw] ip_range
        #   @return [::String]
        #     The IP address range of the public delegated prefix
        # @!attribute [rw] name
        #   @return [::String]
        #     The name of the public delegated prefix
        # @!attribute [rw] project
        #   @return [::String]
        #     The project number of the public delegated prefix
        # @!attribute [rw] region
        #   @return [::String]
        #     The region of the public delegated prefix if it is regional. If absent, the prefix is global.
        # @!attribute [rw] status
        #   @return [::String]
        #     The status of the public delegated prefix. Possible values are: INITIALIZING: The public delegated prefix is being initialized and addresses cannot be created yet. ANNOUNCED: The public delegated prefix is active.
        class PublicAdvertisedPrefixPublicDelegatedPrefix
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A PublicDelegatedPrefix resource represents an IP block within a PublicAdvertisedPrefix that is configured within a single cloud scope (global or region). IPs in the block can be allocated to resources within that scope. Public delegated prefixes may be further broken up into smaller IP blocks in the same scope as the parent block.
        # @!attribute [rw] byoip_api_version
        #   @return [::String]
        #     [Output Only] The version of BYOIP API.
        #     Check the ByoipApiVersion enum for the list of possible values.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] fingerprint
        #   @return [::String]
        #     Fingerprint of this resource. A hash of the contents stored in this object. This field is used in optimistic locking. This field will be ignored when inserting a new PublicDelegatedPrefix. An up-to-date fingerprint must be provided in order to update the PublicDelegatedPrefix, otherwise the request will fail with error 412 conditionNotMet. To see the latest fingerprint, make a get() request to retrieve a PublicDelegatedPrefix.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource type. The server generates this identifier.
        # @!attribute [rw] ip_cidr_range
        #   @return [::String]
        #     The IP address range, in CIDR format, represented by this public delegated prefix.
        # @!attribute [rw] is_live_migration
        #   @return [::Boolean]
        #     If true, the prefix will be live migrated.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#publicDelegatedPrefix for public delegated prefixes.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] parent_prefix
        #   @return [::String]
        #     The URL of parent prefix. Either PublicAdvertisedPrefix or PublicDelegatedPrefix.
        # @!attribute [rw] public_delegated_sub_prefixs
        #   @return [::Array<::Google::Cloud::Compute::V1::PublicDelegatedPrefixPublicDelegatedSubPrefix>]
        #     The list of sub public delegated prefixes that exist for this public delegated prefix.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] URL of the region where the public delegated prefix resides. This field applies only to the region resource. You must specify this field as part of the HTTP request URL. It is not settable as a field in the request body.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] status
        #   @return [::String]
        #     [Output Only] The status of the public delegated prefix, which can be one of following values: - `INITIALIZING` The public delegated prefix is being initialized and addresses cannot be created yet. - `READY_TO_ANNOUNCE` The public delegated prefix is a live migration prefix and is active. - `ANNOUNCED` The public delegated prefix is active. - `DELETING` The public delegated prefix is being deprovsioned.
        #     Check the Status enum for the list of possible values.
        class PublicDelegatedPrefix
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # [Output Only] The version of BYOIP API.
          module ByoipApiVersion
            # A value indicating that the enum field is not set.
            UNDEFINED_BYOIP_API_VERSION = 0

            # This public delegated prefix usually takes 4 weeks to delete, and the BGP status cannot be changed. Announce and Withdraw APIs can not be used on this prefix.
            V1 = 2715

            # This public delegated prefix takes minutes to delete. Announce and Withdraw APIs can be used on this prefix to change the BGP status.
            V2 = 2716
          end

          # [Output Only] The status of the public delegated prefix, which can be one of following values: - `INITIALIZING` The public delegated prefix is being initialized and addresses cannot be created yet. - `READY_TO_ANNOUNCE` The public delegated prefix is a live migration prefix and is active. - `ANNOUNCED` The public delegated prefix is active. - `DELETING` The public delegated prefix is being deprovsioned.
          module Status
            # A value indicating that the enum field is not set.
            UNDEFINED_STATUS = 0

            # The public delegated prefix is active.
            ANNOUNCED = 365_103_355

            # The prefix is announced within Google network.
            ANNOUNCED_TO_GOOGLE = 454_875_705

            # The prefix is announced to Internet and within Google.
            ANNOUNCED_TO_INTERNET = 177_880_897

            # The public delegated prefix is being deprovsioned.
            DELETING = 528_602_024

            # The public delegated prefix is being initialized and addresses cannot be created yet.
            INITIALIZING = 306_588_749

            # The public delegated prefix is currently withdrawn but ready to be announced.
            READY_TO_ANNOUNCE = 64_641_265
          end
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::PublicDelegatedPrefixesScopedList}]
        #     A list of PublicDelegatedPrefixesScopedList resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#publicDelegatedPrefixAggregatedList for aggregated lists of public delegated prefixes.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class PublicDelegatedPrefixAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::PublicDelegatedPrefixesScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::PublicDelegatedPrefix>]
        #     A list of PublicDelegatedPrefix resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#publicDelegatedPrefixList for public delegated prefixes.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class PublicDelegatedPrefixList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a sub PublicDelegatedPrefix.
        # @!attribute [rw] delegatee_project
        #   @return [::String]
        #     Name of the project scoping this PublicDelegatedSubPrefix.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] ip_cidr_range
        #   @return [::String]
        #     The IP address range, in CIDR format, represented by this sub public delegated prefix.
        # @!attribute [rw] is_address
        #   @return [::Boolean]
        #     Whether the sub prefix is delegated to create Address resources in the delegatee project.
        # @!attribute [rw] name
        #   @return [::String]
        #     The name of the sub public delegated prefix.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] The region of the sub public delegated prefix if it is regional. If absent, the sub prefix is global.
        # @!attribute [rw] status
        #   @return [::String]
        #     [Output Only] The status of the sub public delegated prefix.
        #     Check the Status enum for the list of possible values.
        class PublicDelegatedPrefixPublicDelegatedSubPrefix
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # [Output Only] The status of the sub public delegated prefix.
          module Status
            # A value indicating that the enum field is not set.
            UNDEFINED_STATUS = 0

            ACTIVE = 314_733_318

            INACTIVE = 270_421_099
          end
        end

        # @!attribute [rw] public_delegated_prefixes
        #   @return [::Array<::Google::Cloud::Compute::V1::PublicDelegatedPrefix>]
        #     [Output Only] A list of PublicDelegatedPrefixes contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning which replaces the list of public delegated prefixes when the list is empty.
        class PublicDelegatedPrefixesScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A quotas entry.
        # @!attribute [rw] limit
        #   @return [::Float]
        #     [Output Only] Quota limit for this metric.
        # @!attribute [rw] metric
        #   @return [::String]
        #     [Output Only] Name of the quota metric.
        #     Check the Metric enum for the list of possible values.
        # @!attribute [rw] owner
        #   @return [::String]
        #     [Output Only] Owning resource. This is the resource on which this quota is applied.
        # @!attribute [rw] usage
        #   @return [::Float]
        #     [Output Only] Current usage of this metric.
        class Quota
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # [Output Only] Name of the quota metric.
          module Metric
            # A value indicating that the enum field is not set.
            UNDEFINED_METRIC = 0

            A2_CPUS = 153_206_585

            AFFINITY_GROUPS = 108_303_563

            AUTOSCALERS = 471_248_988

            BACKEND_BUCKETS = 137_626_846

            BACKEND_SERVICES = 269_623_753

            C2D_CPUS = 508_182_517

            C2_CPUS = 317_601_211

            C3_CPUS = 346_230_362

            COMMITMENTS = 456_141_790

            COMMITTED_A2_CPUS = 59_330_902

            COMMITTED_C2D_CPUS = 282_390_904

            COMMITTED_C2_CPUS = 223_725_528

            COMMITTED_C3_CPUS = 252_354_679

            COMMITTED_CPUS = 292_394_702

            COMMITTED_E2_CPUS = 388_120_154

            COMMITTED_LICENSES = 357_606_869

            COMMITTED_LOCAL_SSD_TOTAL_GB = 308_393_480

            COMMITTED_M3_CPUS = 585_985

            COMMITTED_MEMORY_OPTIMIZED_CPUS = 489_057_886

            COMMITTED_N2A_CPUS = 40_064_304

            COMMITTED_N2D_CPUS = 125_951_757

            COMMITTED_N2_CPUS = 322_589_603

            COMMITTED_NVIDIA_A100_80GB_GPUS = 464_326_565

            COMMITTED_NVIDIA_A100_GPUS = 375_799_445

            COMMITTED_NVIDIA_K80_GPUS = 3_857_188

            COMMITTED_NVIDIA_L4_GPUS = 19_163_645

            COMMITTED_NVIDIA_P100_GPUS = 107_528_100

            COMMITTED_NVIDIA_P4_GPUS = 347_952_897

            COMMITTED_NVIDIA_T4_GPUS = 139_871_237

            COMMITTED_NVIDIA_V100_GPUS = 219_562

            COMMITTED_T2A_CPUS = 296_378_986

            COMMITTED_T2D_CPUS = 382_266_439

            # Guest CPUs
            CPUS = 2_075_595

            CPUS_ALL_REGIONS = 470_911_149

            DISKS_TOTAL_GB = 353_520_543

            E2_CPUS = 481_995_837

            EXTERNAL_MANAGED_FORWARDING_RULES = 150_790_089

            EXTERNAL_NETWORK_LB_FORWARDING_RULES = 374_298_265

            EXTERNAL_PROTOCOL_FORWARDING_RULES = 63_478_888

            EXTERNAL_VPN_GATEWAYS = 272_457_134

            FIREWALLS = 374_485_843

            FORWARDING_RULES = 432_668_949

            GLOBAL_EXTERNAL_MANAGED_BACKEND_SERVICES = 164_566_753

            GLOBAL_EXTERNAL_MANAGED_FORWARDING_RULES = 327_611_949

            GLOBAL_EXTERNAL_PROXY_LB_BACKEND_SERVICES = 400_256_169

            GLOBAL_INTERNAL_ADDRESSES = 42_738_332

            GLOBAL_INTERNAL_MANAGED_BACKEND_SERVICES = 256_608_303

            GLOBAL_INTERNAL_TRAFFIC_DIRECTOR_BACKEND_SERVICES = 323_514_196

            GPUS_ALL_REGIONS = 39_387_177

            HEALTH_CHECKS = 289_347_502

            IMAGES = 15_562_360

            INSTANCES = 131_337_822

            INSTANCE_GROUPS = 355_919_038

            INSTANCE_GROUP_MANAGERS = 101_798_192

            INSTANCE_TEMPLATES = 226_188_271

            INTERCONNECTS = 415_204_741

            INTERCONNECT_ATTACHMENTS_PER_REGION = 159_968_086

            INTERCONNECT_ATTACHMENTS_TOTAL_MBPS = 425_090_419

            INTERCONNECT_TOTAL_GBPS = 285_341_866

            INTERNAL_ADDRESSES = 197_899_392

            INTERNAL_TRAFFIC_DIRECTOR_FORWARDING_RULES = 266_433_668

            IN_PLACE_SNAPSHOTS = 151_359_133

            IN_USE_ADDRESSES = 402_125_072

            IN_USE_BACKUP_SCHEDULES = 32_786_705

            IN_USE_SNAPSHOT_SCHEDULES = 462_104_083

            LOCAL_SSD_TOTAL_GB = 330_878_021

            M1_CPUS = 37_203_366

            M2_CPUS = 65_832_517

            M3_CPUS = 94_461_668

            MACHINE_IMAGES = 446_986_640

            N2A_CPUS = 265_855_917

            N2D_CPUS = 351_743_370

            N2_CPUS = 416_465_286

            NETWORKS = 485_481_477

            NETWORK_ATTACHMENTS = 149_028_575

            NETWORK_ENDPOINT_GROUPS = 102_144_909

            NETWORK_FIREWALL_POLICIES = 101_117_374

            NET_LB_SECURITY_POLICIES_PER_REGION = 157_892_269

            NET_LB_SECURITY_POLICY_RULES_PER_REGION = 356_090_931

            NET_LB_SECURITY_POLICY_RULE_ATTRIBUTES_PER_REGION = 311_243_888

            NODE_GROUPS = 24_624_817

            NODE_TEMPLATES = 474_896_668

            NVIDIA_A100_80GB_GPUS = 286_389_320

            NVIDIA_A100_GPUS = 504_872_978

            NVIDIA_K80_GPUS = 163_886_599

            NVIDIA_L4_GPUS = 491_923_130

            NVIDIA_P100_GPUS = 236_601_633

            NVIDIA_P100_VWS_GPUS = 213_970_574

            NVIDIA_P4_GPUS = 283_841_470

            NVIDIA_P4_VWS_GPUS = 528_296_619

            NVIDIA_T4_GPUS = 75_759_810

            NVIDIA_T4_VWS_GPUS = 319_813_039

            NVIDIA_V100_GPUS = 129_293_095

            PACKET_MIRRORINGS = 15_578_407

            PD_EXTREME_TOTAL_PROVISIONED_IOPS = 69_593_965

            PREEMPTIBLE_CPUS = 251_184_841

            PREEMPTIBLE_LOCAL_SSD_GB = 260_819_336

            PREEMPTIBLE_NVIDIA_A100_80GB_GPUS = 151_942_410

            PREEMPTIBLE_NVIDIA_A100_GPUS = 68_832_784

            PREEMPTIBLE_NVIDIA_K80_GPUS = 374_960_201

            PREEMPTIBLE_NVIDIA_L4_GPUS = 100_408_376

            PREEMPTIBLE_NVIDIA_P100_GPUS = 337_432_351

            PREEMPTIBLE_NVIDIA_P100_VWS_GPUS = 313_544_076

            PREEMPTIBLE_NVIDIA_P4_GPUS = 429_197_628

            PREEMPTIBLE_NVIDIA_P4_VWS_GPUS = 252_981_545

            PREEMPTIBLE_NVIDIA_T4_GPUS = 221_115_968

            PREEMPTIBLE_NVIDIA_T4_VWS_GPUS = 44_497_965

            PREEMPTIBLE_NVIDIA_V100_GPUS = 230_123_813

            PSC_ILB_CONSUMER_FORWARDING_RULES_PER_PRODUCER_NETWORK = 231_164_291

            PSC_INTERNAL_LB_FORWARDING_RULES = 169_005_435

            PUBLIC_ADVERTISED_PREFIXES = 471_371_980

            PUBLIC_DELEGATED_PREFIXES = 532_465_974

            REGIONAL_AUTOSCALERS = 29_363_772

            REGIONAL_EXTERNAL_MANAGED_BACKEND_SERVICES = 4_240_989

            REGIONAL_EXTERNAL_NETWORK_LB_BACKEND_SERVICES = 409_564_525

            REGIONAL_INSTANCE_GROUP_MANAGERS = 37_543_696

            REGIONAL_INTERNAL_LB_BACKEND_SERVICES = 137_983_760

            REGIONAL_INTERNAL_MANAGED_BACKEND_SERVICES = 96_282_539

            RESERVATIONS = 32_644_647

            RESOURCE_POLICIES = 83_955_297

            ROUTERS = 493_018_666

            ROUTES = 275_680_074

            SECURITY_POLICIES = 189_518_703

            SECURITY_POLICIES_PER_REGION = 249_041_734

            SECURITY_POLICY_ADVANCED_RULES_PER_REGION = 371_815_341

            SECURITY_POLICY_CEVAL_RULES = 470_815_689

            SECURITY_POLICY_RULES = 203_549_225

            SECURITY_POLICY_RULES_PER_REGION = 126_510_156

            SERVICE_ATTACHMENTS = 471_521_510

            # The total number of snapshots allowed for a single project.
            SNAPSHOTS = 343_405_327

            SSD_TOTAL_GB = 161_732_561

            SSL_CERTIFICATES = 378_372_399

            STATIC_ADDRESSES = 93_624_049

            STATIC_BYOIP_ADDRESSES = 275_809_649

            STATIC_EXTERNAL_IPV6_ADDRESS_RANGES = 472_346_774

            SUBNETWORKS = 421_330_469

            T2A_CPUS = 522_170_599

            T2D_CPUS = 71_187_140

            TARGET_HTTPS_PROXIES = 219_522_506

            TARGET_HTTP_PROXIES = 164_117_155

            TARGET_INSTANCES = 284_519_728

            TARGET_POOLS = 348_261_257

            TARGET_SSL_PROXIES = 159_216_235

            TARGET_TCP_PROXIES = 182_243_136

            TARGET_VPN_GATEWAYS = 75_029_928

            URL_MAPS = 378_660_743

            VPN_GATEWAYS = 35_620_282

            VPN_TUNNELS = 104_327_296

            XPN_SERVICE_PROJECTS = 95_191_981
          end
        end

        # Additional details for quota exceeded error for resource quota.
        # @!attribute [rw] dimensions
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     The map holding related quota dimensions.
        # @!attribute [rw] future_limit
        #   @return [::Float]
        #     Future quota limit being rolled out. The limit's unit depends on the quota type or metric.
        # @!attribute [rw] limit
        #   @return [::Float]
        #     Current effective quota limit. The limit's unit depends on the quota type or metric.
        # @!attribute [rw] limit_name
        #   @return [::String]
        #     The name of the quota limit.
        # @!attribute [rw] metric_name
        #   @return [::String]
        #     The Compute Engine quota metric name.
        # @!attribute [rw] rollout_status
        #   @return [::String]
        #     Rollout status of the future quota limit.
        #     Check the RolloutStatus enum for the list of possible values.
        class QuotaExceededInfo
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class DimensionsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # Rollout status of the future quota limit.
          module RolloutStatus
            # A value indicating that the enum field is not set.
            UNDEFINED_ROLLOUT_STATUS = 0

            # IN_PROGRESS - A rollout is in process which will change the limit value to future limit.
            IN_PROGRESS = 469_193_735

            # ROLLOUT_STATUS_UNSPECIFIED - Rollout status is not specified. The default value.
            ROLLOUT_STATUS_UNSPECIFIED = 26_864_568
          end
        end

        # The parameters of the raw disk image.
        # @!attribute [rw] container_type
        #   @return [::String]
        #     The format used to encode and transmit the block device, which should be TAR. This is just a container and transmission format and not a runtime format. Provided by the client when the disk image is created.
        #     Check the ContainerType enum for the list of possible values.
        # @!attribute [rw] sha1_checksum
        #   @return [::String]
        #     [Deprecated] This field is deprecated. An optional SHA1 checksum of the disk image before unpackaging provided by the client when the disk image is created.
        # @!attribute [rw] source
        #   @return [::String]
        #     The full Google Cloud Storage URL where the raw disk image archive is stored. The following are valid formats for the URL: - https://storage.googleapis.com/bucket_name/image_archive_name - https://storage.googleapis.com/bucket_name/folder_name/ image_archive_name In order to create an image, you must provide the full or partial URL of one of the following: - The rawDisk.source URL - The sourceDisk URL - The sourceImage URL - The sourceSnapshot URL
        class RawDisk
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The format used to encode and transmit the block device, which should be TAR. This is just a container and transmission format and not a runtime format. Provided by the client when the disk image is created.
          module ContainerType
            # A value indicating that the enum field is not set.
            UNDEFINED_CONTAINER_TYPE = 0

            TAR = 82_821
          end
        end

        # A request message for InstanceGroupManagers.RecreateInstances. See the method description for details.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     The name of the managed instance group.
        # @!attribute [rw] instance_group_managers_recreate_instances_request_resource
        #   @return [::Google::Cloud::Compute::V1::InstanceGroupManagersRecreateInstancesRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where the managed instance group is located.
        class RecreateInstancesInstanceGroupManagerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionInstanceGroupManagers.RecreateInstances. See the method description for details.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     Name of the managed instance group.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] region_instance_group_managers_recreate_request_resource
        #   @return [::Google::Cloud::Compute::V1::RegionInstanceGroupManagersRecreateRequest]
        #     The body resource for this request
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class RecreateInstancesRegionInstanceGroupManagerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a reference to a resource.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#reference for references.
        # @!attribute [rw] reference_type
        #   @return [::String]
        #     A description of the reference type with no implied semantics. Possible values include: 1. MEMBER_OF
        # @!attribute [rw] referrer
        #   @return [::String]
        #     URL of the resource which refers to the target.
        # @!attribute [rw] target
        #   @return [::String]
        #     URL of the resource to which this reference points.
        class Reference
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a Region resource. A region is a geographical area where a resource is located. For more information, read Regions and Zones.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] deprecated
        #   @return [::Google::Cloud::Compute::V1::DeprecationStatus]
        #     [Output Only] The deprecation status associated with this region.
        # @!attribute [rw] description
        #   @return [::String]
        #     [Output Only] Textual description of the resource.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#region for regions.
        # @!attribute [rw] name
        #   @return [::String]
        #     [Output Only] Name of the resource.
        # @!attribute [rw] quotas
        #   @return [::Array<::Google::Cloud::Compute::V1::Quota>]
        #     [Output Only] Quotas assigned to this region.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] status
        #   @return [::String]
        #     [Output Only] Status of the region, either UP or DOWN.
        #     Check the Status enum for the list of possible values.
        # @!attribute [rw] supports_pzs
        #   @return [::Boolean]
        #     [Output Only] Reserved for future use.
        # @!attribute [rw] zones
        #   @return [::Array<::String>]
        #     [Output Only] A list of zones available in this region, in the form of resource URLs.
        class Region
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # [Output Only] Status of the region, either UP or DOWN.
          module Status
            # A value indicating that the enum field is not set.
            UNDEFINED_STATUS = 0

            DOWN = 2_104_482

            UP = 2715
          end
        end

        # @!attribute [rw] description
        #   @return [::String]
        #     An optional destination address description if intended to be different from the source.
        # @!attribute [rw] destination_address
        #   @return [::String]
        #     The URL of the destination address to move to. This can be a full or partial URL. For example, the following are all valid URLs to a address: - https://www.googleapis.com/compute/v1/projects/project/regions/region /addresses/address - projects/project/regions/region/addresses/address Note that destination project must be different from the source project. So /regions/region/addresses/address is not valid partial url.
        class RegionAddressesMoveRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Contains a list of autoscalers.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::Autoscaler>]
        #     A list of Autoscaler resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     Type of resource.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class RegionAutoscalerList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::DiskType>]
        #     A list of DiskType resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#regionDiskTypeList for region disk types.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class RegionDiskTypeList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] resource_policies
        #   @return [::Array<::String>]
        #     Resource policies to be added to this disk.
        class RegionDisksAddResourcePoliciesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] resource_policies
        #   @return [::Array<::String>]
        #     Resource policies to be removed from this disk.
        class RegionDisksRemoveResourcePoliciesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] size_gb
        #   @return [::Integer]
        #     The new size of the regional persistent disk, which is specified in GB.
        class RegionDisksResizeRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] async_secondary_disk
        #   @return [::String]
        #     The secondary disk to start asynchronous replication to. You can provide this as a partial or full URL to the resource. For example, the following are valid values: - https://www.googleapis.com/compute/v1/projects/project/zones/zone /disks/disk - https://www.googleapis.com/compute/v1/projects/project/regions/region /disks/disk - projects/project/zones/zone/disks/disk - projects/project/regions/region/disks/disk - zones/zone/disks/disk - regions/region/disks/disk
        class RegionDisksStartAsyncReplicationRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Contains a list of InstanceGroup resources.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::InstanceGroup>]
        #     A list of InstanceGroup resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     The resource type.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class RegionInstanceGroupList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # RegionInstanceGroupManagers.deletePerInstanceConfigs
        # @!attribute [rw] names
        #   @return [::Array<::String>]
        #     The list of instance names for which we want to delete per-instance configs on this managed instance group.
        class RegionInstanceGroupManagerDeleteInstanceConfigReq
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Contains a list of managed instance groups.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::InstanceGroupManager>]
        #     A list of InstanceGroupManager resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] The resource type, which is always compute#instanceGroupManagerList for a list of managed instance groups that exist in th regional scope.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class RegionInstanceGroupManagerList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # RegionInstanceGroupManagers.patchPerInstanceConfigs
        # @!attribute [rw] per_instance_configs
        #   @return [::Array<::Google::Cloud::Compute::V1::PerInstanceConfig>]
        #     The list of per-instance configurations to insert or patch on this managed instance group.
        class RegionInstanceGroupManagerPatchInstanceConfigReq
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # RegionInstanceGroupManagers.updatePerInstanceConfigs
        # @!attribute [rw] per_instance_configs
        #   @return [::Array<::Google::Cloud::Compute::V1::PerInstanceConfig>]
        #     The list of per-instance configurations to insert or patch on this managed instance group.
        class RegionInstanceGroupManagerUpdateInstanceConfigReq
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] instances
        #   @return [::Array<::String>]
        #     The URLs of one or more instances to abandon. This can be a full URL or a partial URL, such as zones/[ZONE]/instances/[INSTANCE_NAME].
        class RegionInstanceGroupManagersAbandonInstancesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # RegionInstanceGroupManagers.applyUpdatesToInstances
        # @!attribute [rw] all_instances
        #   @return [::Boolean]
        #     Flag to update all instances instead of specified list of “instances”. If the flag is set to true then the instances may not be specified in the request.
        # @!attribute [rw] instances
        #   @return [::Array<::String>]
        #     The list of URLs of one or more instances for which you want to apply updates. Each URL can be a full URL or a partial URL, such as zones/[ZONE]/instances/[INSTANCE_NAME].
        # @!attribute [rw] minimal_action
        #   @return [::String]
        #     The minimal action that you want to perform on each instance during the update: - REPLACE: At minimum, delete the instance and create it again. - RESTART: Stop the instance and start it again. - REFRESH: Do not stop the instance and limit disruption as much as possible. - NONE: Do not disrupt the instance at all. By default, the minimum action is NONE. If your update requires a more disruptive action than you set with this flag, the necessary action is performed to execute the update.
        #     Check the MinimalAction enum for the list of possible values.
        # @!attribute [rw] most_disruptive_allowed_action
        #   @return [::String]
        #     The most disruptive action that you want to perform on each instance during the update: - REPLACE: Delete the instance and create it again. - RESTART: Stop the instance and start it again. - REFRESH: Do not stop the instance and limit disruption as much as possible. - NONE: Do not disrupt the instance at all. By default, the most disruptive allowed action is REPLACE. If your update requires a more disruptive action than you set with this flag, the update request will fail.
        #     Check the MostDisruptiveAllowedAction enum for the list of possible values.
        class RegionInstanceGroupManagersApplyUpdatesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The minimal action that you want to perform on each instance during the update: - REPLACE: At minimum, delete the instance and create it again. - RESTART: Stop the instance and start it again. - REFRESH: Do not stop the instance and limit disruption as much as possible. - NONE: Do not disrupt the instance at all. By default, the minimum action is NONE. If your update requires a more disruptive action than you set with this flag, the necessary action is performed to execute the update.
          # Additional supported values which may be not listed in the enum directly due to technical reasons:
          # NONE
          # REFRESH
          # REPLACE
          # RESTART
          module MinimalAction
            # A value indicating that the enum field is not set.
            UNDEFINED_MINIMAL_ACTION = 0
          end

          # The most disruptive action that you want to perform on each instance during the update: - REPLACE: Delete the instance and create it again. - RESTART: Stop the instance and start it again. - REFRESH: Do not stop the instance and limit disruption as much as possible. - NONE: Do not disrupt the instance at all. By default, the most disruptive allowed action is REPLACE. If your update requires a more disruptive action than you set with this flag, the update request will fail.
          # Additional supported values which may be not listed in the enum directly due to technical reasons:
          # NONE
          # REFRESH
          # REPLACE
          # RESTART
          module MostDisruptiveAllowedAction
            # A value indicating that the enum field is not set.
            UNDEFINED_MOST_DISRUPTIVE_ALLOWED_ACTION = 0
          end
        end

        # RegionInstanceGroupManagers.createInstances
        # @!attribute [rw] instances
        #   @return [::Array<::Google::Cloud::Compute::V1::PerInstanceConfig>]
        #     [Required] List of specifications of per-instance configs.
        class RegionInstanceGroupManagersCreateInstancesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] instances
        #   @return [::Array<::String>]
        #     The URLs of one or more instances to delete. This can be a full URL or a partial URL, such as zones/[ZONE]/instances/[INSTANCE_NAME].
        # @!attribute [rw] skip_instances_on_validation_error
        #   @return [::Boolean]
        #     Specifies whether the request should proceed despite the inclusion of instances that are not members of the group or that are already in the process of being deleted or abandoned. If this field is set to `false` and such an instance is specified in the request, the operation fails. The operation always fails if the request contains a malformed instance URL or a reference to an instance that exists in a zone or region other than the group's zone or region.
        class RegionInstanceGroupManagersDeleteInstancesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::InstanceManagedByIgmError>]
        #     [Output Only] The list of errors of the managed instance group.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        class RegionInstanceGroupManagersListErrorsResponse
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::PerInstanceConfig>]
        #     [Output Only] The list of PerInstanceConfig.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class RegionInstanceGroupManagersListInstanceConfigsResp
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] managed_instances
        #   @return [::Array<::Google::Cloud::Compute::V1::ManagedInstance>]
        #     A list of managed instances.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        class RegionInstanceGroupManagersListInstancesResponse
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] instances
        #   @return [::Array<::String>]
        #     The URLs of one or more instances to recreate. This can be a full URL or a partial URL, such as zones/[ZONE]/instances/[INSTANCE_NAME].
        class RegionInstanceGroupManagersRecreateRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] fingerprint
        #   @return [::String]
        #     Fingerprint of the target pools information, which is a hash of the contents. This field is used for optimistic locking when you update the target pool entries. This field is optional.
        # @!attribute [rw] target_pools
        #   @return [::Array<::String>]
        #     The URL of all TargetPool resources to which instances in the instanceGroup field are added. The target pools automatically apply to all of the instances in the managed instance group.
        class RegionInstanceGroupManagersSetTargetPoolsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] instance_template
        #   @return [::String]
        #     URL of the InstanceTemplate resource from which all new instances will be created.
        class RegionInstanceGroupManagersSetTemplateRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::InstanceWithNamedPorts>]
        #     A list of InstanceWithNamedPorts resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     The resource type.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class RegionInstanceGroupsListInstances
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] instance_state
        #   @return [::String]
        #     Instances in which state should be returned. Valid options are: 'ALL', 'RUNNING'. By default, it lists all instances.
        #     Check the InstanceState enum for the list of possible values.
        # @!attribute [rw] port_name
        #   @return [::String]
        #     Name of port user is interested in. It is optional. If it is set, only information about this ports will be returned. If it is not set, all the named ports will be returned. Always lists all instances.
        class RegionInstanceGroupsListInstancesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Instances in which state should be returned. Valid options are: 'ALL', 'RUNNING'. By default, it lists all instances.
          module InstanceState
            # A value indicating that the enum field is not set.
            UNDEFINED_INSTANCE_STATE = 0

            # Matches any status of the instances, running, non-running and others.
            ALL = 64_897

            # Instance is in RUNNING state if it is running.
            RUNNING = 121_282_975
          end
        end

        # @!attribute [rw] fingerprint
        #   @return [::String]
        #     The fingerprint of the named ports information for this instance group. Use this optional property to prevent conflicts when multiple users change the named ports settings concurrently. Obtain the fingerprint with the instanceGroups.get method. Then, include the fingerprint in your request to ensure that you do not overwrite changes that were applied from another concurrent request.
        # @!attribute [rw] named_ports
        #   @return [::Array<::Google::Cloud::Compute::V1::NamedPort>]
        #     The list of named ports to set for this instance group.
        class RegionInstanceGroupsSetNamedPortsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Contains a list of region resources.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::Region>]
        #     A list of Region resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#regionList for lists of regions.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class RegionList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] network_endpoints
        #   @return [::Array<::Google::Cloud::Compute::V1::NetworkEndpoint>]
        #     The list of network endpoints to be attached.
        class RegionNetworkEndpointGroupsAttachEndpointsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] network_endpoints
        #   @return [::Array<::Google::Cloud::Compute::V1::NetworkEndpoint>]
        #     The list of network endpoints to be detached.
        class RegionNetworkEndpointGroupsDetachEndpointsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] firewall_policys
        #   @return [::Array<::Google::Cloud::Compute::V1::RegionNetworkFirewallPoliciesGetEffectiveFirewallsResponseEffectiveFirewallPolicy>]
        #     Effective firewalls from firewall policy.
        # @!attribute [rw] firewalls
        #   @return [::Array<::Google::Cloud::Compute::V1::Firewall>]
        #     Effective firewalls on the network.
        class RegionNetworkFirewallPoliciesGetEffectiveFirewallsResponse
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] display_name
        #   @return [::String]
        #     [Output Only] The display name of the firewall policy.
        # @!attribute [rw] name
        #   @return [::String]
        #     [Output Only] The name of the firewall policy.
        # @!attribute [rw] rules
        #   @return [::Array<::Google::Cloud::Compute::V1::FirewallPolicyRule>]
        #     The rules that apply to the network.
        # @!attribute [rw] type
        #   @return [::String]
        #     [Output Only] The type of the firewall policy. Can be one of HIERARCHY, NETWORK, NETWORK_REGIONAL.
        #     Check the Type enum for the list of possible values.
        class RegionNetworkFirewallPoliciesGetEffectiveFirewallsResponseEffectiveFirewallPolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # [Output Only] The type of the firewall policy. Can be one of HIERARCHY, NETWORK, NETWORK_REGIONAL.
          module Type
            # A value indicating that the enum field is not set.
            UNDEFINED_TYPE = 0

            HIERARCHY = 69_902_869

            NETWORK = 413_984_270

            NETWORK_REGIONAL = 190_804_272

            UNSPECIFIED = 526_786_327
          end
        end

        # @!attribute [rw] label_fingerprint
        #   @return [::String]
        #     The fingerprint of the previous set of labels for this resource, used to detect conflicts. The fingerprint is initially generated by Compute Engine and changes after every request to modify or update labels. You must always provide an up-to-date fingerprint hash in order to update or change labels. Make a get() request to the resource to get the latest fingerprint.
        # @!attribute [rw] labels
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     The labels to set for this resource.
        class RegionSetLabelsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class LabelsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # @!attribute [rw] bindings
        #   @return [::Array<::Google::Cloud::Compute::V1::Binding>]
        #     Flatten Policy to create a backwacd compatible wire-format. Deprecated. Use 'policy' to specify bindings.
        # @!attribute [rw] etag
        #   @return [::String]
        #     Flatten Policy to create a backward compatible wire-format. Deprecated. Use 'policy' to specify the etag.
        # @!attribute [rw] policy
        #   @return [::Google::Cloud::Compute::V1::Policy]
        #     REQUIRED: The complete policy to be applied to the 'resource'. The size of the policy is limited to a few 10s of KB. An empty policy is in general a valid policy but certain services (like Projects) might reject them.
        class RegionSetPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] ssl_certificates
        #   @return [::Array<::String>]
        #     New set of SslCertificate resources to associate with this TargetHttpsProxy resource.
        class RegionTargetHttpsProxiesSetSslCertificatesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] resource
        #   @return [::Google::Cloud::Compute::V1::UrlMap]
        #     Content of the UrlMap to be validated.
        class RegionUrlMapsValidateRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for FirewallPolicies.RemoveAssociation. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to update.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name for the attachment that will be removed.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class RemoveAssociationFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkFirewallPolicies.RemoveAssociation. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to update.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name for the attachment that will be removed.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class RemoveAssociationNetworkFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionNetworkFirewallPolicies.RemoveAssociation. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to update.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name for the association that will be removed.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class RemoveAssociationRegionNetworkFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetPools.RemoveHealthCheck. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_pool
        #   @return [::String]
        #     Name of the target pool to remove health checks from.
        # @!attribute [rw] target_pools_remove_health_check_request_resource
        #   @return [::Google::Cloud::Compute::V1::TargetPoolsRemoveHealthCheckRequest]
        #     The body resource for this request
        class RemoveHealthCheckTargetPoolRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetPools.RemoveInstance. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_pool
        #   @return [::String]
        #     Name of the TargetPool resource to remove instances from.
        # @!attribute [rw] target_pools_remove_instance_request_resource
        #   @return [::Google::Cloud::Compute::V1::TargetPoolsRemoveInstanceRequest]
        #     The body resource for this request
        class RemoveInstanceTargetPoolRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceGroups.RemoveInstances. See the method description for details.
        # @!attribute [rw] instance_group
        #   @return [::String]
        #     The name of the instance group where the specified instances will be removed.
        # @!attribute [rw] instance_groups_remove_instances_request_resource
        #   @return [::Google::Cloud::Compute::V1::InstanceGroupsRemoveInstancesRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where the instance group is located.
        class RemoveInstancesInstanceGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Networks.RemovePeering. See the method description for details.
        # @!attribute [rw] network
        #   @return [::String]
        #     Name of the network resource to remove peering from.
        # @!attribute [rw] networks_remove_peering_request_resource
        #   @return [::Google::Cloud::Compute::V1::NetworksRemovePeeringRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class RemovePeeringNetworkRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Disks.RemoveResourcePolicies. See the method description for details.
        # @!attribute [rw] disk
        #   @return [::String]
        #     The disk name for this request.
        # @!attribute [rw] disks_remove_resource_policies_request_resource
        #   @return [::Google::Cloud::Compute::V1::DisksRemoveResourcePoliciesRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class RemoveResourcePoliciesDiskRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.RemoveResourcePolicies. See the method description for details.
        # @!attribute [rw] instance
        #   @return [::String]
        #     The instance name for this request.
        # @!attribute [rw] instances_remove_resource_policies_request_resource
        #   @return [::Google::Cloud::Compute::V1::InstancesRemoveResourcePoliciesRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class RemoveResourcePoliciesInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionDisks.RemoveResourcePolicies. See the method description for details.
        # @!attribute [rw] disk
        #   @return [::String]
        #     The disk name for this request.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] region_disks_remove_resource_policies_request_resource
        #   @return [::Google::Cloud::Compute::V1::RegionDisksRemoveResourcePoliciesRequest]
        #     The body resource for this request
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class RemoveResourcePoliciesRegionDiskRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for FirewallPolicies.RemoveRule. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to update.
        # @!attribute [rw] priority
        #   @return [::Integer]
        #     The priority of the rule to remove from the firewall policy.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class RemoveRuleFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkFirewallPolicies.RemoveRule. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to update.
        # @!attribute [rw] priority
        #   @return [::Integer]
        #     The priority of the rule to remove from the firewall policy.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class RemoveRuleNetworkFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionNetworkFirewallPolicies.RemoveRule. See the method description for details.
        # @!attribute [rw] firewall_policy
        #   @return [::String]
        #     Name of the firewall policy to update.
        # @!attribute [rw] priority
        #   @return [::Integer]
        #     The priority of the rule to remove from the firewall policy.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class RemoveRuleRegionNetworkFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionSecurityPolicies.RemoveRule. See the method description for details.
        # @!attribute [rw] priority
        #   @return [::Integer]
        #     The priority of the rule to remove from the security policy.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] security_policy
        #   @return [::String]
        #     Name of the security policy to update.
        class RemoveRuleRegionSecurityPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for SecurityPolicies.RemoveRule. See the method description for details.
        # @!attribute [rw] priority
        #   @return [::Integer]
        #     The priority of the rule to remove from the security policy.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] security_policy
        #   @return [::String]
        #     Name of the security policy to update.
        class RemoveRuleSecurityPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A policy that specifies how requests intended for the route's backends are shadowed to a separate mirrored backend service. The load balancer doesn't wait for responses from the shadow service. Before sending traffic to the shadow service, the host or authority header is suffixed with -shadow.
        # @!attribute [rw] backend_service
        #   @return [::String]
        #     The full or partial URL to the BackendService resource being mirrored to. The backend service configured for a mirroring policy must reference backends that are of the same type as the original backend service matched in the URL map. Serverless NEG backends are not currently supported as a mirrored backend service.
        class RequestMirrorPolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a reservation resource. A reservation ensures that capacity is held in a specific zone even if the reserved VMs are not running. For more information, read Reserving zonal resources.
        # @!attribute [rw] commitment
        #   @return [::String]
        #     [Output Only] Full or partial URL to a parent commitment. This field displays for reservations that are tied to a commitment.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#reservations for reservations.
        # @!attribute [rw] name
        #   @return [::String]
        #     The name of the resource, provided by the client when initially creating the resource. The resource name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] resource_policies
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     Resource policies to be added to this reservation. The key is defined by user, and the value is resource policy url. This is to define placement policy with reservation.
        # @!attribute [rw] resource_status
        #   @return [::Google::Cloud::Compute::V1::AllocationResourceStatus]
        #     [Output Only] Status information for Reservation resource.
        # @!attribute [rw] satisfies_pzs
        #   @return [::Boolean]
        #     [Output Only] Reserved for future use.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined fully-qualified URL for this resource.
        # @!attribute [rw] share_settings
        #   @return [::Google::Cloud::Compute::V1::ShareSettings]
        #     Specify share-settings to create a shared reservation. This property is optional. For more information about the syntax and options for this field and its subfields, see the guide for creating a shared reservation.
        # @!attribute [rw] specific_reservation
        #   @return [::Google::Cloud::Compute::V1::AllocationSpecificSKUReservation]
        #     Reservation for instances with specific machine shapes.
        # @!attribute [rw] specific_reservation_required
        #   @return [::Boolean]
        #     Indicates whether the reservation can be consumed by VMs with affinity for "any" reservation. If the field is set, then only VMs that target the reservation by name can consume from this reservation.
        # @!attribute [rw] status
        #   @return [::String]
        #     [Output Only] The status of the reservation.
        #     Check the Status enum for the list of possible values.
        # @!attribute [rw] zone
        #   @return [::String]
        #     Zone in which the reservation resides. A zone must be provided if the reservation is created within a commitment.
        class Reservation
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class ResourcePoliciesEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # [Output Only] The status of the reservation.
          module Status
            # A value indicating that the enum field is not set.
            UNDEFINED_STATUS = 0

            # Resources are being allocated for the reservation.
            CREATING = 455_564_985

            # Reservation is currently being deleted.
            DELETING = 528_602_024

            INVALID = 530_283_991

            # Reservation has allocated all its resources.
            READY = 77_848_963

            # Reservation is currently being resized.
            UPDATING = 494_614_342
          end
        end

        # Specifies the reservations that this instance can consume from.
        # @!attribute [rw] consume_reservation_type
        #   @return [::String]
        #     Specifies the type of reservation from which this instance can consume resources: ANY_RESERVATION (default), SPECIFIC_RESERVATION, or NO_RESERVATION. See Consuming reserved instances for examples.
        #     Check the ConsumeReservationType enum for the list of possible values.
        # @!attribute [rw] key
        #   @return [::String]
        #     Corresponds to the label key of a reservation resource. To target a SPECIFIC_RESERVATION by name, specify googleapis.com/reservation-name as the key and specify the name of your reservation as its value.
        # @!attribute [rw] values
        #   @return [::Array<::String>]
        #     Corresponds to the label values of a reservation resource. This can be either a name to a reservation in the same project or "projects/different-project/reservations/some-reservation-name" to target a shared reservation in the same zone but in a different project.
        class ReservationAffinity
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Specifies the type of reservation from which this instance can consume resources: ANY_RESERVATION (default), SPECIFIC_RESERVATION, or NO_RESERVATION. See Consuming reserved instances for examples.
          module ConsumeReservationType
            # A value indicating that the enum field is not set.
            UNDEFINED_CONSUME_RESERVATION_TYPE = 0

            # Consume any allocation available.
            ANY_RESERVATION = 200_008_121

            # Do not consume from any allocated capacity.
            NO_RESERVATION = 169_322_030

            # Must consume from a specific reservation. Must specify key value fields for specifying the reservations.
            SPECIFIC_RESERVATION = 229_889_055

            UNSPECIFIED = 526_786_327
          end
        end

        # Contains a list of reservations.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::ReservationsScopedList}]
        #     A list of Allocation resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     Type of resource.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class ReservationAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::ReservationsScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::Reservation>]
        #     [Output Only] A list of Allocation resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource.Always compute#reservationsList for listsof reservations
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class ReservationList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] specific_sku_count
        #   @return [::Integer]
        #     Number of allocated resources can be resized with minimum = 1 and maximum = 1000.
        class ReservationsResizeRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] reservations
        #   @return [::Array<::Google::Cloud::Compute::V1::Reservation>]
        #     A list of reservations contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     Informational warning which replaces the list of reservations when the list is empty.
        class ReservationsScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.Reset. See the method description for details.
        # @!attribute [rw] instance
        #   @return [::String]
        #     Name of the instance scoping this request.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class ResetInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Disks.Resize. See the method description for details.
        # @!attribute [rw] disk
        #   @return [::String]
        #     The name of the persistent disk.
        # @!attribute [rw] disks_resize_request_resource
        #   @return [::Google::Cloud::Compute::V1::DisksResizeRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class ResizeDiskRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceGroupManagers.Resize. See the method description for details.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     The name of the managed instance group.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] size
        #   @return [::Integer]
        #     The number of running instances that the managed instance group should maintain at any given time. The group automatically adds or removes instances to maintain the number of instances specified by this parameter.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where the managed instance group is located.
        class ResizeInstanceGroupManagerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionDisks.Resize. See the method description for details.
        # @!attribute [rw] disk
        #   @return [::String]
        #     Name of the regional persistent disk.
        # @!attribute [rw] project
        #   @return [::String]
        #     The project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] region_disks_resize_request_resource
        #   @return [::Google::Cloud::Compute::V1::RegionDisksResizeRequest]
        #     The body resource for this request
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class ResizeRegionDiskRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionInstanceGroupManagers.Resize. See the method description for details.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     Name of the managed instance group.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] size
        #   @return [::Integer]
        #     Number of instances that should exist in this instance group manager.
        class ResizeRegionInstanceGroupManagerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Reservations.Resize. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] reservation
        #   @return [::String]
        #     Name of the reservation to update.
        # @!attribute [rw] reservations_resize_request_resource
        #   @return [::Google::Cloud::Compute::V1::ReservationsResizeRequest]
        #     The body resource for this request
        # @!attribute [rw] zone
        #   @return [::String]
        #     Name of the zone for this request.
        class ResizeReservationRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Commitment for a particular resource (a Commitment is composed of one or more of these).
        # @!attribute [rw] accelerator_type
        #   @return [::String]
        #     Name of the accelerator type resource. Applicable only when the type is ACCELERATOR.
        # @!attribute [rw] amount
        #   @return [::Integer]
        #     The amount of the resource purchased (in a type-dependent unit, such as bytes). For vCPUs, this can just be an integer. For memory, this must be provided in MB. Memory must be a multiple of 256 MB, with up to 6.5GB of memory per every vCPU.
        # @!attribute [rw] type
        #   @return [::String]
        #     Type of resource for which this commitment applies. Possible values are VCPU, MEMORY, LOCAL_SSD, and ACCELERATOR.
        #     Check the Type enum for the list of possible values.
        class ResourceCommitment
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Type of resource for which this commitment applies. Possible values are VCPU, MEMORY, LOCAL_SSD, and ACCELERATOR.
          module Type
            # A value indicating that the enum field is not set.
            UNDEFINED_TYPE = 0

            ACCELERATOR = 429_815_371

            LOCAL_SSD = 508_934_896

            MEMORY = 123_056_385

            UNSPECIFIED = 526_786_327

            VCPU = 2_628_978
          end
        end

        # @!attribute [rw] group
        #   @return [::String]
        #     A URI referencing one of the instance groups or network endpoint groups listed in the backend service.
        class ResourceGroupReference
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] resource_policies
        #   @return [::Array<::Google::Cloud::Compute::V1::ResourcePolicy>]
        #     A list of resourcePolicies contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     Informational warning which replaces the list of resourcePolicies when the list is empty.
        class ResourcePoliciesScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a Resource Policy resource. You can use resource policies to schedule actions for some Compute Engine resources. For example, you can use them to schedule persistent disk snapshots.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        # @!attribute [rw] disk_consistency_group_policy
        #   @return [::Google::Cloud::Compute::V1::ResourcePolicyDiskConsistencyGroupPolicy]
        #     Resource policy for disk consistency groups.
        # @!attribute [rw] group_placement_policy
        #   @return [::Google::Cloud::Compute::V1::ResourcePolicyGroupPlacementPolicy]
        #     Resource policy for instances for placement configuration.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] instance_schedule_policy
        #   @return [::Google::Cloud::Compute::V1::ResourcePolicyInstanceSchedulePolicy]
        #     Resource policy for scheduling instance operations.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#resource_policies for resource policies.
        # @!attribute [rw] name
        #   @return [::String]
        #     The name of the resource, provided by the client when initially creating the resource. The resource name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] region
        #   @return [::String]
        # @!attribute [rw] resource_status
        #   @return [::Google::Cloud::Compute::V1::ResourcePolicyResourceStatus]
        #     [Output Only] The system status of the resource policy.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined fully-qualified URL for this resource.
        # @!attribute [rw] snapshot_schedule_policy
        #   @return [::Google::Cloud::Compute::V1::ResourcePolicySnapshotSchedulePolicy]
        #     Resource policy for persistent disks for creating snapshots.
        # @!attribute [rw] status
        #   @return [::String]
        #     [Output Only] The status of resource policy creation.
        #     Check the Status enum for the list of possible values.
        class ResourcePolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # [Output Only] The status of resource policy creation.
          module Status
            # A value indicating that the enum field is not set.
            UNDEFINED_STATUS = 0

            # Resource policy is being created.
            CREATING = 455_564_985

            # Resource policy is being deleted.
            DELETING = 528_602_024

            # Resource policy is expired and will not run again.
            EXPIRED = 482_489_093

            INVALID = 530_283_991

            # Resource policy is ready to be used.
            READY = 77_848_963
          end
        end

        # Contains a list of resourcePolicies.
        # @!attribute [rw] etag
        #   @return [::String]
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::ResourcePoliciesScopedList}]
        #     A list of ResourcePolicy resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     Type of resource.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class ResourcePolicyAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::ResourcePoliciesScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # Time window specified for daily operations.
        # @!attribute [rw] days_in_cycle
        #   @return [::Integer]
        #     Defines a schedule with units measured in days. The value determines how many days pass between the start of each cycle.
        # @!attribute [rw] duration
        #   @return [::String]
        #     [Output only] A predetermined duration for the window, automatically chosen to be the smallest possible in the given scenario.
        # @!attribute [rw] start_time
        #   @return [::String]
        #     Start time of the window. This must be in UTC format that resolves to one of 00:00, 04:00, 08:00, 12:00, 16:00, or 20:00. For example, both 13:00-5 and 08:00 are valid.
        class ResourcePolicyDailyCycle
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Resource policy for disk consistency groups.
        class ResourcePolicyDiskConsistencyGroupPolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A GroupPlacementPolicy specifies resource placement configuration. It specifies the failure bucket separation as well as network locality
        # @!attribute [rw] availability_domain_count
        #   @return [::Integer]
        #     The number of availability domains to spread instances across. If two instances are in different availability domain, they are not in the same low latency network.
        # @!attribute [rw] collocation
        #   @return [::String]
        #     Specifies network collocation
        #     Check the Collocation enum for the list of possible values.
        # @!attribute [rw] vm_count
        #   @return [::Integer]
        #     Number of VMs in this placement group. Google does not recommend that you use this field unless you use a compact policy and you want your policy to work only if it contains this exact number of VMs.
        class ResourcePolicyGroupPlacementPolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Specifies network collocation
          module Collocation
            # A value indicating that the enum field is not set.
            UNDEFINED_COLLOCATION = 0

            COLLOCATED = 103_257_554

            UNSPECIFIED_COLLOCATION = 464_308_205
          end
        end

        # Time window specified for hourly operations.
        # @!attribute [rw] duration
        #   @return [::String]
        #     [Output only] Duration of the time window, automatically chosen to be smallest possible in the given scenario.
        # @!attribute [rw] hours_in_cycle
        #   @return [::Integer]
        #     Defines a schedule with units measured in hours. The value determines how many hours pass between the start of each cycle.
        # @!attribute [rw] start_time
        #   @return [::String]
        #     Time within the window to start the operations. It must be in format "HH:MM", where HH : [00-23] and MM : [00-00] GMT.
        class ResourcePolicyHourlyCycle
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # An InstanceSchedulePolicy specifies when and how frequent certain operations are performed on the instance.
        # @!attribute [rw] expiration_time
        #   @return [::String]
        #     The expiration time of the schedule. The timestamp is an RFC3339 string.
        # @!attribute [rw] start_time
        #   @return [::String]
        #     The start time of the schedule. The timestamp is an RFC3339 string.
        # @!attribute [rw] time_zone
        #   @return [::String]
        #     Specifies the time zone to be used in interpreting Schedule.schedule. The value of this field must be a time zone name from the tz database: https://wikipedia.org/wiki/Tz_database.
        # @!attribute [rw] vm_start_schedule
        #   @return [::Google::Cloud::Compute::V1::ResourcePolicyInstanceSchedulePolicySchedule]
        #     Specifies the schedule for starting instances.
        # @!attribute [rw] vm_stop_schedule
        #   @return [::Google::Cloud::Compute::V1::ResourcePolicyInstanceSchedulePolicySchedule]
        #     Specifies the schedule for stopping instances.
        class ResourcePolicyInstanceSchedulePolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Schedule for an instance operation.
        # @!attribute [rw] schedule
        #   @return [::String]
        #     Specifies the frequency for the operation, using the unix-cron format.
        class ResourcePolicyInstanceSchedulePolicySchedule
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] etag
        #   @return [::String]
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::ResourcePolicy>]
        #     [Output Only] A list of ResourcePolicy resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource.Always compute#resourcePoliciesList for listsof resourcePolicies
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class ResourcePolicyList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Contains output only fields. Use this sub-message for all output fields set on ResourcePolicy. The internal structure of this "status" field should mimic the structure of ResourcePolicy proto specification.
        # @!attribute [rw] instance_schedule_policy
        #   @return [::Google::Cloud::Compute::V1::ResourcePolicyResourceStatusInstanceSchedulePolicyStatus]
        #     [Output Only] Specifies a set of output values reffering to the instance_schedule_policy system status. This field should have the same name as corresponding policy field.
        class ResourcePolicyResourceStatus
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] last_run_start_time
        #   @return [::String]
        #     [Output Only] The last time the schedule successfully ran. The timestamp is an RFC3339 string.
        # @!attribute [rw] next_run_start_time
        #   @return [::String]
        #     [Output Only] The next time the schedule is planned to run. The actual time might be slightly different. The timestamp is an RFC3339 string.
        class ResourcePolicyResourceStatusInstanceSchedulePolicyStatus
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A snapshot schedule policy specifies when and how frequently snapshots are to be created for the target disk. Also specifies how many and how long these scheduled snapshots should be retained.
        # @!attribute [rw] retention_policy
        #   @return [::Google::Cloud::Compute::V1::ResourcePolicySnapshotSchedulePolicyRetentionPolicy]
        #     Retention policy applied to snapshots created by this resource policy.
        # @!attribute [rw] schedule
        #   @return [::Google::Cloud::Compute::V1::ResourcePolicySnapshotSchedulePolicySchedule]
        #     A Vm Maintenance Policy specifies what kind of infrastructure maintenance we are allowed to perform on this VM and when. Schedule that is applied to disks covered by this policy.
        # @!attribute [rw] snapshot_properties
        #   @return [::Google::Cloud::Compute::V1::ResourcePolicySnapshotSchedulePolicySnapshotProperties]
        #     Properties with which snapshots are created such as labels, encryption keys.
        class ResourcePolicySnapshotSchedulePolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Policy for retention of scheduled snapshots.
        # @!attribute [rw] max_retention_days
        #   @return [::Integer]
        #     Maximum age of the snapshot that is allowed to be kept.
        # @!attribute [rw] on_source_disk_delete
        #   @return [::String]
        #     Specifies the behavior to apply to scheduled snapshots when the source disk is deleted.
        #     Check the OnSourceDiskDelete enum for the list of possible values.
        class ResourcePolicySnapshotSchedulePolicyRetentionPolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Specifies the behavior to apply to scheduled snapshots when the source disk is deleted.
          module OnSourceDiskDelete
            # A value indicating that the enum field is not set.
            UNDEFINED_ON_SOURCE_DISK_DELETE = 0

            APPLY_RETENTION_POLICY = 535_071_332

            KEEP_AUTO_SNAPSHOTS = 258_925_689

            UNSPECIFIED_ON_SOURCE_DISK_DELETE = 239_140_769
          end
        end

        # A schedule for disks where the schedueled operations are performed.
        # @!attribute [rw] daily_schedule
        #   @return [::Google::Cloud::Compute::V1::ResourcePolicyDailyCycle]
        # @!attribute [rw] hourly_schedule
        #   @return [::Google::Cloud::Compute::V1::ResourcePolicyHourlyCycle]
        # @!attribute [rw] weekly_schedule
        #   @return [::Google::Cloud::Compute::V1::ResourcePolicyWeeklyCycle]
        class ResourcePolicySnapshotSchedulePolicySchedule
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Specified snapshot properties for scheduled snapshots created by this policy.
        # @!attribute [rw] chain_name
        #   @return [::String]
        #     Chain name that the snapshot is created in.
        # @!attribute [rw] guest_flush
        #   @return [::Boolean]
        #     Indication to perform a 'guest aware' snapshot.
        # @!attribute [rw] labels
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     Labels to apply to scheduled snapshots. These can be later modified by the setLabels method. Label values may be empty.
        # @!attribute [rw] storage_locations
        #   @return [::Array<::String>]
        #     Cloud Storage bucket storage location of the auto snapshot (regional or multi-regional).
        class ResourcePolicySnapshotSchedulePolicySnapshotProperties
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class LabelsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # Time window specified for weekly operations.
        # @!attribute [rw] day_of_weeks
        #   @return [::Array<::Google::Cloud::Compute::V1::ResourcePolicyWeeklyCycleDayOfWeek>]
        #     Up to 7 intervals/windows, one for each day of the week.
        class ResourcePolicyWeeklyCycle
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] day
        #   @return [::String]
        #     Defines a schedule that runs on specific days of the week. Specify one or more days. The following options are available: MONDAY, TUESDAY, WEDNESDAY, THURSDAY, FRIDAY, SATURDAY, SUNDAY.
        #     Check the Day enum for the list of possible values.
        # @!attribute [rw] duration
        #   @return [::String]
        #     [Output only] Duration of the time window, automatically chosen to be smallest possible in the given scenario.
        # @!attribute [rw] start_time
        #   @return [::String]
        #     Time within the window to start the operations. It must be in format "HH:MM", where HH : [00-23] and MM : [00-00] GMT.
        class ResourcePolicyWeeklyCycleDayOfWeek
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Defines a schedule that runs on specific days of the week. Specify one or more days. The following options are available: MONDAY, TUESDAY, WEDNESDAY, THURSDAY, FRIDAY, SATURDAY, SUNDAY.
          module Day
            # A value indicating that the enum field is not set.
            UNDEFINED_DAY = 0

            FRIDAY = 471_398_751

            INVALID = 530_283_991

            MONDAY = 132_310_288

            SATURDAY = 279_037_881

            SUNDAY = 309_626_320

            THURSDAY = 207_198_682

            TUESDAY = 277_509_677

            WEDNESDAY = 422_029_110
          end
        end

        # Contains output only fields. Use this sub-message for actual values set on Instance attributes as compared to the value requested by the user (intent) in their instance CRUD calls.
        # @!attribute [rw] physical_host
        #   @return [::String]
        #     [Output Only] An opaque ID of the host on which the VM is running.
        # @!attribute [rw] upcoming_maintenance
        #   @return [::Google::Cloud::Compute::V1::UpcomingMaintenance]
        class ResourceStatus
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.Resume. See the method description for details.
        # @!attribute [rw] instance
        #   @return [::String]
        #     Name of the instance resource to resume.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class ResumeInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a Route resource. A route defines a path from VM instances in the VPC network to a specific destination. This destination can be inside or outside the VPC network. For more information, read the Routes overview.
        # @!attribute [rw] as_paths
        #   @return [::Array<::Google::Cloud::Compute::V1::RouteAsPath>]
        #     [Output Only] AS path.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this field when you create the resource.
        # @!attribute [rw] dest_range
        #   @return [::String]
        #     The destination range of outgoing packets that this route applies to. Both IPv4 and IPv6 are supported. Must specify an IPv4 range (e.g. 192.0.2.0/24) or an IPv6 range in RFC 4291 format (e.g. 2001:db8::/32). IPv6 range will be displayed using RFC 5952 compressed format.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of this resource. Always compute#routes for Route resources.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?`. The first character must be a lowercase letter, and all following characters (except for the last character) must be a dash, lowercase letter, or digit. The last character must be a lowercase letter or digit.
        # @!attribute [rw] network
        #   @return [::String]
        #     Fully-qualified URL of the network that this route applies to.
        # @!attribute [rw] next_hop_gateway
        #   @return [::String]
        #     The URL to a gateway that should handle matching packets. You can only specify the internet gateway using a full or partial valid URL: projects/ project/global/gateways/default-internet-gateway
        # @!attribute [rw] next_hop_hub
        #   @return [::String]
        #     [Output Only] The full resource name of the Network Connectivity Center hub that will handle matching packets.
        # @!attribute [rw] next_hop_ilb
        #   @return [::String]
        #     The URL to a forwarding rule of type loadBalancingScheme=INTERNAL that should handle matching packets or the IP address of the forwarding Rule. For example, the following are all valid URLs: - 10.128.0.56 - https://www.googleapis.com/compute/v1/projects/project/regions/region /forwardingRules/forwardingRule - regions/region/forwardingRules/forwardingRule
        # @!attribute [rw] next_hop_instance
        #   @return [::String]
        #     The URL to an instance that should handle matching packets. You can specify this as a full or partial URL. For example: https://www.googleapis.com/compute/v1/projects/project/zones/zone/instances/
        # @!attribute [rw] next_hop_ip
        #   @return [::String]
        #     The network IP address of an instance that should handle matching packets. Both IPv6 address and IPv4 addresses are supported. Must specify an IPv4 address in dot-decimal notation (e.g. 192.0.2.99) or an IPv6 address in RFC 4291 format (e.g. 2001:db8::2d9:51:0:0 or 2001:db8:0:0:2d9:51:0:0). IPv6 addresses will be displayed using RFC 5952 compressed format (e.g. 2001:db8::2d9:51:0:0). Should never be an IPv4-mapped IPv6 address.
        # @!attribute [rw] next_hop_network
        #   @return [::String]
        #     The URL of the local network if it should handle matching packets.
        # @!attribute [rw] next_hop_peering
        #   @return [::String]
        #     [Output Only] The network peering name that should handle matching packets, which should conform to RFC1035.
        # @!attribute [rw] next_hop_vpn_tunnel
        #   @return [::String]
        #     The URL to a VpnTunnel that should handle matching packets.
        # @!attribute [rw] priority
        #   @return [::Integer]
        #     The priority of this route. Priority is used to break ties in cases where there is more than one matching route of equal prefix length. In cases where multiple routes have equal prefix length, the one with the lowest-numbered priority value wins. The default value is `1000`. The priority value must be from `0` to `65535`, inclusive.
        # @!attribute [rw] route_status
        #   @return [::String]
        #     [Output only] The status of the route.
        #     Check the RouteStatus enum for the list of possible values.
        # @!attribute [rw] route_type
        #   @return [::String]
        #     [Output Only] The type of this route, which can be one of the following values: - 'TRANSIT' for a transit route that this router learned from another Cloud Router and will readvertise to one of its BGP peers - 'SUBNET' for a route from a subnet of the VPC - 'BGP' for a route learned from a BGP peer of this router - 'STATIC' for a static route
        #     Check the RouteType enum for the list of possible values.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined fully-qualified URL for this resource.
        # @!attribute [rw] tags
        #   @return [::Array<::String>]
        #     A list of instance tags to which this route applies.
        # @!attribute [rw] warnings
        #   @return [::Array<::Google::Cloud::Compute::V1::Warnings>]
        #     [Output Only] If potential misconfigurations are detected for this route, this field will be populated with warning messages.
        class Route
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # [Output only] The status of the route.
          module RouteStatus
            # A value indicating that the enum field is not set.
            UNDEFINED_ROUTE_STATUS = 0

            # This route is processed and active.
            ACTIVE = 314_733_318

            # The route is dropped due to the VPC exceeding the dynamic route limit. For dynamic route limit, please refer to the Learned route example
            DROPPED = 496_235_424

            # This route is processed but inactive due to failure from the backend. The backend may have rejected the route
            INACTIVE = 270_421_099

            # This route is being processed internally. The status will change once processed.
            PENDING = 35_394_935
          end

          # [Output Only] The type of this route, which can be one of the following values: - 'TRANSIT' for a transit route that this router learned from another Cloud Router and will readvertise to one of its BGP peers - 'SUBNET' for a route from a subnet of the VPC - 'BGP' for a route learned from a BGP peer of this router - 'STATIC' for a static route
          module RouteType
            # A value indicating that the enum field is not set.
            UNDEFINED_ROUTE_TYPE = 0

            BGP = 65_707

            STATIC = 308_331_118

            SUBNET = 309_278_557

            TRANSIT = 187_793_843
          end
        end

        # @!attribute [rw] as_lists
        #   @return [::Array<::Integer>]
        #     [Output Only] The AS numbers of the AS Path.
        # @!attribute [rw] path_segment_type
        #   @return [::String]
        #     [Output Only] The type of the AS Path, which can be one of the following values: - 'AS_SET': unordered set of autonomous systems that the route in has traversed - 'AS_SEQUENCE': ordered set of autonomous systems that the route has traversed - 'AS_CONFED_SEQUENCE': ordered set of Member Autonomous Systems in the local confederation that the route has traversed - 'AS_CONFED_SET': unordered set of Member Autonomous Systems in the local confederation that the route has traversed
        #     Check the PathSegmentType enum for the list of possible values.
        class RouteAsPath
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # [Output Only] The type of the AS Path, which can be one of the following values: - 'AS_SET': unordered set of autonomous systems that the route in has traversed - 'AS_SEQUENCE': ordered set of autonomous systems that the route has traversed - 'AS_CONFED_SEQUENCE': ordered set of Member Autonomous Systems in the local confederation that the route has traversed - 'AS_CONFED_SET': unordered set of Member Autonomous Systems in the local confederation that the route has traversed
          module PathSegmentType
            # A value indicating that the enum field is not set.
            UNDEFINED_PATH_SEGMENT_TYPE = 0

            AS_CONFED_SEQUENCE = 222_152_624

            AS_CONFED_SET = 374_040_307

            AS_SEQUENCE = 106_735_918

            AS_SET = 329_846_453
          end
        end

        # Contains a list of Route resources.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::Route>]
        #     A list of Route resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     Type of resource.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class RouteList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a Cloud Router resource. For more information about Cloud Router, read the Cloud Router overview.
        # @!attribute [rw] bgp
        #   @return [::Google::Cloud::Compute::V1::RouterBgp]
        #     BGP information specific to this router.
        # @!attribute [rw] bgp_peers
        #   @return [::Array<::Google::Cloud::Compute::V1::RouterBgpPeer>]
        #     BGP information that must be configured into the routing stack to establish BGP peering. This information must specify the peer ASN and either the interface name, IP address, or peer IP address. Please refer to RFC4273.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] encrypted_interconnect_router
        #   @return [::Boolean]
        #     Indicates if a router is dedicated for use with encrypted VLAN attachments (interconnectAttachments).
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] interfaces
        #   @return [::Array<::Google::Cloud::Compute::V1::RouterInterface>]
        #     Router interfaces. To create a BGP peer that uses a router interface, the interface must have one of the following fields specified: - linkedVpnTunnel - linkedInterconnectAttachment - subnetwork You can create a router interface without any of these fields specified. However, you cannot create a BGP peer that uses that interface.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#router for routers.
        # @!attribute [rw] md5_authentication_keys
        #   @return [::Array<::Google::Cloud::Compute::V1::RouterMd5AuthenticationKey>]
        #     Keys used for MD5 authentication.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] nats
        #   @return [::Array<::Google::Cloud::Compute::V1::RouterNat>]
        #     A list of NAT services created in this router.
        # @!attribute [rw] network
        #   @return [::String]
        #     URI of the network to which this router belongs.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] URI of the region where the router resides. You must specify this field as part of the HTTP request URL. It is not settable as a field in the request body.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        class Router
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Description-tagged IP ranges for the router to advertise.
        # @!attribute [rw] description
        #   @return [::String]
        #     User-specified description for the IP range.
        # @!attribute [rw] range
        #   @return [::String]
        #     The IP range to advertise. The value must be a CIDR-formatted string.
        class RouterAdvertisedIpRange
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Contains a list of routers.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::RoutersScopedList}]
        #     A list of Router resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     Type of resource.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class RouterAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::RoutersScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # @!attribute [rw] advertise_mode
        #   @return [::String]
        #     User-specified flag to indicate which mode to use for advertisement. The options are DEFAULT or CUSTOM.
        #     Check the AdvertiseMode enum for the list of possible values.
        # @!attribute [rw] advertised_groups
        #   @return [::Array<::String>]
        #     User-specified list of prefix groups to advertise in custom mode. This field can only be populated if advertise_mode is CUSTOM and is advertised to all peers of the router. These groups will be advertised in addition to any specified prefixes. Leave this field blank to advertise no custom groups.
        #     Check the AdvertisedGroups enum for the list of possible values.
        # @!attribute [rw] advertised_ip_ranges
        #   @return [::Array<::Google::Cloud::Compute::V1::RouterAdvertisedIpRange>]
        #     User-specified list of individual IP ranges to advertise in custom mode. This field can only be populated if advertise_mode is CUSTOM and is advertised to all peers of the router. These IP ranges will be advertised in addition to any specified groups. Leave this field blank to advertise no custom IP ranges.
        # @!attribute [rw] asn
        #   @return [::Integer]
        #     Local BGP Autonomous System Number (ASN). Must be an RFC6996 private ASN, either 16-bit or 32-bit. The value will be fixed for this router resource. All VPN tunnels that link to this router will have the same local ASN.
        # @!attribute [rw] keepalive_interval
        #   @return [::Integer]
        #     The interval in seconds between BGP keepalive messages that are sent to the peer. Hold time is three times the interval at which keepalive messages are sent, and the hold time is the maximum number of seconds allowed to elapse between successive keepalive messages that BGP receives from a peer. BGP will use the smaller of either the local hold time value or the peer's hold time value as the hold time for the BGP connection between the two peers. If set, this value must be between 20 and 60. The default is 20.
        class RouterBgp
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # User-specified flag to indicate which mode to use for advertisement. The options are DEFAULT or CUSTOM.
          module AdvertiseMode
            # A value indicating that the enum field is not set.
            UNDEFINED_ADVERTISE_MODE = 0

            CUSTOM = 388_595_569

            DEFAULT = 115_302_945
          end

          module AdvertisedGroups
            # A value indicating that the enum field is not set.
            UNDEFINED_ADVERTISED_GROUPS = 0

            # Advertise all available subnets (including peer VPC subnets).
            ALL_SUBNETS = 3_622_872
          end
        end

        # @!attribute [rw] advertise_mode
        #   @return [::String]
        #     User-specified flag to indicate which mode to use for advertisement.
        #     Check the AdvertiseMode enum for the list of possible values.
        # @!attribute [rw] advertised_groups
        #   @return [::Array<::String>]
        #     User-specified list of prefix groups to advertise in custom mode, which currently supports the following option: - ALL_SUBNETS: Advertises all of the router's own VPC subnets. This excludes any routes learned for subnets that use VPC Network Peering. Note that this field can only be populated if advertise_mode is CUSTOM and overrides the list defined for the router (in the "bgp" message). These groups are advertised in addition to any specified prefixes. Leave this field blank to advertise no custom groups.
        #     Check the AdvertisedGroups enum for the list of possible values.
        # @!attribute [rw] advertised_ip_ranges
        #   @return [::Array<::Google::Cloud::Compute::V1::RouterAdvertisedIpRange>]
        #     User-specified list of individual IP ranges to advertise in custom mode. This field can only be populated if advertise_mode is CUSTOM and overrides the list defined for the router (in the "bgp" message). These IP ranges are advertised in addition to any specified groups. Leave this field blank to advertise no custom IP ranges.
        # @!attribute [rw] advertised_route_priority
        #   @return [::Integer]
        #     The priority of routes advertised to this BGP peer. Where there is more than one matching route of maximum length, the routes with the lowest priority value win.
        # @!attribute [rw] bfd
        #   @return [::Google::Cloud::Compute::V1::RouterBgpPeerBfd]
        #     BFD configuration for the BGP peering.
        # @!attribute [rw] custom_learned_ip_ranges
        #   @return [::Array<::Google::Cloud::Compute::V1::RouterBgpPeerCustomLearnedIpRange>]
        #     A list of user-defined custom learned route IP address ranges for a BGP session.
        # @!attribute [rw] custom_learned_route_priority
        #   @return [::Integer]
        #     The user-defined custom learned route priority for a BGP session. This value is applied to all custom learned route ranges for the session. You can choose a value from `0` to `65335`. If you don't provide a value, Google Cloud assigns a priority of `100` to the ranges.
        # @!attribute [rw] enable
        #   @return [::String]
        #     The status of the BGP peer connection. If set to FALSE, any active session with the peer is terminated and all associated routing information is removed. If set to TRUE, the peer connection can be established with routing information. The default is TRUE.
        #     Check the Enable enum for the list of possible values.
        # @!attribute [rw] enable_ipv6
        #   @return [::Boolean]
        #     Enable IPv6 traffic over BGP Peer. If not specified, it is disabled by default.
        # @!attribute [rw] interface_name
        #   @return [::String]
        #     Name of the interface the BGP peer is associated with.
        # @!attribute [rw] ip_address
        #   @return [::String]
        #     IP address of the interface inside Google Cloud Platform. Only IPv4 is supported.
        # @!attribute [rw] ipv6_nexthop_address
        #   @return [::String]
        #     IPv6 address of the interface inside Google Cloud Platform.
        # @!attribute [rw] management_type
        #   @return [::String]
        #     [Output Only] The resource that configures and manages this BGP peer. - MANAGED_BY_USER is the default value and can be managed by you or other users - MANAGED_BY_ATTACHMENT is a BGP peer that is configured and managed by Cloud Interconnect, specifically by an InterconnectAttachment of type PARTNER. Google automatically creates, updates, and deletes this type of BGP peer when the PARTNER InterconnectAttachment is created, updated, or deleted.
        #     Check the ManagementType enum for the list of possible values.
        # @!attribute [rw] md5_authentication_key_name
        #   @return [::String]
        #     Present if MD5 authentication is enabled for the peering. Must be the name of one of the entries in the Router.md5_authentication_keys. The field must comply with RFC1035.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of this BGP peer. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] peer_asn
        #   @return [::Integer]
        #     Peer BGP Autonomous System Number (ASN). Each BGP interface may use a different value.
        # @!attribute [rw] peer_ip_address
        #   @return [::String]
        #     IP address of the BGP interface outside Google Cloud Platform. Only IPv4 is supported.
        # @!attribute [rw] peer_ipv6_nexthop_address
        #   @return [::String]
        #     IPv6 address of the BGP interface outside Google Cloud Platform.
        # @!attribute [rw] router_appliance_instance
        #   @return [::String]
        #     URI of the VM instance that is used as third-party router appliances such as Next Gen Firewalls, Virtual Routers, or Router Appliances. The VM instance must be located in zones contained in the same region as this Cloud Router. The VM instance is the peer side of the BGP session.
        class RouterBgpPeer
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # User-specified flag to indicate which mode to use for advertisement.
          module AdvertiseMode
            # A value indicating that the enum field is not set.
            UNDEFINED_ADVERTISE_MODE = 0

            CUSTOM = 388_595_569

            DEFAULT = 115_302_945
          end

          module AdvertisedGroups
            # A value indicating that the enum field is not set.
            UNDEFINED_ADVERTISED_GROUPS = 0

            # Advertise all available subnets (including peer VPC subnets).
            ALL_SUBNETS = 3_622_872
          end

          # The status of the BGP peer connection. If set to FALSE, any active session with the peer is terminated and all associated routing information is removed. If set to TRUE, the peer connection can be established with routing information. The default is TRUE.
          module Enable
            # A value indicating that the enum field is not set.
            UNDEFINED_ENABLE = 0

            FALSE = 66_658_563

            TRUE = 2_583_950
          end

          # [Output Only] The resource that configures and manages this BGP peer. - MANAGED_BY_USER is the default value and can be managed by you or other users - MANAGED_BY_ATTACHMENT is a BGP peer that is configured and managed by Cloud Interconnect, specifically by an InterconnectAttachment of type PARTNER. Google automatically creates, updates, and deletes this type of BGP peer when the PARTNER InterconnectAttachment is created, updated, or deleted.
          module ManagementType
            # A value indicating that the enum field is not set.
            UNDEFINED_MANAGEMENT_TYPE = 0

            # The BGP peer is automatically created for PARTNER type InterconnectAttachment; Google will automatically create/delete this BGP peer when the PARTNER InterconnectAttachment is created/deleted, and Google will update the ipAddress and peerIpAddress when the PARTNER InterconnectAttachment is provisioned. This type of BGP peer cannot be created or deleted, but can be modified for all fields except for name, ipAddress and peerIpAddress.
            MANAGED_BY_ATTACHMENT = 458_926_411

            # Default value, the BGP peer is manually created and managed by user.
            MANAGED_BY_USER = 317_294_067
          end
        end

        # @!attribute [rw] min_receive_interval
        #   @return [::Integer]
        #     The minimum interval, in milliseconds, between BFD control packets received from the peer router. The actual value is negotiated between the two routers and is equal to the greater of this value and the transmit interval of the other router. If set, this value must be between 1000 and 30000. The default is 1000.
        # @!attribute [rw] min_transmit_interval
        #   @return [::Integer]
        #     The minimum interval, in milliseconds, between BFD control packets transmitted to the peer router. The actual value is negotiated between the two routers and is equal to the greater of this value and the corresponding receive interval of the other router. If set, this value must be between 1000 and 30000. The default is 1000.
        # @!attribute [rw] multiplier
        #   @return [::Integer]
        #     The number of consecutive BFD packets that must be missed before BFD declares that a peer is unavailable. If set, the value must be a value between 5 and 16. The default is 5.
        # @!attribute [rw] session_initialization_mode
        #   @return [::String]
        #     The BFD session initialization mode for this BGP peer. If set to ACTIVE, the Cloud Router will initiate the BFD session for this BGP peer. If set to PASSIVE, the Cloud Router will wait for the peer router to initiate the BFD session for this BGP peer. If set to DISABLED, BFD is disabled for this BGP peer. The default is DISABLED.
        #     Check the SessionInitializationMode enum for the list of possible values.
        class RouterBgpPeerBfd
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The BFD session initialization mode for this BGP peer. If set to ACTIVE, the Cloud Router will initiate the BFD session for this BGP peer. If set to PASSIVE, the Cloud Router will wait for the peer router to initiate the BFD session for this BGP peer. If set to DISABLED, BFD is disabled for this BGP peer. The default is DISABLED.
          module SessionInitializationMode
            # A value indicating that the enum field is not set.
            UNDEFINED_SESSION_INITIALIZATION_MODE = 0

            ACTIVE = 314_733_318

            DISABLED = 516_696_700

            PASSIVE = 462_813_959
          end
        end

        # @!attribute [rw] range
        #   @return [::String]
        #     The custom learned route IP address range. Must be a valid CIDR-formatted prefix. If an IP address is provided without a subnet mask, it is interpreted as, for IPv4, a `/32` singular IP address range, and, for IPv6, `/128`.
        class RouterBgpPeerCustomLearnedIpRange
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] ip_range
        #   @return [::String]
        #     IP address and range of the interface. The IP range must be in the RFC3927 link-local IP address space. The value must be a CIDR-formatted string, for example: 169.254.0.1/30. NOTE: Do not truncate the address as it represents the IP address of the interface.
        # @!attribute [rw] linked_interconnect_attachment
        #   @return [::String]
        #     URI of the linked Interconnect attachment. It must be in the same region as the router. Each interface can have one linked resource, which can be a VPN tunnel, an Interconnect attachment, or a subnetwork.
        # @!attribute [rw] linked_vpn_tunnel
        #   @return [::String]
        #     URI of the linked VPN tunnel, which must be in the same region as the router. Each interface can have one linked resource, which can be a VPN tunnel, an Interconnect attachment, or a subnetwork.
        # @!attribute [rw] management_type
        #   @return [::String]
        #     [Output Only] The resource that configures and manages this interface. - MANAGED_BY_USER is the default value and can be managed directly by users. - MANAGED_BY_ATTACHMENT is an interface that is configured and managed by Cloud Interconnect, specifically, by an InterconnectAttachment of type PARTNER. Google automatically creates, updates, and deletes this type of interface when the PARTNER InterconnectAttachment is created, updated, or deleted.
        #     Check the ManagementType enum for the list of possible values.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of this interface entry. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] private_ip_address
        #   @return [::String]
        #     The regional private internal IP address that is used to establish BGP sessions to a VM instance acting as a third-party Router Appliance, such as a Next Gen Firewall, a Virtual Router, or an SD-WAN VM.
        # @!attribute [rw] redundant_interface
        #   @return [::String]
        #     Name of the interface that will be redundant with the current interface you are creating. The redundantInterface must belong to the same Cloud Router as the interface here. To establish the BGP session to a Router Appliance VM, you must create two BGP peers. The two BGP peers must be attached to two separate interfaces that are redundant with each other. The redundant_interface must be 1-63 characters long, and comply with RFC1035. Specifically, the redundant_interface must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] subnetwork
        #   @return [::String]
        #     The URI of the subnetwork resource that this interface belongs to, which must be in the same region as the Cloud Router. When you establish a BGP session to a VM instance using this interface, the VM instance must belong to the same subnetwork as the subnetwork specified here.
        class RouterInterface
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # [Output Only] The resource that configures and manages this interface. - MANAGED_BY_USER is the default value and can be managed directly by users. - MANAGED_BY_ATTACHMENT is an interface that is configured and managed by Cloud Interconnect, specifically, by an InterconnectAttachment of type PARTNER. Google automatically creates, updates, and deletes this type of interface when the PARTNER InterconnectAttachment is created, updated, or deleted.
          module ManagementType
            # A value indicating that the enum field is not set.
            UNDEFINED_MANAGEMENT_TYPE = 0

            # The interface is automatically created for PARTNER type InterconnectAttachment, Google will automatically create/update/delete this interface when the PARTNER InterconnectAttachment is created/provisioned/deleted. This type of interface cannot be manually managed by user.
            MANAGED_BY_ATTACHMENT = 458_926_411

            # Default value, the interface is manually created and managed by user.
            MANAGED_BY_USER = 317_294_067
          end
        end

        # Contains a list of Router resources.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::Router>]
        #     A list of Router resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#router for routers.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class RouterList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] key
        #   @return [::String]
        #     [Input only] Value of the key. For patch and update calls, it can be skipped to copy the value from the previous configuration. This is allowed if the key with the same name existed before the operation. Maximum length is 80 characters. Can only contain printable ASCII characters.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name used to identify the key. Must be unique within a router. Must be referenced by exactly one bgpPeer. Must comply with RFC1035.
        class RouterMd5AuthenticationKey
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a Nat resource. It enables the VMs within the specified subnetworks to access Internet without external IP addresses. It specifies a list of subnetworks (and the ranges within) that want to use NAT. Customers can also provide the external IPs that would be used for NAT. GCP would auto-allocate ephemeral IPs if no external IPs are provided.
        # @!attribute [rw] auto_network_tier
        #   @return [::String]
        #     The network tier to use when automatically reserving NAT IP addresses. Must be one of: PREMIUM, STANDARD. If not specified, then the current project-level default tier is used.
        #     Check the AutoNetworkTier enum for the list of possible values.
        # @!attribute [rw] drain_nat_ips
        #   @return [::Array<::String>]
        #     A list of URLs of the IP resources to be drained. These IPs must be valid static external IPs that have been assigned to the NAT. These IPs should be used for updating/patching a NAT only.
        # @!attribute [rw] enable_dynamic_port_allocation
        #   @return [::Boolean]
        #     Enable Dynamic Port Allocation. If not specified, it is disabled by default. If set to true, - Dynamic Port Allocation will be enabled on this NAT config. - enableEndpointIndependentMapping cannot be set to true. - If minPorts is set, minPortsPerVm must be set to a power of two greater than or equal to 32. If minPortsPerVm is not set, a minimum of 32 ports will be allocated to a VM from this NAT config.
        # @!attribute [rw] enable_endpoint_independent_mapping
        #   @return [::Boolean]
        # @!attribute [rw] endpoint_types
        #   @return [::Array<::String>]
        #     List of NAT-ted endpoint types supported by the Nat Gateway. If the list is empty, then it will be equivalent to include ENDPOINT_TYPE_VM
        #     Check the EndpointTypes enum for the list of possible values.
        # @!attribute [rw] icmp_idle_timeout_sec
        #   @return [::Integer]
        #     Timeout (in seconds) for ICMP connections. Defaults to 30s if not set.
        # @!attribute [rw] log_config
        #   @return [::Google::Cloud::Compute::V1::RouterNatLogConfig]
        #     Configure logging on this NAT.
        # @!attribute [rw] max_ports_per_vm
        #   @return [::Integer]
        #     Maximum number of ports allocated to a VM from this NAT config when Dynamic Port Allocation is enabled. If Dynamic Port Allocation is not enabled, this field has no effect. If Dynamic Port Allocation is enabled, and this field is set, it must be set to a power of two greater than minPortsPerVm, or 64 if minPortsPerVm is not set. If Dynamic Port Allocation is enabled and this field is not set, a maximum of 65536 ports will be allocated to a VM from this NAT config.
        # @!attribute [rw] min_ports_per_vm
        #   @return [::Integer]
        #     Minimum number of ports allocated to a VM from this NAT config. If not set, a default number of ports is allocated to a VM. This is rounded up to the nearest power of 2. For example, if the value of this field is 50, at least 64 ports are allocated to a VM.
        # @!attribute [rw] name
        #   @return [::String]
        #     Unique name of this Nat service. The name must be 1-63 characters long and comply with RFC1035.
        # @!attribute [rw] nat_ip_allocate_option
        #   @return [::String]
        #     Specify the NatIpAllocateOption, which can take one of the following values: - MANUAL_ONLY: Uses only Nat IP addresses provided by customers. When there are not enough specified Nat IPs, the Nat service fails for new VMs. - AUTO_ONLY: Nat IPs are allocated by Google Cloud Platform; customers can't specify any Nat IPs. When choosing AUTO_ONLY, then nat_ip should be empty.
        #     Check the NatIpAllocateOption enum for the list of possible values.
        # @!attribute [rw] nat_ips
        #   @return [::Array<::String>]
        #     A list of URLs of the IP resources used for this Nat service. These IP addresses must be valid static external IP addresses assigned to the project.
        # @!attribute [rw] rules
        #   @return [::Array<::Google::Cloud::Compute::V1::RouterNatRule>]
        #     A list of rules associated with this NAT.
        # @!attribute [rw] source_subnetwork_ip_ranges_to_nat
        #   @return [::String]
        #     Specify the Nat option, which can take one of the following values: - ALL_SUBNETWORKS_ALL_IP_RANGES: All of the IP ranges in every Subnetwork are allowed to Nat. - ALL_SUBNETWORKS_ALL_PRIMARY_IP_RANGES: All of the primary IP ranges in every Subnetwork are allowed to Nat. - LIST_OF_SUBNETWORKS: A list of Subnetworks are allowed to Nat (specified in the field subnetwork below) The default is SUBNETWORK_IP_RANGE_TO_NAT_OPTION_UNSPECIFIED. Note that if this field contains ALL_SUBNETWORKS_ALL_IP_RANGES then there should not be any other Router.Nat section in any Router for this network in this region.
        #     Check the SourceSubnetworkIpRangesToNat enum for the list of possible values.
        # @!attribute [rw] subnetworks
        #   @return [::Array<::Google::Cloud::Compute::V1::RouterNatSubnetworkToNat>]
        #     A list of Subnetwork resources whose traffic should be translated by NAT Gateway. It is used only when LIST_OF_SUBNETWORKS is selected for the SubnetworkIpRangeToNatOption above.
        # @!attribute [rw] tcp_established_idle_timeout_sec
        #   @return [::Integer]
        #     Timeout (in seconds) for TCP established connections. Defaults to 1200s if not set.
        # @!attribute [rw] tcp_time_wait_timeout_sec
        #   @return [::Integer]
        #     Timeout (in seconds) for TCP connections that are in TIME_WAIT state. Defaults to 120s if not set.
        # @!attribute [rw] tcp_transitory_idle_timeout_sec
        #   @return [::Integer]
        #     Timeout (in seconds) for TCP transitory connections. Defaults to 30s if not set.
        # @!attribute [rw] type
        #   @return [::String]
        #     Indicates whether this NAT is used for public or private IP translation. If unspecified, it defaults to PUBLIC.
        #     Check the Type enum for the list of possible values.
        # @!attribute [rw] udp_idle_timeout_sec
        #   @return [::Integer]
        #     Timeout (in seconds) for UDP connections. Defaults to 30s if not set.
        class RouterNat
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The network tier to use when automatically reserving NAT IP addresses. Must be one of: PREMIUM, STANDARD. If not specified, then the current project-level default tier is used.
          module AutoNetworkTier
            # A value indicating that the enum field is not set.
            UNDEFINED_AUTO_NETWORK_TIER = 0

            # Public internet quality with fixed bandwidth.
            FIXED_STANDARD = 310_464_328

            # High quality, Google-grade network tier, support for all networking products.
            PREMIUM = 399_530_551

            # Public internet quality, only limited support for other networking products.
            STANDARD = 484_642_493

            # (Output only) Temporary tier for FIXED_STANDARD when fixed standard tier is expired or not configured.
            STANDARD_OVERRIDES_FIXED_STANDARD = 465_847_234
          end

          module EndpointTypes
            # A value indicating that the enum field is not set.
            UNDEFINED_ENDPOINT_TYPES = 0

            # This is used for regional Application Load Balancers (internal and external) and regional proxy Network Load Balancers (internal and external) endpoints.
            ENDPOINT_TYPE_MANAGED_PROXY_LB = 439_196_930

            # This is used for Secure Web Gateway endpoints.
            ENDPOINT_TYPE_SWG = 159_344_456

            # This is the default.
            ENDPOINT_TYPE_VM = 57_095_474
          end

          # Specify the NatIpAllocateOption, which can take one of the following values: - MANUAL_ONLY: Uses only Nat IP addresses provided by customers. When there are not enough specified Nat IPs, the Nat service fails for new VMs. - AUTO_ONLY: Nat IPs are allocated by Google Cloud Platform; customers can't specify any Nat IPs. When choosing AUTO_ONLY, then nat_ip should be empty.
          module NatIpAllocateOption
            # A value indicating that the enum field is not set.
            UNDEFINED_NAT_IP_ALLOCATE_OPTION = 0

            # Nat IPs are allocated by GCP; customers can not specify any Nat IPs.
            AUTO_ONLY = 182_333_500

            # Only use Nat IPs provided by customers. When specified Nat IPs are not enough then the Nat service fails for new VMs.
            MANUAL_ONLY = 261_251_205
          end

          # Specify the Nat option, which can take one of the following values: - ALL_SUBNETWORKS_ALL_IP_RANGES: All of the IP ranges in every Subnetwork are allowed to Nat. - ALL_SUBNETWORKS_ALL_PRIMARY_IP_RANGES: All of the primary IP ranges in every Subnetwork are allowed to Nat. - LIST_OF_SUBNETWORKS: A list of Subnetworks are allowed to Nat (specified in the field subnetwork below) The default is SUBNETWORK_IP_RANGE_TO_NAT_OPTION_UNSPECIFIED. Note that if this field contains ALL_SUBNETWORKS_ALL_IP_RANGES then there should not be any other Router.Nat section in any Router for this network in this region.
          module SourceSubnetworkIpRangesToNat
            # A value indicating that the enum field is not set.
            UNDEFINED_SOURCE_SUBNETWORK_IP_RANGES_TO_NAT = 0

            # All the IP ranges in every Subnetwork are allowed to Nat.
            ALL_SUBNETWORKS_ALL_IP_RANGES = 179_964_376

            # All the primary IP ranges in every Subnetwork are allowed to Nat.
            ALL_SUBNETWORKS_ALL_PRIMARY_IP_RANGES = 185_573_819

            # A list of Subnetworks are allowed to Nat (specified in the field subnetwork below)
            LIST_OF_SUBNETWORKS = 517_542_270
          end

          # Indicates whether this NAT is used for public or private IP translation. If unspecified, it defaults to PUBLIC.
          module Type
            # A value indicating that the enum field is not set.
            UNDEFINED_TYPE = 0

            # NAT used for private IP translation.
            PRIVATE = 403_485_027

            # NAT used for public IP translation. This is the default.
            PUBLIC = 223_389_289
          end
        end

        # Configuration of logging on a NAT.
        # @!attribute [rw] enable
        #   @return [::Boolean]
        #     Indicates whether or not to export logs. This is false by default.
        # @!attribute [rw] filter
        #   @return [::String]
        #     Specify the desired filtering of logs on this NAT. If unspecified, logs are exported for all connections handled by this NAT. This option can take one of the following values: - ERRORS_ONLY: Export logs only for connection failures. - TRANSLATIONS_ONLY: Export logs only for successful connections. - ALL: Export logs for all connections, successful and unsuccessful.
        #     Check the Filter enum for the list of possible values.
        class RouterNatLogConfig
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Specify the desired filtering of logs on this NAT. If unspecified, logs are exported for all connections handled by this NAT. This option can take one of the following values: - ERRORS_ONLY: Export logs only for connection failures. - TRANSLATIONS_ONLY: Export logs only for successful connections. - ALL: Export logs for all connections, successful and unsuccessful.
          module Filter
            # A value indicating that the enum field is not set.
            UNDEFINED_FILTER = 0

            # Export logs for all (successful and unsuccessful) connections.
            ALL = 64_897

            # Export logs for connection failures only.
            ERRORS_ONLY = 307_484_672

            # Export logs for successful connections only.
            TRANSLATIONS_ONLY = 357_212_649
          end
        end

        # @!attribute [rw] action
        #   @return [::Google::Cloud::Compute::V1::RouterNatRuleAction]
        #     The action to be enforced for traffic that matches this rule.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this rule.
        # @!attribute [rw] match
        #   @return [::String]
        #     CEL expression that specifies the match condition that egress traffic from a VM is evaluated against. If it evaluates to true, the corresponding `action` is enforced. The following examples are valid match expressions for public NAT: "inIpRange(destination.ip, '1.1.0.0/16') || inIpRange(destination.ip, '2.2.0.0/16')" "destination.ip == '1.1.0.1' || destination.ip == '8.8.8.8'" The following example is a valid match expression for private NAT: "nexthop.hub == '//networkconnectivity.googleapis.com/projects/my-project/locations/global/hubs/hub-1'"
        # @!attribute [rw] rule_number
        #   @return [::Integer]
        #     An integer uniquely identifying a rule in the list. The rule number must be a positive value between 0 and 65000, and must be unique among rules within a NAT.
        class RouterNatRule
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] source_nat_active_ips
        #   @return [::Array<::String>]
        #     A list of URLs of the IP resources used for this NAT rule. These IP addresses must be valid static external IP addresses assigned to the project. This field is used for public NAT.
        # @!attribute [rw] source_nat_active_ranges
        #   @return [::Array<::String>]
        #     A list of URLs of the subnetworks used as source ranges for this NAT Rule. These subnetworks must have purpose set to PRIVATE_NAT. This field is used for private NAT.
        # @!attribute [rw] source_nat_drain_ips
        #   @return [::Array<::String>]
        #     A list of URLs of the IP resources to be drained. These IPs must be valid static external IPs that have been assigned to the NAT. These IPs should be used for updating/patching a NAT rule only. This field is used for public NAT.
        # @!attribute [rw] source_nat_drain_ranges
        #   @return [::Array<::String>]
        #     A list of URLs of subnetworks representing source ranges to be drained. This is only supported on patch/update, and these subnetworks must have previously been used as active ranges in this NAT Rule. This field is used for private NAT.
        class RouterNatRuleAction
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Defines the IP ranges that want to use NAT for a subnetwork.
        # @!attribute [rw] name
        #   @return [::String]
        #     URL for the subnetwork resource that will use NAT.
        # @!attribute [rw] secondary_ip_range_names
        #   @return [::Array<::String>]
        #     A list of the secondary ranges of the Subnetwork that are allowed to use NAT. This can be populated only if "LIST_OF_SECONDARY_IP_RANGES" is one of the values in source_ip_ranges_to_nat.
        # @!attribute [rw] source_ip_ranges_to_nat
        #   @return [::Array<::String>]
        #     Specify the options for NAT ranges in the Subnetwork. All options of a single value are valid except NAT_IP_RANGE_OPTION_UNSPECIFIED. The only valid option with multiple values is: ["PRIMARY_IP_RANGE", "LIST_OF_SECONDARY_IP_RANGES"] Default: [ALL_IP_RANGES]
        #     Check the SourceIpRangesToNat enum for the list of possible values.
        class RouterNatSubnetworkToNat
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods


          module SourceIpRangesToNat
            # A value indicating that the enum field is not set.
            UNDEFINED_SOURCE_IP_RANGES_TO_NAT = 0

            # The primary and all the secondary ranges are allowed to Nat.
            ALL_IP_RANGES = 35_608_496

            # A list of secondary ranges are allowed to Nat.
            LIST_OF_SECONDARY_IP_RANGES = 192_289_308

            # The primary range is allowed to Nat.
            PRIMARY_IP_RANGE = 297_109_954
          end
        end

        # @!attribute [rw] best_routes
        #   @return [::Array<::Google::Cloud::Compute::V1::Route>]
        #     Best routes for this router's network.
        # @!attribute [rw] best_routes_for_router
        #   @return [::Array<::Google::Cloud::Compute::V1::Route>]
        #     Best routes learned by this router.
        # @!attribute [rw] bgp_peer_status
        #   @return [::Array<::Google::Cloud::Compute::V1::RouterStatusBgpPeerStatus>]
        # @!attribute [rw] nat_status
        #   @return [::Array<::Google::Cloud::Compute::V1::RouterStatusNatStatus>]
        # @!attribute [rw] network
        #   @return [::String]
        #     URI of the network to which this router belongs.
        class RouterStatus
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] advertised_routes
        #   @return [::Array<::Google::Cloud::Compute::V1::Route>]
        #     Routes that were advertised to the remote BGP peer
        # @!attribute [rw] bfd_status
        #   @return [::Google::Cloud::Compute::V1::BfdStatus]
        # @!attribute [rw] enable_ipv6
        #   @return [::Boolean]
        #     Enable IPv6 traffic over BGP Peer. If not specified, it is disabled by default.
        # @!attribute [rw] ip_address
        #   @return [::String]
        #     IP address of the local BGP interface.
        # @!attribute [rw] ipv6_nexthop_address
        #   @return [::String]
        #     IPv6 address of the local BGP interface.
        # @!attribute [rw] linked_vpn_tunnel
        #   @return [::String]
        #     URL of the VPN tunnel that this BGP peer controls.
        # @!attribute [rw] md5_auth_enabled
        #   @return [::Boolean]
        #     Informs whether MD5 authentication is enabled on this BGP peer.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of this BGP peer. Unique within the Routers resource.
        # @!attribute [rw] num_learned_routes
        #   @return [::Integer]
        #     Number of routes learned from the remote BGP Peer.
        # @!attribute [rw] peer_ip_address
        #   @return [::String]
        #     IP address of the remote BGP interface.
        # @!attribute [rw] peer_ipv6_nexthop_address
        #   @return [::String]
        #     IPv6 address of the remote BGP interface.
        # @!attribute [rw] router_appliance_instance
        #   @return [::String]
        #     [Output only] URI of the VM instance that is used as third-party router appliances such as Next Gen Firewalls, Virtual Routers, or Router Appliances. The VM instance is the peer side of the BGP session.
        # @!attribute [rw] state
        #   @return [::String]
        #     The state of the BGP session. For a list of possible values for this field, see BGP session states.
        # @!attribute [rw] status
        #   @return [::String]
        #     Status of the BGP peer: \\{UP, DOWN}
        #     Check the Status enum for the list of possible values.
        # @!attribute [rw] status_reason
        #   @return [::String]
        #     Indicates why particular status was returned.
        #     Check the StatusReason enum for the list of possible values.
        # @!attribute [rw] uptime
        #   @return [::String]
        #     Time this session has been up. Format: 14 years, 51 weeks, 6 days, 23 hours, 59 minutes, 59 seconds
        # @!attribute [rw] uptime_seconds
        #   @return [::String]
        #     Time this session has been up, in seconds. Format: 145
        class RouterStatusBgpPeerStatus
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Status of the BGP peer: \\{UP, DOWN}
          module Status
            # A value indicating that the enum field is not set.
            UNDEFINED_STATUS = 0

            DOWN = 2_104_482

            UNKNOWN = 433_141_802

            UP = 2715
          end

          # Indicates why particular status was returned.
          module StatusReason
            # A value indicating that the enum field is not set.
            UNDEFINED_STATUS_REASON = 0

            # Indicates internal problems with configuration of MD5 authentication. This particular reason can only be returned when md5AuthEnabled is true and status is DOWN.
            MD5_AUTH_INTERNAL_PROBLEM = 140_462_259

            STATUS_REASON_UNSPECIFIED = 394_331_913
          end
        end

        # Status of a NAT contained in this router.
        # @!attribute [rw] auto_allocated_nat_ips
        #   @return [::Array<::String>]
        #     A list of IPs auto-allocated for NAT. Example: ["1.1.1.1", "129.2.16.89"]
        # @!attribute [rw] drain_auto_allocated_nat_ips
        #   @return [::Array<::String>]
        #     A list of IPs auto-allocated for NAT that are in drain mode. Example: ["1.1.1.1", "179.12.26.133"].
        # @!attribute [rw] drain_user_allocated_nat_ips
        #   @return [::Array<::String>]
        #     A list of IPs user-allocated for NAT that are in drain mode. Example: ["1.1.1.1", "179.12.26.133"].
        # @!attribute [rw] min_extra_nat_ips_needed
        #   @return [::Integer]
        #     The number of extra IPs to allocate. This will be greater than 0 only if user-specified IPs are NOT enough to allow all configured VMs to use NAT. This value is meaningful only when auto-allocation of NAT IPs is *not* used.
        # @!attribute [rw] name
        #   @return [::String]
        #     Unique name of this NAT.
        # @!attribute [rw] num_vm_endpoints_with_nat_mappings
        #   @return [::Integer]
        #     Number of VM endpoints (i.e., Nics) that can use NAT.
        # @!attribute [rw] rule_status
        #   @return [::Array<::Google::Cloud::Compute::V1::RouterStatusNatStatusNatRuleStatus>]
        #     Status of rules in this NAT.
        # @!attribute [rw] user_allocated_nat_ip_resources
        #   @return [::Array<::String>]
        #     A list of fully qualified URLs of reserved IP address resources.
        # @!attribute [rw] user_allocated_nat_ips
        #   @return [::Array<::String>]
        #     A list of IPs user-allocated for NAT. They will be raw IP strings like "179.12.26.133".
        class RouterStatusNatStatus
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Status of a NAT Rule contained in this NAT.
        # @!attribute [rw] active_nat_ips
        #   @return [::Array<::String>]
        #     A list of active IPs for NAT. Example: ["1.1.1.1", "179.12.26.133"].
        # @!attribute [rw] drain_nat_ips
        #   @return [::Array<::String>]
        #     A list of IPs for NAT that are in drain mode. Example: ["1.1.1.1", "179.12.26.133"].
        # @!attribute [rw] min_extra_ips_needed
        #   @return [::Integer]
        #     The number of extra IPs to allocate. This will be greater than 0 only if the existing IPs in this NAT Rule are NOT enough to allow all configured VMs to use NAT.
        # @!attribute [rw] num_vm_endpoints_with_nat_mappings
        #   @return [::Integer]
        #     Number of VM endpoints (i.e., NICs) that have NAT Mappings from this NAT Rule.
        # @!attribute [rw] rule_number
        #   @return [::Integer]
        #     Rule number of the rule.
        class RouterStatusNatStatusNatRuleStatus
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] kind
        #   @return [::String]
        #     Type of resource.
        # @!attribute [rw] result
        #   @return [::Google::Cloud::Compute::V1::RouterStatus]
        class RouterStatusResponse
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] resource
        #   @return [::Google::Cloud::Compute::V1::Router]
        #     Preview of given router.
        class RoutersPreviewResponse
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] routers
        #   @return [::Array<::Google::Cloud::Compute::V1::Router>]
        #     A list of routers contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     Informational warning which replaces the list of routers when the list is empty.
        class RoutersScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # This is deprecated and has no effect. Do not use.
        # @!attribute [rw] action
        #   @return [::String]
        #     This is deprecated and has no effect. Do not use.
        #     Check the Action enum for the list of possible values.
        # @!attribute [rw] conditions
        #   @return [::Array<::Google::Cloud::Compute::V1::Condition>]
        #     This is deprecated and has no effect. Do not use.
        # @!attribute [rw] description
        #   @return [::String]
        #     This is deprecated and has no effect. Do not use.
        # @!attribute [rw] ins
        #   @return [::Array<::String>]
        #     This is deprecated and has no effect. Do not use.
        # @!attribute [rw] log_configs
        #   @return [::Array<::Google::Cloud::Compute::V1::LogConfig>]
        #     This is deprecated and has no effect. Do not use.
        # @!attribute [rw] not_ins
        #   @return [::Array<::String>]
        #     This is deprecated and has no effect. Do not use.
        # @!attribute [rw] permissions
        #   @return [::Array<::String>]
        #     This is deprecated and has no effect. Do not use.
        class Rule
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # This is deprecated and has no effect. Do not use.
          module Action
            # A value indicating that the enum field is not set.
            UNDEFINED_ACTION = 0

            # This is deprecated and has no effect. Do not use.
            ALLOW = 62_368_553

            # This is deprecated and has no effect. Do not use.
            ALLOW_WITH_LOG = 76_034_177

            # This is deprecated and has no effect. Do not use.
            DENY = 2_094_604

            # This is deprecated and has no effect. Do not use.
            DENY_WITH_LOG = 351_433_982

            # This is deprecated and has no effect. Do not use.
            LOG = 75_556

            # This is deprecated and has no effect. Do not use.
            NO_ACTION = 260_643_444
          end
        end

        # @!attribute [rw] port
        #   @return [::Integer]
        #     The TCP port number to which the health check prober sends packets. The default value is 443. Valid values are 1 through 65535.
        # @!attribute [rw] port_name
        #   @return [::String]
        #     Not supported.
        # @!attribute [rw] port_specification
        #   @return [::String]
        #     Specifies how a port is selected for health checking. Can be one of the following values: USE_FIXED_PORT: Specifies a port number explicitly using the port field in the health check. Supported by backend services for pass-through load balancers and backend services for proxy load balancers. Not supported by target pools. The health check supports all backends supported by the backend service provided the backend can be health checked. For example, GCE_VM_IP network endpoint groups, GCE_VM_IP_PORT network endpoint groups, and instance group backends. USE_NAMED_PORT: Not supported. USE_SERVING_PORT: Provides an indirect method of specifying the health check port by referring to the backend service. Only supported by backend services for proxy load balancers. Not supported by target pools. Not supported by backend services for pass-through load balancers. Supports all backends that can be health checked; for example, GCE_VM_IP_PORT network endpoint groups and instance group backends. For GCE_VM_IP_PORT network endpoint group backends, the health check uses the port number specified for each endpoint in the network endpoint group. For instance group backends, the health check uses the port number determined by looking up the backend service's named port in the instance group's list of named ports.
        #     Check the PortSpecification enum for the list of possible values.
        # @!attribute [rw] proxy_header
        #   @return [::String]
        #     Specifies the type of proxy header to append before sending data to the backend, either NONE or PROXY_V1. The default is NONE.
        #     Check the ProxyHeader enum for the list of possible values.
        # @!attribute [rw] request
        #   @return [::String]
        #     Instructs the health check prober to send this exact ASCII string, up to 1024 bytes in length, after establishing the TCP connection and SSL handshake.
        # @!attribute [rw] response
        #   @return [::String]
        #     Creates a content-based SSL health check. In addition to establishing a TCP connection and the TLS handshake, you can configure the health check to pass only when the backend sends this exact response ASCII string, up to 1024 bytes in length. For details, see: https://cloud.google.com/load-balancing/docs/health-check-concepts#criteria-protocol-ssl-tcp
        class SSLHealthCheck
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Specifies how a port is selected for health checking. Can be one of the following values: USE_FIXED_PORT: Specifies a port number explicitly using the port field in the health check. Supported by backend services for pass-through load balancers and backend services for proxy load balancers. Not supported by target pools. The health check supports all backends supported by the backend service provided the backend can be health checked. For example, GCE_VM_IP network endpoint groups, GCE_VM_IP_PORT network endpoint groups, and instance group backends. USE_NAMED_PORT: Not supported. USE_SERVING_PORT: Provides an indirect method of specifying the health check port by referring to the backend service. Only supported by backend services for proxy load balancers. Not supported by target pools. Not supported by backend services for pass-through load balancers. Supports all backends that can be health checked; for example, GCE_VM_IP_PORT network endpoint groups and instance group backends. For GCE_VM_IP_PORT network endpoint group backends, the health check uses the port number specified for each endpoint in the network endpoint group. For instance group backends, the health check uses the port number determined by looking up the backend service's named port in the instance group's list of named ports.
          module PortSpecification
            # A value indicating that the enum field is not set.
            UNDEFINED_PORT_SPECIFICATION = 0

            # The port number in the health check's port is used for health checking. Applies to network endpoint group and instance group backends.
            USE_FIXED_PORT = 190_235_748

            # Not supported.
            USE_NAMED_PORT = 349_300_671

            # For network endpoint group backends, the health check uses the port number specified on each endpoint in the network endpoint group. For instance group backends, the health check uses the port number specified for the backend service's named port defined in the instance group's named ports.
            USE_SERVING_PORT = 362_637_516
          end

          # Specifies the type of proxy header to append before sending data to the backend, either NONE or PROXY_V1. The default is NONE.
          module ProxyHeader
            # A value indicating that the enum field is not set.
            UNDEFINED_PROXY_HEADER = 0

            NONE = 2_402_104

            PROXY_V1 = 334_352_940
          end
        end

        # DEPRECATED: Please use compute#savedDisk instead. An instance-attached disk resource.
        # @!attribute [rw] auto_delete
        #   @return [::Boolean]
        #     Specifies whether the disk will be auto-deleted when the instance is deleted (but not when the disk is detached from the instance).
        # @!attribute [rw] boot
        #   @return [::Boolean]
        #     Indicates that this is a boot disk. The virtual machine will use the first partition of the disk for its root filesystem.
        # @!attribute [rw] device_name
        #   @return [::String]
        #     Specifies the name of the disk attached to the source instance.
        # @!attribute [rw] disk_encryption_key
        #   @return [::Google::Cloud::Compute::V1::CustomerEncryptionKey]
        #     The encryption key for the disk.
        # @!attribute [rw] disk_size_gb
        #   @return [::Integer]
        #     The size of the disk in base-2 GB.
        # @!attribute [rw] disk_type
        #   @return [::String]
        #     [Output Only] URL of the disk type resource. For example: projects/project /zones/zone/diskTypes/pd-standard or pd-ssd
        # @!attribute [rw] guest_os_features
        #   @return [::Array<::Google::Cloud::Compute::V1::GuestOsFeature>]
        #     A list of features to enable on the guest operating system. Applicable only for bootable images. Read Enabling guest operating system features to see a list of available options.
        # @!attribute [rw] index
        #   @return [::Integer]
        #     Specifies zero-based index of the disk that is attached to the source instance.
        # @!attribute [rw] interface
        #   @return [::String]
        #     Specifies the disk interface to use for attaching this disk, which is either SCSI or NVME.
        #     Check the Interface enum for the list of possible values.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#attachedDisk for attached disks.
        # @!attribute [rw] licenses
        #   @return [::Array<::String>]
        #     [Output Only] Any valid publicly visible licenses.
        # @!attribute [rw] mode
        #   @return [::String]
        #     The mode in which this disk is attached to the source instance, either READ_WRITE or READ_ONLY.
        #     Check the Mode enum for the list of possible values.
        # @!attribute [rw] source
        #   @return [::String]
        #     Specifies a URL of the disk attached to the source instance.
        # @!attribute [rw] storage_bytes
        #   @return [::Integer]
        #     [Output Only] A size of the storage used by the disk's snapshot by this machine image.
        # @!attribute [rw] storage_bytes_status
        #   @return [::String]
        #     [Output Only] An indicator whether storageBytes is in a stable state or it is being adjusted as a result of shared storage reallocation. This status can either be UPDATING, meaning the size of the snapshot is being updated, or UP_TO_DATE, meaning the size of the snapshot is up-to-date.
        #     Check the StorageBytesStatus enum for the list of possible values.
        # @!attribute [rw] type
        #   @return [::String]
        #     Specifies the type of the attached disk, either SCRATCH or PERSISTENT.
        #     Check the Type enum for the list of possible values.
        class SavedAttachedDisk
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Specifies the disk interface to use for attaching this disk, which is either SCSI or NVME.
          module Interface
            # A value indicating that the enum field is not set.
            UNDEFINED_INTERFACE = 0

            NVME = 2_408_800

            SCSI = 2_539_686
          end

          # The mode in which this disk is attached to the source instance, either READ_WRITE or READ_ONLY.
          module Mode
            # A value indicating that the enum field is not set.
            UNDEFINED_MODE = 0

            # Attaches this disk in read-only mode. Multiple virtual machines can use a disk in read-only mode at a time.
            READ_ONLY = 91_950_261

            # *[Default]* Attaches this disk in read-write mode. Only one virtual machine at a time can be attached to a disk in read-write mode.
            READ_WRITE = 173_607_894
          end

          # [Output Only] An indicator whether storageBytes is in a stable state or it is being adjusted as a result of shared storage reallocation. This status can either be UPDATING, meaning the size of the snapshot is being updated, or UP_TO_DATE, meaning the size of the snapshot is up-to-date.
          module StorageBytesStatus
            # A value indicating that the enum field is not set.
            UNDEFINED_STORAGE_BYTES_STATUS = 0

            UPDATING = 494_614_342

            UP_TO_DATE = 101_306_702
          end

          # Specifies the type of the attached disk, either SCRATCH or PERSISTENT.
          module Type
            # A value indicating that the enum field is not set.
            UNDEFINED_TYPE = 0

            PERSISTENT = 460_683_927

            SCRATCH = 496_778_970
          end
        end

        # An instance-attached disk resource.
        # @!attribute [rw] architecture
        #   @return [::String]
        #     [Output Only] The architecture of the attached disk.
        #     Check the Architecture enum for the list of possible values.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#savedDisk for attached disks.
        # @!attribute [rw] source_disk
        #   @return [::String]
        #     Specifies a URL of the disk attached to the source instance.
        # @!attribute [rw] storage_bytes
        #   @return [::Integer]
        #     [Output Only] Size of the individual disk snapshot used by this machine image.
        # @!attribute [rw] storage_bytes_status
        #   @return [::String]
        #     [Output Only] An indicator whether storageBytes is in a stable state or it is being adjusted as a result of shared storage reallocation. This status can either be UPDATING, meaning the size of the snapshot is being updated, or UP_TO_DATE, meaning the size of the snapshot is up-to-date.
        #     Check the StorageBytesStatus enum for the list of possible values.
        class SavedDisk
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # [Output Only] The architecture of the attached disk.
          module Architecture
            # A value indicating that the enum field is not set.
            UNDEFINED_ARCHITECTURE = 0

            # Default value indicating Architecture is not set.
            ARCHITECTURE_UNSPECIFIED = 394_750_507

            # Machines with architecture ARM64
            ARM64 = 62_547_450

            # Machines with architecture X86_64
            X86_64 = 425_300_551
          end

          # [Output Only] An indicator whether storageBytes is in a stable state or it is being adjusted as a result of shared storage reallocation. This status can either be UPDATING, meaning the size of the snapshot is being updated, or UP_TO_DATE, meaning the size of the snapshot is up-to-date.
          module StorageBytesStatus
            # A value indicating that the enum field is not set.
            UNDEFINED_STORAGE_BYTES_STATUS = 0

            UPDATING = 494_614_342

            UP_TO_DATE = 101_306_702
          end
        end

        # @!attribute [rw] last_start_time
        #   @return [::String]
        #     [Output Only] The last time the scaling schedule became active. Note: this is a timestamp when a schedule actually became active, not when it was planned to do so. The timestamp is in RFC3339 text format.
        # @!attribute [rw] next_start_time
        #   @return [::String]
        #     [Output Only] The next time the scaling schedule is to become active. Note: this is a timestamp when a schedule is planned to run, but the actual time might be slightly different. The timestamp is in RFC3339 text format.
        # @!attribute [rw] state
        #   @return [::String]
        #     [Output Only] The current state of a scaling schedule.
        #     Check the State enum for the list of possible values.
        class ScalingScheduleStatus
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # [Output Only] The current state of a scaling schedule.
          module State
            # A value indicating that the enum field is not set.
            UNDEFINED_STATE = 0

            # The current autoscaling recommendation is influenced by this scaling schedule.
            ACTIVE = 314_733_318

            # This scaling schedule has been disabled by the user.
            DISABLED = 516_696_700

            # This scaling schedule will never become active again.
            OBSOLETE = 66_532_761

            # The current autoscaling recommendation is not influenced by this scaling schedule.
            READY = 77_848_963
          end
        end

        # Sets the scheduling options for an Instance.
        # @!attribute [rw] automatic_restart
        #   @return [::Boolean]
        #     Specifies whether the instance should be automatically restarted if it is terminated by Compute Engine (not terminated by a user). You can only set the automatic restart option for standard instances. Preemptible instances cannot be automatically restarted. By default, this is set to true so an instance is automatically restarted if it is terminated by Compute Engine.
        # @!attribute [rw] instance_termination_action
        #   @return [::String]
        #     Specifies the termination action for the instance.
        #     Check the InstanceTerminationAction enum for the list of possible values.
        # @!attribute [rw] local_ssd_recovery_timeout
        #   @return [::Google::Cloud::Compute::V1::Duration]
        #     Specifies the maximum amount of time a Local Ssd Vm should wait while recovery of the Local Ssd state is attempted. Its value should be in between 0 and 168 hours with hour granularity and the default value being 1 hour.
        # @!attribute [rw] location_hint
        #   @return [::String]
        #     An opaque location hint used to place the instance close to other resources. This field is for use by internal tools that use the public API.
        # @!attribute [rw] min_node_cpus
        #   @return [::Integer]
        #     The minimum number of virtual CPUs this instance will consume when running on a sole-tenant node.
        # @!attribute [rw] node_affinities
        #   @return [::Array<::Google::Cloud::Compute::V1::SchedulingNodeAffinity>]
        #     A set of node affinity and anti-affinity configurations. Refer to Configuring node affinity for more information. Overrides reservationAffinity.
        # @!attribute [rw] on_host_maintenance
        #   @return [::String]
        #     Defines the maintenance behavior for this instance. For standard instances, the default behavior is MIGRATE. For preemptible instances, the default and only possible behavior is TERMINATE. For more information, see Set VM host maintenance policy.
        #     Check the OnHostMaintenance enum for the list of possible values.
        # @!attribute [rw] preemptible
        #   @return [::Boolean]
        #     Defines whether the instance is preemptible. This can only be set during instance creation or while the instance is stopped and therefore, in a `TERMINATED` state. See Instance Life Cycle for more information on the possible instance states.
        # @!attribute [rw] provisioning_model
        #   @return [::String]
        #     Specifies the provisioning model of the instance.
        #     Check the ProvisioningModel enum for the list of possible values.
        class Scheduling
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Specifies the termination action for the instance.
          module InstanceTerminationAction
            # A value indicating that the enum field is not set.
            UNDEFINED_INSTANCE_TERMINATION_ACTION = 0

            # Delete the VM.
            DELETE = 402_225_579

            # Default value. This value is unused.
            INSTANCE_TERMINATION_ACTION_UNSPECIFIED = 92_954_803

            # Stop the VM without storing in-memory content. default action.
            STOP = 2_555_906
          end

          # Defines the maintenance behavior for this instance. For standard instances, the default behavior is MIGRATE. For preemptible instances, the default and only possible behavior is TERMINATE. For more information, see Set VM host maintenance policy.
          module OnHostMaintenance
            # A value indicating that the enum field is not set.
            UNDEFINED_ON_HOST_MAINTENANCE = 0

            # *[Default]* Allows Compute Engine to automatically migrate instances out of the way of maintenance events.
            MIGRATE = 165_699_979

            # Tells Compute Engine to terminate and (optionally) restart the instance away from the maintenance activity. If you would like your instance to be restarted, set the automaticRestart flag to true. Your instance may be restarted more than once, and it may be restarted outside the window of maintenance events.
            TERMINATE = 527_617_601
          end

          # Specifies the provisioning model of the instance.
          module ProvisioningModel
            # A value indicating that the enum field is not set.
            UNDEFINED_PROVISIONING_MODEL = 0

            # Heavily discounted, no guaranteed runtime.
            SPOT = 2_552_066

            # Standard provisioning with user controlled runtime, no discounts.
            STANDARD = 484_642_493
          end
        end

        # Node Affinity: the configuration of desired nodes onto which this Instance could be scheduled.
        # @!attribute [rw] key
        #   @return [::String]
        #     Corresponds to the label key of Node resource.
        # @!attribute [rw] operator
        #   @return [::String]
        #     Defines the operation of node selection. Valid operators are IN for affinity and NOT_IN for anti-affinity.
        #     Check the Operator enum for the list of possible values.
        # @!attribute [rw] values
        #   @return [::Array<::String>]
        #     Corresponds to the label values of Node resource.
        class SchedulingNodeAffinity
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Defines the operation of node selection. Valid operators are IN for affinity and NOT_IN for anti-affinity.
          module Operator
            # A value indicating that the enum field is not set.
            UNDEFINED_OPERATOR = 0

            # Requires Compute Engine to seek for matched nodes.
            IN = 2341

            # Requires Compute Engine to avoid certain nodes.
            NOT_IN = 161_144_369

            OPERATOR_UNSPECIFIED = 128_892_924
          end
        end

        # @!attribute [rw] disk_gb
        #   @return [::Integer]
        #     Size of the scratch disk, defined in GB.
        class ScratchDisks
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # An instance's screenshot.
        # @!attribute [rw] contents
        #   @return [::String]
        #     [Output Only] The Base64-encoded screenshot data.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#screenshot for the screenshots.
        class Screenshot
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] etag
        #   @return [::String]
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::SecurityPoliciesScopedList}]
        #     A list of SecurityPoliciesScopedList resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#securityPolicyAggregatedList for lists of Security Policies.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class SecurityPoliciesAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::SecurityPoliciesScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # @!attribute [rw] preconfigured_expression_sets
        #   @return [::Google::Cloud::Compute::V1::SecurityPoliciesWafConfig]
        class SecurityPoliciesListPreconfiguredExpressionSetsResponse
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] security_policies
        #   @return [::Array<::Google::Cloud::Compute::V1::SecurityPolicy>]
        #     A list of SecurityPolicies contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     Informational warning which replaces the list of security policies when the list is empty.
        class SecurityPoliciesScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] waf_rules
        #   @return [::Google::Cloud::Compute::V1::PreconfiguredWafSet]
        class SecurityPoliciesWafConfig
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a Google Cloud Armor security policy resource. Only external backend services that use load balancers can reference a security policy. For more information, see Google Cloud Armor security policy overview.
        # @!attribute [rw] adaptive_protection_config
        #   @return [::Google::Cloud::Compute::V1::SecurityPolicyAdaptiveProtectionConfig]
        # @!attribute [rw] advanced_options_config
        #   @return [::Google::Cloud::Compute::V1::SecurityPolicyAdvancedOptionsConfig]
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] ddos_protection_config
        #   @return [::Google::Cloud::Compute::V1::SecurityPolicyDdosProtectionConfig]
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] fingerprint
        #   @return [::String]
        #     Specifies a fingerprint for this resource, which is essentially a hash of the metadata's contents and used for optimistic locking. The fingerprint is initially generated by Compute Engine and changes after every request to modify or update metadata. You must always provide an up-to-date fingerprint hash in order to update or change metadata, otherwise the request will fail with error 412 conditionNotMet. To see the latest fingerprint, make get() request to the security policy.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output only] Type of the resource. Always compute#securityPolicyfor security policies
        # @!attribute [rw] label_fingerprint
        #   @return [::String]
        #     A fingerprint for the labels being applied to this security policy, which is essentially a hash of the labels set used for optimistic locking. The fingerprint is initially generated by Compute Engine and changes after every request to modify or update labels. You must always provide an up-to-date fingerprint hash in order to update or change labels. To see the latest fingerprint, make get() request to the security policy.
        # @!attribute [rw] labels
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     Labels for this resource. These can only be added or modified by the setLabels method. Each label key/value pair must comply with RFC1035. Label values may be empty.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] recaptcha_options_config
        #   @return [::Google::Cloud::Compute::V1::SecurityPolicyRecaptchaOptionsConfig]
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] URL of the region where the regional security policy resides. This field is not applicable to global security policies.
        # @!attribute [rw] rules
        #   @return [::Array<::Google::Cloud::Compute::V1::SecurityPolicyRule>]
        #     A list of rules that belong to this policy. There must always be a default rule which is a rule with priority 2147483647 and match all condition (for the match condition this means match "*" for srcIpRanges and for the networkMatch condition every field must be either match "*" or not set). If no rules are provided when creating a security policy, a default rule with action "allow" will be added.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] type
        #   @return [::String]
        #     The type indicates the intended use of the security policy. - CLOUD_ARMOR: Cloud Armor backend security policies can be configured to filter incoming HTTP requests targeting backend services. They filter requests before they hit the origin servers. - CLOUD_ARMOR_EDGE: Cloud Armor edge security policies can be configured to filter incoming HTTP requests targeting backend services (including Cloud CDN-enabled) as well as backend buckets (Cloud Storage). They filter requests before the request is served from Google's cache. - CLOUD_ARMOR_INTERNAL_SERVICE: Cloud Armor internal service policies can be configured to filter HTTP requests targeting services managed by Traffic Director in a service mesh. They filter requests before the request is served from the application. - CLOUD_ARMOR_NETWORK: Cloud Armor network policies can be configured to filter packets targeting network load balancing resources such as backend services, target pools, target instances, and instances with external IPs. They filter requests before the request is served from the application. This field can be set only at resource creation time.
        #     Check the Type enum for the list of possible values.
        # @!attribute [rw] user_defined_fields
        #   @return [::Array<::Google::Cloud::Compute::V1::SecurityPolicyUserDefinedField>]
        #     Definitions of user-defined fields for CLOUD_ARMOR_NETWORK policies. A user-defined field consists of up to 4 bytes extracted from a fixed offset in the packet, relative to the IPv4, IPv6, TCP, or UDP header, with an optional mask to select certain bits. Rules may then specify matching values for these fields. Example: userDefinedFields: - name: "ipv4_fragment_offset" base: IPV4 offset: 6 size: 2 mask: "0x1fff"
        class SecurityPolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class LabelsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # The type indicates the intended use of the security policy. - CLOUD_ARMOR: Cloud Armor backend security policies can be configured to filter incoming HTTP requests targeting backend services. They filter requests before they hit the origin servers. - CLOUD_ARMOR_EDGE: Cloud Armor edge security policies can be configured to filter incoming HTTP requests targeting backend services (including Cloud CDN-enabled) as well as backend buckets (Cloud Storage). They filter requests before the request is served from Google's cache. - CLOUD_ARMOR_INTERNAL_SERVICE: Cloud Armor internal service policies can be configured to filter HTTP requests targeting services managed by Traffic Director in a service mesh. They filter requests before the request is served from the application. - CLOUD_ARMOR_NETWORK: Cloud Armor network policies can be configured to filter packets targeting network load balancing resources such as backend services, target pools, target instances, and instances with external IPs. They filter requests before the request is served from the application. This field can be set only at resource creation time.
          module Type
            # A value indicating that the enum field is not set.
            UNDEFINED_TYPE = 0

            CLOUD_ARMOR = 260_640_373

            CLOUD_ARMOR_EDGE = 250_728_775

            CLOUD_ARMOR_NETWORK = 488_527_428
          end
        end

        # Configuration options for Cloud Armor Adaptive Protection (CAAP).
        # @!attribute [rw] layer7_ddos_defense_config
        #   @return [::Google::Cloud::Compute::V1::SecurityPolicyAdaptiveProtectionConfigLayer7DdosDefenseConfig]
        #     If set to true, enables Cloud Armor Machine Learning.
        class SecurityPolicyAdaptiveProtectionConfig
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Configuration options for L7 DDoS detection. This field is only supported in Global Security Policies of type CLOUD_ARMOR.
        # @!attribute [rw] enable
        #   @return [::Boolean]
        #     If set to true, enables CAAP for L7 DDoS detection. This field is only supported in Global Security Policies of type CLOUD_ARMOR.
        # @!attribute [rw] rule_visibility
        #   @return [::String]
        #     Rule visibility can be one of the following: STANDARD - opaque rules. (default) PREMIUM - transparent rules. This field is only supported in Global Security Policies of type CLOUD_ARMOR.
        #     Check the RuleVisibility enum for the list of possible values.
        # @!attribute [rw] threshold_configs
        #   @return [::Array<::Google::Cloud::Compute::V1::SecurityPolicyAdaptiveProtectionConfigLayer7DdosDefenseConfigThresholdConfig>]
        #     Configuration options for layer7 adaptive protection for various customizable thresholds.
        class SecurityPolicyAdaptiveProtectionConfigLayer7DdosDefenseConfig
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Rule visibility can be one of the following: STANDARD - opaque rules. (default) PREMIUM - transparent rules. This field is only supported in Global Security Policies of type CLOUD_ARMOR.
          module RuleVisibility
            # A value indicating that the enum field is not set.
            UNDEFINED_RULE_VISIBILITY = 0

            PREMIUM = 399_530_551

            STANDARD = 484_642_493
          end
        end

        # @!attribute [rw] auto_deploy_confidence_threshold
        #   @return [::Float]
        # @!attribute [rw] auto_deploy_expiration_sec
        #   @return [::Integer]
        # @!attribute [rw] auto_deploy_impacted_baseline_threshold
        #   @return [::Float]
        # @!attribute [rw] auto_deploy_load_threshold
        #   @return [::Float]
        # @!attribute [rw] name
        #   @return [::String]
        #     The name must be 1-63 characters long, and comply with RFC1035. The name must be unique within the security policy.
        class SecurityPolicyAdaptiveProtectionConfigLayer7DdosDefenseConfigThresholdConfig
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] json_custom_config
        #   @return [::Google::Cloud::Compute::V1::SecurityPolicyAdvancedOptionsConfigJsonCustomConfig]
        #     Custom configuration to apply the JSON parsing. Only applicable when json_parsing is set to STANDARD.
        # @!attribute [rw] json_parsing
        #   @return [::String]
        #     Check the JsonParsing enum for the list of possible values.
        # @!attribute [rw] log_level
        #   @return [::String]
        #     Check the LogLevel enum for the list of possible values.
        # @!attribute [rw] user_ip_request_headers
        #   @return [::Array<::String>]
        #     An optional list of case-insensitive request header names to use for resolving the callers client IP address.
        class SecurityPolicyAdvancedOptionsConfig
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods


          module JsonParsing
            # A value indicating that the enum field is not set.
            UNDEFINED_JSON_PARSING = 0

            DISABLED = 516_696_700

            STANDARD = 484_642_493

            STANDARD_WITH_GRAPHQL = 106_979_218
          end

          module LogLevel
            # A value indicating that the enum field is not set.
            UNDEFINED_LOG_LEVEL = 0

            NORMAL = 161_067_239

            VERBOSE = 532_219_234
          end
        end

        # @!attribute [rw] content_types
        #   @return [::Array<::String>]
        #     A list of custom Content-Type header values to apply the JSON parsing. As per RFC 1341, a Content-Type header value has the following format: Content-Type := type "/" subtype *[";" parameter] When configuring a custom Content-Type header value, only the type/subtype needs to be specified, and the parameters should be excluded.
        class SecurityPolicyAdvancedOptionsConfigJsonCustomConfig
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] ddos_protection
        #   @return [::String]
        #     Check the DdosProtection enum for the list of possible values.
        class SecurityPolicyDdosProtectionConfig
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods


          module DdosProtection
            # A value indicating that the enum field is not set.
            UNDEFINED_DDOS_PROTECTION = 0

            ADVANCED = 63_789_090

            STANDARD = 484_642_493
          end
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::SecurityPolicy>]
        #     A list of SecurityPolicy resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#securityPolicyList for listsof securityPolicies
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class SecurityPolicyList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] redirect_site_key
        #   @return [::String]
        #     An optional field to supply a reCAPTCHA site key to be used for all the rules using the redirect action with the type of GOOGLE_RECAPTCHA under the security policy. The specified site key needs to be created from the reCAPTCHA API. The user is responsible for the validity of the specified site key. If not specified, a Google-managed site key is used. This field is only supported in Global Security Policies of type CLOUD_ARMOR.
        class SecurityPolicyRecaptchaOptionsConfig
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] security_policy
        #   @return [::String]
        class SecurityPolicyReference
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a rule that describes one or more match conditions along with the action to be taken when traffic matches this condition (allow or deny).
        # @!attribute [rw] action
        #   @return [::String]
        #     The Action to perform when the rule is matched. The following are the valid actions: - allow: allow access to target. - deny(STATUS): deny access to target, returns the HTTP response code specified. Valid values for `STATUS` are 403, 404, and 502. - rate_based_ban: limit client traffic to the configured threshold and ban the client if the traffic exceeds the threshold. Configure parameters for this action in RateLimitOptions. Requires rate_limit_options to be set. - redirect: redirect to a different target. This can either be an internal reCAPTCHA redirect, or an external URL-based redirect via a 302 response. Parameters for this action can be configured via redirectOptions. This action is only supported in Global Security Policies of type CLOUD_ARMOR. - throttle: limit client traffic to the configured threshold. Configure parameters for this action in rateLimitOptions. Requires rate_limit_options to be set for this.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] header_action
        #   @return [::Google::Cloud::Compute::V1::SecurityPolicyRuleHttpHeaderAction]
        #     Optional, additional actions that are performed on headers. This field is only supported in Global Security Policies of type CLOUD_ARMOR.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output only] Type of the resource. Always compute#securityPolicyRule for security policy rules
        # @!attribute [rw] match
        #   @return [::Google::Cloud::Compute::V1::SecurityPolicyRuleMatcher]
        #     A match condition that incoming traffic is evaluated against. If it evaluates to true, the corresponding 'action' is enforced.
        # @!attribute [rw] network_match
        #   @return [::Google::Cloud::Compute::V1::SecurityPolicyRuleNetworkMatcher]
        #     A match condition that incoming packets are evaluated against for CLOUD_ARMOR_NETWORK security policies. If it matches, the corresponding 'action' is enforced. The match criteria for a rule consists of built-in match fields (like 'srcIpRanges') and potentially multiple user-defined match fields ('userDefinedFields'). Field values may be extracted directly from the packet or derived from it (e.g. 'srcRegionCodes'). Some fields may not be present in every packet (e.g. 'srcPorts'). A user-defined field is only present if the base header is found in the packet and the entire field is in bounds. Each match field may specify which values can match it, listing one or more ranges, prefixes, or exact values that are considered a match for the field. A field value must be present in order to match a specified match field. If no match values are specified for a match field, then any field value is considered to match it, and it's not required to be present. For strings specifying '*' is also equivalent to match all. For a packet to match a rule, all specified match fields must match the corresponding field values derived from the packet. Example: networkMatch: srcIpRanges: - "192.0.2.0/24" - "198.51.100.0/24" userDefinedFields: - name: "ipv4_fragment_offset" values: - "1-0x1fff" The above match condition matches packets with a source IP in 192.0.2.0/24 or 198.51.100.0/24 and a user-defined field named "ipv4_fragment_offset" with a value between 1 and 0x1fff inclusive.
        # @!attribute [rw] preconfigured_waf_config
        #   @return [::Google::Cloud::Compute::V1::SecurityPolicyRulePreconfiguredWafConfig]
        #     Preconfigured WAF configuration to be applied for the rule. If the rule does not evaluate preconfigured WAF rules, i.e., if evaluatePreconfiguredWaf() is not used, this field will have no effect.
        # @!attribute [rw] preview
        #   @return [::Boolean]
        #     If set to true, the specified action is not enforced.
        # @!attribute [rw] priority
        #   @return [::Integer]
        #     An integer indicating the priority of a rule in the list. The priority must be a positive value between 0 and 2147483647. Rules are evaluated from highest to lowest priority where 0 is the highest priority and 2147483647 is the lowest priority.
        # @!attribute [rw] rate_limit_options
        #   @return [::Google::Cloud::Compute::V1::SecurityPolicyRuleRateLimitOptions]
        #     Must be specified if the action is "rate_based_ban" or "throttle". Cannot be specified for any other actions.
        # @!attribute [rw] redirect_options
        #   @return [::Google::Cloud::Compute::V1::SecurityPolicyRuleRedirectOptions]
        #     Parameters defining the redirect action. Cannot be specified for any other actions. This field is only supported in Global Security Policies of type CLOUD_ARMOR.
        class SecurityPolicyRule
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] request_headers_to_adds
        #   @return [::Array<::Google::Cloud::Compute::V1::SecurityPolicyRuleHttpHeaderActionHttpHeaderOption>]
        #     The list of request headers to add or overwrite if they're already present.
        class SecurityPolicyRuleHttpHeaderAction
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] header_name
        #   @return [::String]
        #     The name of the header to set.
        # @!attribute [rw] header_value
        #   @return [::String]
        #     The value to set the named header to.
        class SecurityPolicyRuleHttpHeaderActionHttpHeaderOption
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a match condition that incoming traffic is evaluated against. Exactly one field must be specified.
        # @!attribute [rw] config
        #   @return [::Google::Cloud::Compute::V1::SecurityPolicyRuleMatcherConfig]
        #     The configuration options available when specifying versioned_expr. This field must be specified if versioned_expr is specified and cannot be specified if versioned_expr is not specified.
        # @!attribute [rw] expr
        #   @return [::Google::Cloud::Compute::V1::Expr]
        #     User defined CEVAL expression. A CEVAL expression is used to specify match criteria such as origin.ip, source.region_code and contents in the request header. Expressions containing `evaluateThreatIntelligence` require Cloud Armor Managed Protection Plus tier and are not supported in Edge Policies nor in Regional Policies. Expressions containing `evaluatePreconfiguredExpr('sourceiplist-*')` require Cloud Armor Managed Protection Plus tier and are only supported in Global Security Policies.
        # @!attribute [rw] versioned_expr
        #   @return [::String]
        #     Preconfigured versioned expression. If this field is specified, config must also be specified. Available preconfigured expressions along with their requirements are: SRC_IPS_V1 - must specify the corresponding src_ip_range field in config.
        #     Check the VersionedExpr enum for the list of possible values.
        class SecurityPolicyRuleMatcher
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Preconfigured versioned expression. If this field is specified, config must also be specified. Available preconfigured expressions along with their requirements are: SRC_IPS_V1 - must specify the corresponding src_ip_range field in config.
          module VersionedExpr
            # A value indicating that the enum field is not set.
            UNDEFINED_VERSIONED_EXPR = 0

            # Matches the source IP address of a request to the IP ranges supplied in config.
            SRC_IPS_V1 = 70_925_961
          end
        end

        # @!attribute [rw] src_ip_ranges
        #   @return [::Array<::String>]
        #     CIDR IP address range. Maximum number of src_ip_ranges allowed is 10.
        class SecurityPolicyRuleMatcherConfig
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a match condition that incoming network traffic is evaluated against.
        # @!attribute [rw] dest_ip_ranges
        #   @return [::Array<::String>]
        #     Destination IPv4/IPv6 addresses or CIDR prefixes, in standard text format.
        # @!attribute [rw] dest_ports
        #   @return [::Array<::String>]
        #     Destination port numbers for TCP/UDP/SCTP. Each element can be a 16-bit unsigned decimal number (e.g. "80") or range (e.g. "0-1023").
        # @!attribute [rw] ip_protocols
        #   @return [::Array<::String>]
        #     IPv4 protocol / IPv6 next header (after extension headers). Each element can be an 8-bit unsigned decimal number (e.g. "6"), range (e.g. "253-254"), or one of the following protocol names: "tcp", "udp", "icmp", "esp", "ah", "ipip", or "sctp".
        # @!attribute [rw] src_asns
        #   @return [::Array<::Integer>]
        #     BGP Autonomous System Number associated with the source IP address.
        # @!attribute [rw] src_ip_ranges
        #   @return [::Array<::String>]
        #     Source IPv4/IPv6 addresses or CIDR prefixes, in standard text format.
        # @!attribute [rw] src_ports
        #   @return [::Array<::String>]
        #     Source port numbers for TCP/UDP/SCTP. Each element can be a 16-bit unsigned decimal number (e.g. "80") or range (e.g. "0-1023").
        # @!attribute [rw] src_region_codes
        #   @return [::Array<::String>]
        #     Two-letter ISO 3166-1 alpha-2 country code associated with the source IP address.
        # @!attribute [rw] user_defined_fields
        #   @return [::Array<::Google::Cloud::Compute::V1::SecurityPolicyRuleNetworkMatcherUserDefinedFieldMatch>]
        #     User-defined fields. Each element names a defined field and lists the matching values for that field.
        class SecurityPolicyRuleNetworkMatcher
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the user-defined field, as given in the definition.
        # @!attribute [rw] values
        #   @return [::Array<::String>]
        #     Matching values of the field. Each element can be a 32-bit unsigned decimal or hexadecimal (starting with "0x") number (e.g. "64") or range (e.g. "0x400-0x7ff").
        class SecurityPolicyRuleNetworkMatcherUserDefinedFieldMatch
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] exclusions
        #   @return [::Array<::Google::Cloud::Compute::V1::SecurityPolicyRulePreconfiguredWafConfigExclusion>]
        #     A list of exclusions to apply during preconfigured WAF evaluation.
        class SecurityPolicyRulePreconfiguredWafConfig
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] request_cookies_to_exclude
        #   @return [::Array<::Google::Cloud::Compute::V1::SecurityPolicyRulePreconfiguredWafConfigExclusionFieldParams>]
        #     A list of request cookie names whose value will be excluded from inspection during preconfigured WAF evaluation.
        # @!attribute [rw] request_headers_to_exclude
        #   @return [::Array<::Google::Cloud::Compute::V1::SecurityPolicyRulePreconfiguredWafConfigExclusionFieldParams>]
        #     A list of request header names whose value will be excluded from inspection during preconfigured WAF evaluation.
        # @!attribute [rw] request_query_params_to_exclude
        #   @return [::Array<::Google::Cloud::Compute::V1::SecurityPolicyRulePreconfiguredWafConfigExclusionFieldParams>]
        #     A list of request query parameter names whose value will be excluded from inspection during preconfigured WAF evaluation. Note that the parameter can be in the query string or in the POST body.
        # @!attribute [rw] request_uris_to_exclude
        #   @return [::Array<::Google::Cloud::Compute::V1::SecurityPolicyRulePreconfiguredWafConfigExclusionFieldParams>]
        #     A list of request URIs from the request line to be excluded from inspection during preconfigured WAF evaluation. When specifying this field, the query or fragment part should be excluded.
        # @!attribute [rw] target_rule_ids
        #   @return [::Array<::String>]
        #     A list of target rule IDs under the WAF rule set to apply the preconfigured WAF exclusion. If omitted, it refers to all the rule IDs under the WAF rule set.
        # @!attribute [rw] target_rule_set
        #   @return [::String]
        #     Target WAF rule set to apply the preconfigured WAF exclusion.
        class SecurityPolicyRulePreconfiguredWafConfigExclusion
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] op
        #   @return [::String]
        #     The match operator for the field.
        #     Check the Op enum for the list of possible values.
        # @!attribute [rw] val
        #   @return [::String]
        #     The value of the field.
        class SecurityPolicyRulePreconfiguredWafConfigExclusionFieldParams
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The match operator for the field.
          module Op
            # A value indicating that the enum field is not set.
            UNDEFINED_OP = 0

            # The operator matches if the field value contains the specified value.
            CONTAINS = 215_180_831

            # The operator matches if the field value ends with the specified value.
            ENDS_WITH = 490_402_221

            # The operator matches if the field value equals the specified value.
            EQUALS = 442_201_023

            # The operator matches if the field value is any value.
            EQUALS_ANY = 337_226_060

            # The operator matches if the field value starts with the specified value.
            STARTS_WITH = 139_505_652
          end
        end

        # @!attribute [rw] ban_duration_sec
        #   @return [::Integer]
        #     Can only be specified if the action for the rule is "rate_based_ban". If specified, determines the time (in seconds) the traffic will continue to be banned by the rate limit after the rate falls below the threshold.
        # @!attribute [rw] ban_threshold
        #   @return [::Google::Cloud::Compute::V1::SecurityPolicyRuleRateLimitOptionsThreshold]
        #     Can only be specified if the action for the rule is "rate_based_ban". If specified, the key will be banned for the configured 'ban_duration_sec' when the number of requests that exceed the 'rate_limit_threshold' also exceed this 'ban_threshold'.
        # @!attribute [rw] conform_action
        #   @return [::String]
        #     Action to take for requests that are under the configured rate limit threshold. Valid option is "allow" only.
        # @!attribute [rw] enforce_on_key
        #   @return [::String]
        #     Determines the key to enforce the rate_limit_threshold on. Possible values are: - ALL: A single rate limit threshold is applied to all the requests matching this rule. This is the default value if "enforceOnKey" is not configured. - IP: The source IP address of the request is the key. Each IP has this limit enforced separately. - HTTP_HEADER: The value of the HTTP header whose name is configured under "enforceOnKeyName". The key value is truncated to the first 128 bytes of the header value. If no such header is present in the request, the key type defaults to ALL. - XFF_IP: The first IP address (i.e. the originating client IP address) specified in the list of IPs under X-Forwarded-For HTTP header. If no such header is present or the value is not a valid IP, the key defaults to the source IP address of the request i.e. key type IP. - HTTP_COOKIE: The value of the HTTP cookie whose name is configured under "enforceOnKeyName". The key value is truncated to the first 128 bytes of the cookie value. If no such cookie is present in the request, the key type defaults to ALL. - HTTP_PATH: The URL path of the HTTP request. The key value is truncated to the first 128 bytes. - SNI: Server name indication in the TLS session of the HTTPS request. The key value is truncated to the first 128 bytes. The key type defaults to ALL on a HTTP session. - REGION_CODE: The country/region from which the request originates.
        #     Check the EnforceOnKey enum for the list of possible values.
        # @!attribute [rw] enforce_on_key_configs
        #   @return [::Array<::Google::Cloud::Compute::V1::SecurityPolicyRuleRateLimitOptionsEnforceOnKeyConfig>]
        #     If specified, any combination of values of enforce_on_key_type/enforce_on_key_name is treated as the key on which ratelimit threshold/action is enforced. You can specify up to 3 enforce_on_key_configs. If enforce_on_key_configs is specified, enforce_on_key must not be specified.
        # @!attribute [rw] enforce_on_key_name
        #   @return [::String]
        #     Rate limit key name applicable only for the following key types: HTTP_HEADER -- Name of the HTTP header whose value is taken as the key value. HTTP_COOKIE -- Name of the HTTP cookie whose value is taken as the key value.
        # @!attribute [rw] exceed_action
        #   @return [::String]
        #     Action to take for requests that are above the configured rate limit threshold, to either deny with a specified HTTP response code, or redirect to a different endpoint. Valid options are `deny(STATUS)`, where valid values for `STATUS` are 403, 404, 429, and 502, and `redirect`, where the redirect parameters come from `exceedRedirectOptions` below. The `redirect` action is only supported in Global Security Policies of type CLOUD_ARMOR.
        # @!attribute [rw] exceed_redirect_options
        #   @return [::Google::Cloud::Compute::V1::SecurityPolicyRuleRedirectOptions]
        #     Parameters defining the redirect action that is used as the exceed action. Cannot be specified if the exceed action is not redirect. This field is only supported in Global Security Policies of type CLOUD_ARMOR.
        # @!attribute [rw] rate_limit_threshold
        #   @return [::Google::Cloud::Compute::V1::SecurityPolicyRuleRateLimitOptionsThreshold]
        #     Threshold at which to begin ratelimiting.
        class SecurityPolicyRuleRateLimitOptions
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Determines the key to enforce the rate_limit_threshold on. Possible values are: - ALL: A single rate limit threshold is applied to all the requests matching this rule. This is the default value if "enforceOnKey" is not configured. - IP: The source IP address of the request is the key. Each IP has this limit enforced separately. - HTTP_HEADER: The value of the HTTP header whose name is configured under "enforceOnKeyName". The key value is truncated to the first 128 bytes of the header value. If no such header is present in the request, the key type defaults to ALL. - XFF_IP: The first IP address (i.e. the originating client IP address) specified in the list of IPs under X-Forwarded-For HTTP header. If no such header is present or the value is not a valid IP, the key defaults to the source IP address of the request i.e. key type IP. - HTTP_COOKIE: The value of the HTTP cookie whose name is configured under "enforceOnKeyName". The key value is truncated to the first 128 bytes of the cookie value. If no such cookie is present in the request, the key type defaults to ALL. - HTTP_PATH: The URL path of the HTTP request. The key value is truncated to the first 128 bytes. - SNI: Server name indication in the TLS session of the HTTPS request. The key value is truncated to the first 128 bytes. The key type defaults to ALL on a HTTP session. - REGION_CODE: The country/region from which the request originates.
          module EnforceOnKey
            # A value indicating that the enum field is not set.
            UNDEFINED_ENFORCE_ON_KEY = 0

            ALL = 64_897

            HTTP_COOKIE = 494_981_627

            HTTP_HEADER = 91_597_348

            HTTP_PATH = 311_503_228

            IP = 2343

            REGION_CODE = 79_559_768

            SNI = 82_254

            XFF_IP = 438_707_118
          end
        end

        # @!attribute [rw] enforce_on_key_name
        #   @return [::String]
        #     Rate limit key name applicable only for the following key types: HTTP_HEADER -- Name of the HTTP header whose value is taken as the key value. HTTP_COOKIE -- Name of the HTTP cookie whose value is taken as the key value.
        # @!attribute [rw] enforce_on_key_type
        #   @return [::String]
        #     Determines the key to enforce the rate_limit_threshold on. Possible values are: - ALL: A single rate limit threshold is applied to all the requests matching this rule. This is the default value if "enforceOnKeyConfigs" is not configured. - IP: The source IP address of the request is the key. Each IP has this limit enforced separately. - HTTP_HEADER: The value of the HTTP header whose name is configured under "enforceOnKeyName". The key value is truncated to the first 128 bytes of the header value. If no such header is present in the request, the key type defaults to ALL. - XFF_IP: The first IP address (i.e. the originating client IP address) specified in the list of IPs under X-Forwarded-For HTTP header. If no such header is present or the value is not a valid IP, the key defaults to the source IP address of the request i.e. key type IP. - HTTP_COOKIE: The value of the HTTP cookie whose name is configured under "enforceOnKeyName". The key value is truncated to the first 128 bytes of the cookie value. If no such cookie is present in the request, the key type defaults to ALL. - HTTP_PATH: The URL path of the HTTP request. The key value is truncated to the first 128 bytes. - SNI: Server name indication in the TLS session of the HTTPS request. The key value is truncated to the first 128 bytes. The key type defaults to ALL on a HTTP session. - REGION_CODE: The country/region from which the request originates.
        #     Check the EnforceOnKeyType enum for the list of possible values.
        class SecurityPolicyRuleRateLimitOptionsEnforceOnKeyConfig
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Determines the key to enforce the rate_limit_threshold on. Possible values are: - ALL: A single rate limit threshold is applied to all the requests matching this rule. This is the default value if "enforceOnKeyConfigs" is not configured. - IP: The source IP address of the request is the key. Each IP has this limit enforced separately. - HTTP_HEADER: The value of the HTTP header whose name is configured under "enforceOnKeyName". The key value is truncated to the first 128 bytes of the header value. If no such header is present in the request, the key type defaults to ALL. - XFF_IP: The first IP address (i.e. the originating client IP address) specified in the list of IPs under X-Forwarded-For HTTP header. If no such header is present or the value is not a valid IP, the key defaults to the source IP address of the request i.e. key type IP. - HTTP_COOKIE: The value of the HTTP cookie whose name is configured under "enforceOnKeyName". The key value is truncated to the first 128 bytes of the cookie value. If no such cookie is present in the request, the key type defaults to ALL. - HTTP_PATH: The URL path of the HTTP request. The key value is truncated to the first 128 bytes. - SNI: Server name indication in the TLS session of the HTTPS request. The key value is truncated to the first 128 bytes. The key type defaults to ALL on a HTTP session. - REGION_CODE: The country/region from which the request originates.
          module EnforceOnKeyType
            # A value indicating that the enum field is not set.
            UNDEFINED_ENFORCE_ON_KEY_TYPE = 0

            ALL = 64_897

            HTTP_COOKIE = 494_981_627

            HTTP_HEADER = 91_597_348

            HTTP_PATH = 311_503_228

            IP = 2343

            REGION_CODE = 79_559_768

            SNI = 82_254

            XFF_IP = 438_707_118
          end
        end

        # @!attribute [rw] count
        #   @return [::Integer]
        #     Number of HTTP(S) requests for calculating the threshold.
        # @!attribute [rw] interval_sec
        #   @return [::Integer]
        #     Interval over which the threshold is computed.
        class SecurityPolicyRuleRateLimitOptionsThreshold
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] target
        #   @return [::String]
        #     Target for the redirect action. This is required if the type is EXTERNAL_302 and cannot be specified for GOOGLE_RECAPTCHA.
        # @!attribute [rw] type
        #   @return [::String]
        #     Type of the redirect action.
        #     Check the Type enum for the list of possible values.
        class SecurityPolicyRuleRedirectOptions
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Type of the redirect action.
          module Type
            # A value indicating that the enum field is not set.
            UNDEFINED_TYPE = 0

            EXTERNAL_302 = 395_733_761

            GOOGLE_RECAPTCHA = 518_803_009
          end
        end

        # @!attribute [rw] base
        #   @return [::String]
        #     The base relative to which 'offset' is measured. Possible values are: - IPV4: Points to the beginning of the IPv4 header. - IPV6: Points to the beginning of the IPv6 header. - TCP: Points to the beginning of the TCP header, skipping over any IPv4 options or IPv6 extension headers. Not present for non-first fragments. - UDP: Points to the beginning of the UDP header, skipping over any IPv4 options or IPv6 extension headers. Not present for non-first fragments. required
        #     Check the Base enum for the list of possible values.
        # @!attribute [rw] mask
        #   @return [::String]
        #     If specified, apply this mask (bitwise AND) to the field to ignore bits before matching. Encoded as a hexadecimal number (starting with "0x"). The last byte of the field (in network byte order) corresponds to the least significant byte of the mask.
        # @!attribute [rw] name
        #   @return [::String]
        #     The name of this field. Must be unique within the policy.
        # @!attribute [rw] offset
        #   @return [::Integer]
        #     Offset of the first byte of the field (in network byte order) relative to 'base'.
        # @!attribute [rw] size
        #   @return [::Integer]
        #     Size of the field in bytes. Valid values: 1-4.
        class SecurityPolicyUserDefinedField
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The base relative to which 'offset' is measured. Possible values are: - IPV4: Points to the beginning of the IPv4 header. - IPV6: Points to the beginning of the IPv6 header. - TCP: Points to the beginning of the TCP header, skipping over any IPv4 options or IPv6 extension headers. Not present for non-first fragments. - UDP: Points to the beginning of the UDP header, skipping over any IPv4 options or IPv6 extension headers. Not present for non-first fragments. required
          module Base
            # A value indicating that the enum field is not set.
            UNDEFINED_BASE = 0

            IPV4 = 2_254_341

            IPV6 = 2_254_343

            TCP = 82_881

            UDP = 83_873
          end
        end

        # The authentication and authorization settings for a BackendService.
        # @!attribute [rw] aws_v4_authentication
        #   @return [::Google::Cloud::Compute::V1::AWSV4Signature]
        #     The configuration needed to generate a signature for access to private storage buckets that support AWS's Signature Version 4 for authentication. Allowed only for INTERNET_IP_PORT and INTERNET_FQDN_PORT NEG backends.
        # @!attribute [rw] client_tls_policy
        #   @return [::String]
        #     Optional. A URL referring to a networksecurity.ClientTlsPolicy resource that describes how clients should authenticate with this service's backends. clientTlsPolicy only applies to a global BackendService with the loadBalancingScheme set to INTERNAL_SELF_MANAGED. If left blank, communications are not encrypted.
        # @!attribute [rw] subject_alt_names
        #   @return [::Array<::String>]
        #     Optional. A list of Subject Alternative Names (SANs) that the client verifies during a mutual TLS handshake with an server/endpoint for this BackendService. When the server presents its X.509 certificate to the client, the client inspects the certificate's subjectAltName field. If the field contains one of the specified values, the communication continues. Otherwise, it fails. This additional check enables the client to verify that the server is authorized to run the requested service. Note that the contents of the server certificate's subjectAltName field are configured by the Public Key Infrastructure which provisions server identities. Only applies to a global BackendService with loadBalancingScheme set to INTERNAL_SELF_MANAGED. Only applies when BackendService has an attached clientTlsPolicy with clientCertificate (mTLS mode).
        class SecuritySettings
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.SendDiagnosticInterrupt. See the method description for details.
        # @!attribute [rw] instance
        #   @return [::String]
        #     Name of the instance scoping this request.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class SendDiagnosticInterruptInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A response message for Instances.SendDiagnosticInterrupt. See the method description for details.
        class SendDiagnosticInterruptInstanceResponse
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # An instance serial console output.
        # @!attribute [rw] contents
        #   @return [::String]
        #     [Output Only] The contents of the console output.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#serialPortOutput for serial port output.
        # @!attribute [rw] next
        #   @return [::Integer]
        #     [Output Only] The position of the next byte of content, regardless of whether the content exists, following the output returned in the `contents` property. Use this value in the next request as the start parameter.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] start
        #   @return [::Integer]
        #     The starting byte position of the output that was returned. This should match the start parameter sent with the request. If the serial console output exceeds the size of the buffer (1 MB), older output is overwritten by newer content. The output start value will indicate the byte position of the output that was returned, which might be different than the `start` value that was specified in the request.
        class SerialPortOutput
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] type
        #   @return [::String]
        #     Check the Type enum for the list of possible values.
        class ServerBinding
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods


          module Type
            # A value indicating that the enum field is not set.
            UNDEFINED_TYPE = 0

            # Node may associate with any physical server over its lifetime.
            RESTART_NODE_ON_ANY_SERVER = 502_950_985

            # Node may associate with minimal physical servers over its lifetime.
            RESTART_NODE_ON_MINIMAL_SERVERS = 204_166_495

            SERVER_BINDING_TYPE_UNSPECIFIED = 180_825_512
          end
        end

        # A service account.
        # @!attribute [rw] email
        #   @return [::String]
        #     Email address of the service account.
        # @!attribute [rw] scopes
        #   @return [::Array<::String>]
        #     The list of scopes to be made available for this service account.
        class ServiceAccount
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a ServiceAttachment resource. A service attachment represents a service that a producer has exposed. It encapsulates the load balancer which fronts the service runs and a list of NAT IP ranges that the producers uses to represent the consumers connecting to the service.
        # @!attribute [rw] connected_endpoints
        #   @return [::Array<::Google::Cloud::Compute::V1::ServiceAttachmentConnectedEndpoint>]
        #     [Output Only] An array of connections for all the consumers connected to this service attachment.
        # @!attribute [rw] connection_preference
        #   @return [::String]
        #     The connection preference of service attachment. The value can be set to ACCEPT_AUTOMATIC. An ACCEPT_AUTOMATIC service attachment is one that always accepts the connection from consumer forwarding rules.
        #     Check the ConnectionPreference enum for the list of possible values.
        # @!attribute [rw] consumer_accept_lists
        #   @return [::Array<::Google::Cloud::Compute::V1::ServiceAttachmentConsumerProjectLimit>]
        #     Projects that are allowed to connect to this service attachment.
        # @!attribute [rw] consumer_reject_lists
        #   @return [::Array<::String>]
        #     Projects that are not allowed to connect to this service attachment. The project can be specified using its id or number.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] domain_names
        #   @return [::Array<::String>]
        #     If specified, the domain name will be used during the integration between the PSC connected endpoints and the Cloud DNS. For example, this is a valid domain name: "p.mycompany.com.". Current max number of domain names supported is 1.
        # @!attribute [rw] enable_proxy_protocol
        #   @return [::Boolean]
        #     If true, enable the proxy protocol which is for supplying client TCP/IP address data in TCP connections that traverse proxies on their way to destination servers.
        # @!attribute [rw] fingerprint
        #   @return [::String]
        #     Fingerprint of this resource. A hash of the contents stored in this object. This field is used in optimistic locking. This field will be ignored when inserting a ServiceAttachment. An up-to-date fingerprint must be provided in order to patch/update the ServiceAttachment; otherwise, the request will fail with error 412 conditionNotMet. To see the latest fingerprint, make a get() request to retrieve the ServiceAttachment.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource type. The server generates this identifier.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#serviceAttachment for service attachments.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] nat_subnets
        #   @return [::Array<::String>]
        #     An array of URLs where each entry is the URL of a subnet provided by the service producer to use for NAT in this service attachment.
        # @!attribute [rw] producer_forwarding_rule
        #   @return [::String]
        #     The URL of a forwarding rule with loadBalancingScheme INTERNAL* that is serving the endpoint identified by this service attachment.
        # @!attribute [rw] psc_service_attachment_id
        #   @return [::Google::Cloud::Compute::V1::Uint128]
        #     [Output Only] An 128-bit global unique ID of the PSC service attachment.
        # @!attribute [rw] reconcile_connections
        #   @return [::Boolean]
        #     This flag determines whether a consumer accept/reject list change can reconcile the statuses of existing ACCEPTED or REJECTED PSC endpoints. - If false, connection policy update will only affect existing PENDING PSC endpoints. Existing ACCEPTED/REJECTED endpoints will remain untouched regardless how the connection policy is modified . - If true, update will affect both PENDING and ACCEPTED/REJECTED PSC endpoints. For example, an ACCEPTED PSC endpoint will be moved to REJECTED if its project is added to the reject list. For newly created service attachment, this boolean defaults to false.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] URL of the region where the service attachment resides. This field applies only to the region resource. You must specify this field as part of the HTTP request URL. It is not settable as a field in the request body.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] target_service
        #   @return [::String]
        #     The URL of a service serving the endpoint identified by this service attachment.
        class ServiceAttachment
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The connection preference of service attachment. The value can be set to ACCEPT_AUTOMATIC. An ACCEPT_AUTOMATIC service attachment is one that always accepts the connection from consumer forwarding rules.
          module ConnectionPreference
            # A value indicating that the enum field is not set.
            UNDEFINED_CONNECTION_PREFERENCE = 0

            ACCEPT_AUTOMATIC = 75_250_580

            ACCEPT_MANUAL = 373_061_341

            CONNECTION_PREFERENCE_UNSPECIFIED = 34_590_772
          end
        end

        # Contains a list of ServiceAttachmentsScopedList.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::ServiceAttachmentsScopedList}]
        #     A list of ServiceAttachmentsScopedList resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     Type of resource.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class ServiceAttachmentAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::ServiceAttachmentsScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # [Output Only] A connection connected to this service attachment.
        # @!attribute [rw] consumer_network
        #   @return [::String]
        #     The url of the consumer network.
        # @!attribute [rw] endpoint
        #   @return [::String]
        #     The url of a connected endpoint.
        # @!attribute [rw] psc_connection_id
        #   @return [::Integer]
        #     The PSC connection id of the connected endpoint.
        # @!attribute [rw] status
        #   @return [::String]
        #     The status of a connected endpoint to this service attachment.
        #     Check the Status enum for the list of possible values.
        class ServiceAttachmentConnectedEndpoint
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The status of a connected endpoint to this service attachment.
          module Status
            # A value indicating that the enum field is not set.
            UNDEFINED_STATUS = 0

            # The connection has been accepted by the producer.
            ACCEPTED = 246_714_279

            # The connection has been closed by the producer.
            CLOSED = 380_163_436

            # The connection has been accepted by the producer, but the producer needs to take further action before the forwarding rule can serve traffic.
            NEEDS_ATTENTION = 344_491_452

            # The connection is pending acceptance by the producer.
            PENDING = 35_394_935

            # The consumer is still connected but not using the connection.
            REJECTED = 174_130_302

            STATUS_UNSPECIFIED = 42_133_066
          end
        end

        # @!attribute [rw] connection_limit
        #   @return [::Integer]
        #     The value of the limit to set.
        # @!attribute [rw] network_url
        #   @return [::String]
        #     The network URL for the network to set the limit for.
        # @!attribute [rw] project_id_or_num
        #   @return [::String]
        #     The project id or number for the project to set the limit for.
        class ServiceAttachmentConsumerProjectLimit
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::ServiceAttachment>]
        #     A list of ServiceAttachment resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#serviceAttachment for service attachments.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class ServiceAttachmentList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] service_attachments
        #   @return [::Array<::Google::Cloud::Compute::V1::ServiceAttachment>]
        #     A list of ServiceAttachments contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     Informational warning which replaces the list of service attachments when the list is empty.
        class ServiceAttachmentsScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetSslProxies.SetBackendService. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_ssl_proxies_set_backend_service_request_resource
        #   @return [::Google::Cloud::Compute::V1::TargetSslProxiesSetBackendServiceRequest]
        #     The body resource for this request
        # @!attribute [rw] target_ssl_proxy
        #   @return [::String]
        #     Name of the TargetSslProxy resource whose BackendService resource is to be set.
        class SetBackendServiceTargetSslProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetTcpProxies.SetBackendService. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_tcp_proxies_set_backend_service_request_resource
        #   @return [::Google::Cloud::Compute::V1::TargetTcpProxiesSetBackendServiceRequest]
        #     The body resource for this request
        # @!attribute [rw] target_tcp_proxy
        #   @return [::String]
        #     Name of the TargetTcpProxy resource whose BackendService resource is to be set.
        class SetBackendServiceTargetTcpProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetPools.SetBackup. See the method description for details.
        # @!attribute [rw] failover_ratio
        #   @return [::Float]
        #     New failoverRatio value for the target pool.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_pool
        #   @return [::String]
        #     Name of the TargetPool resource to set a backup pool for.
        # @!attribute [rw] target_reference_resource
        #   @return [::Google::Cloud::Compute::V1::TargetReference]
        #     The body resource for this request
        class SetBackupTargetPoolRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetHttpsProxies.SetCertificateMap. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_https_proxies_set_certificate_map_request_resource
        #   @return [::Google::Cloud::Compute::V1::TargetHttpsProxiesSetCertificateMapRequest]
        #     The body resource for this request
        # @!attribute [rw] target_https_proxy
        #   @return [::String]
        #     Name of the TargetHttpsProxy resource whose CertificateMap is to be set. The name must be 1-63 characters long, and comply with RFC1035.
        class SetCertificateMapTargetHttpsProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetSslProxies.SetCertificateMap. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_ssl_proxies_set_certificate_map_request_resource
        #   @return [::Google::Cloud::Compute::V1::TargetSslProxiesSetCertificateMapRequest]
        #     The body resource for this request
        # @!attribute [rw] target_ssl_proxy
        #   @return [::String]
        #     Name of the TargetSslProxy resource whose CertificateMap is to be set. The name must be 1-63 characters long, and comply with RFC1035.
        class SetCertificateMapTargetSslProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] client_operation_id
        #   @return [::String]
        #     [Output Only] The client operation id.
        # @!attribute [rw] per_location_operations
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::SetCommonInstanceMetadataOperationMetadataPerLocationOperationInfo}]
        #     [Output Only] Status information per location (location name is key). Example key: zones/us-central1-a
        class SetCommonInstanceMetadataOperationMetadata
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::SetCommonInstanceMetadataOperationMetadataPerLocationOperationInfo]
          class PerLocationOperationsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # @!attribute [rw] error
        #   @return [::Google::Cloud::Compute::V1::Status]
        #     [Output Only] If state is `ABANDONED` or `FAILED`, this field is populated.
        # @!attribute [rw] state
        #   @return [::String]
        #     [Output Only] Status of the action, which can be one of the following: `PROPAGATING`, `PROPAGATED`, `ABANDONED`, `FAILED`, or `DONE`.
        #     Check the State enum for the list of possible values.
        class SetCommonInstanceMetadataOperationMetadataPerLocationOperationInfo
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # [Output Only] Status of the action, which can be one of the following: `PROPAGATING`, `PROPAGATED`, `ABANDONED`, `FAILED`, or `DONE`.
          module State
            # A value indicating that the enum field is not set.
            UNDEFINED_STATE = 0

            # Operation not tracked in this location e.g. zone is marked as DOWN.
            ABANDONED = 81_797_556

            # Operation has completed successfully.
            DONE = 2_104_194

            # Operation is in an error state.
            FAILED = 455_706_685

            # Operation is confirmed to be in the location.
            PROPAGATED = 507_550_299

            # Operation is not yet confirmed to have been created in the location.
            PROPAGATING = 164_807_046

            UNSPECIFIED = 526_786_327
          end
        end

        # A request message for Projects.SetCommonInstanceMetadata. See the method description for details.
        # @!attribute [rw] metadata_resource
        #   @return [::Google::Cloud::Compute::V1::Metadata]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class SetCommonInstanceMetadataProjectRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Projects.SetDefaultNetworkTier. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] projects_set_default_network_tier_request_resource
        #   @return [::Google::Cloud::Compute::V1::ProjectsSetDefaultNetworkTierRequest]
        #     The body resource for this request
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class SetDefaultNetworkTierProjectRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.SetDeletionProtection. See the method description for details.
        # @!attribute [rw] deletion_protection
        #   @return [::Boolean]
        #     Whether the resource should be protected against deletion.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class SetDeletionProtectionInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.SetDiskAutoDelete. See the method description for details.
        # @!attribute [rw] auto_delete
        #   @return [::Boolean]
        #     Whether to auto-delete the disk when the instance is deleted.
        # @!attribute [rw] device_name
        #   @return [::String]
        #     The device name of the disk to modify. Make a get() request on the instance to view currently attached disks and device names.
        # @!attribute [rw] instance
        #   @return [::String]
        #     The instance name for this request.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class SetDiskAutoDeleteInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for BackendBuckets.SetEdgeSecurityPolicy. See the method description for details.
        # @!attribute [rw] backend_bucket
        #   @return [::String]
        #     Name of the BackendService resource to which the security policy should be set. The name should conform to RFC1035.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] security_policy_reference_resource
        #   @return [::Google::Cloud::Compute::V1::SecurityPolicyReference]
        #     The body resource for this request
        class SetEdgeSecurityPolicyBackendBucketRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for BackendServices.SetEdgeSecurityPolicy. See the method description for details.
        # @!attribute [rw] backend_service
        #   @return [::String]
        #     Name of the BackendService resource to which the edge security policy should be set. The name should conform to RFC1035.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] security_policy_reference_resource
        #   @return [::Google::Cloud::Compute::V1::SecurityPolicyReference]
        #     The body resource for this request
        class SetEdgeSecurityPolicyBackendServiceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for BackendBuckets.SetIamPolicy. See the method description for details.
        # @!attribute [rw] global_set_policy_request_resource
        #   @return [::Google::Cloud::Compute::V1::GlobalSetPolicyRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class SetIamPolicyBackendBucketRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for BackendServices.SetIamPolicy. See the method description for details.
        # @!attribute [rw] global_set_policy_request_resource
        #   @return [::Google::Cloud::Compute::V1::GlobalSetPolicyRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class SetIamPolicyBackendServiceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Disks.SetIamPolicy. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        # @!attribute [rw] zone_set_policy_request_resource
        #   @return [::Google::Cloud::Compute::V1::ZoneSetPolicyRequest]
        #     The body resource for this request
        class SetIamPolicyDiskRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for FirewallPolicies.SetIamPolicy. See the method description for details.
        # @!attribute [rw] global_organization_set_policy_request_resource
        #   @return [::Google::Cloud::Compute::V1::GlobalOrganizationSetPolicyRequest]
        #     The body resource for this request
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class SetIamPolicyFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Images.SetIamPolicy. See the method description for details.
        # @!attribute [rw] global_set_policy_request_resource
        #   @return [::Google::Cloud::Compute::V1::GlobalSetPolicyRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class SetIamPolicyImageRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.SetIamPolicy. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        # @!attribute [rw] zone_set_policy_request_resource
        #   @return [::Google::Cloud::Compute::V1::ZoneSetPolicyRequest]
        #     The body resource for this request
        class SetIamPolicyInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceTemplates.SetIamPolicy. See the method description for details.
        # @!attribute [rw] global_set_policy_request_resource
        #   @return [::Google::Cloud::Compute::V1::GlobalSetPolicyRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class SetIamPolicyInstanceTemplateRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Licenses.SetIamPolicy. See the method description for details.
        # @!attribute [rw] global_set_policy_request_resource
        #   @return [::Google::Cloud::Compute::V1::GlobalSetPolicyRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class SetIamPolicyLicenseRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for MachineImages.SetIamPolicy. See the method description for details.
        # @!attribute [rw] global_set_policy_request_resource
        #   @return [::Google::Cloud::Compute::V1::GlobalSetPolicyRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class SetIamPolicyMachineImageRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkAttachments.SetIamPolicy. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] region_set_policy_request_resource
        #   @return [::Google::Cloud::Compute::V1::RegionSetPolicyRequest]
        #     The body resource for this request
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class SetIamPolicyNetworkAttachmentRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkFirewallPolicies.SetIamPolicy. See the method description for details.
        # @!attribute [rw] global_set_policy_request_resource
        #   @return [::Google::Cloud::Compute::V1::GlobalSetPolicyRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class SetIamPolicyNetworkFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NodeGroups.SetIamPolicy. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        # @!attribute [rw] zone_set_policy_request_resource
        #   @return [::Google::Cloud::Compute::V1::ZoneSetPolicyRequest]
        #     The body resource for this request
        class SetIamPolicyNodeGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NodeTemplates.SetIamPolicy. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] region_set_policy_request_resource
        #   @return [::Google::Cloud::Compute::V1::RegionSetPolicyRequest]
        #     The body resource for this request
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class SetIamPolicyNodeTemplateRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionBackendServices.SetIamPolicy. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] region_set_policy_request_resource
        #   @return [::Google::Cloud::Compute::V1::RegionSetPolicyRequest]
        #     The body resource for this request
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class SetIamPolicyRegionBackendServiceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionDisks.SetIamPolicy. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] region_set_policy_request_resource
        #   @return [::Google::Cloud::Compute::V1::RegionSetPolicyRequest]
        #     The body resource for this request
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class SetIamPolicyRegionDiskRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionNetworkFirewallPolicies.SetIamPolicy. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] region_set_policy_request_resource
        #   @return [::Google::Cloud::Compute::V1::RegionSetPolicyRequest]
        #     The body resource for this request
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class SetIamPolicyRegionNetworkFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Reservations.SetIamPolicy. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        # @!attribute [rw] zone_set_policy_request_resource
        #   @return [::Google::Cloud::Compute::V1::ZoneSetPolicyRequest]
        #     The body resource for this request
        class SetIamPolicyReservationRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ResourcePolicies.SetIamPolicy. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] region_set_policy_request_resource
        #   @return [::Google::Cloud::Compute::V1::RegionSetPolicyRequest]
        #     The body resource for this request
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class SetIamPolicyResourcePolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ServiceAttachments.SetIamPolicy. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] region_set_policy_request_resource
        #   @return [::Google::Cloud::Compute::V1::RegionSetPolicyRequest]
        #     The body resource for this request
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class SetIamPolicyServiceAttachmentRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Snapshots.SetIamPolicy. See the method description for details.
        # @!attribute [rw] global_set_policy_request_resource
        #   @return [::Google::Cloud::Compute::V1::GlobalSetPolicyRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class SetIamPolicySnapshotRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Subnetworks.SetIamPolicy. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] region_set_policy_request_resource
        #   @return [::Google::Cloud::Compute::V1::RegionSetPolicyRequest]
        #     The body resource for this request
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class SetIamPolicySubnetworkRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceGroupManagers.SetInstanceTemplate. See the method description for details.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     The name of the managed instance group.
        # @!attribute [rw] instance_group_managers_set_instance_template_request_resource
        #   @return [::Google::Cloud::Compute::V1::InstanceGroupManagersSetInstanceTemplateRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where the managed instance group is located.
        class SetInstanceTemplateInstanceGroupManagerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionInstanceGroupManagers.SetInstanceTemplate. See the method description for details.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     The name of the managed instance group.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] region_instance_group_managers_set_template_request_resource
        #   @return [::Google::Cloud::Compute::V1::RegionInstanceGroupManagersSetTemplateRequest]
        #     The body resource for this request
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class SetInstanceTemplateRegionInstanceGroupManagerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Addresses.SetLabels. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The region for this request.
        # @!attribute [rw] region_set_labels_request_resource
        #   @return [::Google::Cloud::Compute::V1::RegionSetLabelsRequest]
        #     The body resource for this request
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class SetLabelsAddressRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Disks.SetLabels. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        # @!attribute [rw] zone_set_labels_request_resource
        #   @return [::Google::Cloud::Compute::V1::ZoneSetLabelsRequest]
        #     The body resource for this request
        class SetLabelsDiskRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ExternalVpnGateways.SetLabels. See the method description for details.
        # @!attribute [rw] global_set_labels_request_resource
        #   @return [::Google::Cloud::Compute::V1::GlobalSetLabelsRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class SetLabelsExternalVpnGatewayRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ForwardingRules.SetLabels. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The region for this request.
        # @!attribute [rw] region_set_labels_request_resource
        #   @return [::Google::Cloud::Compute::V1::RegionSetLabelsRequest]
        #     The body resource for this request
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class SetLabelsForwardingRuleRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for GlobalAddresses.SetLabels. See the method description for details.
        # @!attribute [rw] global_set_labels_request_resource
        #   @return [::Google::Cloud::Compute::V1::GlobalSetLabelsRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class SetLabelsGlobalAddressRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for GlobalForwardingRules.SetLabels. See the method description for details.
        # @!attribute [rw] global_set_labels_request_resource
        #   @return [::Google::Cloud::Compute::V1::GlobalSetLabelsRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class SetLabelsGlobalForwardingRuleRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Images.SetLabels. See the method description for details.
        # @!attribute [rw] global_set_labels_request_resource
        #   @return [::Google::Cloud::Compute::V1::GlobalSetLabelsRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class SetLabelsImageRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.SetLabels. See the method description for details.
        # @!attribute [rw] instance
        #   @return [::String]
        #     Name of the instance scoping this request.
        # @!attribute [rw] instances_set_labels_request_resource
        #   @return [::Google::Cloud::Compute::V1::InstancesSetLabelsRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class SetLabelsInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InterconnectAttachments.SetLabels. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The region for this request.
        # @!attribute [rw] region_set_labels_request_resource
        #   @return [::Google::Cloud::Compute::V1::RegionSetLabelsRequest]
        #     The body resource for this request
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class SetLabelsInterconnectAttachmentRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Interconnects.SetLabels. See the method description for details.
        # @!attribute [rw] global_set_labels_request_resource
        #   @return [::Google::Cloud::Compute::V1::GlobalSetLabelsRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class SetLabelsInterconnectRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionDisks.SetLabels. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The region for this request.
        # @!attribute [rw] region_set_labels_request_resource
        #   @return [::Google::Cloud::Compute::V1::RegionSetLabelsRequest]
        #     The body resource for this request
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class SetLabelsRegionDiskRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for SecurityPolicies.SetLabels. See the method description for details.
        # @!attribute [rw] global_set_labels_request_resource
        #   @return [::Google::Cloud::Compute::V1::GlobalSetLabelsRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class SetLabelsSecurityPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Snapshots.SetLabels. See the method description for details.
        # @!attribute [rw] global_set_labels_request_resource
        #   @return [::Google::Cloud::Compute::V1::GlobalSetLabelsRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class SetLabelsSnapshotRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetVpnGateways.SetLabels. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The region for this request.
        # @!attribute [rw] region_set_labels_request_resource
        #   @return [::Google::Cloud::Compute::V1::RegionSetLabelsRequest]
        #     The body resource for this request
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class SetLabelsTargetVpnGatewayRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for VpnGateways.SetLabels. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The region for this request.
        # @!attribute [rw] region_set_labels_request_resource
        #   @return [::Google::Cloud::Compute::V1::RegionSetLabelsRequest]
        #     The body resource for this request
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class SetLabelsVpnGatewayRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for VpnTunnels.SetLabels. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The region for this request.
        # @!attribute [rw] region_set_labels_request_resource
        #   @return [::Google::Cloud::Compute::V1::RegionSetLabelsRequest]
        #     The body resource for this request
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        class SetLabelsVpnTunnelRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.SetMachineResources. See the method description for details.
        # @!attribute [rw] instance
        #   @return [::String]
        #     Name of the instance scoping this request.
        # @!attribute [rw] instances_set_machine_resources_request_resource
        #   @return [::Google::Cloud::Compute::V1::InstancesSetMachineResourcesRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class SetMachineResourcesInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.SetMachineType. See the method description for details.
        # @!attribute [rw] instance
        #   @return [::String]
        #     Name of the instance scoping this request.
        # @!attribute [rw] instances_set_machine_type_request_resource
        #   @return [::Google::Cloud::Compute::V1::InstancesSetMachineTypeRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class SetMachineTypeInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.SetMetadata. See the method description for details.
        # @!attribute [rw] instance
        #   @return [::String]
        #     Name of the instance scoping this request.
        # @!attribute [rw] metadata_resource
        #   @return [::Google::Cloud::Compute::V1::Metadata]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class SetMetadataInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.SetMinCpuPlatform. See the method description for details.
        # @!attribute [rw] instance
        #   @return [::String]
        #     Name of the instance scoping this request.
        # @!attribute [rw] instances_set_min_cpu_platform_request_resource
        #   @return [::Google::Cloud::Compute::V1::InstancesSetMinCpuPlatformRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class SetMinCpuPlatformInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.SetName. See the method description for details.
        # @!attribute [rw] instance
        #   @return [::String]
        #     The instance name for this request.
        # @!attribute [rw] instances_set_name_request_resource
        #   @return [::Google::Cloud::Compute::V1::InstancesSetNameRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class SetNameInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceGroups.SetNamedPorts. See the method description for details.
        # @!attribute [rw] instance_group
        #   @return [::String]
        #     The name of the instance group where the named ports are updated.
        # @!attribute [rw] instance_groups_set_named_ports_request_resource
        #   @return [::Google::Cloud::Compute::V1::InstanceGroupsSetNamedPortsRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where the instance group is located.
        class SetNamedPortsInstanceGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionInstanceGroups.SetNamedPorts. See the method description for details.
        # @!attribute [rw] instance_group
        #   @return [::String]
        #     The name of the regional instance group where the named ports are updated.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] region_instance_groups_set_named_ports_request_resource
        #   @return [::Google::Cloud::Compute::V1::RegionInstanceGroupsSetNamedPortsRequest]
        #     The body resource for this request
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class SetNamedPortsRegionInstanceGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NodeGroups.SetNodeTemplate. See the method description for details.
        # @!attribute [rw] node_group
        #   @return [::String]
        #     Name of the NodeGroup resource to update.
        # @!attribute [rw] node_groups_set_node_template_request_resource
        #   @return [::Google::Cloud::Compute::V1::NodeGroupsSetNodeTemplateRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class SetNodeTemplateNodeGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Subnetworks.SetPrivateIpGoogleAccess. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] subnetwork
        #   @return [::String]
        #     Name of the Subnetwork resource.
        # @!attribute [rw] subnetworks_set_private_ip_google_access_request_resource
        #   @return [::Google::Cloud::Compute::V1::SubnetworksSetPrivateIpGoogleAccessRequest]
        #     The body resource for this request
        class SetPrivateIpGoogleAccessSubnetworkRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetSslProxies.SetProxyHeader. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_ssl_proxies_set_proxy_header_request_resource
        #   @return [::Google::Cloud::Compute::V1::TargetSslProxiesSetProxyHeaderRequest]
        #     The body resource for this request
        # @!attribute [rw] target_ssl_proxy
        #   @return [::String]
        #     Name of the TargetSslProxy resource whose ProxyHeader is to be set.
        class SetProxyHeaderTargetSslProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetTcpProxies.SetProxyHeader. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_tcp_proxies_set_proxy_header_request_resource
        #   @return [::Google::Cloud::Compute::V1::TargetTcpProxiesSetProxyHeaderRequest]
        #     The body resource for this request
        # @!attribute [rw] target_tcp_proxy
        #   @return [::String]
        #     Name of the TargetTcpProxy resource whose ProxyHeader is to be set.
        class SetProxyHeaderTargetTcpProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetHttpsProxies.SetQuicOverride. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_https_proxies_set_quic_override_request_resource
        #   @return [::Google::Cloud::Compute::V1::TargetHttpsProxiesSetQuicOverrideRequest]
        #     The body resource for this request
        # @!attribute [rw] target_https_proxy
        #   @return [::String]
        #     Name of the TargetHttpsProxy resource to set the QUIC override policy for. The name should conform to RFC1035.
        class SetQuicOverrideTargetHttpsProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.SetScheduling. See the method description for details.
        # @!attribute [rw] instance
        #   @return [::String]
        #     Instance name for this request.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] scheduling_resource
        #   @return [::Google::Cloud::Compute::V1::Scheduling]
        #     The body resource for this request
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class SetSchedulingInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for BackendServices.SetSecurityPolicy. See the method description for details.
        # @!attribute [rw] backend_service
        #   @return [::String]
        #     Name of the BackendService resource to which the security policy should be set. The name should conform to RFC1035.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] security_policy_reference_resource
        #   @return [::Google::Cloud::Compute::V1::SecurityPolicyReference]
        #     The body resource for this request
        class SetSecurityPolicyBackendServiceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.SetSecurityPolicy. See the method description for details.
        # @!attribute [rw] instance
        #   @return [::String]
        #     Name of the Instance resource to which the security policy should be set. The name should conform to RFC1035.
        # @!attribute [rw] instances_set_security_policy_request_resource
        #   @return [::Google::Cloud::Compute::V1::InstancesSetSecurityPolicyRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     Name of the zone scoping this request.
        class SetSecurityPolicyInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionBackendServices.SetSecurityPolicy. See the method description for details.
        # @!attribute [rw] backend_service
        #   @return [::String]
        #     Name of the BackendService resource to which the security policy should be set. The name should conform to RFC1035.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] security_policy_reference_resource
        #   @return [::Google::Cloud::Compute::V1::SecurityPolicyReference]
        #     The body resource for this request
        class SetSecurityPolicyRegionBackendServiceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetInstances.SetSecurityPolicy. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] security_policy_reference_resource
        #   @return [::Google::Cloud::Compute::V1::SecurityPolicyReference]
        #     The body resource for this request
        # @!attribute [rw] target_instance
        #   @return [::String]
        #     Name of the TargetInstance resource to which the security policy should be set. The name should conform to RFC1035.
        # @!attribute [rw] zone
        #   @return [::String]
        #     Name of the zone scoping this request.
        class SetSecurityPolicyTargetInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetPools.SetSecurityPolicy. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] security_policy_reference_resource
        #   @return [::Google::Cloud::Compute::V1::SecurityPolicyReference]
        #     The body resource for this request
        # @!attribute [rw] target_pool
        #   @return [::String]
        #     Name of the TargetPool resource to which the security policy should be set. The name should conform to RFC1035.
        class SetSecurityPolicyTargetPoolRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.SetServiceAccount. See the method description for details.
        # @!attribute [rw] instance
        #   @return [::String]
        #     Name of the instance resource to start.
        # @!attribute [rw] instances_set_service_account_request_resource
        #   @return [::Google::Cloud::Compute::V1::InstancesSetServiceAccountRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class SetServiceAccountInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.SetShieldedInstanceIntegrityPolicy. See the method description for details.
        # @!attribute [rw] instance
        #   @return [::String]
        #     Name or id of the instance scoping this request.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] shielded_instance_integrity_policy_resource
        #   @return [::Google::Cloud::Compute::V1::ShieldedInstanceIntegrityPolicy]
        #     The body resource for this request
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class SetShieldedInstanceIntegrityPolicyInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionTargetHttpsProxies.SetSslCertificates. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] region_target_https_proxies_set_ssl_certificates_request_resource
        #   @return [::Google::Cloud::Compute::V1::RegionTargetHttpsProxiesSetSslCertificatesRequest]
        #     The body resource for this request
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_https_proxy
        #   @return [::String]
        #     Name of the TargetHttpsProxy resource to set an SslCertificates resource for.
        class SetSslCertificatesRegionTargetHttpsProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetHttpsProxies.SetSslCertificates. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_https_proxies_set_ssl_certificates_request_resource
        #   @return [::Google::Cloud::Compute::V1::TargetHttpsProxiesSetSslCertificatesRequest]
        #     The body resource for this request
        # @!attribute [rw] target_https_proxy
        #   @return [::String]
        #     Name of the TargetHttpsProxy resource to set an SslCertificates resource for.
        class SetSslCertificatesTargetHttpsProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetSslProxies.SetSslCertificates. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_ssl_proxies_set_ssl_certificates_request_resource
        #   @return [::Google::Cloud::Compute::V1::TargetSslProxiesSetSslCertificatesRequest]
        #     The body resource for this request
        # @!attribute [rw] target_ssl_proxy
        #   @return [::String]
        #     Name of the TargetSslProxy resource whose SslCertificate resource is to be set.
        class SetSslCertificatesTargetSslProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetHttpsProxies.SetSslPolicy. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] ssl_policy_reference_resource
        #   @return [::Google::Cloud::Compute::V1::SslPolicyReference]
        #     The body resource for this request
        # @!attribute [rw] target_https_proxy
        #   @return [::String]
        #     Name of the TargetHttpsProxy resource whose SSL policy is to be set. The name must be 1-63 characters long, and comply with RFC1035.
        class SetSslPolicyTargetHttpsProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetSslProxies.SetSslPolicy. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] ssl_policy_reference_resource
        #   @return [::Google::Cloud::Compute::V1::SslPolicyReference]
        #     The body resource for this request
        # @!attribute [rw] target_ssl_proxy
        #   @return [::String]
        #     Name of the TargetSslProxy resource whose SSL policy is to be set. The name must be 1-63 characters long, and comply with RFC1035.
        class SetSslPolicyTargetSslProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.SetTags. See the method description for details.
        # @!attribute [rw] instance
        #   @return [::String]
        #     Name of the instance scoping this request.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] tags_resource
        #   @return [::Google::Cloud::Compute::V1::Tags]
        #     The body resource for this request
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class SetTagsInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ForwardingRules.SetTarget. See the method description for details.
        # @!attribute [rw] forwarding_rule
        #   @return [::String]
        #     Name of the ForwardingRule resource in which target is to be set.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_reference_resource
        #   @return [::Google::Cloud::Compute::V1::TargetReference]
        #     The body resource for this request
        class SetTargetForwardingRuleRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for GlobalForwardingRules.SetTarget. See the method description for details.
        # @!attribute [rw] forwarding_rule
        #   @return [::String]
        #     Name of the ForwardingRule resource in which target is to be set.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_reference_resource
        #   @return [::Google::Cloud::Compute::V1::TargetReference]
        #     The body resource for this request
        class SetTargetGlobalForwardingRuleRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceGroupManagers.SetTargetPools. See the method description for details.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     The name of the managed instance group.
        # @!attribute [rw] instance_group_managers_set_target_pools_request_resource
        #   @return [::Google::Cloud::Compute::V1::InstanceGroupManagersSetTargetPoolsRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where the managed instance group is located.
        class SetTargetPoolsInstanceGroupManagerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionInstanceGroupManagers.SetTargetPools. See the method description for details.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     Name of the managed instance group.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] region_instance_group_managers_set_target_pools_request_resource
        #   @return [::Google::Cloud::Compute::V1::RegionInstanceGroupManagersSetTargetPoolsRequest]
        #     The body resource for this request
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class SetTargetPoolsRegionInstanceGroupManagerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionTargetHttpProxies.SetUrlMap. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_http_proxy
        #   @return [::String]
        #     Name of the TargetHttpProxy to set a URL map for.
        # @!attribute [rw] url_map_reference_resource
        #   @return [::Google::Cloud::Compute::V1::UrlMapReference]
        #     The body resource for this request
        class SetUrlMapRegionTargetHttpProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionTargetHttpsProxies.SetUrlMap. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_https_proxy
        #   @return [::String]
        #     Name of the TargetHttpsProxy to set a URL map for.
        # @!attribute [rw] url_map_reference_resource
        #   @return [::Google::Cloud::Compute::V1::UrlMapReference]
        #     The body resource for this request
        class SetUrlMapRegionTargetHttpsProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetHttpProxies.SetUrlMap. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_http_proxy
        #   @return [::String]
        #     Name of the TargetHttpProxy to set a URL map for.
        # @!attribute [rw] url_map_reference_resource
        #   @return [::Google::Cloud::Compute::V1::UrlMapReference]
        #     The body resource for this request
        class SetUrlMapTargetHttpProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for TargetHttpsProxies.SetUrlMap. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] target_https_proxy
        #   @return [::String]
        #     Name of the TargetHttpsProxy resource whose URL map is to be set.
        # @!attribute [rw] url_map_reference_resource
        #   @return [::Google::Cloud::Compute::V1::UrlMapReference]
        #     The body resource for this request
        class SetUrlMapTargetHttpsProxyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Projects.SetUsageExportBucket. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] usage_export_location_resource
        #   @return [::Google::Cloud::Compute::V1::UsageExportLocation]
        #     The body resource for this request
        class SetUsageExportBucketProjectRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # The share setting for reservations and sole tenancy node groups.
        # @!attribute [rw] project_map
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::ShareSettingsProjectConfig}]
        #     A map of project id and project config. This is only valid when share_type's value is SPECIFIC_PROJECTS.
        # @!attribute [rw] share_type
        #   @return [::String]
        #     Type of sharing for this shared-reservation
        #     Check the ShareType enum for the list of possible values.
        class ShareSettings
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::ShareSettingsProjectConfig]
          class ProjectMapEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # Type of sharing for this shared-reservation
          module ShareType
            # A value indicating that the enum field is not set.
            UNDEFINED_SHARE_TYPE = 0

            # Default value.
            LOCAL = 72_607_563

            # Shared-reservation is open to entire Organization
            ORGANIZATION = 274_978_099

            # Default value. This value is unused.
            SHARE_TYPE_UNSPECIFIED = 494_771_730

            # Shared-reservation is open to specific projects
            SPECIFIC_PROJECTS = 347_838_695
          end
        end

        # Config for each project in the share settings.
        # @!attribute [rw] project_id
        #   @return [::String]
        #     The project ID, should be same as the key of this project config in the parent map.
        class ShareSettingsProjectConfig
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A set of Shielded Instance options.
        # @!attribute [rw] enable_integrity_monitoring
        #   @return [::Boolean]
        #     Defines whether the instance has integrity monitoring enabled. Enabled by default.
        # @!attribute [rw] enable_secure_boot
        #   @return [::Boolean]
        #     Defines whether the instance has Secure Boot enabled. Disabled by default.
        # @!attribute [rw] enable_vtpm
        #   @return [::Boolean]
        #     Defines whether the instance has the vTPM enabled. Enabled by default.
        class ShieldedInstanceConfig
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A Shielded Instance Identity.
        # @!attribute [rw] encryption_key
        #   @return [::Google::Cloud::Compute::V1::ShieldedInstanceIdentityEntry]
        #     An Endorsement Key (EK) made by the RSA 2048 algorithm issued to the Shielded Instance's vTPM.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#shieldedInstanceIdentity for shielded Instance identity entry.
        # @!attribute [rw] signing_key
        #   @return [::Google::Cloud::Compute::V1::ShieldedInstanceIdentityEntry]
        #     An Attestation Key (AK) made by the RSA 2048 algorithm issued to the Shielded Instance's vTPM.
        class ShieldedInstanceIdentity
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A Shielded Instance Identity Entry.
        # @!attribute [rw] ek_cert
        #   @return [::String]
        #     A PEM-encoded X.509 certificate. This field can be empty.
        # @!attribute [rw] ek_pub
        #   @return [::String]
        #     A PEM-encoded public key.
        class ShieldedInstanceIdentityEntry
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # The policy describes the baseline against which Instance boot integrity is measured.
        # @!attribute [rw] update_auto_learn_policy
        #   @return [::Boolean]
        #     Updates the integrity policy baseline using the measurements from the VM instance's most recent boot.
        class ShieldedInstanceIntegrityPolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a customer-supplied Signing Key used by Cloud CDN Signed URLs
        # @!attribute [rw] key_name
        #   @return [::String]
        #     Name of the key. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] key_value
        #   @return [::String]
        #     128-bit key value used for signing the URL. The key value must be a valid RFC 4648 Section 5 base64url encoded string.
        class SignedUrlKey
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.SimulateMaintenanceEvent. See the method description for details.
        # @!attribute [rw] instance
        #   @return [::String]
        #     Name of the instance scoping this request.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class SimulateMaintenanceEventInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NodeGroups.SimulateMaintenanceEvent. See the method description for details.
        # @!attribute [rw] node_group
        #   @return [::String]
        #     Name of the NodeGroup resource whose nodes will go under maintenance simulation.
        # @!attribute [rw] node_groups_simulate_maintenance_event_request_resource
        #   @return [::Google::Cloud::Compute::V1::NodeGroupsSimulateMaintenanceEventRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class SimulateMaintenanceEventNodeGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a Persistent Disk Snapshot resource. You can use snapshots to back up data on a regular interval. For more information, read Creating persistent disk snapshots.
        # @!attribute [rw] architecture
        #   @return [::String]
        #     [Output Only] The architecture of the snapshot. Valid values are ARM64 or X86_64.
        #     Check the Architecture enum for the list of possible values.
        # @!attribute [rw] auto_created
        #   @return [::Boolean]
        #     [Output Only] Set to true if snapshots are automatically created by applying resource policy on the target disk.
        # @!attribute [rw] chain_name
        #   @return [::String]
        #     Creates the new snapshot in the snapshot chain labeled with the specified name. The chain name must be 1-63 characters long and comply with RFC1035. This is an uncommon option only for advanced service owners who needs to create separate snapshot chains, for example, for chargeback tracking. When you describe your snapshot resource, this field is visible only if it has a non-empty value.
        # @!attribute [rw] creation_size_bytes
        #   @return [::Integer]
        #     [Output Only] Size in bytes of the snapshot at creation time.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] disk_size_gb
        #   @return [::Integer]
        #     [Output Only] Size of the source disk, specified in GB.
        # @!attribute [rw] download_bytes
        #   @return [::Integer]
        #     [Output Only] Number of bytes downloaded to restore a snapshot to a disk.
        # @!attribute [rw] guest_os_features
        #   @return [::Array<::Google::Cloud::Compute::V1::GuestOsFeature>]
        #     [Output Only] A list of features to enable on the guest operating system. Applicable only for bootable images. Read Enabling guest operating system features to see a list of available options.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#snapshot for Snapshot resources.
        # @!attribute [rw] label_fingerprint
        #   @return [::String]
        #     A fingerprint for the labels being applied to this snapshot, which is essentially a hash of the labels set used for optimistic locking. The fingerprint is initially generated by Compute Engine and changes after every request to modify or update labels. You must always provide an up-to-date fingerprint hash in order to update or change labels, otherwise the request will fail with error 412 conditionNotMet. To see the latest fingerprint, make a get() request to retrieve a snapshot.
        # @!attribute [rw] labels
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     Labels to apply to this snapshot. These can be later modified by the setLabels method. Label values may be empty.
        # @!attribute [rw] license_codes
        #   @return [::Array<::Integer>]
        #     [Output Only] Integer license codes indicating which licenses are attached to this snapshot.
        # @!attribute [rw] licenses
        #   @return [::Array<::String>]
        #     [Output Only] A list of public visible licenses that apply to this snapshot. This can be because the original image had licenses attached (such as a Windows image).
        # @!attribute [rw] location_hint
        #   @return [::String]
        #     An opaque location hint used to place the snapshot close to other resources. This field is for use by internal tools that use the public API.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource; provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] satisfies_pzs
        #   @return [::Boolean]
        #     [Output Only] Reserved for future use.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] snapshot_encryption_key
        #   @return [::Google::Cloud::Compute::V1::CustomerEncryptionKey]
        #     Encrypts the snapshot using a customer-supplied encryption key. After you encrypt a snapshot using a customer-supplied key, you must provide the same key if you use the snapshot later. For example, you must provide the encryption key when you create a disk from the encrypted snapshot in a future request. Customer-supplied encryption keys do not protect access to metadata of the snapshot. If you do not provide an encryption key when creating the snapshot, then the snapshot will be encrypted using an automatically generated key and you do not need to provide a key to use the snapshot later.
        # @!attribute [rw] snapshot_type
        #   @return [::String]
        #     Indicates the type of the snapshot.
        #     Check the SnapshotType enum for the list of possible values.
        # @!attribute [rw] source_disk
        #   @return [::String]
        #     The source disk used to create this snapshot.
        # @!attribute [rw] source_disk_encryption_key
        #   @return [::Google::Cloud::Compute::V1::CustomerEncryptionKey]
        #     The customer-supplied encryption key of the source disk. Required if the source disk is protected by a customer-supplied encryption key.
        # @!attribute [rw] source_disk_for_recovery_checkpoint
        #   @return [::String]
        #     The source disk whose recovery checkpoint will be used to create this snapshot.
        # @!attribute [rw] source_disk_id
        #   @return [::String]
        #     [Output Only] The ID value of the disk used to create this snapshot. This value may be used to determine whether the snapshot was taken from the current or a previous instance of a given disk name.
        # @!attribute [rw] source_snapshot_schedule_policy
        #   @return [::String]
        #     [Output Only] URL of the resource policy which created this scheduled snapshot.
        # @!attribute [rw] source_snapshot_schedule_policy_id
        #   @return [::String]
        #     [Output Only] ID of the resource policy which created this scheduled snapshot.
        # @!attribute [rw] status
        #   @return [::String]
        #     [Output Only] The status of the snapshot. This can be CREATING, DELETING, FAILED, READY, or UPLOADING.
        #     Check the Status enum for the list of possible values.
        # @!attribute [rw] storage_bytes
        #   @return [::Integer]
        #     [Output Only] A size of the storage used by the snapshot. As snapshots share storage, this number is expected to change with snapshot creation/deletion.
        # @!attribute [rw] storage_bytes_status
        #   @return [::String]
        #     [Output Only] An indicator whether storageBytes is in a stable state or it is being adjusted as a result of shared storage reallocation. This status can either be UPDATING, meaning the size of the snapshot is being updated, or UP_TO_DATE, meaning the size of the snapshot is up-to-date.
        #     Check the StorageBytesStatus enum for the list of possible values.
        # @!attribute [rw] storage_locations
        #   @return [::Array<::String>]
        #     Cloud Storage bucket storage location of the snapshot (regional or multi-regional).
        class Snapshot
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class LabelsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # [Output Only] The architecture of the snapshot. Valid values are ARM64 or X86_64.
          module Architecture
            # A value indicating that the enum field is not set.
            UNDEFINED_ARCHITECTURE = 0

            # Default value indicating Architecture is not set.
            ARCHITECTURE_UNSPECIFIED = 394_750_507

            # Machines with architecture ARM64
            ARM64 = 62_547_450

            # Machines with architecture X86_64
            X86_64 = 425_300_551
          end

          # Indicates the type of the snapshot.
          module SnapshotType
            # A value indicating that the enum field is not set.
            UNDEFINED_SNAPSHOT_TYPE = 0

            ARCHIVE = 506_752_162

            STANDARD = 484_642_493
          end

          # [Output Only] The status of the snapshot. This can be CREATING, DELETING, FAILED, READY, or UPLOADING.
          module Status
            # A value indicating that the enum field is not set.
            UNDEFINED_STATUS = 0

            # Snapshot creation is in progress.
            CREATING = 455_564_985

            # Snapshot is currently being deleted.
            DELETING = 528_602_024

            # Snapshot creation failed.
            FAILED = 455_706_685

            # Snapshot has been created successfully.
            READY = 77_848_963

            # Snapshot is being uploaded.
            UPLOADING = 267_603_489
          end

          # [Output Only] An indicator whether storageBytes is in a stable state or it is being adjusted as a result of shared storage reallocation. This status can either be UPDATING, meaning the size of the snapshot is being updated, or UP_TO_DATE, meaning the size of the snapshot is up-to-date.
          module StorageBytesStatus
            # A value indicating that the enum field is not set.
            UNDEFINED_STORAGE_BYTES_STATUS = 0

            UPDATING = 494_614_342

            UP_TO_DATE = 101_306_702
          end
        end

        # Contains a list of Snapshot resources.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::Snapshot>]
        #     A list of Snapshot resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     Type of resource.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class SnapshotList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] storage_location
        #   @return [::Google::Cloud::Compute::V1::SnapshotSettingsStorageLocationSettings]
        #     Policy of which storage location is going to be resolved, and additional data that particularizes how the policy is going to be carried out.
        class SnapshotSettings
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] locations
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::SnapshotSettingsStorageLocationSettingsStorageLocationPreference}]
        #     When the policy is SPECIFIC_LOCATIONS, snapshots will be stored in the locations listed in this field. Keys are GCS bucket locations.
        # @!attribute [rw] policy
        #   @return [::String]
        #     The chosen location policy.
        #     Check the Policy enum for the list of possible values.
        class SnapshotSettingsStorageLocationSettings
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::SnapshotSettingsStorageLocationSettingsStorageLocationPreference]
          class LocationsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # The chosen location policy.
          module Policy
            # A value indicating that the enum field is not set.
            UNDEFINED_POLICY = 0

            # Store snapshot in the same region as with the originating disk. No additional parameters are needed.
            LOCAL_REGION = 403_535_464

            # Store snapshot to the nearest multi region GCS bucket, relative to the originating disk. No additional parameters are needed.
            NEAREST_MULTI_REGION = 212_467_515

            # Store snapshot in the specific locations, as specified by the user. The list of regions to store must be defined under the `locations` field.
            SPECIFIC_LOCATIONS = 280_093_809

            STORAGE_LOCATION_POLICY_UNSPECIFIED = 250_644_592
          end
        end

        # A structure for specifying storage locations.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the location. It should be one of the GCS buckets.
        class SnapshotSettingsStorageLocationSettingsStorageLocationPreference
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] disk_encryption_key
        #   @return [::Google::Cloud::Compute::V1::CustomerEncryptionKey]
        #     The customer-supplied encryption key of the source disk. Required if the source disk is protected by a customer-supplied encryption key.
        # @!attribute [rw] source_disk
        #   @return [::String]
        #     URL of the disk attached to the source instance. This can be a full or valid partial URL. For example, the following are valid values: - https://www.googleapis.com/compute/v1/projects/project/zones/zone /disks/disk - projects/project/zones/zone/disks/disk - zones/zone/disks/disk
        class SourceDiskEncryptionKey
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A specification of the parameters to use when creating the instance template from a source instance.
        # @!attribute [rw] disk_configs
        #   @return [::Array<::Google::Cloud::Compute::V1::DiskInstantiationConfig>]
        #     Attached disks configuration. If not provided, defaults are applied: For boot disk and any other R/W disks, the source images for each disk will be used. For read-only disks, they will be attached in read-only mode. Local SSD disks will be created as blank volumes.
        class SourceInstanceParams
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # DEPRECATED: Please use compute#instanceProperties instead. New properties will not be added to this field.
        # @!attribute [rw] can_ip_forward
        #   @return [::Boolean]
        #     Enables instances created based on this machine image to send packets with source IP addresses other than their own and receive packets with destination IP addresses other than their own. If these instances will be used as an IP gateway or it will be set as the next-hop in a Route resource, specify true. If unsure, leave this set to false. See the Enable IP forwarding documentation for more information.
        # @!attribute [rw] deletion_protection
        #   @return [::Boolean]
        #     Whether the instance created from this machine image should be protected against deletion.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional text description for the instances that are created from this machine image.
        # @!attribute [rw] disks
        #   @return [::Array<::Google::Cloud::Compute::V1::SavedAttachedDisk>]
        #     An array of disks that are associated with the instances that are created from this machine image.
        # @!attribute [rw] guest_accelerators
        #   @return [::Array<::Google::Cloud::Compute::V1::AcceleratorConfig>]
        #     A list of guest accelerator cards' type and count to use for instances created from this machine image.
        # @!attribute [rw] key_revocation_action_type
        #   @return [::String]
        #     KeyRevocationActionType of the instance. Supported options are "STOP" and "NONE". The default value is "NONE" if it is not specified.
        #     Check the KeyRevocationActionType enum for the list of possible values.
        # @!attribute [rw] labels
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     Labels to apply to instances that are created from this machine image.
        # @!attribute [rw] machine_type
        #   @return [::String]
        #     The machine type to use for instances that are created from this machine image.
        # @!attribute [rw] metadata
        #   @return [::Google::Cloud::Compute::V1::Metadata]
        #     The metadata key/value pairs to assign to instances that are created from this machine image. These pairs can consist of custom metadata or predefined keys. See Project and instance metadata for more information.
        # @!attribute [rw] min_cpu_platform
        #   @return [::String]
        #     Minimum cpu/platform to be used by instances created from this machine image. The instance may be scheduled on the specified or newer cpu/platform. Applicable values are the friendly names of CPU platforms, such as minCpuPlatform: "Intel Haswell" or minCpuPlatform: "Intel Sandy Bridge". For more information, read Specifying a Minimum CPU Platform.
        # @!attribute [rw] network_interfaces
        #   @return [::Array<::Google::Cloud::Compute::V1::NetworkInterface>]
        #     An array of network access configurations for this interface.
        # @!attribute [rw] scheduling
        #   @return [::Google::Cloud::Compute::V1::Scheduling]
        #     Specifies the scheduling options for the instances that are created from this machine image.
        # @!attribute [rw] service_accounts
        #   @return [::Array<::Google::Cloud::Compute::V1::ServiceAccount>]
        #     A list of service accounts with specified scopes. Access tokens for these service accounts are available to the instances that are created from this machine image. Use metadata queries to obtain the access tokens for these instances.
        # @!attribute [rw] tags
        #   @return [::Google::Cloud::Compute::V1::Tags]
        #     A list of tags to apply to the instances that are created from this machine image. The tags identify valid sources or targets for network firewalls. The setTags method can modify this list of tags. Each tag within the list must comply with RFC1035.
        class SourceInstanceProperties
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class LabelsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # KeyRevocationActionType of the instance. Supported options are "STOP" and "NONE". The default value is "NONE" if it is not specified.
          module KeyRevocationActionType
            # A value indicating that the enum field is not set.
            UNDEFINED_KEY_REVOCATION_ACTION_TYPE = 0

            # Default value. This value is unused.
            KEY_REVOCATION_ACTION_TYPE_UNSPECIFIED = 467_110_106

            # Indicates user chose no operation.
            NONE = 2_402_104

            # Indicates user chose to opt for VM shutdown on key revocation.
            STOP = 2_555_906
          end
        end

        # Represents an SSL certificate resource. Google Compute Engine has two SSL certificate resources: * [Global](/compute/docs/reference/rest/v1/sslCertificates) * [Regional](/compute/docs/reference/rest/v1/regionSslCertificates) The global SSL certificates (sslCertificates) are used by: - Global external Application Load Balancers - Classic Application Load Balancers - Proxy Network Load Balancers (with target SSL proxies) The regional SSL certificates (regionSslCertificates) are used by: - Regional external Application Load Balancers - Regional internal Application Load Balancers Optionally, certificate file contents that you upload can contain a set of up to five PEM-encoded certificates. The API call creates an object (sslCertificate) that holds this data. You can use SSL keys and certificates to secure connections to a load balancer. For more information, read Creating and using SSL certificates, SSL certificates quotas and limits, and Troubleshooting SSL certificates.
        # @!attribute [rw] certificate
        #   @return [::String]
        #     A value read into memory from a certificate file. The certificate file must be in PEM format. The certificate chain must be no greater than 5 certs long. The chain must include at least one intermediate cert.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] expire_time
        #   @return [::String]
        #     [Output Only] Expire time of the certificate. RFC3339
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#sslCertificate for SSL certificates.
        # @!attribute [rw] managed
        #   @return [::Google::Cloud::Compute::V1::SslCertificateManagedSslCertificate]
        #     Configuration and status of a managed SSL certificate.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] private_key
        #   @return [::String]
        #     A value read into memory from a write-only private key file. The private key file must be in PEM format. For security, only insert requests include this field.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] URL of the region where the regional SSL Certificate resides. This field is not applicable to global SSL Certificate.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output only] Server-defined URL for the resource.
        # @!attribute [rw] self_managed
        #   @return [::Google::Cloud::Compute::V1::SslCertificateSelfManagedSslCertificate]
        #     Configuration and status of a self-managed SSL certificate.
        # @!attribute [rw] subject_alternative_names
        #   @return [::Array<::String>]
        #     [Output Only] Domains associated with the certificate via Subject Alternative Name.
        # @!attribute [rw] type
        #   @return [::String]
        #     (Optional) Specifies the type of SSL certificate, either "SELF_MANAGED" or "MANAGED". If not specified, the certificate is self-managed and the fields certificate and private_key are used.
        #     Check the Type enum for the list of possible values.
        class SslCertificate
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # (Optional) Specifies the type of SSL certificate, either "SELF_MANAGED" or "MANAGED". If not specified, the certificate is self-managed and the fields certificate and private_key are used.
          module Type
            # A value indicating that the enum field is not set.
            UNDEFINED_TYPE = 0

            # Google-managed SSLCertificate.
            MANAGED = 479_501_183

            # Certificate uploaded by user.
            SELF_MANAGED = 434_437_516

            TYPE_UNSPECIFIED = 437_714_322
          end
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::SslCertificatesScopedList}]
        #     A list of SslCertificatesScopedList resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#sslCertificateAggregatedList for lists of SSL Certificates.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class SslCertificateAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::SslCertificatesScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # Contains a list of SslCertificate resources.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::SslCertificate>]
        #     A list of SslCertificate resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     Type of resource.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class SslCertificateList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Configuration and status of a managed SSL certificate.
        # @!attribute [rw] domain_status
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     [Output only] Detailed statuses of the domains specified for managed certificate resource.
        # @!attribute [rw] domains
        #   @return [::Array<::String>]
        #     The domains for which a managed SSL certificate will be generated. Each Google-managed SSL certificate supports up to the [maximum number of domains per Google-managed SSL certificate](/load-balancing/docs/quotas#ssl_certificates).
        # @!attribute [rw] status
        #   @return [::String]
        #     [Output only] Status of the managed certificate resource.
        #     Check the Status enum for the list of possible values.
        class SslCertificateManagedSslCertificate
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class DomainStatusEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # [Output only] Status of the managed certificate resource.
          module Status
            # A value indicating that the enum field is not set.
            UNDEFINED_STATUS = 0

            # The certificate management is working, and a certificate has been provisioned.
            ACTIVE = 314_733_318

            MANAGED_CERTIFICATE_STATUS_UNSPECIFIED = 474_800_850

            # The certificate management is working. GCP will attempt to provision the first certificate.
            PROVISIONING = 290_896_621

            # Certificate provisioning failed due to an issue with the DNS or load balancing configuration. For details of which domain failed, consult domain_status field.
            PROVISIONING_FAILED = 76_813_775

            # Certificate provisioning failed due to an issue with the DNS or load balancing configuration. It won't be retried. To try again delete and create a new managed SslCertificate resource. For details of which domain failed, consult domain_status field.
            PROVISIONING_FAILED_PERMANENTLY = 275_036_203

            # Renewal of the certificate has failed due to an issue with the DNS or load balancing configuration. The existing cert is still serving; however, it will expire shortly. To provision a renewed certificate, delete and create a new managed SslCertificate resource. For details on which domain failed, consult domain_status field.
            RENEWAL_FAILED = 434_659_076
          end
        end

        # Configuration and status of a self-managed SSL certificate.
        # @!attribute [rw] certificate
        #   @return [::String]
        #     A local certificate file. The certificate must be in PEM format. The certificate chain must be no greater than 5 certs long. The chain must include at least one intermediate cert.
        # @!attribute [rw] private_key
        #   @return [::String]
        #     A write-only private key in PEM format. Only insert requests will include this field.
        class SslCertificateSelfManagedSslCertificate
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] ssl_certificates
        #   @return [::Array<::Google::Cloud::Compute::V1::SslCertificate>]
        #     List of SslCertificates contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     Informational warning which replaces the list of backend services when the list is empty.
        class SslCertificatesScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] etag
        #   @return [::String]
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::SslPoliciesScopedList}]
        #     A list of SslPoliciesScopedList resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#sslPolicyAggregatedList for lists of SSL Policies.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class SslPoliciesAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::SslPoliciesScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::SslPolicy>]
        #     A list of SslPolicy resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#sslPoliciesList for lists of sslPolicies.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class SslPoliciesList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] features
        #   @return [::Array<::String>]
        class SslPoliciesListAvailableFeaturesResponse
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] ssl_policies
        #   @return [::Array<::Google::Cloud::Compute::V1::SslPolicy>]
        #     A list of SslPolicies contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     Informational warning which replaces the list of SSL policies when the list is empty.
        class SslPoliciesScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents an SSL Policy resource. Use SSL policies to control the SSL features, such as versions and cipher suites, offered by an HTTPS or SSL Proxy load balancer. For more information, read SSL Policy Concepts.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] custom_features
        #   @return [::Array<::String>]
        #     A list of features enabled when the selected profile is CUSTOM. The method returns the set of features that can be specified in this list. This field must be empty if the profile is not CUSTOM.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] enabled_features
        #   @return [::Array<::String>]
        #     [Output Only] The list of features enabled in the SSL policy.
        # @!attribute [rw] fingerprint
        #   @return [::String]
        #     Fingerprint of this resource. A hash of the contents stored in this object. This field is used in optimistic locking. This field will be ignored when inserting a SslPolicy. An up-to-date fingerprint must be provided in order to update the SslPolicy, otherwise the request will fail with error 412 conditionNotMet. To see the latest fingerprint, make a get() request to retrieve an SslPolicy.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output only] Type of the resource. Always compute#sslPolicyfor SSL policies.
        # @!attribute [rw] min_tls_version
        #   @return [::String]
        #     The minimum version of SSL protocol that can be used by the clients to establish a connection with the load balancer. This can be one of TLS_1_0, TLS_1_1, TLS_1_2.
        #     Check the MinTlsVersion enum for the list of possible values.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] profile
        #   @return [::String]
        #     Profile specifies the set of SSL features that can be used by the load balancer when negotiating SSL with clients. This can be one of COMPATIBLE, MODERN, RESTRICTED, or CUSTOM. If using CUSTOM, the set of SSL features to enable must be specified in the customFeatures field.
        #     Check the Profile enum for the list of possible values.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] URL of the region where the regional SSL policy resides. This field is not applicable to global SSL policies.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] warnings
        #   @return [::Array<::Google::Cloud::Compute::V1::Warnings>]
        #     [Output Only] If potential misconfigurations are detected for this SSL policy, this field will be populated with warning messages.
        class SslPolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The minimum version of SSL protocol that can be used by the clients to establish a connection with the load balancer. This can be one of TLS_1_0, TLS_1_1, TLS_1_2.
          module MinTlsVersion
            # A value indicating that the enum field is not set.
            UNDEFINED_MIN_TLS_VERSION = 0

            # TLS 1.0
            TLS_1_0 = 33_116_734

            # TLS 1.1
            TLS_1_1 = 33_116_735

            # TLS 1.2
            TLS_1_2 = 33_116_736
          end

          # Profile specifies the set of SSL features that can be used by the load balancer when negotiating SSL with clients. This can be one of COMPATIBLE, MODERN, RESTRICTED, or CUSTOM. If using CUSTOM, the set of SSL features to enable must be specified in the customFeatures field.
          module Profile
            # A value indicating that the enum field is not set.
            UNDEFINED_PROFILE = 0

            # Compatible profile. Allows the broadset set of clients, even those which support only out-of-date SSL features to negotiate with the load balancer.
            COMPATIBLE = 179_357_396

            # Custom profile. Allow only the set of allowed SSL features specified in the customFeatures field.
            CUSTOM = 388_595_569

            # Modern profile. Supports a wide set of SSL features, allowing modern clients to negotiate SSL with the load balancer.
            MODERN = 132_013_855

            # Restricted profile. Supports a reduced set of SSL features, intended to meet stricter compliance requirements.
            RESTRICTED = 261_551_195
          end
        end

        # @!attribute [rw] ssl_policy
        #   @return [::String]
        #     URL of the SSL policy resource. Set this to empty string to clear any existing SSL policy associated with the target proxy resource.
        class SslPolicyReference
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Disks.StartAsyncReplication. See the method description for details.
        # @!attribute [rw] disk
        #   @return [::String]
        #     The name of the persistent disk.
        # @!attribute [rw] disks_start_async_replication_request_resource
        #   @return [::Google::Cloud::Compute::V1::DisksStartAsyncReplicationRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class StartAsyncReplicationDiskRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionDisks.StartAsyncReplication. See the method description for details.
        # @!attribute [rw] disk
        #   @return [::String]
        #     The name of the persistent disk.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] region_disks_start_async_replication_request_resource
        #   @return [::Google::Cloud::Compute::V1::RegionDisksStartAsyncReplicationRequest]
        #     The body resource for this request
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class StartAsyncReplicationRegionDiskRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.Start. See the method description for details.
        # @!attribute [rw] instance
        #   @return [::String]
        #     Name of the instance resource to start.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class StartInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.StartWithEncryptionKey. See the method description for details.
        # @!attribute [rw] instance
        #   @return [::String]
        #     Name of the instance resource to start.
        # @!attribute [rw] instances_start_with_encryption_key_request_resource
        #   @return [::Google::Cloud::Compute::V1::InstancesStartWithEncryptionKeyRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class StartWithEncryptionKeyInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] preserved_state
        #   @return [::Google::Cloud::Compute::V1::StatefulPolicyPreservedState]
        class StatefulPolicy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Configuration of preserved resources.
        # @!attribute [rw] disks
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::StatefulPolicyPreservedStateDiskDevice}]
        #     Disks created on the instances that will be preserved on instance delete, update, etc. This map is keyed with the device names of the disks.
        # @!attribute [rw] external_i_ps
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::StatefulPolicyPreservedStateNetworkIp}]
        #     External network IPs assigned to the instances that will be preserved on instance delete, update, etc. This map is keyed with the network interface name.
        # @!attribute [rw] internal_i_ps
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::StatefulPolicyPreservedStateNetworkIp}]
        #     Internal network IPs assigned to the instances that will be preserved on instance delete, update, etc. This map is keyed with the network interface name.
        class StatefulPolicyPreservedState
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::StatefulPolicyPreservedStateDiskDevice]
          class DisksEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::StatefulPolicyPreservedStateNetworkIp]
          class ExternalIPsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::StatefulPolicyPreservedStateNetworkIp]
          class InternalIPsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # @!attribute [rw] auto_delete
        #   @return [::String]
        #     These stateful disks will never be deleted during autohealing, update or VM instance recreate operations. This flag is used to configure if the disk should be deleted after it is no longer used by the group, e.g. when the given instance or the whole group is deleted. Note: disks attached in READ_ONLY mode cannot be auto-deleted.
        #     Check the AutoDelete enum for the list of possible values.
        class StatefulPolicyPreservedStateDiskDevice
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # These stateful disks will never be deleted during autohealing, update or VM instance recreate operations. This flag is used to configure if the disk should be deleted after it is no longer used by the group, e.g. when the given instance or the whole group is deleted. Note: disks attached in READ_ONLY mode cannot be auto-deleted.
          module AutoDelete
            # A value indicating that the enum field is not set.
            UNDEFINED_AUTO_DELETE = 0

            NEVER = 74_175_084

            ON_PERMANENT_INSTANCE_DELETION = 95_727_719
          end
        end

        # @!attribute [rw] auto_delete
        #   @return [::String]
        #     These stateful IPs will never be released during autohealing, update or VM instance recreate operations. This flag is used to configure if the IP reservation should be deleted after it is no longer used by the group, e.g. when the given instance or the whole group is deleted.
        #     Check the AutoDelete enum for the list of possible values.
        class StatefulPolicyPreservedStateNetworkIp
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # These stateful IPs will never be released during autohealing, update or VM instance recreate operations. This flag is used to configure if the IP reservation should be deleted after it is no longer used by the group, e.g. when the given instance or the whole group is deleted.
          module AutoDelete
            # A value indicating that the enum field is not set.
            UNDEFINED_AUTO_DELETE = 0

            NEVER = 74_175_084

            ON_PERMANENT_INSTANCE_DELETION = 95_727_719
          end
        end

        # The `Status` type defines a logical error model that is suitable for different programming environments, including REST APIs and RPC APIs. It is used by [gRPC](https://github.com/grpc). Each `Status` message contains three pieces of data: error code, error message, and error details. You can find out more about this error model and how to work with it in the [API Design Guide](https://cloud.google.com/apis/design/errors).
        # @!attribute [rw] code
        #   @return [::Integer]
        #     The status code, which should be an enum value of google.rpc.Code.
        # @!attribute [rw] details
        #   @return [::Array<::Google::Protobuf::Any>]
        #     A list of messages that carry the error details. There is a common set of message types for APIs to use.
        # @!attribute [rw] message
        #   @return [::String]
        #     A developer-facing error message, which should be in English. Any user-facing error message should be localized and sent in the google.rpc.Status.details field, or localized by the client.
        class Status
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Disks.StopAsyncReplication. See the method description for details.
        # @!attribute [rw] disk
        #   @return [::String]
        #     The name of the persistent disk.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class StopAsyncReplicationDiskRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionDisks.StopAsyncReplication. See the method description for details.
        # @!attribute [rw] disk
        #   @return [::String]
        #     The name of the persistent disk.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class StopAsyncReplicationRegionDiskRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Disks.StopGroupAsyncReplication. See the method description for details.
        # @!attribute [rw] disks_stop_group_async_replication_resource_resource
        #   @return [::Google::Cloud::Compute::V1::DisksStopGroupAsyncReplicationResource]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request. This must be the zone of the primary or secondary disks in the consistency group.
        class StopGroupAsyncReplicationDiskRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionDisks.StopGroupAsyncReplication. See the method description for details.
        # @!attribute [rw] disks_stop_group_async_replication_resource_resource
        #   @return [::Google::Cloud::Compute::V1::DisksStopGroupAsyncReplicationResource]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request. This must be the region of the primary or secondary disks in the consistency group.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class StopGroupAsyncReplicationRegionDiskRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.Stop. See the method description for details.
        # @!attribute [rw] discard_local_ssd
        #   @return [::Boolean]
        #     If true, discard the contents of any attached localSSD partitions. Default value is false.
        # @!attribute [rw] instance
        #   @return [::String]
        #     Name of the instance resource to stop.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class StopInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a Subnetwork resource. A subnetwork (also known as a subnet) is a logical partition of a Virtual Private Cloud network with one primary IP range and zero or more secondary IP ranges. For more information, read Virtual Private Cloud (VPC) Network.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource. This field can be set only at resource creation time.
        # @!attribute [rw] enable_flow_logs
        #   @return [::Boolean]
        #     Whether to enable flow logging for this subnetwork. If this field is not explicitly set, it will not appear in get listings. If not set the default behavior is determined by the org policy, if there is no org policy specified, then it will default to disabled. This field isn't supported if the subnet purpose field is set to REGIONAL_MANAGED_PROXY.
        # @!attribute [rw] external_ipv6_prefix
        #   @return [::String]
        #     The external IPv6 address range that is owned by this subnetwork.
        # @!attribute [rw] fingerprint
        #   @return [::String]
        #     Fingerprint of this resource. A hash of the contents stored in this object. This field is used in optimistic locking. This field will be ignored when inserting a Subnetwork. An up-to-date fingerprint must be provided in order to update the Subnetwork, otherwise the request will fail with error 412 conditionNotMet. To see the latest fingerprint, make a get() request to retrieve a Subnetwork.
        # @!attribute [rw] gateway_address
        #   @return [::String]
        #     [Output Only] The gateway address for default routes to reach destination addresses outside this subnetwork.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] internal_ipv6_prefix
        #   @return [::String]
        #     [Output Only] The internal IPv6 address range that is assigned to this subnetwork.
        # @!attribute [rw] ip_cidr_range
        #   @return [::String]
        #     The range of internal addresses that are owned by this subnetwork. Provide this property when you create the subnetwork. For example, 10.0.0.0/8 or 100.64.0.0/10. Ranges must be unique and non-overlapping within a network. Only IPv4 is supported. This field is set at resource creation time. The range can be any range listed in the Valid ranges list. The range can be expanded after creation using expandIpCidrRange.
        # @!attribute [rw] ipv6_access_type
        #   @return [::String]
        #     The access type of IPv6 address this subnet holds. It's immutable and can only be specified during creation or the first time the subnet is updated into IPV4_IPV6 dual stack.
        #     Check the Ipv6AccessType enum for the list of possible values.
        # @!attribute [rw] ipv6_cidr_range
        #   @return [::String]
        #     [Output Only] This field is for internal use.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#subnetwork for Subnetwork resources.
        # @!attribute [rw] log_config
        #   @return [::Google::Cloud::Compute::V1::SubnetworkLogConfig]
        #     This field denotes the VPC flow logging options for this subnetwork. If logging is enabled, logs are exported to Cloud Logging.
        # @!attribute [rw] name
        #   @return [::String]
        #     The name of the resource, provided by the client when initially creating the resource. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] network
        #   @return [::String]
        #     The URL of the network to which this subnetwork belongs, provided by the client when initially creating the subnetwork. This field can be set only at resource creation time.
        # @!attribute [rw] private_ip_google_access
        #   @return [::Boolean]
        #     Whether the VMs in this subnet can access Google services without assigned external IP addresses. This field can be both set at resource creation time and updated using setPrivateIpGoogleAccess.
        # @!attribute [rw] private_ipv6_google_access
        #   @return [::String]
        #     This field is for internal use. This field can be both set at resource creation time and updated using patch.
        #     Check the PrivateIpv6GoogleAccess enum for the list of possible values.
        # @!attribute [rw] purpose
        #   @return [::String]
        #     The purpose of the resource. This field can be either PRIVATE, REGIONAL_MANAGED_PROXY, PRIVATE_SERVICE_CONNECT, or INTERNAL_HTTPS_LOAD_BALANCER. PRIVATE is the default purpose for user-created subnets or subnets that are automatically created in auto mode networks. A subnet with purpose set to REGIONAL_MANAGED_PROXY is a user-created subnetwork that is reserved for regional Envoy-based load balancers. A subnet with purpose set to PRIVATE_SERVICE_CONNECT is used to publish services using Private Service Connect. A subnet with purpose set to INTERNAL_HTTPS_LOAD_BALANCER is a proxy-only subnet that can be used only by regional internal HTTP(S) load balancers. Note that REGIONAL_MANAGED_PROXY is the preferred setting for all regional Envoy load balancers. If unspecified, the subnet purpose defaults to PRIVATE. The enableFlowLogs field isn't supported if the subnet purpose field is set to REGIONAL_MANAGED_PROXY.
        #     Check the Purpose enum for the list of possible values.
        # @!attribute [rw] region
        #   @return [::String]
        #     URL of the region where the Subnetwork resides. This field can be set only at resource creation time.
        # @!attribute [rw] role
        #   @return [::String]
        #     The role of subnetwork. Currently, this field is only used when purpose = REGIONAL_MANAGED_PROXY. The value can be set to ACTIVE or BACKUP. An ACTIVE subnetwork is one that is currently being used for Envoy-based load balancers in a region. A BACKUP subnetwork is one that is ready to be promoted to ACTIVE or is currently draining. This field can be updated with a patch request.
        #     Check the Role enum for the list of possible values.
        # @!attribute [rw] secondary_ip_ranges
        #   @return [::Array<::Google::Cloud::Compute::V1::SubnetworkSecondaryRange>]
        #     An array of configurations for secondary IP ranges for VM instances contained in this subnetwork. The primary IP of such VM must belong to the primary ipCidrRange of the subnetwork. The alias IPs may belong to either primary or secondary ranges. This field can be updated with a patch request.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] stack_type
        #   @return [::String]
        #     The stack type for the subnet. If set to IPV4_ONLY, new VMs in the subnet are assigned IPv4 addresses only. If set to IPV4_IPV6, new VMs in the subnet can be assigned both IPv4 and IPv6 addresses. If not specified, IPV4_ONLY is used. This field can be both set at resource creation time and updated using patch.
        #     Check the StackType enum for the list of possible values.
        # @!attribute [rw] state
        #   @return [::String]
        #     [Output Only] The state of the subnetwork, which can be one of the following values: READY: Subnetwork is created and ready to use DRAINING: only applicable to subnetworks that have the purpose set to INTERNAL_HTTPS_LOAD_BALANCER and indicates that connections to the load balancer are being drained. A subnetwork that is draining cannot be used or modified until it reaches a status of READY
        #     Check the State enum for the list of possible values.
        class Subnetwork
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The access type of IPv6 address this subnet holds. It's immutable and can only be specified during creation or the first time the subnet is updated into IPV4_IPV6 dual stack.
          module Ipv6AccessType
            # A value indicating that the enum field is not set.
            UNDEFINED_IPV6_ACCESS_TYPE = 0

            # VMs on this subnet will be assigned IPv6 addresses that are accessible via the Internet, as well as the VPC network.
            EXTERNAL = 35_607_499

            # VMs on this subnet will be assigned IPv6 addresses that are only accessible over the VPC network.
            INTERNAL = 279_295_677

            UNSPECIFIED_IPV6_ACCESS_TYPE = 313_080_613
          end

          # This field is for internal use. This field can be both set at resource creation time and updated using patch.
          module PrivateIpv6GoogleAccess
            # A value indicating that the enum field is not set.
            UNDEFINED_PRIVATE_IPV6_GOOGLE_ACCESS = 0

            # Disable private IPv6 access to/from Google services.
            DISABLE_GOOGLE_ACCESS = 450_958_579

            # Bidirectional private IPv6 access to/from Google services.
            ENABLE_BIDIRECTIONAL_ACCESS_TO_GOOGLE = 427_975_994

            # Outbound private IPv6 access from VMs in this subnet to Google services.
            ENABLE_OUTBOUND_VM_ACCESS_TO_GOOGLE = 288_210_263
          end

          # The purpose of the resource. This field can be either PRIVATE, REGIONAL_MANAGED_PROXY, PRIVATE_SERVICE_CONNECT, or INTERNAL_HTTPS_LOAD_BALANCER. PRIVATE is the default purpose for user-created subnets or subnets that are automatically created in auto mode networks. A subnet with purpose set to REGIONAL_MANAGED_PROXY is a user-created subnetwork that is reserved for regional Envoy-based load balancers. A subnet with purpose set to PRIVATE_SERVICE_CONNECT is used to publish services using Private Service Connect. A subnet with purpose set to INTERNAL_HTTPS_LOAD_BALANCER is a proxy-only subnet that can be used only by regional internal HTTP(S) load balancers. Note that REGIONAL_MANAGED_PROXY is the preferred setting for all regional Envoy load balancers. If unspecified, the subnet purpose defaults to PRIVATE. The enableFlowLogs field isn't supported if the subnet purpose field is set to REGIONAL_MANAGED_PROXY.
          module Purpose
            # A value indicating that the enum field is not set.
            UNDEFINED_PURPOSE = 0

            # Subnet reserved for Global Envoy-based Load Balancing.
            GLOBAL_MANAGED_PROXY = 236_463_602

            # Subnet reserved for Internal HTTP(S) Load Balancing.
            INTERNAL_HTTPS_LOAD_BALANCER = 248_748_889

            # Regular user created or automatically created subnet.
            PRIVATE = 403_485_027

            # Subnetwork used as source range for Private NAT Gateways.
            PRIVATE_NAT = 367_764_517

            # Regular user created or automatically created subnet.
            PRIVATE_RFC_1918 = 254_902_107

            # Subnetworks created for Private Service Connect in the producer network.
            PRIVATE_SERVICE_CONNECT = 48_134_724

            # Subnetwork used for Regional Envoy-based Load Balancing.
            REGIONAL_MANAGED_PROXY = 153_049_966
          end

          # The role of subnetwork. Currently, this field is only used when purpose = REGIONAL_MANAGED_PROXY. The value can be set to ACTIVE or BACKUP. An ACTIVE subnetwork is one that is currently being used for Envoy-based load balancers in a region. A BACKUP subnetwork is one that is ready to be promoted to ACTIVE or is currently draining. This field can be updated with a patch request.
          module Role
            # A value indicating that the enum field is not set.
            UNDEFINED_ROLE = 0

            # The ACTIVE subnet that is currently used.
            ACTIVE = 314_733_318

            # The BACKUP subnet that could be promoted to ACTIVE.
            BACKUP = 341_010_882
          end

          # The stack type for the subnet. If set to IPV4_ONLY, new VMs in the subnet are assigned IPv4 addresses only. If set to IPV4_IPV6, new VMs in the subnet can be assigned both IPv4 and IPv6 addresses. If not specified, IPV4_ONLY is used. This field can be both set at resource creation time and updated using patch.
          module StackType
            # A value indicating that the enum field is not set.
            UNDEFINED_STACK_TYPE = 0

            # New VMs in this subnet can have both IPv4 and IPv6 addresses.
            IPV4_IPV6 = 22_197_249

            # New VMs in this subnet will only be assigned IPv4 addresses.
            IPV4_ONLY = 22_373_798

            UNSPECIFIED_STACK_TYPE = 298_084_569
          end

          # [Output Only] The state of the subnetwork, which can be one of the following values: READY: Subnetwork is created and ready to use DRAINING: only applicable to subnetworks that have the purpose set to INTERNAL_HTTPS_LOAD_BALANCER and indicates that connections to the load balancer are being drained. A subnetwork that is draining cannot be used or modified until it reaches a status of READY
          module State
            # A value indicating that the enum field is not set.
            UNDEFINED_STATE = 0

            # Subnetwork is being drained.
            DRAINING = 480_455_402

            # Subnetwork is ready for use.
            READY = 77_848_963
          end
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::SubnetworksScopedList}]
        #     A list of SubnetworksScopedList resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#subnetworkAggregatedList for aggregated lists of subnetworks.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class SubnetworkAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::SubnetworksScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # Contains a list of Subnetwork resources.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::Subnetwork>]
        #     A list of Subnetwork resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#subnetworkList for lists of subnetworks.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class SubnetworkList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # The available logging options for this subnetwork.
        # @!attribute [rw] aggregation_interval
        #   @return [::String]
        #     Can only be specified if VPC flow logging for this subnetwork is enabled. Toggles the aggregation interval for collecting flow logs. Increasing the interval time will reduce the amount of generated flow logs for long lasting connections. Default is an interval of 5 seconds per connection.
        #     Check the AggregationInterval enum for the list of possible values.
        # @!attribute [rw] enable
        #   @return [::Boolean]
        #     Whether to enable flow logging for this subnetwork. If this field is not explicitly set, it will not appear in get listings. If not set the default behavior is determined by the org policy, if there is no org policy specified, then it will default to disabled. Flow logging isn't supported if the subnet purpose field is set to REGIONAL_MANAGED_PROXY.
        # @!attribute [rw] filter_expr
        #   @return [::String]
        #     Can only be specified if VPC flow logs for this subnetwork is enabled. The filter expression is used to define which VPC flow logs should be exported to Cloud Logging.
        # @!attribute [rw] flow_sampling
        #   @return [::Float]
        #     Can only be specified if VPC flow logging for this subnetwork is enabled. The value of the field must be in [0, 1]. Set the sampling rate of VPC flow logs within the subnetwork where 1.0 means all collected logs are reported and 0.0 means no logs are reported. Default is 0.5 unless otherwise specified by the org policy, which means half of all collected logs are reported.
        # @!attribute [rw] metadata
        #   @return [::String]
        #     Can only be specified if VPC flow logs for this subnetwork is enabled. Configures whether all, none or a subset of metadata fields should be added to the reported VPC flow logs. Default is EXCLUDE_ALL_METADATA.
        #     Check the Metadata enum for the list of possible values.
        # @!attribute [rw] metadata_fields
        #   @return [::Array<::String>]
        #     Can only be specified if VPC flow logs for this subnetwork is enabled and "metadata" was set to CUSTOM_METADATA.
        class SubnetworkLogConfig
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Can only be specified if VPC flow logging for this subnetwork is enabled. Toggles the aggregation interval for collecting flow logs. Increasing the interval time will reduce the amount of generated flow logs for long lasting connections. Default is an interval of 5 seconds per connection.
          module AggregationInterval
            # A value indicating that the enum field is not set.
            UNDEFINED_AGGREGATION_INTERVAL = 0

            INTERVAL_10_MIN = 487_155_916

            INTERVAL_15_MIN = 491_773_521

            INTERVAL_1_MIN = 69_052_714

            INTERVAL_30_SEC = 7_548_937

            INTERVAL_5_MIN = 72_746_798

            INTERVAL_5_SEC = 72_752_429
          end

          # Can only be specified if VPC flow logs for this subnetwork is enabled. Configures whether all, none or a subset of metadata fields should be added to the reported VPC flow logs. Default is EXCLUDE_ALL_METADATA.
          module Metadata
            # A value indicating that the enum field is not set.
            UNDEFINED_METADATA = 0

            CUSTOM_METADATA = 62_450_749

            EXCLUDE_ALL_METADATA = 334_519_954

            INCLUDE_ALL_METADATA = 164_619_908
          end
        end

        # Represents a secondary IP range of a subnetwork.
        # @!attribute [rw] ip_cidr_range
        #   @return [::String]
        #     The range of IP addresses belonging to this subnetwork secondary range. Provide this property when you create the subnetwork. Ranges must be unique and non-overlapping with all primary and secondary IP ranges within a network. Only IPv4 is supported. The range can be any range listed in the Valid ranges list.
        # @!attribute [rw] range_name
        #   @return [::String]
        #     The name associated with this subnetwork secondary range, used when adding an alias IP range to a VM instance. The name must be 1-63 characters long, and comply with RFC1035. The name must be unique within the subnetwork.
        class SubnetworkSecondaryRange
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] ip_cidr_range
        #   @return [::String]
        #     The IP (in CIDR format or netmask) of internal addresses that are legal on this Subnetwork. This range should be disjoint from other subnetworks within this network. This range can only be larger than (i.e. a superset of) the range previously defined before the update.
        class SubnetworksExpandIpCidrRangeRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] subnetworks
        #   @return [::Array<::Google::Cloud::Compute::V1::Subnetwork>]
        #     A list of subnetworks contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     An informational warning that appears when the list of addresses is empty.
        class SubnetworksScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] private_ip_google_access
        #   @return [::Boolean]
        class SubnetworksSetPrivateIpGoogleAccessRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Subsetting configuration for this BackendService. Currently this is applicable only for Internal TCP/UDP load balancing, Internal HTTP(S) load balancing and Traffic Director.
        # @!attribute [rw] policy
        #   @return [::String]
        #     Check the Policy enum for the list of possible values.
        class Subsetting
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods


          module Policy
            # A value indicating that the enum field is not set.
            UNDEFINED_POLICY = 0

            # Subsetting based on consistent hashing. For Traffic Director, the number of backends per backend group (the subset size) is based on the `subset_size` parameter. For Internal HTTP(S) load balancing, the number of backends per backend group (the subset size) is dynamically adjusted in two cases: - As the number of proxy instances participating in Internal HTTP(S) load balancing increases, the subset size decreases. - When the total number of backends in a network exceeds the capacity of a single proxy instance, subset sizes are reduced automatically for each service that has backend subsetting enabled.
            CONSISTENT_HASH_SUBSETTING = 108_989_492

            # No Subsetting. Clients may open connections and send traffic to all backends of this backend service. This can lead to performance issues if there is substantial imbalance in the count of clients and backends.
            NONE = 2_402_104
          end
        end

        # A request message for Instances.Suspend. See the method description for details.
        # @!attribute [rw] discard_local_ssd
        #   @return [::Boolean]
        #     If true, discard the contents of any attached localSSD partitions. Default value is false.
        # @!attribute [rw] instance
        #   @return [::String]
        #     Name of the instance resource to suspend.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class SuspendInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Networks.SwitchToCustomMode. See the method description for details.
        # @!attribute [rw] network
        #   @return [::String]
        #     Name of the network to be updated.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class SwitchToCustomModeNetworkRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] port
        #   @return [::Integer]
        #     The TCP port number to which the health check prober sends packets. The default value is 80. Valid values are 1 through 65535.
        # @!attribute [rw] port_name
        #   @return [::String]
        #     Not supported.
        # @!attribute [rw] port_specification
        #   @return [::String]
        #     Specifies how a port is selected for health checking. Can be one of the following values: USE_FIXED_PORT: Specifies a port number explicitly using the port field in the health check. Supported by backend services for pass-through load balancers and backend services for proxy load balancers. Not supported by target pools. The health check supports all backends supported by the backend service provided the backend can be health checked. For example, GCE_VM_IP network endpoint groups, GCE_VM_IP_PORT network endpoint groups, and instance group backends. USE_NAMED_PORT: Not supported. USE_SERVING_PORT: Provides an indirect method of specifying the health check port by referring to the backend service. Only supported by backend services for proxy load balancers. Not supported by target pools. Not supported by backend services for pass-through load balancers. Supports all backends that can be health checked; for example, GCE_VM_IP_PORT network endpoint groups and instance group backends. For GCE_VM_IP_PORT network endpoint group backends, the health check uses the port number specified for each endpoint in the network endpoint group. For instance group backends, the health check uses the port number determined by looking up the backend service's named port in the instance group's list of named ports.
        #     Check the PortSpecification enum for the list of possible values.
        # @!attribute [rw] proxy_header
        #   @return [::String]
        #     Specifies the type of proxy header to append before sending data to the backend, either NONE or PROXY_V1. The default is NONE.
        #     Check the ProxyHeader enum for the list of possible values.
        # @!attribute [rw] request
        #   @return [::String]
        #     Instructs the health check prober to send this exact ASCII string, up to 1024 bytes in length, after establishing the TCP connection.
        # @!attribute [rw] response
        #   @return [::String]
        #     Creates a content-based TCP health check. In addition to establishing a TCP connection, you can configure the health check to pass only when the backend sends this exact response ASCII string, up to 1024 bytes in length. For details, see: https://cloud.google.com/load-balancing/docs/health-check-concepts#criteria-protocol-ssl-tcp
        class TCPHealthCheck
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Specifies how a port is selected for health checking. Can be one of the following values: USE_FIXED_PORT: Specifies a port number explicitly using the port field in the health check. Supported by backend services for pass-through load balancers and backend services for proxy load balancers. Not supported by target pools. The health check supports all backends supported by the backend service provided the backend can be health checked. For example, GCE_VM_IP network endpoint groups, GCE_VM_IP_PORT network endpoint groups, and instance group backends. USE_NAMED_PORT: Not supported. USE_SERVING_PORT: Provides an indirect method of specifying the health check port by referring to the backend service. Only supported by backend services for proxy load balancers. Not supported by target pools. Not supported by backend services for pass-through load balancers. Supports all backends that can be health checked; for example, GCE_VM_IP_PORT network endpoint groups and instance group backends. For GCE_VM_IP_PORT network endpoint group backends, the health check uses the port number specified for each endpoint in the network endpoint group. For instance group backends, the health check uses the port number determined by looking up the backend service's named port in the instance group's list of named ports.
          module PortSpecification
            # A value indicating that the enum field is not set.
            UNDEFINED_PORT_SPECIFICATION = 0

            # The port number in the health check's port is used for health checking. Applies to network endpoint group and instance group backends.
            USE_FIXED_PORT = 190_235_748

            # Not supported.
            USE_NAMED_PORT = 349_300_671

            # For network endpoint group backends, the health check uses the port number specified on each endpoint in the network endpoint group. For instance group backends, the health check uses the port number specified for the backend service's named port defined in the instance group's named ports.
            USE_SERVING_PORT = 362_637_516
          end

          # Specifies the type of proxy header to append before sending data to the backend, either NONE or PROXY_V1. The default is NONE.
          module ProxyHeader
            # A value indicating that the enum field is not set.
            UNDEFINED_PROXY_HEADER = 0

            NONE = 2_402_104

            PROXY_V1 = 334_352_940
          end
        end

        # A set of instance tags.
        # @!attribute [rw] fingerprint
        #   @return [::String]
        #     Specifies a fingerprint for this request, which is essentially a hash of the tags' contents and used for optimistic locking. The fingerprint is initially generated by Compute Engine and changes after every request to modify or update tags. You must always provide an up-to-date fingerprint hash in order to update or change tags. To see the latest fingerprint, make get() request to the instance.
        # @!attribute [rw] items
        #   @return [::Array<::String>]
        #     An array of tags. Each tag must be 1-63 characters long, and comply with RFC1035.
        class Tags
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a Target gRPC Proxy resource. A target gRPC proxy is a component of load balancers intended for load balancing gRPC traffic. Only global forwarding rules with load balancing scheme INTERNAL_SELF_MANAGED can reference a target gRPC proxy. The target gRPC Proxy references a URL map that specifies how traffic is routed to gRPC backend services.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] fingerprint
        #   @return [::String]
        #     Fingerprint of this resource. A hash of the contents stored in this object. This field is used in optimistic locking. This field will be ignored when inserting a TargetGrpcProxy. An up-to-date fingerprint must be provided in order to patch/update the TargetGrpcProxy; otherwise, the request will fail with error 412 conditionNotMet. To see the latest fingerprint, make a get() request to retrieve the TargetGrpcProxy.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource type. The server generates this identifier.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#targetGrpcProxy for target grpc proxies.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] self_link_with_id
        #   @return [::String]
        #     [Output Only] Server-defined URL with id for the resource.
        # @!attribute [rw] url_map
        #   @return [::String]
        #     URL to the UrlMap resource that defines the mapping from URL to the BackendService. The protocol field in the BackendService must be set to GRPC.
        # @!attribute [rw] validate_for_proxyless
        #   @return [::Boolean]
        #     If true, indicates that the BackendServices referenced by the urlMap may be accessed by gRPC applications without using a sidecar proxy. This will enable configuration checks on urlMap and its referenced BackendServices to not allow unsupported features. A gRPC application must use "xds:///" scheme in the target URI of the service it is connecting to. If false, indicates that the BackendServices referenced by the urlMap will be accessed by gRPC applications via a sidecar proxy. In this case, a gRPC application must not use "xds:///" scheme in the target URI of the service it is connecting to
        class TargetGrpcProxy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::TargetGrpcProxy>]
        #     A list of TargetGrpcProxy resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#targetGrpcProxy for target grpc proxies.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class TargetGrpcProxyList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] target_http_proxies
        #   @return [::Array<::Google::Cloud::Compute::V1::TargetHttpProxy>]
        #     A list of TargetHttpProxies contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     Informational warning which replaces the list of backend services when the list is empty.
        class TargetHttpProxiesScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a Target HTTP Proxy resource. Google Compute Engine has two Target HTTP Proxy resources: * [Global](/compute/docs/reference/rest/v1/targetHttpProxies) * [Regional](/compute/docs/reference/rest/v1/regionTargetHttpProxies) A target HTTP proxy is a component of Google Cloud HTTP load balancers. * targetHttpProxies are used by global external Application Load Balancers, classic Application Load Balancers, cross-region internal Application Load Balancers, and Traffic Director. * regionTargetHttpProxies are used by regional internal Application Load Balancers and regional external Application Load Balancers. Forwarding rules reference a target HTTP proxy, and the target proxy then references a URL map. For more information, read Using Target Proxies and Forwarding rule concepts.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] fingerprint
        #   @return [::String]
        #     Fingerprint of this resource. A hash of the contents stored in this object. This field is used in optimistic locking. This field will be ignored when inserting a TargetHttpProxy. An up-to-date fingerprint must be provided in order to patch/update the TargetHttpProxy; otherwise, the request will fail with error 412 conditionNotMet. To see the latest fingerprint, make a get() request to retrieve the TargetHttpProxy.
        # @!attribute [rw] http_keep_alive_timeout_sec
        #   @return [::Integer]
        #     Specifies how long to keep a connection open, after completing a response, while there is no matching traffic (in seconds). If an HTTP keep-alive is not specified, a default value (610 seconds) will be used. For global external Application Load Balancers, the minimum allowed value is 5 seconds and the maximum allowed value is 1200 seconds. For classic Application Load Balancers, this option is not supported.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#targetHttpProxy for target HTTP proxies.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] proxy_bind
        #   @return [::Boolean]
        #     This field only applies when the forwarding rule that references this target proxy has a loadBalancingScheme set to INTERNAL_SELF_MANAGED. When this field is set to true, Envoy proxies set up inbound traffic interception and bind to the IP address and port specified in the forwarding rule. This is generally useful when using Traffic Director to configure Envoy as a gateway or middle proxy (in other words, not a sidecar proxy). The Envoy proxy listens for inbound requests and handles requests when it receives them. The default is false.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] URL of the region where the regional Target HTTP Proxy resides. This field is not applicable to global Target HTTP Proxies.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] url_map
        #   @return [::String]
        #     URL to the UrlMap resource that defines the mapping from URL to the BackendService.
        class TargetHttpProxy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::TargetHttpProxiesScopedList}]
        #     A list of TargetHttpProxiesScopedList resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#targetHttpProxyAggregatedList for lists of Target HTTP Proxies.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        class TargetHttpProxyAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::TargetHttpProxiesScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # A list of TargetHttpProxy resources.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::TargetHttpProxy>]
        #     A list of TargetHttpProxy resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     Type of resource. Always compute#targetHttpProxyList for lists of target HTTP proxies.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class TargetHttpProxyList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] target_https_proxies
        #   @return [::Array<::Google::Cloud::Compute::V1::TargetHttpsProxy>]
        #     A list of TargetHttpsProxies contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     Informational warning which replaces the list of backend services when the list is empty.
        class TargetHttpsProxiesScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] certificate_map
        #   @return [::String]
        #     URL of the Certificate Map to associate with this TargetHttpsProxy. Accepted format is //certificatemanager.googleapis.com/projects/\\{project }/locations/\\{location}/certificateMaps/\\{resourceName}.
        class TargetHttpsProxiesSetCertificateMapRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] quic_override
        #   @return [::String]
        #     QUIC policy for the TargetHttpsProxy resource.
        #     Check the QuicOverride enum for the list of possible values.
        class TargetHttpsProxiesSetQuicOverrideRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # QUIC policy for the TargetHttpsProxy resource.
          module QuicOverride
            # A value indicating that the enum field is not set.
            UNDEFINED_QUIC_OVERRIDE = 0

            # The load balancer will not attempt to negotiate QUIC with clients.
            DISABLE = 241_807_048

            # The load balancer will attempt to negotiate QUIC with clients.
            ENABLE = 438_835_587

            # No overrides to the default QUIC policy. This option is implicit if no QUIC override has been specified in the request.
            NONE = 2_402_104
          end
        end

        # @!attribute [rw] ssl_certificates
        #   @return [::Array<::String>]
        #     New set of SslCertificate resources to associate with this TargetHttpsProxy resource. At least one SSL certificate must be specified. Currently, you may specify up to 15 SSL certificates.
        class TargetHttpsProxiesSetSslCertificatesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a Target HTTPS Proxy resource. Google Compute Engine has two Target HTTPS Proxy resources: * [Global](/compute/docs/reference/rest/v1/targetHttpsProxies) * [Regional](/compute/docs/reference/rest/v1/regionTargetHttpsProxies) A target HTTPS proxy is a component of GCP HTTPS load balancers. * targetHttpProxies are used by global external Application Load Balancers, classic Application Load Balancers, cross-region internal Application Load Balancers, and Traffic Director. * regionTargetHttpProxies are used by regional internal Application Load Balancers and regional external Application Load Balancers. Forwarding rules reference a target HTTPS proxy, and the target proxy then references a URL map. For more information, read Using Target Proxies and Forwarding rule concepts.
        # @!attribute [rw] authorization_policy
        #   @return [::String]
        #     Optional. A URL referring to a networksecurity.AuthorizationPolicy resource that describes how the proxy should authorize inbound traffic. If left blank, access will not be restricted by an authorization policy. Refer to the AuthorizationPolicy resource for additional details. authorizationPolicy only applies to a global TargetHttpsProxy attached to globalForwardingRules with the loadBalancingScheme set to INTERNAL_SELF_MANAGED. Note: This field currently has no impact.
        # @!attribute [rw] certificate_map
        #   @return [::String]
        #     URL of a certificate map that identifies a certificate map associated with the given target proxy. This field can only be set for global target proxies. If set, sslCertificates will be ignored. Accepted format is //certificatemanager.googleapis.com/projects/\\{project }/locations/\\{location}/certificateMaps/\\{resourceName}.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] fingerprint
        #   @return [::String]
        #     Fingerprint of this resource. A hash of the contents stored in this object. This field is used in optimistic locking. This field will be ignored when inserting a TargetHttpsProxy. An up-to-date fingerprint must be provided in order to patch the TargetHttpsProxy; otherwise, the request will fail with error 412 conditionNotMet. To see the latest fingerprint, make a get() request to retrieve the TargetHttpsProxy.
        # @!attribute [rw] http_keep_alive_timeout_sec
        #   @return [::Integer]
        #     Specifies how long to keep a connection open, after completing a response, while there is no matching traffic (in seconds). If an HTTP keep-alive is not specified, a default value (610 seconds) will be used. For global external Application Load Balancers, the minimum allowed value is 5 seconds and the maximum allowed value is 1200 seconds. For classic Application Load Balancers, this option is not supported.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#targetHttpsProxy for target HTTPS proxies.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] proxy_bind
        #   @return [::Boolean]
        #     This field only applies when the forwarding rule that references this target proxy has a loadBalancingScheme set to INTERNAL_SELF_MANAGED. When this field is set to true, Envoy proxies set up inbound traffic interception and bind to the IP address and port specified in the forwarding rule. This is generally useful when using Traffic Director to configure Envoy as a gateway or middle proxy (in other words, not a sidecar proxy). The Envoy proxy listens for inbound requests and handles requests when it receives them. The default is false.
        # @!attribute [rw] quic_override
        #   @return [::String]
        #     Specifies the QUIC override policy for this TargetHttpsProxy resource. This setting determines whether the load balancer attempts to negotiate QUIC with clients. You can specify NONE, ENABLE, or DISABLE. - When quic-override is set to NONE, Google manages whether QUIC is used. - When quic-override is set to ENABLE, the load balancer uses QUIC when possible. - When quic-override is set to DISABLE, the load balancer doesn't use QUIC. - If the quic-override flag is not specified, NONE is implied.
        #     Check the QuicOverride enum for the list of possible values.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] URL of the region where the regional TargetHttpsProxy resides. This field is not applicable to global TargetHttpsProxies.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] server_tls_policy
        #   @return [::String]
        #     Optional. A URL referring to a networksecurity.ServerTlsPolicy resource that describes how the proxy should authenticate inbound traffic. serverTlsPolicy only applies to a global TargetHttpsProxy attached to globalForwardingRules with the loadBalancingScheme set to INTERNAL_SELF_MANAGED or EXTERNAL or EXTERNAL_MANAGED. For details which ServerTlsPolicy resources are accepted with INTERNAL_SELF_MANAGED and which with EXTERNAL, EXTERNAL_MANAGED loadBalancingScheme consult ServerTlsPolicy documentation. If left blank, communications are not encrypted.
        # @!attribute [rw] ssl_certificates
        #   @return [::Array<::String>]
        #     URLs to SslCertificate resources that are used to authenticate connections between users and the load balancer. At least one SSL certificate must be specified. Currently, you may specify up to 15 SSL certificates. sslCertificates do not apply when the load balancing scheme is set to INTERNAL_SELF_MANAGED.
        # @!attribute [rw] ssl_policy
        #   @return [::String]
        #     URL of SslPolicy resource that will be associated with the TargetHttpsProxy resource. If not set, the TargetHttpsProxy resource has no SSL policy configured.
        # @!attribute [rw] url_map
        #   @return [::String]
        #     A fully-qualified or valid partial URL to the UrlMap resource that defines the mapping from URL to the BackendService. For example, the following are all valid URLs for specifying a URL map: - https://www.googleapis.compute/v1/projects/project/global/urlMaps/ url-map - projects/project/global/urlMaps/url-map - global/urlMaps/url-map
        class TargetHttpsProxy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Specifies the QUIC override policy for this TargetHttpsProxy resource. This setting determines whether the load balancer attempts to negotiate QUIC with clients. You can specify NONE, ENABLE, or DISABLE. - When quic-override is set to NONE, Google manages whether QUIC is used. - When quic-override is set to ENABLE, the load balancer uses QUIC when possible. - When quic-override is set to DISABLE, the load balancer doesn't use QUIC. - If the quic-override flag is not specified, NONE is implied.
          module QuicOverride
            # A value indicating that the enum field is not set.
            UNDEFINED_QUIC_OVERRIDE = 0

            # The load balancer will not attempt to negotiate QUIC with clients.
            DISABLE = 241_807_048

            # The load balancer will attempt to negotiate QUIC with clients.
            ENABLE = 438_835_587

            # No overrides to the default QUIC policy. This option is implicit if no QUIC override has been specified in the request.
            NONE = 2_402_104
          end
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::TargetHttpsProxiesScopedList}]
        #     A list of TargetHttpsProxiesScopedList resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#targetHttpsProxyAggregatedList for lists of Target HTTP Proxies.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class TargetHttpsProxyAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::TargetHttpsProxiesScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # Contains a list of TargetHttpsProxy resources.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::TargetHttpsProxy>]
        #     A list of TargetHttpsProxy resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     Type of resource. Always compute#targetHttpsProxyList for lists of target HTTPS proxies.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class TargetHttpsProxyList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a Target Instance resource. You can use a target instance to handle traffic for one or more forwarding rules, which is ideal for forwarding protocol traffic that is managed by a single source. For example, ESP, AH, TCP, or UDP. For more information, read Target instances.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] instance
        #   @return [::String]
        #     A URL to the virtual machine instance that handles traffic for this target instance. When creating a target instance, you can provide the fully-qualified URL or a valid partial URL to the desired virtual machine. For example, the following are all valid URLs: - https://www.googleapis.com/compute/v1/projects/project/zones/zone /instances/instance - projects/project/zones/zone/instances/instance - zones/zone/instances/instance
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] The type of the resource. Always compute#targetInstance for target instances.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] nat_policy
        #   @return [::String]
        #     Must have a value of NO_NAT. Protocol forwarding delivers packets while preserving the destination IP address of the forwarding rule referencing the target instance.
        #     Check the NatPolicy enum for the list of possible values.
        # @!attribute [rw] network
        #   @return [::String]
        #     The URL of the network this target instance uses to forward traffic. If not specified, the traffic will be forwarded to the network that the default network interface belongs to.
        # @!attribute [rw] security_policy
        #   @return [::String]
        #     [Output Only] The resource URL for the security policy associated with this target instance.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] zone
        #   @return [::String]
        #     [Output Only] URL of the zone where the target instance resides. You must specify this field as part of the HTTP request URL. It is not settable as a field in the request body.
        class TargetInstance
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Must have a value of NO_NAT. Protocol forwarding delivers packets while preserving the destination IP address of the forwarding rule referencing the target instance.
          module NatPolicy
            # A value indicating that the enum field is not set.
            UNDEFINED_NAT_POLICY = 0

            # No NAT performed.
            NO_NAT = 161_455_491
          end
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::TargetInstancesScopedList}]
        #     A list of TargetInstance resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     Type of resource.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class TargetInstanceAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::TargetInstancesScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # Contains a list of TargetInstance resources.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::TargetInstance>]
        #     A list of TargetInstance resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     Type of resource.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class TargetInstanceList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] target_instances
        #   @return [::Array<::Google::Cloud::Compute::V1::TargetInstance>]
        #     A list of target instances contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     Informational warning which replaces the list of addresses when the list is empty.
        class TargetInstancesScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a Target Pool resource. Target pools are used for network TCP/UDP load balancing. A target pool references member instances, an associated legacy HttpHealthCheck resource, and, optionally, a backup target pool. For more information, read Using target pools.
        # @!attribute [rw] backup_pool
        #   @return [::String]
        #     The server-defined URL for the resource. This field is applicable only when the containing target pool is serving a forwarding rule as the primary pool, and its failoverRatio field is properly set to a value between [0, 1]. backupPool and failoverRatio together define the fallback behavior of the primary target pool: if the ratio of the healthy instances in the primary pool is at or below failoverRatio, traffic arriving at the load-balanced IP will be directed to the backup pool. In case where failoverRatio and backupPool are not set, or all the instances in the backup pool are unhealthy, the traffic will be directed back to the primary pool in the "force" mode, where traffic will be spread to the healthy instances with the best effort, or to all instances when no instance is healthy.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] failover_ratio
        #   @return [::Float]
        #     This field is applicable only when the containing target pool is serving a forwarding rule as the primary pool (i.e., not as a backup pool to some other target pool). The value of the field must be in [0, 1]. If set, backupPool must also be set. They together define the fallback behavior of the primary target pool: if the ratio of the healthy instances in the primary pool is at or below this number, traffic arriving at the load-balanced IP will be directed to the backup pool. In case where failoverRatio is not set or all the instances in the backup pool are unhealthy, the traffic will be directed back to the primary pool in the "force" mode, where traffic will be spread to the healthy instances with the best effort, or to all instances when no instance is healthy.
        # @!attribute [rw] health_checks
        #   @return [::Array<::String>]
        #     The URL of the HttpHealthCheck resource. A member instance in this pool is considered healthy if and only if the health checks pass. Only legacy HttpHealthChecks are supported. Only one health check may be specified.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] instances
        #   @return [::Array<::String>]
        #     A list of resource URLs to the virtual machine instances serving this pool. They must live in zones contained in the same region as this pool.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#targetPool for target pools.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] URL of the region where the target pool resides.
        # @!attribute [rw] security_policy
        #   @return [::String]
        #     [Output Only] The resource URL for the security policy associated with this target pool.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] session_affinity
        #   @return [::String]
        #     Session affinity option, must be one of the following values: NONE: Connections from the same client IP may go to any instance in the pool. CLIENT_IP: Connections from the same client IP will go to the same instance in the pool while that instance remains healthy. CLIENT_IP_PROTO: Connections from the same client IP with the same IP protocol will go to the same instance in the pool while that instance remains healthy.
        #     Check the SessionAffinity enum for the list of possible values.
        class TargetPool
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Session affinity option, must be one of the following values: NONE: Connections from the same client IP may go to any instance in the pool. CLIENT_IP: Connections from the same client IP will go to the same instance in the pool while that instance remains healthy. CLIENT_IP_PROTO: Connections from the same client IP with the same IP protocol will go to the same instance in the pool while that instance remains healthy.
          module SessionAffinity
            # A value indicating that the enum field is not set.
            UNDEFINED_SESSION_AFFINITY = 0

            # 2-tuple hash on packet's source and destination IP addresses. Connections from the same source IP address to the same destination IP address will be served by the same backend VM while that VM remains healthy.
            CLIENT_IP = 345_665_051

            # 1-tuple hash only on packet's source IP address. Connections from the same source IP address will be served by the same backend VM while that VM remains healthy. This option can only be used for Internal TCP/UDP Load Balancing.
            CLIENT_IP_NO_DESTINATION = 106_122_516

            # 5-tuple hash on packet's source and destination IP addresses, IP protocol, and source and destination ports. Connections for the same IP protocol from the same source IP address and port to the same destination IP address and port will be served by the same backend VM while that VM remains healthy. This option cannot be used for HTTP(S) load balancing.
            CLIENT_IP_PORT_PROTO = 221_722_926

            # 3-tuple hash on packet's source and destination IP addresses, and IP protocol. Connections for the same IP protocol from the same source IP address to the same destination IP address will be served by the same backend VM while that VM remains healthy. This option cannot be used for HTTP(S) load balancing.
            CLIENT_IP_PROTO = 25_322_148

            # Hash based on a cookie generated by the L7 loadbalancer. Only valid for HTTP(S) load balancing.
            GENERATED_COOKIE = 370_321_204

            # The hash is based on a user specified header field.
            HEADER_FIELD = 200_737_960

            # The hash is based on a user provided cookie.
            HTTP_COOKIE = 494_981_627

            # No session affinity. Connections from the same client IP may go to any instance in the pool.
            NONE = 2_402_104
          end
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::TargetPoolsScopedList}]
        #     A list of TargetPool resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#targetPoolAggregatedList for aggregated lists of target pools.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class TargetPoolAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::TargetPoolsScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # @!attribute [rw] health_status
        #   @return [::Array<::Google::Cloud::Compute::V1::HealthStatus>]
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#targetPoolInstanceHealth when checking the health of an instance.
        class TargetPoolInstanceHealth
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Contains a list of TargetPool resources.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::TargetPool>]
        #     A list of TargetPool resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#targetPoolList for lists of target pools.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class TargetPoolList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] health_checks
        #   @return [::Array<::Google::Cloud::Compute::V1::HealthCheckReference>]
        #     The HttpHealthCheck to add to the target pool.
        class TargetPoolsAddHealthCheckRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] instances
        #   @return [::Array<::Google::Cloud::Compute::V1::InstanceReference>]
        #     A full or partial URL to an instance to add to this target pool. This can be a full or partial URL. For example, the following are valid URLs: - https://www.googleapis.com/compute/v1/projects/project-id/zones/zone /instances/instance-name - projects/project-id/zones/zone/instances/instance-name - zones/zone/instances/instance-name
        class TargetPoolsAddInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] health_checks
        #   @return [::Array<::Google::Cloud::Compute::V1::HealthCheckReference>]
        #     Health check URL to be removed. This can be a full or valid partial URL. For example, the following are valid URLs: - https://www.googleapis.com/compute/beta/projects/project /global/httpHealthChecks/health-check - projects/project/global/httpHealthChecks/health-check - global/httpHealthChecks/health-check
        class TargetPoolsRemoveHealthCheckRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] instances
        #   @return [::Array<::Google::Cloud::Compute::V1::InstanceReference>]
        #     URLs of the instances to be removed from target pool.
        class TargetPoolsRemoveInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] target_pools
        #   @return [::Array<::Google::Cloud::Compute::V1::TargetPool>]
        #     A list of target pools contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     Informational warning which replaces the list of addresses when the list is empty.
        class TargetPoolsScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] target
        #   @return [::String]
        class TargetReference
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] service
        #   @return [::String]
        #     The URL of the new BackendService resource for the targetSslProxy.
        class TargetSslProxiesSetBackendServiceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] certificate_map
        #   @return [::String]
        #     URL of the Certificate Map to associate with this TargetSslProxy. Accepted format is //certificatemanager.googleapis.com/projects/\\{project }/locations/\\{location}/certificateMaps/\\{resourceName}.
        class TargetSslProxiesSetCertificateMapRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] proxy_header
        #   @return [::String]
        #     The new type of proxy header to append before sending data to the backend. NONE or PROXY_V1 are allowed.
        #     Check the ProxyHeader enum for the list of possible values.
        class TargetSslProxiesSetProxyHeaderRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The new type of proxy header to append before sending data to the backend. NONE or PROXY_V1 are allowed.
          module ProxyHeader
            # A value indicating that the enum field is not set.
            UNDEFINED_PROXY_HEADER = 0

            NONE = 2_402_104

            PROXY_V1 = 334_352_940
          end
        end

        # @!attribute [rw] ssl_certificates
        #   @return [::Array<::String>]
        #     New set of URLs to SslCertificate resources to associate with this TargetSslProxy. At least one SSL certificate must be specified. Currently, you may specify up to 15 SSL certificates.
        class TargetSslProxiesSetSslCertificatesRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a Target SSL Proxy resource. A target SSL proxy is a component of a SSL Proxy load balancer. Global forwarding rules reference a target SSL proxy, and the target proxy then references an external backend service. For more information, read Using Target Proxies.
        # @!attribute [rw] certificate_map
        #   @return [::String]
        #     URL of a certificate map that identifies a certificate map associated with the given target proxy. This field can only be set for global target proxies. If set, sslCertificates will be ignored. Accepted format is //certificatemanager.googleapis.com/projects/\\{project }/locations/\\{location}/certificateMaps/\\{resourceName}.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#targetSslProxy for target SSL proxies.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] proxy_header
        #   @return [::String]
        #     Specifies the type of proxy header to append before sending data to the backend, either NONE or PROXY_V1. The default is NONE.
        #     Check the ProxyHeader enum for the list of possible values.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] service
        #   @return [::String]
        #     URL to the BackendService resource.
        # @!attribute [rw] ssl_certificates
        #   @return [::Array<::String>]
        #     URLs to SslCertificate resources that are used to authenticate connections to Backends. At least one SSL certificate must be specified. Currently, you may specify up to 15 SSL certificates. sslCertificates do not apply when the load balancing scheme is set to INTERNAL_SELF_MANAGED.
        # @!attribute [rw] ssl_policy
        #   @return [::String]
        #     URL of SslPolicy resource that will be associated with the TargetSslProxy resource. If not set, the TargetSslProxy resource will not have any SSL policy configured.
        class TargetSslProxy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Specifies the type of proxy header to append before sending data to the backend, either NONE or PROXY_V1. The default is NONE.
          module ProxyHeader
            # A value indicating that the enum field is not set.
            UNDEFINED_PROXY_HEADER = 0

            NONE = 2_402_104

            PROXY_V1 = 334_352_940
          end
        end

        # Contains a list of TargetSslProxy resources.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::TargetSslProxy>]
        #     A list of TargetSslProxy resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     Type of resource.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class TargetSslProxyList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] target_tcp_proxies
        #   @return [::Array<::Google::Cloud::Compute::V1::TargetTcpProxy>]
        #     A list of TargetTcpProxies contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     Informational warning which replaces the list of backend services when the list is empty.
        class TargetTcpProxiesScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] service
        #   @return [::String]
        #     The URL of the new BackendService resource for the targetTcpProxy.
        class TargetTcpProxiesSetBackendServiceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] proxy_header
        #   @return [::String]
        #     The new type of proxy header to append before sending data to the backend. NONE or PROXY_V1 are allowed.
        #     Check the ProxyHeader enum for the list of possible values.
        class TargetTcpProxiesSetProxyHeaderRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The new type of proxy header to append before sending data to the backend. NONE or PROXY_V1 are allowed.
          module ProxyHeader
            # A value indicating that the enum field is not set.
            UNDEFINED_PROXY_HEADER = 0

            NONE = 2_402_104

            PROXY_V1 = 334_352_940
          end
        end

        # Represents a Target TCP Proxy resource. A target TCP proxy is a component of a TCP Proxy load balancer. Global forwarding rules reference target TCP proxy, and the target proxy then references an external backend service. For more information, read TCP Proxy Load Balancing overview.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#targetTcpProxy for target TCP proxies.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] proxy_bind
        #   @return [::Boolean]
        #     This field only applies when the forwarding rule that references this target proxy has a loadBalancingScheme set to INTERNAL_SELF_MANAGED. When this field is set to true, Envoy proxies set up inbound traffic interception and bind to the IP address and port specified in the forwarding rule. This is generally useful when using Traffic Director to configure Envoy as a gateway or middle proxy (in other words, not a sidecar proxy). The Envoy proxy listens for inbound requests and handles requests when it receives them. The default is false.
        # @!attribute [rw] proxy_header
        #   @return [::String]
        #     Specifies the type of proxy header to append before sending data to the backend, either NONE or PROXY_V1. The default is NONE.
        #     Check the ProxyHeader enum for the list of possible values.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] URL of the region where the regional TCP proxy resides. This field is not applicable to global TCP proxy.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] service
        #   @return [::String]
        #     URL to the BackendService resource.
        class TargetTcpProxy
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Specifies the type of proxy header to append before sending data to the backend, either NONE or PROXY_V1. The default is NONE.
          module ProxyHeader
            # A value indicating that the enum field is not set.
            UNDEFINED_PROXY_HEADER = 0

            NONE = 2_402_104

            PROXY_V1 = 334_352_940
          end
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::TargetTcpProxiesScopedList}]
        #     A list of TargetTcpProxiesScopedList resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#targetTcpProxyAggregatedList for lists of Target TCP Proxies.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class TargetTcpProxyAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::TargetTcpProxiesScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # Contains a list of TargetTcpProxy resources.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::TargetTcpProxy>]
        #     A list of TargetTcpProxy resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     Type of resource.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class TargetTcpProxyList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a Target VPN Gateway resource. The target VPN gateway resource represents a Classic Cloud VPN gateway. For more information, read the the Cloud VPN Overview.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] forwarding_rules
        #   @return [::Array<::String>]
        #     [Output Only] A list of URLs to the ForwardingRule resources. ForwardingRules are created using compute.forwardingRules.insert and associated with a VPN gateway.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#targetVpnGateway for target VPN gateways.
        # @!attribute [rw] label_fingerprint
        #   @return [::String]
        #     A fingerprint for the labels being applied to this TargetVpnGateway, which is essentially a hash of the labels set used for optimistic locking. The fingerprint is initially generated by Compute Engine and changes after every request to modify or update labels. You must always provide an up-to-date fingerprint hash in order to update or change labels, otherwise the request will fail with error 412 conditionNotMet. To see the latest fingerprint, make a get() request to retrieve a TargetVpnGateway.
        # @!attribute [rw] labels
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     Labels for this resource. These can only be added or modified by the setLabels method. Each label key/value pair must comply with RFC1035. Label values may be empty.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] network
        #   @return [::String]
        #     URL of the network to which this VPN gateway is attached. Provided by the client when the VPN gateway is created.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] URL of the region where the target VPN gateway resides. You must specify this field as part of the HTTP request URL. It is not settable as a field in the request body.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] status
        #   @return [::String]
        #     [Output Only] The status of the VPN gateway, which can be one of the following: CREATING, READY, FAILED, or DELETING.
        #     Check the Status enum for the list of possible values.
        # @!attribute [rw] tunnels
        #   @return [::Array<::String>]
        #     [Output Only] A list of URLs to VpnTunnel resources. VpnTunnels are created using the compute.vpntunnels.insert method and associated with a VPN gateway.
        class TargetVpnGateway
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class LabelsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # [Output Only] The status of the VPN gateway, which can be one of the following: CREATING, READY, FAILED, or DELETING.
          module Status
            # A value indicating that the enum field is not set.
            UNDEFINED_STATUS = 0

            CREATING = 455_564_985

            DELETING = 528_602_024

            FAILED = 455_706_685

            READY = 77_848_963
          end
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::TargetVpnGatewaysScopedList}]
        #     A list of TargetVpnGateway resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#targetVpnGateway for target VPN gateways.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class TargetVpnGatewayAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::TargetVpnGatewaysScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # Contains a list of TargetVpnGateway resources.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::TargetVpnGateway>]
        #     A list of TargetVpnGateway resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#targetVpnGateway for target VPN gateways.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class TargetVpnGatewayList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] target_vpn_gateways
        #   @return [::Array<::Google::Cloud::Compute::V1::TargetVpnGateway>]
        #     [Output Only] A list of target VPN gateways contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning which replaces the list of addresses when the list is empty.
        class TargetVpnGatewaysScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] actual_output_url
        #   @return [::String]
        #     The actual output URL evaluated by a load balancer containing the scheme, host, path and query parameters.
        # @!attribute [rw] actual_redirect_response_code
        #   @return [::Integer]
        #     Actual HTTP status code for rule with `urlRedirect` calculated by load balancer
        # @!attribute [rw] actual_service
        #   @return [::String]
        #     BackendService or BackendBucket returned by load balancer.
        # @!attribute [rw] expected_output_url
        #   @return [::String]
        #     The expected output URL evaluated by a load balancer containing the scheme, host, path and query parameters.
        # @!attribute [rw] expected_redirect_response_code
        #   @return [::Integer]
        #     Expected HTTP status code for rule with `urlRedirect` calculated by load balancer
        # @!attribute [rw] expected_service
        #   @return [::String]
        #     Expected BackendService or BackendBucket resource the given URL should be mapped to.
        # @!attribute [rw] headers
        #   @return [::Array<::Google::Cloud::Compute::V1::UrlMapTestHeader>]
        #     HTTP headers of the request.
        # @!attribute [rw] host
        #   @return [::String]
        #     Host portion of the URL.
        # @!attribute [rw] path
        #   @return [::String]
        #     Path portion including query parameters in the URL.
        class TestFailure
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for BackendBuckets.TestIamPermissions. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] test_permissions_request_resource
        #   @return [::Google::Cloud::Compute::V1::TestPermissionsRequest]
        #     The body resource for this request
        class TestIamPermissionsBackendBucketRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for BackendServices.TestIamPermissions. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] test_permissions_request_resource
        #   @return [::Google::Cloud::Compute::V1::TestPermissionsRequest]
        #     The body resource for this request
        class TestIamPermissionsBackendServiceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Disks.TestIamPermissions. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] test_permissions_request_resource
        #   @return [::Google::Cloud::Compute::V1::TestPermissionsRequest]
        #     The body resource for this request
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class TestIamPermissionsDiskRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ExternalVpnGateways.TestIamPermissions. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] test_permissions_request_resource
        #   @return [::Google::Cloud::Compute::V1::TestPermissionsRequest]
        #     The body resource for this request
        class TestIamPermissionsExternalVpnGatewayRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for FirewallPolicies.TestIamPermissions. See the method description for details.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] test_permissions_request_resource
        #   @return [::Google::Cloud::Compute::V1::TestPermissionsRequest]
        #     The body resource for this request
        class TestIamPermissionsFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Images.TestIamPermissions. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] test_permissions_request_resource
        #   @return [::Google::Cloud::Compute::V1::TestPermissionsRequest]
        #     The body resource for this request
        class TestIamPermissionsImageRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.TestIamPermissions. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] test_permissions_request_resource
        #   @return [::Google::Cloud::Compute::V1::TestPermissionsRequest]
        #     The body resource for this request
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class TestIamPermissionsInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceTemplates.TestIamPermissions. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] test_permissions_request_resource
        #   @return [::Google::Cloud::Compute::V1::TestPermissionsRequest]
        #     The body resource for this request
        class TestIamPermissionsInstanceTemplateRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for LicenseCodes.TestIamPermissions. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] test_permissions_request_resource
        #   @return [::Google::Cloud::Compute::V1::TestPermissionsRequest]
        #     The body resource for this request
        class TestIamPermissionsLicenseCodeRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Licenses.TestIamPermissions. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] test_permissions_request_resource
        #   @return [::Google::Cloud::Compute::V1::TestPermissionsRequest]
        #     The body resource for this request
        class TestIamPermissionsLicenseRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for MachineImages.TestIamPermissions. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] test_permissions_request_resource
        #   @return [::Google::Cloud::Compute::V1::TestPermissionsRequest]
        #     The body resource for this request
        class TestIamPermissionsMachineImageRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkAttachments.TestIamPermissions. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] test_permissions_request_resource
        #   @return [::Google::Cloud::Compute::V1::TestPermissionsRequest]
        #     The body resource for this request
        class TestIamPermissionsNetworkAttachmentRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkEndpointGroups.TestIamPermissions. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] test_permissions_request_resource
        #   @return [::Google::Cloud::Compute::V1::TestPermissionsRequest]
        #     The body resource for this request
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class TestIamPermissionsNetworkEndpointGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NetworkFirewallPolicies.TestIamPermissions. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] test_permissions_request_resource
        #   @return [::Google::Cloud::Compute::V1::TestPermissionsRequest]
        #     The body resource for this request
        class TestIamPermissionsNetworkFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NodeGroups.TestIamPermissions. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] test_permissions_request_resource
        #   @return [::Google::Cloud::Compute::V1::TestPermissionsRequest]
        #     The body resource for this request
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class TestIamPermissionsNodeGroupRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for NodeTemplates.TestIamPermissions. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] test_permissions_request_resource
        #   @return [::Google::Cloud::Compute::V1::TestPermissionsRequest]
        #     The body resource for this request
        class TestIamPermissionsNodeTemplateRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for PacketMirrorings.TestIamPermissions. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] test_permissions_request_resource
        #   @return [::Google::Cloud::Compute::V1::TestPermissionsRequest]
        #     The body resource for this request
        class TestIamPermissionsPacketMirroringRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionBackendServices.TestIamPermissions. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] test_permissions_request_resource
        #   @return [::Google::Cloud::Compute::V1::TestPermissionsRequest]
        #     The body resource for this request
        class TestIamPermissionsRegionBackendServiceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionDisks.TestIamPermissions. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] test_permissions_request_resource
        #   @return [::Google::Cloud::Compute::V1::TestPermissionsRequest]
        #     The body resource for this request
        class TestIamPermissionsRegionDiskRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionNetworkFirewallPolicies.TestIamPermissions. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] test_permissions_request_resource
        #   @return [::Google::Cloud::Compute::V1::TestPermissionsRequest]
        #     The body resource for this request
        class TestIamPermissionsRegionNetworkFirewallPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Reservations.TestIamPermissions. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] test_permissions_request_resource
        #   @return [::Google::Cloud::Compute::V1::TestPermissionsRequest]
        #     The body resource for this request
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class TestIamPermissionsReservationRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ResourcePolicies.TestIamPermissions. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] test_permissions_request_resource
        #   @return [::Google::Cloud::Compute::V1::TestPermissionsRequest]
        #     The body resource for this request
        class TestIamPermissionsResourcePolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ServiceAttachments.TestIamPermissions. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] test_permissions_request_resource
        #   @return [::Google::Cloud::Compute::V1::TestPermissionsRequest]
        #     The body resource for this request
        class TestIamPermissionsServiceAttachmentRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Snapshots.TestIamPermissions. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] test_permissions_request_resource
        #   @return [::Google::Cloud::Compute::V1::TestPermissionsRequest]
        #     The body resource for this request
        class TestIamPermissionsSnapshotRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Subnetworks.TestIamPermissions. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] test_permissions_request_resource
        #   @return [::Google::Cloud::Compute::V1::TestPermissionsRequest]
        #     The body resource for this request
        class TestIamPermissionsSubnetworkRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for VpnGateways.TestIamPermissions. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] resource
        #   @return [::String]
        #     Name or id of the resource for this request.
        # @!attribute [rw] test_permissions_request_resource
        #   @return [::Google::Cloud::Compute::V1::TestPermissionsRequest]
        #     The body resource for this request
        class TestIamPermissionsVpnGatewayRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] permissions
        #   @return [::Array<::String>]
        #     The set of permissions to check for the 'resource'. Permissions with wildcards (such as '*' or 'storage.*') are not allowed.
        class TestPermissionsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] permissions
        #   @return [::Array<::String>]
        #     A subset of `TestPermissionsRequest.permissions` that the caller is allowed.
        class TestPermissionsResponse
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] high
        #   @return [::Integer]
        # @!attribute [rw] low
        #   @return [::Integer]
        class Uint128
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Upcoming Maintenance notification information.
        # @!attribute [rw] can_reschedule
        #   @return [::Boolean]
        #     Indicates if the maintenance can be customer triggered.
        # @!attribute [rw] latest_window_start_time
        #   @return [::String]
        #     The latest time for the planned maintenance window to start. This timestamp value is in RFC3339 text format.
        # @!attribute [rw] maintenance_status
        #   @return [::String]
        #     Check the MaintenanceStatus enum for the list of possible values.
        # @!attribute [rw] type
        #   @return [::String]
        #     Defines the type of maintenance.
        #     Check the Type enum for the list of possible values.
        # @!attribute [rw] window_end_time
        #   @return [::String]
        #     The time by which the maintenance disruption will be completed. This timestamp value is in RFC3339 text format.
        # @!attribute [rw] window_start_time
        #   @return [::String]
        #     The current start time of the maintenance window. This timestamp value is in RFC3339 text format.
        class UpcomingMaintenance
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods


          module MaintenanceStatus
            # A value indicating that the enum field is not set.
            UNDEFINED_MAINTENANCE_STATUS = 0

            # There is ongoing maintenance on this VM.
            ONGOING = 473_158_491

            # There is pending maintenance.
            PENDING = 35_394_935

            # Unknown maintenance status. Do not use this value.
            UNKNOWN = 433_141_802
          end

          # Defines the type of maintenance.
          module Type
            # A value indicating that the enum field is not set.
            UNDEFINED_TYPE = 0

            # Scheduled maintenance (e.g. maintenance after uptime guarantee is complete).
            SCHEDULED = 478_400_653

            # No type specified. Do not use this value.
            UNKNOWN_TYPE = 490_705_455

            # Unscheduled maintenance (e.g. emergency maintenance during uptime guarantee).
            UNSCHEDULED = 450_077_204
          end
        end

        # A request message for Instances.UpdateAccessConfig. See the method description for details.
        # @!attribute [rw] access_config_resource
        #   @return [::Google::Cloud::Compute::V1::AccessConfig]
        #     The body resource for this request
        # @!attribute [rw] instance
        #   @return [::String]
        #     The instance name for this request.
        # @!attribute [rw] network_interface
        #   @return [::String]
        #     The name of the network interface where the access config is attached.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class UpdateAccessConfigInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Autoscalers.Update. See the method description for details.
        # @!attribute [rw] autoscaler
        #   @return [::String]
        #     Name of the autoscaler to update.
        # @!attribute [rw] autoscaler_resource
        #   @return [::Google::Cloud::Compute::V1::Autoscaler]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     Name of the zone for this request.
        class UpdateAutoscalerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for BackendBuckets.Update. See the method description for details.
        # @!attribute [rw] backend_bucket
        #   @return [::String]
        #     Name of the BackendBucket resource to update.
        # @!attribute [rw] backend_bucket_resource
        #   @return [::Google::Cloud::Compute::V1::BackendBucket]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class UpdateBackendBucketRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for BackendServices.Update. See the method description for details.
        # @!attribute [rw] backend_service
        #   @return [::String]
        #     Name of the BackendService resource to update.
        # @!attribute [rw] backend_service_resource
        #   @return [::Google::Cloud::Compute::V1::BackendService]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class UpdateBackendServiceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Disks.Update. See the method description for details.
        # @!attribute [rw] disk
        #   @return [::String]
        #     The disk name for this request.
        # @!attribute [rw] disk_resource
        #   @return [::Google::Cloud::Compute::V1::Disk]
        #     The body resource for this request
        # @!attribute [rw] paths
        #   @return [::String]
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] update_mask
        #   @return [::String]
        #     update_mask indicates fields to be updated as part of this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class UpdateDiskRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.UpdateDisplayDevice. See the method description for details.
        # @!attribute [rw] display_device_resource
        #   @return [::Google::Cloud::Compute::V1::DisplayDevice]
        #     The body resource for this request
        # @!attribute [rw] instance
        #   @return [::String]
        #     Name of the instance scoping this request.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class UpdateDisplayDeviceInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Firewalls.Update. See the method description for details.
        # @!attribute [rw] firewall
        #   @return [::String]
        #     Name of the firewall rule to update.
        # @!attribute [rw] firewall_resource
        #   @return [::Google::Cloud::Compute::V1::Firewall]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class UpdateFirewallRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for HealthChecks.Update. See the method description for details.
        # @!attribute [rw] health_check
        #   @return [::String]
        #     Name of the HealthCheck resource to update.
        # @!attribute [rw] health_check_resource
        #   @return [::Google::Cloud::Compute::V1::HealthCheck]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class UpdateHealthCheckRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.Update. See the method description for details.
        # @!attribute [rw] instance
        #   @return [::String]
        #     Name of the instance resource to update.
        # @!attribute [rw] instance_resource
        #   @return [::Google::Cloud::Compute::V1::Instance]
        #     The body resource for this request
        # @!attribute [rw] minimal_action
        #   @return [::String]
        #     Specifies the action to take when updating an instance even if the updated properties do not require it. If not specified, then Compute Engine acts based on the minimum action that the updated properties require.
        #     Check the MinimalAction enum for the list of possible values.
        # @!attribute [rw] most_disruptive_allowed_action
        #   @return [::String]
        #     Specifies the most disruptive action that can be taken on the instance as part of the update. Compute Engine returns an error if the instance properties require a more disruptive action as part of the instance update. Valid options from lowest to highest are NO_EFFECT, REFRESH, and RESTART.
        #     Check the MostDisruptiveAllowedAction enum for the list of possible values.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class UpdateInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Specifies the action to take when updating an instance even if the updated properties do not require it. If not specified, then Compute Engine acts based on the minimum action that the updated properties require.
          # Additional supported values which may be not listed in the enum directly due to technical reasons:
          # INVALID
          # NO_EFFECT
          # REFRESH
          # RESTART
          module MinimalAction
            # A value indicating that the enum field is not set.
            UNDEFINED_MINIMAL_ACTION = 0
          end

          # Specifies the most disruptive action that can be taken on the instance as part of the update. Compute Engine returns an error if the instance properties require a more disruptive action as part of the instance update. Valid options from lowest to highest are NO_EFFECT, REFRESH, and RESTART.
          # Additional supported values which may be not listed in the enum directly due to technical reasons:
          # INVALID
          # NO_EFFECT
          # REFRESH
          # RESTART
          module MostDisruptiveAllowedAction
            # A value indicating that the enum field is not set.
            UNDEFINED_MOST_DISRUPTIVE_ALLOWED_ACTION = 0
          end
        end

        # A request message for Instances.UpdateNetworkInterface. See the method description for details.
        # @!attribute [rw] instance
        #   @return [::String]
        #     The instance name for this request.
        # @!attribute [rw] network_interface
        #   @return [::String]
        #     The name of the network interface to update.
        # @!attribute [rw] network_interface_resource
        #   @return [::Google::Cloud::Compute::V1::NetworkInterface]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class UpdateNetworkInterfaceInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Networks.UpdatePeering. See the method description for details.
        # @!attribute [rw] network
        #   @return [::String]
        #     Name of the network resource which the updated peering is belonging to.
        # @!attribute [rw] networks_update_peering_request_resource
        #   @return [::Google::Cloud::Compute::V1::NetworksUpdatePeeringRequest]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class UpdatePeeringNetworkRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for InstanceGroupManagers.UpdatePerInstanceConfigs. See the method description for details.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     The name of the managed instance group. It should conform to RFC1035.
        # @!attribute [rw] instance_group_managers_update_per_instance_configs_req_resource
        #   @return [::Google::Cloud::Compute::V1::InstanceGroupManagersUpdatePerInstanceConfigsReq]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone where the managed instance group is located. It should conform to RFC1035.
        class UpdatePerInstanceConfigsInstanceGroupManagerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionInstanceGroupManagers.UpdatePerInstanceConfigs. See the method description for details.
        # @!attribute [rw] instance_group_manager
        #   @return [::String]
        #     The name of the managed instance group. It should conform to RFC1035.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request, should conform to RFC1035.
        # @!attribute [rw] region_instance_group_manager_update_instance_config_req_resource
        #   @return [::Google::Cloud::Compute::V1::RegionInstanceGroupManagerUpdateInstanceConfigReq]
        #     The body resource for this request
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class UpdatePerInstanceConfigsRegionInstanceGroupManagerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionAutoscalers.Update. See the method description for details.
        # @!attribute [rw] autoscaler
        #   @return [::String]
        #     Name of the autoscaler to update.
        # @!attribute [rw] autoscaler_resource
        #   @return [::Google::Cloud::Compute::V1::Autoscaler]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class UpdateRegionAutoscalerRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionBackendServices.Update. See the method description for details.
        # @!attribute [rw] backend_service
        #   @return [::String]
        #     Name of the BackendService resource to update.
        # @!attribute [rw] backend_service_resource
        #   @return [::Google::Cloud::Compute::V1::BackendService]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class UpdateRegionBackendServiceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionCommitments.Update. See the method description for details.
        # @!attribute [rw] commitment
        #   @return [::String]
        #     Name of the commitment for which auto renew is being updated.
        # @!attribute [rw] commitment_resource
        #   @return [::Google::Cloud::Compute::V1::Commitment]
        #     The body resource for this request
        # @!attribute [rw] paths
        #   @return [::String]
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] update_mask
        #   @return [::String]
        #     update_mask indicates fields to be updated as part of this request.
        class UpdateRegionCommitmentRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionDisks.Update. See the method description for details.
        # @!attribute [rw] disk
        #   @return [::String]
        #     The disk name for this request.
        # @!attribute [rw] disk_resource
        #   @return [::Google::Cloud::Compute::V1::Disk]
        #     The body resource for this request
        # @!attribute [rw] paths
        #   @return [::String]
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] update_mask
        #   @return [::String]
        #     update_mask indicates fields to be updated as part of this request.
        class UpdateRegionDiskRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionHealthChecks.Update. See the method description for details.
        # @!attribute [rw] health_check
        #   @return [::String]
        #     Name of the HealthCheck resource to update.
        # @!attribute [rw] health_check_resource
        #   @return [::Google::Cloud::Compute::V1::HealthCheck]
        #     The body resource for this request
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class UpdateRegionHealthCheckRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionUrlMaps.Update. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     begin_interface: MixerMutationRequestBuilder Request ID to support idempotency.
        # @!attribute [rw] url_map
        #   @return [::String]
        #     Name of the UrlMap resource to update.
        # @!attribute [rw] url_map_resource
        #   @return [::Google::Cloud::Compute::V1::UrlMap]
        #     The body resource for this request
        class UpdateRegionUrlMapRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Reservations.Update. See the method description for details.
        # @!attribute [rw] paths
        #   @return [::String]
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] reservation
        #   @return [::String]
        #     Name of the reservation to update.
        # @!attribute [rw] reservation_resource
        #   @return [::Google::Cloud::Compute::V1::Reservation]
        #     The body resource for this request
        # @!attribute [rw] update_mask
        #   @return [::String]
        #     Update_mask indicates fields to be updated as part of this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     Name of the zone for this request.
        class UpdateReservationRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Routers.Update. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] router
        #   @return [::String]
        #     Name of the Router resource to update.
        # @!attribute [rw] router_resource
        #   @return [::Google::Cloud::Compute::V1::Router]
        #     The body resource for this request
        class UpdateRouterRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for Instances.UpdateShieldedInstanceConfig. See the method description for details.
        # @!attribute [rw] instance
        #   @return [::String]
        #     Name or id of the instance scoping this request.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] shielded_instance_config_resource
        #   @return [::Google::Cloud::Compute::V1::ShieldedInstanceConfig]
        #     The body resource for this request
        # @!attribute [rw] zone
        #   @return [::String]
        #     The name of the zone for this request.
        class UpdateShieldedInstanceConfigInstanceRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for UrlMaps.Update. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        # @!attribute [rw] url_map
        #   @return [::String]
        #     Name of the UrlMap resource to update.
        # @!attribute [rw] url_map_resource
        #   @return [::Google::Cloud::Compute::V1::UrlMap]
        #     The body resource for this request
        class UpdateUrlMapRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a URL Map resource. Compute Engine has two URL Map resources: * [Global](/compute/docs/reference/rest/v1/urlMaps) * [Regional](/compute/docs/reference/rest/v1/regionUrlMaps) A URL map resource is a component of certain types of cloud load balancers and Traffic Director: * urlMaps are used by global external Application Load Balancers, classic Application Load Balancers, and cross-region internal Application Load Balancers. * regionUrlMaps are used by internal Application Load Balancers, regional external Application Load Balancers and regional internal Application Load Balancers. For a list of supported URL map features by the load balancer type, see the Load balancing features: Routing and traffic management table. For a list of supported URL map features for Traffic Director, see the Traffic Director features: Routing and traffic management table. This resource defines mappings from hostnames and URL paths to either a backend service or a backend bucket. To use the global urlMaps resource, the backend service must have a loadBalancingScheme of either EXTERNAL or INTERNAL_SELF_MANAGED. To use the regionUrlMaps resource, the backend service must have a loadBalancingScheme of INTERNAL_MANAGED. For more information, read URL Map Concepts.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] default_route_action
        #   @return [::Google::Cloud::Compute::V1::HttpRouteAction]
        #     defaultRouteAction takes effect when none of the hostRules match. The load balancer performs advanced routing actions, such as URL rewrites and header transformations, before forwarding the request to the selected backend. If defaultRouteAction specifies any weightedBackendServices, defaultService must not be set. Conversely if defaultService is set, defaultRouteAction cannot contain any weightedBackendServices. Only one of defaultRouteAction or defaultUrlRedirect must be set. URL maps for classic Application Load Balancers only support the urlRewrite action within defaultRouteAction. defaultRouteAction has no effect when the URL map is bound to a target gRPC proxy that has the validateForProxyless field set to true.
        # @!attribute [rw] default_service
        #   @return [::String]
        #     The full or partial URL of the defaultService resource to which traffic is directed if none of the hostRules match. If defaultRouteAction is also specified, advanced routing actions, such as URL rewrites, take effect before sending the request to the backend. However, if defaultService is specified, defaultRouteAction cannot contain any weightedBackendServices. Conversely, if routeAction specifies any weightedBackendServices, service must not be specified. Only one of defaultService, defaultUrlRedirect , or defaultRouteAction.weightedBackendService must be set. defaultService has no effect when the URL map is bound to a target gRPC proxy that has the validateForProxyless field set to true.
        # @!attribute [rw] default_url_redirect
        #   @return [::Google::Cloud::Compute::V1::HttpRedirectAction]
        #     When none of the specified hostRules match, the request is redirected to a URL specified by defaultUrlRedirect. If defaultUrlRedirect is specified, defaultService or defaultRouteAction must not be set. Not supported when the URL map is bound to a target gRPC proxy.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] fingerprint
        #   @return [::String]
        #     Fingerprint of this resource. A hash of the contents stored in this object. This field is used in optimistic locking. This field is ignored when inserting a UrlMap. An up-to-date fingerprint must be provided in order to update the UrlMap, otherwise the request will fail with error 412 conditionNotMet. To see the latest fingerprint, make a get() request to retrieve a UrlMap.
        # @!attribute [rw] header_action
        #   @return [::Google::Cloud::Compute::V1::HttpHeaderAction]
        #     Specifies changes to request and response headers that need to take effect for the selected backendService. The headerAction specified here take effect after headerAction specified under pathMatcher. headerAction is not supported for load balancers that have their loadBalancingScheme set to EXTERNAL. Not supported when the URL map is bound to a target gRPC proxy that has validateForProxyless field set to true.
        # @!attribute [rw] host_rules
        #   @return [::Array<::Google::Cloud::Compute::V1::HostRule>]
        #     The list of host rules to use against the URL.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#urlMaps for url maps.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] path_matchers
        #   @return [::Array<::Google::Cloud::Compute::V1::PathMatcher>]
        #     The list of named PathMatchers to use against the URL.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] URL of the region where the regional URL map resides. This field is not applicable to global URL maps. You must specify this field as part of the HTTP request URL. It is not settable as a field in the request body.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] tests
        #   @return [::Array<::Google::Cloud::Compute::V1::UrlMapTest>]
        #     The list of expected URL mapping tests. Request to update the UrlMap succeeds only if all test cases pass. You can specify a maximum of 100 tests per UrlMap. Not supported when the URL map is bound to a target gRPC proxy that has validateForProxyless field set to true.
        class UrlMap
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Contains a list of UrlMap resources.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::UrlMap>]
        #     A list of UrlMap resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     Type of resource.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class UrlMapList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] url_map
        #   @return [::String]
        class UrlMapReference
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Message for the expected URL mappings.
        # @!attribute [rw] description
        #   @return [::String]
        #     Description of this test case.
        # @!attribute [rw] expected_output_url
        #   @return [::String]
        #     The expected output URL evaluated by the load balancer containing the scheme, host, path and query parameters. For rules that forward requests to backends, the test passes only when expectedOutputUrl matches the request forwarded by the load balancer to backends. For rules with urlRewrite, the test verifies that the forwarded request matches hostRewrite and pathPrefixRewrite in the urlRewrite action. When service is specified, expectedOutputUrl`s scheme is ignored. For rules with urlRedirect, the test passes only if expectedOutputUrl matches the URL in the load balancer's redirect response. If urlRedirect specifies https_redirect, the test passes only if the scheme in expectedOutputUrl is also set to HTTPS. If urlRedirect specifies strip_query, the test passes only if expectedOutputUrl does not contain any query parameters. expectedOutputUrl is optional when service is specified.
        # @!attribute [rw] expected_redirect_response_code
        #   @return [::Integer]
        #     For rules with urlRedirect, the test passes only if expectedRedirectResponseCode matches the HTTP status code in load balancer's redirect response. expectedRedirectResponseCode cannot be set when service is set.
        # @!attribute [rw] headers
        #   @return [::Array<::Google::Cloud::Compute::V1::UrlMapTestHeader>]
        #     HTTP headers for this request. If headers contains a host header, then host must also match the header value.
        # @!attribute [rw] host
        #   @return [::String]
        #     Host portion of the URL. If headers contains a host header, then host must also match the header value.
        # @!attribute [rw] path
        #   @return [::String]
        #     Path portion of the URL.
        # @!attribute [rw] service
        #   @return [::String]
        #     Expected BackendService or BackendBucket resource the given URL should be mapped to. The service field cannot be set if expectedRedirectResponseCode is set.
        class UrlMapTest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # HTTP headers used in UrlMapTests.
        # @!attribute [rw] name
        #   @return [::String]
        #     Header name.
        # @!attribute [rw] value
        #   @return [::String]
        #     Header value.
        class UrlMapTestHeader
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Message representing the validation result for a UrlMap.
        # @!attribute [rw] load_errors
        #   @return [::Array<::String>]
        # @!attribute [rw] load_succeeded
        #   @return [::Boolean]
        #     Whether the given UrlMap can be successfully loaded. If false, 'loadErrors' indicates the reasons.
        # @!attribute [rw] test_failures
        #   @return [::Array<::Google::Cloud::Compute::V1::TestFailure>]
        # @!attribute [rw] test_passed
        #   @return [::Boolean]
        #     If successfully loaded, this field indicates whether the test passed. If false, 'testFailures's indicate the reason of failure.
        class UrlMapValidationResult
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::UrlMapsScopedList}]
        #     A list of UrlMapsScopedList resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     Type of resource.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class UrlMapsAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::UrlMapsScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # @!attribute [rw] url_maps
        #   @return [::Array<::Google::Cloud::Compute::V1::UrlMap>]
        #     A list of UrlMaps contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     Informational warning which replaces the list of backend services when the list is empty.
        class UrlMapsScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] load_balancing_schemes
        #   @return [::Array<::String>]
        #     Specifies the load balancer type(s) this validation request is for. Use EXTERNAL_MANAGED for global external Application Load Balancers and regional external Application Load Balancers. Use EXTERNAL for classic Application Load Balancers. Use INTERNAL_MANAGED for internal Application Load Balancers. For more information, refer to Choosing a load balancer. If unspecified, the load balancing scheme will be inferred from the backend service resources this URL map references. If that can not be inferred (for example, this URL map only references backend buckets, or this Url map is for rewrites and redirects only and doesn't reference any backends), EXTERNAL will be used as the default type. If specified, the scheme(s) must not conflict with the load balancing scheme of the backend service resources this Url map references.
        #     Check the LoadBalancingSchemes enum for the list of possible values.
        # @!attribute [rw] resource
        #   @return [::Google::Cloud::Compute::V1::UrlMap]
        #     Content of the UrlMap to be validated.
        class UrlMapsValidateRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods


          module LoadBalancingSchemes
            # A value indicating that the enum field is not set.
            UNDEFINED_LOAD_BALANCING_SCHEMES = 0

            # Signifies that this will be used for classic Application Load Balancers.
            EXTERNAL = 35_607_499

            # Signifies that this will be used for Envoy-based global external Application Load Balancers.
            EXTERNAL_MANAGED = 512_006_923

            # If unspecified, the validation will try to infer the scheme from the backend service resources this Url map references. If the inference is not possible, EXTERNAL will be used as the default type.
            LOAD_BALANCING_SCHEME_UNSPECIFIED = 526_507_452
          end
        end

        # @!attribute [rw] result
        #   @return [::Google::Cloud::Compute::V1::UrlMapValidationResult]
        class UrlMapsValidateResponse
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # The spec for modifying the path before sending the request to the matched backend service.
        # @!attribute [rw] host_rewrite
        #   @return [::String]
        #     Before forwarding the request to the selected service, the request's host header is replaced with contents of hostRewrite. The value must be from 1 to 255 characters.
        # @!attribute [rw] path_prefix_rewrite
        #   @return [::String]
        #     Before forwarding the request to the selected backend service, the matching portion of the request's path is replaced by pathPrefixRewrite. The value must be from 1 to 1024 characters.
        # @!attribute [rw] path_template_rewrite
        #   @return [::String]
        #     If specified, the pattern rewrites the URL path (based on the :path header) using the HTTP template syntax. A corresponding path_template_match must be specified. Any template variables must exist in the path_template_match field. - -At least one variable must be specified in the path_template_match field - You can omit variables from the rewritten URL - The * and ** operators cannot be matched unless they have a corresponding variable name - e.g. \\{format=*} or \\{var=**}. For example, a path_template_match of /static/\\{format=**} could be rewritten as /static/content/\\{format} to prefix /content to the URL. Variables can also be re-ordered in a rewrite, so that /\\{country}/\\{format}/\\{suffix=**} can be rewritten as /content/\\{format}/\\{country}/\\{suffix}. At least one non-empty routeRules[].matchRules[].path_template_match is required. Only one of path_prefix_rewrite or path_template_rewrite may be specified.
        class UrlRewrite
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Subnetwork which the current user has compute.subnetworks.use permission on.
        # @!attribute [rw] external_ipv6_prefix
        #   @return [::String]
        #     [Output Only] The external IPv6 address range that is assigned to this subnetwork.
        # @!attribute [rw] internal_ipv6_prefix
        #   @return [::String]
        #     [Output Only] The internal IPv6 address range that is assigned to this subnetwork.
        # @!attribute [rw] ip_cidr_range
        #   @return [::String]
        #     The range of internal addresses that are owned by this subnetwork.
        # @!attribute [rw] ipv6_access_type
        #   @return [::String]
        #     The access type of IPv6 address this subnet holds. It's immutable and can only be specified during creation or the first time the subnet is updated into IPV4_IPV6 dual stack.
        #     Check the Ipv6AccessType enum for the list of possible values.
        # @!attribute [rw] network
        #   @return [::String]
        #     Network URL.
        # @!attribute [rw] purpose
        #   @return [::String]
        #     The purpose of the resource. This field can be either PRIVATE, REGIONAL_MANAGED_PROXY, PRIVATE_SERVICE_CONNECT, or INTERNAL_HTTPS_LOAD_BALANCER. PRIVATE is the default purpose for user-created subnets or subnets that are automatically created in auto mode networks. A subnet with purpose set to REGIONAL_MANAGED_PROXY is a user-created subnetwork that is reserved for regional Envoy-based load balancers. A subnet with purpose set to PRIVATE_SERVICE_CONNECT is used to publish services using Private Service Connect. A subnet with purpose set to INTERNAL_HTTPS_LOAD_BALANCER is a proxy-only subnet that can be used only by regional internal HTTP(S) load balancers. Note that REGIONAL_MANAGED_PROXY is the preferred setting for all regional Envoy load balancers. If unspecified, the subnet purpose defaults to PRIVATE. The enableFlowLogs field isn't supported if the subnet purpose field is set to REGIONAL_MANAGED_PROXY.
        #     Check the Purpose enum for the list of possible values.
        # @!attribute [rw] role
        #   @return [::String]
        #     The role of subnetwork. Currently, this field is only used when purpose = REGIONAL_MANAGED_PROXY. The value can be set to ACTIVE or BACKUP. An ACTIVE subnetwork is one that is currently being used for Envoy-based load balancers in a region. A BACKUP subnetwork is one that is ready to be promoted to ACTIVE or is currently draining. This field can be updated with a patch request.
        #     Check the Role enum for the list of possible values.
        # @!attribute [rw] secondary_ip_ranges
        #   @return [::Array<::Google::Cloud::Compute::V1::UsableSubnetworkSecondaryRange>]
        #     Secondary IP ranges.
        # @!attribute [rw] stack_type
        #   @return [::String]
        #     The stack type for the subnet. If set to IPV4_ONLY, new VMs in the subnet are assigned IPv4 addresses only. If set to IPV4_IPV6, new VMs in the subnet can be assigned both IPv4 and IPv6 addresses. If not specified, IPV4_ONLY is used. This field can be both set at resource creation time and updated using patch.
        #     Check the StackType enum for the list of possible values.
        # @!attribute [rw] subnetwork
        #   @return [::String]
        #     Subnetwork URL.
        class UsableSubnetwork
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The access type of IPv6 address this subnet holds. It's immutable and can only be specified during creation or the first time the subnet is updated into IPV4_IPV6 dual stack.
          module Ipv6AccessType
            # A value indicating that the enum field is not set.
            UNDEFINED_IPV6_ACCESS_TYPE = 0

            # VMs on this subnet will be assigned IPv6 addresses that are accessible via the Internet, as well as the VPC network.
            EXTERNAL = 35_607_499

            # VMs on this subnet will be assigned IPv6 addresses that are only accessible over the VPC network.
            INTERNAL = 279_295_677
          end

          # The purpose of the resource. This field can be either PRIVATE, REGIONAL_MANAGED_PROXY, PRIVATE_SERVICE_CONNECT, or INTERNAL_HTTPS_LOAD_BALANCER. PRIVATE is the default purpose for user-created subnets or subnets that are automatically created in auto mode networks. A subnet with purpose set to REGIONAL_MANAGED_PROXY is a user-created subnetwork that is reserved for regional Envoy-based load balancers. A subnet with purpose set to PRIVATE_SERVICE_CONNECT is used to publish services using Private Service Connect. A subnet with purpose set to INTERNAL_HTTPS_LOAD_BALANCER is a proxy-only subnet that can be used only by regional internal HTTP(S) load balancers. Note that REGIONAL_MANAGED_PROXY is the preferred setting for all regional Envoy load balancers. If unspecified, the subnet purpose defaults to PRIVATE. The enableFlowLogs field isn't supported if the subnet purpose field is set to REGIONAL_MANAGED_PROXY.
          module Purpose
            # A value indicating that the enum field is not set.
            UNDEFINED_PURPOSE = 0

            # Subnet reserved for Global Envoy-based Load Balancing.
            GLOBAL_MANAGED_PROXY = 236_463_602

            # Subnet reserved for Internal HTTP(S) Load Balancing.
            INTERNAL_HTTPS_LOAD_BALANCER = 248_748_889

            # Regular user created or automatically created subnet.
            PRIVATE = 403_485_027

            # Subnetwork used as source range for Private NAT Gateways.
            PRIVATE_NAT = 367_764_517

            # Regular user created or automatically created subnet.
            PRIVATE_RFC_1918 = 254_902_107

            # Subnetworks created for Private Service Connect in the producer network.
            PRIVATE_SERVICE_CONNECT = 48_134_724

            # Subnetwork used for Regional Envoy-based Load Balancing.
            REGIONAL_MANAGED_PROXY = 153_049_966
          end

          # The role of subnetwork. Currently, this field is only used when purpose = REGIONAL_MANAGED_PROXY. The value can be set to ACTIVE or BACKUP. An ACTIVE subnetwork is one that is currently being used for Envoy-based load balancers in a region. A BACKUP subnetwork is one that is ready to be promoted to ACTIVE or is currently draining. This field can be updated with a patch request.
          module Role
            # A value indicating that the enum field is not set.
            UNDEFINED_ROLE = 0

            # The ACTIVE subnet that is currently used.
            ACTIVE = 314_733_318

            # The BACKUP subnet that could be promoted to ACTIVE.
            BACKUP = 341_010_882
          end

          # The stack type for the subnet. If set to IPV4_ONLY, new VMs in the subnet are assigned IPv4 addresses only. If set to IPV4_IPV6, new VMs in the subnet can be assigned both IPv4 and IPv6 addresses. If not specified, IPV4_ONLY is used. This field can be both set at resource creation time and updated using patch.
          module StackType
            # A value indicating that the enum field is not set.
            UNDEFINED_STACK_TYPE = 0

            # New VMs in this subnet can have both IPv4 and IPv6 addresses.
            IPV4_IPV6 = 22_197_249

            # New VMs in this subnet will only be assigned IPv4 addresses.
            IPV4_ONLY = 22_373_798
          end
        end

        # Secondary IP range of a usable subnetwork.
        # @!attribute [rw] ip_cidr_range
        #   @return [::String]
        #     The range of IP addresses belonging to this subnetwork secondary range.
        # @!attribute [rw] range_name
        #   @return [::String]
        #     The name associated with this subnetwork secondary range, used when adding an alias IP range to a VM instance. The name must be 1-63 characters long, and comply with RFC1035. The name must be unique within the subnetwork.
        class UsableSubnetworkSecondaryRange
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::UsableSubnetwork>]
        #     [Output] A list of usable subnetwork URLs.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#usableSubnetworksAggregatedList for aggregated lists of usable subnetworks.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results. In special cases listUsable may return 0 subnetworks and nextPageToken which still should be used to get the next page of results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class UsableSubnetworksAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # The location in Cloud Storage and naming method of the daily usage report. Contains bucket_name and report_name prefix.
        # @!attribute [rw] bucket_name
        #   @return [::String]
        #     The name of an existing bucket in Cloud Storage where the usage report object is stored. The Google Service Account is granted write access to this bucket. This can either be the bucket name by itself, such as example-bucket, or the bucket name with gs:// or https://storage.googleapis.com/ in front of it, such as gs://example-bucket.
        # @!attribute [rw] report_name_prefix
        #   @return [::String]
        #     An optional prefix for the name of the usage report object stored in bucketName. If not supplied, defaults to usage_gce. The report is stored as a CSV file named report_name_prefix_gce_YYYYMMDD.csv where YYYYMMDD is the day of the usage according to Pacific Time. If you supply a prefix, it should conform to Cloud Storage object naming conventions.
        class UsageExportLocation
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionUrlMaps.Validate. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region scoping this request.
        # @!attribute [rw] region_url_maps_validate_request_resource
        #   @return [::Google::Cloud::Compute::V1::RegionUrlMapsValidateRequest]
        #     The body resource for this request
        # @!attribute [rw] url_map
        #   @return [::String]
        #     Name of the UrlMap resource to be validated as.
        class ValidateRegionUrlMapRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for UrlMaps.Validate. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] url_map
        #   @return [::String]
        #     Name of the UrlMap resource to be validated as.
        # @!attribute [rw] url_maps_validate_request_resource
        #   @return [::Google::Cloud::Compute::V1::UrlMapsValidateRequest]
        #     The body resource for this request
        class ValidateUrlMapRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Contain information of Nat mapping for a VM endpoint (i.e., NIC).
        # @!attribute [rw] instance_name
        #   @return [::String]
        #     Name of the VM instance which the endpoint belongs to
        # @!attribute [rw] interface_nat_mappings
        #   @return [::Array<::Google::Cloud::Compute::V1::VmEndpointNatMappingsInterfaceNatMappings>]
        class VmEndpointNatMappings
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Contain information of Nat mapping for an interface of this endpoint.
        # @!attribute [rw] drain_nat_ip_port_ranges
        #   @return [::Array<::String>]
        #     List of all drain IP:port-range mappings assigned to this interface. These ranges are inclusive, that is, both the first and the last ports can be used for NAT. Example: ["2.2.2.2:12345-12355", "1.1.1.1:2234-2234"].
        # @!attribute [rw] nat_ip_port_ranges
        #   @return [::Array<::String>]
        #     A list of all IP:port-range mappings assigned to this interface. These ranges are inclusive, that is, both the first and the last ports can be used for NAT. Example: ["2.2.2.2:12345-12355", "1.1.1.1:2234-2234"].
        # @!attribute [rw] num_total_drain_nat_ports
        #   @return [::Integer]
        #     Total number of drain ports across all NAT IPs allocated to this interface. It equals to the aggregated port number in the field drain_nat_ip_port_ranges.
        # @!attribute [rw] num_total_nat_ports
        #   @return [::Integer]
        #     Total number of ports across all NAT IPs allocated to this interface. It equals to the aggregated port number in the field nat_ip_port_ranges.
        # @!attribute [rw] rule_mappings
        #   @return [::Array<::Google::Cloud::Compute::V1::VmEndpointNatMappingsInterfaceNatMappingsNatRuleMappings>]
        #     Information about mappings provided by rules in this NAT.
        # @!attribute [rw] source_alias_ip_range
        #   @return [::String]
        #     Alias IP range for this interface endpoint. It will be a private (RFC 1918) IP range. Examples: "10.33.4.55/32", or "192.168.5.0/24".
        # @!attribute [rw] source_virtual_ip
        #   @return [::String]
        #     Primary IP of the VM for this NIC.
        class VmEndpointNatMappingsInterfaceNatMappings
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Contains information of NAT Mappings provided by a NAT Rule.
        # @!attribute [rw] drain_nat_ip_port_ranges
        #   @return [::Array<::String>]
        #     List of all drain IP:port-range mappings assigned to this interface by this rule. These ranges are inclusive, that is, both the first and the last ports can be used for NAT. Example: ["2.2.2.2:12345-12355", "1.1.1.1:2234-2234"].
        # @!attribute [rw] nat_ip_port_ranges
        #   @return [::Array<::String>]
        #     A list of all IP:port-range mappings assigned to this interface by this rule. These ranges are inclusive, that is, both the first and the last ports can be used for NAT. Example: ["2.2.2.2:12345-12355", "1.1.1.1:2234-2234"].
        # @!attribute [rw] num_total_drain_nat_ports
        #   @return [::Integer]
        #     Total number of drain ports across all NAT IPs allocated to this interface by this rule. It equals the aggregated port number in the field drain_nat_ip_port_ranges.
        # @!attribute [rw] num_total_nat_ports
        #   @return [::Integer]
        #     Total number of ports across all NAT IPs allocated to this interface by this rule. It equals the aggregated port number in the field nat_ip_port_ranges.
        # @!attribute [rw] rule_number
        #   @return [::Integer]
        #     Rule number of the NAT Rule.
        class VmEndpointNatMappingsInterfaceNatMappingsNatRuleMappings
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Contains a list of VmEndpointNatMappings.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#vmEndpointNatMappingsList for lists of Nat mappings of VM endpoints.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] result
        #   @return [::Array<::Google::Cloud::Compute::V1::VmEndpointNatMappings>]
        #     [Output Only] A list of Nat mapping information of VM endpoints.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class VmEndpointNatMappingsList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a HA VPN gateway. HA VPN is a high-availability (HA) Cloud VPN solution that lets you securely connect your on-premises network to your Google Cloud Virtual Private Cloud network through an IPsec VPN connection in a single region. For more information about Cloud HA VPN solutions, see Cloud VPN topologies .
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#vpnGateway for VPN gateways.
        # @!attribute [rw] label_fingerprint
        #   @return [::String]
        #     A fingerprint for the labels being applied to this VpnGateway, which is essentially a hash of the labels set used for optimistic locking. The fingerprint is initially generated by Compute Engine and changes after every request to modify or update labels. You must always provide an up-to-date fingerprint hash in order to update or change labels, otherwise the request will fail with error 412 conditionNotMet. To see the latest fingerprint, make a get() request to retrieve a VpnGateway.
        # @!attribute [rw] labels
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     Labels for this resource. These can only be added or modified by the setLabels method. Each label key/value pair must comply with RFC1035. Label values may be empty.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] network
        #   @return [::String]
        #     URL of the network to which this VPN gateway is attached. Provided by the client when the VPN gateway is created.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] URL of the region where the VPN gateway resides.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] stack_type
        #   @return [::String]
        #     The stack type for this VPN gateway to identify the IP protocols that are enabled. Possible values are: IPV4_ONLY, IPV4_IPV6. If not specified, IPV4_ONLY will be used.
        #     Check the StackType enum for the list of possible values.
        # @!attribute [rw] vpn_interfaces
        #   @return [::Array<::Google::Cloud::Compute::V1::VpnGatewayVpnGatewayInterface>]
        #     The list of VPN interfaces associated with this VPN gateway.
        class VpnGateway
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class LabelsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # The stack type for this VPN gateway to identify the IP protocols that are enabled. Possible values are: IPV4_ONLY, IPV4_IPV6. If not specified, IPV4_ONLY will be used.
          module StackType
            # A value indicating that the enum field is not set.
            UNDEFINED_STACK_TYPE = 0

            # Enable VPN gateway with both IPv4 and IPv6 protocols.
            IPV4_IPV6 = 22_197_249

            # Enable VPN gateway with only IPv4 protocol.
            IPV4_ONLY = 22_373_798
          end
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::VpnGatewaysScopedList}]
        #     A list of VpnGateway resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#vpnGateway for VPN gateways.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class VpnGatewayAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::VpnGatewaysScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # Contains a list of VpnGateway resources.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::VpnGateway>]
        #     A list of VpnGateway resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#vpnGateway for VPN gateways.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class VpnGatewayList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] vpn_connections
        #   @return [::Array<::Google::Cloud::Compute::V1::VpnGatewayStatusVpnConnection>]
        #     List of VPN connection for this VpnGateway.
        class VpnGatewayStatus
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Describes the high availability requirement state for the VPN connection between this Cloud VPN gateway and a peer gateway.
        # @!attribute [rw] state
        #   @return [::String]
        #     Indicates the high availability requirement state for the VPN connection. Valid values are CONNECTION_REDUNDANCY_MET, CONNECTION_REDUNDANCY_NOT_MET.
        #     Check the State enum for the list of possible values.
        # @!attribute [rw] unsatisfied_reason
        #   @return [::String]
        #     Indicates the reason why the VPN connection does not meet the high availability redundancy criteria/requirement. Valid values is INCOMPLETE_TUNNELS_COVERAGE.
        #     Check the UnsatisfiedReason enum for the list of possible values.
        class VpnGatewayStatusHighAvailabilityRequirementState
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # Indicates the high availability requirement state for the VPN connection. Valid values are CONNECTION_REDUNDANCY_MET, CONNECTION_REDUNDANCY_NOT_MET.
          module State
            # A value indicating that the enum field is not set.
            UNDEFINED_STATE = 0

            # VPN tunnels are configured with adequate redundancy from Cloud VPN gateway to the peer VPN gateway. For both GCP-to-non-GCP and GCP-to-GCP connections, the adequate redundancy is a pre-requirement for users to get 99.99% availability on GCP side; please note that for any connection, end-to-end 99.99% availability is subject to proper configuration on the peer VPN gateway.
            CONNECTION_REDUNDANCY_MET = 505_242_907

            # VPN tunnels are not configured with adequate redundancy from the Cloud VPN gateway to the peer gateway
            CONNECTION_REDUNDANCY_NOT_MET = 511_863_311
          end

          # Indicates the reason why the VPN connection does not meet the high availability redundancy criteria/requirement. Valid values is INCOMPLETE_TUNNELS_COVERAGE.
          module UnsatisfiedReason
            # A value indicating that the enum field is not set.
            UNDEFINED_UNSATISFIED_REASON = 0

            INCOMPLETE_TUNNELS_COVERAGE = 55_917_437
          end
        end

        # Contains some information about a VPN tunnel.
        # @!attribute [rw] local_gateway_interface
        #   @return [::Integer]
        #     The VPN gateway interface this VPN tunnel is associated with.
        # @!attribute [rw] peer_gateway_interface
        #   @return [::Integer]
        #     The peer gateway interface this VPN tunnel is connected to, the peer gateway could either be an external VPN gateway or a Google Cloud VPN gateway.
        # @!attribute [rw] tunnel_url
        #   @return [::String]
        #     URL reference to the VPN tunnel.
        class VpnGatewayStatusTunnel
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A VPN connection contains all VPN tunnels connected from this VpnGateway to the same peer gateway. The peer gateway could either be an external VPN gateway or a Google Cloud VPN gateway.
        # @!attribute [rw] peer_external_gateway
        #   @return [::String]
        #     URL reference to the peer external VPN gateways to which the VPN tunnels in this VPN connection are connected. This field is mutually exclusive with peer_gcp_gateway.
        # @!attribute [rw] peer_gcp_gateway
        #   @return [::String]
        #     URL reference to the peer side VPN gateways to which the VPN tunnels in this VPN connection are connected. This field is mutually exclusive with peer_gcp_gateway.
        # @!attribute [rw] state
        #   @return [::Google::Cloud::Compute::V1::VpnGatewayStatusHighAvailabilityRequirementState]
        #     HighAvailabilityRequirementState for the VPN connection.
        # @!attribute [rw] tunnels
        #   @return [::Array<::Google::Cloud::Compute::V1::VpnGatewayStatusTunnel>]
        #     List of VPN tunnels that are in this VPN connection.
        class VpnGatewayStatusVpnConnection
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A VPN gateway interface.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] Numeric identifier for this VPN interface associated with the VPN gateway.
        # @!attribute [rw] interconnect_attachment
        #   @return [::String]
        #     URL of the VLAN attachment (interconnectAttachment) resource for this VPN gateway interface. When the value of this field is present, the VPN gateway is used for HA VPN over Cloud Interconnect; all egress or ingress traffic for this VPN gateway interface goes through the specified VLAN attachment resource.
        # @!attribute [rw] ip_address
        #   @return [::String]
        #     [Output Only] IP address for this VPN interface associated with the VPN gateway. The IP address could be either a regional external IP address or a regional internal IP address. The two IP addresses for a VPN gateway must be all regional external or regional internal IP addresses. There cannot be a mix of regional external IP addresses and regional internal IP addresses. For HA VPN over Cloud Interconnect, the IP addresses for both interfaces could either be regional internal IP addresses or regional external IP addresses. For regular (non HA VPN over Cloud Interconnect) HA VPN tunnels, the IP address must be a regional external IP address.
        class VpnGatewayVpnGatewayInterface
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] result
        #   @return [::Google::Cloud::Compute::V1::VpnGatewayStatus]
        class VpnGatewaysGetStatusResponse
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] vpn_gateways
        #   @return [::Array<::Google::Cloud::Compute::V1::VpnGateway>]
        #     [Output Only] A list of VPN gateways contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning which replaces the list of addresses when the list is empty.
        class VpnGatewaysScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Represents a Cloud VPN Tunnel resource. For more information about VPN, read the the Cloud VPN Overview.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] description
        #   @return [::String]
        #     An optional description of this resource. Provide this property when you create the resource.
        # @!attribute [rw] detailed_status
        #   @return [::String]
        #     [Output Only] Detailed status message for the VPN tunnel.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] ike_version
        #   @return [::Integer]
        #     IKE protocol version to use when establishing the VPN tunnel with the peer VPN gateway. Acceptable IKE versions are 1 or 2. The default version is 2.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#vpnTunnel for VPN tunnels.
        # @!attribute [rw] label_fingerprint
        #   @return [::String]
        #     A fingerprint for the labels being applied to this VpnTunnel, which is essentially a hash of the labels set used for optimistic locking. The fingerprint is initially generated by Compute Engine and changes after every request to modify or update labels. You must always provide an up-to-date fingerprint hash in order to update or change labels, otherwise the request will fail with error 412 conditionNotMet. To see the latest fingerprint, make a get() request to retrieve a VpnTunnel.
        # @!attribute [rw] labels
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     Labels for this resource. These can only be added or modified by the setLabels method. Each label key/value pair must comply with RFC1035. Label values may be empty.
        # @!attribute [rw] local_traffic_selector
        #   @return [::Array<::String>]
        #     Local traffic selector to use when establishing the VPN tunnel with the peer VPN gateway. The value should be a CIDR formatted string, for example: 192.168.0.0/16. The ranges must be disjoint. Only IPv4 is supported.
        # @!attribute [rw] name
        #   @return [::String]
        #     Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
        # @!attribute [rw] peer_external_gateway
        #   @return [::String]
        #     URL of the peer side external VPN gateway to which this VPN tunnel is connected. Provided by the client when the VPN tunnel is created. This field is exclusive with the field peerGcpGateway.
        # @!attribute [rw] peer_external_gateway_interface
        #   @return [::Integer]
        #     The interface ID of the external VPN gateway to which this VPN tunnel is connected. Provided by the client when the VPN tunnel is created. Possible values are: `0`, `1`, `2`, `3`. The number of IDs in use depends on the external VPN gateway redundancy type.
        # @!attribute [rw] peer_gcp_gateway
        #   @return [::String]
        #     URL of the peer side HA VPN gateway to which this VPN tunnel is connected. Provided by the client when the VPN tunnel is created. This field can be used when creating highly available VPN from VPC network to VPC network, the field is exclusive with the field peerExternalGateway. If provided, the VPN tunnel will automatically use the same vpnGatewayInterface ID in the peer Google Cloud VPN gateway.
        # @!attribute [rw] peer_ip
        #   @return [::String]
        #     IP address of the peer VPN gateway. Only IPv4 is supported.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] URL of the region where the VPN tunnel resides. You must specify this field as part of the HTTP request URL. It is not settable as a field in the request body.
        # @!attribute [rw] remote_traffic_selector
        #   @return [::Array<::String>]
        #     Remote traffic selectors to use when establishing the VPN tunnel with the peer VPN gateway. The value should be a CIDR formatted string, for example: 192.168.0.0/16. The ranges should be disjoint. Only IPv4 is supported.
        # @!attribute [rw] router
        #   @return [::String]
        #     URL of the router resource to be used for dynamic routing.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] shared_secret
        #   @return [::String]
        #     Shared secret used to set the secure session between the Cloud VPN gateway and the peer VPN gateway.
        # @!attribute [rw] shared_secret_hash
        #   @return [::String]
        #     Hash of the shared secret.
        # @!attribute [rw] status
        #   @return [::String]
        #     [Output Only] The status of the VPN tunnel, which can be one of the following: - PROVISIONING: Resource is being allocated for the VPN tunnel. - WAITING_FOR_FULL_CONFIG: Waiting to receive all VPN-related configs from the user. Network, TargetVpnGateway, VpnTunnel, ForwardingRule, and Route resources are needed to setup the VPN tunnel. - FIRST_HANDSHAKE: Successful first handshake with the peer VPN. - ESTABLISHED: Secure session is successfully established with the peer VPN. - NETWORK_ERROR: Deprecated, replaced by NO_INCOMING_PACKETS - AUTHORIZATION_ERROR: Auth error (for example, bad shared secret). - NEGOTIATION_FAILURE: Handshake failed. - DEPROVISIONING: Resources are being deallocated for the VPN tunnel. - FAILED: Tunnel creation has failed and the tunnel is not ready to be used. - NO_INCOMING_PACKETS: No incoming packets from peer. - REJECTED: Tunnel configuration was rejected, can be result of being denied access. - ALLOCATING_RESOURCES: Cloud VPN is in the process of allocating all required resources. - STOPPED: Tunnel is stopped due to its Forwarding Rules being deleted for Classic VPN tunnels or the project is in frozen state. - PEER_IDENTITY_MISMATCH: Peer identity does not match peer IP, probably behind NAT. - TS_NARROWING_NOT_ALLOWED: Traffic selector narrowing not allowed for an HA-VPN tunnel.
        #     Check the Status enum for the list of possible values.
        # @!attribute [rw] target_vpn_gateway
        #   @return [::String]
        #     URL of the Target VPN gateway with which this VPN tunnel is associated. Provided by the client when the VPN tunnel is created.
        # @!attribute [rw] vpn_gateway
        #   @return [::String]
        #     URL of the VPN gateway with which this VPN tunnel is associated. Provided by the client when the VPN tunnel is created. This must be used (instead of target_vpn_gateway) if a High Availability VPN gateway resource is created.
        # @!attribute [rw] vpn_gateway_interface
        #   @return [::Integer]
        #     The interface ID of the VPN gateway with which this VPN tunnel is associated. Possible values are: `0`, `1`.
        class VpnTunnel
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class LabelsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end

          # [Output Only] The status of the VPN tunnel, which can be one of the following: - PROVISIONING: Resource is being allocated for the VPN tunnel. - WAITING_FOR_FULL_CONFIG: Waiting to receive all VPN-related configs from the user. Network, TargetVpnGateway, VpnTunnel, ForwardingRule, and Route resources are needed to setup the VPN tunnel. - FIRST_HANDSHAKE: Successful first handshake with the peer VPN. - ESTABLISHED: Secure session is successfully established with the peer VPN. - NETWORK_ERROR: Deprecated, replaced by NO_INCOMING_PACKETS - AUTHORIZATION_ERROR: Auth error (for example, bad shared secret). - NEGOTIATION_FAILURE: Handshake failed. - DEPROVISIONING: Resources are being deallocated for the VPN tunnel. - FAILED: Tunnel creation has failed and the tunnel is not ready to be used. - NO_INCOMING_PACKETS: No incoming packets from peer. - REJECTED: Tunnel configuration was rejected, can be result of being denied access. - ALLOCATING_RESOURCES: Cloud VPN is in the process of allocating all required resources. - STOPPED: Tunnel is stopped due to its Forwarding Rules being deleted for Classic VPN tunnels or the project is in frozen state. - PEER_IDENTITY_MISMATCH: Peer identity does not match peer IP, probably behind NAT. - TS_NARROWING_NOT_ALLOWED: Traffic selector narrowing not allowed for an HA-VPN tunnel.
          module Status
            # A value indicating that the enum field is not set.
            UNDEFINED_STATUS = 0

            # Cloud VPN is in the process of allocating all required resources (specifically, a borg task).
            ALLOCATING_RESOURCES = 320_922_816

            # Auth error (e.g. bad shared secret).
            AUTHORIZATION_ERROR = 23_580_290

            # Resources is being deallocated for the VPN tunnel.
            DEPROVISIONING = 428_935_662

            # Secure session is successfully established with peer VPN.
            ESTABLISHED = 88_852_344

            # Tunnel creation has failed and the tunnel is not ready to be used.
            FAILED = 455_706_685

            # Successful first handshake with peer VPN.
            FIRST_HANDSHAKE = 191_393_000

            # Handshake failed.
            NEGOTIATION_FAILURE = 360_325_868

            # Deprecated, replaced by NO_INCOMING_PACKETS
            NETWORK_ERROR = 193_912_951

            # No incoming packets from peer
            NO_INCOMING_PACKETS = 119_983_216

            # Resource is being allocated for the VPN tunnel.
            PROVISIONING = 290_896_621

            # Tunnel configuration was rejected, can be result of being denylisted.
            REJECTED = 174_130_302

            # Tunnel is stopped due to its Forwarding Rules being deleted.
            STOPPED = 444_276_141

            # Waiting to receive all VPN-related configs from user. Network, TargetVpnGateway, VpnTunnel, ForwardingRule and Route resources are needed to setup VPN tunnel.
            WAITING_FOR_FULL_CONFIG = 41_640_522
          end
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Google::Protobuf::Map{::String => ::Google::Cloud::Compute::V1::VpnTunnelsScopedList}]
        #     A list of VpnTunnelsScopedList resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#vpnTunnel for VPN tunnels.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] unreachables
        #   @return [::Array<::String>]
        #     [Output Only] Unreachable resources.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class VpnTunnelAggregatedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::Google::Cloud::Compute::V1::VpnTunnelsScopedList]
          class ItemsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # Contains a list of VpnTunnel resources.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::VpnTunnel>]
        #     A list of VpnTunnel resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#vpnTunnel for VPN tunnels.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class VpnTunnelList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] vpn_tunnels
        #   @return [::Array<::Google::Cloud::Compute::V1::VpnTunnel>]
        #     A list of VPN tunnels contained in this scope.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     Informational warning which replaces the list of addresses when the list is empty.
        class VpnTunnelsScopedList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] aliases
        #   @return [::Array<::String>]
        #     A list of alternate IDs. The format should be: - E.g. XSS-stable Generic suffix like "stable" is particularly useful if a policy likes to avail newer set of expressions without having to change the policy. A given alias name can't be used for more than one entity set.
        # @!attribute [rw] expressions
        #   @return [::Array<::Google::Cloud::Compute::V1::WafExpressionSetExpression>]
        #     List of available expressions.
        # @!attribute [rw] id
        #   @return [::String]
        #     Google specified expression set ID. The format should be: - E.g. XSS-20170329 required
        class WafExpressionSet
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     Expression ID should uniquely identify the origin of the expression. E.g. owasp-crs-v020901-id973337 identifies Owasp core rule set version 2.9.1 rule id 973337. The ID could be used to determine the individual attack definition that has been detected. It could also be used to exclude it from the policy in case of false positive. required
        # @!attribute [rw] sensitivity
        #   @return [::Integer]
        #     The sensitivity value associated with the WAF rule ID. This corresponds to the ModSecurity paranoia level, ranging from 1 to 4. 0 is reserved for opt-in only rules.
        class WafExpressionSetExpression
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for GlobalOperations.Wait. See the method description for details.
        # @!attribute [rw] operation
        #   @return [::String]
        #     Name of the Operations resource to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        class WaitGlobalOperationRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for RegionOperations.Wait. See the method description for details.
        # @!attribute [rw] operation
        #   @return [::String]
        #     Name of the Operations resource to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] region
        #   @return [::String]
        #     Name of the region for this request.
        class WaitRegionOperationRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for ZoneOperations.Wait. See the method description for details.
        # @!attribute [rw] operation
        #   @return [::String]
        #     Name of the Operations resource to return.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] zone
        #   @return [::String]
        #     Name of the zone for this request.
        class WaitZoneOperationRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # [Output Only] Informational warning message.
        # @!attribute [rw] code
        #   @return [::String]
        #     [Output Only] A warning code, if applicable. For example, Compute Engine returns NO_RESULTS_ON_PAGE if there are no results in the response.
        #     Check the Code enum for the list of possible values.
        # @!attribute [rw] data
        #   @return [::Array<::Google::Cloud::Compute::V1::Data>]
        #     [Output Only] Metadata about this warning in key: value format. For example: "data": [ { "key": "scope", "value": "zones/us-east1-d" }
        # @!attribute [rw] message
        #   @return [::String]
        #     [Output Only] A human-readable description of the warning code.
        class Warning
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # [Output Only] A warning code, if applicable. For example, Compute Engine returns NO_RESULTS_ON_PAGE if there are no results in the response.
          module Code
            # A value indicating that the enum field is not set.
            UNDEFINED_CODE = 0

            # Warning about failed cleanup of transient changes made by a failed operation.
            CLEANUP_FAILED = 150_308_440

            # A link to a deprecated resource was created.
            DEPRECATED_RESOURCE_USED = 391_835_586

            # When deploying and at least one of the resources has a type marked as deprecated
            DEPRECATED_TYPE_USED = 346_526_230

            # The user created a boot disk that is larger than image size.
            DISK_SIZE_LARGER_THAN_IMAGE_SIZE = 369_442_967

            # When deploying and at least one of the resources has a type marked as experimental
            EXPERIMENTAL_TYPE_USED = 451_954_443

            # Warning that is present in an external api call
            EXTERNAL_API_WARNING = 175_546_307

            # Warning that value of a field has been overridden. Deprecated unused field.
            FIELD_VALUE_OVERRIDEN = 329_669_423

            # The operation involved use of an injected kernel, which is deprecated.
            INJECTED_KERNELS_DEPRECATED = 417_377_419

            # A WEIGHTED_MAGLEV backend service is associated with a health check that is not of type HTTP/HTTPS/HTTP2.
            INVALID_HEALTH_CHECK_FOR_DYNAMIC_WIEGHTED_LB = 401_542_606

            # When deploying a deployment with a exceedingly large number of resources
            LARGE_DEPLOYMENT_WARNING = 481_440_678

            # Resource can't be retrieved due to list overhead quota exceed which captures the amount of resources filtered out by user-defined list filter.
            LIST_OVERHEAD_QUOTA_EXCEED = 47_618_117

            # A resource depends on a missing type
            MISSING_TYPE_DEPENDENCY = 344_505_463

            # The route's nextHopIp address is not assigned to an instance on the network.
            NEXT_HOP_ADDRESS_NOT_ASSIGNED = 324_964_999

            # The route's next hop instance cannot ip forward.
            NEXT_HOP_CANNOT_IP_FORWARD = 383_382_887

            # The route's nextHopInstance URL refers to an instance that does not have an ipv6 interface on the same network as the route.
            NEXT_HOP_INSTANCE_HAS_NO_IPV6_INTERFACE = 146_748_434

            # The route's nextHopInstance URL refers to an instance that does not exist.
            NEXT_HOP_INSTANCE_NOT_FOUND = 464_250_446

            # The route's nextHopInstance URL refers to an instance that is not on the same network as the route.
            NEXT_HOP_INSTANCE_NOT_ON_NETWORK = 243_758_146

            # The route's next hop instance does not have a status of RUNNING.
            NEXT_HOP_NOT_RUNNING = 417_081_265

            # Error which is not critical. We decided to continue the process despite the mentioned error.
            NOT_CRITICAL_ERROR = 105_763_924

            # No results are present on a particular list page.
            NO_RESULTS_ON_PAGE = 30_036_744

            # Success is reported, but some results may be missing due to errors
            PARTIAL_SUCCESS = 39_966_469

            # The user attempted to use a resource that requires a TOS they have not accepted.
            REQUIRED_TOS_AGREEMENT = 3_745_539

            # Warning that a resource is in use.
            RESOURCE_IN_USE_BY_OTHER_RESOURCE_WARNING = 496_728_641

            # One or more of the resources set to auto-delete could not be deleted because they were in use.
            RESOURCE_NOT_DELETED = 168_598_460

            # When a resource schema validation is ignored.
            SCHEMA_VALIDATION_IGNORED = 275_245_642

            # Instance template used in instance group manager is valid as such, but its application does not make a lot of sense, because it allows only single instance in instance group.
            SINGLE_INSTANCE_PROPERTY_TEMPLATE = 268_305_617

            # When undeclared properties in the schema are present
            UNDECLARED_PROPERTIES = 390_513_439

            # A given scope cannot be reached.
            UNREACHABLE = 13_328_052
          end
        end

        # @!attribute [rw] code
        #   @return [::String]
        #     [Output Only] A warning code, if applicable. For example, Compute Engine returns NO_RESULTS_ON_PAGE if there are no results in the response.
        #     Check the Code enum for the list of possible values.
        # @!attribute [rw] data
        #   @return [::Array<::Google::Cloud::Compute::V1::Data>]
        #     [Output Only] Metadata about this warning in key: value format. For example: "data": [ { "key": "scope", "value": "zones/us-east1-d" }
        # @!attribute [rw] message
        #   @return [::String]
        #     [Output Only] A human-readable description of the warning code.
        class Warnings
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # [Output Only] A warning code, if applicable. For example, Compute Engine returns NO_RESULTS_ON_PAGE if there are no results in the response.
          module Code
            # A value indicating that the enum field is not set.
            UNDEFINED_CODE = 0

            # Warning about failed cleanup of transient changes made by a failed operation.
            CLEANUP_FAILED = 150_308_440

            # A link to a deprecated resource was created.
            DEPRECATED_RESOURCE_USED = 391_835_586

            # When deploying and at least one of the resources has a type marked as deprecated
            DEPRECATED_TYPE_USED = 346_526_230

            # The user created a boot disk that is larger than image size.
            DISK_SIZE_LARGER_THAN_IMAGE_SIZE = 369_442_967

            # When deploying and at least one of the resources has a type marked as experimental
            EXPERIMENTAL_TYPE_USED = 451_954_443

            # Warning that is present in an external api call
            EXTERNAL_API_WARNING = 175_546_307

            # Warning that value of a field has been overridden. Deprecated unused field.
            FIELD_VALUE_OVERRIDEN = 329_669_423

            # The operation involved use of an injected kernel, which is deprecated.
            INJECTED_KERNELS_DEPRECATED = 417_377_419

            # A WEIGHTED_MAGLEV backend service is associated with a health check that is not of type HTTP/HTTPS/HTTP2.
            INVALID_HEALTH_CHECK_FOR_DYNAMIC_WIEGHTED_LB = 401_542_606

            # When deploying a deployment with a exceedingly large number of resources
            LARGE_DEPLOYMENT_WARNING = 481_440_678

            # Resource can't be retrieved due to list overhead quota exceed which captures the amount of resources filtered out by user-defined list filter.
            LIST_OVERHEAD_QUOTA_EXCEED = 47_618_117

            # A resource depends on a missing type
            MISSING_TYPE_DEPENDENCY = 344_505_463

            # The route's nextHopIp address is not assigned to an instance on the network.
            NEXT_HOP_ADDRESS_NOT_ASSIGNED = 324_964_999

            # The route's next hop instance cannot ip forward.
            NEXT_HOP_CANNOT_IP_FORWARD = 383_382_887

            # The route's nextHopInstance URL refers to an instance that does not have an ipv6 interface on the same network as the route.
            NEXT_HOP_INSTANCE_HAS_NO_IPV6_INTERFACE = 146_748_434

            # The route's nextHopInstance URL refers to an instance that does not exist.
            NEXT_HOP_INSTANCE_NOT_FOUND = 464_250_446

            # The route's nextHopInstance URL refers to an instance that is not on the same network as the route.
            NEXT_HOP_INSTANCE_NOT_ON_NETWORK = 243_758_146

            # The route's next hop instance does not have a status of RUNNING.
            NEXT_HOP_NOT_RUNNING = 417_081_265

            # Error which is not critical. We decided to continue the process despite the mentioned error.
            NOT_CRITICAL_ERROR = 105_763_924

            # No results are present on a particular list page.
            NO_RESULTS_ON_PAGE = 30_036_744

            # Success is reported, but some results may be missing due to errors
            PARTIAL_SUCCESS = 39_966_469

            # The user attempted to use a resource that requires a TOS they have not accepted.
            REQUIRED_TOS_AGREEMENT = 3_745_539

            # Warning that a resource is in use.
            RESOURCE_IN_USE_BY_OTHER_RESOURCE_WARNING = 496_728_641

            # One or more of the resources set to auto-delete could not be deleted because they were in use.
            RESOURCE_NOT_DELETED = 168_598_460

            # When a resource schema validation is ignored.
            SCHEMA_VALIDATION_IGNORED = 275_245_642

            # Instance template used in instance group manager is valid as such, but its application does not make a lot of sense, because it allows only single instance in instance group.
            SINGLE_INSTANCE_PROPERTY_TEMPLATE = 268_305_617

            # When undeclared properties in the schema are present
            UNDECLARED_PROPERTIES = 390_513_439

            # A given scope cannot be reached.
            UNREACHABLE = 13_328_052
          end
        end

        # In contrast to a single BackendService in HttpRouteAction to which all matching traffic is directed to, WeightedBackendService allows traffic to be split across multiple backend services. The volume of traffic for each backend service is proportional to the weight specified in each WeightedBackendService
        # @!attribute [rw] backend_service
        #   @return [::String]
        #     The full or partial URL to the default BackendService resource. Before forwarding the request to backendService, the load balancer applies any relevant headerActions specified as part of this backendServiceWeight.
        # @!attribute [rw] header_action
        #   @return [::Google::Cloud::Compute::V1::HttpHeaderAction]
        #     Specifies changes to request and response headers that need to take effect for the selected backendService. headerAction specified here take effect before headerAction in the enclosing HttpRouteRule, PathMatcher and UrlMap. headerAction is not supported for load balancers that have their loadBalancingScheme set to EXTERNAL. Not supported when the URL map is bound to a target gRPC proxy that has validateForProxyless field set to true.
        # @!attribute [rw] weight
        #   @return [::Integer]
        #     Specifies the fraction of traffic sent to a backend service, computed as weight / (sum of all weightedBackendService weights in routeAction) . The selection of a backend service is determined only for new traffic. Once a user's request has been directed to a backend service, subsequent requests are sent to the same backend service as determined by the backend service's session affinity policy. The value must be from 0 to 1000.
        class WeightedBackendService
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for PublicAdvertisedPrefixes.Withdraw. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] public_advertised_prefix
        #   @return [::String]
        #     The name of the public advertised prefix. It should comply with RFC1035.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class WithdrawPublicAdvertisedPrefixeRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # A request message for PublicDelegatedPrefixes.Withdraw. See the method description for details.
        # @!attribute [rw] project
        #   @return [::String]
        #     Project ID for this request.
        # @!attribute [rw] public_delegated_prefix
        #   @return [::String]
        #     The name of the public delegated prefix. It should comply with RFC1035.
        # @!attribute [rw] region
        #   @return [::String]
        #     The name of the region where the public delegated prefix is located. It should comply with RFC1035.
        # @!attribute [rw] request_id
        #   @return [::String]
        #     An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
        class WithdrawPublicDelegatedPrefixeRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::Project>]
        #     [Output Only] A list of shared VPC host project URLs.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of resource. Always compute#xpnHostList for lists of shared VPC hosts.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class XpnHostList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Service resource (a.k.a service project) ID.
        # @!attribute [rw] id
        #   @return [::String]
        #     The ID of the service resource. In the case of projects, this field supports project id (e.g., my-project-123) and project number (e.g. 12345678).
        # @!attribute [rw] type
        #   @return [::String]
        #     The type of the service resource.
        #     Check the Type enum for the list of possible values.
        class XpnResourceId
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # The type of the service resource.
          module Type
            # A value indicating that the enum field is not set.
            UNDEFINED_TYPE = 0

            PROJECT = 408_671_993

            XPN_RESOURCE_TYPE_UNSPECIFIED = 151_607_034
          end
        end

        # Represents a Zone resource. A zone is a deployment area. These deployment areas are subsets of a region. For example the zone us-east1-a is located in the us-east1 region. For more information, read Regions and Zones.
        # @!attribute [rw] available_cpu_platforms
        #   @return [::Array<::String>]
        #     [Output Only] Available cpu/platform selections for the zone.
        # @!attribute [rw] creation_timestamp
        #   @return [::String]
        #     [Output Only] Creation timestamp in RFC3339 text format.
        # @!attribute [rw] deprecated
        #   @return [::Google::Cloud::Compute::V1::DeprecationStatus]
        #     [Output Only] The deprecation status associated with this zone.
        # @!attribute [rw] description
        #   @return [::String]
        #     [Output Only] Textual description of the resource.
        # @!attribute [rw] id
        #   @return [::Integer]
        #     [Output Only] The unique identifier for the resource. This identifier is defined by the server.
        # @!attribute [rw] kind
        #   @return [::String]
        #     [Output Only] Type of the resource. Always compute#zone for zones.
        # @!attribute [rw] name
        #   @return [::String]
        #     [Output Only] Name of the resource.
        # @!attribute [rw] region
        #   @return [::String]
        #     [Output Only] Full URL reference to the region which hosts the zone.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for the resource.
        # @!attribute [rw] status
        #   @return [::String]
        #     [Output Only] Status of the zone, either UP or DOWN.
        #     Check the Status enum for the list of possible values.
        # @!attribute [rw] supports_pzs
        #   @return [::Boolean]
        #     [Output Only] Reserved for future use.
        class Zone
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # [Output Only] Status of the zone, either UP or DOWN.
          module Status
            # A value indicating that the enum field is not set.
            UNDEFINED_STATUS = 0

            DOWN = 2_104_482

            UP = 2715
          end
        end

        # Contains a list of zone resources.
        # @!attribute [rw] id
        #   @return [::String]
        #     [Output Only] Unique identifier for the resource; defined by the server.
        # @!attribute [rw] items
        #   @return [::Array<::Google::Cloud::Compute::V1::Zone>]
        #     A list of Zone resources.
        # @!attribute [rw] kind
        #   @return [::String]
        #     Type of resource.
        # @!attribute [rw] next_page_token
        #   @return [::String]
        #     [Output Only] This token allows you to get the next page of results for list requests. If the number of results is larger than maxResults, use the nextPageToken as a value for the query parameter pageToken in the next list request. Subsequent list requests will have their own nextPageToken to continue paging through the results.
        # @!attribute [rw] self_link
        #   @return [::String]
        #     [Output Only] Server-defined URL for this resource.
        # @!attribute [rw] warning
        #   @return [::Google::Cloud::Compute::V1::Warning]
        #     [Output Only] Informational warning message.
        class ZoneList
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # @!attribute [rw] label_fingerprint
        #   @return [::String]
        #     The fingerprint of the previous set of labels for this resource, used to detect conflicts. The fingerprint is initially generated by Compute Engine and changes after every request to modify or update labels. You must always provide an up-to-date fingerprint hash in order to update or change labels. Make a get() request to the resource to get the latest fingerprint.
        # @!attribute [rw] labels
        #   @return [::Google::Protobuf::Map{::String => ::String}]
        #     The labels to set for this resource.
        class ZoneSetLabelsRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods

          # @!attribute [rw] key
          #   @return [::String]
          # @!attribute [rw] value
          #   @return [::String]
          class LabelsEntry
            include ::Google::Protobuf::MessageExts
            extend ::Google::Protobuf::MessageExts::ClassMethods
          end
        end

        # @!attribute [rw] bindings
        #   @return [::Array<::Google::Cloud::Compute::V1::Binding>]
        #     Flatten Policy to create a backwacd compatible wire-format. Deprecated. Use 'policy' to specify bindings.
        # @!attribute [rw] etag
        #   @return [::String]
        #     Flatten Policy to create a backward compatible wire-format. Deprecated. Use 'policy' to specify the etag.
        # @!attribute [rw] policy
        #   @return [::Google::Cloud::Compute::V1::Policy]
        #     REQUIRED: The complete policy to be applied to the 'resource'. The size of the policy is limited to a few 10s of KB. An empty policy is in general a valid policy but certain services (like Projects) might reject them.
        class ZoneSetPolicyRequest
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end
      end
    end
  end
end
